/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AE_H
#define AE_H

#include <aqfinance/engine/ae_api.h>

#include <aqfinance/print/af_print_table_descr.h>

#include <aqdatabase/aqdb_db.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/configmgr.h>


#define AE_PM_LIBNAME    "aqfinance"
#define AE_PM_SYSCONFDIR "sysconfdir"
#define AE_PM_DATADIR    "datadir"
#define AE_PM_LOCALEDIR  "localedir"

#define AE_IMEXPORTER_PLUGIN_FOLDER "imexporter"


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



typedef enum {
  AE_ProcessEventType_Unknown=-1,
  AE_ProcessEventType_None=0,
  AE_ProcessEventType_Request,
  AE_ProcessEventType_Offer,
  AE_ProcessEventType_Order,
  AE_ProcessEventType_Confirmation,
  AE_ProcessEventType_Delivery,
  AE_ProcessEventType_Invoice,
  AE_ProcessEventType_Payment
} AE_PROCESS_EVENT_TYPE;


enum {
  AE_Units_Unknown = -1,
  AE_Units_None,
  AE_Units_Liters,
  AE_Units_Grammes,
  AE_Units_Meters,
  AE_Units_SquareMeters,
  AE_Units_CubicMeters,
  AE_Units_Minutes,
  AE_Units_Hours,
  AE_Units_Miles,
  AE_Units_KiloMeters
};



#define AE_OBJECT_FLAGS_ATTENTION 0x80000000
#define AE_OBJECT_FLAGS_TODO      0x40000000
#define AE_OBJECT_FLAGS_IMPORTANT 0x20000000


#define AE_CURRENCY_NAMESPACE_ISO_4217_CODE "iso4217code"


#ifdef __cplusplus
extern "C" {
#endif


AE_API int AE_Init(const char *dname);
AE_API int AE_Fini();


AE_API const char *AE_GetUserDataFolder();
AE_API GWEN_CONFIGMGR *AE_GetConfigManager();

AE_API int AE_Units_fromString(const char *s);

AE_API const char *AE_Units_toString(int i);


AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsInvoice();
AE_API void AE_SetPsInvoice(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsRequest();
AE_API void AE_SetPsRequest(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsOffer();
AE_API void AE_SetPsOffer(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsConfirmation();
AE_API void AE_SetPsConfirmation(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsDelivery();
AE_API void AE_SetPsDelivery(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsRelease();
AE_API void AE_SetPsRelease(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsReminder();
AE_API void AE_SetPsReminder(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsBuyOrder();
AE_API void AE_SetPsBuyOrder(const AF_PRINT_TABLE_DESCR *ps);

AE_API const AF_PRINT_TABLE_DESCR *AE_GetPsSellOrder();
AE_API void AE_SetPsSellOrder(const AF_PRINT_TABLE_DESCR *ps);


/**
 * Reads invoice settings for the given partner. If no settings could be found
 * then new settings will be derived from the global invoice settings.
 *
 * @return description owned by the caller (i.e. you need to call @ref AF_PrintTableDescr_free
 *   if you are done with it)
 *
 * @param parterId id of the partner for which to read the settings
 */
AE_API AF_PRINT_TABLE_DESCR *AE_ReadPsInvoiceForPartner(AQDB_ID partnerId);
AE_API void AE_WritePsInvoiceForPartner(const AF_PRINT_TABLE_DESCR *ps, AQDB_ID partnerId);



#ifdef __cplusplus
}
#endif


#endif

