/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef AE_TRANSACTION_FNS_H
#define AE_TRANSACTION_FNS_H

#ifdef __cplusplus
extern "C" {
#endif


AE_API void AE_Transaction_AddSplit(AE_TRANSACTION *t, AE_SPLIT *sp);

/**
 * This function calculates the total amount moved (of the given commodity), e.g. if there is one
 * split with + 10 EUR and another one with - 10 EUR (correct balance) then the amount moved (and
 * returned here)  is 10 EUR. The value returned (if any) is always positive.
 */
AE_API AQDB_VALUE *AE_Transaction_CalculateAmount(const AE_TRANSACTION *t, AQDB_ID commodityId);

/**
 * Calculates the total sum of all splits of the given commodity. Correct transactions always
 * sum up to zero. If any value other than zero is returned then the transaction isn't balanced
 * regarding the given commodity.
 */
AE_API AQDB_VALUE *AE_Transaction_CalculateSum(const AE_TRANSACTION *t, AQDB_ID commodityId);

/**
 * Samples the commodities used in the given transaction.
 */
AE_API int AE_Transaction_SampleCommodities(const AE_TRANSACTION *t, AQDB_ID *idBuffer, int size);

/**
 * Returns the most positive split in the given transaction.
 */
AE_API AE_SPLIT *AE_Transaction_GetMaxDebitSplit(const AE_TRANSACTION *t);

/**
 * Returns the most negative split in the given transaction.
 */
AE_API AE_SPLIT *AE_Transaction_GetMaxCreditSplit(const AE_TRANSACTION *t);

/**
 * Returns the split which balances most of the amount of the split which is assigned
 * the given account id.
 */
AE_API AE_SPLIT *AE_Transaction_GetCounterSplit(const AE_TRANSACTION *t, AQDB_ID aid);


AE_API int AE_Transaction_Matches(const AE_TRANSACTION *t, const char *s);
AE_API void AE_Transaction_List_SelectMatching(AE_TRANSACTION_LIST *tl, const char *s);


#ifdef __cplusplus
}
#endif


#endif
