/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AE_UTILS_H
#define AE_UTILS_H

#include <aqfinance/engine/ae.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_item.h>

#include <aqdatabase/aqdb.h>
#include <aqdatabase/aqdb_db.h>

#include <gwenhywfar/buffer.h>
#include <gwenhywfar/db.h>


#ifdef __cplusplus
extern "C" {
#endif


typedef struct {
  const char *name;
  const char *text;
  const char *tableHeader;
} AE_FIELD_DESCRIPTION;





AE_API int AE_Utils_ParseValueAndUnits(const char *s,
				       GWEN_BUFFER *valueBuf,
				       GWEN_BUFFER *unitsBuf);

AE_API int AE_Utils_AmountFromString(const char *text,
				     const AE_COMMODITY_LIST *cl,
                                     AQDB_VALUE **pValue,
				     AE_COMMODITY **pCommodity);

AE_API int AE_Utils_AmountToString(const AQDB_VALUE *value,
				   AQDB_ID commodityId,
				   const AE_COMMODITY_LIST *cl,
				   GWEN_BUFFER *tbuf);

AE_API int AE_Utils_AmountToStringNoSymbol(const AQDB_VALUE *value,
					   AQDB_ID commodityId,
					   const AE_COMMODITY_LIST *cl,
					   GWEN_BUFFER *tbuf);


/** @TODO: remove this function! */
AE_API int AE_Utils_ValueAndItemUnitsFromString(const char *text,
						AQDB_VALUE **pValue,
						int *pUnits);

/** @TODO: remove this function! */
AE_API const char *AE_Utils_ItemUnitsToString(int units);


AE_API int AE_Utils_ValueAndUnitsFromString(const char *text,
                                            AQDB_VALUE **pValue,
                                            int *pUnits);
AE_API void AE_Utils_ValueAndUnitsToString(const AQDB_VALUE *v, int units, GWEN_BUFFER *buf, int precision);

AE_API const char *AE_Utils_UnitsToString(int u);
AE_API const char *AE_Utils_FuelType_toString(int i);


AE_API const char *AE_Utils_StrCaseStr(const char *haystack, const char *needle);

AE_API void AE_Utils_NormalizeUtf8Umlauts(const char *p, int size, GWEN_BUFFER *buf);


/**
 * This function replaces variable names in the given text by their values from the
 * given db.
 * Variables are expected in one of the following formats:
 * - $(VARNAME)
 * - $(VARNAME?DEFAULTVALUE)
 * - $(VARNAME?DEFAULTVALUE:PARAMS)
 * - $(VARNAME:PARAMS)
 *
 */
AE_API int AE_Utils_Replace_Vars(const char *s, GWEN_BUFFER *dbuf, GWEN_DB_NODE *dbValues);


AE_API const AE_FIELD_DESCRIPTION *AE_Utils_GetItemFieldDescriptions();
AE_API const AE_FIELD_DESCRIPTION *AE_Utils_GetJobFieldDescriptions();


AE_API const AE_FIELD_DESCRIPTION *AE_Utils_GetItemFieldDescriptionByIndex(const AE_FIELD_DESCRIPTION *fd, int idx);
AE_API const AE_FIELD_DESCRIPTION *AE_Utils_GetItemFieldDescriptionByName(const AE_FIELD_DESCRIPTION *fd, const char *name);


#ifdef __cplusplus
}
#endif


#endif

