/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AE_MODULES_STATEMENTIMPORT_H
#define AE_MODULES_STATEMENTIMPORT_H


#include <aqfinance/engine/book/ae_book.h>
#include <aqfinance/engine/book/ae_statement.h>

#include <aqbanking/imexporter.h>
#include <aqbanking/transaction.h>


#ifdef __cplusplus
extern "C" {
#endif


AE_API
int AE_StatementImport_TransactionToStatement(const AB_TRANSACTION *t,
					      AE_STATEMENT *st);

AE_API
AB_ACCOUNT_STATUS *AE_StatementImport_GetBestBalance(AB_IMEXPORTER_ACCOUNTINFO *ai,
                                                     int noted);


AE_API
int AE_StatementImport_ImportTransactions(AE_BOOK *b,
                                          AB_IMEXPORTER_ACCOUNTINFO *iea,
                                          AE_BOOK_TABLE_TYPE tt,
                                          AQDB_ID bankAccountId,
                                          uint32_t guiid);


AE_API
int AE_StatementImport_UpdateTransfers(AE_BOOK *b,
                                       AB_IMEXPORTER_ACCOUNTINFO *iea,
                                       AE_BOOK_TABLE_TYPE tt,
                                       AQDB_ID bankAccountId,
                                       uint32_t guiid);

AE_API
int AE_StatementImport_ImportStandingOrders(AE_BOOK *b,
                                            AB_IMEXPORTER_ACCOUNTINFO *iea,
                                            AE_BOOK_TABLE_TYPE tt,
                                            AQDB_ID bankAccountId,
                                            uint32_t guiid);


AE_API
int AE_StatementImport_ImportAccountInfo(AE_BOOK *b,
                                         AB_IMEXPORTER_ACCOUNTINFO *iea,
                                         AQDB_ID bankAccountId,
                                         uint32_t guiid);

AE_API
int AE_StatementImport_ImportContext(AE_BOOK *b,
				     AB_IMEXPORTER_CONTEXT *iec,
                                     uint32_t guiid);


/**
 * This function compares the transactions from the given AccountInfo structure against
 * already known statements from the database. New statements are added to the given list
 * while known statements are discarded.
 */
AE_API
int AE_StatementImport_ExtractNewStatements(AE_BOOK *b,
                                            AB_IMEXPORTER_ACCOUNTINFO *iea,
                                            AE_BOOK_TABLE_TYPE tt,
                                            AQDB_ID bankAccountId,
                                            AE_STATEMENT_LIST *newStatements,
                                            uint32_t guiid);

AE_API
int AE_StatementImport_GetAccountForCategory(AE_BOOK *b, const char *cat,
					     AE_ACCOUNT_TREE *at,
					     AQDB_ID commodityId,
					     AE_ACCOUNT **pAcc, int doLock);

enum {
  AE_StatementImport_FindPartner_ResultFound=0,
  AE_StatementImport_FindPartner_ResultByAccount,
  AE_StatementImport_FindPartner_ResultByIban,
  AE_StatementImport_FindPartner_ResultByName
};

AE_API
int AE_StatementImport_FindPartnerForStatement(const AE_STATEMENT *st,
					       const AE_PARTNER_LIST *pl,
					       AE_PARTNER **pPartner);

AE_API
int AE_StatementImport_GetPartnerForStatementNoCreate(AE_BOOK *b,
                                                      AB_BANKING *ab,
						      const AE_STATEMENT *st,
						      AE_COMMODITY_LIST *cl,
						      AE_GROUP_TREE *gt,
						      AE_PARTNER_LIST *pl,
						      AE_PARTNER **pPartner,
						      int doLock);

AE_API
int AE_StatementImport_GetPartnerForStatement(AE_BOOK *b,
                                              AB_BANKING *ab,
					      const AE_STATEMENT *st,
					      AE_COMMODITY_LIST *cl,
                                              AE_GROUP_TREE *gt,
					      AE_PARTNER_LIST *pl,
					      AE_PARTNER **pPartner,
					      int doLock);


AE_API
int AE_StatementImport_AssignPartnersToStatements(AE_BOOK *b,
                                                  AB_BANKING *ab,
						  AE_BOOK_TABLE_TYPE tt,
						  AE_STATEMENT_LIST *sl,
						  int doLock);


AE_API
int AE_StatementImport_GetPartnerForStatementSimple(AE_BOOK *b,
                                                    AB_BANKING *ab,
						    AE_BOOK_TABLE_TYPE tt,
						    const AE_STATEMENT *st,
						    AE_PARTNER **pPartner,
						    int doLock);

#ifdef __cplusplus
}
#endif


#endif
