/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AF_GRAPH_H
#define AF_GRAPH_H


#include <aqfinance/graphics/af_graphics.h>
#include <aqfinance/graphs/af_graph_dataset.h>
#include <aqfinance/graphs/af_graph_subgraph.h>
#include <aqfinance/graphs/af_graph_settings.h>

#include <gwenhywfar/db.h>



#ifdef __cplusplus
extern "C" {
#endif



typedef struct AF_GRAPH AF_GRAPH;
GWEN_LIST_FUNCTION_LIB_DEFS(AF_GRAPH, AF_Graph, AE_API)
GWEN_LIST2_FUNCTION_LIB_DEFS(AF_GRAPH, AF_Graph, AE_API)



#define AF_GRAPH_FLAGS_WITH_GRID 0x00000001


enum {
  AF_GRAPH_TYPE_UNKNOWN=-1,
  AF_GRAPH_TYPE_NONE=0,
  AF_GRAPH_TYPE_LINE,
  AF_GRAPH_TYPE_POINTS,
  AF_GRAPH_TYPE_BARS
};


/**
 * Takes over AF_GRAPHICS object (use AF_Graphics_Attach if you still need that object after destroying AF_GRAPH).
*/
AE_API AF_GRAPH *AF_Graph_new(AF_GRAPH_DATASET *ds, AF_GRAPHICS *graphics, int width, int height);
AE_API void AF_Graph_free(AF_GRAPH *gr);

AE_API int AF_Graph_Init_fromDb(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig);
AE_API int AF_Graph_Init(AF_GRAPH *gr, const AF_GRAPH_SETTINGS *gs);
AE_API int AF_Graph_Fini(AF_GRAPH *gr);

AE_API int AF_Graph_Draw(AF_GRAPH *gr);



#ifdef __cplusplus
}
#endif



#endif



