/***************************************************************************
 begin       : Sat Sep 08 2013
 copyright   : (C) 2013 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "globals.h"
#include "af_report2_l.h"
#include <gwenhywfar/text.h>

#include <aqfinance/engine/modules/ae_recontransfers.h>
#include <aqfinance/engine/modules/ae_hhd.h>

#include <aqfinance/graphics/af_graphics_cairo.h>

#include <aqfinance/graphs/af_graph_dataset.h>
#include <aqfinance/graphs/af_graph.h>

#include <aqfinance/report2/af_report2.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>



static int graphReport(AB_BANKING *ab,
                       GWEN_DB_NODE *dbArgs,
                       int argc,
                       char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *iAccountPattern;
  const char *eAccountPattern;
  const char *outFile;
  const char *title;
  const char *subTitle;
  const char *s;
  int graphWidth;
  int graphHeight;
  int accumulate;
  uint32_t flags=0;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "outfile",                    /* name */
    1,                            /* minnum */
    1,                            /* maxnum */
    "o",                          /* short option */
    "outfile",                      /* long option */
    I18S("Specify the output file name (a PDF file)"),
    I18S("Specify the output file name (a PDF file)")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Int,            /* type */
    "graphWidth",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "w",                          /* short option */
    "graphWidth",                   /* long option */
    "Specify the graph width", /* short */
    "Specify the graph width" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Int,            /* type */
    "graphHeight",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "h",                          /* short option */
    "graphHeight",                /* long option */
    "Specify the graph height",   /* short */
    "Specify the graph height"    /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "title",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "t",                          /* short option */
    "title",                      /* long option */
    I18S("Specify the title for the graph"),
    I18S("Specify the title for the graph")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "subtitle",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "st",                          /* short option */
    "subtitle",                      /* long option */
    I18S("Specify the subtitle for the graph"),
    I18S("Specify the subtitle for the graph")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "iAccount",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "ia",                          /* short option */
    "income account",                    /* long option */
    "Specify the income account for which splits are wanted", /* short */
    "Specify the income account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "eAccount",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "ea",                          /* short option */
    "eaccount",                    /* long option */
    "Specify the expense account for which splits are wanted", /* short */
    "Specify the expense account for which splits are wanted" /* long */
  },
  {
    0,                            /* flags */
    GWEN_ArgsType_Int,            /* type */
    "accumulate",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "accumulate",                 /* long option */
    "Accumulate data",            /* short */
    "Accumulate data"             /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);
  outFile=GWEN_DB_GetCharValue(db, "outFile", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  iAccountPattern=GWEN_DB_GetCharValue(db, "iAccount", 0, 0);
  eAccountPattern=GWEN_DB_GetCharValue(db, "eAccount", 0, 0);

  graphWidth=GWEN_DB_GetIntValue(db, "graphWidth", 0, 1024);
  graphHeight=GWEN_DB_GetIntValue(db, "graphHeight", 0, 800);

  accumulate=GWEN_DB_GetIntValue(db, "accumulate", 0, 0);

  title=GWEN_DB_GetCharValue(db, "title", 0, NULL);
  subTitle=GWEN_DB_GetCharValue(db, "subTitle", 0, NULL);

  if (accumulate)
    flags|=AF_REPORT2_FLAGS_ACCUMULATE;

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Png_new(outFile, graphWidth, graphHeight);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_REPORT2 *rp;
      AF_REPORT2_CATEGORY_TREE *catTree;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, title);
      AF_Graph_DataSet_SetSubTitle(ds, subTitle);

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
      AF_Graph_DataSet_SetLabelX(ds, "Date");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      /* graph 1 */
      if (iAccountPattern) {
        rp=AF_Report2_new();
        catTree=AF_Report2_GetCategoryTree(rp);
        rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
        if (rv<0) {
          DBG_ERROR(0, "Error opening book (%d)", rv);
          return 4;
        }

        rv=AE_Book_Report2_ReadMatchingSplits(b, rp, fromDate, toDate, iAccountPattern, "EUR");
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
          return 4;
        }
        AF_Report2_CollapseAllCategories(rp);
        AF_Report2_AddMonthData2DataSet(rp, ds, flags|AF_REPORT2_FLAGS_SUM_TOTAL, iAccountPattern,
                                        1, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0x00ff0000, 1, 0);
      }

      if (eAccountPattern) {
        /* graph 2 */
        rp=AF_Report2_new();
        catTree=AF_Report2_GetCategoryTree(rp);
        rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
          return 4;
        }

        rv=AE_Book_Report2_ReadMatchingSplits(b, rp, fromDate, toDate, eAccountPattern, "EUR");
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
          return 4;
        }
        AF_Report2_CollapseAllCategories(rp);
        AF_Report2_AddMonthData2DataSet(rp, ds, flags|AF_REPORT2_FLAGS_SUM_TOTAL|AF_REPORT2_FLAGS_REVERSE_SIGN, eAccountPattern,
                                        2, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0xff000000, 1, 0);
      }


      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 1);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffffff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 9);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, graphWidth, graphHeight);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}








