/***************************************************************************
 begin       : Sat Apr 03 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "globals.h"
#include <gwenhywfar/text.h>

#include <aqfinance/engine/modules/ae_recontransfers.h>
#include <aqfinance/report/af_report_data.h>
#include <aqfinance/report/af_report_axis.h>
#include <aqfinance/report/af_report.h>

#include <aqfinance/report/csv.h>
#include <aqfinance/report/af_report_cfg_csv.h>

#include <aqfinance/report/htmlbase.h>
#include <aqfinance/report/af_report_cfg_htmlbase.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>






static int dbHtmlSplitReport(AB_BANKING *ab,
			     GWEN_DB_NODE *dbArgs,
			     int argc,
			     char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  AE_SPLIT_LIST *sl;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *commodityCode;
  const char *outFile;
  uint32_t reportFlags;
  const char *s;
  GWEN_SYNCIO *sio;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "commodity",                  /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "c",                          /* short option */
    "commodity",                  /* long option */
    "Specify the commodity for which splits are wanted", /* short */
    "Specify the commodity for which splits are wanted"  /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "output",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "o",                          /* short option */
    "output",                     /* long option */
    "Specify the output file",    /* short */
    "Specify the output file"     /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "accountType",                /* name */
    0,                            /* minnum */
    99,                            /* maxnum */
    "t",                          /* short option */
    "atype",                      /* long option */
    "Specify the account types which should be listed in the report", /* short */
    "Specify the account types which should be listed in the report"  /* long */
  },
  {
    0,                            /* flags */
    GWEN_ArgsType_Int,            /* type */
    "sumParents",                 /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "p",                          /* short option */
    "sumparents",                 /* long option */
    "Sum up categories into parent categories", /* short */
    "Sum up categories into parent categories"  /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);
  commodityCode=GWEN_DB_GetCharValue(db, "commodity", 0, "EUR");
  outFile=GWEN_DB_GetCharValue(db, "output", 0, NULL);

  s=GWEN_DB_GetCharValue(db, "accountType", 0, NULL);
  if (!(s && *s))
    reportFlags=
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_ASSETS |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_LIABILITIES |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_INCOME |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_EXPENSE |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_EQUITY;
  else {
    int i;

    reportFlags=0;
    for (i=0; i<99; i++) {
      s=GWEN_DB_GetCharValue(db, "accountType", i, NULL);
      if (s==NULL)
	break;
      if (strcasecmp(s, "assets")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_ASSETS;
      else if (strcasecmp(s, "liabilities")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_LIABILITIES;
      else if (strcasecmp(s, "income")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_INCOME;
      else if (strcasecmp(s, "expense")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_EXPENSE;
      else if (strcasecmp(s, "equity")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_EQUITY;
    }
  }

  if (GWEN_DB_GetIntValue(db, "sumParents", 0, 0))
    reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_SUM_PARENTS;

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }


  sl=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndAccount(b, fromDate, toDate, accountPattern, commodityCode, sl);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to read splits (%d)\n", rv);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    AB_Banking_Fini(ab);
    return rv;
  }

  if (outFile==NULL) {
    sio=GWEN_SyncIo_File_fromStdout();
    assert(sio);
  }
  else {
    sio=GWEN_SyncIo_File_new(outFile, GWEN_SyncIo_File_CreationMode_CreateAlways);
    GWEN_SyncIo_AddFlags(sio,
			 GWEN_SYNCIO_FILE_FLAGS_READ |
			 GWEN_SYNCIO_FILE_FLAGS_WRITE |
			 GWEN_SYNCIO_FILE_FLAGS_UREAD |
			 GWEN_SYNCIO_FILE_FLAGS_UWRITE |
			 GWEN_SYNCIO_FILE_FLAGS_GREAD |
			 GWEN_SYNCIO_FILE_FLAGS_GWRITE);
    rv=GWEN_SyncIo_Connect(sio);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sio);
      AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
      AE_Book_free(b);
      AB_Banking_Fini(ab);
      return rv;
    }
  }

  rv=AF_Report_BuildHtmlSplitReport(b, "AqFinance Report", sl, reportFlags, sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_free(sio);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    AB_Banking_Fini(ab);
    return rv;
  }

  /* done, close SIO */
  if (outFile!=NULL) {
    rv=GWEN_SyncIo_Disconnect(sio);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sio);
      return rv;
    }
  }
  GWEN_SyncIo_free(sio);


  rv=AE_Book_Close(b, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error closing book (%d)", rv);
    return 4;
  }

  rv=AB_Banking_Fini(ab);
  if (rv) {
    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
    return 5;
  }

  return 0;
}







