/***************************************************************************
 begin       : Sat Apr 03 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <gwenhywfar/logger.h>
#include <gwenhywfar/db.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/cgui.h>

#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/logger.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/directory.h>
#include <aqbanking/banking.h>
#include <aqbanking/abgui.h>

#include "globals.h"


#include "opendb.c"
#include "exportstm.c"

#include "dbinit.c"
#include "toxml.c"
#include "dbrecon.c"
#include "dblisttrans.c"
#include "dblisttransfers.c"
#include "dbimport.c"
#include "htmlsplitrep.c"
#include "graphreport.c"
#include "graphreport2.c"
#include "update.c"

#include "util.c"

#include "test.c"




int main(int argc, char **argv) {
  GWEN_DB_NODE *db;
  const char *cmd;
  int rv;
  AB_BANKING *ab;
  GWEN_GUI *gui;
  int nonInteractive=0;
  const char *pinFile;
  const GWEN_ARGS args[]={
  {
    0,                            /* flags */
    GWEN_ArgsType_Int,            /* type */
    "nonInteractive",             /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "n",                          /* short option */
    "noninteractive",             /* long option */
    "Select non-interactive mode",/* short description */
    "Select non-interactive mode.\n"        /* long description */
    "This automatically returns a confirmative answer to any non-critical\n"
    "message."
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "pinfile",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "P",                          /* short option */
    "pinfile",                    /* long option */
    "Specify the PIN file",       /* short description */
    "Specify the PIN file"        /* long description */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,            /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",
    I18S("Show this help screen. For help on commands, "
	 "run aqfinance-cli <COMMAND> --help."),
    I18S("Show this help screen. For help on commands, run aqfinance-cli <COMMAND> --help.")
  }
  };

  rv=GWEN_Init();
  if (rv) {
    fprintf(stderr, "ERROR: Unable to init Gwen.\n");
    exit(2);
  }

  rv=AQDB_Init();
  if (rv) {
    fprintf(stderr, "ERROR: Unable to init AqDatabase.\n");
    exit(2);
  }

  AE_Init(NULL);

  GWEN_Logger_Open(0, "aqfinance-cli", 0,
		   GWEN_LoggerType_Console,
		   GWEN_LoggerFacility_User);
  GWEN_Logger_SetLevel(0, GWEN_LoggerLevel_Warning);

  rv=GWEN_I18N_BindTextDomain_Dir(PACKAGE, LOCALEDIR);
  if (rv) {
    DBG_ERROR(0, "Could not bind textdomain (%d)", rv);
  }
  else {
    rv=GWEN_I18N_BindTextDomain_Codeset(PACKAGE, "UTF-8");
    if (rv) {
      DBG_ERROR(0, "Could not set codeset (%d)", rv);
    }
  }

  db=GWEN_DB_Group_new("arguments");
  rv=GWEN_Args_Check(argc, argv, 1,
		     GWEN_ARGS_MODE_ALLOW_FREEPARAM |
		     GWEN_ARGS_MODE_STOP_AT_FREEPARAM,
		     args,
		     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments main\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    GWEN_Buffer_AppendString(ubuf,
                             I18N("Usage: "));
    GWEN_Buffer_AppendString(ubuf, argv[0]);
    GWEN_Buffer_AppendString(ubuf,
                             I18N(" [GLOBAL OPTIONS] COMMAND "
                                  "[LOCAL OPTIONS]\n"));
    GWEN_Buffer_AppendString(ubuf,
                             I18N("\nGlobal Options:\n"));
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    GWEN_Buffer_AppendString(ubuf,
                             I18N("\nCommands:\n"));
    GWEN_Buffer_AppendString(ubuf, " dbinit\n");
    GWEN_Buffer_AppendString(ubuf,
			     I18N("   Initialize AqFinance database\n"));

    GWEN_Buffer_AppendString(ubuf, " toxml\n");
    GWEN_Buffer_AppendString(ubuf,
			     I18N("   Dump an AqFinance database to an XML file\n"));

    GWEN_Buffer_AppendString(ubuf, " dbrecon\n");
    GWEN_Buffer_AppendString(ubuf,
			     I18N("   Reconcile transfers using transactions (both from AqFinance database)\n"));

    GWEN_Buffer_AppendString(ubuf, " dblisttrans\n");
    GWEN_Buffer_AppendString(ubuf,
			     I18N("   Export transactions from the AqFinance database\n"));

    GWEN_Buffer_AppendString(ubuf, " dblisttransfers\n");
    GWEN_Buffer_AppendString(ubuf,
			     I18N("   Export transfers from the AqFinance database\n"));

    GWEN_Buffer_AppendString(ubuf, " dbimport\n");
    GWEN_Buffer_AppendString(ubuf,
                             I18N("   import data into the AqFinance database\n"));

    GWEN_Buffer_AppendString(ubuf, " dbHtmlSplitReport\n");
    GWEN_Buffer_AppendString(ubuf,
                             I18N("   Create a HTML split report using data from the AqFinance database\n"));

    GWEN_Buffer_AppendString(ubuf, " update\n");
    GWEN_Buffer_AppendString(ubuf,
                             I18N("   Update AqFinance)\n"));

    /* end of text */
    GWEN_Buffer_AppendString(ubuf, "\n");

    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }
  if (rv) {
    argc-=rv-1;
    argv+=rv-1;
  }

  nonInteractive=GWEN_DB_GetIntValue(db, "nonInteractive", 0, 0);

  cmd=GWEN_DB_GetCharValue(db, "params", 0, 0);
  if (!cmd) {
    fprintf(stderr, "ERROR: Command needed.\n");
    return 1;
  }

  gui=GWEN_Gui_CGui_new();
  GWEN_Gui_SetCharSet(gui, "ISO-8859-15");
  if (nonInteractive)
    GWEN_Gui_AddFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
  else
    GWEN_Gui_SubFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);

  pinFile=GWEN_DB_GetCharValue(db, "pinFile", 0, NULL);
  if (pinFile) {
    GWEN_DB_NODE *dbPins;

    dbPins=GWEN_DB_Group_new("pins");
    if (GWEN_DB_ReadFile(dbPins, pinFile,
			 GWEN_DB_FLAGS_DEFAULT |
			 GWEN_PATH_FLAGS_CREATE_GROUP)) {
      fprintf(stderr, "Error reading pinfile \"%s\"\n", pinFile);
      return 2;
    }
    GWEN_Gui_SetPasswordDb(gui, dbPins, 1);
  }
				    
  GWEN_Gui_SetGui(gui);

  ab=AB_Banking_new("aqfinance-cli", NULL, 0);
  AB_Gui_Extend(gui, ab);

  if (strcasecmp(cmd, "dbinit")==0) {
    rv=dbInit(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "toxml")==0) {
    rv=toXml(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dbrecon")==0) {
    rv=dbRecon(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dblisttrans")==0) {
    rv=dbListTrans(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dblisttransfers")==0) {
    rv=dbListTransfers(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dbimport")==0) {
    rv=dbImport(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dbhtmlsplitreport")==0) {
    rv=dbHtmlSplitReport(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dbgraphreport")==0) {
    rv=graphReport(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dbgraphreport2")==0) {
    rv=graphReport2(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "dbtest")==0) {
    rv=dbTest(ab, db, argc, argv);
  }
  else if (strcasecmp(cmd, "update")==0) {
    rv=update(ab, db, argc, argv);
  }
  else {
    fprintf(stderr, "ERROR: Unknown command \"%s\".\n", cmd);
    rv=1;
  }

  AQDB_Fini();
  GWEN_Fini();

  return rv;
}



