/***************************************************************************
 begin       : Sat Apr 03 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "globals.h"
#include <gwenhywfar/text.h>

#include "af_report2_l.h"

#include <aqfinance/engine/modules/ae_recontransfers.h>
#include <aqfinance/engine/modules/ae_hhd.h>
#include <aqfinance/report/af_report_data.h>
#include <aqfinance/report/af_report_axis.h>
#include <aqfinance/report/af_report.h>

#include <aqfinance/report/csv.h>
#include <aqfinance/report/af_report_cfg_csv.h>

#include <aqfinance/report/htmlbase.h>
#include <aqfinance/report/af_report_cfg_htmlbase.h>

#include <aqfinance/graphics/af_graphics_cairo.h>

#include <aqfinance/graphs/af_graph_dataset.h>
#include <aqfinance/graphs/af_graph.h>

#include <aqfinance/report2/af_report2.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>



#define GRAPH_TEST_WIDTH  1440
#define GRAPH_TEST_HEIGHT 1024

#define GRAPH_TEST_WIDTH_SMALL  800
#define GRAPH_TEST_HEIGHT_SMALL 200


#if 0
static int dbTest1(AB_BANKING *ab,
                   GWEN_DB_NODE *dbArgs,
                   int argc,
                   char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  AE_SPLIT_LIST *sl;
  AF_REPORT_DATA *rdata;
  AF_REPORT_CFG *cfg;
  AF_REPORT_CFG_COLUMN_LIST *cl;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }


  sl=AE_Split_List_new();
  rv=AE_Book_ReadSplitList(b, AE_Book_TableType_Split, sl);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to read splits (%d)\n", rv);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    AB_Banking_Fini(ab);
    return rv;
  }

  rdata=AF_Report_Data_new();
  AF_Report_Data_SetTitle(rdata, "AqFinance Report");


  cfg=AF_Report_Cfg_new();
  cl=AE_Split_CreateReportColumnList();
  AF_Report_Cfg_SetColumnList(cfg, cl);
  AE_Book_Report_CreateColumns(b, cfg, rdata);

  rv=AE_Book_ReadSplitListSubObjects(b, sl);
  if (rv<0) {
    DBG_ERROR(0, "Error building file (%d)", rv);
    return 4;
  }

  AE_Book_Report_AddSplitsToReportData(b, rdata, cfg, sl);

#if 1
  AE_Book_Report_ToFlatList(b, rdata);
  AF_Report_Data_SortDateByIdx(rdata, 0, 8);
#else
  DBG_ERROR(0, "Creating flat day list...");
  AE_Book_Report_ToFlatDayList(b, rdata,
                               AE_BOOK_REPORT_FLAG_ADD_POS_AMT |
                               AE_BOOK_REPORT_FLAG_ADD_NEG_AMT |
                               AE_BOOK_REPORT_FLAG_ADD_SUM_AMT |
                               0);
#endif
  //AF_Report_Data_SortDateByIdx(rdata, 1, 0);

  if (1) {
    GWEN_XMLNODE *n;
    GWEN_XMLNODE *nn;

    n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "root");
    nn=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "report");
    GWEN_XMLNode_AddChild(n, nn);
    AF_Report_Data_toXml(rdata, nn);
    GWEN_XMLNode_WriteFile(n, "/tmp/test-splits.xml", GWEN_XML_FLAGS_SIMPLE | GWEN_XML_FLAGS_INDENT);
    GWEN_XMLNode_free(n);
  }

  if (1) {
    int rv;
    AF_REPORT_CFG_CSV *cfg;

    cfg=AF_Report_Cfg_Csv_new();
    AF_Report_Cfg_Csv_AddFlags(cfg, AF_REPORT_CFG_CSV_FLAG_VALUE_LOCAL);

    rv=AF_Report_Builder_Csv("/tmp/test-splits.csv", rdata, cfg);
    if (rv<0) {
      DBG_ERROR(0, "Error building file (%d)", rv);
      return 4;
    }
  }

  if (1) {
    int rv;
    AF_REPORT_CFG_HTMLBASE *cfg;

    DBG_ERROR(0, "Creating HTML report...");
    cfg=AF_Report_Cfg_HtmlBase_new();
    AF_Report_Cfg_HtmlBase_AddFlags(cfg,
                                    AF_REPORT_CFG_HTMLBASE_FLAG_VALUE_LOCAL |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_FULL_DOC |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_RED_NEGATIVES |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_SHOW_TITLE |
                                    0);
    AF_Report_Cfg_HtmlBase_SetBorderWidth(cfg, 1);
    //AF_Report_Cfg_HtmlBase_SetColumnWidth1(cfg, 400);

    rv=AF_Report_Builder_HtmlBase("/tmp/test-splits.html", rdata, cfg);
    if (rv<0) {
      DBG_ERROR(0, "Error building file (%d)", rv);
      return 4;
    }
    DBG_ERROR(0, "Done");
  }

  rv=AE_Book_Close(b, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error closing book (%d)", rv);
    return 4;
  }

  rv=AB_Banking_Fini(ab);
  if (rv) {
    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
    return 5;
  }

  return 0;
}



static int dbTest2(AB_BANKING *ab,
                   GWEN_DB_NODE *dbArgs,
                   int argc,
                   char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  AE_SPLIT_LIST *sl;
  AF_REPORT_DATA *rdata;
  AF_REPORT_CFG *cfg;
  AF_REPORT_CFG_COLUMN_LIST *cl;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *s;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }


  sl=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndAccount(b, fromDate, toDate, accountPattern, "EUR", sl);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to read splits (%d)\n", rv);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    AB_Banking_Fini(ab);
    return rv;
  }

  rdata=AF_Report_Data_new();
  AF_Report_Data_SetTitle(rdata, "AqFinance Report");


  cfg=AF_Report_Cfg_new();
  cl=AE_Split_CreateReportColumnList();
  AF_Report_Cfg_SetColumnList(cfg, cl);
  AE_Book_Report_CreateColumns(b, cfg, rdata);

  AE_Book_Report_AddSplitsToReportData(b, rdata, cfg, sl);

#if 1
  AE_Book_Report_ToFlatList(b, rdata);
  AF_Report_Data_SortDateByIdx(rdata, 0, 8);
#else
  DBG_ERROR(0, "Creating flat day list...");
  AE_Book_Report_ToFlatDayList(b, rdata,
                               AE_BOOK_REPORT_FLAG_ADD_POS_AMT |
                               AE_BOOK_REPORT_FLAG_ADD_NEG_AMT |
                               AE_BOOK_REPORT_FLAG_ADD_SUM_AMT |
                               0);
#endif
  if (1) {
    GWEN_XMLNODE *n;
    GWEN_XMLNODE *nn;

    n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "root");
    nn=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "report");
    GWEN_XMLNode_AddChild(n, nn);
    AF_Report_Data_toXml(rdata, nn);
    GWEN_XMLNode_WriteFile(n, "/tmp/test-splits.xml", GWEN_XML_FLAGS_SIMPLE | GWEN_XML_FLAGS_INDENT);
    GWEN_XMLNode_free(n);
  }

  if (1) {
    int rv;
    AF_REPORT_CFG_CSV *cfg;

    cfg=AF_Report_Cfg_Csv_new();
    AF_Report_Cfg_Csv_AddFlags(cfg, AF_REPORT_CFG_CSV_FLAG_VALUE_LOCAL);

    rv=AF_Report_Builder_Csv("/tmp/test-splits.csv", rdata, cfg);
    if (rv<0) {
      DBG_ERROR(0, "Error building file (%d)", rv);
      return 4;
    }
  }

  if (1) {
    int rv;
    AF_REPORT_CFG_HTMLBASE *cfg;

    DBG_ERROR(0, "Creating HTML report...");
    cfg=AF_Report_Cfg_HtmlBase_new();
    AF_Report_Cfg_HtmlBase_AddFlags(cfg,
                                    AF_REPORT_CFG_HTMLBASE_FLAG_VALUE_LOCAL |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_FULL_DOC |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_RED_NEGATIVES |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_SHOW_TITLE |
                                    0);
    AF_Report_Cfg_HtmlBase_SetBorderWidth(cfg, 1);
    //AF_Report_Cfg_HtmlBase_SetColumnWidth1(cfg, 400);

    rv=AF_Report_Builder_HtmlBase("/tmp/test-splits.html", rdata, cfg);
    if (rv<0) {
      DBG_ERROR(0, "Error building file (%d)", rv);
      return 4;
    }
    DBG_ERROR(0, "Done");
  }

  rv=AE_Book_Close(b, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error closing book (%d)", rv);
    return 4;
  }

  rv=AB_Banking_Fini(ab);
  if (rv) {
    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
    return 5;
  }

  return 0;
}
#endif




#if 0
static int dbTest3(AB_BANKING *ab,
                   GWEN_DB_NODE *dbArgs,
                   int argc,
                   char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  AE_SPLIT_LIST *sl;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *commodityCode;
  const char *outFile;
  uint32_t reportFlags;
  const char *s;
  GWEN_SYNCIO *sio;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "commodity",                  /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "c",                          /* short option */
    "commodity",                  /* long option */
    "Specify the commodity for which splits are wanted", /* short */
    "Specify the commodity for which splits are wanted"  /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "output",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "o",                          /* short option */
    "output",                     /* long option */
    "Specify the output file",    /* short */
    "Specify the output file"     /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "accountType",                /* name */
    0,                            /* minnum */
    99,                            /* maxnum */
    "t",                          /* short option */
    "atype",                      /* long option */
    "Specify the account types which should be listed in the report", /* short */
    "Specify the account types which should be listed in the report"  /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);
  commodityCode=GWEN_DB_GetCharValue(db, "commodity", 0, "EUR");
  outFile=GWEN_DB_GetCharValue(db, "output", 0, NULL);

  s=GWEN_DB_GetCharValue(db, "accountType", 0, NULL);
  if (!(s && *s))
    reportFlags=
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_ASSETS |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_LIABILITIES |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_INCOME |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_EXPENSE |
      AF_REPORT_HTMLSPLIT_FLAGS_WITH_EQUITY;
  else {
    int i;

    reportFlags=0;
    for (i=0; i<99; i++) {
      s=GWEN_DB_GetCharValue(db, "accountType", i, NULL);
      if (s==NULL)
	break;
      if (strcasecmp(s, "assets")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_ASSETS;
      else if (strcasecmp(s, "liabilities")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_LIABILITIES;
      else if (strcasecmp(s, "income")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_INCOME;
      else if (strcasecmp(s, "expense")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_EXPENSE;
      else if (strcasecmp(s, "equity")==0)
	reportFlags|=AF_REPORT_HTMLSPLIT_FLAGS_WITH_EQUITY;
    }
  }


  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }


  sl=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndAccount(b, fromDate, toDate, accountPattern, commodityCode, sl);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to read splits (%d)\n", rv);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    AB_Banking_Fini(ab);
    return rv;
  }

  if (outFile==NULL) {
    sio=GWEN_SyncIo_File_fromStdout();
    assert(sio);
  }
  else {
    sio=GWEN_SyncIo_File_new(outFile, GWEN_SyncIo_File_CreationMode_CreateAlways);
    GWEN_SyncIo_AddFlags(sio,
			 GWEN_SYNCIO_FILE_FLAGS_READ |
			 GWEN_SYNCIO_FILE_FLAGS_WRITE |
			 GWEN_SYNCIO_FILE_FLAGS_UREAD |
			 GWEN_SYNCIO_FILE_FLAGS_UWRITE |
			 GWEN_SYNCIO_FILE_FLAGS_GREAD |
			 GWEN_SYNCIO_FILE_FLAGS_GWRITE);
    rv=GWEN_SyncIo_Connect(sio);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sio);
      AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
      AE_Book_free(b);
      AB_Banking_Fini(ab);
      return rv;
    }
  }

  rv=AF_Report_BuildHtmlSplitReport(b, "AqFinance Report", sl, reportFlags, sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_free(sio);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    AB_Banking_Fini(ab);
    return rv;
  }

  /* done, close SIO */
  if (outFile!=NULL) {
    rv=GWEN_SyncIo_Disconnect(sio);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sio);
      return rv;
    }
  }
  GWEN_SyncIo_free(sio);


  rv=AE_Book_Close(b, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error closing book (%d)", rv);
    return 4;
  }

  rv=AB_Banking_Fini(ab);
  if (rv) {
    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
    return 5;
  }

  return 0;
}
#endif




#if 0
static int hhdTest1(AB_BANKING *ab,
                    GWEN_DB_NODE *dbArgs,
                    int argc,
                    char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *s;
  GWEN_BUFFER *tbuf;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "hhd",                      /* name */
    1,                            /* minnum */
    1,                            /* maxnum */
    "s",                          /* short option */
    "hhd",                      /* long option */
    I18S("Specify string to translate"),
    I18S("Specify string to translate")
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  s=GWEN_DB_GetCharValue(db, "hhd", 0, 0);
  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AE_HHD14_Translate(s, tbuf);
  if (rv<0) {
    fprintf(stderr, "Error translating hhd string (%d)\n", rv);
    return 2;
  }

  fprintf(stdout, "HHD translates to: [%s]\n", GWEN_Buffer_GetStart(tbuf));

  return 0;
}
#endif



static int graphicsTest1(AB_BANKING *ab,
                         GWEN_DB_NODE *dbArgs,
                         int argc,
                         char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", 800, 400);
    if (g) {
      int rv;
      int pen1;
      int pen2;
      int pen3;

      rv=AF_Graphics_PenCreate(g, 0x00ff0000, 1, 0);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_PenCreate(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }
      pen1=rv;

      rv=AF_Graphics_PenCreate(g, 0xff000000, 1, 0);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_PenCreate(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }
      pen2=rv;

      rv=AF_Graphics_PenCreate(g, 0x0000ff00, 1, 0);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_PenCreate(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }
      pen3=rv;

      rv=AF_Graphics_DrawLine(g, pen1, 100, 300, 700, 500);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_DrawLine(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }

      rv=AF_Graphics_DrawRect(g, pen2, 100, 200, 200, 100);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_DrawLine(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }

      rv=AF_Graphics_DrawFilledRect(g, pen3, 200, 300, 200, 50);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_DrawLine(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }

      rv=AF_Graphics_Finish(g);
      if (rv<0) {
        DBG_ERROR(0, "Error on AF_Graphics_Finish(): %d", rv);
        AF_Graphics_free(g);
        return 2;
      }


    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest1(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_GRAPH_DATAPAIR_LIST *dpList;
      int i, j;
      double v;
      GWEN_DATE *dt;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
      AF_Graph_DataSet_SetLabelX(ds, "Date");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      dpList=AF_Graph_DataSet_GetDataPairList(ds);
      if (dpList==NULL) {
	dpList=AF_Graph_DataPair_List_new();
	AF_Graph_DataSet_SetDataPairList(ds, dpList);
      }

      v=-100.0;
      dt=GWEN_Date_fromGregorian(2001, 01, 5);
      j=GWEN_Date_GetJulian(dt);
      for (i=0; i<5; i++) {
	AF_GRAPH_DATAPAIR *dp;

	dp=AF_Graph_DataPair_new();
	AF_Graph_DataPair_SetLevel(dp, 1);
	AF_Graph_DataPair_SetValueX(dp, j+i);
	AF_Graph_DataPair_SetValueY(dp, v);
	AF_Graph_DataPair_List_Add(dp, dpList);

	v+=100.0;
      }

      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 10);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest2(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_GRAPH_DATAPAIR_LIST *dpList;
      int i;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;
      double y, yStep;
      double x, xStep;
      int steps;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelX(ds, "Double");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      dpList=AF_Graph_DataSet_GetDataPairList(ds);
      if (dpList==NULL) {
	dpList=AF_Graph_DataPair_List_new();
	AF_Graph_DataSet_SetDataPairList(ds, dpList);
      }

      y=-1000.0;
      yStep=1000;
      x=-10.0;
      xStep=20;
      steps=5;
      for (i=0; i<steps; i++) {
	AF_GRAPH_DATAPAIR *dp;

	dp=AF_Graph_DataPair_new();
	AF_Graph_DataPair_SetLevel(dp, 1);
	AF_Graph_DataPair_SetValueX(dp, x);
	AF_Graph_DataPair_SetValueY(dp, y);
	AF_Graph_DataPair_List_Add(dp, dpList);

	x+=xStep;
	y+=yStep;
      }

      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 2);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 10);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest3(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH_SMALL, GRAPH_TEST_HEIGHT_SMALL);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_GRAPH_DATAPAIR_LIST *dpList;
      int i;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;
      double y, yStep;
      double x, xStep;
      int steps;

      ds=AF_Graph_DataSet_new();

#if 0
      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");
#endif

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Double);
      //AF_Graph_DataSet_SetLabelX(ds, "Double");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      //AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      dpList=AF_Graph_DataSet_GetDataPairList(ds);
      if (dpList==NULL) {
	dpList=AF_Graph_DataPair_List_new();
	AF_Graph_DataSet_SetDataPairList(ds, dpList);
      }

      y=-1000.0;
      yStep=1000;
      x=-10.0;
      xStep=20;
      steps=5;
      for (i=0; i<steps; i++) {
	AF_GRAPH_DATAPAIR *dp;

	dp=AF_Graph_DataPair_new();
	AF_Graph_DataPair_SetLevel(dp, 1);
	AF_Graph_DataPair_SetValueX(dp, x);
	AF_Graph_DataPair_SetValueY(dp, y);
	AF_Graph_DataPair_List_Add(dp, dpList);

	x+=xStep;
	y+=yStep;
      }

      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 2);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 10);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH_SMALL, GRAPH_TEST_HEIGHT_SMALL);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest4(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_GRAPH_DATAPAIR_LIST *dpList;
      int i;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;
      GWEN_DATE *dt;
      int j;
      double v;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
      AF_Graph_DataSet_SetLabelX(ds, "Date");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      AF_Graph_DataSet_AddSubGraph(ds, "Label", 1, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0x00ff0000, 1, 0);

      dpList=AF_Graph_DataSet_GetDataPairList(ds);
      if (dpList==NULL) {
	dpList=AF_Graph_DataPair_List_new();
	AF_Graph_DataSet_SetDataPairList(ds, dpList);
      }

      v=-100.0;
      dt=GWEN_Date_fromGregorian(2001, 01, 5);
      j=GWEN_Date_GetJulian(dt);
      for (i=0; i<32; i++) {
	AF_GRAPH_DATAPAIR *dp;

	dp=AF_Graph_DataPair_new();
	AF_Graph_DataPair_SetLevel(dp, 1);
	AF_Graph_DataPair_SetValueX(dp, j+i);
	AF_Graph_DataPair_SetValueY(dp, v);
	AF_Graph_DataPair_List_Add(dp, dpList);

	v+=100.0;
      }

      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 2);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 9);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest5(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *s;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_REPORT2 *rp;
      AF_REPORT2_CATEGORY_TREE *catTree;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
      AF_Graph_DataSet_SetLabelX(ds, "Date");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      rp=AF_Report2_new();
      catTree=AF_Report2_GetCategoryTree(rp);
      rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
      if (rv<0) {
        DBG_ERROR(0, "Error opening book (%d)", rv);
        return 4;
      }

      rv=AE_Book_Report2_ReadMatchingSplits(b, rp, fromDate, toDate, accountPattern, "EUR");
      if (rv<0) {
        DBG_ERROR(0, "Error opening book (%d)", rv);
        return 4;
      }
      AF_Report2_CollapseAllCategories(rp);
      AF_Report2_AddDayData2DataSet(rp, ds, AF_REPORT2_FLAGS_SUM_TOTAL, "Summe",
                                    1, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0x00ff0000, 1, 0);

      AF_Graph_DataSet_SortDataPairs(ds, 1);
      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 2);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 9);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest6(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *s;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_REPORT2 *rp;
      AF_REPORT2_CATEGORY_TREE *catTree;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
      AF_Graph_DataSet_SetLabelX(ds, "Date");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      rp=AF_Report2_new();
      catTree=AF_Report2_GetCategoryTree(rp);
      rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
      if (rv<0) {
        DBG_ERROR(0, "Error opening book (%d)", rv);
        return 4;
      }

      rv=AE_Book_Report2_ReadMatchingSplits(b, rp, fromDate, toDate, accountPattern, "EUR");
      if (rv<0) {
        DBG_ERROR(0, "Error opening book (%d)", rv);
        return 4;
      }
      AF_Report2_CollapseAllCategories(rp);
      AF_Report2_AddMonthData2DataSet(rp, ds, AF_REPORT2_FLAGS_SUM_TOTAL, "Summe",
                                      1, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0x00ff0000, 1, 0);

      AF_Graph_DataSet_SortDataPairs(ds, 1);
      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 2);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 9);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest7(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *s;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  if (1) {
    AF_GRAPHICS *g;

    g=AF_GraphicsCairo_Pdf_new("/tmp/test.pdf", GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
    if (g) {
      AF_GRAPH *gr;
      AF_GRAPH_DATASET *ds;
      AF_REPORT2 *rp;
      AF_REPORT2_CATEGORY_TREE *catTree;
      GWEN_DB_NODE *dbConfig;
      GWEN_DB_NODE *dbT;

      ds=AF_Graph_DataSet_new();

      AF_Graph_DataSet_SetTitle(ds, "Title");
      AF_Graph_DataSet_SetSubTitle(ds, "Subtitle");

      AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
      AF_Graph_DataSet_SetLabelX(ds, "Date");

      AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
      AF_Graph_DataSet_SetLabelY(ds, "EUR");
      AF_Graph_DataSet_SetPrecisionY(ds, 2);

      /* graph 1 */
      rp=AF_Report2_new();
      catTree=AF_Report2_GetCategoryTree(rp);
      rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
      if (rv<0) {
        DBG_ERROR(0, "Error opening book (%d)", rv);
        return 4;
      }

      rv=AE_Book_Report2_ReadMatchingSplits(b, rp, fromDate, toDate, "Ertraege*", "EUR");
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
        return 4;
      }
      AF_Report2_CollapseAllCategories(rp);
      AF_Report2_AddMonthData2DataSet(rp, ds, AF_REPORT2_FLAGS_SUM_TOTAL, "Einnahmen",
                                      1, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0x00ff0000, 1, 0);

      /* graph 2 */
      rp=AF_Report2_new();
      catTree=AF_Report2_GetCategoryTree(rp);
      rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
        return 4;
      }

      rv=AE_Book_Report2_ReadMatchingSplits(b, rp, fromDate, toDate, "Aufwendungen*", "EUR");
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
        return 4;
      }
      AF_Report2_CollapseAllCategories(rp);
      AF_Report2_AddMonthData2DataSet(rp, ds, AF_REPORT2_FLAGS_SUM_TOTAL|AF_REPORT2_FLAGS_REVERSE_SIGN, "Ausgaben",
                                      2, AF_GRAPH_TYPE_LINE, AF_Report2_DataMode_Direct, (int)0xff000000, 1, 0);

      AF_Graph_DataSet_SortDataPairs(ds, 1);
      AF_Graph_DataSet_AdjustXRange(ds);
      AF_Graph_DataSet_AdjustYRange(ds);
      AF_Graph_DataSet_AddTicks(ds);

      dbConfig=GWEN_DB_Group_new("config");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxisLable");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xff000000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penAxis");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x0000ff00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraph");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0x00ff0000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGraphBackground");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xffec5b00);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 0);

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "penGrid");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "frontColour", (int)0xc0c0c000);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "lineWidth", 1);
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "dashType", 2);



      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 20);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "bold");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontSubTitle");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 16);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "italic");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontAxisLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 12);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "fontTickLabel");
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSize", 9);
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontSlant", "none");
      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "fontWeight", "none");

      GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS, "flags", "grid");


      AF_Graphics_Attach(g);
      gr=AF_Graph_new(ds, g, GRAPH_TEST_WIDTH, GRAPH_TEST_HEIGHT);
      rv=AF_Graph_Init_fromDb(gr, dbConfig);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Init_fromDb(): %d", rv);
	return 2;
      }

      rv=AF_Graph_Draw(gr);
      if (rv<0) {
	DBG_ERROR(0, "Error on AF_Graph_Draw(): %d", rv);
	return 2;
      }

      AF_Graph_Fini(gr);
      AF_Graph_free(gr);

      AF_Graphics_Finish(g);

    }
    else {
      DBG_ERROR(0, "Could not create graphics context");
      AF_Graphics_free(g);
      return 2;
    }


    AF_Graphics_free(g);
  }

  return 0;
}



static int graphTest8(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *s;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  if (1) {
    AF_REPORT2_SETTINGS *settings;
    AF_GRAPH_SETTINGS *gs;
    AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
    AF_REPORT2_SETTINGS_GRAPH *sg;

    settings=AF_Report2_Settings_new();

    /* graph settings */
    AF_Report2_Settings_SetDefaults(settings);
    gs=AF_Report2_Settings_GetGraphSettings(settings);
    AF_Graph_Settings_SetFlags(gs, AF_GRAPH_FLAGS_WITH_GRID);

    AF_Report2_Settings_SetTitle(settings, "Title");
    AF_Report2_Settings_SetSubTitle(settings, "Subtitle");
    AF_Report2_Settings_SetLabelY(settings, "EUR");
    AF_Report2_Settings_SetCommodity(settings, "EUR");
    AF_Report2_Settings_SetPrecisionY(settings, 2);
    AF_Report2_Settings_SetFlags(settings, AF_REPORT2_FLAGS_SUM_TOTAL);
    AF_Report2_Settings_SetDateSpanType(settings, AF_Report2_DateSpanType_UserDefined);
    AF_Report2_Settings_SetFirstDate(settings, fromDate);
    AF_Report2_Settings_SetLastDate(settings, toDate);
    AF_Report2_Settings_SetWidth(settings, GRAPH_TEST_WIDTH);
    AF_Report2_Settings_SetHeight(settings, GRAPH_TEST_HEIGHT);
    AF_Report2_Settings_SetDateUnits(settings, AF_Report2_DateUnits_Months);

    /* graphs */
    sgList=AF_Report2_SettingsGraph_List_new();
    sg=AF_Report2_SettingsGraph_new();
    AF_Report2_SettingsGraph_SetLabel(sg, "Graph1");
    AF_Report2_SettingsGraph_SetGraphType(sg, AF_GRAPH_TYPE_LINE);
    AF_Report2_SettingsGraph_SetDataMode(sg, AF_Report2_DataMode_Direct);
    AF_Report2_SettingsGraph_SetColour(sg, 0xff000000);
    AF_Report2_SettingsGraph_SetLineWidth(sg, 1);
    AF_Report2_SettingsGraph_SetDashType(sg, 0);
    AF_Report2_SettingsGraph_SetFlags(sg, 0);
    AF_Report2_SettingsGraph_AddCategoryName(sg, accountPattern);
    AF_Report2_SettingsGraph_List_Add(sg, sgList);
    AF_Report2_Settings_SetGraphList(settings, sgList);

    rv=AF_Report2_CreateReport(b, settings, "/tmp/test2.png");
    if (rv<0) {
      DBG_ERROR(0, "Error creating report (%d)", rv);
      return 4;
    }

    AF_Report2_Settings_free(settings);
  }


  return 0;
}



static int graphTest9(AB_BANKING *ab,
                      GWEN_DB_NODE *dbArgs,
                      int argc,
                      char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  GWEN_DATE *fromDate=NULL;
  GWEN_DATE *toDate=NULL;
  const char *accountPattern;
  const char *s;
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "dburl",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "dburl",                      /* long option */
    I18S("Specify the database URL"),
    I18S("Specify the database URL")
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "fromDate",                   /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "fromdate",                   /* long option */
    "Specify the first date for which transactions are wanted (YYYYMMDD)", /* short */
    "Specify the first date for which transactions are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "toDate",                     /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    0,                            /* short option */
    "todate",                     /* long option */
    "Specify the last date for which splits are wanted (YYYYMMDD)", /* short */
    "Specify the last date for which splits are wanted (YYYYMMDD)" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,            /* type */
    "account",                    /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "a",                          /* short option */
    "account",                    /* long option */
    "Specify the account for which splits are wanted", /* short */
    "Specify the account for which splits are wanted" /* long */
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* fromDate */
  s=GWEN_DB_GetCharValue(db, "fromDate", 0, 0);
  if (s && *s) {
    fromDate=GWEN_Date_fromString(s);
    if (fromDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  /* toDate */
  s=GWEN_DB_GetCharValue(db, "toDate", 0, 0);
  if (s && *s) {
    toDate=GWEN_Date_fromString(s);
    if (toDate==0) {
      DBG_ERROR(0, "Invalid fromdate value \"%s\"", s);
      return 1;
    }
  }

  accountPattern=GWEN_DB_GetCharValue(db, "account", 0, 0);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  if (1) {
    AF_REPORT2_SETTINGS *settings;
    AF_GRAPH_SETTINGS *gs;
    AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
    AF_REPORT2_SETTINGS_GRAPH *sg;

    settings=AF_Report2_Settings_new();

    /* graph settings */
    AF_Report2_Settings_SetDefaults(settings);
    gs=AF_Report2_Settings_GetGraphSettings(settings);
    AF_Graph_Settings_SetFlags(gs, AF_GRAPH_FLAGS_WITH_GRID);

    AF_Report2_Settings_SetTitle(settings, "Title");
    AF_Report2_Settings_SetSubTitle(settings, "Subtitle");
    AF_Report2_Settings_SetLabelY(settings, "EUR");
    AF_Report2_Settings_SetCommodity(settings, "EUR");
    AF_Report2_Settings_SetPrecisionY(settings, 2);
    AF_Report2_Settings_SetFlags(settings, AF_REPORT2_FLAGS_SUM_TOTAL);
    AF_Report2_Settings_SetDateSpanType(settings, AF_Report2_DateSpanType_UserDefined);
    AF_Report2_Settings_SetFirstDate(settings, fromDate);
    AF_Report2_Settings_SetLastDate(settings, toDate);
    AF_Report2_Settings_SetWidth(settings, GRAPH_TEST_WIDTH);
    AF_Report2_Settings_SetHeight(settings, GRAPH_TEST_HEIGHT);
    AF_Report2_Settings_SetDateUnits(settings, AF_Report2_DateUnits_Months);

    /* graphs */
    sgList=AF_Report2_SettingsGraph_List_new();

    sg=AF_Report2_SettingsGraph_new();
    AF_Report2_SettingsGraph_SetLabel(sg, "Graph1");
    AF_Report2_SettingsGraph_SetGraphType(sg, AF_GRAPH_TYPE_LINE);
    AF_Report2_SettingsGraph_SetDataMode(sg, AF_Report2_DataMode_Direct);
    AF_Report2_SettingsGraph_SetColour(sg, 0xff000000);
    AF_Report2_SettingsGraph_SetLineWidth(sg, 1);
    AF_Report2_SettingsGraph_SetDashType(sg, 0);
    AF_Report2_SettingsGraph_SetFlags(sg, AF_REPORT2_FLAGS_REVERSE_SIGN | AF_REPORT2_FLAGS_SUM_TOTAL);
    AF_Report2_SettingsGraph_AddCategoryName(sg, "Aufwendungen*");
    AF_Report2_SettingsGraph_List_Add(sg, sgList);

    sg=AF_Report2_SettingsGraph_new();
    AF_Report2_SettingsGraph_SetLabel(sg, "Graph2");
    AF_Report2_SettingsGraph_SetGraphType(sg, AF_GRAPH_TYPE_LINE);
    AF_Report2_SettingsGraph_SetDataMode(sg, AF_Report2_DataMode_Direct);
    AF_Report2_SettingsGraph_SetColour(sg, 0x00ff0000);
    AF_Report2_SettingsGraph_SetLineWidth(sg, 1);
    AF_Report2_SettingsGraph_SetDashType(sg, 0);
    AF_Report2_SettingsGraph_SetFlags(sg, AF_REPORT2_FLAGS_SUM_TOTAL);
    AF_Report2_SettingsGraph_AddCategoryName(sg, "Ertraege*");
    AF_Report2_SettingsGraph_List_Add(sg, sgList);

    AF_Report2_Settings_SetGraphList(settings, sgList);

    rv=AF_Report2_CreateReport(b, settings, "/tmp/test2.png");
    if (rv<0) {
      DBG_ERROR(0, "Error creating report (%d)", rv);
      return 4;
    }

    AF_Report2_Settings_free(settings);
  }


  return 0;
}







static int dbTest(AB_BANKING *ab,
                  GWEN_DB_NODE *dbArgs,
                  int argc,
                  char **argv) {
  //return dbTest3(ab, dbArgs, argc, argv);
  //return hhdTest1(ab, dbArgs, argc, argv);
  //return graphicsTest1(ab, dbArgs, argc, argv);
  //return graphTest2(ab, dbArgs, argc, argv);
  //return graphTest3(ab, dbArgs, argc, argv);
  //return graphTest4(ab, dbArgs, argc, argv);
  //return graphTest5(ab, dbArgs, argc, argv);
  //return graphTest6(ab, dbArgs, argc, argv);
  //return graphTest7(ab, dbArgs, argc, argv);
  //return graphTest8(ab, dbArgs, argc, argv);
  return graphTest9(ab, dbArgs, argc, argv);
}




