/***************************************************************************
 begin       : Sat Jun 25 2011
 copyright   : (C) 2011 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "globals.h"
#include <gwenhywfar/text.h>

#include <aqfinance/engine/modules/ae_writexml.h>
#include <aqfinance/update/af_update.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>



static int update(AB_BANKING *ab,
                  GWEN_DB_NODE *dbArgs,
                  int argc,
                  char **argv) {
  GWEN_DB_NODE *db;
  int rv;
  AF_UPDATE *upd;
  AF_UPDATE_RELEASE_LIST *rl;
  const char *dataDir;
  char homebuf[256];
  const GWEN_ARGS args[]={
  {
    GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
    GWEN_ArgsType_Char,           /* type */
    "datadir",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    "D",                          /* short option */
    "datadir",                      /* long option */
    I18S("Specify the data folder"),
    I18S("Specify the data folder")
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,             /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",                       /* long option */
    "Show this help screen",      /* short description */
    "Show this help screen"       /* long description */
  }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dataDir=GWEN_DB_GetCharValue(db, "dataDir", 0, NULL);
  if (!(dataDir && *dataDir)) {
    /* get prefix folder */
    rv=GWEN_Directory_GetPrefixDirectory(homebuf, sizeof(homebuf)-1);
    if (rv<0) {
      fprintf(stderr, "ERROR: Unable to get prefix directory (%d)\n", rv);
      return 2;
    }
    else {
      GWEN_BUFFER *tbuf;
      char cwdbuf[256];
  
      homebuf[sizeof(homebuf)-1]=0;
      if (NULL==getcwd(cwdbuf, sizeof(cwdbuf)-1)) {
        fprintf(stderr, "ERROR: getcwd: %s\n", strerror(errno));
        return 1;
      }
      cwdbuf[sizeof(cwdbuf)-1]=0;
  
      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Buffer_AppendString(tbuf, homebuf);
      GWEN_Buffer_AppendString(tbuf,
                               GWEN_DIR_SEPARATOR_S
                               ".."                      /* initial */
                               GWEN_DIR_SEPARATOR_S
                               ".."                      /* program */
                               GWEN_DIR_SEPARATOR_S);
      if (chdir(GWEN_Buffer_GetStart(tbuf))) {
        fprintf(stderr, "ERROR: chdir(%s): %s\n", GWEN_Buffer_GetStart(tbuf), strerror(errno));
        GWEN_Buffer_free(tbuf);
        return 2;
      }
      GWEN_Buffer_free(tbuf);
  
      if (NULL==getcwd(homebuf, sizeof(homebuf)-1)) {
        fprintf(stderr, "ERROR: getcwd: %s", strerror(errno));
        return 2;
      }
      homebuf[sizeof(homebuf)-1]=0;
  
      if (chdir(cwdbuf)) {
        fprintf(stderr, "ERROR: chdir(%s): %s\n", cwdbuf, strerror(errno));
        return 2;
      }
    }
    dataDir=homebuf;
  }

  /* create update manager */
  upd=AF_Update_new(UPDATE_URL1, dataDir);
  rv=AF_Update_Init(upd);
  if (rv<0) {
    fprintf(stderr, "ERROR: Error on AF_Update_Init (%d)\n", rv);
    AF_Update_free(upd);
    return 2;
  }

  /* download release info */
  rl=AF_UpdateRelease_List_new();
  rv=AF_Update_DownloadNewReleaseInfos(upd, rl);
  if (rv<0 || (rv>0 && rv<200) || rv>=300) {
    fprintf(stderr, "ERROR: Error downloading new releases (%d)\n", rv);
    AF_Update_free(upd);
    return 2;
  }

  if (AF_UpdateRelease_List_GetCount(rl)==0) {
    fprintf(stderr, "No new releases.\n");
  }
  else {
    AF_UPDATE_RELEASE *r;

    r=AF_UpdateRelease_List_First(rl);
    while(r) {
      fprintf(stderr, "Newer release: %d.%d.%d.%d-%d\n",
              AF_UpdateRelease_GetVersionMajor(r),
              AF_UpdateRelease_GetVersionMinor(r),
              AF_UpdateRelease_GetVersionPatchLevel(r),
              AF_UpdateRelease_GetVersionBuild(r),
              AF_UpdateRelease_GetMaturityLevel(r));
      r=AF_UpdateRelease_List_Next(r);
    }

    r=AF_UpdateRelease_List_Last(rl);
    if (r) {
      /* download release */
      rv=AF_Update_InstallRelease(upd, r, 0);
      if (rv!=0) {
        fprintf(stderr, "ERROR: Error downloading release (%d)\n", rv);
        AF_Update_free(upd);
        return 3;
      }
    }
  }

  return 0;
}


