/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_about.hpp"
#include "ff_app.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_About) FF_AboutMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_About::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_About::onCmdCancel),
};


FXIMPLEMENT(FF_About, FXDialogBox, FF_AboutMap, ARRAYNUMBER(FF_AboutMap))




FF_About::FF_About(FF_App* a,
		   FXComposite *parent,
		   const FXString& title,
		   FXuint opts,
		   FXint x, FXint y, FXint w, FXint h,
		   FXint pl, FXint pr, FXint pt, FXint pb,
		   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_label=new FOX16_HtmlLabel(vf, "", LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label->setMinimumWidth(200);

  new FXSpring(vf, LAYOUT_FILL_Y);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  new FXButton(vf, I18N("Dismiss"), NULL, this, ID_ACCEPT,
	       BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_CENTER_X);
}



FF_About::FF_About()
:FXDialogBox()
,m_app(NULL)
{

}



FF_About::~FF_About() {
}



FXint FF_About::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+50;
}



void FF_About::init() {
  GWEN_DB_NODE *db;
  int w, h;
  FXString str;

  db=m_app->getDialogConfig("FF_About");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(480, 400);

  GWEN_DB_Group_free(db);

  str=FXStringFormat(I18N("<h2>AqFinance v%s</h2>"
                          "<p>(c) 2007-2013 Martin Preuss</p>"
			  "<p>AqFinance is a financial application based on double-entry "
			  "bookkeeping.</p>"
			  "<p>It uses <i>AqBanking</i> to perform online banking tasks.</p>"
			  "<p>Please visit <i>http://www2.aquamaniac.de/sites/aqfinance/index.php</i> "
			  "for more details.</p>"),
                     AF_VERSION_PACK);
  m_label->setText(str);
}



void FF_About::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_About", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



long FF_About::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_About::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



bool FF_About::about(FF_App* app,
		     FXComposite *parent,
		     const FXString &title) {
  FF_About dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.show();
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());
  dlg.init();

  dlgres=dlg.execute();
  dlg.fini();

  if (dlgres==0) {
    return false;
  }

  return false;
}

