/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_askupdate.hpp"
#include "ff_app.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_AskUpdate) FF_AskUpdateMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_AskUpdate::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_AskUpdate::onCmdCancel),
};


FXIMPLEMENT(FF_AskUpdate, FXDialogBox, FF_AskUpdateMap, ARRAYNUMBER(FF_AskUpdateMap))




FF_AskUpdate::FF_AskUpdate(FF_App* a,
			   FXComposite *parent,
			   const FXString& title,
			   const FXString& text,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr, FXint pt, FXint pb,
			   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;

  m_textBrowser=new FXText(vf, NULL, 0,
			   TEXT_READONLY | TEXT_WORDWRAP | LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_textBrowser->setText(text);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new FXButton(hf, I18N("Abort"), NULL, this, ID_CANCEL,
	       BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
  new FXButton(hf, I18N("Update"), NULL, this, ID_ACCEPT,
	       BUTTON_NORMAL | LAYOUT_RIGHT);
}



FF_AskUpdate::FF_AskUpdate()
:FXDialogBox()
,m_app(NULL)
{

}



FF_AskUpdate::~FF_AskUpdate() {
}



FXint FF_AskUpdate::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+50;
}



void FF_AskUpdate::init() {
  GWEN_DB_NODE *db;
  int w, h;
  FXString str;

  db=m_app->getDialogConfig("FF_AskUpdate");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(480, 400);

  GWEN_DB_Group_free(db);
}



void FF_AskUpdate::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_AskUpdate", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



long FF_AskUpdate::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_AskUpdate::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



bool FF_AskUpdate::askUpdate(FF_App* app,
			     FXComposite *parent,
			     const FXString &title,
			     const FXString& text) {
  FF_AskUpdate dlg(app, parent, title, text, DECOR_ALL);
  FXuint dlgres;

  dlg.show();
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());
  dlg.init();

  dlgres=dlg.execute();
  dlg.fini();

  if (dlgres==0) {
    return false;
  }

  return true;
}

