/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef MAINWINDOW_HPP
#define MAINWINDOW_HPP


#include <fx.h>

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>


class FF_App;
class FF_ModuleManager;
class FXXMenuContainer;
class FXXAction;
class FOX16_HtmlLabel;


class MainWindow: public FXMainWindow {
  FXDECLARE(MainWindow)

public:

  enum {
    ID_FILE_CREATE=FXMainWindow::ID_LAST,
    ID_FILE_OPEN,
    ID_FILE_FROMXML,
    ID_FILE_CLOSE,
    ID_FILE_QUIT,
    ID_FILE_RECENT1,
    ID_FILE_RECENT2,
    ID_FILE_RECENT3,
    ID_FILE_RECENT4,
    ID_FILE_RECENT5,
    ID_FILE_IMPORT,
    ID_FILE_IMPORTTRANSFERS,
    ID_FILE_IMPORTFROMQBM,
    ID_EDIT_CURRENCIES,
    ID_EDIT_TAXES,
    ID_EDIT_ITEMS,
    ID_EDIT_TRANSFER_TEMPLATES,
    ID_EDIT_DOCTMPL_INVOICE,
    ID_EDIT_DOCTMPL_REQUEST,
    ID_EDIT_DOCTMPL_OFFER,
    ID_EDIT_DOCTMPL_CONFIRMATION,
    ID_EDIT_DOCTMPL_DELIVERY,
    ID_EDIT_DOCTMPL_RELEASE,
    ID_EDIT_DOCTMPL_REMINDER,
    ID_EDIT_DOCTMPL_BUYORDER,
    ID_EDIT_DOCTMPL_SELLORDER,
    ID_EDIT_SETTINGS,
    ID_VIEW_REFRESH,
    ID_EXTRA_WRITEXML,
    ID_EXTRA_SESSIONS,
    ID_EXTRA_ONLINE_SETUP,
    ID_EXTRA_UPDATE,
    ID_EXTRA_PARTNERSTOTRANSFERS,
    ID_EXTRA_PARTNERSTOSTATEMENTS,
    ID_EXTRA_EXISTING_PINTAN,
    ID_EXTRA_EXISTING_CHIPCARD,
    ID_EXTRA_CREATE_KEYFILE,
    ID_EXTRA_LOAD_CELLPHONE,
    ID_DEBUG_1,
    ID_DEBUG_2,
    ID_DEBUG_3,
    ID_DEBUG_4,
    ID_HELP_ABOUT,
    ID_LAST
  };


  MainWindow(FF_App* a, const FXString& name,
             FXIcon *ic=NULL,
             FXIcon *mi=NULL,
             FXuint opts=DECOR_ALL,
             FXint x=0, FXint y=0, FXint w=0, FXint h=0,
             FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
             FXint hs=0,FXint vs=0);

  ~MainWindow();

  FXXMenuContainer *getMenuBar() const { return m_menuBar;};
  FXToolBar *getToolBar() const { return m_toolBar;};

  const FXString &getLastOpened() const { return m_lastOpened;};

  int openConfig();
  void closeConfig();

  void create();

  void dump(int indent);

  void layout();

  int openDb(const FXString &url);
  int createDb(const FXString &url);
  int closeDb();

  long onMenuFileCreate(FXObject*, FXSelector, void*);
  long onMenuFileOpen(FXObject*, FXSelector, void*);
  long onMenuFileFromXml(FXObject*, FXSelector, void*);
  long onMenuFileClose(FXObject*, FXSelector, void*);

  long onMenuFileImport(FXObject*, FXSelector, void*);
  long onMenuFileImportTransfers(FXObject*, FXSelector, void*);
  long onMenuFileImportFromQbm(FXObject*, FXSelector, void*);

  long onMenuFileQuit(FXObject*, FXSelector, void*);

  long onMenuFileRecent1(FXObject*, FXSelector, void*);
  long onMenuFileRecent2(FXObject*, FXSelector, void*);
  long onMenuFileRecent3(FXObject*, FXSelector, void*);
  long onMenuFileRecent4(FXObject*, FXSelector, void*);
  long onMenuFileRecent5(FXObject*, FXSelector, void*);

  long onMenuEditCurrencies(FXObject*, FXSelector, void*);
  long onMenuEditTaxes(FXObject*, FXSelector, void*);
  long onMenuEditItems(FXObject*, FXSelector, void*);
  long onMenuEditTransferItems(FXObject*, FXSelector, void*);

  long onMenuEditDocTmplInvoice(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplRequest(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplOffer(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplConfirmation(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplDelivery(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplRelease(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplReminder(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplBuyOrder(FXObject*, FXSelector, void*);
  long onMenuEditDocTmplSellOrder(FXObject*, FXSelector, void*);
  long onMenuEditSettings(FXObject*, FXSelector, void*);

  long onMenuViewRefresh(FXObject*, FXSelector, void*);

  long onMenuExtraWriteXml(FXObject*, FXSelector, void*);
  long onMenuExtraSessions(FXObject*, FXSelector, void*);
  long onMenuExtraOnlineSetup(FXObject*, FXSelector, void*);
  long onMenuExtraUpdate(FXObject*, FXSelector, void*);
  long onMenuExtraAssignPartners2Transfers(FXObject*, FXSelector, void*);
  long onMenuExtraAssignPartners2Statements(FXObject*, FXSelector, void*);

  long onMenuExtraHbciExistingPinTan(FXObject*, FXSelector, void*);
  long onMenuExtraHbciExistingChipCard(FXObject*, FXSelector, void*);
  long onMenuExtraHbciCreateKeyFile(FXObject*, FXSelector, void*);

  long onMenuExtraLoadCellPhone(FXObject*, FXSelector, void*);

  long onMenuDebug1(FXObject*, FXSelector, void*);
  long onMenuDebug2(FXObject*, FXSelector, void*);
  long onMenuDebug3(FXObject*, FXSelector, void*);
  long onMenuDebug4(FXObject*, FXSelector, void*);

  long onMenuHelpAbout(FXObject*, FXSelector, void*);

  FXbool close(FXbool notify=FALSE);

protected:
  MainWindow();

  void rebuildRecentFiles();
  void addRecentFile(const FXString &url);

  int openOrCreateTable(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt, uint32_t flags);
  void setBookOpen(bool b);

  int softwareUpdate();
  int _softwareUpdate(uint32_t pid);

  FF_App *m_app;
  FXSwitcher *m_switcher;
  FOX16_HtmlLabel *m_noBookWindow;
  FF_ModuleManager *m_moduleManager;

  GWEN_DB_NODE *m_mainConfig;

  FXToolTip *m_toolTip;

  FXXMenuContainer *m_menuBar;
  FXToolBar *m_toolBar;

  FXXMenuContainer *m_menuFile;
  FXXAction *m_actionFileCreate;
  FXXAction *m_actionFileFromXml;
  FXXAction *m_actionFileOpen;
  FXXAction *m_actionFileClose;
  FXXAction *m_actionFileQuit;
  FXXAction *m_actionFileImport;
  FXXAction *m_actionFileImportTransfers;
  FXXAction *m_actionFileImportFromQbm;

  FXXMenuContainer *m_menuFileRecent;
  FXXAction *m_actionFileRecent1;
  FXXAction *m_actionFileRecent2;
  FXXAction *m_actionFileRecent3;
  FXXAction *m_actionFileRecent4;
  FXXAction *m_actionFileRecent5;

  FXXMenuContainer *m_menuEdit;
  FXXAction *m_actionEditCurrencies;
  FXXAction *m_actionEditTaxes;
  FXXAction *m_actionEditItems;
  FXXAction *m_actionEditTransferTemplates;
  FXXAction *m_actionEditSettings;

  FXXMenuContainer *m_menuEditDocTmpl;
  FXXAction *m_actionEditDocTmplInvoice;
  FXXAction *m_actionEditDocTmplRequest;
  FXXAction *m_actionEditDocTmplOffer;
  FXXAction *m_actionEditDocTmplConfirmation;
  FXXAction *m_actionEditDocTmplDelivery;
  FXXAction *m_actionEditDocTmplRelease;
  FXXAction *m_actionEditDocTmplReminder;
  FXXAction *m_actionEditDocTmplBuyOrder;
  FXXAction *m_actionEditDocTmplSellOrder;

  FXXMenuContainer *m_menuView;
  FXXAction *m_actionViewRefresh;

  FXXMenuContainer *m_menuExtra;
  FXXMenuContainer *m_menuExtraDataBase;
  FXXAction *m_actionExtraWriteXml;
  FXXAction *m_actionExtraSessions;
  FXXAction *m_actionExtraOnlineSetup;
  FXXAction *m_actionExtraUpdate;
  FXXAction *m_actionExtraAssignPartners2Transfers;
  FXXAction *m_actionExtraAssignPartners2Statements;
  FXXAction *m_actionExtraLoadCellPhone;

  FXXMenuContainer *m_menuExtraHbci;
  FXXAction *m_actionExtraHbciExistingPinTan;
  FXXAction *m_actionExtraHbciExistingChipCard;
  FXXAction *m_actionExtraHbciCreateKeyFile;

  FXXAction *m_actionExtraPaypalNewUser;

  FXXMenuContainer *m_menuDebug;
  FXXAction *m_actionDebug1;
  FXXAction *m_actionDebug2;
  FXXAction *m_actionDebug3;
  FXXAction *m_actionDebug4;

  FXXMenuContainer *m_menuHelp;
  FXXAction *m_actionHelpAbout;

  FXString m_recentFiles[5];
  FXString m_lastOpened;
};


#endif

