/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_conflicthandler.hpp"
#include "ff_conflictwidget.hpp"
#include "ff_app.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


FXDEFMAP(FF_ConflictHandler) FF_ConflictHandlerMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_ConflictHandler::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_ConflictHandler::onCmdCancel),
};


FXIMPLEMENT(FF_ConflictHandler, FXDialogBox, FF_ConflictHandlerMap, ARRAYNUMBER(FF_ConflictHandlerMap))




FF_ConflictHandler::FF_ConflictHandler(AE_DBCONFLICT_LIST *clist,
				       FF_App* a,
				       const FXString& title,
				       const FXString& text,
				       FXuint opts,
				       FXint x, FXint y, FXint w, FXint h,
				       FXint pl, FXint pr, FXint pt, FXint pb,
				       FXint hs, FXint vs)
:FXDialogBox(a, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_conflictList(NULL){
  FXVerticalFrame *vf;
  FXVerticalFrame *vf2;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXScrollWindow *sw;
  AE_DBCONFLICT *cfl;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new FOX16_HtmlLabel(vf, text, LAYOUT_FILL_X);

  gb=new FXGroupBox(vf, I18N("Conflicts"),
		    LAYOUT_FILL_X | LAYOUT_FILL_Y |
		    FRAME_LINE);
  sw=new FXScrollWindow(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  vf2=new FXVerticalFrame(sw, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  cfl=AE_DbConflict_List_First(clist);
  while(cfl) {
    new FF_ConflictWidget(vf2, cfl, LAYOUT_FILL_X);
    cfl=AE_DbConflict_List_Next(cfl);
  }

  /* last line (OK / Cancel) */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_ConflictHandler::FF_ConflictHandler()
:FXDialogBox()
,m_conflictList(NULL) {
}



FF_ConflictHandler::~FF_ConflictHandler() {
}



void FF_ConflictHandler::init() {
  FF_App *app;
  GWEN_DB_NODE *db;
  int w, h;

  app=dynamic_cast<FF_App*>(getApp());
  db=app->getDialogConfig("FF_ConflictHandler");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight());

  GWEN_DB_Group_free(db);
}



void FF_ConflictHandler::fini() {
  FF_App *app;
  GWEN_DB_NODE *db;
  int rv;

  app=dynamic_cast<FF_App*>(getApp());
  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=app->setDialogConfig("FF_ConflictHandler", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



long FF_ConflictHandler::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_ConflictHandler::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_ConflictHandler::handleConflicts(FF_App* app,
					 const FXString &title,
					 const FXString &text,
					 AE_DBCONFLICT_LIST *clist) {
  FF_ConflictHandler dlg(clist, app, title, text, DECOR_ALL);
  FXuint dlgres;

  dlg.create();
  dlg.place(PLACEMENT_VISIBLE);

  dlg.init();
  dlg.flags|=FLAG_DIRTY;
  dlg.layout();
  dlg.show(PLACEMENT_VISIBLE);
  dlgres=dlg.execute();
  dlg.fini();
  if (dlgres==0) {
    return false;
  }

  return true;
}



