/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_datepicker.hpp"

#include <aqfoxext/FXXDatePicker.hpp>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_DatePicker) FF_DatePickerMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_DatePicker::ID_DATEPICKER, FF_DatePicker::onCmdDatePicker),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_DatePicker::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_DatePicker::onCmdCancel),
};


FXIMPLEMENT(FF_DatePicker, FXDialogBox, FF_DatePickerMap, ARRAYNUMBER(FF_DatePickerMap))



FF_DatePicker::FF_DatePicker(FXComposite *parent,
                             const FXString& title,
                             FXuint opts,
                             FXint x, FXint y, FXint w, FXint h,
                             FXint pl, FXint pr, FXint pt, FXint pb,
                             FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_datePicker(NULL) {
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;

  m_datePicker=new FXXDatePicker(vf,
                                 NULL, /* holiday picker */
                                 this, ID_DATEPICKER,
                                 LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new FXSpring(vf, LAYOUT_FILL_Y);
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);


}




FF_DatePicker::~FF_DatePicker() {
}




FXint FF_DatePicker::checkGui(FXbool quiet) {
  FXDate d;

  d=m_datePicker->getSelectedDate();
  if (d<FXDate(1900, 1, 1) || d>FXDate(2500, 12, 31)) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
			  I18N("Invalid date selected"));
    }
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}



FXint FF_DatePicker::toGui(const FXDate &d) {
  m_datePicker->setSelectedDate(d);
  return 0;
}



FXint FF_DatePicker::fromGui(FXDate &result) {
  FXDate d;

  d=m_datePicker->getSelectedDate();
  if (d<FXDate(1900, 1, 1) || d>FXDate(2500, 12, 31))
    return GWEN_ERROR_BAD_DATA;

  result=d;
  return 0;
}



void FF_DatePicker::create() {
  FXDialogBox::create();
  m_datePicker->create();
}



long FF_DatePicker::onCmdDatePicker(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_DatePicker::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_DatePicker::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}




FF_DatePicker::FF_DatePicker()
:FXDialogBox()
,m_datePicker(NULL) {
}




bool FF_DatePicker::pickDate(FXComposite *parent,
                             const FXString &title,
                             FXDate &result) {
  FF_DatePicker dlg(parent, title, DECOR_ALL | DECOR_SHRINKABLE);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.toGui(result);
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(result);
  if (res==0)
    return true;
  else
    return false;
}





