/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITACCOUNT_HPP
#define FF_EDITACCOUNT_HPP


#include <fx.h>

#include <aqfinance/engine/book/ae_account.h>


class FF_App;
class FF_AccountTree;
class FF_RuleView;
class FF_CommodityEdit;


class FF_EditAccount: public FXDialogBox {
  FXDECLARE(FF_EditAccount)

public:

  enum {
    ID_ACCOUNTTREE=FXDialogBox::ID_LAST,
    ID_TEXTFIELD_NAME,
    ID_TEXTFIELD_CODE,
    ID_TEXTFIELD_DESCR,
    ID_COMBO_TYPE,
    ID_TABBOOK,
    ID_RULEVIEW,
    ID_LAST
  };

  FF_EditAccount(FF_App* a,
                 FXComposite *parent,
                 const FXString& title,
                 FXuint opts=DECOR_TITLE|DECOR_BORDER,
                 FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                 FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
                 FXint hs=0, FXint vs=0);
  ~FF_EditAccount();

  static bool editAccount(FF_App* app,
                          FXComposite *parent,
                          const FXString &title,
                          AE_ACCOUNT *acc);

  void init();
  void fini();

  FXint checkGui(FXbool quiet);
  FXint toGui(const AE_ACCOUNT *acc);
  FXint fromGui(AE_ACCOUNT *acc);

  void create();

  long onInsertedRule(FXObject*, FXSelector, void *ptr);
  long onReplacedRule(FXObject*, FXSelector, void *ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

  FXint getDefaultHeight();

protected:
  FF_EditAccount();

  void reload();
  void loadRules(AE_RULE_LIST *rules);

  FF_App *m_app;
  FXTextField *m_nameEdit;
  FXTextField *m_codeEdit;
  FXTextField *m_descrEdit;

  FXComboBox *m_typeCombo;
  FXCheckButton *m_placeHolderCheck;

  FF_CommodityEdit *m_primaryCurrencyEdit;
  FF_CommodityEdit *m_secondaryCurrencyEdit;

  FXTabBook *m_tabBook;
  FF_AccountTree *m_accountTree;

  FF_RuleView *m_ruleView;

  AQDB_ID m_accountId;

};




#endif




