/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_editaccstmrule.hpp"
#include "ff_accountedit.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg




FF_EditAccountStatementRule::FF_EditAccountStatementRule(FF_App* a,
                                                         FXComposite *parent,
                                                         const FXString& title,
                                                         FXuint opts,
                                                         FXint x, FXint y, FXint w, FXint h,
                                                         FXint pl, FXint pr, FXint pt, FXint pb,
                                                         FXint hs, FXint vs)
:FF_EditStatementRule(a, parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_accountEdit(NULL) {
  FXHorizontalFrame *hf;

  new FXHorizontalSeparator(m_verticalFrame, LAYOUT_FILL_X | SEPARATOR_GROOVE);

  hf=new FXHorizontalFrame(m_verticalFrame, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Account"), NULL, LABEL_NORMAL);
  m_accountEdit=new FF_AccountEdit(hf, NULL, 0, LAYOUT_FILL_X);
}



FF_EditAccountStatementRule::FF_EditAccountStatementRule()
:FF_EditStatementRule()
,m_accountEdit(NULL) {
}



FF_EditAccountStatementRule::~FF_EditAccountStatementRule() {
}



FXint FF_EditAccountStatementRule::checkGui(FXbool quiet) {
  int rv;

  rv=FF_EditStatementRule::checkGui(quiet);
  if (rv<0)
    return rv;

  /* TODO: check account */
  if (m_accountEdit->getAccountId()==0) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
                          "%s",
			  I18N("Please select the destination account"));
      m_accountEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}



FXint FF_EditAccountStatementRule::toGui(const AE_RULE *r) {
  int rv;
  AQDB_ID aid;

  rv=FF_EditStatementRule::toGui(r);
  if (rv<0)
    return rv;

  /* set account */
  m_accountEdit->reload();

  aid=AE_Rule_GetOwnerId(r);
  if (aid)
    m_accountEdit->setAccountId(aid);

  return 0;
}



FXint FF_EditAccountStatementRule::fromGui(AE_RULE *r) {
  int rv;

  rv=FF_EditStatementRule::fromGui(r);
  if (rv<0)
    return rv;

  /* get account */
  AE_Rule_SetOwnerId(r, m_accountEdit->getAccountId());

  return 0;
}



bool FF_EditAccountStatementRule::editRule(FF_App* app,
                                           FXComposite *parent,
                                           const FXString &title,
                                           AE_RULE *r) {
  FF_EditAccountStatementRule dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.addDialogLine();
  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.toGui(r);

  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (dlgres==0)
    return false;

  dlg.fromGui(r);
  return true;
}


