/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editaddress.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditAddress) FF_EditAddressMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditAddress::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditAddress::onCmdCancel),

};


FXIMPLEMENT(FF_EditAddress, FXDialogBox, FF_EditAddressMap, ARRAYNUMBER(FF_EditAddressMap))



FF_EditAddress::FF_EditAddress(FF_App* a,
                                 FXComposite *parent,
                                 const FXString& title,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h,
                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_nameEdit(NULL)
,m_deptEdit(NULL)
,m_streetNumEdit(NULL)
,m_zipCodeEdit(NULL)
,m_cityEdit(NULL)
,m_stateEdit(NULL)
,m_regionEdit(NULL)
,m_countryEdit(NULL)
,m_pPhoneEdit(NULL)
,m_pMobilePhoneEdit(NULL)
,m_oPhoneEdit(NULL)
,m_oMobilePhoneEdit(NULL)
,m_faxEdit(NULL)
,m_emailEdit(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 40, this, ID_EDIT_NAME,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Department"), NULL, LABEL_NORMAL);
  m_deptEdit=new FXTextField(m, 40, this, ID_EDIT_DEPT,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Street/Nr"), NULL, LABEL_NORMAL);
  m_streetNumEdit=new FXTextField(m, 40, this, ID_EDIT_STREETNUM,
				  TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Zipcode"), NULL, LABEL_NORMAL);
  m_zipCodeEdit=new FXTextField(m, 16, this, ID_EDIT_ZIPCODE,
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("City"), NULL, LABEL_NORMAL);
  m_cityEdit=new FXTextField(m, 40, this, ID_EDIT_CITY,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("State"), NULL, LABEL_NORMAL);
  m_stateEdit=new FXTextField(m, 40, this, ID_EDIT_STATE,
			      TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Region"), NULL, LABEL_NORMAL);
  m_regionEdit=new FXTextField(m, 40, this, ID_EDIT_REGION,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Country"), NULL, LABEL_NORMAL);
  m_countryEdit=new FXTextField(m, 40, this, ID_EDIT_COUNTRY,
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Private Phone"), NULL, LABEL_NORMAL);
  m_pPhoneEdit=new FXTextField(m, 40, this, ID_EDIT_PPHONE,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Private Mobile Phone"), NULL, LABEL_NORMAL);
  m_pMobilePhoneEdit=new FXTextField(m, 40, this, ID_EDIT_PMOBILE,
				     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Office Phone"), NULL, LABEL_NORMAL);
  m_oPhoneEdit=new FXTextField(m, 40, this, ID_EDIT_OPHONE,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Office Mobile Phone"), NULL, LABEL_NORMAL);
  m_oMobilePhoneEdit=new FXTextField(m, 40, this, ID_EDIT_OMOBILE,
				     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Faxnumber"), NULL, LABEL_NORMAL);
  m_faxEdit=new FXTextField(m, 40, this, ID_EDIT_FAX,
			    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Email"), NULL, LABEL_NORMAL);
  m_emailEdit=new FXTextField(m, 40, this, ID_EDIT_EMAIL,
			      TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  m_billingCheck=new FXCheckButton(vf, I18N("Use for billing"), NULL, 0, CHECKBUTTON_NORMAL);
  m_deliveryCheck=new FXCheckButton(vf, I18N("Use for delivery"), NULL, 0, CHECKBUTTON_NORMAL);

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditAddress::FF_EditAddress()
:FXDialogBox()
,m_app(NULL)
,m_nameEdit(NULL)
,m_deptEdit(NULL)
,m_streetNumEdit(NULL)
,m_zipCodeEdit(NULL)
,m_cityEdit(NULL)
,m_stateEdit(NULL)
,m_regionEdit(NULL)
,m_countryEdit(NULL)
,m_pPhoneEdit(NULL)
,m_pMobilePhoneEdit(NULL)
,m_oPhoneEdit(NULL)
,m_oMobilePhoneEdit(NULL)
,m_faxEdit(NULL)
,m_emailEdit(NULL)
{

}




FF_EditAddress::~FF_EditAddress() {
}






int FF_EditAddress::checkGui(FXbool quiet) {
  FXString str;

  if (m_nameEdit->getText().simplify().empty() &&
      m_deptEdit->getText().simplify().empty() &&
      m_streetNumEdit->getText().simplify().empty() &&
      m_zipCodeEdit->getText().simplify().empty() &&
      m_cityEdit->getText().simplify().empty() &&
      m_stateEdit->getText().simplify().empty() &&
      m_regionEdit->getText().simplify().empty() &&
      m_countryEdit->getText().simplify().empty() &&
      m_pPhoneEdit->getText().simplify().empty() &&
      m_pMobilePhoneEdit->getText().simplify().empty() &&
      m_oPhoneEdit->getText().simplify().empty() &&
      m_oMobilePhoneEdit->getText().simplify().empty() &&
      m_faxEdit->getText().simplify().empty() &&
      m_emailEdit->getText().simplify().empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please fillout at least a single field."));
    }
    return GWEN_ERROR_NO_DATA;
  }

  return 0;
}



int FF_EditAddress::toGui(const AE_ADDRESS *addr) {
  const char *s;
  uint32_t flags;

  s=AE_Address_GetName(addr);
  if (s)
    m_nameEdit->setText(s);

  s=AE_Address_GetDepartment(addr);
  if (s)
    m_deptEdit->setText(s);

  s=AE_Address_GetStreetNum(addr);
  if (s)
    m_streetNumEdit->setText(s);

  s=AE_Address_GetZipCode(addr);
  if (s)
    m_zipCodeEdit->setText(s);

  s=AE_Address_GetCity(addr);
  if (s)
    m_cityEdit->setText(s);

  s=AE_Address_GetState(addr);
  if (s)
    m_stateEdit->setText(s);

  s=AE_Address_GetRegion(addr);
  if (s)
    m_regionEdit->setText(s);

  s=AE_Address_GetCountry(addr);
  if (s)
    m_countryEdit->setText(s);

  s=AE_Address_GetPrivatePhone(addr);
  if (s)
    m_pPhoneEdit->setText(s);

  s=AE_Address_GetPrivateMobilePhone(addr);
  if (s)
    m_pMobilePhoneEdit->setText(s);

  s=AE_Address_GetOfficePhone(addr);
  if (s)
    m_oPhoneEdit->setText(s);

  s=AE_Address_GetOfficeMobilePhone(addr);
  if (s)
    m_oMobilePhoneEdit->setText(s);

  s=AE_Address_GetFaxNumber(addr);
  if (s)
    m_faxEdit->setText(s);

  s=AE_Address_GetEmail(addr);
  if (s)
    m_emailEdit->setText(s);

  flags=AE_Address_GetFlags(addr);

  m_billingCheck->setCheck((flags & AE_ADDRESS_FLAGS_BILLING)?TRUE:FALSE);
  m_deliveryCheck->setCheck((flags & AE_ADDRESS_FLAGS_DELIVERY)?TRUE:FALSE);

  return 0;
}



int FF_EditAddress::fromGui(AE_ADDRESS *addr) {
  FXString str;
  uint32_t flags;

#define TXTFROMGUI(n, m)                   \
  str=n->getText().simplify();             \
  if (str.empty())                         \
    AE_Address_Set##m(addr, NULL);         \
  else                                     \
    AE_Address_Set##m(addr, str.text())

  TXTFROMGUI(m_nameEdit, Name);
  TXTFROMGUI(m_deptEdit, Department);
  TXTFROMGUI(m_streetNumEdit, StreetNum);
  TXTFROMGUI(m_zipCodeEdit, ZipCode);
  TXTFROMGUI(m_cityEdit, City);
  TXTFROMGUI(m_stateEdit, State);
  TXTFROMGUI(m_regionEdit, Region);
  TXTFROMGUI(m_countryEdit, Country);
  TXTFROMGUI(m_pPhoneEdit, PrivatePhone);
  TXTFROMGUI(m_pMobilePhoneEdit, PrivateMobilePhone);
  TXTFROMGUI(m_oPhoneEdit, OfficePhone);
  TXTFROMGUI(m_oMobilePhoneEdit, OfficeMobilePhone);
  TXTFROMGUI(m_faxEdit, FaxNumber);
  TXTFROMGUI(m_emailEdit, Email);

#undef TXTFROMGUI

  flags=0;
  if (m_billingCheck->getCheck()==TRUE)
    flags|=AE_ADDRESS_FLAGS_BILLING;
  if (m_deliveryCheck->getCheck()==TRUE)
    flags|=AE_ADDRESS_FLAGS_DELIVERY;
  AE_Address_SetFlags(addr, flags);

  return 0;
}



void FF_EditAddress::create() {
  FXDialogBox::create();
}



long FF_EditAddress::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditAddress::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditAddress::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditAddress::editAddress(FF_App* app,
				 FXComposite *parent,
				 const FXString &title,
				 AE_ADDRESS *addr) {
  FF_EditAddress dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(addr);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(addr);
  if (res==0)
    return true;

  return false;
}









