/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editbankaccount.hpp"
#include "ff_commodityedit.hpp"
#include "ff_app.hpp"
#include "ff_grouptree.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <ctype.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditBankAccount) FF_EditBankAccountMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditBankAccount::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditBankAccount::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccount::ID_TEXTFIELD_ACCTID,
	    FF_EditBankAccount::onCmdTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccount::ID_TEXTFIELD_BANKCODE,
	    FF_EditBankAccount::onCmdBankCode),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccount::ID_TEXTFIELD_BIC,
	    FF_EditBankAccount::onCmdTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccount::ID_TEXTFIELD_IBAN,
	    FF_EditBankAccount::onCmdTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccount::ID_TEXTFIELD_BANKNAME,
	    FF_EditBankAccount::onCmdTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccount::ID_TEXTFIELD_ACCOUNTNAME,
	    FF_EditBankAccount::onCmdTextField),

};


FXIMPLEMENT(FF_EditBankAccount, FXDialogBox, FF_EditBankAccountMap, ARRAYNUMBER(FF_EditBankAccountMap))




FF_EditBankAccount::FF_EditBankAccount(FF_App* a,
				       FXComposite *parent,
				       const FXString& title,
				       FXuint opts,
				       FXint x, FXint y, FXint w, FXint h,
				       FXint pl, FXint pr, FXint pt, FXint pb,
				       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_ownerNameEdit(NULL)
,m_countryCombo(NULL)
,m_bankNameEdit(NULL)
,m_accountNameEdit(NULL)
,m_acctIdEdit(NULL)
,m_bankCodeEdit(NULL)
,m_ibanEdit(NULL)
,m_bicEdit(NULL)
,m_currencyEdit(NULL)
,m_groupTree(NULL){
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXIcon *ic;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Owner Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_ownerNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_OWNERNAME,
                                  TEXTFIELD_NORMAL);

  new FXLabel(m, I18N("Country"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_countryCombo=new FXListBox(m, this, ID_COMBO_COUNTRY,
			       LISTBOX_NORMAL | FRAME_SUNKEN | FRAME_THICK);

  new FXLabel(m, I18N("Bank Code"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_bankCodeEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_BANKCODE,
				 TEXTFIELD_NORMAL);

  new FXLabel(m, I18N("Bank Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_bankNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_BANKNAME,
                                 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Account Number"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_acctIdEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_ACCTID,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Account Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_accountNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_ACCOUNTNAME,
                                    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("SWIFT BIC"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_bicEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_ACCTID,
			    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("IBAN"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_ibanEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_ACCTID,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* currency */
  new FXLabel(m, I18N("Currency"), NULL, LABEL_NORMAL);
  m_currencyEdit=new FF_CommodityEdit(m, NULL, 0, LAYOUT_FILL_X);

  if (opts & FF_EDITBANKACCOUNT_OPTS_SHOWGROUP) {
    gb=new FXGroupBox(vf, I18N("Account Group"),
		      LAYOUT_FILL_X | LAYOUT_FILL_Y |
		      FRAME_LINE);
    m_groupTree=new FF_GroupTree(gb, this, ID_GROUP,
				 LAYOUT_FILL_X | LAYOUT_FILL_Y |
				 FOLDINGLIST_SINGLESELECT |
				 FOLDINGLIST_SHOWS_LINES |
				 FOLDINGLIST_SHOWS_BOXES |
				 FOLDINGLIST_ROOT_BOXES);
    m_groupTree->appendHeader(I18N("Group Name"));
    m_groupTree->setHeaderSize(0, m_acctIdEdit->getDefaultWidth());

    ic=a->getIcon("minifolderopen");
    if (ic)
      m_groupTree->setIconOpen(ic);
    ic=a->getIcon("minifolder");
    if (ic)
      m_groupTree->setIconClosed(ic);
  }

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditBankAccount::FF_EditBankAccount()
:FXDialogBox()
,m_app(NULL)
,m_ownerNameEdit(NULL)
,m_countryCombo(NULL)
,m_acctIdEdit(NULL)
,m_bankCodeEdit(NULL)
,m_ibanEdit(NULL)
,m_bicEdit(NULL)
,m_currencyEdit(NULL)
,m_groupTree(NULL) {

}



FF_EditBankAccount::~FF_EditBankAccount() {
}



void FF_EditBankAccount::create() {
  FXDialogBox::create();
  m_acctIdEdit->create();
  m_bankCodeEdit->create();
  m_ibanEdit->create();
  m_bicEdit->create();
  if (m_groupTree)
    m_groupTree->create();
}



FXint FF_EditBankAccount::getDefaultHeight() {
  /* add minimum height of table to be usefull */
  if (options & FF_EDITBANKACCOUNT_OPTS_SHOWGROUP)
    return FXDialogBox::getDefaultHeight()+100;
  else
    return FXDialogBox::getDefaultHeight();
}



FXint FF_EditBankAccount::checkGui(FXbool quiet) {
  FXString acctId;
  FXString bankCode;
  FXString bic;
  FXString iban;
  FXString currency;
  AB_BANKING *ab;

  acctId=m_acctIdEdit->getText().simplify();
  bankCode=m_bankCodeEdit->getText().simplify();
  bic=m_bicEdit->getText().simplify();
  iban=m_ibanEdit->getText().simplify();
  currency=m_currencyEdit->getCurrency();

  if (acctId.empty() && bankCode.empty() &&
      iban.empty()) {
    if (!quiet) {
      DBG_ERROR(0, "Missing info");
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (currency.empty()) {
    DBG_ERROR(0, "Missing info");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Missing Input"),
			  "%s",
			  I18N("Please select a currency."));
    }
    return GWEN_ERROR_NO_DATA;
  }

  ab=m_app->getBanking();
  if (ab) {
    AB_BANKINFO_CHECKRESULT cr;
    FXString str;

    str=m_countryCombo->getItem(m_countryCombo->getCurrentItem());
    if (str.empty())
      str="de";
    cr=AB_Banking_CheckAccount(ab, str.text(), NULL, bankCode.text(), acctId.text());
    if (cr!=AB_BankInfoCheckResult_Ok) {
      if (!quiet) {
	if (cr==AB_BankInfoCheckResult_NotOk) {
	  if (FXMessageBox::warning(this, MBOX_YES_NO,
				    I18N("Invalid Combination"),
				    "%s",
				    I18N("The combination of bank code and account number is invalid\n"
					 "Do you still want to use it?"))!=FX::MBOX_CLICKED_YES) {
	    return GWEN_ERROR_BAD_DATA;
	  }
	}
	else if (cr==AB_BankInfoCheckResult_UnknownBank) {
	  if (FXMessageBox::warning(this, MBOX_YES_NO,
				    I18N("Unknown Bank"),
                                    "%s",
				    I18N("The bank code you entered is unknown.\n"
					 "Do you still want to use it?"))!=
	      FX::MBOX_CLICKED_YES) {
	    return GWEN_ERROR_BAD_DATA;
	  }
	}
	else {
	  if (FXMessageBox::warning(this, MBOX_YES_NO,
				    I18N("Unable to Verify"),
                                    "%s",
				    I18N("The combination of bank code and account number can not\n"
					 "be verified. The most likely reason is that this version of "
					 "AqFinance has been compiled without support for KtoBlzCheck.\n"
					 "Do you still want to use the entered bank account?"))!=
	      FX::MBOX_CLICKED_YES) {
	    return GWEN_ERROR_BAD_DATA;
	  }
	}
      }
    }
  }

  return 0;
}



FXint FF_EditBankAccount::toGui(const AE_BANKACCOUNT *ba) {
  const char *s;
  AE_BOOK *b;
  AB_BANKING *ab;
  int i;
  const char *baCountry="de";

  m_countryCombo->clearItems();
  m_countryCombo->setNumVisible(2);

  m_currencyEdit->reload();

  ab=m_app->getBanking();
  i=0;
  if (ab) {
    AB_COUNTRY_CONSTLIST2 *countries;

    baCountry=AE_BankAccount_GetCountry(ba);
    if (baCountry==NULL || *baCountry==0)
      baCountry="de";
    countries=AB_Banking_ListCountriesByName(ab, "*");
    if (countries) {
      AB_COUNTRY_CONSTLIST2_ITERATOR *it;

      it=AB_Country_ConstList2_First(countries);
      if (it) {
	const AB_COUNTRY *c;

	c=AB_Country_ConstList2Iterator_Data(it);
	while(c) {
	  s=AB_Country_GetCode(c);
	  if (s && *s)
	    m_countryCombo->appendItem(s);
	  c=AB_Country_ConstList2Iterator_Next(it);
	}
        AB_Country_ConstList2Iterator_free(it);
      }
      AB_Country_ConstList2_free(countries);
    }
  }

  i=m_countryCombo->getNumItems();
  if (i==0) {
    m_countryCombo->appendItem("de");
    m_countryCombo->appendItem("us");
    i=m_countryCombo->getNumItems();
  }
  if (i>10)
    i=10;
  m_countryCombo->setNumVisible(i);

  for (i=0; i<m_countryCombo->getNumItems(); i++) {
    FXString str;

    str=m_countryCombo->getItem(i);
    if (!(str.empty()) && strcasecmp(str.text(), baCountry)==0) {
      m_countryCombo->setCurrentItem(i);
      break;
    }
  }

  b=m_app->getBook();
  if (m_groupTree) {
    if (b) {
      AE_GROUP_TREE *gt;
      int rv;

      gt=AE_Group_Tree_new();
      rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_BankAccountGroup, gt);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	DBG_ERROR(0, "Adding %d groups", AE_Group_Tree_GetCount(gt));
	m_groupTree->addGroups(gt);
      }
      AE_Group_Tree_free(gt);
      m_groupTree->sortItems();
      m_groupTree->expandItems();
    }

    if (AE_BankAccount_GetGroupId(ba)) {
      FF_GroupTreeItem *item=NULL;

      item=m_groupTree->findGroupItem(AE_BankAccount_GetGroupId(ba));
      if (item)
	item->setSelected(true);
    }
  }

  s=AE_BankAccount_GetOwnerName(ba);
  if (s)
    m_ownerNameEdit->setText(FXString(s));

  s=AE_BankAccount_GetAccountName(ba);
  if (s)
    m_accountNameEdit->setText(FXString(s));

  s=AE_BankAccount_GetAccountNumber(ba);
  if (s)
    m_acctIdEdit->setText(FXString(s));

  s=AE_BankAccount_GetBankCode(ba);
  if (s)
    m_bankCodeEdit->setText(FXString(s));

  s=AE_BankAccount_GetBic(ba);
  if (s)
    m_bicEdit->setText(FXString(s));

  s=AE_BankAccount_GetIban(ba);
  if (s)
    m_ibanEdit->setText(FXString(s));

  s=AE_BankAccount_GetBankName(ba);
  if (s)
    m_bankNameEdit->setText(FXString(s));
  else
    updateBankName();

  s=AE_BankAccount_GetCurrency(ba);
  if (!s || !*s)
    /* default to EUR */
    s="EUR";
  m_currencyEdit->setCurrency(s);

  return 0;
}



FXint FF_EditBankAccount::fromGui(AE_BANKACCOUNT *ba) {
  FXString acctId;
  FXString bankCode;
  FXString bic;
  FXString iban;
  FXString bankName;
  FXString accountName;
  FXString currency;
  FXString ownerName;
  AE_GROUP *gr;

  acctId=m_acctIdEdit->getText().simplify();
  bankCode=m_bankCodeEdit->getText().simplify();
  bic=m_bicEdit->getText().simplify();
  iban=m_ibanEdit->getText().simplify();
  bankName=m_bankNameEdit->getText().simplify();
  accountName=m_accountNameEdit->getText().simplify();
  ownerName=m_ownerNameEdit->getText().simplify();

  if (acctId.empty() && bankCode.empty() &&
      iban.empty()) {
    DBG_ERROR(0, "Missing info");
    return GWEN_ERROR_NO_DATA;
  }

  if (ownerName.empty())
    AE_BankAccount_SetOwnerName(ba, NULL);
  else
    AE_BankAccount_SetOwnerName(ba, ownerName.text());

  if (acctId.empty())
    AE_BankAccount_SetAccountNumber(ba, NULL);
  else
    AE_BankAccount_SetAccountNumber(ba, acctId.text());

  if (bankCode.empty())
    AE_BankAccount_SetBankCode(ba, NULL);
  else
    AE_BankAccount_SetBankCode(ba, bankCode.text());

  if (iban.empty())
    AE_BankAccount_SetIban(ba, NULL);
  else
    AE_BankAccount_SetIban(ba, iban.text());

  if (bic.empty())
    AE_BankAccount_SetBic(ba, NULL);
  else
    AE_BankAccount_SetBic(ba, bic.text());

  if (bankName.empty())
    AE_BankAccount_SetBankName(ba, NULL);
  else
    AE_BankAccount_SetBankName(ba, bankName.text());

  if (accountName.empty())
    AE_BankAccount_SetAccountName(ba, NULL);
  else
    AE_BankAccount_SetAccountName(ba, accountName.text());

  currency=m_currencyEdit->getCurrency();
  if (currency.empty())
    AE_BankAccount_SetCurrency(ba, NULL);
  else
    AE_BankAccount_SetCurrency(ba, currency.text());

  if (m_groupTree) {
    gr=m_groupTree->getFirstSelectedGroup();
    if (gr) {
      DBG_ERROR(0, "Current group: %d", AE_Group_GetId(gr));
      AE_BankAccount_SetGroupId(ba, AE_Group_GetId(gr));
    }
    else {
      DBG_ERROR(0, "No current group");
      AE_BankAccount_SetGroupId(ba, 0);
    }
  }

  return 0;
}



long FF_EditBankAccount::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



void FF_EditBankAccount::updateBankName() {
  AB_BANKING *ab;

  ab=m_app->getBanking();
  if (ab) {
    AB_BANKINFO *bi;
    FXString country;
    FXString str;
    FXString str2;
    int i;

    country=m_countryCombo->getItem(m_countryCombo->getCurrentItem()).simplify().trim();
    if (country.empty())
      country="de";
    str=m_bankCodeEdit->getText().simplify().trim();
    for (i=0; i<str.count(); i++) {
      if (isdigit(str[i]))
	str2+=str[i];
    }

    bi=AB_Banking_GetBankInfo(ab,
			      country.text(),
			      NULL,
			      str2.text());
    if (bi) {
      const char *s;

      s=AB_BankInfo_GetBic(bi);
      if (s && *s)
	m_bicEdit->setText(FXString(s));

      s=AB_BankInfo_GetBankName(bi);
      if (s && *s)
	m_bankNameEdit->setText(FXString(s));

      AB_BankInfo_free(bi);
    }
  }
}



long FF_EditBankAccount::onCmdBankCode(FXObject*, FXSelector, void *ptr) {
  updateBankName();
  return 1;
}


long FF_EditBankAccount::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0)
    getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_EditBankAccount::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}




bool FF_EditBankAccount::editBankAccount(FF_App* app,
					 FXComposite *parent,
					 const FXString &title,
					 FXuint opts,
					 AE_BANKACCOUNT *ba) {
  FF_EditBankAccount dlg(app, parent, title, opts);
  FXuint dlgres;
  FXint res;

  dlg.create();

  res=dlg.toGui(ba);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0)
    return false;

  res=dlg.fromGui(ba);
  if (res!=0)
    return false;
  return true;
}





