/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_edititem.hpp"
#include "ff_app.hpp"
#include "ff_valueedit.hpp"
#include "ff_accountedit.hpp"
#include "ff_taxedit.hpp"

#include <aqfinance/engine/book/ae_group.h>
#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditItem) FF_EditItemMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditItem::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditItem::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditItem::ID_TEXTFIELD, FF_EditItem::onCmdTextField),
};


FXIMPLEMENT(FF_EditItem, FXDialogBox, FF_EditItemMap, ARRAYNUMBER(FF_EditItemMap))




FF_EditItem::FF_EditItem(FF_App* a,
			 FXComposite *parent,
			 const FXString& title,
			 FXuint opts,
			 FXint x, FXint y, FXint w, FXint h,
			 FXint pl, FXint pr, FXint pt, FXint pb,
			 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_codeEdit(NULL)
,m_nameEdit(NULL)
,m_costOfSalesAccountEdit(NULL)
,m_buyTaxEdit(NULL)
,m_buyValueEdit(NULL)
,m_buyAccountEdit(NULL)
,m_sellTaxEdit(NULL)
,m_sellValueEdit(NULL)
,m_sellAccountEdit(NULL)
,m_memoEdit(NULL)
,m_commodities(NULL)
,m_accountTree(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  FXHorizontalFrame *hf;
  FXGroupBox *gb;

  new FXLabel(m, I18N("FF_EditItem|Code"), NULL, LABEL_NORMAL);
  m_codeEdit=new FXTextField(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  m_codeEdit->setTipText(I18N("Test-Tooltip"));

  new FXLabel(m, I18N("FF_EditItem|Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("FF_EditItem|Amount"), NULL, LABEL_NORMAL);
  m_amountEdit=new FF_ValueEdit(m, 16, NULL, 0, LAYOUT_FILL_X);

  /* buy */
  gb=new FXGroupBox(vf, I18N("FF_EditItem|Buy"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("FF_EditItem|Tax"), NULL, LABEL_NORMAL);
  m_buyTaxEdit=new FF_TaxEdit(m, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("FF_EditItem|Price"), NULL, LABEL_NORMAL);
  m_buyValueEdit=new FF_ValueEdit(m, 16, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("FF_EditItem|Account"), NULL, LABEL_NORMAL);
  m_buyAccountEdit=new FF_AccountEdit(m, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
  m_buyAccountEdit->setTipText(I18N("Choose or create an asset account for wares bought"));

  /* sell */
  gb=new FXGroupBox(vf, I18N("FF_EditItem|Sell"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("FF_EditItem|Tax"), NULL, LABEL_NORMAL);
  m_sellTaxEdit=new FF_TaxEdit(m, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("FF_EditItem|Price"), NULL, LABEL_NORMAL);
  m_sellValueEdit=new FF_ValueEdit(m, 16, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("FF_EditItem|Account"), NULL, LABEL_NORMAL);
  m_sellAccountEdit=new FF_AccountEdit(m, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
  m_sellAccountEdit->setTipText(I18N("Choose or create an income account for wares sold"));

  new FXLabel(m, I18N("FF_EditItem|Cost of Sales Account"), NULL, LABEL_NORMAL);
  m_costOfSalesAccountEdit=new FF_AccountEdit(m, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
  m_costOfSalesAccountEdit->setTipText(I18N("Choose or create an expense account for costs of wares bought"));

  /* memo */
  gb=new FXGroupBox(vf, I18N("Memo"),
		    LAYOUT_FILL_X | LAYOUT_FILL_Y |
		    FRAME_LINE);
  m_memoEdit=new FXText(gb, NULL, 0,
			TEXT_SHOWACTIVE | TEXT_NO_TABS |
			LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* dialog line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditItem::FF_EditItem()
:FXDialogBox()
,m_app(NULL)
,m_codeEdit(NULL)
,m_nameEdit(NULL)
,m_costOfSalesAccountEdit(NULL)
,m_buyTaxEdit(NULL)
,m_buyValueEdit(NULL)
,m_buyAccountEdit(NULL)
,m_sellTaxEdit(NULL)
,m_sellValueEdit(NULL)
,m_sellAccountEdit(NULL)
,m_memoEdit(NULL)
,m_commodities(NULL)
,m_accountTree(NULL)
{

}



FF_EditItem::~FF_EditItem() {
  AE_Commodity_List_free(m_commodities);
  AE_Account_Tree_free(m_accountTree);
}



FXint FF_EditItem::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



void FF_EditItem::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditItem");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight());

  GWEN_DB_Group_free(db);
}



void FF_EditItem::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditItem", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_EditItem::checkGui(FXbool quiet) {
  FXString str;
  AQDB_ID id;
  AQDB_VALUE *v=NULL;
  int units;
  int rv;

  str=m_codeEdit->getText().simplify();
  if (str.empty()) {
    DBG_ERROR(0, "Empty name");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing Code"));
      m_codeEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }

  /* amount */
  str=m_amountEdit->getText().simplify();
  rv=AE_Utils_ValueAndItemUnitsFromString(str.text(), &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Invalid amount"));
      m_amountEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }
  else {
    AQDB_Value_free(v);
  }

  /* inventory account */
  if (!(m_costOfSalesAccountEdit->getText().simplify().empty())) {
    id=m_costOfSalesAccountEdit->getAccountId();
    if (id==0) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid inventory account."));
	m_costOfSalesAccountEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
  }

  /* buy tax */
  if (!(m_buyTaxEdit->getText().simplify().empty())) {
    id=m_buyTaxEdit->getTaxId();
    if (id==0) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid buy tax"));
	m_buyTaxEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
  }

  /* buy account */
  if (!(m_buyAccountEdit->getText().simplify().empty())) {
    id=m_buyAccountEdit->getAccountId();
    if (id==0) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid buy account."));
	m_buyAccountEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
  }

  /* buy value */
  str=m_buyValueEdit->getText().simplify();
  if (!(str.empty())) {
    AQDB_VALUE *v=NULL;
    AE_COMMODITY *co=NULL;
    int rv;

    rv=AE_Utils_AmountFromString(str.text(), m_commodities, &v, &co);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid value."));
	m_buyValueEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
    else {
      AQDB_Value_free(v);
      if (co==NULL) {
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Currency missing in amount."));
	  m_buyValueEdit->setFocus();
	}
	return GWEN_ERROR_INVALID;
      }
    }
  }


  /* sell tax */
  if (!(m_sellTaxEdit->getText().simplify().empty())) {
    id=m_sellTaxEdit->getTaxId();
    if (id==0) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid sell tax"));
	m_sellTaxEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
  }

  /* sell account */
  if (!(m_sellAccountEdit->getText().simplify().empty())) {
    id=m_sellAccountEdit->getAccountId();
    if (id==0) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid sell account."));
	m_sellAccountEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
  }

  /* sell value */
  str=m_sellValueEdit->getText().simplify();
  if (!(str.empty())) {
    AQDB_VALUE *v=NULL;
    AE_COMMODITY *co=NULL;
    int rv;

    rv=AE_Utils_AmountFromString(str.text(), m_commodities, &v, &co);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid value."));
	m_sellValueEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
    else {
      AQDB_Value_free(v);
      if (co==NULL) {
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Currency missing in amount."));
	  m_sellValueEdit->setFocus();
	}
	return GWEN_ERROR_INVALID;
      }
    }
  }

  return 0;
}



FXint FF_EditItem::toGui(const AE_ITEM *item, AE_BOOK_TABLE_TYPE tt) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    int rv;
    const char *s;
    const AQDB_VALUE *v;
    GWEN_BUFFER *tbuf;

    m_costOfSalesAccountEdit->reload();

    m_buyTaxEdit->reload();
    m_buyAccountEdit->reload();

    m_sellTaxEdit->reload();
    m_sellAccountEdit->reload();

    reload();

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    s=AE_Item_GetCode(item);
    if (s && *s)
      m_codeEdit->setText(FXString(s));

    s=AE_Item_GetName(item);
    if (s && *s)
      m_nameEdit->setText(FXString(s));

    /* amount */
    v=AE_Item_GetValue(item);
    if (v) {
      const char *units;

      units=AE_Utils_ItemUnitsToString(AE_Item_GetUnits(item));
      m_amountEdit->setValueAndUnits(v, units);
      m_amountEdit->selectAll();
    }

    m_costOfSalesAccountEdit->setAccountId(AE_Item_GetCostOfSalesAccountId(item));

    /* buy */
    m_buyTaxEdit->setTaxId(AE_Item_GetBuyTaxId(item));
    m_buyAccountEdit->setAccountId(AE_Item_GetBuyAccountId(item));

    v=AE_Item_GetBuyPriceValue(item);
    if (v) {
      rv=AE_Utils_AmountToString(v,
				 AE_Item_GetCommodityId(item),
				 m_commodities,
				 tbuf);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_sellValueEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	m_sellValueEdit->selectAll();
      }
      GWEN_Buffer_Reset(tbuf);
    }

    /* sell */
    m_sellTaxEdit->setTaxId(AE_Item_GetSellTaxId(item));
    m_sellAccountEdit->setAccountId(AE_Item_GetSellAccountId(item));

    v=AE_Item_GetSellPriceValue(item);
    if (v) {
      rv=AE_Utils_AmountToString(v,
				 AE_Item_GetCommodityId(item),
				 m_commodities,
				 tbuf);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_sellValueEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	m_sellValueEdit->selectAll();
      }
      GWEN_Buffer_Reset(tbuf);
    }

    /* memo */
    s=AE_Item_GetMemo(item);
    if (s && *s)
      m_memoEdit->setText(s);

    GWEN_Buffer_free(tbuf);
  }
  else {
    DBG_ERROR(0, "No book");
    return GWEN_ERROR_INTERNAL;
  }

  return 0;
}



FXint FF_EditItem::fromGui(AE_ITEM *item) {
  FXString str;
  AQDB_ID id;
  AQDB_VALUE *v=NULL;
  int units;
  int rv;

  str=m_codeEdit->getText().simplify();
  if (str.empty()) {
    DBG_ERROR(0, "Empty name");
    return GWEN_ERROR_INVALID;
  }
  AE_Item_SetCode(item, str.text());

  str=m_nameEdit->getText().simplify();
  if (str.empty())
    AE_Item_SetName(item, NULL);
  else
    AE_Item_SetName(item, str.text());

  /* amount */
  str=m_amountEdit->getText().simplify();
  rv=AE_Utils_ValueAndItemUnitsFromString(str.text(), &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else {
    AE_Item_SetValue(item, v);
    AE_Item_SetUnits(item, units);
    AQDB_Value_free(v);
  }

  id=m_costOfSalesAccountEdit->getAccountId();
  AE_Item_SetCostOfSalesAccountId(item, id);

  /* buy */
  id=m_buyTaxEdit->getTaxId();
  AE_Item_SetBuyTaxId(item, id);

  id=m_buyAccountEdit->getAccountId();
  AE_Item_SetBuyAccountId(item, id);

  str=m_buyValueEdit->getText().simplify();
  if (str.empty())
    AE_Item_SetBuyPriceValue(item, NULL);
  else {
    AQDB_VALUE *v=NULL;
    AE_COMMODITY *co=NULL;
    int rv;

    rv=AE_Utils_AmountFromString(str.text(), m_commodities, &v, &co);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      AQDB_ID coId=AE_Item_GetCommodityId(item);

      if (co)
	coId=AE_Commodity_GetId(co);

      AE_Item_SetBuyPriceValue(item, v);
      AQDB_Value_free(v);

      AE_Item_SetCommodityId(item, coId);
    }
  }

  /* sell */
  id=m_sellTaxEdit->getTaxId();
  AE_Item_SetSellTaxId(item, id);

  id=m_sellAccountEdit->getAccountId();
  AE_Item_SetSellAccountId(item, id);

  str=m_sellValueEdit->getText().simplify();
  if (str.empty())
    AE_Item_SetSellPriceValue(item, NULL);
  else {
    AQDB_VALUE *v=NULL;
    AE_COMMODITY *co=NULL;
    int rv;

    rv=AE_Utils_AmountFromString(str.text(), m_commodities, &v, &co);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      AQDB_ID coId=AE_Item_GetCommodityId(item);

      if (co)
	coId=AE_Commodity_GetId(co);

      AE_Item_SetSellPriceValue(item, v);
      AQDB_Value_free(v);

      AE_Item_SetCommodityId(item, coId);
    }
  }

  /* memo */
  str=m_memoEdit->getText().simplify();
  if (str.empty())
    AE_Item_SetMemo(item, NULL);
  else
    AE_Item_SetMemo(item, str.text());

  /* done */
  return 0;
}



long FF_EditItem::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_EditItem::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditItem::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



void FF_EditItem::reload() {
  FF_App *app;
  AE_BOOK *b;
  
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ACCOUNT_TREE *at;
    AE_COMMODITY_LIST *cList;
    int rv;

    /* read accounts */
    at=AE_Account_Tree_new();
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Account_Tree_free(at);
    }
    else {
      AE_Account_Tree_free(m_accountTree);
      m_accountTree=at;
    }

    /* read commodities */
    cList=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cList);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Commodity_List_free(cList);
    }
    else {
      AE_Commodity_List_free(m_commodities);
      m_commodities=cList;
    }
  }
}




bool FF_EditItem::editItem(FF_App* app,
			   FXComposite *parent,
			   const FXString &title,
			   AE_BOOK_TABLE_TYPE tt,
			   AE_ITEM *item) {
  FF_EditItem dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.place(PLACEMENT_VISIBLE);
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  dlg.init();
  res=dlg.toGui(item, tt);
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(item);
  if (res==0) {
    return true;
  }

  return false;
}





