/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITITEM_HPP
#define FF_EDITITEM_HPP


#include <fx.h>

#include <aqfinance/engine/book/ae_book.h>
#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_item.h>


class FF_App;
class FF_TaxEdit;
class FF_ValueEdit;
class FF_AccountEdit;


class FF_EditItem: public FXDialogBox {
  FXDECLARE(FF_EditItem)

public:

  enum {
    ID_COMBOBOX=FXDialogBox::ID_LAST,
    ID_TEXTFIELD,
    ID_LAST
  };

  FF_EditItem(FF_App* a,
	      FXComposite *parent,
	      const FXString& title,
	      FXuint opts=DECOR_TITLE|DECOR_BORDER,
	      FXint x=0, FXint y=0, FXint w=0, FXint h=0,
	      FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
	      FXint hs=0, FXint vs=0);
  ~FF_EditItem();

  FXint getDefaultHeight();

  static bool editItem(FF_App* app,
		       FXComposite *parent,
		       const FXString &title,
		       AE_BOOK_TABLE_TYPE tt,
		       AE_ITEM *item);

  void init();
  void fini();

  FXint checkGui(FXbool quiet);
  FXint toGui(const AE_ITEM *item, AE_BOOK_TABLE_TYPE tt);
  FXint fromGui(AE_ITEM *item);

  long onCmdTextField(FXObject*, FXSelector, void *ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

  void reload();

protected:
  FF_EditItem();

  FF_App *m_app;

  FXTextField *m_codeEdit;
  FXTextField *m_nameEdit;

  FF_ValueEdit *m_amountEdit;
  FF_AccountEdit *m_costOfSalesAccountEdit;

  FF_TaxEdit *m_buyTaxEdit;
  FF_ValueEdit *m_buyValueEdit;
  FF_AccountEdit *m_buyAccountEdit;

  FF_TaxEdit *m_sellTaxEdit;
  FF_ValueEdit *m_sellValueEdit;
  FF_AccountEdit *m_sellAccountEdit;

  FXText *m_memoEdit;

  AE_COMMODITY_LIST *m_commodities;
  AE_ACCOUNT_TREE *m_accountTree;
};




#endif




