/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editjob.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_itemtable.hpp"
#include "ff_selectpartner.hpp"
#include "ff_partneredit.hpp"

#include <aqfinance/engine/book/ae_group.h>
#include <aqfinance/engine/modules/ae_idgen.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_EditJob) FF_EditJobMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditJob::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditJob::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditJob::ID_TEXTFIELD, FF_EditJob::onCmdTextField),
  FXMAPFUNC(SEL_REPLACED, FF_EditJob::ID_ITEM_TABLE, FF_EditJob::onRplItemTable),
  FXMAPFUNC(SEL_COMMAND, FF_EditJob::ID_PARTNER_EDIT, FF_EditJob::onCmdPartnerEdit),
  FXMAPFUNC(SEL_COMMAND, FF_EditJob::ID_ADDRESS_BUTTON, FF_EditJob::onCmdAddressButton),
  FXMAPFUNC(SEL_COMMAND, FF_EditJob::ID_ACCOUNT_BUTTON, FF_EditJob::onCmdAccountButton),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FF_EditJob::ID_ITEM_TABLE,
	    FF_EditJob::onListRightBtnReleased),
  FXMAPFUNC(SEL_COMMAND, FF_EditJob::ID_CMD_DELITEM,
	    FF_EditJob::onCmdDelItem),
  FXMAPFUNC(SEL_COMMAND, FF_EditJob::ID_CODE_BUTTON, FF_EditJob::onCmdGenerateCode),
};


FXIMPLEMENT(FF_EditJob, FXDialogBox, FF_EditJobMap, ARRAYNUMBER(FF_EditJobMap))




FF_EditJob::FF_EditJob(int jobType,
		       bool isBuy,
		       FF_App* a,
		       FXComposite *parent,
		       const FXString& title,
		       FXuint opts,
		       FXint x, FXint y, FXint w, FXint h,
		       FXint pl, FXint pr, FXint pt, FXint pb,
		       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_jobType(jobType)
,m_totalLabel(NULL)
,m_showValues(true)
,m_isBuy(isBuy)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXVerticalFrame *vf2;
  FXMatrix *m;
  FXHorizontalFrame *hf;

  if (m_jobType==AE_Job_TypeDelivery ||
      m_jobType==AE_Job_TypeRelease)
    m_showValues=false;

  /* begin: first matrix */
  m=new FXMatrix(vf, 3, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Enter Date"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_enterDateEdit=new FF_DateEdit(m, this, ID_DATEEDIT, I18N("YYYY/MM/DD"));
  new FXSpring(m, LAYOUT_FILL_X);

  new FXLabel(m, I18N("Due Date"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_dueDateEdit=new FF_DateEdit(m, this, ID_DATEEDIT, I18N("YYYY/MM/DD"));
  new FXSpring(m, LAYOUT_FILL_X);

  new FXLabel(m, I18N("Code"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_codeEdit=new FXTextField(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  new THEMEBUTTON(m, I18N("Generate"), NULL, this, ID_CODE_BUTTON,
	       BUTTON_NORMAL | LAYOUT_RIGHT);

  new FXLabel(m, I18N("Partner"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_partnerEdit=new FF_PartnerEdit(m, this, ID_PARTNER_EDIT, LAYOUT_FILL_X);
  new FXSpring(m, LAYOUT_FILL_X);

  new FXLabel(m, I18N("Priority"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_priorityCombo=new FXListBox(m, this, ID_PRIORITY_COMBO, 0);
  m_priorityCombo->appendItem(I18N("Priority|none"));
  m_priorityCombo->appendItem(I18N("Priority|low"));
  m_priorityCombo->appendItem(I18N("Priority|medium"));
  m_priorityCombo->appendItem(I18N("Priority|high"));
  m_priorityCombo->appendItem(I18N("Priority|urgent"));
  m_priorityCombo->setNumVisible(5);
  new FXSpring(m, LAYOUT_FILL_X);

  new FXLabel(m, I18N("Memo"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_memoEdit=new FXTextField(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  new FXSpring(m, LAYOUT_FILL_X);

  /* end: first matrix */


  m_tabBook=new FXTabBook(vf, this, ID_TABBOOK,
			  TABBOOK_NORMAL |
			  /*FRAME_RAISED | FRAME_THICK | */
			  LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* items tab */
  new FXTabItem(m_tabBook, I18N("Items"));
  vf2=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y,
			  0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
  m_itemTable=new FF_ItemTable(vf2, this, ID_ITEM_TABLE,
			       LAYOUT_FILL_X | LAYOUT_FILL_Y |
			       FF_ItemTable::ITEMTABLE_USE_ITEM_TEMPLATES);
  if (m_isBuy) {
    m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_CODE);
    m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_MEMO);
    if (m_showValues)
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_BUY_TAX);
    m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_AMOUNT);
    if (m_showValues) {
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_BUY_NETSUM);
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_BUY_TAXSUM);
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_BUY_TOTALSUM);
    }
  }
  else {
    m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_CODE);
    m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_MEMO);
    if (m_showValues)
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_SELL_TAX);
    m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_AMOUNT);
    if (m_showValues) {
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_SELL_NETSUM);
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_SELL_TAXSUM);
      m_itemTable->addColumn(FF_ItemTable::COLUMN_TYPE_SELL_TOTALSUM);
    }
  }

  if (m_showValues)
    m_totalLabel=new FOX16_HtmlLabel(vf2, "<b>Total:</b>000000000000 EUR", LAYOUT_FILL_X);

  /* address tab */
  new FXTabItem(m_tabBook, I18N("Address"));
  vf2=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Department"), NULL, LABEL_NORMAL);
  m_deptEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Street/Nr"), NULL, LABEL_NORMAL);
  m_streetNumEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
				  TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Zipcode"), NULL, LABEL_NORMAL);
  m_zipCodeEdit=new FXTextField(m, 16, this, ID_TEXTFIELD,
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("City"), NULL, LABEL_NORMAL);
  m_cityEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Country"), NULL, LABEL_NORMAL);
  m_countryEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Email"), NULL, LABEL_NORMAL);
  m_emailEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
			      TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Faxnumber"), NULL, LABEL_NORMAL);
  m_faxEdit=new FXTextField(m, 40, this, ID_TEXTFIELD,
			    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new THEMEBUTTON(vf2, I18N("Select from Partner"), NULL, this, ID_ADDRESS_BUTTON, BUTTON_NORMAL);

  /* account tab */
  new FXTabItem(m_tabBook, I18N("Account"));
  vf2=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  new FXLabel(m, I18N("Bank Name"), NULL, LABEL_NORMAL);
  m_bankNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
				 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Account Name"), NULL, LABEL_NORMAL);
  m_accountNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
                                    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Account Number"), NULL, LABEL_NORMAL);
  m_acctIdEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Bank Code"), NULL, LABEL_NORMAL);
  m_bankCodeEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
				 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("SWIFT BIC"), NULL, LABEL_NORMAL);
  m_bicEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
			    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("IBAN"), NULL, LABEL_NORMAL);
  m_ibanEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Purpose"), NULL, LABEL_NORMAL);
  m_purposeEdit=new FXTextField(m, 32, this, ID_TEXTFIELD,
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new THEMEBUTTON(vf2, I18N("Select from Partner"), NULL, this, ID_ACCOUNT_BUTTON, BUTTON_NORMAL);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditJob::FF_EditJob()
:FXDialogBox()
,m_app(NULL)
,m_totalLabel(NULL)
,m_nameEdit(NULL)
,m_isBuy(false)
{

}



FF_EditJob::~FF_EditJob() {
}



void FF_EditJob::create() {
  FXDialogBox::create();
}



FXint FF_EditJob::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



FXint FF_EditJob::checkGui(FXbool quiet) {
  FXString s;
  GWEN_DATE *d;
  int cnt;

  s=m_codeEdit->getText().simplify();
  if (s.empty()) {
    DBG_ERROR(0, "Empty code");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing Code"));
      m_codeEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }

  if (!m_enterDateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Invalid Entry Date"));
      m_enterDateEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }
  GWEN_Date_free(d);

  if (!m_dueDateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Invalid Due Date"));
      m_dueDateEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }
  GWEN_Date_free(d);

  cnt=m_itemTable->getNumberOfRows()-1; /* ignore last line */
  if (cnt) {
    if (m_showValues) {
      AQDB_ID coId=0;
      int i;

      for (i=0; i<cnt; i++) {
	AE_ITEM *t;

	t=m_itemTable->getItemByRow(i);
	if (t) {
	  if (coId==0) {
	    coId=AE_Item_GetCommodityId(t);
	    if (coId==0) {
	      DBG_ERROR(0, "Bad commodity");
	      if (!quiet)
		FXMessageBox::error(this, MBOX_OK,
				    I18N("Input Error"),
				    "%s",
				    I18N("Please enter amounts with currencies for all items"));
	      return GWEN_ERROR_INVALID;
	    }
	  }

	  if (AE_Item_GetCommodityId(t)!=coId) {
	    DBG_ERROR(0, "Bad commodity");
	    if (!quiet)
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Input Error"),
				  "%s",
				  I18N("You must use the same currency for all items"));
	    return GWEN_ERROR_INVALID;
	  }
	}
      }
    }
  }
  else {
    DBG_ERROR(0, "No items");
    if (!quiet)
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please enter at least a single item."));
    return GWEN_ERROR_INVALID;
  }
  return 0;
}



FXint FF_EditJob::toGui(const AE_JOB *j) {
  const char *s;
  const GWEN_DATE *d;
  const AE_ITEM_LIST *il;
  AQDB_ID id;

  m_itemTable->reload();
  m_partnerEdit->reload();

  m_jobType=AE_Job_GetType(j);

  d=AE_Job_GetEnterDate(j);
  if (j)
    m_enterDateEdit->setDate(d);

  d=AE_Job_GetDueDate(j);
  if (j)
    m_dueDateEdit->setDate(d);

  s=AE_Job_GetCode(j);
  if (s)
    m_codeEdit->setText(FXString(s));

  s=AE_Job_GetMemo(j);
  if (s)
    m_memoEdit->setText(FXString(s));

  id=AE_Job_GetPartnerId(j);
  if (id)
    m_partnerEdit->setPartnerId(id);

  /* account group */
  s=AE_Job_GetAccountBankName(j);
  if (s)
    m_bankNameEdit->setText(FXString(s));

  s=AE_Job_GetAccountBankCode(j);
  if (s)
    m_bankCodeEdit->setText(FXString(s));

  s=AE_Job_GetAccountName(j);
  if (s)
    m_accountNameEdit->setText(FXString(s));

  s=AE_Job_GetAccountNumber(j);
  if (s)
    m_acctIdEdit->setText(FXString(s));

  s=AE_Job_GetAccountBic(j);
  if (s)
    m_bicEdit->setText(FXString(s));

  s=AE_Job_GetAccountIban(j);
  if (s)
    m_ibanEdit->setText(FXString(s));


  /* address group */
  s=AE_Job_GetAddressName(j);
  if (s)
    m_nameEdit->setText(FXString(s));

  s=AE_Job_GetAddressDept(j);
  if (s)
    m_deptEdit->setText(FXString(s));

  s=AE_Job_GetAddressStreetNum(j);
  if (s)
    m_streetNumEdit->setText(FXString(s));

  s=AE_Job_GetAddressZipCode(j);
  if (s)
    m_zipCodeEdit->setText(FXString(s));

  s=AE_Job_GetAddressLocation(j);
  if (s)
    m_cityEdit->setText(FXString(s));

  s=AE_Job_GetAddressCountry(j);
  if (s)
    m_countryEdit->setText(FXString(s));

  s=AE_Job_GetAddressEmail(j);
  if (s)
    m_emailEdit->setText(FXString(s));

  s=AE_Job_GetAddressFaxNumber(j);
  if (s)
    m_faxEdit->setText(FXString(s));

  il=AE_Job_GetItems(j);
  if (il) {
    AE_ITEM *item;

    item=AE_Item_List_First(il);
    while(item) {
      m_itemTable->addItem(AE_Item_dup(item));
      item=AE_Item_List_Next(item);
    }

    /* add empty item */
    item=AE_Item_new();
    m_itemTable->addItem(item);
  }

  /* setup address and account from partner if necessary */
  id=AE_Job_GetPartnerId(j);
  if (id)
    setFromPartner(id);

  switch(AE_Job_GetPriority(j)) {
  case AE_Job_PriorityLow:    m_priorityCombo->setCurrentItem(1); break;
  case AE_Job_PriorityMedium: m_priorityCombo->setCurrentItem(2); break;
  case AE_Job_PriorityHigh:   m_priorityCombo->setCurrentItem(3); break;
  case AE_Job_PriorityUrgent: m_priorityCombo->setCurrentItem(4); break;
  case AE_Job_PriorityNone:
  case AE_Job_PriorityUnknown:
    m_priorityCombo->setCurrentItem(0);
    break;
  }

  s=AE_Job_GetPurpose(j);
  if (s && *s)
    m_purposeEdit->setText(FXString(s));

  updateTotal();

  return 0;
}



FXint FF_EditJob::fromGui(AE_JOB *j) {
  FXString s;
  GWEN_DATE *d;
  AQDB_ID id;
  AE_ITEM_LIST *items;
  int cnt;
  int i;

  if (!m_enterDateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    return GWEN_ERROR_INVALID;
  }
  AE_Job_SetEnterDate(j, d);
  GWEN_Date_free(d);

  if (!m_dueDateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    return GWEN_ERROR_INVALID;
  }
  AE_Job_SetDueDate(j, d);
  GWEN_Date_free(d);

  s=m_codeEdit->getText().simplify();
  if (s.empty()) {
    DBG_ERROR(0, "Empty code");
    return GWEN_ERROR_INVALID;
  }
  AE_Job_SetCode(j, s.text());

  id=m_partnerEdit->getPartnerId();
  AE_Job_SetPartnerId(j, id);

  s=m_memoEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetMemo(j, NULL);
  else
    AE_Job_SetMemo(j, s.text());

  /* account group */
  s=m_bankNameEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAccountBankName(j, NULL);
  else
    AE_Job_SetAccountBankName(j, s.text());

  s=m_accountNameEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAccountName(j, NULL);
  else
    AE_Job_SetAccountName(j, s.text());

  s=m_acctIdEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAccountNumber(j, NULL);
  else
    AE_Job_SetAccountNumber(j, s.text());

  s=m_bankCodeEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAccountBankCode(j, NULL);
  else
    AE_Job_SetAccountBankCode(j, s.text());

  s=m_ibanEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAccountIban(j, NULL);
  else
    AE_Job_SetAccountIban(j, s.text());

  s=m_bicEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAccountBic(j, NULL);
  else
    AE_Job_SetAccountBic(j, s.text());

  s=m_purposeEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetPurpose(j, NULL);
  else
    AE_Job_SetPurpose(j, s.text());

  /* address code */
  s=m_nameEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressName(j, NULL);
  else
    AE_Job_SetAddressName(j, s.text());

  s=m_deptEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressDept(j, NULL);
  else
    AE_Job_SetAddressDept(j, s.text());

  s=m_streetNumEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressStreetNum(j, NULL);
  else
    AE_Job_SetAddressStreetNum(j, s.text());

  s=m_zipCodeEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressZipCode(j, NULL);
  else
    AE_Job_SetAddressZipCode(j, s.text());

  s=m_cityEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressLocation(j, NULL);
  else
    AE_Job_SetAddressLocation(j, s.text());

  s=m_countryEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressCountry(j, NULL);
  else
    AE_Job_SetAddressCountry(j, s.text());

  s=m_emailEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressEmail(j, NULL);
  else
    AE_Job_SetAddressEmail(j, s.text());

  s=m_faxEdit->getText().simplify();
  if (s.empty())
    AE_Job_SetAddressFaxNumber(j, NULL);
  else
    AE_Job_SetAddressFaxNumber(j, s.text());

  switch(m_priorityCombo->getCurrentItem()) {
  case 0:  AE_Job_SetPriority(j, AE_Job_PriorityNone); break;
  case 1:  AE_Job_SetPriority(j, AE_Job_PriorityLow); break;
  case 2:  AE_Job_SetPriority(j, AE_Job_PriorityMedium); break;
  case 3:  AE_Job_SetPriority(j, AE_Job_PriorityHigh); break;
  case 4:  AE_Job_SetPriority(j, AE_Job_PriorityUrgent); break;
  default: AE_Job_SetPriority(j, AE_Job_PriorityNone); break;
  }

  cnt=m_itemTable->getNumberOfRows()-1; /* ignore last line */
  items=AE_Item_List_new();
  for (i=0; i<cnt; i++) {
    AE_ITEM *t;

    t=m_itemTable->getItemByRow(i);
    if (t) {
      /* add copy to list because the item table owns the list elements */
      AE_Item_List_Add(AE_Item_dup(t), items);
    }
  }
  AE_Job_SetItems(j, items);

  return 0;
}



void FF_EditJob::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;
  FXString gname;

  gname="FF_EditJob-";
  gname+=AE_Job_Type_toString(m_jobType);

  db=m_app->getDialogConfig(gname.text());
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+50);

  sortByColumn=GWEN_DB_GetIntValue(db, "itemsSortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_itemTable->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "itemsColumns", i, m_itemTable->getHeaderSize(i));
    m_itemTable->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_itemTable->setHeaderArrowDir(i, TRUE);
      else
        m_itemTable->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_EditJob::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;
  FXString gname;

  gname="FF_EditJob-";
  gname+=AE_Job_Type_toString(m_jobType);

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_itemTable->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "itemsColumns",
                        m_itemTable->getHeaderSize(i));
    b=m_itemTable->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "itemsSortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig(gname.text(), db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



long FF_EditJob::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_EditJob::onRplItemTable(FXObject*, FXSelector, void *ptr) {
  FXXTable::ColumnInfo *ci;

  DBG_ERROR(0, "CMD ItemTable");
  ci=(FXXTable::ColumnInfo*) ptr;

  if (ci) {
    if ((ci->getRow()+1)==m_itemTable->getNumberOfRows()) {
      AE_ITEM *item;

      item=AE_Item_new();
      m_itemTable->addItem(item);
      m_itemTable->update();
    }
  }

  updateTotal();

  return 1;
}



void FF_EditJob::updateTotal() {
  if (m_showValues) {
    int i;
    AQDB_VALUE *v;
    GWEN_BUFFER *vbuf;
    FXString str;

    /* calculate total sum */
    v=AQDB_Value_new();
    for (i=0; i<m_itemTable->getNumberOfRows()-1; i++) {
      AE_ITEM *item;

      item=m_itemTable->getItemByRow(i);
      if (item) {
	const AQDB_VALUE *vt;

	if (m_isBuy) {
	  vt=AE_Item_GetBuyPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetBuyTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	}
	else {
	  vt=AE_Item_GetSellPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetSellTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	}
      }
    }

    /* no currency for now */
    vbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AQDB_Value_toHumanReadableString(v, vbuf, 2);

    str+="<b>Total:</b> ";
    str+=FXString(GWEN_Buffer_GetStart(vbuf));
    GWEN_Buffer_free(vbuf);

    m_totalLabel->setText(str);
  }
}



long FF_EditJob::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditJob::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_EditJob::onCmdPartnerEdit(FXObject*, FXSelector, void *ptr) {
  AQDB_ID id;

  id=m_partnerEdit->getPartnerId();
  if (id)
    setFromPartner(id);
  return 1;
}



long FF_EditJob::onCmdAddressButton(FXObject*, FXSelector, void *ptr) {
  AQDB_ID id;

  id=m_partnerEdit->getPartnerId();
  if (id)
    setAddressFromPartner(id);
  return 1;
}



long FF_EditJob::onCmdAccountButton(FXObject*, FXSelector, void *ptr) {
  AQDB_ID id;

  id=m_partnerEdit->getPartnerId();
  if (id)
    setAccountFromPartner(id);
  return 1;
}



void FF_EditJob::setAddressFromPartner(AQDB_ID pid) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_PARTNER *pt=NULL;
    int rv;
    AE_ADDRESS *addr;
    uint32_t flags=0;

    rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner, pid, &pt);
    if (rv<0) {
      DBG_ERROR(0, "Unable to read partner %08x: %d", pid, rv);
      return;
    }

    rv=AE_Book_ReadPartnerSubObjects(b, pt);
    if (rv<0) {
      DBG_ERROR(0, "Unable to read subObjects for partner %08x: %d", pid, rv);
      AE_Partner_free(pt);
      return;
    }

    switch(m_jobType) {
    case AE_Job_TypeRequest:
    case AE_Job_TypeOffer:
    case AE_Job_TypeOrder:
    case AE_Job_TypeConfirmation:
    case AE_Job_TypeRelease:
    case AE_Job_TypeInvoice:
      flags=AE_ADDRESS_FLAGS_BILLING;
      break;
    case AE_Job_TypePayment:
    case AE_Job_TypeCancellation:
    case AE_Job_TypeDelivery:
    case AE_Job_TypeUnknown:
      flags=AE_ADDRESS_FLAGS_DELIVERY;
      break;
    }

    /* set address */
    addr=AE_Partner_GetFirstAddress(pt);
    /* first try to find a matching address */
    while(addr) {
      if ((AE_Address_GetFlags(addr) & flags)==flags)
	break;
      addr=AE_Address_List_Next(addr);
    }
    if (addr==NULL)
      /* none found, simply use the first address */
      addr=AE_Partner_GetFirstAddress(pt);

    if (addr) {
      const char *s;

      s=AE_Address_GetName(addr);
      if (s && *s)
	m_nameEdit->setText(s);
      else {
	FXString str;

	s=AE_Partner_GetTitle(pt);
	if (s && *s)
	  str=FXString(s);
	s=AE_Partner_GetFirstName(pt);
	if (s && *s) {
	  if (!(str.empty()))
	    str+=" ";
	  str+=FXString(s);
	}
	s=AE_Partner_GetLastName(pt);
	if (s) {
	  if (!(str.empty()))
	    str+=" ";
	  str+=FXString(s);
	}
	m_nameEdit->setText(str);
      }

      s=AE_Address_GetStreetNum(addr);
      if (s && *s)
	m_streetNumEdit->setText(s);

      s=AE_Address_GetZipCode(addr);
      if (s && *s)
	m_zipCodeEdit->setText(s);

      s=AE_Address_GetCity(addr);
      if (s && *s)
	m_cityEdit->setText(s);

      s=AE_Address_GetCountry(addr);
      if (s && *s)
	m_countryEdit->setText(s);

      s=AE_Address_GetEmail(addr);
      if (s && *s)
	m_emailEdit->setText(s);

      s=AE_Address_GetFaxNumber(addr);
      if (s && *s)
	m_faxEdit->setText(s);
    }

    AE_Partner_free(pt);
  }
}



void FF_EditJob::setAccountFromPartner(AQDB_ID pid) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_PARTNER *pt=NULL;
    AE_BANKACCOUNT *acc;
    int rv;

    rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner, pid, &pt);
    if (rv<0) {
      DBG_ERROR(0, "Unable to read partner %08x: %d", pid, rv);
      return;
    }

    rv=AE_Book_ReadPartnerSubObjects(b, pt);
    if (rv<0) {
      DBG_ERROR(0, "Unable to read subObjects for partner %08x: %d", pid, rv);
      AE_Partner_free(pt);
      return;
    }

    /* set bank account */
    acc=AE_Partner_GetFirstBankAccount(pt);
    if (acc) {
      const char *s;

      s=AE_BankAccount_GetBankName(acc);
      if (s && *s)
	m_bankNameEdit->setText(s);

      s=AE_BankAccount_GetBankCode(acc);
      if (s && *s)
	m_bankCodeEdit->setText(s);

      s=AE_BankAccount_GetAccountName(acc);
      if (s && *s)
	m_accountNameEdit->setText(s);

      s=AE_BankAccount_GetAccountNumber(acc);
      if (s && *s)
	m_acctIdEdit->setText(s);

      s=AE_BankAccount_GetBic(acc);
      if (s && *s)
	m_bicEdit->setText(s);

      s=AE_BankAccount_GetIban(acc);
      if (s && *s)
	m_ibanEdit->setText(s);
    }

    AE_Partner_free(pt);
  }
}



void FF_EditJob::setFromPartner(AQDB_ID pid) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_PARTNER *pt=NULL;
    int rv;

    rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner, pid, &pt);
    if (rv<0) {
      DBG_ERROR(0, "Unable to read partner %08x: %d", pid, rv);
      return;
    }

    rv=AE_Book_ReadPartnerSubObjects(b, pt);
    if (rv<0) {
      DBG_ERROR(0, "Unable to read subObjects for partner %08x: %d", pid, rv);
      AE_Partner_free(pt);
      return;
    }

    if (m_nameEdit->getText().empty() &&
	m_deptEdit->getText().empty() &&
	m_streetNumEdit->getText().empty() &&
	m_zipCodeEdit->getText().empty() &&
	m_cityEdit->getText().empty() &&
	m_countryEdit->getText().empty()) {
      AE_ADDRESS *addr;
      uint32_t flags=0;

      switch(m_jobType) {
      case AE_Job_TypeRequest:
      case AE_Job_TypeOffer:
      case AE_Job_TypeOrder:
      case AE_Job_TypeConfirmation:
      case AE_Job_TypeRelease:
      case AE_Job_TypeInvoice:
	flags=AE_ADDRESS_FLAGS_BILLING;
        break;
      case AE_Job_TypePayment:
      case AE_Job_TypeCancellation:
      case AE_Job_TypeDelivery:
      case AE_Job_TypeUnknown:
	flags=AE_ADDRESS_FLAGS_DELIVERY;
        break;
      }

      /* set address */
      addr=AE_Partner_GetFirstAddress(pt);
      /* first try to find a matching address */
      while(addr) {
	if ((AE_Address_GetFlags(addr) & flags)==flags)
	  break;
	addr=AE_Address_List_Next(addr);
      }
      if (addr==NULL)
        /* none found, simply use the first address */
	addr=AE_Partner_GetFirstAddress(pt);

      if (addr) {
	const char *s;

	s=AE_Address_GetName(addr);
        if (s && *s)
	  m_nameEdit->setText(s);
	else {
	  FXString str;

	  s=AE_Partner_GetTitle(pt);
	  if (s && *s)
	    str=FXString(s);
	  s=AE_Partner_GetFirstName(pt);
	  if (s && *s) {
	    if (!(str.empty()))
	      str+=" ";
	    str+=FXString(s);
	  }
	  s=AE_Partner_GetLastName(pt);
	  if (s) {
	    if (!(str.empty()))
	      str+=" ";
	    str+=FXString(s);
	  }
	  m_nameEdit->setText(str);
	}

	s=AE_Address_GetStreetNum(addr);
	if (s && *s)
	  m_streetNumEdit->setText(s);

	s=AE_Address_GetZipCode(addr);
	if (s && *s)
	  m_zipCodeEdit->setText(s);

	s=AE_Address_GetCity(addr);
	if (s && *s)
	  m_cityEdit->setText(s);

	s=AE_Address_GetCountry(addr);
	if (s && *s)
	  m_countryEdit->setText(s);

	s=AE_Address_GetEmail(addr);
	if (s && *s)
	  m_emailEdit->setText(s);
  
	s=AE_Address_GetFaxNumber(addr);
	if (s && *s)
	  m_faxEdit->setText(s);
      }
    }

    if (m_bankNameEdit->getText().empty() &&
	m_accountNameEdit->getText().empty() &&
	m_acctIdEdit->getText().empty() &&
	m_bankCodeEdit->getText().empty() &&
	m_ibanEdit->getText().empty() &&
	m_bicEdit->getText().empty()) {
      AE_BANKACCOUNT *acc;

      /* set bank account */
      acc=AE_Partner_GetFirstBankAccount(pt);
      if (acc) {
	const char *s;

	s=AE_BankAccount_GetBankName(acc);
	if (s && *s)
	  m_bankNameEdit->setText(s);

	s=AE_BankAccount_GetBankCode(acc);
	if (s && *s)
	  m_bankCodeEdit->setText(s);

	s=AE_BankAccount_GetAccountName(acc);
	if (s && *s)
	  m_accountNameEdit->setText(s);

	s=AE_BankAccount_GetAccountNumber(acc);
	if (s && *s)
	  m_acctIdEdit->setText(s);

	s=AE_BankAccount_GetBic(acc);
	if (s && *s)
	  m_bicEdit->setText(s);

	s=AE_BankAccount_GetIban(acc);
	if (s && *s)
	  m_ibanEdit->setText(s);
      }
    }

    AE_Partner_free(pt);
  }
}



long FF_EditJob::onListRightBtnReleased(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;

  if (!event->moved){
    FXMenuPane pmenu(this);

    new FXMenuCommand(&pmenu, I18N("Delete Item"), NULL, this, ID_CMD_DELITEM);

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FF_EditJob::onCmdDelItem(FXObject*, FXSelector, void* ptr) {
  int r;

  DBG_ERROR(0, "Delete item");
  r=m_itemTable->getCurrentRow();
  if (r>=0) {
    AE_ITEM *item;

    item=m_itemTable->getCurrentItem();
    if (item) {
      FF_App *app;
      AE_BOOK *b;
      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
        FXuint res;

        res=FXMessageBox::question(this, MBOX_YES_NO,
				   I18N("Delete Item"),
				   "%s",
                                   I18N("Are you sure you want to delete the selected "
					"item?"));
        if (res!=MBOX_CLICKED_YES) {
          DBG_ERROR(0, "Aborted");
        }
	else {
	  m_itemTable->delRow(r);
	  m_itemTable->rebuild();
          updateTotal();
	}
      }
    }
  }

  return 1;
}



long FF_EditJob::onCmdGenerateCode(FXObject*, FXSelector, void* ptr) {
  GWEN_BUFFER *tbuf;
  const char *prefix=NULL;
  int rv;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);

  switch(m_jobType) {
  case AE_Job_TypeRequest:          prefix="A"; break;
  case AE_Job_TypeOffer:            prefix="B"; break;
  case AE_Job_TypeOrder:            prefix="C"; break;
  case AE_Job_TypeConfirmation:     prefix="D"; break;
  case AE_Job_TypeRelease:          prefix="E"; break;
  case AE_Job_TypeDelivery:         prefix="F"; break;
  case AE_Job_TypeInvoice:          prefix="R"; break;
  case AE_Job_TypePayment:          prefix="H"; break;
  case AE_Job_TypeReminder:         prefix="J"; break;
  case AE_Job_TypeCancellation:     prefix="K"; break;

  case AE_Job_TypeUnknown:
  case AE_Job_TypeNone:
    break;
  }

  rv=AE_IdGen_MkId(prefix, tbuf);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "Could not generate an id");
  }
  else {
    m_codeEdit->setText(GWEN_Buffer_GetStart(tbuf));
  }
  GWEN_Buffer_free(tbuf);
  return 1;
}



bool FF_EditJob::editJob(FF_App* app,
			 FXComposite *parent,
			 const FXString &title,
			 AE_JOB *j,
			 bool isBuy) {
  FF_EditJob dlg(AE_Job_GetType(j), isBuy, app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  res=dlg.toGui(j);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());
  dlg.init();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(j);
  if (res==0) {
    return true;
  }

  return false;
}





