/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITJOB_HPP
#define FF_EDITJOB_HPP


#include <fx.h>

#include <aqfinance/engine/book/ae_book.h>
#include <aqfinance/engine/book/ae_job.h>


class FF_App;
class FF_DateEdit;
class FF_ItemTable;
class FF_PartnerEdit;
class FOX16_HtmlLabel;


class FF_EditJob: public FXDialogBox {
  FXDECLARE(FF_EditJob)

public:

  enum {
    ID_COMBOBOX=FXDialogBox::ID_LAST,
    ID_TABBOOK,
    ID_TEXTFIELD,
    ID_ITEM_TABLE,
    ID_DATEEDIT,
    ID_CODE_BUTTON,
    ID_PARTNER_EDIT,
    ID_ADDRESS_BUTTON,
    ID_ACCOUNT_BUTTON,
    ID_PRIORITY_COMBO,
    ID_CMD_DELITEM,
    ID_LAST
  };

  FF_EditJob(int jobType,
             bool isIncoming,
	     FF_App* a,
	     FXComposite *parent,
	     const FXString& title,
	     FXuint opts=DECOR_TITLE|DECOR_BORDER,
	     FXint x=0, FXint y=0, FXint w=0, FXint h=0,
	     FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
	     FXint hs=0, FXint vs=0);
  ~FF_EditJob();

  FXint getDefaultHeight();

  static bool editJob(FF_App* app,
		      FXComposite *parent,
		      const FXString &title,
		      AE_JOB *j,
		      bool isBuy);

  FXint checkGui(FXbool quiet);
  FXint toGui(const AE_JOB *j);
  FXint fromGui(AE_JOB *j);

  void init();
  void fini();

  void create();

  long onCmdTextField(FXObject*, FXSelector, void *ptr);
  long onRplItemTable(FXObject*, FXSelector, void *ptr);
  long onCmdPartnerEdit(FXObject*, FXSelector, void *ptr);

  long onCmdAddressButton(FXObject*, FXSelector, void *ptr);
  long onCmdAccountButton(FXObject*, FXSelector, void *ptr);

  long onListRightBtnReleased(FXObject*, FXSelector, void *ptr);
  long onCmdDelItem(FXObject*, FXSelector, void* ptr);
  long onCmdGenerateCode(FXObject*, FXSelector, void* ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

protected:
  FF_EditJob();

  void updateTotal();
  void setFromPartner(AQDB_ID pid);
  void setAccountFromPartner(AQDB_ID pid);
  void setAddressFromPartner(AQDB_ID pid);

  FF_App *m_app;
  int m_jobType;

  FXTabBook *m_tabBook;

  FF_DateEdit *m_enterDateEdit;
  FF_DateEdit *m_dueDateEdit;
  FXTextField *m_codeEdit;

  FF_PartnerEdit *m_partnerEdit;

  FXListBox *m_priorityCombo;
  FXTextField *m_memoEdit;

  FF_ItemTable *m_itemTable;
  FOX16_HtmlLabel *m_totalLabel;

  FXTextField *m_bankNameEdit;
  FXTextField *m_accountNameEdit;
  FXTextField *m_acctIdEdit;
  FXTextField *m_bankCodeEdit;
  FXTextField *m_ibanEdit;
  FXTextField *m_bicEdit;
  FXTextField *m_purposeEdit;

  FXTextField *m_nameEdit;
  FXTextField *m_deptEdit;
  FXTextField *m_streetNumEdit;
  FXTextField *m_zipCodeEdit;
  FXTextField *m_cityEdit;
  FXTextField *m_countryEdit;
  FXTextField *m_emailEdit;
  FXTextField *m_faxEdit;

  bool m_showValues;
  bool m_isBuy;
};




#endif




