/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editloadcellphone.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_group.h>

#include <aqbanking/jobloadcellphone.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditLoadCellPhone) FF_EditLoadCellPhoneMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_EditLoadCellPhone::ID_COMBO_LOCAL_ACC, FF_EditLoadCellPhone::onCmdLocalAccount),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditLoadCellPhone::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditLoadCellPhone::onCmdCancel),
};


FXIMPLEMENT(FF_EditLoadCellPhone, FXDialogBox, FF_EditLoadCellPhoneMap, ARRAYNUMBER(FF_EditLoadCellPhoneMap))




FF_EditLoadCellPhone::FF_EditLoadCellPhone(FF_App* a,
					   FXComposite *parent,
					   const FXString& title,
					   FXuint opts,
					   FXint x, FXint y, FXint w, FXint h,
					   FXint pl, FXint pr, FXint pt, FXint pb,
					   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_localAccountCombo(NULL)
,m_productCombo(NULL)
,m_phoneNumberEdit(NULL)
,m_memoEdit(NULL)
,m_amountEdit(NULL)
,m_currencyCombo(NULL)
,m_bankAccountList(NULL)
,m_productList(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  FXHorizontalFrame *hf;

  new FXLabel(m, I18N("Bank Account"), NULL, LABEL_NORMAL);
  m_localAccountCombo=new FXComboBox(m, 1, this, ID_COMBO_LOCAL_ACC,
				     COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN | LAYOUT_FILL_ROW);

  new FXLabel(m, I18N("Phone Number"), NULL, LABEL_NORMAL);
  m_phoneNumberEdit=new FXTextField(m, 32, this, ID_PHONE_NUMBER, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
				    LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Product"), NULL, LABEL_NORMAL);
  m_productCombo=new FXComboBox(m, 1, this, ID_COMBO_PRODUCT,
				COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("Memo"), NULL, LABEL_NORMAL);
  m_memoEdit=new FXTextField(m, 32, this,
			     ID_PHONE_NUMBER, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
			     LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Amount"), NULL, LABEL_NORMAL);
  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_amountEdit=new FXTextField(hf, 12, this, ID_EDIT_AMOUNT,
			       LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
			       JUSTIFY_RIGHT | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_currencyCombo=new FXComboBox(hf, 6, this, ID_COMBO_CURRENCY,
				 COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK);


  new FXSpring(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditLoadCellPhone::FF_EditLoadCellPhone()
:FXDialogBox()
,m_app(NULL)
,m_localAccountCombo(NULL)
,m_productCombo(NULL)
,m_phoneNumberEdit(NULL)
,m_memoEdit(NULL)
,m_amountEdit(NULL)
,m_currencyCombo(NULL)
,m_bankAccountList(NULL)
,m_productList(NULL)
{

}



FF_EditLoadCellPhone::~FF_EditLoadCellPhone() {
  AE_BankAccount_List_free(m_bankAccountList);
  m_bankAccountList=(AE_BANKACCOUNT_LIST*) -1;

  AB_CellPhoneProduct_List_free(m_productList);
  m_productList=(AB_CELLPHONE_PRODUCT_LIST*) -1;
}



void FF_EditLoadCellPhone::create() {
  FXDialogBox::create();
}



FXint FF_EditLoadCellPhone::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



void FF_EditLoadCellPhone::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditLoadCellPhone");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);


  GWEN_DB_Group_free(db);
}



void FF_EditLoadCellPhone::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditLoadCellPhone", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }

  GWEN_DB_Group_free(db);
}



FXint FF_EditLoadCellPhone::checkGui(FXbool quiet) {
  FXString str;
  int i;
  AB_BANKING *ab;

  ab=m_app->getBanking();

  i=m_localAccountCombo->getCurrentItem();
  if (i<1) {
    DBG_ERROR(0, "Missing account");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please select a local account"));
      m_localAccountCombo->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }
  else {
    AE_BANKACCOUNT *ba;

    i--;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }
  
    if (ba==NULL) {
      DBG_ERROR(0, "Missing account");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Missing local account"));
	m_localAccountCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  i=m_productCombo->getCurrentItem();
  if (i<1) {
    DBG_ERROR(0, "Missing product");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please select a phone product"));
      m_productCombo->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }
  else {
    AB_CELLPHONE_PRODUCT *p;

    p=AB_CellPhoneProduct_List_First(m_productList);
    i--;
    while(p) {
      if ((i--)==0)
	break;
      p=AB_CellPhoneProduct_List_Next(p);
    }
    if (p==NULL) {
      DBG_ERROR(0, "Missing product");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Missing phone product"));
	m_productCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  str=m_amountEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing amount"));
      m_amountEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Two types of komma detected, "
                                 "please specify the decimal point only"));
        m_amountEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad amount input"));
          m_amountEdit->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
      if (AQDB_Value_IsZero(v) || AQDB_Value_IsNegative(v)) {
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Please enter a positive amount"));
          m_amountEdit->setFocus();
        }
	AQDB_Value_free(v);
	return GWEN_ERROR_BAD_DATA;
      }
      AQDB_Value_free(v);
    }
  }

  str=m_currencyCombo->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing currency"));
      m_currencyCombo->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  str=m_phoneNumberEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing phone number"));
      m_phoneNumberEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  return 0;
}



FXString FF_EditLoadCellPhone::createCellPhoneComboString(const char *providerName, const char *productName) {
  FXString str;

  if (providerName && *providerName==0) providerName=NULL;
  if (productName && *productName==0) productName=NULL;

  if (providerName && productName)
    str=FXString(providerName)+"/"+FXString(productName);
  else if (providerName)
    str=FXString(providerName);
  else if (productName)
    str=FXString(productName);
  else
    str=FXString(I18N("unnamed product"));
  return str;
}



FXint FF_EditLoadCellPhone::toGui(const AE_LOAD_CELLPHONE *lc) {
  AE_BOOK *b;
  AB_BANKING *ab;

  ab=m_app->getBanking();
  b=m_app->getBook();
  if (b && ab) {
    int rv;
    AE_BANKACCOUNT *ba;
    int i;
    int selected=0;
    const char *s;
    FXString str;
    const AQDB_VALUE *v;

    /* fill local account combo box */
    m_localAccountCombo->clearItems();
    m_currencyCombo->clearItems();

    m_localAccountCombo->appendItem(I18N("Select an Account"));
    m_productCombo->appendItem(I18N("Select a Product"));

    if (m_bankAccountList!=NULL)
      AE_BankAccount_List_free(m_bankAccountList);
    m_bankAccountList=AE_BankAccount_List_new();
    rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, m_bankAccountList);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_BankAccount_List_free(m_bankAccountList);
      m_bankAccountList=NULL;
      return rv;
    }

    /* remove offline accounts */
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      AE_BANKACCOUNT *n;
      AQDB_ID oid;

      n=AE_BankAccount_List_Next(ba);
      oid=AE_BankAccount_GetBankingId(ba);
      if (oid) {
	if (AB_Banking_GetAccount(ab, oid)==NULL) {
	  /* oid, but there is no account for it; remove from list */
	  AE_BankAccount_List_Del(ba);
	  AE_BankAccount_free(ba);
	}
      }
      else {
	/* offline account, remove from list and free */
	AE_BankAccount_List_Del(ba);
	AE_BankAccount_free(ba);
      }

      ba=n;
    }

    i=1;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      const char *s1;
      const char *s2;
      FXString str;

      s1=AE_BankAccount_GetBankName(ba);
      if (s1==NULL || *s1==0)
        s1=AE_BankAccount_GetBankCode(ba);
      s2=AE_BankAccount_GetAccountName(ba);
      if (s2==NULL || *s1==0)
        s2=AE_BankAccount_GetAccountNumber(ba);
      if (*s1==0) s1=NULL;
      if (*s2==0) s2=NULL;

      if (s1 && s2)
        str=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        str=FXString(s1);
      else if (s2)
        str=FXString(s2);
      else
        str=FXString(I18N("unnamed account"));
      m_localAccountCombo->appendItem(str);
      if (AE_LoadCellPhone_GetBankAccountId(lc)==AE_BankAccount_GetBankingId(ba))
	selected=i;

      i++;
      ba=AE_BankAccount_List_Next(ba);
    }

    i=m_localAccountCombo->getNumItems();
    if (i>10)
      m_localAccountCombo->setNumVisible(10);
    else
      m_localAccountCombo->setNumVisible(i);

    m_localAccountCombo->setCurrentItem(selected);
    if (selected)
      selectLocalAccount(selected-1);

    /* set product entry */
    str=createCellPhoneComboString(AE_LoadCellPhone_GetProviderName(lc),
				   AE_LoadCellPhone_GetProductName(lc));
    for (i=1; i<m_productCombo->getNumItems(); i++) {
      FXString s;

      s=m_productCombo->getItem(i);
      if (!(s.empty()) && FX::comparecase(s, str)==0)
	break;
    }
    if (i<m_productCombo->getNumItems())
      m_productCombo->setCurrentItem(i);

    /* set currency and amount */
    v=AE_LoadCellPhone_GetValue(lc);
    if (v) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      m_amountEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }
    s=AE_LoadCellPhone_GetCurrency(lc);
    if (s)
      m_currencyCombo->appendItem(FXString(s));
    else
      m_currencyCombo->appendItem(FXString("EUR"));

    s=AE_LoadCellPhone_GetPhoneNumber(lc);
    if (s && *s)
      m_phoneNumberEdit->setText(FXString(s));

    s=AE_LoadCellPhone_GetMemo(lc);
    if (s && *s)
      m_memoEdit->setText(FXString(s));
  }


  return 0;
}



FXint FF_EditLoadCellPhone::fromGui(AE_LOAD_CELLPHONE *lc) {
  FXString str;
  FXString str2;
  int i;
  AE_BANKACCOUNT *ba;

  i=m_localAccountCombo->getCurrentItem();
  if (i==0) {
    return GWEN_ERROR_NO_DATA;
  }
  else {
    i--;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }
  
    if (ba==NULL)
      return GWEN_ERROR_NO_DATA;
    else
      AE_LoadCellPhone_SetBankAccountId(lc, AE_BankAccount_GetBankingId(ba));
  }

  str=m_phoneNumberEdit->getText().simplify();
  if (str.empty()) {
    DBG_ERROR(0, "Empty phone number");
    return GWEN_ERROR_INVALID;
  }
  AE_LoadCellPhone_SetPhoneNumber(lc, str.text());

  str=m_memoEdit->getText().simplify();
  if (str.empty())
    AE_LoadCellPhone_SetMemo(lc, NULL);
  else
    AE_LoadCellPhone_SetMemo(lc, str.text());

  assert(m_productList);
  i=m_productCombo->getCurrentItem();
  if (i>0) {
    AB_CELLPHONE_PRODUCT *p;

    p=AB_CellPhoneProduct_List_First(m_productList);
    i--;
    while(p) {
      if ((i--)==0)
	break;
      p=AB_CellPhoneProduct_List_Next(p);
    }
    if (p) {
      AE_LoadCellPhone_SetProviderName(lc, AB_CellPhoneProduct_GetProviderName(p));
      AE_LoadCellPhone_SetProductName(lc, AB_CellPhoneProduct_GetProductName(p));
    }
  }

  str=m_amountEdit->getText().simplify();
  if (str.empty()) {
    return GWEN_ERROR_NO_DATA;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL)
        return GWEN_ERROR_BAD_DATA;
      AE_LoadCellPhone_SetValue(lc, v);
      AQDB_Value_free(v);
    }
  }

  str=m_currencyCombo->getText().simplify();
  if (str.empty())
    return GWEN_ERROR_NO_DATA;
  else
    AE_LoadCellPhone_SetCurrency(lc, str.text());

  return 0;
}



void FF_EditLoadCellPhone::selectLocalAccount(int i) {
  AB_BANKING *ab;
  int j;

  m_productCombo->clearItems();
  m_productCombo->appendItem(I18N("Select a Product"));

  if (m_productList)
    AB_CellPhoneProduct_List_free(m_productList);

  ab=m_app->getBanking();
  if (ab) {
    AE_BANKACCOUNT *ba;

    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }

    if (ba) {
      AB_ACCOUNT *a;

      a=AB_Banking_GetAccount(ab, AE_BankAccount_GetBankingId(ba));
      if (a) {
        AB_JOB *j;
        int rv;

	j=AB_JobLoadCellPhone_new(a);
	rv=AB_Job_CheckAvailability(j);
	if (rv<0) {
	  DBG_ERROR(0, "LoadCellPhone not available (%d) with %s/%s", rv,
		    AB_Account_GetBankCode(a),
		    AB_Account_GetAccountNumber(a));
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Banking Error"),
			      I18N("The selected local account does not support this jobs (%d)"), rv);

	}
	else {
	  const AB_CELLPHONE_PRODUCT_LIST *pl;

	  pl=AB_JobLoadCellPhone_GetCellPhoneProductList(j);
	  if (pl) {
	    const AB_CELLPHONE_PRODUCT *p;
	    AB_CELLPHONE_PRODUCT_LIST *cpl;

	    cpl=AB_CellPhoneProduct_List_new();

	    p=AB_CellPhoneProduct_List_First(pl);
	    while(p) {
	      AB_CELLPHONE_PRODUCT *cp;
	      FXString str;

	      cp=AB_CellPhoneProduct_dup(p);
	      AB_CellPhoneProduct_List_Add(cp, cpl);

	      str=createCellPhoneComboString(AB_CellPhoneProduct_GetProviderName(cp),
					     AB_CellPhoneProduct_GetProductName(cp));
	      m_productCombo->appendItem(str);

	      p=AB_CellPhoneProduct_List_Next(p);
	    }
	    m_productList=cpl;
	  }
        }
        AB_Job_free(j);
      }
    }
  }

  j=m_productCombo->getNumItems();
  if (j>10)
    m_productCombo->setNumVisible(10);
  else
    m_productCombo->setNumVisible(j);

}



long FF_EditLoadCellPhone::onCmdLocalAccount(FXObject*, FXSelector, void *ptr) {
  int i;

  i=m_localAccountCombo->getCurrentItem();
  if (i>0) {
    selectLocalAccount(i-1);
  }

  return 1;
}



long FF_EditLoadCellPhone::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditLoadCellPhone::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this, TRUE);
  }
  return 1;
}



long FF_EditLoadCellPhone::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this, FALSE);
  return 1;
}




bool FF_EditLoadCellPhone::editLoadCellPhone(FF_App* app,
					     FXComposite *parent,
					     const FXString &title,
					     AE_LOAD_CELLPHONE *lc) {
  FF_EditLoadCellPhone dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.place(PLACEMENT_VISIBLE);
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  dlg.init();
  res=dlg.toGui(lc);
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(lc);
  if (res==0) {
    return true;
  }

  return false;
}





