/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editpartner.hpp"
#include "ff_app.hpp"
#include "ff_grouptree.hpp"
#include "ff_accountedit.hpp"
#include "ff_bankaccountview.hpp"
#include "ff_groupview.hpp"
#include "ff_addressview.hpp"
#include "ff_ruleview.hpp"
#include "ff_commodityedit.hpp"
#include "ff_edit_printtabledescr.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>
#include <aqfinance/engine/book/ae_address.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditPartner) FF_EditPartnerMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_EditPartner::ID_EDIT_INVOICE_BUTTON, FF_EditPartner::onCmdPartnerEditInvoice),
};


FXIMPLEMENT(FF_EditPartner, FXDialogBox, FF_EditPartnerMap, ARRAYNUMBER(FF_EditPartnerMap))




FF_EditPartner::FF_EditPartner(FF_App* a,
			       FXComposite *parent,
			       const FXString& title,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h,
			       FXint pl, FXint pr, FXint pt, FXint pb,
			       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_partnerId(0)
,m_lastNameEdit(NULL)
,m_firstNameEdit(NULL)
,m_titleEdit(NULL)
,m_tabBook(NULL)
,m_addrTab(NULL)
,m_accTab(NULL)
,m_groupTab(NULL)
,m_noteTab(NULL)
,m_ruleTab(NULL)
,m_addressView(NULL)
,m_bankAccountView(NULL)
,m_groupView(NULL)
,m_noteEdit(NULL)
,m_ruleView(NULL)
{
  FXVerticalFrame *vf;
  FXVerticalFrame *vf2;
  FXHorizontalFrame *hf;
  FXMatrix *m;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);

  m=new FXMatrix(hf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Title"), NULL, LABEL_NORMAL | LAYOUT_FILL_X | JUSTIFY_LEFT);
  m_titleEdit=new FXTextField(m, 16, this, ID_TEXTFIELD_TITLE,
			      TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("First Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X | JUSTIFY_LEFT);
  m_firstNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_FIRSTNAME,
				  TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Last Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X | JUSTIFY_LEFT);
  m_lastNameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_LASTNAME,
				 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Currency"), NULL, LABEL_NORMAL | LAYOUT_FILL_X | JUSTIFY_LEFT);
  m_commodityEdit=new FF_CommodityEdit(m, NULL, 0, LAYOUT_FILL_X);

  m_customerCheck=new FXCheckButton(vf, I18N("Customer"), NULL, 0, CHECKBUTTON_NORMAL);
  m_vendorCheck=new FXCheckButton(vf, I18N("Vendor"), NULL, 0, CHECKBUTTON_NORMAL);

  /* buttons */
  vf2=new FXVerticalFrame(hf, LAYOUT_FILL_Y);
  m_editInvoiceButton=new THEMEBUTTON(vf2, I18N("Edit Invoice Template"), NULL, this, ID_EDIT_INVOICE_BUTTON, BUTTON_NORMAL);

  m_tabBook=new FXTabBook(vf, this, ID_TABBOOK,
			  LAYOUT_FILL_X|LAYOUT_FILL_Y);

  /* address tab */
  m_addrTab=new FXTabItem(m_tabBook, I18N("Addresses"), NULL,
			  JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  m_addressView=new FF_AddressView(m_tabBook, ID_VIEW_ADDRESS, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* account tab */
  m_accTab=new FXTabItem(m_tabBook, I18N("Bank Accounts"), NULL,
			 JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  m_bankAccountView=new FF_BankAccountView(m_tabBook, ID_VIEW_BANKACCOUNT, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* group tab */
  m_groupTab=new FXTabItem(m_tabBook, I18N("Grouping"), NULL,
			   JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  m_groupView=new FF_GroupView(AE_Book_TableType_PartnerGroup, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_groupView->setTarget(this);
  m_groupView->setSelector(ID_VIEW_GROUP);

  /* rule tab */
  m_ruleTab=new FXTabItem(m_tabBook, I18N("Matching Rules"), NULL);
  m_ruleView=new FF_RuleView(m_tabBook, this, ID_VIEW_RULE, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* note tab */
  m_noteTab=new FXTabItem(m_tabBook, I18N("Note"), NULL,
			  JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  m_noteEdit=new FXText(m_tabBook, this,
			ID_TEXT_NOTE, TEXT_WORDWRAP | FRAME_RAISED | FRAME_THICK |
			LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* dialog buttons */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditPartner::FF_EditPartner()
:FXDialogBox()
,m_app(NULL)
,m_partnerId(0)
,m_lastNameEdit(NULL)
,m_firstNameEdit(NULL)
,m_titleEdit(NULL)
,m_tabBook(NULL)
,m_addrTab(NULL)
,m_accTab(NULL)
,m_groupTab(NULL)
,m_noteTab(NULL)
,m_ruleTab(NULL)
,m_addressView(NULL)
,m_bankAccountView(NULL)
,m_groupView(NULL)
,m_noteEdit(NULL)
,m_ruleView(NULL)
{
}



FF_EditPartner::~FF_EditPartner() {
}



void FF_EditPartner::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;

  db=m_app->getDialogConfig("FF_EditPartner");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "addressView");
  m_addressView->init(dbT);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "bankAccountView");
  m_bankAccountView->init(dbT);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "ruleView");
  m_ruleView->init(dbT);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "groupView");
  m_groupView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_EditPartner::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "addressView");
  m_addressView->fini(dbT);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "bankAccountView");
  m_bankAccountView->fini(dbT);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "ruleView");
  m_ruleView->fini(dbT);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "groupView");
  m_groupView->fini(dbT);

  rv=m_app->setDialogConfig("FF_EditPartner", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}




FXint FF_EditPartner::checkGui(FXbool quiet) {
  AQDB_ID coId;

  coId=m_commodityEdit->getCommodityId();
  if (coId==0) {
    DBG_ERROR(0, "No commodity id");
    if (!quiet) {
      FXMessageBox::error(this, FX::MBOX_OK,
			  I18N("Input Error"),
                          "%s",
			  I18N("Please select a currency"));
      m_commodityEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  return 0;
}



FXint FF_EditPartner::toGui(const AE_PARTNER *pt) {
  const char *s;
  AE_ADDRESS_LIST *al;
  AE_BANKACCOUNT_LIST *bl;
  AE_RULE_LIST *rl;
  AQDB_ID id;
  uint32_t flags;

  m_groupView->reload();
  m_commodityEdit->reload();

#define TXTTOGUI(n, m)                   \
  s=AE_Partner_Get##m(pt);               \
  if (s)                                 \
    n->setText(s);

  TXTTOGUI(m_titleEdit, Title);
  TXTTOGUI(m_lastNameEdit, LastName);
  TXTTOGUI(m_firstNameEdit, FirstName);
  TXTTOGUI(m_noteEdit, Note);

  m_commodityEdit->setCommodityId(AE_Partner_GetCommodityId(pt));

  m_partnerId=AE_Partner_GetId(pt);
  flags=AE_Partner_GetFlags(pt);
  m_customerCheck->setCheck((flags & AE_PARTNER_FLAGS_CUSTOMER)?TRUE:FALSE);
  m_vendorCheck->setCheck((flags & AE_PARTNER_FLAGS_VENDOR)?TRUE:FALSE);

  al=AE_Partner_GetAddressList(pt);
  if (al)
    m_addressView->toGui(al);

  bl=AE_Partner_GetBankAccountList(pt);
  if (bl)
    m_bankAccountView->toGui(bl);

  rl=AE_Partner_GetRules(pt);
  if (rl)
    m_ruleView->toGui(rl);

  id=AE_Partner_GetGroupId(pt);
  if (id!=0) {
    m_groupView->selectGroup(id, true);
    m_groupView->makeGroupVisible(id);
  }

#undef TXTTOGUI

  if (m_partnerId==0)
    m_editInvoiceButton->disable();
  else
    m_editInvoiceButton->enable();

  return 0;
}



FXint FF_EditPartner::fromGui(AE_PARTNER *pt) {
  FXString str;
  AE_ADDRESS_LIST *al;
  AE_BANKACCOUNT_LIST *bl;
  AE_RULE_LIST *rl;
  AE_GROUP *gr;
  uint32_t flags;

#define TXTFROMGUI(n, m)                   \
  str=n->getText().simplify();             \
  if (str.empty())                         \
    AE_Partner_Set##m(pt, NULL);         \
  else                                     \
    AE_Partner_Set##m(pt, str.text())

  TXTFROMGUI(m_titleEdit, Title);
  TXTFROMGUI(m_lastNameEdit, LastName);
  TXTFROMGUI(m_firstNameEdit, FirstName);
  TXTFROMGUI(m_noteEdit, Note);

  AE_Partner_SetCommodityId(pt, m_commodityEdit->getCommodityId());

  flags=AE_Partner_GetFlags(pt);
  flags&=~(AE_PARTNER_FLAGS_CUSTOMER | AE_PARTNER_FLAGS_VENDOR);
  if (m_customerCheck->getCheck())
    flags|=AE_PARTNER_FLAGS_CUSTOMER;
  if (m_vendorCheck->getCheck())
    flags|=AE_PARTNER_FLAGS_VENDOR;
  AE_Partner_SetFlags(pt, flags);

  al=AE_Address_List_new();
  m_addressView->fromGui(al);
  AE_Partner_SetAddressList(pt, al);

  bl=AE_BankAccount_List_new();
  m_bankAccountView->fromGui(bl);
  AE_Partner_SetBankAccountList(pt, bl);

  rl=AE_Rule_List_new();
  m_ruleView->fromGui(rl);
  AE_Partner_SetRules(pt, rl);

  gr=m_groupView->getFirstSelectedGroup();
  if (gr)
    AE_Partner_SetGroupId(pt, AE_Group_GetId(gr));
  else
    AE_Partner_SetGroupId(pt, 0);

#undef TXTFROMGUI

  return 0;
}



void FF_EditPartner::create() {
  FXDialogBox::create();
}



long FF_EditPartner::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditPartner::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this, TRUE);
  }
  return 1;
}



long FF_EditPartner::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_EditPartner::onCmdPartnerEditInvoice(FXObject*, FXSelector, void *ptr) {
  AF_PRINT_TABLE_DESCR *cd;

  cd=AE_ReadPsInvoiceForPartner(m_partnerId);
  if (cd) {
    bool b;

    b=FF_EditPrintTableDescr::editForJob(m_app, this, "Partner's Template for Invoices", cd);
    if (b) {
      AE_WritePsInvoiceForPartner(cd, m_partnerId);
    }
    AF_PrintTableDescr_free(cd);
  }

  return 1;
}



FXint FF_EditPartner::getDefaultHeight() {
  /* add minimum height to be usefull */
  return FXDialogBox::getDefaultHeight()+100;
}



bool FF_EditPartner::editPartner(FF_App* app,
				 FXComposite *parent,
				 const FXString &title,
				 AE_PARTNER *pt) {
  FF_EditPartner dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.place(PLACEMENT_VISIBLE);
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  dlg.init();
  res=dlg.toGui(pt);
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(pt);
  if (res==0) {
    return true;
  }

  return false;
}



bool FF_EditPartner::selectPartnerAccount(FF_App* app,
					  FXComposite *parent,
					  const FXString &title,
					  AE_PARTNER *pt,
					  AE_BANKACCOUNT **pBankAccount) {
  FF_EditPartner dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  dlg.init();
  res=dlg.toGui(pt);
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(pt);
  if (res==0) {
    AE_BANKACCOUNT *ba;

    ba=dlg.m_bankAccountView->getCurrentBankAccount();
    if (ba) {
      *pBankAccount=AE_BankAccount_dup(ba);
      return true;
    }
  }


  return false;
}







