/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITPARTNER_HPP
#define FF_EDITPARTNER_HPP


#include <fx.h>

#include <aqfinance/engine/book/ae_partner.h>


class FF_App;
class FF_GroupTree;
class FF_AccountEdit;
class FF_BankAccountView;
class FF_GroupView;
class FF_AddressView;
class FF_RuleView;
class FF_CommodityEdit;


class FF_EditPartner: public FXDialogBox {
  FXDECLARE(FF_EditPartner)

public:

  enum {
    ID_TEXTFIELD_TITLE=FXDialogBox::ID_LAST,
    ID_TEXTFIELD_LASTNAME,
    ID_TEXTFIELD_FIRSTNAME,
    ID_CHECKBOX_CREDIT,
    ID_ACCEDIT_CREDIT,
    ID_CHECKBOX_DEBIT,
    ID_ACCEDIT_DEBIT,
    ID_GROUP,
    ID_TABBOOK,
    ID_TEXT_NOTE,
    ID_VIEW_BANKACCOUNT,
    ID_VIEW_GROUP,
    ID_VIEW_ADDRESS,
    ID_VIEW_RULE,
    ID_EDIT_INVOICE_BUTTON,
    ID_LAST
  };

  FF_EditPartner(FF_App* a,
		 FXComposite *parent,
		 const FXString& title,
		 FXuint opts=DECOR_TITLE|DECOR_BORDER,
		 FXint x=0, FXint y=0, FXint w=0, FXint h=0,
		 FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
		 FXint hs=0, FXint vs=0);
  ~FF_EditPartner();

  static bool editPartner(FF_App* app,
			  FXComposite *parent,
			  const FXString &title,
			  AE_PARTNER *pt);

  static bool selectPartnerAccount(FF_App* app,
				   FXComposite *parent,
				   const FXString &title,
				   AE_PARTNER *pt,
				   AE_BANKACCOUNT **pBankAccount);

  void init();
  void fini();

  FXint checkGui(FXbool quiet);
  FXint toGui(const AE_PARTNER *pt);
  FXint fromGui(AE_PARTNER *pt);

  void create();

  long onCmdComboBox(FXObject*, FXSelector, void *ptr);
  long onCmdTextField(FXObject*, FXSelector, void *ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

  long onCmdPartnerEditInvoice(FXObject*, FXSelector, void *ptr);

  FXint getDefaultHeight();

protected:
  FF_EditPartner();

  FF_App *m_app;
  AQDB_ID m_partnerId;

  FXTextField *m_lastNameEdit;
  FXTextField *m_firstNameEdit;
  FXTextField *m_titleEdit;
  FF_CommodityEdit *m_commodityEdit;

  FXCheckButton *m_customerCheck;
  FXCheckButton *m_vendorCheck;

  FXButton *m_editInvoiceButton;

  FXTabBook *m_tabBook;
  FXTabItem *m_addrTab;
  FXTabItem *m_accTab;
  FXTabItem *m_groupTab;
  FXTabItem *m_noteTab;
  FXTabItem *m_ruleTab;

  FF_AddressView *m_addressView;
  FF_BankAccountView *m_bankAccountView;
  FF_GroupView *m_groupView;
  FXText *m_noteEdit;

  FF_RuleView *m_ruleView;
};




#endif




