/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editrefueling.hpp"
#include "ff_dateedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/buffer.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditRefueling) FF_EditRefuelingMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditRefueling::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditRefueling::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditRefueling::ID_BUTTON_ANOTHER, FF_EditRefueling::onCmdWantAnother),
};


FXIMPLEMENT(FF_EditRefueling, FXDialogBox, FF_EditRefuelingMap, ARRAYNUMBER(FF_EditRefuelingMap))



FF_EditRefueling::FF_EditRefueling(FF_App* a,
                                   FXComposite *parent,
                                   const FXString& title,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h,
                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_dateEdit(NULL)
,m_amountEdit(NULL)
,m_priceEdit(NULL)
,m_mileageEdit(NULL)
,m_fullCheck(NULL)
,m_petrolStationEdit(NULL)
,m_fuelTypeCombo(NULL)
,m_anotherButton(NULL)
,m_wantAnother(false)
,m_commodityList(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("AE_Refueling|Date"), NULL, LABEL_NORMAL);
  m_dateEdit=new FF_DateEdit(m, this, ID_EDIT_DATE,
			     I18N("YYYY/MM/DD"),
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Refueling|Mileage"), NULL, LABEL_NORMAL);
  m_mileageEdit=new FXTextField(m, 40, this, ID_EDIT_MILEAGE,
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Refueling|Fuel Amount"), NULL, LABEL_NORMAL);
  m_amountEdit=new FXTextField(m, 40, this, ID_EDIT_AMOUNT,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Refueling|Fuel Price"), NULL, LABEL_NORMAL);
  m_priceEdit=new FXTextField(m, 40, this, ID_EDIT_PRICE,
			      TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Refueling|Petrol Station"), NULL, LABEL_NORMAL);
  m_petrolStationEdit=new FXTextField(m, 40, this, ID_EDIT_PETROLSTATION,
				      TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Refueling|Fuel Type"), NULL, LABEL_NORMAL);
  m_fuelTypeCombo=new FXComboBox(m, 40, this, ID_COMBO_FUELTYPE,
				 COMBOBOX_STATIC);
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|--select--"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|unknown"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Regular (87 octane)"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Regular (88 octane)"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Regular (89 octane)"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Regular (90 octane)"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Regular (91 octane)"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Super (95 octane)"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Super E10 (95 octane, 10% ethanol))"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Diesel"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|Gas"));
  m_fuelTypeCombo->appendItem(I18N("FF_EditRefueling|other"));

  new FXLabel(m, I18N("AE_Refueling|Memo"), NULL, LABEL_NORMAL);
  m_memoEdit=new FXTextField(m, 40, this, ID_EDIT_MEMO,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  m_fullCheck=new FXCheckButton(vf, I18N("AE_Refueling|Fully Fueled"), this, ID_CHECK_FULL,
				CHECKBUTTON_NORMAL);

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new FXSpring(hf, LAYOUT_FILL_X);
  if (opts & OPTIONS_MULTI)
    m_anotherButton=new THEMEBUTTON(hf, I18N("FF_EditRefueling|Another"), NULL, this, ID_BUTTON_ANOTHER,
				    BUTTON_NORMAL);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL);
}



FF_EditRefueling::FF_EditRefueling()
:FXDialogBox()
,m_app(NULL)
,m_dateEdit(NULL)
,m_amountEdit(NULL)
,m_priceEdit(NULL)
,m_mileageEdit(NULL)
,m_fullCheck(NULL)
,m_petrolStationEdit(NULL)
,m_fuelTypeCombo(NULL)
,m_anotherButton(NULL)
,m_wantAnother(false)
,m_commodityList(NULL)
{

}




FF_EditRefueling::~FF_EditRefueling() {
  AE_Commodity_List_free(m_commodityList);
  m_commodityList=(AE_COMMODITY_LIST*)(-1);
}



void FF_EditRefueling::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditRefueling");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditRefueling::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditRefueling", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



bool FF_EditRefueling::_checkValueAndUnits(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle) {
  FXString str;

  str=tf->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    errorTitle.text(),
			    "%s",
			    I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	tf->setFocus();
      }
      return false;
    }
    else {
      AQDB_VALUE *v=NULL;
      int units=0;
      int i;
      int rv;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(str.text(), &v, &units);
      if (rv<0) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Invalid value in input"));
          tf->setFocus();
        }
        return false;
      }
      AQDB_Value_free(v);
    }
  }
  else if (needed) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  errorTitle.text(),
			  "%s",
			  I18N("Missing input."));
      tf->setFocus();
    }
    return false;
  }

  return true;
}



bool FF_EditRefueling::_checkValueAndCommodity(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle) {
  FXString str;

  str=tf->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    errorTitle.text(),
			    "%s",
			    I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	tf->setFocus();
      }
      return false;
    }
    else {
      AQDB_VALUE *v=NULL;
      int i;
      int rv;
      AE_COMMODITY *co=NULL;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      rv=AE_Utils_AmountFromString(str.text(), m_commodityList, &v, &co);
      if (rv<0) {
        DBG_ERROR(0, "Internal error here (%d)", rv);
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Invalid amount or commodity in input"));
          tf->setFocus();
        }
        return false;
      }
      AQDB_Value_free(v);
    }
  }
  else if (needed) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  errorTitle.text(),
			  "%s",
			  I18N("Missing input."));
      tf->setFocus();
    }
    return false;
  }

  return true;
}



int FF_EditRefueling::_strToValue(FXbool needed, const FXString &str, AQDB_VALUE **pValue, int *pUnits) {
  if (str.empty()) {
    if (needed) {
      DBG_ERROR(0, "No data when needed");
      return GWEN_ERROR_NO_DATA;
    }
    return 0;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      int i;
      FXString nstr;
      int rv;

      /* replace textual comma with decimal point */
      nstr=str;
      i=nstr.find(',');
      if (i>=0)
	nstr[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(nstr.text(), pValue, pUnits);
      return rv;
    }
  }

  return 0;
}



int FF_EditRefueling::checkGui(FXbool quiet) {
  GWEN_DATE *ti;

  if (!_checkValueAndUnits(quiet, FALSE, m_amountEdit, I18N("FF_EditRefueling|Bad Fuel Amount")))
    return GWEN_ERROR_BAD_DATA;

  if (!_checkValueAndCommodity(quiet, FALSE, m_priceEdit, I18N("FF_EditRefueling|Bad Fuel Price")))
    return GWEN_ERROR_BAD_DATA;

  if (!_checkValueAndUnits(quiet, TRUE, m_mileageEdit, I18N("FF_EditRefueling|Bad Mileage")))
    return GWEN_ERROR_BAD_DATA;

  if (m_fuelTypeCombo->getCurrentItem()==0) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please select the fuel type"));
      m_fuelTypeCombo->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }

  if (!m_dateEdit->getDate(&ti)) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Bad date input"));
      m_dateEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  else
    GWEN_Date_free(ti);


  return 0;
}



int FF_EditRefueling::toGui(const AE_REFUELING *re) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_COMMODITY_LIST *col;
    int rv;
    AQDB_ID coId=0;
    const char *s;
    const AQDB_VALUE *v;
    const GWEN_DATE *dt;
    int units;

    /* read commodity list, we need it later */
    col=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, col);
    if (rv<0) {
      DBG_ERROR(0, "Internal error here (%d)", rv);
    }
    AE_Commodity_List_free(m_commodityList);
    m_commodityList=col;

#define TXTTOGUI(n, m)              \
  s=AE_Refueling_Get##m(re);         \
  if (s)                            \
    n->setText(s);

    TXTTOGUI(m_petrolStationEdit, PetrolStation);
    TXTTOGUI(m_memoEdit, Memo);

#undef TXTTOGUI

    units=AE_Refueling_GetFuelAmountUnits(re);
    v=AE_Refueling_GetFuelAmount(re);
    if (v) {
      GWEN_BUFFER *tbuf;
  
      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AE_Utils_ValueAndUnitsToString(v, units, tbuf, 2);
      m_amountEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }
  
    coId=AE_Refueling_GetCommodityId(re);
    v=AE_Refueling_GetFuelPrice(re);
    if (v) {
      GWEN_BUFFER *tbuf;
      int rv;
  
      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
  
      rv=AE_Utils_AmountToString(v, coId, m_commodityList, tbuf);
      if (rv<0) {
        DBG_ERROR(0, "Internal error here (%d)", rv);
        GWEN_Buffer_free(tbuf);
        return rv;
      }
      m_priceEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }
  
    units=AE_Refueling_GetMileageUnits(re);
    v=AE_Refueling_GetMileage(re);
    if (v) {
      GWEN_BUFFER *tbuf;
  
      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AE_Utils_ValueAndUnitsToString(v, units, tbuf, 2);
      m_mileageEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }
  
  
    dt=AE_Refueling_GetDate(re);
    if (dt)
      m_dateEdit->setDate(dt);
  
  
    switch(AE_Refueling_GetFuelType(re)) {
    case AE_Vehicle_FuelType_Unknown:       m_fuelTypeCombo->setCurrentItem(1, FALSE); break;
    case AE_Vehicle_FuelType_Octane87:      m_fuelTypeCombo->setCurrentItem(2, FALSE); break;
    case AE_Vehicle_FuelType_Octane88:      m_fuelTypeCombo->setCurrentItem(3, FALSE); break;
    case AE_Vehicle_FuelType_Octane89:      m_fuelTypeCombo->setCurrentItem(4, FALSE); break;
    case AE_Vehicle_FuelType_Octane90:      m_fuelTypeCombo->setCurrentItem(5, FALSE); break;
    case AE_Vehicle_FuelType_Octane91:      m_fuelTypeCombo->setCurrentItem(6, FALSE); break;
    case AE_Vehicle_FuelType_Octane95:      m_fuelTypeCombo->setCurrentItem(7, FALSE); break;
    case AE_Vehicle_FuelType_Octane95e10:   m_fuelTypeCombo->setCurrentItem(8, FALSE); break;
    case AE_Vehicle_FuelType_OtherGasoline: m_fuelTypeCombo->setCurrentItem(9, FALSE); break;
    case AE_Vehicle_FuelType_Diesel:        m_fuelTypeCombo->setCurrentItem(10, FALSE); break;
    case AE_Vehicle_FuelType_Gas:           m_fuelTypeCombo->setCurrentItem(11, FALSE); break;
    }
  
    if (AE_Refueling_GetFlags(re) & AE_REFUELING_FLAGS_FULL)
      m_fullCheck->setCheck(TRUE);
    else
      m_fullCheck->setCheck(FALSE);
  
    return 0;
  }
  else {
    DBG_ERROR(0, "Internal error: No book??");
    return GWEN_ERROR_INTERNAL;
  }
}



int FF_EditRefueling::fromGui(AE_REFUELING *re) {
  FXString str;
  int rv;
  AQDB_VALUE *v;
  int units;
  AE_VEHICLE_FUELTYPE ft=AE_Vehicle_FuelType_Unknown;

#define TXTFROMGUI(n, m)                   \
  str=n->getText().simplify();             \
  if (str.empty())                         \
    AE_Refueling_Set##m(re, NULL);         \
  else                                     \
    AE_Refueling_Set##m(re, str.text())

  TXTFROMGUI(m_petrolStationEdit, PetrolStation);
  TXTFROMGUI(m_memoEdit, Memo);

#undef TXTFROMGUI


  str=m_amountEdit->getText().simplify();
  rv=_strToValue(TRUE, str, &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "Bad amount");
    return rv;
  }
  AE_Refueling_SetFuelAmount(re, v);
  AQDB_Value_free(v);
  AE_Refueling_SetFuelAmountUnits(re, units);

  str=m_priceEdit->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      int i;
      FXString nstr;
      int rv;
      AE_COMMODITY *co=NULL;

      /* replace textual comma with decimal point */
      nstr=str;
      i=nstr.find(',');
      if (i>=0)
        nstr[i]='.';

      rv=AE_Utils_AmountFromString(nstr.text(), m_commodityList, &v, &co);
      if (rv<0) {
        DBG_ERROR(0, "Internal error here (%d)", rv);
        return rv;
      }
      AE_Refueling_SetFuelPrice(re, v);
      if (co)
        AE_Refueling_SetCommodityId(re, AE_Commodity_GetId(co));
      else {
        DBG_ERROR(0, "No matching commodity");
      }
      AQDB_Value_free(v);
    }
  }

  str=m_mileageEdit->getText().simplify();
  rv=_strToValue(TRUE, str, &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "Bad amount");
    return rv;
  }
  AE_Refueling_SetMileage(re, v);
  AE_Refueling_SetMileageUnits(re, units);
  AQDB_Value_free(v);

  if (m_dateEdit) {
    GWEN_DATE *ti;

    if (!m_dateEdit->getDate(&ti)) {
      DBG_ERROR(0, "Bad date");
      return GWEN_ERROR_BAD_DATA;
    }
    AE_Refueling_SetDate(re, ti);
    GWEN_Date_free(ti);
  }

  switch(m_fuelTypeCombo->getCurrentItem()) {
  case 2:  ft=AE_Vehicle_FuelType_Octane87; break;
  case 3:  ft=AE_Vehicle_FuelType_Octane88; break;
  case 4:  ft=AE_Vehicle_FuelType_Octane89; break;
  case 5:  ft=AE_Vehicle_FuelType_Octane90; break;
  case 6:  ft=AE_Vehicle_FuelType_Octane91; break;
  case 7:  ft=AE_Vehicle_FuelType_Octane95; break;
  case 8:  ft=AE_Vehicle_FuelType_Octane95e10; break;
  case 9:  ft=AE_Vehicle_FuelType_OtherGasoline; break;
  case 10: ft=AE_Vehicle_FuelType_Diesel; break;
  case 11: ft=AE_Vehicle_FuelType_Gas; break;
  default:
  case 1:  ft=AE_Vehicle_FuelType_Unknown; break;
  }
  AE_Refueling_SetFuelType(re, ft);

  if (m_fullCheck->getCheck()==TRUE)
    AE_Refueling_AddFlags(re, AE_REFUELING_FLAGS_FULL);
  else
    AE_Refueling_SubFlags(re, AE_REFUELING_FLAGS_FULL);

  return 0;
}



void FF_EditRefueling::create() {
  FXDialogBox::create();
}



long FF_EditRefueling::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditRefueling::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    m_wantAnother=false;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditRefueling::onCmdWantAnother(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    m_wantAnother=true;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditRefueling::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



bool FF_EditRefueling::editRefueling(FF_App* app,
                                     FXComposite *parent,
                                     const FXString &title,
                                     AE_REFUELING *re) {
  FF_EditRefueling dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(re);
  if (res)
    return false;
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);

  dlg.m_dateEdit->setFocus();
  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(re);
  if (res==0)
    return true;

  return false;
}



bool FF_EditRefueling::createRefuelings(FF_App* app,
					FXComposite *parent,
					const FXString &title,
					const AE_REFUELING *origTmpl) {
  FF_EditRefueling dlg(app, parent, title, DECOR_ALL|FF_EditRefueling::OPTIONS_MULTI);
  FXuint dlgres;
  FXint res;
  AE_REFUELING *tmpl;
  AE_BOOK *b;

  b=app->getBook();
  if (b) {
    dlg.create();
    dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
    dlg.init();
    dlg.show(PLACEMENT_VISIBLE);
  
    tmpl=AE_Refueling_dup(origTmpl);
    for(;;) {
      AE_REFUELING *re;
      int rv;
  
      re=AE_Refueling_dup(tmpl);
      res=dlg.toGui(re);
      if (res) {
	dlg.fini();
	AE_Refueling_free(re);
	AE_Refueling_free(tmpl);
	dlg.hide();
	return false;
      }
  
      dlg.m_wantAnother=false;
      dlg.m_dateEdit->setFocus();
      dlgres=dlg.execute(PLACEMENT_VISIBLE);
      DBG_ERROR(0, "Result: %d", dlgres);
      if (dlgres==0) {
	dlg.fini();
	AE_Refueling_free(re);
	AE_Refueling_free(tmpl);
	dlg.hide();
	return false;
      }
  
      res=dlg.fromGui(re);
      if (res!=0) {
	DBG_ERROR(0, "Should not happen");
	dlg.fini();
	AE_Refueling_free(re);
	AE_Refueling_free(tmpl);
	dlg.hide();
	return false;
      }

      /* create refueling */
      rv=AE_Book_AddRefueling(b, AE_Book_TableType_Refueling, re, 1);
      if (rv<0) {
	dlg.fini();
	AE_Refueling_free(re);
	AE_Refueling_free(tmpl);
	dlg.hide();
	return false;
      }
      app->signalDbReload(AE_Book_TableType_Refueling);

      /* make current refueling the template for the next round */
      AE_Refueling_free(tmpl);
      tmpl=re;
      AE_Refueling_SetId(tmpl, 0);
      AE_Refueling_SetDbId(tmpl, 0);
      AE_Refueling_SetFuelAmount(tmpl, NULL);
      AE_Refueling_SetFuelPrice(tmpl, NULL);
      AE_Refueling_SetMileage(tmpl, NULL);

      if (!dlg.wantAnother()) {
	dlg.fini();
	AE_Refueling_free(tmpl);
	dlg.hide();
	return true;
      }
  
    }
  }
  else {
    DBG_ERROR(0, "No open book");
    return false;
  }
}









