/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITREFUELING_HPP
#define FF_EDITREFUELING_HPP


#include <fx.h>


#include <aqfinance/engine/book/ae_refueling.h>
#include <aqfinance/engine/book/ae_commodity.h>


class FF_App;
class FF_DateEdit;


class FF_EditRefueling: public FXDialogBox {
  FXDECLARE(FF_EditRefueling)

public:

  enum {
    ID_EDIT_NAME=FXDialogBox::ID_LAST,
    ID_EDIT_DATE,
    ID_EDIT_AMOUNT,
    ID_EDIT_PRICE,
    ID_EDIT_MILEAGE,
    ID_EDIT_MEMO,
    ID_EDIT_PETROLSTATION,
    ID_CHECK_FULL,
    ID_COMBO_FUELTYPE,
    ID_BUTTON_ANOTHER,
    ID_LAST
  };

  enum {
    OPTIONS_MULTI=0x04000000
  };


  FF_EditRefueling(FF_App* a,
                   FXComposite *parent,
                   const FXString& title,
                   FXuint opts=DECOR_TITLE|DECOR_BORDER,
                   FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                   FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
                   FXint hs=0, FXint vs=0);
  virtual ~FF_EditRefueling();


  static bool editRefueling(FF_App* app,
                            FXComposite *parent,
                            const FXString &title,
                            AE_REFUELING *re);
  static bool createRefuelings(FF_App* app,
                               FXComposite *parent,
                               const FXString &title,
                               const AE_REFUELING *origTmpl);


  bool wantAnother() const { return m_wantAnother;};


  void init();
  void fini();

  int checkGui(FXbool quiet);
  int toGui(const AE_REFUELING *re);
  int fromGui(AE_REFUELING *re);

  void create();

  long onCmdTextField(FXObject*, FXSelector, void *ptr);
  long onCmdWantAnother(FXObject*, FXSelector, void *ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

protected:
  FF_EditRefueling();

  bool _checkValueAndCommodity(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle);
  bool _checkValueAndUnits(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle);
  int _strToValue(FXbool needed, const FXString &str, AQDB_VALUE **pValue, int *pUnits);

  FF_App *m_app;

  FF_DateEdit *m_dateEdit;
  FXTextField *m_amountEdit;
  FXTextField *m_priceEdit;
  FXTextField *m_mileageEdit;
  FXCheckButton *m_fullCheck;
  FXTextField *m_petrolStationEdit;
  FXComboBox *m_fuelTypeCombo;

  FXTextField *m_memoEdit;
  FXButton *m_anotherButton;

  bool m_wantAnother;
  AE_COMMODITY_LIST *m_commodityList;
};




#endif




