/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editsimpletransaction.hpp"
#include "ff_dateedit.hpp"
#include "ff_accountedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/buffer.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditSimpleTransaction) FF_EditSimpleTransactionMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditSimpleTransaction::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditSimpleTransaction::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditSimpleTransaction::ID_BUTTON_SAMEDAY, FF_EditSimpleTransaction::onCmdSameDay),
  FXMAPFUNC(SEL_COMMAND, FF_EditSimpleTransaction::ID_BUTTON_NEXTDAY, FF_EditSimpleTransaction::onCmdNextDay),
  FXMAPFUNC(SEL_COMMAND, FF_EditSimpleTransaction::ID_BUTTON_NEXTWEEK, FF_EditSimpleTransaction::onCmdNextWeek),
  FXMAPFUNC(SEL_COMMAND, FF_EditSimpleTransaction::ID_BUTTON_NEXTMONTH, FF_EditSimpleTransaction::onCmdNextMonth),
  FXMAPFUNC(SEL_COMMAND, FF_EditSimpleTransaction::ID_EDIT_BOOKINGDATE, FF_EditSimpleTransaction::onCmdBookingDate),
};


FXIMPLEMENT(FF_EditSimpleTransaction, FXDialogBox, FF_EditSimpleTransactionMap, ARRAYNUMBER(FF_EditSimpleTransactionMap))



FF_EditSimpleTransaction::FF_EditSimpleTransaction(FF_App* a,
                                                   FXComposite *parent,
                                                   const FXString& title,
                                                   FXuint opts,
                                                   FXint x, FXint y, FXint w, FXint h,
                                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                                   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_bookingDateEdit(NULL)
,m_valutaDateEdit(NULL)
,m_amountEdit(NULL)
,m_checkNumberEdit(NULL)
,m_memoEdit(NULL)
,m_sameDayButton(NULL)
,m_nextDayButton(NULL)
,m_nextWeekButton(NULL)
,m_nextMonthButton(NULL)
,m_wantAnother(Another_None)
,m_accountTree(NULL)
,m_commodityList(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("AE_Transaction|Booking Date"), NULL, LABEL_NORMAL);
  m_bookingDateEdit=new FF_DateEdit(m, this, ID_EDIT_BOOKINGDATE,
                                    I18N("YYYY/MM/DD"),
                                    TEXTFIELD_NORMAL);

  new FXLabel(m, I18N("AE_Transaction|Valuta Date"), NULL, LABEL_NORMAL);
  m_valutaDateEdit=new FF_DateEdit(m, this, ID_EDIT_VALUTADATE,
                                   I18N("YYYY/MM/DD"),
                                   TEXTFIELD_NORMAL);

  gb=new FXGroupBox(vf, I18N("FF_EditSimpleTransaction|Transaction from/to"),
                    LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X );

  new FXLabel(m, I18N("AE_Transaction|From Account"), NULL, LABEL_NORMAL);
  m_fromAccountEdit=new FF_AccountEdit(m,  this, ID_EDIT_FROMACCOUNT, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("AE_Transaction|To Account"), NULL, LABEL_NORMAL);
  m_toAccountEdit=new FF_AccountEdit(m,  this, ID_EDIT_TOACCOUNT, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);


  gb=new FXGroupBox(vf, I18N("FF_EditSimpleTransaction|Basic Information"),
                    LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("AE_Transaction|Amount"), NULL, LABEL_NORMAL);
  m_amountEdit=new FXTextField(m, 40, this, ID_EDIT_AMOUNT,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Transaction|Check Number"), NULL, LABEL_NORMAL);
  m_checkNumberEdit=new FXTextField(m, 40, this, ID_EDIT_CHECKNUMBER,
                                    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Transaction|Memo"), NULL, LABEL_NORMAL);
  m_memoEdit=new FXTextField(m, 40, this, ID_EDIT_MEMO,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  if (opts & OPTIONS_MULTI) {
    gb=new FXGroupBox(vf, I18N("FF_EditSimpleTransaction|Create another transaction..."),
                      LAYOUT_FILL_X | LAYOUT_FILL_Y |
                      FRAME_LINE);
    hf=new FXHorizontalFrame(gb, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
    new THEMEBUTTON(hf, I18N("&Same Day"), NULL, this, ID_BUTTON_SAMEDAY, BUTTON_NORMAL);
    new THEMEBUTTON(hf, I18N("Next &Day"), NULL, this, ID_BUTTON_NEXTDAY, BUTTON_NORMAL);
    new THEMEBUTTON(hf, I18N("Next &Week"), NULL, this, ID_BUTTON_NEXTWEEK, BUTTON_NORMAL);
    new THEMEBUTTON(hf, I18N("Next &Month"), NULL, this, ID_BUTTON_NEXTMONTH, BUTTON_NORMAL);
  }

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new FXSpring(hf, LAYOUT_FILL_X);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT, BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL, BUTTON_NORMAL);
}



FF_EditSimpleTransaction::FF_EditSimpleTransaction()
:FXDialogBox()
,m_app(NULL)
,m_bookingDateEdit(NULL)
,m_valutaDateEdit(NULL)
,m_amountEdit(NULL)
,m_checkNumberEdit(NULL)
,m_memoEdit(NULL)
,m_sameDayButton(NULL)
,m_nextDayButton(NULL)
,m_nextWeekButton(NULL)
,m_nextMonthButton(NULL)
,m_wantAnother(Another_None)
,m_accountTree(NULL)
,m_commodityList(NULL)
{

}




FF_EditSimpleTransaction::~FF_EditSimpleTransaction() {
  AE_Commodity_List_free(m_commodityList);
  m_commodityList=(AE_COMMODITY_LIST*)(-1);

  AE_Account_Tree_free(m_accountTree);
  m_accountTree=(AE_ACCOUNT_TREE*)(-1);
}



void FF_EditSimpleTransaction::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditSimpleTransaction");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditSimpleTransaction::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditSimpleTransaction", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



bool FF_EditSimpleTransaction::_checkValueAndUnits(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle) {
  FXString str;

  str=tf->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    errorTitle.text(),
			    "%s",
			    I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	tf->setFocus();
      }
      return false;
    }
    else {
      AQDB_VALUE *v=NULL;
      int units=0;
      int i;
      int rv;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(str.text(), &v, &units);
      if (rv<0) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Invalid value in input"));
          tf->setFocus();
        }
        return false;
      }
      AQDB_Value_free(v);
    }
  }
  else if (needed) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  errorTitle.text(),
			  "%s",
			  I18N("Missing input."));
      tf->setFocus();
    }
    return false;
  }

  return true;
}



bool FF_EditSimpleTransaction::_checkValueAndCommodity(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle) {
  FXString str;

  str=tf->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    errorTitle.text(),
			    "%s",
			    I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	tf->setFocus();
      }
      return false;
    }
    else {
      AQDB_VALUE *v=NULL;
      int i;
      int rv;
      AE_COMMODITY *co=NULL;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      rv=AE_Utils_AmountFromString(str.text(), m_commodityList, &v, &co);
      if (rv<0) {
        DBG_ERROR(0, "Internal error here (%d)", rv);
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Invalid amount or commodity in input"));
          tf->setFocus();
        }
        return false;
      }
      AQDB_Value_free(v);
    }
  }
  else if (needed) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  errorTitle.text(),
			  "%s",
			  I18N("Missing input."));
      tf->setFocus();
    }
    return false;
  }

  return true;
}



int FF_EditSimpleTransaction::_strToValue(FXbool needed, const FXString &str, AQDB_VALUE **pValue, int *pUnits) {
  if (str.empty()) {
    if (needed) {
      DBG_ERROR(0, "No data when needed");
      return GWEN_ERROR_NO_DATA;
    }
    return 0;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      int i;
      FXString nstr;
      int rv;

      /* replace textual comma with decimal point */
      nstr=str;
      i=nstr.find(',');
      if (i>=0)
	nstr[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(nstr.text(), pValue, pUnits);
      return rv;
    }
  }

  return 0;
}



int FF_EditSimpleTransaction::checkGui(FXbool quiet) {
  AE_TRANSACTION *t;
  int rv;

  t=AE_Transaction_new();
  rv=_fromGui(t, quiet);
  if (rv<0) {
    DBG_INFO(0, "here");
    AE_Transaction_free(t);
    return GWEN_ERROR_BAD_DATA;
  }

  if (!m_app->checkTransaction(this, m_accountTree, m_commodityList, t, quiet)) {
    DBG_INFO(0, "here");
    AE_Transaction_free(t);
    return GWEN_ERROR_BAD_DATA;
  }

  AE_Transaction_free(t);

  return 0;
}



int FF_EditSimpleTransaction::reload() {
  AE_BOOK *b;
  AE_COMMODITY_LIST *col;
  AE_ACCOUNT_TREE *at;
  int rv;

  b=m_app->getBook();
  assert(b);

  /* read commodity list, we need it later */
  col=AE_Commodity_List_new();
  rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, col);
  if (rv<0) {
    DBG_ERROR(0, "Internal error here (%d)", rv);
    AE_Commodity_List_free(col);
    return rv;
  }
  AE_Commodity_List_free(m_commodityList);
  m_commodityList=col;

  /* read account tree */
  at=AE_Account_Tree_new();
  rv=AE_Book_ReadAccountTree(m_app->getBook(), AE_Book_TableType_Account, at);
  if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Account_Tree_free(at);
      return rv;
  }
  AE_Account_Tree_free(m_accountTree);
  m_accountTree=at;

  /* let widgets reload needed data */
  m_fromAccountEdit->reload();
  m_toAccountEdit->reload();

  /* done */
  return 0;
}



int FF_EditSimpleTransaction::toGui(const AE_TRANSACTION *t) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    const char *s;
    const char *transMemo=NULL;
    const GWEN_DATE *dt;
    const AE_SPLIT_LIST *sl;
    AQDB_ID fromAccId=0;
    AQDB_ID toAccId=0;

    m_fromAccountEdit->reload();
    m_toAccountEdit->reload();

    dt=AE_Transaction_GetDate(t);
    if (dt)
      m_bookingDateEdit->setDate(dt);

    dt=AE_Transaction_GetValutaDate(t);
    if (dt)
      m_valutaDateEdit->setDate(dt);

    s=AE_Transaction_GetCheckNumber(t);
    if (s && *s)
      m_checkNumberEdit->setText(s);

    transMemo=AE_Transaction_GetMemo(t);
    if (transMemo && *transMemo)
      m_memoEdit->setText(transMemo);

    sl=AE_Transaction_GetSplitList(t);
    if (sl) {
      AE_SPLIT *sp;

      sp=AE_Split_List_First(sl);
      if (sp) {
        const AQDB_VALUE *v;
        AQDB_ID coId=0;

        if (!(transMemo && *transMemo)) {
          s=AE_Split_GetMemo(sp);
          if (s && *s) {
            transMemo=s;
            m_memoEdit->setText(s);
          }
        }

        v=AE_Split_GetAmount(sp);
        if (v) {
          AQDB_ID accId;

          accId=AE_Split_GetAccountId(sp);
          if (AQDB_Value_IsNegative(v)) {
            fromAccId=accId;
            m_fromAccountEdit->setAccountId(accId);
          }
          else {
            toAccId=accId;
            m_toAccountEdit->setAccountId(accId);
          }
        }
        coId=AE_Split_GetCommodityId(sp);

        sp=AE_Split_List_Next(sp);
        if (sp) {
          AQDB_ID accId;

          if (!(transMemo && *transMemo)) {
            s=AE_Split_GetMemo(sp);
            if (s && *s) {
              transMemo=s;
              m_memoEdit->setText(s);
            }
          }

          accId=AE_Split_GetAccountId(sp);
          if (AQDB_Value_IsNegative(v)) {
            if (fromAccId==0)
              m_fromAccountEdit->setAccountId(accId);
          }
          else {
            if (toAccId==0)
              m_toAccountEdit->setAccountId(accId);
          }
          if (v==NULL)
            v=AE_Split_GetAmount(sp);
          if (coId==0)
            coId=AE_Split_GetCommodityId(sp);
        }

        if (v) {
          if (AQDB_Value_IsNegative(v)) {
            AQDB_VALUE *vpos;
            GWEN_BUFFER *tbuf;
            int rv;

            vpos=AQDB_Value_dup(v);
            AQDB_Value_Negate(vpos);

            tbuf=GWEN_Buffer_new(0, 32, 0, 1);

            rv=AE_Utils_AmountToString(vpos, coId, m_commodityList, tbuf);
            if (rv<0) {
              DBG_ERROR(0, "Internal error here (%d)", rv);
              GWEN_Buffer_free(tbuf);
              return rv;
            }
            m_amountEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
            GWEN_Buffer_free(tbuf);
          }
          else {
            GWEN_BUFFER *tbuf;
            int rv;

            tbuf=GWEN_Buffer_new(0, 32, 0, 1);

            rv=AE_Utils_AmountToString(v, coId, m_commodityList, tbuf);
            if (rv<0) {
              DBG_ERROR(0, "Internal error here (%d)", rv);
              GWEN_Buffer_free(tbuf);
              return rv;
            }
            m_amountEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
            GWEN_Buffer_free(tbuf);
          }
        }
      }
    }

    return 0;
  }
  else {
    DBG_ERROR(0, "Internal error: No book??");
    return GWEN_ERROR_INTERNAL;
  }
}



int FF_EditSimpleTransaction::_fromGui(AE_TRANSACTION *t, FXbool quiet) {
  FXString str;
  GWEN_DATE *dt;
  AQDB_VALUE *v;
  AE_SPLIT_LIST *sl;
  AE_SPLIT *sp1;
  AE_SPLIT *sp2;
  AQDB_ID id;

  sl=AE_Transaction_GetSplitList(t);
  if (sl==NULL) {
    sl=AE_Split_List_new();
    AE_Transaction_SetSplitList(t, sl);
  }
  else
    AE_Split_List_Clear(sl);

  /* create empty splits, add them */
  sp1=AE_Split_new();
  AE_Split_List_Add(sp1, sl);
  sp2=AE_Split_new();
  AE_Split_List_Add(sp2, sl);

  /* date */
  if (!m_bookingDateEdit->getDate(&dt)) {
    DBG_ERROR(0, "Bad booking date");
    if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
                            "%s",
                            I18N("Bad booking date input"));
        m_bookingDateEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  AE_Transaction_SetDate(t, dt);
  GWEN_Date_free(dt);

  /* valuta date */
  if (!m_valutaDateEdit->getDate(&dt)) {
    DBG_ERROR(0, "Bad valuta date");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Bad booking date input"));
      m_bookingDateEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  AE_Transaction_SetValutaDate(t, dt);
  GWEN_Date_free(dt);

  /* checkNumber */
  str=m_checkNumberEdit->getText().simplify();
  if (str.empty())
    AE_Transaction_SetCheckNumber(t, NULL);
  else
    AE_Transaction_SetCheckNumber(t, str.text());

  /* memo */
  str=m_memoEdit->getText().simplify();
  if (str.empty())
    AE_Transaction_SetMemo(t, NULL);
  else
    AE_Transaction_SetMemo(t, str.text());

  /* from account */
  id=m_fromAccountEdit->getAccountId();
  if (id==0) {
    DBG_ERROR(0, "No \"from\" account");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing source account"));
      m_fromAccountEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  AE_Split_SetAccountId(sp1, id);

  /* to account */
  id=m_toAccountEdit->getAccountId();
  if (id==0) {
    DBG_ERROR(0, "No \"to\" account");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Missing destination account"));
      m_toAccountEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  AE_Split_SetAccountId(sp2, id);

  /* amount */
  str=m_amountEdit->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Multiple types of comma detected, use only one comma type."));
	m_amountEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      int i;
      FXString nstr;
      int rv;
      AE_COMMODITY *co=NULL;

      /* replace textual comma with decimal point */
      nstr=str;
      i=nstr.find(',');
      if (i>=0)
        nstr[i]='.';

      rv=AE_Utils_AmountFromString(nstr.text(), m_commodityList, &v, &co);
      if (rv<0) {
        DBG_ERROR(0, "Internal error here (%d)", rv);
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Invalid amount entered."));
	  m_amountEdit->setFocus();
	}
        return rv;
      }
      if (co==NULL) {
        AE_ACCOUNT *a;
        AQDB_ID coId=0;

        a=AE_Account_Tree_GetById(m_accountTree, AE_Split_GetAccountId(sp1));
        if (a)
          coId=AE_Account_GetPrimaryCommodityId(a);
        if (coId==0) {
          a=AE_Account_Tree_GetById(m_accountTree, AE_Split_GetAccountId(sp2));
          if (a)
            coId=AE_Account_GetPrimaryCommodityId(a);
        }
        if (coId)
          co=AE_Commodity_List_GetById(m_commodityList, coId);
      }

      /* direct value to 2nd split */
      AE_Split_SetAmount(sp2, v);
      if (co)
        AE_Split_SetCommodityId(sp2, AE_Commodity_GetId(co));

      /* negated amount to 1st split */
      AQDB_Value_Negate(v);
      AE_Split_SetAmount(sp1, v);
      if (co)
        AE_Split_SetCommodityId(sp1, AE_Commodity_GetId(co));
    }
  }

  return 0;
}



int FF_EditSimpleTransaction::fromGui(AE_TRANSACTION *t) {
  int rv;

  rv=_fromGui(t, TRUE);
  if (rv<0) {
    DBG_INFO(0, "here");
    return rv;
  }

  if (!m_app->checkTransaction(this, m_accountTree, m_commodityList, t, TRUE)) {
    DBG_INFO(0, "here");
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}



void FF_EditSimpleTransaction::create() {
  FXDialogBox::create();
}



long FF_EditSimpleTransaction::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditSimpleTransaction::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    m_wantAnother=Another_None;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditSimpleTransaction::onCmdSameDay(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Another on same day");
  if (checkGui(FALSE)==0) {
    DBG_ERROR(0, "GUI data seems to be ok");
    m_wantAnother=Another_SameDay;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditSimpleTransaction::onCmdNextDay(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Another on next day");
  if (checkGui(FALSE)==0) {
    m_wantAnother=Another_NextDay;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditSimpleTransaction::onCmdNextWeek(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Another on next week");
  if (checkGui(FALSE)==0) {
    m_wantAnother=Another_NextWeek;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditSimpleTransaction::onCmdNextMonth(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Another on next month");
  if (checkGui(FALSE)==0) {
    m_wantAnother=Another_NextMonth;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditSimpleTransaction::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_EditSimpleTransaction::onCmdBookingDate(FXObject*, FXSelector, void *ptr) {
  GWEN_DATE *dt;

  /* date */
  if (m_bookingDateEdit->getDate(&dt)) {
    DBG_ERROR(0, "Setting valuta date to booking date");
    m_valutaDateEdit->setDate(dt);
    GWEN_Date_free(dt);
  }
  else {
    DBG_ERROR(0, "No valid date yet");
  }

  return 1;
}



bool FF_EditSimpleTransaction::editTransaction(FF_App* app,
                                               FXComposite *parent,
                                               const FXString &title,
                                               AE_TRANSACTION *t) {
  FF_EditSimpleTransaction dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(t);
  if (res)
    return false;
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
  dlg.init();
  dlg.reload();
  dlg.show(PLACEMENT_VISIBLE);

  dlg.m_bookingDateEdit->setFocus();
  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(t);
  if (res==0)
    return true;

  return false;
}



bool FF_EditSimpleTransaction::createTransactions(FF_App* app,
                                                  FXComposite *parent,
                                                  const FXString &title,
                                                  const AE_TRANSACTION *origTmpl) {
  FF_EditSimpleTransaction dlg(app, parent, title, DECOR_ALL|FF_EditSimpleTransaction::OPTIONS_MULTI);
  FXuint dlgres;
  FXint res;
  AE_TRANSACTION *tmpl;
  AE_BOOK *b;
  int loop;

  b=app->getBook();
  if (b) {
    dlg.create();
    dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
    dlg.init();
    dlg.show(PLACEMENT_VISIBLE);
  
    tmpl=AE_Transaction_dup(origTmpl);
    for(loop=0;;loop++) {
      AE_TRANSACTION *t;
      const GWEN_DATE *dt;
      int rv;
  
      dlg.reload();
      t=AE_Transaction_dup(tmpl);
      res=dlg.toGui(t);
      if (res) {
	dlg.fini();
	AE_Transaction_free(t);
	AE_Transaction_free(tmpl);
	dlg.hide();
	return false;
      }
  
      dlg.m_wantAnother=Another_None;
      if (loop==0)
        dlg.m_bookingDateEdit->setFocus();
      else
        dlg.m_memoEdit->setFocus();
      dlgres=dlg.execute(PLACEMENT_VISIBLE);
      DBG_ERROR(0, "Result: %d", dlgres);
      if (dlgres==0) {
	dlg.fini();
	AE_Transaction_free(t);
	AE_Transaction_free(tmpl);
	dlg.hide();
	return false;
      }
  
      res=dlg.fromGui(t);
      if (res!=0) {
	DBG_ERROR(0, "Should not happen");
	dlg.fini();
	AE_Transaction_free(t);
	AE_Transaction_free(tmpl);
	dlg.hide();
	return false;
      }

      /* apply transaction (i.e. add it) */
      rv=AE_Book_ApplyTransaction(b, t, NULL, 1);
      if (rv<0) {
	dlg.fini();
	AE_Transaction_free(t);
	AE_Transaction_free(tmpl);
	dlg.hide();
	return false;
      }
      app->signalDbReload(AE_Book_TableType_Transaction);

      /* make current refueling the template for the next round */
      AE_Transaction_free(tmpl);
      tmpl=t;
      AE_Transaction_SetId(tmpl, 0);
      AE_Transaction_SetDbId(tmpl, 0);

      switch(dlg.wantAnother()) {
      case Another_None:
	/* not another one, done */
	dlg.fini();
        AE_Transaction_free(tmpl);
        dlg.hide();
	return true;

      case Another_SameDay:
	/* same day, no changes */
	break;

      case Another_NextDay:
	dt=AE_Transaction_GetDate(tmpl);
	if (dt) {
	  int j;
	  GWEN_DATE *nd;

	  j=GWEN_Date_GetJulian(dt);
	  j++;
	  nd=GWEN_Date_fromJulian(j);
	  AE_Transaction_SetDate(tmpl, nd);
	  AE_Transaction_SetValutaDate(tmpl, nd);
	  GWEN_Date_free(nd);
	}
	break;

      case Another_NextWeek:
	dt=AE_Transaction_GetDate(tmpl);
	if (dt) {
	  int j;
	  GWEN_DATE *nd;

	  j=GWEN_Date_GetJulian(dt);
	  j+=7;
	  nd=GWEN_Date_fromJulian(j);
	  AE_Transaction_SetDate(tmpl, nd);
	  AE_Transaction_SetValutaDate(tmpl, nd);
	  GWEN_Date_free(nd);
	}
	break;

      case Another_NextMonth:
	dt=AE_Transaction_GetDate(tmpl);
	if (dt) {
	  int y, m, j;
	  GWEN_DATE *nd;

	  j=GWEN_Date_GetJulian(dt);
	  y=GWEN_Date_GetYear(dt);
	  m=GWEN_Date_GetMonth(dt);
	  switch(m) {
	  case 1:
	  case 3:
	  case 5:
	  case 7:
	  case 8:
	  case 10:
	  case 12: /* months with 31 days */
	    j+=31;
	    break;
	  case 2:
	    if (GWEN_Date_IsLeapYear(y))
	      j+=29;
	    else
	      j+=28;
	    break;
	  default:
	    j+=30;
	    break;
	  }
	  nd=GWEN_Date_fromJulian(j);
	  AE_Transaction_SetDate(tmpl, nd);
	  AE_Transaction_SetValutaDate(tmpl, nd);
	  GWEN_Date_free(nd);

	}
	break;

      default:
        break;
      }
    }
  }
  else {
    DBG_ERROR(0, "No open book");
    return false;
  }
}









