/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITSIMPLETRANSACTION_HPP
#define FF_EDITSIMPLETRANSACTION_HPP


#include <fx.h>


#include <aqfinance/engine/book/ae_transaction.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_account.h>


class FF_App;
class FF_DateEdit;
class FF_AccountEdit;


class FF_EditSimpleTransaction: public FXDialogBox {
  FXDECLARE(FF_EditSimpleTransaction)

public:

  enum {
    ID_EDIT_NAME=FXDialogBox::ID_LAST,
    ID_EDIT_BOOKINGDATE,
    ID_EDIT_VALUTADATE,
    ID_EDIT_FROMACCOUNT,
    ID_EDIT_TOACCOUNT,
    ID_EDIT_AMOUNT,
    ID_EDIT_CHECKNUMBER,
    ID_EDIT_MEMO,

    ID_BUTTON_SAMEDAY,
    ID_BUTTON_NEXTDAY,
    ID_BUTTON_NEXTWEEK,
    ID_BUTTON_NEXTMONTH,
    ID_LAST
  };

  enum {
    OPTIONS_MULTI=0x04000000
  };


  FF_EditSimpleTransaction(FF_App* a,
                           FXComposite *parent,
                           const FXString& title,
                           FXuint opts=DECOR_TITLE|DECOR_BORDER,
                           FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                           FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
                           FXint hs=0, FXint vs=0);
  virtual ~FF_EditSimpleTransaction();


  static bool editTransaction(FF_App* app,
                              FXComposite *parent,
                              const FXString &title,
                              AE_TRANSACTION *t);

  static bool createTransactions(FF_App* app,
                                 FXComposite *parent,
                                 const FXString &title,
                                 const AE_TRANSACTION *origTmpl);


  int wantAnother() const { return m_wantAnother;};


  void init();
  void fini();

  int checkGui(FXbool quiet);
  int toGui(const AE_TRANSACTION *t);
  int fromGui(AE_TRANSACTION *t);

  void create();

  int reload();

  long onCmdTextField(FXObject*, FXSelector, void *ptr);
  long onCmdBookingDate(FXObject*, FXSelector, void *ptr);

  long onCmdSameDay(FXObject*, FXSelector, void *ptr);
  long onCmdNextDay(FXObject*, FXSelector, void *ptr);
  long onCmdNextWeek(FXObject*, FXSelector, void *ptr);
  long onCmdNextMonth(FXObject*, FXSelector, void *ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

protected:
  enum {
    Another_None=0,
    Another_SameDay,
    Another_NextDay,
    Another_NextWeek,
    Another_NextMonth
  };

  FF_EditSimpleTransaction();

  bool _checkValueAndCommodity(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle);
  bool _checkValueAndUnits(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle);
  int _strToValue(FXbool needed, const FXString &str, AQDB_VALUE **pValue, int *pUnits);
  int _fromGui(AE_TRANSACTION *t, FXbool quiet);

  FF_App *m_app;

  FF_DateEdit *m_bookingDateEdit;
  FF_DateEdit *m_valutaDateEdit;
  FF_AccountEdit *m_fromAccountEdit;
  FF_AccountEdit *m_toAccountEdit;
  FXTextField *m_amountEdit;
  FXTextField *m_checkNumberEdit;
  FXTextField *m_memoEdit;

  FXButton *m_sameDayButton;
  FXButton *m_nextDayButton;
  FXButton *m_nextWeekButton;
  FXButton *m_nextMonthButton;

  int m_wantAnother;
  AE_ACCOUNT_TREE *m_accountTree;
  AE_COMMODITY_LIST *m_commodityList;
};




#endif




