/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_editstmrule.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg


static FF_EditRule::FieldDescription _fields[]={
  {"id", I18S("Id")},
  {"bankAccountId", I18S("Bank Account Id")},
  {"localCountry", I18S("Local Country")},
  {"localBankCode", I18S("Local Bank Code")},
  {"localAccountNumber", I18S("Local Account Number")},
  {"localiban", I18S("Local IBAN")},
  {"localbic", I18S("Local BIC")},
  {"localname", I18S("Local Name")},
  {"remoteCountry", I18S("Remote Country")},
  {"remoteBankCode", I18S("Remote Bank Code")},
  {"remoteAccountNumber", I18S("Remote Account Number")},
  {"remoteiban", I18S("Remote IBAN")},
  {"remotebic", I18S("Remote BIC")},
  {"remotename", I18S("Remote Name")},
  {"date", I18S("Date")},
  {"valutadate", I18S("Valuta Date")},
  {"amount", I18S("Value (Amount)")},
  {"currency", I18S("Value (Currency)")},
  {"fiid", I18S("Id from Financial Institute")},
  {"textkey", I18S("Text Key")},
  {"textkeyext", I18S("Text Key Extension")},
  {"infoText", I18S("Info Text")},
  {"customerRef", I18S("Customer Reference")},
  {"bankRef", I18S("Bank Reference")},
  {"purpose", I18S("Purpose")},
  {"status", I18S("Status")},
  {"transferid", I18S("Transfer Id")},
  {"groupid", I18S("Group Id")},
  {"category", I18S("Category")},
  {NULL, NULL}
};



FF_EditStatementRule::FF_EditStatementRule(FF_App* a,
                                           FXComposite *parent,
                                           const FXString& title,
                                           FXuint opts,
                                           FXint x, FXint y, FXint w, FXint h,
                                           FXint pl, FXint pr, FXint pt, FXint pb,
                                           FXint hs, FXint vs)
:FF_EditRule(a, _fields, parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs) {
}



FF_EditStatementRule::FF_EditStatementRule()
:FF_EditRule() {
}



FF_EditStatementRule::~FF_EditStatementRule() {
}



bool FF_EditStatementRule::editRule(FF_App* app,
                                    FXComposite *parent,
                                    const FXString &title,
                                    AE_RULE *r) {
  FF_EditStatementRule dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.addDialogLine();
  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.toGui(r);

  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (dlgres==0)
    return false;

  dlg.fromGui(r);
  return true;
}



