/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_edittax.hpp"
#include "ff_app.hpp"
#include "ff_amountedit.hpp"
#include "ff_groupview.hpp"
#include "ff_accountedit.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditTax) FF_EditTaxMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditTax::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditTax::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditTax::ID_LIST_TYPE, FF_EditTax::onCmdTypeList),

};


FXIMPLEMENT(FF_EditTax, FXDialogBox, FF_EditTaxMap, ARRAYNUMBER(FF_EditTaxMap))



FF_EditTax::FF_EditTax(FF_App* a,
		       FXComposite *parent,
		       const FXString& title,
		       FXuint opts,
		       FXint x, FXint y, FXint w, FXint h,
		       FXint pl, FXint pr, FXint pt, FXint pb,
		       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;

  /* description */
  m_label=new FOX16_HtmlLabel(vf, I18N("Edit the tax."),
			      LAYOUT_FILL_X);

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 32, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Code"), NULL, LABEL_NORMAL);
  m_codeEdit=new FXTextField(m, 32, this, ID_EDIT_CODE,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Description"), NULL, LABEL_NORMAL);
  m_descrEdit=new FXTextField(m, 32, this, ID_EDIT_DESCR,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Account"), NULL, LABEL_NORMAL);
  m_taxAccountEdit=new FF_AccountEdit(m, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("Type"), NULL, LABEL_NORMAL);
  m_typeList=new FXListBox(m, this, ID_LIST_TYPE, LISTBOX_NORMAL);
  m_typeList->appendItem(I18N("Percentual Amount"));
  m_typeList->appendItem(I18N("Fixed Amount"));

  m_valueLabel=new FXLabel(m, I18N("Value"), NULL, LABEL_NORMAL);
  m_valueEdit=new FF_AmountEdit(m, this, ID_EDIT_AMOUNT, 0);

  /* group tab */
  m_groupView=new FF_GroupView(AE_Book_TableType_TaxGroup, vf,
			       LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_groupView->setTarget(this);
  m_groupView->setSelector(ID_VIEW_GROUP);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_EditTax::FF_EditTax()
:FXDialogBox()
,m_app(NULL)
{

}




FF_EditTax::~FF_EditTax() {
}



void FF_EditTax::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;

  db=m_app->getDialogConfig("FF_EditTax");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "groupView");
  m_groupView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_EditTax::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "groupView");
  m_groupView->fini(dbT);

  rv=m_app->setDialogConfig("FF_EditTax", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}






int FF_EditTax::checkGui(FXbool quiet) {
  FXString str;

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
			  I18N("Missing tax name"));
      m_nameEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  str=m_valueEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
                          I18N("Missing amount"));
      m_valueEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	m_valueEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad input for smallest unit"));
	  m_valueEdit->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
      if (AQDB_Value_IsZero(v) ||
          AQDB_Value_IsNegative(v)) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad input for smallest unit: Must be greater than zero."));
	  m_valueEdit->setFocus();
	}

        AQDB_Value_free(v);
        return GWEN_ERROR_BAD_DATA;
      }

      AQDB_Value_free(v);
    }
  }

  /* account */
  if (!(m_taxAccountEdit->getText().simplify().empty())) {
    if (m_taxAccountEdit->getAccountId()==0) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid tax account."));
	m_taxAccountEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
  }

  return 0;
}



int FF_EditTax::toGui(const AE_TAX *tax) {
  const char *s;
  const AQDB_VALUE *v;
  AQDB_ID id;

  m_valueEdit->reload();
  m_groupView->reload();
  m_taxAccountEdit->reload();

  s=AE_Tax_GetName(tax);
  if (s)
    m_nameEdit->setText(s);

  s=AE_Tax_GetCode(tax);
  if (s)
    m_codeEdit->setText(s);

  s=AE_Tax_GetDescription(tax);
  if (s)
    m_descrEdit->setText(s);

  m_taxAccountEdit->setAccountId(AE_Tax_GetAccountId(tax));

  v=AE_Tax_GetAmount(tax);
  if (v) {
    m_valueEdit->setValue(v);
    m_valueEdit->setCurrencyById(AE_Tax_GetCommodityId(tax));
  }

  switch(AE_Tax_GetType(tax)) {
  case AE_Tax_TypeFixedAmount:
    m_typeList->setCurrentItem(1, TRUE);
    break;
  case AE_Tax_TypePercentualAmount:
  default:
    m_typeList->setCurrentItem(0, TRUE);
    break;
  }

  id=AE_Tax_GetGroupId(tax);
  if (id!=0) {
    m_groupView->selectGroup(id, true);
    m_groupView->makeGroupVisible(id);
  }

  return 0;
}



int FF_EditTax::fromGui(AE_TAX *tax) {
  FXString str;
  AQDB_VALUE *v;
  AQDB_ID id;
  AE_GROUP *gr;

  str=m_nameEdit->getText().simplify();
  if (str.empty())
    return GWEN_ERROR_NO_DATA;
  AE_Tax_SetName(tax, str.text());

  str=m_codeEdit->getText().simplify();
  if (str.empty())
    AE_Tax_SetCode(tax, NULL);
  else
    AE_Tax_SetCode(tax, str.text());

  str=m_descrEdit->getText().simplify();
  if (str.empty())
    AE_Tax_SetDescription(tax, NULL);
  else
    AE_Tax_SetDescription(tax, str.text());

  AE_Tax_SetAccountId(tax, m_taxAccountEdit->getAccountId());

  switch(m_typeList->getCurrentItem()) {
  case 1:
    AE_Tax_SetType(tax, AE_Tax_TypeFixedAmount);
    break;
  case 0:
  default:
    AE_Tax_SetType(tax, AE_Tax_TypePercentualAmount);
    break;
  }

  v=m_valueEdit->getValue();
  if (v==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad or missing amount");
    return GWEN_ERROR_NO_DATA;
  }
  AE_Tax_SetAmount(tax, v);
  AQDB_Value_free(v);

  id=m_valueEdit->getCurrencyId();
  AE_Tax_SetCommodityId(tax, id);

  gr=m_groupView->getFirstSelectedGroup();
  if (gr)
    AE_Tax_SetGroupId(tax, AE_Group_GetId(gr));
  else
    AE_Tax_SetGroupId(tax, 0);

  return 0;
}



void FF_EditTax::create() {
  FXDialogBox::create();
}



long FF_EditTax::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditTax::onCmdTypeList(FXObject*, FXSelector, void *ptr) {
  switch(m_typeList->getCurrentItem()) {
  case 1: m_valueLabel->setText(I18N("Amount")); break;
  default:
  case 0: m_valueLabel->setText(I18N("Percentage")); break;
  }

  return 1;
}



long FF_EditTax::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditTax::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditTax::editTax(FF_App* app,
			 FXComposite *parent,
			 const FXString &title,
			 AE_TAX *tax) {
  FF_EditTax dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.place(PLACEMENT_VISIBLE);
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  dlg.init();
  res=dlg.toGui(tax);
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(tax);
  if (res==0) {
    return true;
  }

  return false;
}





