/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_edittransactions.hpp"
#include "ff_app.hpp"
#include "ff_editaccstmrule.hpp"
#include "ff_edittransaction.hpp"

#include <aqfinance/engine/book/ae_query_stm.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_EditTransactions) FF_EditTransactionsMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditTransactions::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditTransactions::onCmdCancel),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FF_EditTransactions::ID_TRANSACTION_LIST,
            FF_EditTransactions::onListRightBtnReleased),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransactions::ID_FILTER_BY_PURPOSE,
	    FF_EditTransactions::onCmdFilterByPurpose),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransactions::ID_FILTER_BY_ACCOUNT,
	    FF_EditTransactions::onCmdFilterByAccount),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransactions::ID_FILTER_BY_NAME,
	    FF_EditTransactions::onCmdFilterByName),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransactions::ID_AUTOMATCH,
	    FF_EditTransactions::onCmdAutoMatch),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransactions::ID_EDIT_SPLITS,
	    FF_EditTransactions::onCmdSplitEditor),

};


FXIMPLEMENT(FF_EditTransactions, FXDialogBox,
            FF_EditTransactionsMap, ARRAYNUMBER(FF_EditTransactionsMap))




FF_EditTransactions::FF_EditTransactions(FF_App* a,
					 FXComposite *parent,
					 const FXString& title,
					 FXuint opts,
					 FXint x, FXint y, FXint w, FXint h,
					 FXint pl, FXint pr, FXint pt, FXint pb,
					 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_transactionTable(NULL) {
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
			      I18N("Please edit the transactions below."),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  m_transactionTable=new FF_TransactionTable(vf, this, ID_TRANSACTION_LIST,
					     FXXTable::TABLE_TRACK_SCROLLBARS |
                                             LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_DATE, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_MEMO, 100);
#if 0
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_VALUE, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_DEBIT_ACCOUNT, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_CREDIT_ACCOUNT, 100);
#endif
#if 0
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_PARTNER, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_CATEGORY, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_VALUE, 100);
#endif
#if 1
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_VALUE, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_COUNTER_ACCOUNT, 100);
#endif

  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_FLAGS, 100);
  m_transactionTable->addColumn(FF_TransactionTable::COLUMN_TYPE_IMPORT, 100);

  m_transactionTable->setMinRowHeight(20);
  m_transactionTable->setColorizeMatches(true);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);

  new THEMEBUTTON(hf, I18N("Automatch"), NULL, this, ID_AUTOMATCH,
		  BUTTON_NORMAL);
  new FXSpring(hf, LAYOUT_FILL_X);

  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL);
}



FF_EditTransactions::FF_EditTransactions()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_transactionTable(NULL) {
}



FF_EditTransactions::~FF_EditTransactions() {
}



void FF_EditTransactions::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_EditTransactions");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_transactionTable->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_transactionTable->getHeaderSize(i));
    m_transactionTable->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_transactionTable->setHeaderArrowDir(i, TRUE);
      else
        m_transactionTable->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_EditTransactions::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_transactionTable->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_transactionTable->getHeaderSize(i));
    b=m_transactionTable->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_EditTransactions", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_EditTransactions::checkGui(FXbool quiet) {
  return 0;
}



void FF_EditTransactions::toGui(const AE_TRANSACTION_LIST *tlist) {
  AE_TRANSACTION *t;

  m_transactionTable->reload();
  t=AE_Transaction_List_First(tlist);
  while(t) {
    AE_TRANSACTION *cpy;

    cpy=AE_Transaction_dup(t);
    m_transactionTable->addTransaction(cpy);
    t=AE_Transaction_List_Next(t);
  }
  m_transactionTable->sort();

  autoMatch();
}



void FF_EditTransactions::fromGui(AE_TRANSACTION_LIST *tlist) {
  int i;
  int cnt;

  cnt=m_transactionTable->getNumberOfRows();
  for (i=0; i<cnt; i++) {
    AE_TRANSACTION *t;

    t=m_transactionTable->getTransactionByRow(i);
    if (t) {
      /* add copy to list because the transaction table owns the list elements */
      AE_Transaction_List_Add(AE_Transaction_dup(t), tlist);
    }
  }
}



void FF_EditTransactions::create() {
  FXDialogBox::create();
  m_label->create();
  m_transactionTable->create();
}



long FF_EditTransactions::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditTransactions::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  FXuint res;

  DBG_ERROR(0, "Cancel");

  res=FXMessageBox::question(this, MBOX_YES_NO,
			     I18N("Cancel Booking Import"),
                             "%s",
			     I18N("Cancelling this dialog will cause the import "
				  "of booking transactions to be aborted.\n"
				  "However, at this point the bank statements have "
				  "already been imported, so cancelling this dialog "
				  "will cause the already imported bank statements "
				  "to be left without corresponding booking "
				  "transactions."
				  "Are you sure you want to abort this dialog?"));
  if (res==MBOX_CLICKED_YES) {
    DBG_ERROR(0, "Aborted");
    getApp()->stopModal(this,FALSE);
    hide();
  }
  return 1;
}



long FF_EditTransactions::onListRightBtnReleased(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_TRANSACTION *t;
  AE_STATEMENT *st=NULL;

  t=m_transactionTable->getCurrentTransaction();
  if (t)
    st=AE_Transaction_GetStatement(t);

  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuPane paneFilter(this);
    FXMenuCommand *c;
    const char *s;

    new FXMenuCommand(&pmenu, I18N("Split Editor"), NULL, this, ID_EDIT_SPLITS);
    new FXMenuSeparator(&pmenu);

    new FXMenuCommand(&paneFilter, I18N("by purpose"), NULL, this, ID_FILTER_BY_PURPOSE);
    c=new FXMenuCommand(&paneFilter, I18N("by account"), NULL, this, ID_FILTER_BY_ACCOUNT);
    c->disable();
    if (st) {
      s=AE_Statement_GetRemoteBankCode(st);
      if (s && *s) {
	s=AE_Statement_GetRemoteAccountNumber(st);
	if (s && *s)
	  c->enable();
      }
    }
    c=new FXMenuCommand(&paneFilter, I18N("by name"), NULL, this, ID_FILTER_BY_NAME);
    c->disable();
    if (st) {
      s=AE_Statement_GetRemoteName(st);
      if (s && *s)
        c->enable();
    }

    new FXMenuCascade(&pmenu, I18N("Create a Filter Rule"), NULL, &paneFilter);
    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FF_EditTransactions::onCmdFilterByPurpose(FXObject*, FXSelector, void *ptr) {
  AE_TRANSACTION *t;

  DBG_ERROR(0, "Filter by Purpose");
  t=m_transactionTable->getCurrentTransaction();
  if (t) {
    AE_STATEMENT *st;
    AE_RULE *r;

    r=AE_Rule_new();
    AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
    AE_Rule_SetTypeId(r, AE_Book_TableType_BankStatement);
    st=AE_Transaction_GetStatement(t);
    if (st) {
      const char *s;

      s=AE_Statement_GetPurpose(st);
      if (s && *s) {
	GWEN_BUFFER *tbuf;

	tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	GWEN_Buffer_AppendString(tbuf, "purpose\t?contains\t");
	GWEN_Buffer_AppendString(tbuf, s);
	GWEN_Buffer_AppendString(tbuf, "\n");
	AE_Rule_SetExpression(r, GWEN_Buffer_GetStart(tbuf));
	GWEN_Buffer_free(tbuf);
      }
    }
    if (FF_EditAccountStatementRule::editRule(m_app, this,
					      I18N("Create a Filter Rule"),
					      r)) {
      int rv;

      rv=AE_Book_AddRule(m_app->getBook(), AE_Book_TableType_Rule, r, 1);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	// TODO: change account
	autoMatch();
      }
    }
    m_transactionTable->reload();

    AE_Rule_free(r);
  }

  return 1;
}



long FF_EditTransactions::onCmdFilterByAccount(FXObject*, FXSelector, void *ptr) {
  AE_TRANSACTION *t;

  DBG_ERROR(0, "Filter by Account");
  t=m_transactionTable->getCurrentTransaction();
  if (t) {
    AE_STATEMENT *st;
    AE_RULE *r;

    r=AE_Rule_new();
    AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
    AE_Rule_SetTypeId(r, AE_Book_TableType_BankStatement);
    AE_Rule_SetCombineMode(r, AE_RuleCombineMode_And);
    st=AE_Transaction_GetStatement(t);
    if (st) {
      const char *s;
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);

      s=AE_Statement_GetRemoteBankCode(st);
      if (s && *s) {
	GWEN_Buffer_AppendString(tbuf, "remotebankcode\t==\t");
	GWEN_Buffer_AppendString(tbuf, s);
	GWEN_Buffer_AppendString(tbuf, "\n");
      }

      s=AE_Statement_GetRemoteAccountNumber(st);
      if (s && *s) {
	GWEN_Buffer_AppendString(tbuf, "remoteaccountnumber\t==\t");
	GWEN_Buffer_AppendString(tbuf, s);
	GWEN_Buffer_AppendString(tbuf, "\n");
      }

      if (GWEN_Buffer_GetUsedBytes(tbuf))
	AE_Rule_SetExpression(r, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
    if (FF_EditAccountStatementRule::editRule(m_app, this,
					      I18N("Create a Filter Rule"),
					      r)) {
      int rv;

      rv=AE_Book_AddRule(m_app->getBook(), AE_Book_TableType_Rule, r, 1);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	// TODO: change account
	autoMatch();
      }
    }
    m_transactionTable->reload();

    AE_Rule_free(r);
  }

  return 1;
}



long FF_EditTransactions::onCmdFilterByName(FXObject*, FXSelector, void *ptr) {
  AE_TRANSACTION *t;

  DBG_ERROR(0, "Filter by Name");
  t=m_transactionTable->getCurrentTransaction();
  if (t) {
    AE_STATEMENT *st;
    AE_RULE *r;

    r=AE_Rule_new();
    AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
    AE_Rule_SetTypeId(r, AE_Book_TableType_BankStatement);
    st=AE_Transaction_GetStatement(t);
    if (st) {
      const char *s;

      s=AE_Statement_GetRemoteName(st);
      if (s && *s) {
	GWEN_BUFFER *tbuf;

	tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	GWEN_Buffer_AppendString(tbuf, "remotename\t?contains\t");
	GWEN_Buffer_AppendString(tbuf, s);
	GWEN_Buffer_AppendString(tbuf, "\n");
	AE_Rule_SetExpression(r, GWEN_Buffer_GetStart(tbuf));
	GWEN_Buffer_free(tbuf);
      }
    }
    if (FF_EditAccountStatementRule::editRule(m_app, this,
					      I18N("Create a Filter Rule"),
					      r)) {
      int rv;

      rv=AE_Book_AddRule(m_app->getBook(), AE_Book_TableType_Rule, r, 1);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	// TODO: change account
	autoMatch();
      }
    }
    m_transactionTable->reload();

    AE_Rule_free(r);
  }
  return 1;
}



long FF_EditTransactions::onCmdAutoMatch(FXObject*, FXSelector, void *ptr) {
  autoMatch();
  return 1;
}



long FF_EditTransactions::onCmdSplitEditor(FXObject*, FXSelector, void *ptr) {
  AE_TRANSACTION *t;

  DBG_ERROR(0, "Split Editor");
  t=m_transactionTable->getCurrentTransaction();
  if (t) {
    if (FF_EditTransaction::editTransaction(m_app, this,
					    I18N("Edit Splits"),
					    t)) {
      m_transactionTable->rebuildTransactionflags(t);
    }
    m_transactionTable->reload();
    m_transactionTable->rebuild();
  }

  return 1;
}



void FF_EditTransactions::autoMatch() {
  int cnt;

  cnt=m_transactionTable->getNumberOfRows();
  if (cnt) {
    AE_RULE_LIST *rules;
    int rv;
    char qbuf[64];

    snprintf(qbuf, sizeof(qbuf)-1, "$ownerType==%d", AE_Book_TableType_Account);
    qbuf[sizeof(qbuf)-1]=0;

    rules=AE_Rule_List_new();
    rv=AE_Book_QueryRuleList(m_app->getBook(), AE_Book_TableType_Rule, 0, qbuf, rules);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Rule_List_free(rules);
    }
    else {
      if (AE_Rule_List_GetCount(rules)==0) {
        DBG_ERROR(0, "No rules");
      }
      else {
	int i;
	AE_RULE *r;
	GWEN_BUFFER *tbuf;
	bool needRebuild=false;

	/* create queries for rules and store them */
	tbuf=GWEN_Buffer_new(0, 256, 0, 1);
        r=AE_Rule_List_First(rules);
	while(r) {
	  rv=AE_Rule_toQueryString(r, tbuf);
	  if (rv<0) {
	    DBG_ERROR(0, "here (%d)", rv);
	  }
	  else {
	    AQDB_QUERY *q;

	    q=AE_StatementQuery_new(GWEN_Buffer_GetStart(tbuf));
	    if (q) {
	      DBG_ERROR(0, "Created query for [%s]", GWEN_Buffer_GetStart(tbuf));
	      AE_Rule_SetQuery(r, q);
	    }
	    else {
	      DBG_ERROR(0, "Bad query string [%s]", GWEN_Buffer_GetStart(tbuf));
	    }
	  }
	  GWEN_Buffer_Reset(tbuf);
	  r=AE_Rule_List_Next(r);
	}
	GWEN_Buffer_free(tbuf);

	for (i=0; i<cnt; i++) {
	  AE_TRANSACTION *t;

	  t=m_transactionTable->getTransactionByRow(i);
	  if (t) {
	    AE_STATEMENT *st;

	    st=AE_Transaction_GetStatement(t);
	    if (st) {
	      AE_RULE *r;
	      int matches=0;
	      AQDB_ID matchedAid=0;

	      r=AE_Rule_List_First(rules);
	      while(r) {
		AQDB_QUERY *q;
		AQDB_ID aid;

		q=AE_Rule_GetQuery(r);
		aid=AE_Rule_GetOwnerId(r);
		if (q && aid) {
		  rv=AE_StatementQuery_Check(q, st);
                  if (rv>0) {
                    if (matchedAid==0) {
                      matchedAid=aid;
                      matches++;
                    }
                    else {
                      if (matchedAid!=aid)
                        matches++;
                    }
		  }
		  else if (rv<0) {
		    DBG_ERROR(0, "here (%d)", rv);
		  }
		}

		r=AE_Rule_List_Next(r);
	      }
	      if (matches) {
		AE_SPLIT_LIST *sl;

		sl=AE_Transaction_GetSplitList(t);
		if (sl) {
		  AE_SPLIT *sp;

		  sp=AE_Split_List_First(sl);
		  while(sp) {
		    uint32_t flags;

		    flags=AE_Split_GetRuntimeFlags(sp);
                    if (!(flags & AE_SPLIT_RTFLAGS_FIX)) {
                      /* clear match mask */
		      flags&=~AE_SPLIT_RTFLAGS_MATCHMASK;
		      if (matches==1)
			/* only one rule matches, so we are quite sure abouth the match */
			flags|=AE_SPLIT_RTFLAGS_MATCHAUTO2;
		      else
			/* at least two rules matched, so we can't be sure about the result */
			flags|=AE_SPLIT_RTFLAGS_MATCHAUTO1;
		      AE_Split_SetRuntimeFlags(sp, flags);
		      AE_Split_SetAccountId(sp, matchedAid);
		      needRebuild=true;
		      break;
		    }
		    sp=AE_Split_List_Next(sp);
		  }
		}
	      }
	    }
	  }
	}
	if (needRebuild)
	  m_transactionTable->rebuild();
	AE_Rule_List_free(rules);
      }
    }
  }
  else {
    DBG_ERROR(0, "No transactions");
  }
}



bool FF_EditTransactions::editTransactions(FF_App* app,
                                           FXComposite *parent,
                                           const FXString &title,
                                           AE_TRANSACTION_LIST *tlist) {
  FF_EditTransactions dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.toGui(tlist);

  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  AE_Transaction_List_Clear(tlist);
  dlg.fromGui(tlist);
  return true;
}



