/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDITTRANSFER2_HPP
#define FF_EDITTRANSFER2_HPP


#include <fx.h>


#include <aqfinance/engine/book/ae_book.h>
#include <aqfinance/engine/book/ae_statement.h>
#include <aqfinance/engine/book/ae_bankaccount.h>


class FF_App;
class FOX16_HtmlLabel;
class FF_TransferEdit;


class FF_EditTransfer2: public FXDialogBox {
  FXDECLARE(FF_EditTransfer2)

public:

  enum {
    OPTS_SHOWDATE     = 0x80000000,
    OPTS_SHOWLOADTMPL = 0x40000000,
    OPTS_SHOWSAVETMPL = 0x20000000
  };

  enum {
    ID_EDIT_TRANSFER=FXDialogBox::ID_LAST,
    ID_TABBOOK,
    ID_BUTTON_LOAD_TMPL,
    ID_BUTTON_SAVE_TMPL,
    ID_LAST
  };


  FF_EditTransfer2(FF_App* a,
                   int editMode,
                   int rules,
                   FXComposite *parent,
                   const FXString& title,
                   FXuint opts=DECOR_TITLE|DECOR_BORDER,
                   FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                   FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
                   FXint hs=0, FXint vs=0);
  virtual ~FF_EditTransfer2();


  static bool editSingleTransfer(FF_App* app,
                                 int editMode,
                                 FXComposite *parent,
                                 const FXString &title,
                                 AE_STATEMENT *stm,
                                 FXuint opts=DECOR_ALL);

  static bool editInternalTransfer(FF_App* app,
                                   int editMode,
                                   FXComposite *parent,
                                   const FXString &title,
                                   AE_STATEMENT *stm,
                                   FXuint opts=DECOR_ALL);

  static bool createStandingOrder(FF_App* app,
                                  int editMode,
                                  FXComposite *parent,
                                  const FXString &title,
                                  AE_STATEMENT *stm,
                                  FXuint opts=DECOR_ALL);

  static bool editStandingOrder(FF_App* app,
                                int editMode,
                                FXComposite *parent,
                                const FXString &title,
                                AE_STATEMENT *stm,
                                FXuint opts=DECOR_ALL);

  static bool editSepaTransfer(FF_App* app,
                               int editMode,
                               FXComposite *parent,
                               const FXString &title,
                               AE_STATEMENT *stm,
                               FXuint opts=DECOR_ALL);


  void init();
  void fini();


  int checkGui(FXbool quiet);
  int toGui(const AE_STATEMENT *stm);
  int fromGui(AE_STATEMENT *stml);

  int copyTransfer(AE_STATEMENT *st, AE_BOOK_TABLE_TYPE tt);

  long onCmdLoadTmpl(FXObject*, FXSelector, void *ptr);
  long onCmdSaveTmpl(FXObject*, FXSelector, void *ptr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

protected:
  FF_EditTransfer2();

  FF_App *m_app;
  int m_rules;

  FOX16_HtmlLabel *m_label;
  FF_TransferEdit *m_transferEdit;
  FXButton *m_loadTmplButton;
  FXButton *m_saveTmplButton;

};




#endif




