/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editvehicle.hpp"
#include "ff_dateedit.hpp"
#include "ff_grouptree.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/buffer.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditVehicle) FF_EditVehicleMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditVehicle::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditVehicle::onCmdCancel),

};


FXIMPLEMENT(FF_EditVehicle, FXDialogBox, FF_EditVehicleMap, ARRAYNUMBER(FF_EditVehicleMap))



FF_EditVehicle::FF_EditVehicle(FF_App* a,
                               FXComposite *parent,
                               const FXString& title,
                               FXuint opts,
                               FXint x, FXint y, FXint w, FXint h,
                               FXint pl, FXint pr, FXint pt, FXint pb,
                               FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_nameEdit(NULL)
,m_makeEdit(NULL)
,m_modelEdit(NULL)
,m_buildYearEdit(NULL)
,m_baseEdit(NULL)
,m_licensePlateEdit(NULL)
,m_identificationNumberEdit(NULL)
,m_memoEdit(NULL)
,m_fuelCapacityEdit(NULL)
,m_buyDateEdit(NULL)
,m_groupTree(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXIcon *ic;

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("AE_Vehicle|Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 40, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Make"), NULL, LABEL_NORMAL);
  m_makeEdit=new FXTextField(m, 40, this, ID_EDIT_MAKE,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Model"), NULL, LABEL_NORMAL);
  m_modelEdit=new FXTextField(m, 40, this, ID_EDIT_MODEL,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Build Year"), NULL, LABEL_NORMAL);
  m_buildYearEdit=new FXTextField(m, 40, this, ID_EDIT_BUILDYEAR,
                                  TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Base"), NULL, LABEL_NORMAL);
  m_baseEdit=new FXTextField(m, 40, this, ID_EDIT_BASE,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|License Plate"), NULL, LABEL_NORMAL);
  m_licensePlateEdit=new FXTextField(m, 40, this, ID_EDIT_LICENSEPLATE,
                                     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Identification Number (VIN)"), NULL, LABEL_NORMAL);
  m_identificationNumberEdit=new FXTextField(m, 40, this, ID_EDIT_VIN,
                            TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Memo"), NULL, LABEL_NORMAL);
  m_memoEdit=new FXTextField(m, 40, this, ID_EDIT_MEMO,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Fuel Capacity"), NULL, LABEL_NORMAL);
  m_fuelCapacityEdit=new FXTextField(m, 40, this, ID_EDIT_FUELCAPACITY,
                                     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_Vehicle|Buy Date"), NULL, LABEL_NORMAL);
  m_buyDateEdit=new FF_DateEdit(m, this, ID_EDIT_BUYDATE,
				I18N("YYYY/MM/DD"),
				TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  gb=new FXGroupBox(vf, I18N("Vehicle Group"),
                    LAYOUT_FILL_X | LAYOUT_FILL_Y |
                    FRAME_LINE);
  m_groupTree=new FF_GroupTree(gb, this, ID_GROUP,
                               LAYOUT_FILL_X | LAYOUT_FILL_Y |
                               FOLDINGLIST_SINGLESELECT |
                               FOLDINGLIST_SHOWS_LINES |
                               FOLDINGLIST_SHOWS_BOXES |
                               FOLDINGLIST_ROOT_BOXES);
  m_groupTree->appendHeader(I18N("Group Name"));
  m_groupTree->setHeaderSize(0, m_nameEdit->getDefaultWidth());

  ic=a->getIcon("minifolderopen");
  if (ic)
    m_groupTree->setIconOpen(ic);
  ic=a->getIcon("minifolder");
  if (ic)
    m_groupTree->setIconClosed(ic);

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditVehicle::FF_EditVehicle()
:FXDialogBox()
,m_app(NULL)
,m_nameEdit(NULL)
,m_makeEdit(NULL)
,m_modelEdit(NULL)
,m_buildYearEdit(NULL)
,m_baseEdit(NULL)
,m_licensePlateEdit(NULL)
,m_identificationNumberEdit(NULL)
,m_memoEdit(NULL)
,m_fuelCapacityEdit(NULL)
,m_buyDateEdit(NULL)
,m_groupTree(NULL)
{

}




FF_EditVehicle::~FF_EditVehicle() {
}



FXint FF_EditVehicle::getDefaultHeight() {
  /* add minimum height of table to be usefull */
  return FXDialogBox::getDefaultHeight()+100;
}



void FF_EditVehicle::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditVehicle");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditVehicle::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditVehicle", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



bool FF_EditVehicle::_checkValueAndUnits(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle) {
  FXString str;

  str=tf->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    errorTitle.text(),
			    "%s",
			    I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	tf->setFocus();
      }
      return false;
    }
    else {
      AQDB_VALUE *v=NULL;
      int units=0;
      int i;
      int rv;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(str.text(), &v, &units);
      if (rv<0) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Invalid value in input"));
          tf->setFocus();
        }
        return false;
      }
      AQDB_Value_free(v);

      if (units==AE_Units_Unknown ||
          units==AE_Units_None) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Please enter units (like in \"123,45 km\" or in \"12,34 L\""));
          tf->setFocus();
        }
        return false;
      }
    }
  }
  else if (needed) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  errorTitle.text(),
			  "%s",
			  I18N("Missing input."));
      tf->setFocus();
    }
    return false;
  }

  return true;
}



int FF_EditVehicle::_strToValue(FXbool needed, const FXString &str, AQDB_VALUE **pValue, int *pUnits) {
  if (str.empty()) {
    if (needed) {
      DBG_ERROR(0, "No data when needed");
      return GWEN_ERROR_NO_DATA;
    }
    return 0;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      int i;
      FXString nstr;
      int rv;

      /* replace textual comma with decimal point */
      nstr=str;
      i=nstr.find(',');
      if (i>=0)
	nstr[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(nstr.text(), pValue, pUnits);
      return rv;
    }
  }

  return 0;
}



int FF_EditVehicle::checkGui(FXbool quiet) {
  FXString str;

  if (!_checkValueAndUnits(quiet, FALSE, m_fuelCapacityEdit, I18N("FF_EditVehicle|Bad Fuel Capacity")))
    return GWEN_ERROR_BAD_DATA;
  return 0;
}



int FF_EditVehicle::toGui(const AE_VEHICLE *veh) {
  const char *s;
  int i;
  const AQDB_VALUE *v;
  const GWEN_DATE *dt;
  int units;
  AE_BOOK *b;

#define TXTTOGUI(n, m)              \
  s=AE_Vehicle_Get##m(veh);         \
  if (s)                            \
    n->setText(s);

  TXTTOGUI(m_nameEdit, Name);
  TXTTOGUI(m_makeEdit, Make);
  TXTTOGUI(m_modelEdit, Model);
  TXTTOGUI(m_baseEdit, Base);
  TXTTOGUI(m_licensePlateEdit, LicensePlate);
  TXTTOGUI(m_identificationNumberEdit, IdentificationNumber);
  TXTTOGUI(m_memoEdit, Memo);

  i=AE_Vehicle_GetBuildYear(veh);
  m_buildYearEdit->setText(FXStringVal(i, 10));

  units=AE_Vehicle_GetFuelCapacityUnits(veh);
  v=AE_Vehicle_GetFuelCapacity(veh);
  if (v) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AE_Utils_ValueAndUnitsToString(v, units, tbuf, 2);
    m_fuelCapacityEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
    GWEN_Buffer_free(tbuf);
  }

  dt=AE_Vehicle_GetBuyDate(veh);
  if (dt)
    m_buyDateEdit->setDate(dt);

#undef TXTTOGUI

  b=m_app->getBook();
  if (b) {
    AE_GROUP_TREE *gt;
    int rv;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_VehicleGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      DBG_ERROR(0, "Adding %d groups", AE_Group_Tree_GetCount(gt));
      m_groupTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);
    m_groupTree->sortItems();
    m_groupTree->expandItems();
  }

  if (AE_Vehicle_GetGroupId(veh)) {
    FF_GroupTreeItem *item=NULL;

    item=m_groupTree->findGroupItem(AE_Vehicle_GetGroupId(veh));
    if (item)
      item->setSelected(true);
  }

  return 0;
}



int FF_EditVehicle::fromGui(AE_VEHICLE *veh) {
  FXString str;
  GWEN_DATE *dt;
  AE_GROUP *gr;
  AQDB_VALUE *v=NULL;
  int units=0;
  int rv;

#define TXTFROMGUI(n, m)                   \
  str=n->getText().simplify();             \
  if (str.empty())                         \
    AE_Vehicle_Set##m(veh, NULL);         \
  else                                     \
    AE_Vehicle_Set##m(veh, str.text())

  TXTFROMGUI(m_nameEdit, Name);
  TXTFROMGUI(m_makeEdit, Make);
  TXTFROMGUI(m_modelEdit, Model);
  TXTFROMGUI(m_baseEdit, Base);
  TXTFROMGUI(m_licensePlateEdit, LicensePlate);
  TXTFROMGUI(m_identificationNumberEdit, IdentificationNumber);
  TXTFROMGUI(m_memoEdit, Memo);

#undef TXTFROMGUI

  str=m_fuelCapacityEdit->getText().simplify();
  rv=_strToValue(TRUE, str, &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "Bad amount");
    return rv;
  }
  AE_Vehicle_SetFuelCapacity(veh, v);
  AE_Vehicle_SetFuelCapacityUnits(veh, units);
  AQDB_Value_free(v);

  str=m_buildYearEdit->getText().simplify();
  if (!str.empty()) {
    int i;

    i=FXIntVal(str, 10);
    AE_Vehicle_SetBuildYear(veh, i);
  }

  if (m_buyDateEdit->getDate(&dt)) {
    AE_Vehicle_SetBuyDate(veh, dt);
    GWEN_Date_free(dt);
  }

  gr=m_groupTree->getFirstSelectedGroup();
  if (gr) {
    DBG_ERROR(0, "Current group: %d", AE_Group_GetId(gr));
    AE_Vehicle_SetGroupId(veh, AE_Group_GetId(gr));
  }
  else {
    DBG_ERROR(0, "No current group");
    AE_Vehicle_SetGroupId(veh, 0);
  }

  return 0;
}



void FF_EditVehicle::create() {
  FXDialogBox::create();
}



long FF_EditVehicle::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditVehicle::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditVehicle::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditVehicle::editVehicle(FF_App* app,
				 FXComposite *parent,
				 const FXString &title,
				 AE_VEHICLE *veh) {
  FF_EditVehicle dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(veh);
  if (res)
    return false;
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(veh);
  if (res==0)
    return true;

  return false;
}









