/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

/* NO LONGER USED */


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editxfer.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_datepicker.hpp"
#include "ff_partneredit.hpp"

#include <aqbanking/jobsingletransfer.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)

#define MAX_PURPOSE_LINES 4



FXDEFMAP(FF_EditTransfer) FF_EditTransferMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_EditTransfer::ID_COMBO_LOCAL_ACC,
            FF_EditTransfer::onCmdLocalAccount),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransfer::ID_COMBO_PARTNER_ACC,
            FF_EditTransfer::onCmdPartnerAccount),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransfer::ID_EDIT_PARTNER, FF_EditTransfer::onCmdPartner),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransfer::ID_BUTTON_DATE, FF_EditTransfer::onCmdDate),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditTransfer::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditTransfer::onCmdCancel),
  FXMAPFUNCS(SEL_CHANGED, FF_EditTransfer::ID_EDIT_PURPOSE1, FF_EditTransfer::ID_EDIT_PURPOSE4, FF_EditTransfer::onChgPurpose),
  FXMAPFUNC(SEL_CHANGED, FF_EditTransfer::ID_EDIT_NAME, FF_EditTransfer::onChgName),

};


FXIMPLEMENT(FF_EditTransfer, FXDialogBox, FF_EditTransferMap, ARRAYNUMBER(FF_EditTransferMap))




FF_EditTransfer::FF_EditTransfer(FF_App* a,
				 int editMode,
				 FXComposite *parent,
                                 const FXString& title,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h,
                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_editMode(editMode)
,m_maxNumPurposeLines(2)
,m_maxPurposeLen(27)
,m_maxRemoteNameLen(27)
,m_label(NULL)
,m_localAccountCombo(NULL)
,m_nameEdit(NULL)
,m_accNumEdit(NULL)
,m_bankCodeEdit(NULL)
,m_purposeEdit1(NULL)
,m_purposeEdit2(NULL)
,m_amountEdit(NULL)
,m_currencyCombo(NULL)
,m_textKeyCombo(NULL)
,m_internalCheck(NULL)
,m_bankAccountList(NULL)
,m_partnerAccountList(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXVerticalFrame *vf2;
  FXGroupBox *gb;
  FXMatrix *m;
  FXHorizontalFrame *hf;
  FXHorizontalFrame *hf2;

  /* description */
  m_label=new FOX16_HtmlLabel(vf, I18N("Transfer money from your account to others"),
			      LAYOUT_FILL_X);

  /* local account */
  gb=new FXGroupBox(vf, I18N("FF_EditTransfer|Local Account"), LAYOUT_FILL_X | FRAME_LINE);

  m_localAccountCombo=new FXComboBox(gb, 1, this, ID_COMBO_LOCAL_ACC,
                                     COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  /* business partner */
  gb=new FXGroupBox(vf, I18N("FF_EditTransfer|Partner"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Partner"), NULL, LABEL_NORMAL);
  m_partnerEdit=new FF_PartnerEdit(m, this, ID_EDIT_PARTNER,
				   COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK |
				   LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Partner Account"), NULL, LABEL_NORMAL);
  m_partnerAccountCombo=new FXComboBox(m, 1, this, ID_COMBO_PARTNER_ACC,
				       COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  /* remote account */
  gb=new FXGroupBox(vf, I18N("FF_EditTransfer|Remote Account"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);


  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 32, this, ID_EDIT_NAME,
			     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN |
			     LAYOUT_FILL_X);

  new FXLabel(m, I18N("Bank Code"), NULL, LABEL_NORMAL);
  m_bankCodeEdit=new FXTextField(m, 32, this, ID_EDIT_BANK_CODE,
				 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Account Number"), NULL, LABEL_NORMAL);
  m_accNumEdit=new FXTextField(m, 32, this, ID_EDIT_ACCOUNT_NUM,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
			       LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* purpose, amount, text key */
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);

  /* purpose */
  gb=new FXGroupBox(hf, I18N("FF_EditTransfer|Purpose"), FRAME_LINE);
  vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_purposeEdit1=new FXTextField(vf2, 27, this, ID_EDIT_PURPOSE1,
				 LAYOUT_FILL_X |
				 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | TEXTFIELD_LIMITED |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_purposeEdit2=new FXTextField(vf2, 27, this, ID_EDIT_PURPOSE2,
				 LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  m_purposeEdit3=new FXTextField(vf2, 27, this, ID_EDIT_PURPOSE3,
				 LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_purposeEdit4=new FXTextField(vf2, 27, this, ID_EDIT_PURPOSE4,
                                 LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
                                 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* amount */
  gb=new FXGroupBox(hf, I18N("FF_EditTransfer|Amount"), FRAME_LINE);
  hf2=new FXHorizontalFrame(gb, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_amountEdit=new FXTextField(hf2, 12, this, ID_EDIT_AMOUNT,
			       LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
			       JUSTIFY_RIGHT | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_currencyCombo=new FXComboBox(hf2, 1, this, ID_COMBO_CURRENCY,
				 COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* textkey and date */
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

  /* date */
  gb=new FXGroupBox(hf, I18N("Date"), FRAME_LINE);
  hf2=new FXHorizontalFrame(gb, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_dateEdit=new FF_DateEdit(hf2, this, ID_EDIT_DATE,
                             I18N("YYYY/MM/DD"),
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  m_dateButton=new THEMEBUTTON(hf2, I18N("Pick"), NULL, this, ID_BUTTON_DATE, BUTTON_NORMAL);

  /* textkey */
  gb=new FXGroupBox(hf, I18N("FF_EditTransfer|Textkey"), FRAME_LINE);
  m_textKeyCombo=new FXComboBox(gb, 1, this, ID_COMBO_TEXTKEY,
                                COMBOBOX_STATIC | LAYOUT_FILL_X | FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN);

  new FXSpring(hf, LAYOUT_FILL_X);

  m_internalCheck=new FXCheckButton(vf, I18N("FF_EditXfer|Internal Transfer"));
  m_internalCheck->setTipText(I18N("FF_EditXfer|Internal transfers are transfers between\n"
				   "two accounts at the same bank."));

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditTransfer::FF_EditTransfer()
:FXDialogBox()
,m_app(NULL)
,m_maxNumPurposeLines(2)
,m_maxPurposeLen(27)
,m_maxRemoteNameLen(27)
,m_label(NULL)
,m_localAccountCombo(NULL)
,m_nameEdit(NULL)
,m_accNumEdit(NULL)
,m_bankCodeEdit(NULL)
,m_purposeEdit1(NULL)
,m_purposeEdit2(NULL)
,m_amountEdit(NULL)
,m_currencyCombo(NULL)
,m_textKeyCombo(NULL)
,m_bankAccountList(NULL)
,m_partnerAccountList(NULL)
{
}



FF_EditTransfer::~FF_EditTransfer() {
  AE_BankAccount_List_free(m_bankAccountList);
  m_bankAccountList=(AE_BANKACCOUNT_LIST*) -1;

  AE_BankAccount_List_free(m_partnerAccountList);
  m_partnerAccountList=(AE_BANKACCOUNT_LIST*) -1;

}



void FF_EditTransfer::setMaxNumPurposeLines(int i) {
  m_purposeEdit1->disable();
  m_purposeEdit2->disable();
  m_purposeEdit3->disable();
  m_purposeEdit4->disable();

  if (i>3)
    m_purposeEdit4->enable();
  if (i>2)
    m_purposeEdit3->enable();
  if (i>1)
    m_purposeEdit2->enable();
  if (i>0)
    m_purposeEdit1->enable();
  m_maxNumPurposeLines=i;
}



void FF_EditTransfer::setMaxPurposeLen(int i) {
#if 0
  m_purposeEdit1->setNumColumns(i);
  m_purposeEdit2->setNumColumns(i);
  m_purposeEdit3->setNumColumns(i);
  m_purposeEdit4->setNumColumns(i);
#endif

  m_maxPurposeLen=i;
  checkPurposeLengths();
}



void FF_EditTransfer::setMaxRemoteNameLen(int i) {
#if 0
  m_nameEdit->setNumColumns(i);
#endif

  m_maxRemoteNameLen=i;
  checkRemoteNameLength();
}



void FF_EditTransfer::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditTransfer");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditTransfer::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditTransfer", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



int FF_EditTransfer::checkGui(FXbool quiet) {
  FXString str;
  FXString str2;
  FXString str3;
  FXString str4;
  int i;
  AE_BANKACCOUNT *ba;
  AB_BANKING *ab;
  GWEN_TIME *ti;

  ab=m_app->getBanking();

  i=m_localAccountCombo->getCurrentItem();
  if (i==0) {
    if (m_editMode==EDITMODE_STRICT) {
      DBG_ERROR(0, "Missing account");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
			    I18N("Please select a local account"));
        m_localAccountCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    i--;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }
  
    if (ba==NULL) {
      if (m_editMode==EDITMODE_STRICT) {
        DBG_ERROR(0, "Missing account");
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
                              I18N("Missing local account"));
          m_localAccountCombo->setFocus();
        }
        return GWEN_ERROR_NO_DATA;
      }
    }
  }

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing peer name"));
        m_nameEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    if (str.count()>m_maxRemoteNameLen) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Remote name has too many characters"));
	m_nameEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  str=m_accNumEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing peer account number"));
        m_accNumEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  str2=m_bankCodeEdit->getText().simplify();
  if (str2.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing peer bank code"));
        m_bankCodeEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  if (ab) {
    AB_BANKINFO_CHECKRESULT res;

    res=AB_Banking_CheckAccount(ab, "de", 0, str2.text(), str.text());
    switch(res) {
      case AB_BankInfoCheckResult_NotOk:
        DBG_ERROR(0,
                  "Invalid combination of bank code and account number "
                  "for remote account");
        if (!quiet) {
          if (FXMessageBox::warning(this, MBOX_YES_NO,
                                    I18N("Input Error"),
				    "%s",
                                    I18N("The given combination of bank code and account "
                                         "number seems to be invalid.\n"
                                         "Please verify your input.\n"
                                         "Do you still want to use the information entered?"))
              !=MBOX_CLICKED_YES) {
            return GWEN_ERROR_BAD_DATA;
          }
        }
        break;

      case AB_BankInfoCheckResult_UnknownBank:
        DBG_ERROR(0, "Remote bank code is unknown");
        if (!quiet) {
          if (FXMessageBox::warning(this, MBOX_YES_NO,
                                    I18N("Input Error"),
				    "%s",
                                    I18N("The given bank code is unknown.\n"
                                         "Please verify your input.\n"
                                         "Do you still want to use the information entered?"))
              !=MBOX_CLICKED_YES) {
            return GWEN_ERROR_BAD_DATA;
          }
        }
        break;
    
      case AB_BankInfoCheckResult_UnknownResult:
        DBG_ERROR(0,
                  "Indifferent result for remote account check");
        if (!quiet) {
          if (FXMessageBox::warning(this, MBOX_YES_NO,
                                    I18N("Input Error"),
				    "%s",
                                    I18N("The validity of the given combination of bank code "
                                         "and account couldn't be established.\n"
                                         "Please verify your input.\n"
                                         "Do you still want to use the information entered?"))
              !=MBOX_CLICKED_YES) {
            return GWEN_ERROR_BAD_DATA;
          }
        }
        break;

      case AB_BankInfoCheckResult_Ok:
        break;

      default:
        DBG_ERROR(0, "Unknown check result %d", res);
        return 4;
    }
  }

  /* extensive purpose testing */
  str=m_purposeEdit1->getText().simplify();
  str2=m_purposeEdit2->getText().simplify();
  str3=m_purposeEdit3->getText().simplify();
  str4=m_purposeEdit4->getText().simplify();

  if (str.empty() &&
      str2.empty() &&
      str3.empty() &&
      str4.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing purpose"));
        m_purposeEdit1->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  if (str.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit1->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (str2.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit2->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (str3.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit3->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (str4.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit4->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }


  str=m_amountEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing amount"));
        m_amountEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Two types of komma detected, "
                                 "please specify the decimal point only"));
        m_amountEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad amount input"));
          m_amountEdit->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
      AQDB_Value_free(v);
    }
  }

  str=m_currencyCombo->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing currency"));
        m_currencyCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  str=m_textKeyCombo->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
                            "%s",
                            I18N("Missing text key"));
        m_textKeyCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    int i;

    if (1!=sscanf(str.text(), "%d", &i)) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Bad text key input"));
        m_textKeyCombo->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
  }

  if (!m_dateEdit->getDate(&ti)) {
    if (m_editMode==EDITMODE_STRICT) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
                            "%s",
			    I18N("Bad date input"));
	m_dateEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
  }
  else
    GWEN_Time_free(ti);

  return 0;
}



int FF_EditTransfer::toGui(const AE_STATEMENT *stm) {
  AE_BOOK *b;
  AB_BANKING *ab;

  setMaxNumPurposeLines(4);
  setMaxPurposeLen(27);

  ab=m_app->getBanking();
  b=m_app->getBook();
  if (b && ab) {
    int rv;
    AE_BANKACCOUNT *ba;
    int i;
    int selected=0;
    const char *s;
    const AQDB_VALUE *v;
    const GWEN_DATE *ti;

    m_partnerEdit->reload();

    /* fill local account combo box */
    m_localAccountCombo->clearItems();
    m_currencyCombo->clearItems();

    m_localAccountCombo->appendItem(I18N("Select an Account"));

    if (m_bankAccountList==NULL) {
      m_bankAccountList=AE_BankAccount_List_new();
      rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, m_bankAccountList);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
        AE_BankAccount_List_free(m_bankAccountList);
	m_bankAccountList=NULL;
        return rv;
      }

      /* remove offline accounts */
      ba=AE_BankAccount_List_First(m_bankAccountList);
      while(ba) {
	AE_BANKACCOUNT *n;
	AQDB_ID oid;

	n=AE_BankAccount_List_Next(ba);
	oid=AE_BankAccount_GetBankingId(ba);
	if (oid) {
	  if (AB_Banking_GetAccount(ab, oid)==NULL) {
	    /* oid, but there is no account for it; remove from list */
	    AE_BankAccount_List_Del(ba);
	    AE_BankAccount_free(ba);
	  }
	}
	else {
	  /* offline account, remove from list and free */
	  AE_BankAccount_List_Del(ba);
	  AE_BankAccount_free(ba);
	}

	ba=n;
      }
    }

    i=1;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      const char *s1;
      const char *s2;
      FXString str;

      s1=AE_BankAccount_GetBankName(ba);
      if (s1==NULL || *s1==0)
        s1=AE_BankAccount_GetBankCode(ba);
      s2=AE_BankAccount_GetAccountName(ba);
      if (s2==NULL || *s1==0)
        s2=AE_BankAccount_GetAccountNumber(ba);
      if (*s1==0) s1=NULL;
      if (*s2==0) s2=NULL;

      if (s1 && s2)
        str=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        str=FXString(s1);
      else if (s2)
        str=FXString(s2);
      else
        str=FXString(I18N("unnamed account"));
      m_localAccountCombo->appendItem(str);
      if (AE_Statement_GetBankAccountId(stm)==AE_BankAccount_GetBankingId(ba))
        selected=i;

      i++;
      ba=AE_BankAccount_List_Next(ba);
    }

    i=m_localAccountCombo->getNumItems();
    if (i>10)
      m_localAccountCombo->setNumVisible(10);
    else
      m_localAccountCombo->setNumVisible(i);

    m_localAccountCombo->setCurrentItem(selected);
    if (selected)
      selectLocalAccount(selected-1);

    /* partner */
    m_partnerEdit->setPartnerId(AE_Statement_GetPartnerId(stm));
    updatePartnerWidgets();

    /* name */
    s=AE_Statement_GetRemoteName(stm);
    if (s && *s)
      m_nameEdit->setText(FXString(s));

    /* select parter account according to remote bank account */
    i=m_partnerAccountCombo->getNumItems();
    if (i<1)
      m_partnerAccountCombo->setCurrentItem(0);
    else {
      int j;
      const char *sBankCode;
      const char *sAccNum;
      const char *sIban;

      sBankCode=AE_Statement_GetRemoteBankCode(stm);
      sAccNum=AE_Statement_GetRemoteAccountNumber(stm);
      sIban=AE_Statement_GetRemoteIban(stm);

      if (sBankCode)
	while(*sBankCode=='0')
	  sBankCode++;

      if (sAccNum)
	while(*sAccNum=='0')
	  sAccNum++;

      for (j=1; j<i; j++) {
	AE_BANKACCOUNT *ba;

	ba=(AE_BANKACCOUNT*) (m_partnerAccountCombo->getItemData(j));
	if (ba) {
	  const char *xBankCode;
	  const char *xAccNum;
	  const char *xIban;

	  xBankCode=AE_BankAccount_GetBankCode(ba);
	  xAccNum=AE_BankAccount_GetAccountNumber(ba);
	  xIban=AE_BankAccount_GetIban(ba);

	  if (sBankCode && *sBankCode && sAccNum && *sAccNum &&
	      xBankCode && *xBankCode && xAccNum && *xAccNum) {
	    if (xBankCode)
	      while(*xBankCode=='0')
		xBankCode++;
	    if (xAccNum)
	      while(*xAccNum=='0')
		xAccNum++;
	    if (strcasecmp(sBankCode, xBankCode)==0 &&
		strcasecmp(sAccNum, xAccNum)==0)
              break;
	  }
	  else if (sIban && *sIban && xIban && *xIban) {
	    if (strcasecmp(sIban, xIban)==0)
              break;
	  }
	}
      }
      if (j<i)
        m_partnerAccountCombo->setCurrentItem(j);
    }

    /* bank code */
    s=AE_Statement_GetRemoteBankCode(stm);
    if (s && *s)
      m_bankCodeEdit->setText(FXString(s));

    /* account number */
    s=AE_Statement_GetRemoteAccountNumber(stm);
    if (s && *s)
      m_accNumEdit->setText(FXString(s));

    /* purpose (split lines) */
    s=AE_Statement_GetPurpose(stm);
    if (s && *s) {
      FXString str[MAX_PURPOSE_LINES];
      int i=0;
  
      if (strchr(s, '\n')) {
	/* multi-line purpose, handle appropriately */
	while(*s) {
	  const char *ts;
  
	  ts=strchr(s, '\n');
	  if (ts) {
	    int l;
	    char *cpy;
  
	    l=ts-s;
	    cpy=(char*) malloc(l+1);
	    assert(cpy);
	    memmove(cpy, s, l);
	    cpy[l]=0;
	    if (i<MAX_PURPOSE_LINES) {
	      str[i]=FXString(cpy);
	      i++;
	    }
	    free(cpy);
	    s=ts;
	    s++;
	  }
	  else {
	    if (i<MAX_PURPOSE_LINES) {
	      str[i]=FXString(s);
	      i++;
	    }
	    s+=strlen(s);
	  }
	}
      }
      else {
	/* single-line purpose, simply add */
	if (i<MAX_PURPOSE_LINES) {
	  str[i]=FXString(s);
	  i++;
	}
      }

      if (i>0)
	m_purposeEdit1->setText(str[0]);
      if (i>1)
	m_purposeEdit2->setText(str[1]);
      if (i>2)
	m_purposeEdit3->setText(str[2]);
      if (i>3)
	m_purposeEdit4->setText(str[3]);
    }

    /* set currency and amount */
    v=AE_Statement_GetValue(stm);
    if (v) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      m_amountEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }
    s=AE_Statement_GetCurrency(stm);
    if (s)
      m_currencyCombo->appendItem(FXString(s));
    else
      m_currencyCombo->appendItem(FXString("EUR"));

    /* set textkey */
    i=AE_Statement_GetTextKey(stm);
    if (i) {
      FXString str;

      str=FXStringVal(i, 10);
      i=m_textKeyCombo->findItem(str);
      if (i>0) {
        i=m_textKeyCombo->appendItem(str);
        m_textKeyCombo->setCurrentItem(i);
      }
    }

    /* set date */
    ti=AE_Statement_GetDate(stm);
    if (ti)
      m_dateEdit->setDate(ti);

    m_internalCheck->setCheck((AE_Statement_GetFlags(stm) & AE_STATEMENT_FLAGS_INTERNAL_XFER)?TRUE:FALSE);
  }
  else {
    DBG_ERROR(0, "No book, no accounts");
    return GWEN_ERROR_INVALID;
  }


  return 0;
}



int FF_EditTransfer::fromGui(AE_STATEMENT *stm) {
  FXString str;
  FXString str2;
  int i;
  AE_BANKACCOUNT *ba;
  GWEN_DATE *ti;

  i=m_localAccountCombo->getCurrentItem();
  if (i==0) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetBankAccountId(stm, 0);
    AE_Statement_SetLocalBankCode(stm, NULL);
    AE_Statement_SetLocalAccountNumber(stm, NULL);
    AE_Statement_SetLocalIban(stm, NULL);
    AE_Statement_SetLocalBic(stm, NULL);
  }
  else {
    i--;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }
  
    if (ba==NULL) {
      if (m_editMode==EDITMODE_STRICT)
        return GWEN_ERROR_NO_DATA;
      AE_Statement_SetBankAccountId(stm, 0);
      AE_Statement_SetLocalBankCode(stm, NULL);
      AE_Statement_SetLocalAccountNumber(stm, NULL);
      AE_Statement_SetLocalIban(stm, NULL);
      AE_Statement_SetLocalBic(stm, NULL);
    }
    else {
      AE_Statement_SetBankAccountId(stm, AE_BankAccount_GetBankingId(ba));
      AE_Statement_SetLocalBankCode(stm, AE_BankAccount_GetBankCode(ba));
      AE_Statement_SetLocalAccountNumber(stm, AE_BankAccount_GetAccountNumber(ba));
      AE_Statement_SetLocalIban(stm, AE_BankAccount_GetIban(ba));
      AE_Statement_SetLocalBic(stm, AE_BankAccount_GetBic(ba));
    }
  }

  AE_Statement_SetPartnerId(stm, m_partnerEdit->getPartnerId());

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetRemoteName(stm, NULL);
  }
  else
    AE_Statement_SetRemoteName(stm, str.text());

  str=m_accNumEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetRemoteAccountNumber(stm, NULL);
  }
  else
    AE_Statement_SetRemoteAccountNumber(stm, str.text());

  str=m_bankCodeEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetRemoteBankCode(stm, NULL);
  }
  else
    AE_Statement_SetRemoteBankCode(stm, str.text());

  str=FXString("");
  i=0;

  str2=m_purposeEdit1->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  str2=m_purposeEdit2->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  str2=m_purposeEdit3->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  str2=m_purposeEdit4->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  if (str.empty()) {
    DBG_ERROR(AE_LOGDOMAIN, "Empty purpose");
    if (m_editMode==EDITMODE_STRICT) {
      return GWEN_ERROR_NO_DATA;
    }
    AE_Statement_SetPurpose(stm, NULL);
  }
  else
    AE_Statement_SetPurpose(stm, str.text());

  str=m_amountEdit->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetValue(stm, NULL);
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL)
        return GWEN_ERROR_BAD_DATA;
      AE_Statement_SetValue(stm, v);
      AQDB_Value_free(v);
    }
  }

  str=m_currencyCombo->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetCurrency(stm, NULL);
  }
  else
    AE_Statement_SetCurrency(stm, str.text());

  str=m_textKeyCombo->getText().simplify();
  if (str.empty()) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetTextKey(stm, 0);
  }
  else {
    if (1!=sscanf(str.text(), "%d", &i))
      return GWEN_ERROR_BAD_DATA;
    AE_Statement_SetTextKey(stm, i);
  }

  if (!m_dateEdit->getDate(&ti)) {
    if (m_editMode==EDITMODE_STRICT)
      return GWEN_ERROR_BAD_DATA;
  }
  AE_Statement_SetDate(stm, ti);
  GWEN_Date_free(ti);

  if (m_internalCheck->getCheck()==TRUE)
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_INTERNAL_XFER);
  else
    AE_Statement_SubFlags(stm, AE_STATEMENT_FLAGS_INTERNAL_XFER);

  return 0;
}



void FF_EditTransfer::selectLocalAccount(int i) {
  AB_BANKING *ab;
  FXString currentKey;
  int j;

  currentKey=m_textKeyCombo->getText();
  m_textKeyCombo->clearItems();
  setMaxNumPurposeLines(1);
  setMaxPurposeLen(27);

  ab=m_app->getBanking();
  if (ab) {
    AE_BANKACCOUNT *ba;

    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }

    if (ba) {
      AB_ACCOUNT *a;

      a=AB_Banking_GetAccount(ab, AE_BankAccount_GetBankingId(ba));
      if (a) {
        AB_JOB *j;
        int rv;

        j=AB_JobSingleTransfer_new(a);
        rv=AB_Job_CheckAvailability(j);
        if (rv<0) {
          DBG_ERROR(0, "SingleTransfer not available (%d) with %s/%s", rv,
                    AB_Account_GetBankCode(a),
                    AB_Account_GetAccountNumber(a));
        }
        else {
          const AB_TRANSACTION_LIMITS *tl;

          tl=AB_Job_GetFieldLimits(j);
          if (tl) {
            GWEN_STRINGLIST *keys=AB_TransactionLimits_GetValuesTextKey(tl);
	    GWEN_STRINGLISTENTRY *se=GWEN_StringList_FirstEntry(keys);
	    int i;

            while(se) {
              const char *s;

              s=GWEN_StringListEntry_Data(se);
              if (s)
                m_textKeyCombo->appendItem(FXString(s));
              se=GWEN_StringListEntry_Next(se);
	    }
	    i=AB_TransactionLimits_GetMaxLinesPurpose(tl);
	    if (i<1)
	      i=2;
	    setMaxNumPurposeLines(i);
	    i=AB_TransactionLimits_GetMaxLenPurpose(tl);
	    if (i<1)
	      i=27;
	    setMaxPurposeLen(27);

	    i=AB_TransactionLimits_GetMaxLenRemoteName(tl);
	    if (i<1)
	      i=27;
	    setMaxRemoteNameLen(27);
	  }
        }
        AB_Job_free(j);
      }
    }
  }
  if (!currentKey.empty()) {
    FXint i=m_textKeyCombo->findItem(currentKey);
    if (i>=0)
      m_textKeyCombo->setCurrentItem(i);
  }

  j=m_textKeyCombo->getNumItems();
  if (j>10)
    m_textKeyCombo->setNumVisible(10);
  else
    m_textKeyCombo->setNumVisible(j);

}



long FF_EditTransfer::onCmdLocalAccount(FXObject*, FXSelector, void *ptr) {
  int i;

  i=m_localAccountCombo->getCurrentItem();
  if (i>0) {
    selectLocalAccount(i-1);
  }

  return 1;
}



void FF_EditTransfer::updatePartnerWidgets() {
  AE_BOOK *b;

  m_partnerAccountCombo->clearItems();
  if (m_partnerAccountList)
    AE_BankAccount_List_Clear(m_partnerAccountList);
  else
    m_partnerAccountList=AE_BankAccount_List_new();
  m_partnerAccountCombo->appendItem(I18N("--select--"), NULL);

  b=m_app->getBook();
  if (b) {
    AQDB_ID id;
    int i;

    // read name and account from partner
    id=m_partnerEdit->getPartnerId();
    if (id) {
      AE_PARTNER *pt=NULL;
      AE_BANKACCOUNT *ba;
      AE_ADDRESS *addr;
      const char *s;
      int rv;
      int idx;

      /* read partner */
      rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner, id, &pt);
      if (rv<0) {
	DBG_ERROR(0, "Partner %08x not found", (unsigned int) id);
	return;
      }

      /* read subobjects (like account, address etc) */
      rv=AE_Book_ReadPartnerSubObjects(b, pt);
      if (rv<0) {
	DBG_ERROR(0, "Error reading subobjects of partner %08x", (unsigned int) id);
        AE_Partner_free(pt);
	return;
      }

      ba=AE_BankAccount_List_First(AE_Partner_GetBankAccountList(pt));
      while(ba) {
	FXString str;
	const char *bankCode;
        const char *accountNumber;

	bankCode=AE_BankAccount_GetBankCode(ba);
        accountNumber=AE_BankAccount_GetAccountNumber(ba);
	if (bankCode && *bankCode && accountNumber && *accountNumber)
	  str=FXStringFormat(I18N("%s at %s"), accountNumber, bankCode);
        else if (accountNumber && *accountNumber)
	  str=FXStringFormat(I18N("%s"), accountNumber);

	if (!str.empty()) {
	  AE_BANKACCOUNT *cpy;

          cpy=AE_BankAccount_dup(ba);
	  m_partnerAccountCombo->appendItem(str, cpy);
	  AE_BankAccount_List_Add(cpy, m_partnerAccountList);
	}

	ba=AE_BankAccount_List_Next(ba);
      }
      idx=m_partnerAccountCombo->getNumItems();
      if (idx>0) {
	m_partnerAccountCombo->setCurrentItem(idx-1);
	selectPartnerAccount(idx-1);
      }

      addr=AE_Address_List_First(AE_Partner_GetAddressList(pt));
      if (addr) {
	s=AE_Address_GetName(addr);
	if (s && *s)
	  m_nameEdit->setText(FXString(s));
      }
      else {
	FXString str;

	s=AE_Partner_GetFirstName(pt);
	if (s && *s)
	  str+=FXString(s);
	s=AE_Partner_GetLastName(pt);
	if (s && *s) {
	  if (!str.empty())
	    str+=" ";
          str+=FXString(s);
	}
        m_nameEdit->setText(str);
      }

      AE_Partner_free(pt);
    }
    i=m_partnerAccountCombo->getNumItems();
    if (i>10)
      m_partnerAccountCombo->setNumVisible(10);
    else
      m_partnerAccountCombo->setNumVisible(i);
  }
}



long FF_EditTransfer::onCmdPartner(FXObject*, FXSelector, void *ptr) {
  updatePartnerWidgets();
  return 1;
}



void FF_EditTransfer::selectPartnerAccount(int i) {
  if (i>=0) {
    AE_BANKACCOUNT *ba;

    ba=(AE_BANKACCOUNT*) m_partnerAccountCombo->getItemData(i);
    if (ba) {
      const char *s;

      s=AE_BankAccount_GetBankCode(ba);
      if (s && *s)
	m_bankCodeEdit->setText(FXString(s));

      s=AE_BankAccount_GetAccountNumber(ba);
      if (s && *s)
	m_accNumEdit->setText(FXString(s));
    }
  }
}



long FF_EditTransfer::onCmdPartnerAccount(FXObject*, FXSelector, void *ptr) {
  int i;

  i=m_partnerAccountCombo->getCurrentItem();
  if (i>0)
    selectPartnerAccount(i);

  return 1;
}



long FF_EditTransfer::onCmdSearchBank(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditTransfer::onCmdCurrency(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditTransfer::onCmdTextKey(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditTransfer::onCmdAmount(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditTransfer::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



void FF_EditTransfer::checkPurposeLengths() {
  FXTextField *tf;

  tf=m_purposeEdit1;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }

  tf=m_purposeEdit2;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }

  tf=m_purposeEdit3;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }

  tf=m_purposeEdit4;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }
}



void FF_EditTransfer::checkRemoteNameLength() {
  FXTextField *tf;

  tf=m_nameEdit;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxRemoteNameLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }
}



long FF_EditTransfer::onChgPurpose(FXObject *sender, FXSelector, void *ptr) {
  checkPurposeLengths();
  return 1;
}



long FF_EditTransfer::onChgName(FXObject *sender, FXSelector, void *ptr) {
  checkRemoteNameLength();
  return 1;
}



long FF_EditTransfer::onCmdDate(FXObject*, FXSelector, void *ptr) {
  FXDate d;

  if (!m_dateEdit->getDate(d))
    d=FF_DateEdit::today();
  if (FF_DatePicker::pickDate(this, I18N("Date Picker"), d))
    m_dateEdit->setDate(d);
  return 1;
}



long FF_EditTransfer::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditTransfer::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditTransfer::editTransfer(FF_App* app,
				   int editMode,
				   FXComposite *parent,
				   const FXString &title,
				   AE_STATEMENT *stm) {
  FF_EditTransfer dlg(app, editMode, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.init();
  res=dlg.toGui(stm);
  if (res)
    return false;
  dlg.place(PLACEMENT_VISIBLE);
  dlg.show();
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(stm);
  if (res==0)
    return true;
  return false;
}



