/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_flickerdialog.hpp"
#include "ff_app.hpp"

#include "ff_flickerwidget.hpp"


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_FlickerDialog) FF_FlickerDialogMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_FlickerDialog::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_FlickerDialog::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_BUTTON_WIDTH_UP, FF_FlickerDialog::onCmdWidthUp),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_BUTTON_WIDTH_DOWN, FF_FlickerDialog::onCmdWidthDown),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_BUTTON_HEIGHT_UP, FF_FlickerDialog::onCmdHeightUp),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_BUTTON_HEIGHT_DOWN, FF_FlickerDialog::onCmdHeightDown),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_BUTTON_SPEED_UP, FF_FlickerDialog::onCmdSpeedUp),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_BUTTON_SPEED_DOWN, FF_FlickerDialog::onCmdSpeedDown),
  FXMAPFUNC(SEL_COMMAND, FF_FlickerDialog::ID_EDIT_TAN, FF_FlickerDialog::onCmdTan),
  FXMAPFUNC(SEL_CHANGED, FF_FlickerDialog::ID_EDIT_TAN, FF_FlickerDialog::onChgTan),
};


FXIMPLEMENT(FF_FlickerDialog, FXDialogBox, FF_FlickerDialogMap, ARRAYNUMBER(FF_FlickerDialogMap))




FF_FlickerDialog::FF_FlickerDialog(FF_App* a,
                                   FXWindow *parent,
                                   const FXString& title,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h,
                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_tanMinLen(4)
,m_tanMaxLen(6)
,m_widthDownButton(NULL)
,m_widthUpButton(NULL)
,m_heightDownButton(NULL)
,m_heightUpButton(NULL)
,m_speedDownButton(NULL)
,m_speedUpButton(NULL)
,m_flickerWidget(NULL)
,m_tanInput(NULL)
,m_okButton(NULL)
,m_iconWidthDown(NULL)
,m_iconWidthUp(NULL)
,m_iconHeightDown(NULL)
,m_iconHeightUp(NULL)
,m_iconSpeedDown(NULL)
,m_iconSpeedUp(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;
  FXMatrix *m;

  m_label=new FOX16_HtmlLabel(vf, "", LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label->setMinimumWidth(200);

  m_iconWidthDown=m_app->getIcon("arrow-left");
  m_iconWidthUp=m_app->getIcon("arrow-right");

  m_iconHeightDown=m_app->getIcon("arrow-down");
  m_iconHeightUp=m_app->getIcon("arrow-up");

  m_iconSpeedDown=m_app->getIcon("minus");
  m_iconSpeedUp=m_app->getIcon("plus");

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m=new FXMatrix(hf, 2, MATRIX_BY_COLUMNS | PACK_UNIFORM_WIDTH | PACK_UNIFORM_HEIGHT);
      m_widthDownButton=new THEMEBUTTON(m, "", m_iconWidthDown, this, ID_BUTTON_WIDTH_DOWN, BUTTON_NORMAL);
      m_widthUpButton=new THEMEBUTTON(m, "", m_iconWidthUp, this, ID_BUTTON_WIDTH_UP, BUTTON_NORMAL);
      m_heightDownButton=new THEMEBUTTON(m, "", m_iconHeightDown, this, ID_BUTTON_HEIGHT_DOWN, BUTTON_NORMAL);
      m_heightUpButton=new THEMEBUTTON(m, "", m_iconHeightUp, this, ID_BUTTON_HEIGHT_UP, BUTTON_NORMAL);
      m_speedDownButton=new THEMEBUTTON(m, "", m_iconSpeedDown, this, ID_BUTTON_SPEED_DOWN, BUTTON_NORMAL);
      m_speedUpButton=new THEMEBUTTON(m, "", m_iconSpeedUp, this, ID_BUTTON_SPEED_UP, BUTTON_NORMAL);
    new FXSpring(hf, LAYOUT_FILL_Y);

    m_flickerWidget=new FF_FlickerWidget(hf, LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT);

  new FXSpring(vf, LAYOUT_FILL_X);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Please enter the TAN"), NULL, LABEL_NORMAL);
  m_tanInput=new FXTextField(hf, 40, this, ID_EDIT_TAN, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
                  BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
                             BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton->disable();
}



FF_FlickerDialog::FF_FlickerDialog()
:FXDialogBox()
,m_app(NULL)
,m_tanMinLen(4)
,m_tanMaxLen(6)
,m_widthDownButton(NULL)
,m_widthUpButton(NULL)
,m_heightDownButton(NULL)
,m_heightUpButton(NULL)
,m_speedDownButton(NULL)
,m_speedUpButton(NULL)
,m_flickerWidget(NULL)
,m_tanInput(NULL)
,m_okButton(NULL)
,m_iconWidthDown(NULL)
,m_iconWidthUp(NULL)
,m_iconHeightDown(NULL)
,m_iconHeightUp(NULL)
,m_iconSpeedDown(NULL)
,m_iconSpeedUp(NULL)
{

}



FF_FlickerDialog::~FF_FlickerDialog() {
}



void FF_FlickerDialog::init() {
  GWEN_DB_NODE *db;
  int w, h;
  FXString str;

  db=m_app->getDialogConfig("FF_FlickerDialog");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  w=GWEN_DB_GetIntValue(db, "flickerWidth", 0, 0);
  h=GWEN_DB_GetIntValue(db, "flickerHeight", 0, 0);
  if (w==0 || h==0) {
    w=205;
    h=100;
  }

  if (w>0 && h>0)
    m_flickerWidget->resize(w, h);

  GWEN_DB_Group_free(db);
}



void FF_FlickerDialog::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "flickerWidth", m_flickerWidget->getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "flickerHeight", m_flickerWidget->getHeight());

  rv=m_app->setDialogConfig("FF_FlickerDialog", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);

  m_flickerWidget->stopFlicker();

}



long FF_FlickerDialog::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  FXString s;
  int len;

  s=m_tanInput->getText();
  len=s.length();
  if (len>=m_tanMinLen && len<m_tanMaxLen) {
    getApp()->stopModal(this, TRUE);
  }

  return 1;
}



long FF_FlickerDialog::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_FlickerDialog::onCmdWidthUp(FXObject*, FXSelector, void *ptr) {
  FXint w;

  w=m_flickerWidget->getWidth();
  w+=2;
  m_flickerWidget->resize(w, m_flickerWidget->getHeight());
  layout();
  return 1;
}



long FF_FlickerDialog::onCmdWidthDown(FXObject*, FXSelector, void *ptr) {
  FXint w;

  w=m_flickerWidget->getWidth();
  if (w>50) {
    w-=2;
    m_flickerWidget->resize(w, m_flickerWidget->getHeight());
    layout();
  }
  return 1;
}



long FF_FlickerDialog::onCmdHeightUp(FXObject*, FXSelector, void *ptr) {
  FXint h;

  h=m_flickerWidget->getHeight();
  h+=2;
  m_flickerWidget->resize(m_flickerWidget->getWidth(), h);
  layout();
  return 1;
}



long FF_FlickerDialog::onCmdHeightDown(FXObject*, FXSelector, void *ptr) {
  FXint h;

  h=m_flickerWidget->getHeight();
  if (h>20) {
    h-=2;
    m_flickerWidget->resize(m_flickerWidget->getWidth(), h);
    layout();
  }
  return 1;
}



long FF_FlickerDialog::onCmdSpeedUp(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_FlickerDialog::onCmdSpeedDown(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_FlickerDialog::onCmdTan(FXObject*, FXSelector, void *ptr) {
  FXString s;
  int len;

  s=m_tanInput->getText();
  len=s.length();
  if (len>=m_tanMinLen && len<m_tanMaxLen) {
    getApp()->stopModal(this, TRUE);
  }

  return 1;
}



long FF_FlickerDialog::onChgTan(FXObject*, FXSelector, void *ptr) {
  FXString s;
  int len;

  s=m_tanInput->getText();
  len=s.length();
  if (len>=m_tanMinLen && len<m_tanMaxLen)
    m_okButton->enable();
  else
    m_okButton->disable();

  return 1;
}



bool FF_FlickerDialog::getTan(FF_App* app,
                              FXWindow *parent,
                              const FXString &title,
                              const FXString &message,
                              const FXString &code,
                              int minLen,
                              int maxLen,
                              FXString &response) {
  FF_FlickerDialog dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.m_tanMaxLen=maxLen;
  dlg.m_tanMinLen=minLen;

  dlg.m_label->setText(message);
  dlg.init();
  dlg.layout();
  dlg.show(PLACEMENT_VISIBLE);

  DBG_ERROR(0, "Setting flickercode [%s]", code.text());
  dlg.m_flickerWidget->setFlickerCode(code.text());
  dlg.m_flickerWidget->startFlicker();


  dlg.m_tanInput->setFocus();
  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  if (dlgres==0)
    return false;

  response=dlg.m_tanInput->getText();

  return true;
}







