/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_managecurrencies.hpp"
#include "ff_app.hpp"
#include "ff_currencyview.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ManageCurrencies) FF_ManageCurrenciesMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_ManageCurrencies::onCmdAccept),

};


FXIMPLEMENT(FF_ManageCurrencies, FXDialogBox,
            FF_ManageCurrenciesMap, ARRAYNUMBER(FF_ManageCurrenciesMap))




FF_ManageCurrencies::FF_ManageCurrencies(FF_App* a,
                                         FXComposite *parent,
                                         const FXString& title,
                                         FXuint opts,
                                         FXint x, FXint y, FXint w, FXint h,
                                         FXint pl, FXint pr, FXint pt, FXint pb,
                                         FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_currencyView(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_currencyView=new FF_CurrencyView(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Close"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_ManageCurrencies::FF_ManageCurrencies()
:FXDialogBox()
,m_app(NULL)
{
}



FF_ManageCurrencies::~FF_ManageCurrencies() {
}



void FF_ManageCurrencies::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_ManageCurrencies");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  m_currencyView->init(db);
  GWEN_DB_Group_free(db);
}



void FF_ManageCurrencies::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  m_currencyView->fini(db);

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_ManageCurrencies", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



void FF_ManageCurrencies::create() {
  FXDialogBox::create();
}



long FF_ManageCurrencies::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  /*hide(); */
  return 1;
}



void FF_ManageCurrencies::manageCurrencies(FF_App* app,
                                           FXComposite *parent,
                                           const FXString &title) {
  FF_ManageCurrencies dlg(app, parent, title, DECOR_ALL);

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);

  dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  dlg.hide();
}



