/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_pickstartdate.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/gwentime.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_PickStartDate) FF_PickStartDateMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_PickStartDate::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_PickStartDate::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_PickStartDate::ID_CUSTOM_EDIT,
	    FF_PickStartDate::onCmdTextField),
  FXMAPFUNC(SEL_COMMAND, FF_PickStartDate::ID_NODATE,
	    FF_PickStartDate::onCmdNoDate),
  FXMAPFUNC(SEL_COMMAND, FF_PickStartDate::ID_LAST_UPDATE,
	    FF_PickStartDate::onCmdLastUpdate),
  FXMAPFUNC(SEL_COMMAND, FF_PickStartDate::ID_FIRST_POSSIBLE,
	    FF_PickStartDate::onCmdFirstPossible),
  FXMAPFUNC(SEL_COMMAND, FF_PickStartDate::ID_CUSTOM_BUTTON,
            FF_PickStartDate::onCmdCustomDate),

};


FXIMPLEMENT(FF_PickStartDate, FXDialogBox,
            FF_PickStartDateMap, ARRAYNUMBER(FF_PickStartDateMap))



FF_PickStartDate::FF_PickStartDate(FXComposite *parent,
				   const FXString& title,
				   FXuint opts,
				   FXint x, FXint y, FXint w, FXint h,
				   FXint pl, FXint pr, FXint pt, FXint pb,
				   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_firstPossibleDate(FXDate(1800, 1, 1))
,m_lastUpdateDate(FXDate(1800, 1, 1))
,m_defaultChoice(0)
,m_buttonNoDate(NULL)
,m_buttonLastDate(NULL)
,m_lastDateLabel(NULL)
,m_buttonFirstDate(NULL)
,m_firstDateLabel(NULL)
,m_buttonCustomDate(NULL)
,m_customDateEdit(NULL)
,m_okButton(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXMatrix *m;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_label=new FOX16_HtmlLabel(vf,
			      I18N("<p>Please select the first date for which "
				   "transactions are to be retrieved.</p>"
				   "<p>If you specify no date the bank will choose one "
				   "for you.</p>"),
			      LAYOUT_FILL_X);
  gb=new FXGroupBox(vf, I18N("Start Date"),
                    LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 3, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  /* line 1 */
  m_buttonNoDate=new FXRadioButton(m, I18N("No date"),
                                   this, ID_NODATE, RADIOBUTTON_NORMAL);
  new FXSpring(m, LAYOUT_FILL_X);
  new FXSpring(m);

  /* line 2 */
  m_buttonLastDate=new FXRadioButton(m, I18N("Last update date"),
                                     this, ID_LAST_UPDATE, RADIOBUTTON_NORMAL);
  new FXSpring(m, LAYOUT_FILL_X);
  m_lastDateLabel=new FXLabel(m, "", NULL, FRAME_SUNKEN);

  /* line 3 */
  m_buttonFirstDate=new FXRadioButton(m, I18N("First possible date"),
                                      this, ID_FIRST_POSSIBLE, RADIOBUTTON_NORMAL);
  new FXSpring(m, LAYOUT_FILL_X);
  m_firstDateLabel=new FXLabel(m, "", NULL, FRAME_SUNKEN);

  /* line 4 */
  m_buttonCustomDate=new FXRadioButton(m, I18N("Custom date"),
                                       this, ID_CUSTOM_BUTTON, RADIOBUTTON_NORMAL);
  new FXSpring(m, LAYOUT_FILL_X);
  m_customDateEdit=new FF_DateEdit(m, this, ID_CUSTOM_EDIT,
                                   I18N("YYYY/MM/DD"),
                                   TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* ok, cancel */
  new FXSpring(vf);
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
			     BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_PickStartDate::FF_PickStartDate()
:FXDialogBox()
,m_defaultChoice(0)
,m_buttonNoDate(NULL)
,m_buttonLastDate(NULL)
,m_lastDateLabel(NULL)
,m_buttonFirstDate(NULL)
,m_firstDateLabel(NULL)
,m_buttonCustomDate(NULL)
,m_customDateEdit(NULL)
{
}



FF_PickStartDate::~FF_PickStartDate() {
}



void FF_PickStartDate::toGui() {
  GWEN_TIME *ti;
  GWEN_BUFFER *tbuf;

  tbuf=GWEN_Buffer_new(0, 32, 0, 1);

  m_firstDateLabel->disable();
  m_buttonFirstDate->disable();
  m_firstDateLabel->setText(I18N("YYYY/MM/DD"));
  if (m_firstPossibleDate>=FXDate(1900, 1, 1) && m_firstPossibleDate<FXDate(2500, 12, 31)) {
    /* set label for first possible date */
    ti=GWEN_Time_new(m_firstPossibleDate.year(),
                     m_firstPossibleDate.month()-1,
                     m_firstPossibleDate.day(),
                     0, 0, 0, 0);
    if (ti) {
      GWEN_Time_toString(ti, I18N("YYYY/MM/DD"), tbuf);
      m_firstDateLabel->setText(GWEN_Buffer_GetStart(tbuf));
      m_firstDateLabel->enable();
      m_buttonFirstDate->enable();
      GWEN_Buffer_Reset(tbuf);
      GWEN_Time_free(ti);
    }
  }

  m_lastDateLabel->disable();
  m_buttonLastDate->disable();
  m_lastDateLabel->setText(I18N("YYYY/MM/DD"));
  if (m_lastUpdateDate>=FXDate(1900, 1, 1) && m_lastUpdateDate<FXDate(2500, 12, 31)) {
    /* set label for last update date */
    ti=GWEN_Time_new(m_lastUpdateDate.year(),
                     m_lastUpdateDate.month()-1,
                     m_lastUpdateDate.day(),
                     0, 0, 0, 0);
    if (ti) {
      GWEN_Time_toString(ti, I18N("YYYY/MM/DD"), tbuf);
      m_lastDateLabel->setText(GWEN_Buffer_GetStart(tbuf));
      m_buttonLastDate->enable();
      GWEN_Buffer_Reset(tbuf);
      GWEN_Time_free(ti);
    }
  }

  GWEN_Buffer_free(tbuf);

  selectRadioButton(m_defaultChoice);

  if (checkGui(true)==0)
    m_okButton->enable();
  else
    m_okButton->disable();
}



void FF_PickStartDate::selectRadioButton(int i) {
  m_buttonNoDate->setCheck((i==0)?TRUE:FALSE, false);
  m_buttonLastDate->setCheck((i==1)?TRUE:FALSE, false);
  m_buttonFirstDate->setCheck((i==2)?TRUE:FALSE, false);
  m_buttonCustomDate->setCheck((i==3)?TRUE:FALSE, false);

  if (i==3)
    m_customDateEdit->enable();
  else
    m_customDateEdit->disable();
}


int FF_PickStartDate::fromGui(FXDate &result) {
  FXDate d;
  FXDate today;

  today=FF_DateEdit::today();
  if (m_buttonNoDate->getCheck()==TRUE)
    d=FXDate(1800, 1, 1);
  else if (m_buttonLastDate->getCheck()==TRUE)
    d=m_lastUpdateDate;
  else if (m_buttonFirstDate->getCheck()==TRUE)
    d=m_lastUpdateDate;
  else if (m_buttonCustomDate->getCheck()==TRUE) {
    if (m_customDateEdit->getDate(d)!=TRUE) {
      DBG_ERROR(0, "Bad date");
      return GWEN_ERROR_INVALID;
    }
  }

  DBG_ERROR(0, "Date: %04d / %02d / %02d", d.year(), d.month(), d.day());

  if (d>=FXDate(1900, 1, 1) && d<=today) {
    result=d;
    return 0;
  }
  else
    return GWEN_ERROR_INVALID;
}



int FF_PickStartDate::checkGui(bool quiet) {
  FXDate d;
  FXDate today;

  today=FF_DateEdit::today();

  if (m_buttonNoDate->getCheck()==TRUE) {
    d=FXDate(1800, 1, 1);
    return 0;
  }
  else if (m_buttonLastDate->getCheck()==TRUE)
    d=m_lastUpdateDate;
  else if (m_buttonFirstDate->getCheck()==TRUE)
    d=m_lastUpdateDate;
  else if (m_buttonCustomDate->getCheck()==TRUE) {
    if (m_customDateEdit->getDate(d)!=TRUE) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
			    I18N("Error"),
                            "%s",
			    I18N("Invalid date entered"));
      }
      DBG_ERROR(0, "Bad date");
      return GWEN_ERROR_INVALID;
    }
  }

  if (d>=FXDate(1900, 1, 1) && d<=today) {
    return 0;
  }
  else {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Error"),
			  "%s",
			  I18N("Invalid date entered"));
    }
    return GWEN_ERROR_INVALID;
  }
}



long FF_PickStartDate::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_PickStartDate::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



long FF_PickStartDate::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  if (checkGui(true)==0)
    m_okButton->enable();
  else
    m_okButton->disable();

  return 1;
}



long FF_PickStartDate::onCmdNoDate(FXObject*, FXSelector, void *ptr) {
  bool check=(FXuval) ptr;

  DBG_ERROR(0, "Button (%s)", check?"on":"off");
  if (check) {
    selectRadioButton(0);
    if (checkGui(true)==0)
      m_okButton->enable();
    else
      m_okButton->disable();
  }
  return 1;
}



long FF_PickStartDate::onCmdLastUpdate(FXObject*, FXSelector, void *ptr) {
  bool check=(FXuval) ptr;

  DBG_ERROR(0, "Button (%s)", check?"on":"off");
  if (check) {
    selectRadioButton(1);
    if (checkGui(true)==0)
      m_okButton->enable();
    else
      m_okButton->disable();
  }
  return 1;
}



long FF_PickStartDate::onCmdFirstPossible(FXObject*, FXSelector, void *ptr) {
  bool check=(FXuval) ptr;

  DBG_ERROR(0, "Button (%s)", check?"on":"off");
  if (check) {
    selectRadioButton(2);
    if (checkGui(true)==0)
      m_okButton->enable();
    else
      m_okButton->disable();
  }
  return 1;
}



long FF_PickStartDate::onCmdCustomDate(FXObject*, FXSelector, void *ptr) {
  bool check=(FXuval) ptr;

  DBG_ERROR(0, "Button (%s)", check?"on":"off");
  if (check) {
    selectRadioButton(3);
    if (checkGui(true)==0)
      m_okButton->enable();
    else
      m_okButton->disable();
  }
  return 1;
}



bool FF_PickStartDate::pickStartDate(FXComposite *parent,
                                     const FXString &title,
                                     const FXDate &firstPossible,
                                     const FXDate &lastUpdate,
                                     int defaultChoice,
                                     FXDate &result) {
  FF_PickStartDate dlg(parent, title, DECOR_ALL | DECOR_SHRINKABLE);
  FXuint dlgres;
  FXint res;

  dlg.setFirstPossibleDate(firstPossible);
  dlg.setLastUpdateDate(lastUpdate);
  dlg.setDefaultChoice(defaultChoice);

  dlg.create();
  dlg.toGui();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return false;
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(result);
      if (res==0) {
        return true;
        break;
      }
    }
  }
  /* should never reach this point */
  return false;
}



