/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectaccount.hpp"
#include "ff_app.hpp"
#include "ff_accounttree.hpp"
#include "ff_editaccount.hpp"

#include <aqfinance/engine/book/ae_account.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SelectAccount) FF_SelectAccountMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectAccount::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectAccount::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_SelectAccount::ID_CREATE_ACCOUNT, FF_SelectAccount::onCmdCreateAccount),
  FXMAPFUNC(SEL_COMMAND, FF_SelectAccount::ID_EDIT_ACCOUNT, FF_SelectAccount::onCmdEditAccount),
  FXMAPFUNC(SEL_CHANGED, FF_SelectAccount::ID_MATCHER, FF_SelectAccount::onChgMatcher),

};


FXIMPLEMENT(FF_SelectAccount, FXDialogBox,
            FF_SelectAccountMap, ARRAYNUMBER(FF_SelectAccountMap))




FF_SelectAccount::FF_SelectAccount(FF_App* a,
                                   FXComposite *parent,
                                   const FXString& title,
                                   AQDB_ID id,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h,
                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_accountTree(NULL)
,m_accountId(id)
,m_matcher(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
                           I18N("Please select the account from the "
                                "list below"),
                           LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  m_accountTree=new FF_AccountTree(vf, this, ID_ACCOUNT_LIST,
                                   LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                   LIST_SINGLESELECT);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Create Account"), NULL, this, ID_CREATE_ACCOUNT,
               BUTTON_NORMAL);
  new FXSpring(hf, LAYOUT_FILL_X);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
               BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
               BUTTON_NORMAL);
}



FF_SelectAccount::FF_SelectAccount(FF_App* a,
                                   const FXString& title,
                                   AQDB_ID id,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h,
                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                   FXint hs, FXint vs)
:FXDialogBox(a, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_accountTree(NULL)
,m_accountId(id)
,m_matcher(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
			      I18N("Please select the account from the "
				   "list below"),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  m_accountTree=new FF_AccountTree(vf, this, ID_ACCOUNT_LIST,
                                   FOLDINGLIST_SHOWS_LINES |
                                   FOLDINGLIST_SHOWS_BOXES |
                                   FOLDINGLIST_ROOT_BOXES |
                                   FOLDINGLIST_SINGLESELECT |
                                   LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Create Account"), NULL, this, ID_CREATE_ACCOUNT,
		  BUTTON_NORMAL);
  new THEMEBUTTON(hf, I18N("Edit Account"), NULL, this, ID_EDIT_ACCOUNT,
		  BUTTON_NORMAL);
  new FXSpring(hf, LAYOUT_FILL_X);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL);
}



FF_SelectAccount::FF_SelectAccount()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_accountTree(NULL)
,m_accountId(0)
,m_matcher(NULL)
{
}



FF_SelectAccount::~FF_SelectAccount() {
}



void FF_SelectAccount::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_SelectAccount");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_accountTree->getHeaderSize(i));
    m_accountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_accountTree->setHeaderArrowDir(i, TRUE);
      else
        m_accountTree->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_SelectAccount::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_accountTree->getHeaderSize(i));
    b=m_accountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_SelectAccount", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_SelectAccount::checkGui(FXbool quiet) {
  if (m_accountTree->getFirstSelectedAccount())
    return 0;
  return GWEN_ERROR_GENERIC;
}



void FF_SelectAccount::rebuild() {
  AE_ACCOUNT_TREE *at;
  int rv;

  m_accountTree->clearItems();
  m_accountTree->reload();

  at=AE_Account_Tree_new();
  rv=AE_Book_ReadAccountTree(m_app->getBook(), AE_Book_TableType_Account, at);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return;
  }

  FXString str=m_matcher->getText().simplify();

  /* select matching accounts */
  AE_Account_Tree_SelectMatching(at, str.text());
  m_accountTree->addSelectedAccounts(at, 0);
  AE_Account_Tree_free(at);

  if (1) {
    FXFoldingItem *fi;

    /* colorize */
    fi=m_accountTree->getFirstItem();
    while(fi) {
      FF_AccountTreeItem *item;

      item=dynamic_cast<FF_AccountTreeItem*>(fi);
      if (item) {
	AE_ACCOUNT *a;

	a=item->getAccount();
	if (a) {
	  AE_ACCOUNT_GROUP gr;

	  gr=AE_AccountType_GetGroup(AE_Account_GetType(a));
	  switch(gr) {
	  case AE_AccountGroup_Asset:
	    item->setBackColor(m_app->getAccountColorAsset());
	    item->setTextColor(m_accountTree->getTextColor());
	    item->setUseOwnColors(TRUE);
	    m_accountTree->updateItem(item);
	    break;
	  case AE_AccountGroup_Liability:
	    item->setBackColor(m_app->getAccountColorLiability());
	    item->setTextColor(m_accountTree->getTextColor());
	    item->setUseOwnColors(TRUE);
	    m_accountTree->updateItem(item);
	    break;
	  case AE_AccountGroup_Equity:
	    item->setBackColor(m_app->getAccountColorEquity());
	    item->setTextColor(m_accountTree->getTextColor());
	    item->setUseOwnColors(TRUE);
	    m_accountTree->updateItem(item);
	    break;
	  case AE_AccountGroup_Income:
	    item->setBackColor(m_app->getAccountColorIncome());
	    item->setTextColor(m_accountTree->getTextColor());
	    item->setUseOwnColors(TRUE);
	    m_accountTree->updateItem(item);
	    break;
	  case AE_AccountGroup_Expense:
	    item->setBackColor(m_app->getAccountColorExpense());
	    item->setTextColor(m_accountTree->getTextColor());
	    item->setUseOwnColors(TRUE);
	    m_accountTree->updateItem(item);
	    break;
	  default:
	    break;
	  }
	}
      }

      fi=fi->getBelow();
    }
  }

  m_accountTree->sort();
  m_accountTree->expandItems();
  m_accountTree->killSelection();
  m_accountTree->selectAccount(m_accountId, true);

  m_accountTree->makeAccountVisible(m_accountId);
  m_accountTree->setCurrentAccount(m_accountId, false);
}




FXint FF_SelectAccount::toGui(AQDB_ID id) {
  m_accountId=id;
  rebuild();
  return 0;
}



FXint FF_SelectAccount::fromGui(AQDB_ID &id) {
  AE_ACCOUNT *a;

  a=m_accountTree->getFirstSelectedAccount();
  if (a) {
    id=AE_Account_GetId(a);
    return 0;
  }

  return GWEN_ERROR_GENERIC;
}



void FF_SelectAccount::create() {
  FXDialogBox::create();
  m_label->create();
  m_accountTree->create();
}



long FF_SelectAccount::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  rebuild();
  return 1;
}



long FF_SelectAccount::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_SelectAccount::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_SelectAccount::selectAccount(FF_App* app,
                                     FXComposite *parent,
                                     const FXString &title,
                                     AQDB_ID &id) {
  FF_SelectAccount dlg(app, parent, title, id, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(400, dlg.getDefaultHeight()+100);
  dlg.create();
  dlg.init();
  res=dlg.toGui(id);
  if (res)
    return false;
  dlg.layout();
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(id);
  if (res!=0)
    return false;
  return true;
}



bool FF_SelectAccount::selectAccount(FF_App* app,
                                     const FXString &title,
                                     AQDB_ID &id) {
  FF_SelectAccount dlg(app, title, id, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(400, dlg.getDefaultHeight()+100);
  dlg.create();
  dlg.init();
  res=dlg.toGui(id);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(id);
  if (res!=0)
    return false;
  return true;
}



long FF_SelectAccount::onCmdCreateAccount(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ACCOUNT *pacc;
    AE_ACCOUNT *acc;
    AQDB_ID aid;

    acc=AE_Account_new();
    pacc=m_accountTree->getCurrentAccount();
    if (pacc) {
      AE_Account_SetParentId(acc, AE_Account_GetId(pacc));
      AE_Account_SetType(acc, AE_Account_GetType(pacc));
      AE_Account_SetPrimaryCommodityId(acc, AE_Account_GetPrimaryCommodityId(pacc));
    }
    else
      AE_Account_SetParentId(acc, 0);
    if (FF_EditAccount::editAccount(app, this, I18N("Create Account"), acc)) {
      int rv;

      rv=AE_Book_AddAccount(b, AE_Book_TableType_Account, acc, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add account (%d)", rv);
      }
      else {
        AE_RULE_LIST *rules;

        rules=AE_Account_GetRules(acc);
        if (rules) {
          AE_RULE *r;

          r=AE_Rule_List_First(rules);
          while(r) {
            DBG_ERROR(0, "Rule added");
            /* rule is new, create */
            AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
            AE_Rule_SetOwnerId(r, AE_Account_GetId(acc));
            rv=AE_Book_AddRule(b, AE_Book_TableType_Rule, r, 1);
            if (rv<0) {
              DBG_ERROR(0, "here (%d)", rv);
            }
            r=AE_Rule_List_Next(r);
          }
        }
      }

      app->signalDbReload(AE_Book_TableType_Account);
    }
    aid=AE_Account_GetId(acc);
    AE_Account_free(acc);

    m_accountTree->clearItems();
    toGui(aid);
  }

  return 1;
}



long FF_SelectAccount::onCmdEditAccount(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ACCOUNT *acc;
    AQDB_ID aid;

    acc=m_accountTree->getCurrentAccount();
    if (acc) {
      if (FF_EditAccount::editAccount(app, this, I18N("Create Account"), acc)) {
        int rv;

        rv=AE_Book_WriteAccount(b, AE_Book_TableType_Account, acc, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add account (%d)", rv);
        }
        else {
          AE_RULE_LIST *rules;

          rules=AE_Account_GetRules(acc);
          if (rules) {
            AE_RULE *r;

            r=AE_Rule_List_First(rules);
            while(r) {
              AQDB_ID rid;

              rid=AE_Rule_GetId(r);
              if (rid) {
                /* rule is new, create */
                AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
                AE_Rule_SetOwnerId(r, AE_Account_GetId(acc));
                rv=AE_Book_WriteRule(b, AE_Book_TableType_Rule, r, 1);
                if (rv<0) {
                  DBG_ERROR(0, "here (%d)", rv);
                }
              }
              else {
                /* rule is new, create */
                AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
                AE_Rule_SetOwnerId(r, AE_Account_GetId(acc));
                rv=AE_Book_AddRule(b, AE_Book_TableType_Rule, r, 1);
                if (rv<0) {
                  DBG_ERROR(0, "here (%d)", rv);
                }
              }
              r=AE_Rule_List_Next(r);
            }
          }
        }

        app->signalDbReload(AE_Book_TableType_Account);
      }
      aid=AE_Account_GetId(acc);
      m_accountTree->clearItems();
      toGui(aid);
    }
  }

  return 1;
}



