/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectcurrency.hpp"
#include "ff_app.hpp"
#include "ff_currencytree.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SelectCurrency) FF_SelectCurrencyMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectCurrency::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectCurrency::onCmdCancel),

};


FXIMPLEMENT(FF_SelectCurrency, FXDialogBox,
            FF_SelectCurrencyMap, ARRAYNUMBER(FF_SelectCurrencyMap))




FF_SelectCurrency::FF_SelectCurrency(FF_App* a,
                                     FXComposite *parent,
                                     const FXString& title,
                                     AQDB_ID id,
                                     FXuint opts,
                                     FXint x, FXint y, FXint w, FXint h,
                                     FXint pl, FXint pr, FXint pt, FXint pb,
                                     FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_currencyTree(NULL)
,m_currencyId(id) {
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
                           I18N("Please select the currency from the "
                                "list below"),
                           LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);
  m_currencyTree=new FF_CurrencyTree(vf, this, ID_CURRENCY_LIST,
                                     LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                     LIST_SINGLESELECT);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_SelectCurrency::FF_SelectCurrency()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_currencyTree(NULL)
,m_currencyId(0) {
}



FF_SelectCurrency::~FF_SelectCurrency() {
}



FXint FF_SelectCurrency::checkGui(FXbool quiet) {
  if (m_currencyTree->getFirstSelectedCurrency())
    return 0;
  return GWEN_ERROR_GENERIC;
}



FXint FF_SelectCurrency::toGui(AQDB_ID id) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_COMMODITY_LIST *cl;
    int rv;

    cl=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Commodity_List_free(cl);
    }
    else {
      AE_COMMODITY *co;

      while ( (co=AE_Commodity_List_First(cl)) ) {
        AE_Commodity_List_Del(co);
        m_currencyTree->addCurrency(co);
      }
    }
  }


  m_currencyTree->killSelection();
  m_currencyTree->selectCurrency(id, true);
  m_currencyTree->makeCurrencyVisible(id);

  return 0;
}



FXint FF_SelectCurrency::fromGui(AQDB_ID &id) {
  AE_COMMODITY *co;

  co=m_currencyTree->getFirstSelectedCurrency();
  if (co) {
    id=AE_Commodity_GetId(co);
    return 0;
  }

  return GWEN_ERROR_GENERIC;
}



void FF_SelectCurrency::create() {
  FXDialogBox::create();
  m_label->create();
  m_currencyTree->create();
}



long FF_SelectCurrency::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_SelectCurrency::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_SelectCurrency::selectCurrency(FF_App* app,
                                       FXComposite *parent,
                                       const FXString &title,
                                       AQDB_ID &id) {
  FF_SelectCurrency dlg(app, parent, title, id, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(440, 400);
  dlg.create();
  res=dlg.toGui(id);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight()+100);

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return false;
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(id);
      if (res==0) {
        return true;
      }
    }
  }
  /* should never reach this point */
  return false;
}



