/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectpartner.hpp"

#include "ff_app.hpp"
#include "ff_partnerview.hpp"

#include <aqfinance/engine/book/ae_partner.h>


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_SelectPartner) FF_SelectPartnerMap[]={
  FXMAPFUNC(SEL_INSERTED, FF_SelectPartner::ID_PARTNER_VIEW, FF_SelectPartner::onPartnerInserted),
  FXMAPFUNC(SEL_REPLACED, FF_SelectPartner::ID_PARTNER_VIEW, FF_SelectPartner::onPartnerReplaced),
  FXMAPFUNC(SEL_OPENED, FF_SelectPartner::ID_PARTNER_VIEW, FF_SelectPartner::onPartnerOpened),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectPartner::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectPartner::onCmdCancel),
  FXMAPFUNC(SEL_CHANGED, FF_SelectPartner::ID_MATCHER, FF_SelectPartner::onChgMatcher)
};


FXIMPLEMENT(FF_SelectPartner, FXDialogBox, FF_SelectPartnerMap, ARRAYNUMBER(FF_SelectPartnerMap))




FF_SelectPartner::FF_SelectPartner(FF_App* a,
				   FXComposite *parent,
				   const FXString& title,
				   FXuint opts,
				   FXint x, FXint y, FXint w, FXint h,
				   FXint pl, FXint pr, FXint pt, FXint pb,
				   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_partnerView(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FF_PartnerTree *tr;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
			      I18N("Please select the business partner from the "
				   "list below"),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  m_partnerView=new FF_PartnerView(vf, LAYOUT_FILL_X|LAYOUT_FILL_Y);
  m_partnerView->setSelector(ID_PARTNER_VIEW);
  m_partnerView->setTarget(this);

  tr=m_partnerView->getPartnerTree();
  tr->setListStyle(FOLDINGLIST_BROWSESELECT |
		   FOLDINGLIST_SHOWS_LINES |
		   FOLDINGLIST_SHOWS_BOXES |
		   FOLDINGLIST_ROOT_BOXES |
		   FRAME_SUNKEN|FRAME_THICK);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_SelectPartner::FF_SelectPartner()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_partnerView(NULL)
{

}



FF_SelectPartner::~FF_SelectPartner() {
}



FXint FF_SelectPartner::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



long FF_SelectPartner::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_SelectPartner::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



void FF_SelectPartner::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;

  db=m_app->getDialogConfig("FF_SelectPartner");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "partnerView");
  m_partnerView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_SelectPartner::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "partnerView");
  m_partnerView->fini(dbT);

  rv=m_app->setDialogConfig("FF_SelectPartner", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



AE_PARTNER *FF_SelectPartner::getSelectedPartner() {
  return m_partnerView->getPartnerTree()->getCurrentPartner();
}



void FF_SelectPartner::reload() {
  FF_App *app;
  AE_BOOK *b;
  FXString str;

  str=m_matcher->getText().simplify().trim();

  m_partnerView->clear();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_PARTNER_LIST *lst;
    int rv;

    lst=AE_Partner_List_new();
    rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      AE_PARTNER *pt;
  
      if (!str.empty()) {
	AE_Partner_List_SelectMatching(lst, str.text());
	/* remove non-matching partners */
	pt=AE_Partner_List_First(lst);
	while(pt) {
	  AE_PARTNER *next;
  
	  next=AE_Partner_List_Next(pt);
	  if (!(AE_Partner_GetRuntimeFlags(pt) & AE_PARTNER_RTFLAGS_SELECTED)) {
	    AE_Partner_List_Del(pt);
	    AE_Partner_free(pt);
	  }
	  pt=next;
	}
      }

      m_partnerView->toGui(lst);
      m_partnerView->sortPartners();
    }
    AE_Partner_List_free(lst);
  }
}



long FF_SelectPartner::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  reload();
  return 1;
}



long FF_SelectPartner::onPartnerInserted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Partner inserted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_PARTNER *pt;
    int rv;
    uint32_t flags;

    pt=(AE_PARTNER*) ptr;

    /* assign payable and receivable accounts */
    flags=AE_Partner_GetFlags(pt);
    if (((flags & AE_PARTNER_FLAGS_CUSTOMER) && AE_Partner_GetReceivableAccountId(pt)==0) ||
	((flags & AE_PARTNER_FLAGS_VENDOR) && AE_Partner_GetPayableAccountId(pt)==0)) {
      AE_ACCOUNT_TREE *at;
      AE_ACCOUNT *acc;

      at=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	if (flags & AE_PARTNER_FLAGS_CUSTOMER) {
	  if (AE_Partner_GetReceivableAccountId(pt)==0) {
	    acc=app->getPartnerReceivableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetReceivableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}

	if (flags & AE_PARTNER_FLAGS_VENDOR) {
	  if (AE_Partner_GetPayableAccountId(pt)==0) {
	    acc=app->getPartnerPayableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetPayableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}
      }
      AE_Account_Tree_free(at);
    }


    rv=AE_Book_ApplyPartner(b, pt, NULL, 1);
    if (rv<0) {
      DBG_ERROR(0, "Return code: %d", rv);
    }
  }

  return 1;
}



long FF_SelectPartner::onPartnerReplaced(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Partner replaced");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_PARTNER *pt;
    AE_PARTNER *oldpt=NULL;
    int rv;
    uint32_t flags;

    pt=(AE_PARTNER*) ptr;

    /* assign payable and receivable accounts */
    flags=AE_Partner_GetFlags(pt);
    if (((flags & AE_PARTNER_FLAGS_CUSTOMER) && AE_Partner_GetReceivableAccountId(pt)==0) ||
	((flags & AE_PARTNER_FLAGS_VENDOR) && AE_Partner_GetPayableAccountId(pt)==0)) {
      AE_ACCOUNT_TREE *at;
      AE_ACCOUNT *acc;

      at=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	if (flags & AE_PARTNER_FLAGS_CUSTOMER) {
	  if (AE_Partner_GetReceivableAccountId(pt)==0) {
	    acc=app->getPartnerReceivableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetReceivableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}

	if (flags & AE_PARTNER_FLAGS_VENDOR) {
	  if (AE_Partner_GetPayableAccountId(pt)==0) {
	    acc=app->getPartnerPayableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetPayableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}
      }
      AE_Account_Tree_free(at);
    }

    /* read original partner */
    rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner,
			   AE_Partner_GetId(pt), &oldpt);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      /* get saved state */
      AE_Book_ReadPartnerSubObjects(b, oldpt);

      /* apply any changes */
      rv=AE_Book_ApplyPartner(b, pt, oldpt, 1);
      AE_Partner_free(oldpt);
      if (rv<0) {
	DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      }
    }
  }

  return 1;
}



long FF_SelectPartner::onPartnerOpened(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_PARTNER *pt;

    DBG_ERROR(0, "Partner opened");
    pt=(AE_PARTNER*) ptr;

    AE_Book_ReadPartnerSubObjects(b, pt);
  }

  return 1;
}



AE_PARTNER *FF_SelectPartner::selectPartner(FF_App* a,
					    FXComposite *parent,
					    const FXString& title,
					    FXuint opts) {
  FF_SelectPartner dlg(a, parent, title, opts);
  int rv;
  AE_PARTNER *pt;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.init();
  dlg.reload();
  rv=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (rv==0) {
    DBG_ERROR(AE_LOGDOMAIN, "User aborted");
    return NULL;
  }

  pt=dlg.getSelectedPartner();
  if (pt)
    AE_Partner_Attach(pt);

  return pt;
}




