/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_sessiondlg.hpp"
#include "ff_app.hpp"
#include "ff_sessionlist.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SessionDlg) FF_SessionDlgMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_SessionDlg::ID_BUTTON_ROLLBACK,
            FF_SessionDlg::onCmdRollback),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SessionDlg::onCmdClose),
};


            
FXIMPLEMENT(FF_SessionDlg, FXDialogBox,
            FF_SessionDlgMap, ARRAYNUMBER(FF_SessionDlgMap))




FF_SessionDlg::FF_SessionDlg(FXComposite *parent,
                             const FXString& title,
                             FXuint opts,
                             FXint x, FXint y, FXint w, FXint h,
                             FXint pl, FXint pr, FXint pt, FXint pb,
                             FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_sessionList(NULL)
,m_rollbackButton(NULL)
,m_closeButton(NULL) {
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_sessionList=new FF_SessionList(vf, this, ID_SESSION_LIST,
                                   LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                   FOLDINGLIST_BROWSESELECT);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  m_closeButton=new THEMEBUTTON(hf, I18N("Close"), NULL, this, ID_CANCEL,
				BUTTON_NORMAL | LAYOUT_RIGHT);
  m_rollbackButton=new THEMEBUTTON(hf, I18N("Rollback"), NULL, this, ID_BUTTON_ROLLBACK,
				   BUTTON_NORMAL | LAYOUT_RIGHT);
}



FF_SessionDlg::FF_SessionDlg()
:FXDialogBox()
,m_sessionList(NULL)
,m_rollbackButton(NULL)
,m_closeButton(NULL) {
}



FF_SessionDlg::~FF_SessionDlg() {
}



void FF_SessionDlg::init() {
  GWEN_DB_NODE *db;
  int w, h;
  FF_App *app;
  int i;
  int sortByColumn;
  int sortUp;

  app=dynamic_cast<FF_App*>(getApp());

  db=app->getDialogConfig("FF_SessionDlg");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_sessionList->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_sessionList->getHeaderSize(i));
    m_sessionList->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_sessionList->setHeaderArrowDir(i, TRUE);
      else
        m_sessionList->setHeaderArrowDir(i, FALSE);
    }
  }
  GWEN_DB_Group_free(db);

  rebuild();
}



void FF_SessionDlg::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());

  db=GWEN_DB_Group_new("config");
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_sessionList->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_sessionList->getHeaderSize(i));
    b=m_sessionList->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=app->setDialogConfig("FF_SessionDlg", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);

}



void FF_SessionDlg::rebuild() {
  FF_App *app;
  AE_BOOK *b;

  m_sessionList->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AQDB_SESSION_DESCR_LIST *sll;
    int rv;

    sll=AQDB_SessionDescr_List_new();
    rv=AQDB_DB_GetRollbackSessions(AE_Book_GetDataBase(b), sll);
    if (rv<0) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Session Error"),
                          I18N("Unable to retrieve the list of rollback sessions (%d)"), rv);
    }
    else {
      AQDB_SESSION_DESCR *sd;

      while ( (sd=AQDB_SessionDescr_List_First(sll)) ) {
        AQDB_SessionDescr_List_Del(sd);
        m_sessionList->addSession(sd);
      }
    }
    AQDB_SessionDescr_List_free(sll);

    /* sort */
    m_sessionList->sort();
  }
}



void FF_SessionDlg::create() {
  FXDialogBox::create();
  m_sessionList->create();
  m_rollbackButton->create();
  m_closeButton->create();
}



long FF_SessionDlg::onCmdRollback(FXObject*, FXSelector, void* ptr) {
  AQDB_SESSION_DESCR *sd;

  sd=m_sessionList->getFirstSelectedSession();
  if (sd) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint res;

      res=FXMessageBox::question(this, MBOX_YES_NO,
				 I18N("Rollback Changes?"),
				 "%s",
                                 I18N("Rolling back the selected change and all changes\n"
                                      "after that means that really all those changes\n"
                                      "will be undone.\n"
				      "\n"
				      "This action can NOT be undone!!\n"
				      "\n"
                                      "If you rollback to before the first session entry\n"
                                      "in the list then the whole database becomes empty!!\n"
                                      "\n"
                                      "Are you sure you want to undo the selected change and\n"
                                      "all changes after that one, removing all those changes\n"
                                      "irretrievably?"));
      if (res!=MBOX_CLICKED_YES) {
	DBG_ERROR(0, "Aborted");
      }
      else {
	int rv;

	rv=AQDB_DB_UndoSession(AE_Book_GetDataBase(b), AQDB_SessionDescr_GetSessionId(sd));
	if (rv<0) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Session Error"),
			      I18N("Error undoing session %lu (%d)"),
			      (unsigned long int) AQDB_SessionDescr_GetSessionId(sd), rv);
	}

	app->signalDbReload(AE_Book_TableType_None);
	rebuild();
      }
    }
  }

  //getApp()->stopModal(this, TRUE);

  return 1;
}



long FF_SessionDlg::onCmdClose(FXObject*, FXSelector, void* ptr) {
  getApp()->stopModal(this, FALSE);
  return 1;
}




bool FF_SessionDlg::manageSessions(FXComposite *parent,
                                   const FXString& title) {
  FF_SessionDlg dlg(parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();

  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }
  return true;
}




