/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_settings.hpp"

#include "ff_app.hpp"
#include "ff_commodityedit.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg



FXDEFMAP(FF_Settings) FF_SettingsMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_Settings::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_Settings::onCmdCancel),
};


FXIMPLEMENT(FF_Settings, FXDialogBox, FF_SettingsMap, ARRAYNUMBER(FF_SettingsMap))



FF_Settings::FF_Settings(FF_App* a,
			 FXComposite *parent,
			 const FXString& title,
			 FXuint opts,
			 FXint x, FXint y, FXint w, FXint h,
			 FXint pl, FXint pr, FXint pt, FXint pb,
			 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
{
  FXVerticalFrame *vf;
  FXVerticalFrame *vf2;
  FXHorizontalFrame *hf;
  FXGroupBox *fg;
  FXMatrix *m;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_tabBook=new FXTabBook(vf, this, ID_TABBOOK,
			  LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new THEMETABITEM(m_tabBook, I18N("General"));
  vf2=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  fg=new FXGroupBox(vf2, I18N("External Applications"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m=new FXMatrix(fg, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new THEMELABEL(m, I18N("PDF Viewer"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_genericPdfCombo=new THEMECOMBOBOX(m, 16, this, ID_GENERIC_PDF_COMBO,
				      COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new THEMELABEL(m, I18N("Email Client"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_genericEmailCombo=new THEMECOMBOBOX(m, 16, this, ID_GENERIC_EMAIL_COMBO,
					COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);


  fg=new FXGroupBox(vf2, I18N("Cache Handling"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m=new FXMatrix(fg, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new THEMELABEL(m, I18N("Cache Entries"), NULL, JUSTIFY_LEFT | LAYOUT_FILL_X | LAYOUT_SIDE_LEFT);
  m_cacheEntriesSpinner=new FXSpinner(m, 10, this, ID_CACHE_ENTRIES, SPIN_NOMAX);

  new THEMELABEL(m, I18N("Cache Size (MB)"), NULL, JUSTIFY_LEFT | LAYOUT_FILL_X | LAYOUT_SIDE_LEFT);
  m_cacheSizeSpinner=new FXSpinner(m, 10, this, ID_CACHE_SIZE, SPIN_NOMAX);


  m_showBankAccountGraphButton=new FXCheckButton(vf2, I18N("Show Bank Account Graphs"), this, ID_BUTTON_SHOW_BA_GRAPHS,
                                                 LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_Settings::FF_Settings()
:FXDialogBox()
,m_app(NULL)
{
}



FF_Settings::~FF_Settings() {
}



void FF_Settings::init() {
  const FF_APPSPEC_LIST *appSpecList;
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_Settings");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  m_genericPdfCombo->clearItems();
  m_genericPdfCombo->appendItem(I18N("(select)"));
  appSpecList=m_app->getAppSpecsPdfViewer();
  if (appSpecList) {
    const FF_APPSPEC *as;
    int i;

    as=FF_AppSpec_List_First(appSpecList);
    while(as) {
      const char *s;

      s=FF_AppSpec_GetName(as);
      if (s && *s)
	m_genericPdfCombo->appendItem(s);
      as=FF_AppSpec_List_Next(as);
    }

    i=m_genericPdfCombo->getNumItems();
    if (i>10)
      i=10;
    m_genericPdfCombo->setNumVisible(i);
  }

  m_genericEmailCombo->clearItems();
  m_genericEmailCombo->appendItem(I18N("(select)"));
  appSpecList=m_app->getAppSpecsEmailClient();
  if (appSpecList) {
    const FF_APPSPEC *as;
    int i;

    as=FF_AppSpec_List_First(appSpecList);
    while(as) {
      const char *s;

      s=FF_AppSpec_GetName(as);
      if (s && *s)
	m_genericEmailCombo->appendItem(s);
      as=FF_AppSpec_List_Next(as);
    }
    i=m_genericEmailCombo->getNumItems();
    if (i>10)
      i=10;
    m_genericEmailCombo->setNumVisible(i);
  }

  m_cacheEntriesSpinner->setRange(0, 2147483647);
  m_cacheSizeSpinner->setRange(0, 2147483647);

  GWEN_DB_Group_free(db);
}



void FF_Settings::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
		      "height", getHeight());

  rv=m_app->setDialogConfig("FF_Settings", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_Settings::checkGui(FXbool quiet) {
  return 0;
}



FXint FF_Settings::toGui() {
  FXString str;
  uint32_t u;

  str=m_app->getPdfViewerName();
  if (!str.empty()) {
    int i;
    int j;

    j=m_genericPdfCombo->getNumItems();
    for (i=1; i<j; i++) {
      if (strcasecmp(m_genericPdfCombo->getItem(i).text(), str.text())==0)
        break;
    }
    if (i<j)
      m_genericPdfCombo->setCurrentItem(i);
  }

  str=m_app->getEmailClientName();
  if (!str.empty()) {
    int i;
    int j;

    j=m_genericEmailCombo->getNumItems();
    for (i=1; i<j; i++) {
      if (strcasecmp(m_genericEmailCombo->getItem(i).text(), str.text())==0)
        break;
    }
    if (i<j)
      m_genericEmailCombo->setCurrentItem(i);
  }

  u=m_app->getCacheMaxEntries();
  m_cacheEntriesSpinner->setValue(u);

  u=m_app->getCacheMaxSize();
  m_cacheSizeSpinner->setValue(u/(1024*1024));

  m_showBankAccountGraphButton->setCheck((m_app->getShowBankAccountGraphs())?TRUE:FALSE);
  return 0;
}



FXint FF_Settings::fromGui() {
  int i;
  uint32_t u;

  i=m_genericPdfCombo->getCurrentItem();
  if (i>0)
    m_app->setPdfViewerName(m_genericPdfCombo->getItem(i));

  i=m_genericEmailCombo->getCurrentItem();
  if (i>0)
    m_app->setEmailClientName(m_genericEmailCombo->getItem(i));

  u=m_cacheEntriesSpinner->getValue();
  m_app->setCacheMaxEntries(u);

  u=m_cacheSizeSpinner->getValue();
  m_app->setCacheMaxSize(u*(1024*1024));

  m_app->setShowBankAccountGraphs((m_showBankAccountGraphButton->getCheck())?true:false);

  return 0;
}




long FF_Settings::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_Settings::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



bool FF_Settings::edit(FF_App* app,
		       FXComposite *parent,
		       const FXString &title) {
  FF_Settings dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.init();
  res=dlg.toGui();
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui();
  if (res==0) {
    return true;
  }

  return false;
}








