/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_wizard.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_WizardPage) FF_WizardPageMap[]={
  //FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_WizardPage::onCmdAccept),
};


FXIMPLEMENT(FF_WizardPage, FXVerticalFrame, FF_WizardPageMap, ARRAYNUMBER(FF_WizardPageMap))




FF_WizardPage::FF_WizardPage(FF_Wizard *p,
                             const FXString &name,
			     FXuint opts,
			     FXint x, FXint y, FXint w, FXint h,
			     FXint pl, FXint pr,
			     FXint pt, FXint pb,
			     FXint hs, FXint vs)
:FXVerticalFrame(p->m_stack, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_wizard(p)
,m_name(name)
,m_index((p->m_pageCounter)++)
{
}



FF_WizardPage::FF_WizardPage()
:FXVerticalFrame()
,m_wizard(NULL)
,m_index(0)
{
}



FF_WizardPage::~FF_WizardPage() {
}



void FF_WizardPage::enterPageForwards() {
}



void FF_WizardPage::enterPageBackwards() {
}



void FF_WizardPage::leavePageForwards() {
}



void FF_WizardPage::leavePageBackwards() {
}



void FF_WizardPage::init(GWEN_DB_NODE *db) {
}



void FF_WizardPage::fini(GWEN_DB_NODE *db) {
}








FXDEFMAP(FF_Wizard) FF_WizardMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_Wizard::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_Wizard::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_Wizard::ID_BUTTON_NEXT, FF_Wizard::onCmdNext),
  FXMAPFUNC(SEL_COMMAND, FF_Wizard::ID_BUTTON_BACK, FF_Wizard::onCmdBack),
};


FXIMPLEMENT(FF_Wizard, FXDialogBox, FF_WizardMap, ARRAYNUMBER(FF_WizardMap))



FF_Wizard::FF_Wizard(FXWindow *owner,
		     const FXString &title,
		     const FXString &name,
		     FXImage *image,
		     FXuint opts,
		     FXint x, FXint y, FXint w, FXint h,
		     FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FXDialogBox(owner, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_name(name)
,m_image(image)
,m_pageCounter(0)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;

  if (m_image) {
    hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_imageFrame=new FXImageFrame(hf, NULL, FRAME_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y);
    m_stack=new FXSwitcher(hf, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0);
  }
  else
    m_stack=new FXSwitcher(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);

  m_nextButton=new THEMEBUTTON(hf, I18N("Next"), NULL, this, ID_BUTTON_NEXT,
			       BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
  m_backButton=new THEMEBUTTON(hf, I18N("Back"), NULL, this, ID_BUTTON_BACK,
			       BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);

}



FF_Wizard::FF_Wizard()
:FXDialogBox()
,m_image(NULL)
{
}



FF_Wizard::~FF_Wizard() {
}



void FF_Wizard::create() {
  FXDialogBox::create();
  if (m_image)
    m_image->create();
}



FF_WizardPage *FF_Wizard::pageAtIndex(int i) {
  FXWindow *w;

  w=m_stack->childAtIndex(i);
  if (w) {
    FF_WizardPage *wp;

    wp=dynamic_cast<FF_WizardPage*>(w);
    assert(wp);
    return wp;
  }

  return NULL;
}



void FF_Wizard::enableNext() {
  m_nextButton->enable();
}



void FF_Wizard::disableNext() {
  m_nextButton->disable();
}



long FF_Wizard::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_Wizard::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



long FF_Wizard::onCmdNext(FXObject*, FXSelector, void *ptr) {
  int curr;
  FF_WizardPage *wp;

  curr=m_stack->getCurrent();

  if (curr>=(m_stack->numChildren()-1)) {
    /* last page, finished, leave dialog */
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
  }

  wp=pageAtIndex(curr);
  if (wp)
    wp->leavePageForwards();

  curr++;

  m_stack->setCurrent(curr);
  wp=pageAtIndex(curr);
  if (wp)
    wp->enterPageForwards();

  if (curr>=(m_stack->numChildren()-1))
    m_nextButton->setText(I18N("Finish"));
  else
    m_nextButton->setText(I18N("Next"));

  m_backButton->enable();

  return 1;
}



long FF_Wizard::onCmdBack(FXObject*, FXSelector, void *ptr) {
  int curr;
  FF_WizardPage *wp;

  curr=m_stack->getCurrent();

  if (curr<=0) {
    DBG_ERROR(0, "No previous page");
    return 1;
  }

  wp=pageAtIndex(curr);
  if (wp)
    wp->leavePageBackwards();

  curr--;

  m_stack->setCurrent(curr);
  wp=pageAtIndex(curr);
  if (wp)
    wp->enterPageBackwards();

  if (curr==0)
    m_backButton->disable();
  else
    m_backButton->enable();

  m_nextButton->setText(I18N("Next"));

  return 1;
}



FXuint FF_Wizard::execute(FXuint placement) {
  FF_WizardPage *wp;
  FXuint res;

  init();
  m_stack->setCurrent(0);
  if (m_stack->numChildren()<=1)
    m_nextButton->setText(I18N("Finish"));
  else
    m_nextButton->setText(I18N("Next"));
  m_backButton->disable();

  wp=pageAtIndex(0);
  if (wp)
    wp->enterPageForwards();

  res=FXDialogBox::execute(placement);
  fini();
  return res;
}



void FF_Wizard::init() {
  FF_App *app;
  GWEN_DB_NODE *db;
  int w, h;
  int i, j;

  app=dynamic_cast<FF_App*>(getApp());
  db=app->getDialogConfig(m_name.text());
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getDefaultWidth(), getDefaultHeight());

  j=m_stack->numChildren();
  for (i=0; i<j; i++) {
    FF_WizardPage *wp;
  
    wp=pageAtIndex(i);
    if (wp) {
      GWEN_DB_NODE *dbT;

      dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, wp->getName().text());
      wp->init(dbT);
    }
  }
  GWEN_DB_Group_free(db);
}



void FF_Wizard::fini() {
  FF_App *app;
  GWEN_DB_NODE *db;
  int rv;
  int i, j;

  app=dynamic_cast<FF_App*>(getApp());

  db=GWEN_DB_Group_new("config");
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
		      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
		      "height", getHeight());

  j=m_stack->numChildren();
  for (i=0; i<j; i++) {
    FF_WizardPage *wp;
  
    wp=pageAtIndex(i);
    if (wp) {
      GWEN_DB_NODE *dbT;

      dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, wp->getName().text());
      wp->fini(dbT);
    }
  }

  rv=app->setDialogConfig(m_name.text(), db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}







