/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_WIZARD_HPP
#define FF_WIZARD_HPP


#include <fx.h>

#include <gwenhywfar/db.h>



class FF_Wizard;


class FF_WizardPage: public FXVerticalFrame {
  FXDECLARE(FF_WizardPage)
public:
  FF_WizardPage(FF_Wizard *p,
		const FXString &name,
		FXuint opts=0,
		FXint x=0, FXint y=0, FXint w=0, FXint h=0,
		FXint pl=DEFAULT_SPACING, FXint pr=DEFAULT_SPACING,
		FXint pt=DEFAULT_SPACING, FXint pb=DEFAULT_SPACING,
		FXint hs=DEFAULT_SPACING, FXint vs=DEFAULT_SPACING);
  FF_WizardPage();

  virtual ~FF_WizardPage();

  const FXString &getName() const { return m_name;};

  virtual void enterPageForwards();
  virtual void enterPageBackwards();

  virtual void leavePageForwards();
  virtual void leavePageBackwards();

  virtual void init(GWEN_DB_NODE *db);
  virtual void fini(GWEN_DB_NODE *db);

protected:
  FF_Wizard *m_wizard;
  FXString m_name;
  int m_index;
};



class FF_Wizard: public FXDialogBox {
  FXDECLARE(FF_Wizard)
  friend class FF_WizardPage;
public:

  enum {
    ID_BUTTON_NEXT=FXDialogBox::ID_LAST,
    ID_BUTTON_BACK,
    ID_LAST
  };

  FF_Wizard(FXWindow *owner, const FXString &title,
	    const FXString &name,
	    FXImage *image,
	    FXuint opts=DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,
	    FXint x=0, FXint y=0, FXint w=0, FXint h=0,
	    FXint pl=10, FXint pr=10, FXint pt=10, FXint pb=10, FXint hs=10, FXint vs=10);

  ~FF_Wizard();

  void create();

  void enableNext();
  void disableNext();

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

  long onCmdNext(FXObject*, FXSelector, void *ptr);
  long onCmdBack(FXObject*, FXSelector, void *ptr);

  virtual void init();
  virtual void fini();

  virtual FXuint execute(FXuint placement=PLACEMENT_CURSOR);

protected:
  FF_Wizard();

  FF_WizardPage *pageAtIndex(int i);

  FXString m_name;

  FXImage *m_image;
  int m_pageCounter;

  FXImageFrame *m_imageFrame;
  FXSwitcher *m_stack;

  FXButton *m_nextButton;
  FXButton *m_backButton;
  FXButton *m_cancelButton;
};



#endif




