/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_edit_printdoc.hpp"

#include "ff_app.hpp"
#include "ff_printdocview.hpp"

#include <aqfinance/print/af_print.h>
#include <aqfinance/print/af_print_doc.h>
#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmltext.hpp>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/directory.h>

#include <fx.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg


#define RESIZE_GRIP_SIZE 7
#define MOVE_RASTER      7

#define PAGE_DPI            300.0
#define PAGE_DEFAULT_WIDTH  2490
#define PAGE_DEFAULT_HEIGHT 3510

#define BOX_LEFT_BORDER     (0.787*PAGE_DPI)
#define BOX_RIGHT_BORDER    (0.787*PAGE_DPI)
#define BOX_TOP_BORDER      (0.787*PAGE_DPI)
#define BOX_BOTTOM_BORDER   (0.787*PAGE_DPI)



FXDEFMAP(FF_EditPrintDoc) FF_EditPrintDocMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditPrintDoc::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditPrintDoc::onCmdCancel),
  FXMAPFUNC(SEL_PAINT, FF_EditPrintDoc::ID_CANVAS, FF_EditPrintDoc::onPaintCanvas),
  FXMAPFUNC(SEL_MOTION, FF_EditPrintDoc::ID_CANVAS, FF_EditPrintDoc::onMotionCanvas),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, FF_EditPrintDoc::ID_CANVAS, FF_EditPrintDoc::onLeftBtnPressCanvas),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE, FF_EditPrintDoc::ID_CANVAS, FF_EditPrintDoc::onLeftBtnReleaseCanvas),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FF_EditPrintDoc::ID_CANVAS, FF_EditPrintDoc::onRightBtnReleaseCanvas),

  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_PAGE_WIDTH, FF_EditPrintDoc::onCmdPageTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_PAGE_HEIGHT, FF_EditPrintDoc::onCmdPageTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_PAGE_SPINNER, FF_EditPrintDoc::onCmdPageSpinner),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_PAGE_ADDCOVERPAGE, FF_EditPrintDoc::onCmdPageAddCoverPage),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_PAGE_ADDFIRSTPAGE, FF_EditPrintDoc::onCmdPageAddFirstPage),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_PAGE_ADDFOLLOWINGPAGE, FF_EditPrintDoc::onCmdPageAddFollowingPage),

  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_X, FF_EditPrintDoc::onCmdBoxTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_Y, FF_EditPrintDoc::onCmdBoxTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_WIDTH, FF_EditPrintDoc::onCmdBoxTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_HEIGHT, FF_EditPrintDoc::onCmdBoxTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_CONTENT, FF_EditPrintDoc::onCmdBoxTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_FONTSIZE, FF_EditPrintDoc::onCmdBoxTextField),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDCONTENTBOX, FF_EditPrintDoc::onCmdBoxAddContentBox),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDHEADERBOX, FF_EditPrintDoc::onCmdBoxAddHeaderBox),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDFOOTERBOX, FF_EditPrintDoc::onCmdBoxAddFooterBox),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDSENDERADDRBOX, FF_EditPrintDoc::onCmdBoxAddSenderAddressBox),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDRECIPIENTADDRBOX, FF_EditPrintDoc::onCmdBoxAddRecipientAddressBox),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDLOGOBOX, FF_EditPrintDoc::onCmdBoxAddLogoBox),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDUSERBOX1, FF_EditPrintDoc::onCmdBoxAddUserBox1),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDUSERBOX2, FF_EditPrintDoc::onCmdBoxAddUserBox2),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDUSERBOX3, FF_EditPrintDoc::onCmdBoxAddUserBox3),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDUSERBOX4, FF_EditPrintDoc::onCmdBoxAddUserBox4),
  FXMAPFUNC(SEL_COMMAND, FF_EditPrintDoc::ID_BOX_ADDUSERBOX5, FF_EditPrintDoc::onCmdBoxAddUserBox5),

};


FXIMPLEMENT(FF_EditPrintDoc, FXDialogBox,
            FF_EditPrintDocMap, ARRAYNUMBER(FF_EditPrintDocMap))




FF_EditPrintDoc::FF_EditPrintDoc(FF_App* a,
				 FXComposite *parent,
				 const FXString& title,
				 FXuint opts,
				 FXint x, FXint y, FXint w, FXint h,
				 FXint pl, FXint pr, FXint pt, FXint pb,
				 FXint hs, FXint vs)
:FXDialogBox(a, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_workDoc(NULL)
,m_currentPage(0)
,m_currentBox(0)
,m_xFactor(1.0)
,m_yFactor(1.0)
,m_moveMode(MoveMode_None)
,m_leftRightCursor(NULL)
,m_upDownCursor(NULL)
,m_moveCursor(NULL)
,m_arrowCursor(NULL)
,m_menuAddPage(NULL)
,m_menuAddBox(NULL)
{
  FXVerticalFrame *vf;
  FXVerticalFrame *vf2;
  FXVerticalFrame *vf3;
  FXHorizontalFrame *hf;
  FXHorizontalFrame *hf2;
  FXGroupBox *gb;
  FXMatrix *m;

  m_leftRightCursor=new FXCursor(a, CURSOR_LEFTRIGHT);
  m_upDownCursor=new FXCursor(a, CURSOR_UPDOWN);
  m_moveCursor=new FXCursor(a, CURSOR_MOVE);
  m_arrowCursor=new FXCursor(a, CURSOR_ARROW);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  gb=new FXGroupBox(hf, I18N("Page Layout"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_canvas=new FXCanvas(gb, this, ID_CANVAS, FRAME_NORMAL | LAYOUT_FILL_X | LAYOUT_FILL_Y);

  vf2=new FXVerticalFrame(hf, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  /* Document settings */
  gb=new FXGroupBox(vf2, I18N("FF_EditPrintDoc|Document"), FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("FF_EditPrintDoc|Category"), NULL, LABEL_NORMAL);
  m_docCategoryEdit=new FXTextField(m, 24, this, ID_DOC_CATEGORY, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("FF_EditPrintDoc|Name"), NULL, LABEL_NORMAL);
  m_docNameEdit=new FXTextField(m, 24, this, ID_DOC_NAME, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* Page settings */
  gb=new FXGroupBox(vf2, I18N("Page"), FRAME_LINE);
  vf3=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m=new FXMatrix(vf3, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new THEMELABEL(m, I18N("Type"), NULL, LABEL_NORMAL);
  m_pageTypeLabel=new THEMELABEL(m, "", NULL, JUSTIFY_LEFT|LABEL_NORMAL|LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Width"), NULL, LABEL_NORMAL);
  m_pageWidthEdit=new FXTextField(m, 24, this, ID_PAGE_WIDTH, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Height"), NULL, LABEL_NORMAL);
  m_pageHeightEdit=new FXTextField(m, 24, this, ID_PAGE_HEIGHT, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Current Page"), NULL, LABEL_NORMAL);
  m_pageSpinner=new FXSpinner(m, 24, this, ID_PAGE_SPINNER, SPIN_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  m_menuAddPage=new FXMenuPane(this, POPUP_VERTICAL|FRAME_RAISED|FRAME_THICK);
  m_cmdAddCoverPage=new FXMenuCommand(m_menuAddPage, I18N("FF_EditPrintDoc|Add Cover Page"), NULL, this, ID_PAGE_ADDCOVERPAGE);
  m_cmdAddFirstPage=new FXMenuCommand(m_menuAddPage, I18N("FF_EditPrintDoc|Add First Page"), NULL, this, ID_PAGE_ADDFIRSTPAGE);
  m_cmdAddFollowingPages=new FXMenuCommand(m_menuAddPage, I18N("FF_EditPrintDoc|Add Following Page"), NULL, this, ID_PAGE_ADDFOLLOWINGPAGE);
  new THEMEMENUBUTTON(vf3, I18N("Add Page"), NULL, m_menuAddPage, JUSTIFY_NORMAL|ICON_AFTER_TEXT | MENUBUTTON_DOWN|FRAME_RAISED);

  /* Box settings */
  gb=new FXGroupBox(vf2, I18N("Current Box"), FRAME_LINE);
  vf3=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m=new FXMatrix(vf3, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new THEMELABEL(m, I18N("Type"), NULL, LABEL_NORMAL);
  m_boxTypeLabel=new THEMELABEL(m, "", NULL, JUSTIFY_LEFT|LABEL_NORMAL|LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("X"), NULL, LABEL_NORMAL);
  m_boxXEdit=new FXTextField(m, 24, this, ID_BOX_X, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Y"), NULL, LABEL_NORMAL);
  m_boxYEdit=new FXTextField(m, 24, this, ID_BOX_Y, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Width"), NULL, LABEL_NORMAL);
  m_boxWidthEdit=new FXTextField(m, 24, this, ID_BOX_WIDTH, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Height"), NULL, LABEL_NORMAL);
  m_boxHeightEdit=new FXTextField(m, 24, this, ID_BOX_HEIGHT, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Font Size"), NULL, LABEL_NORMAL);
  m_boxFontSizeEdit=new FXTextField(m, 24, this, ID_BOX_FONTSIZE, TEXTFIELD_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Content"), NULL, LABEL_NORMAL);
  hf2=new FXHorizontalFrame(m, LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FILL_COLUMN, 0, 0, 0, 0, 0, 0, 0, 0);
  m_boxContentEdit=new FXTextField(hf2, 24, this, ID_BOX_CONTENT, TEXTFIELD_NORMAL | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
  new THEMEBUTTON(hf2, I18N("Enter"), NULL, this, ID_BOX_CONTENT_BUTTON, BUTTON_NORMAL);

  m_menuAddBox=new FXMenuPane(this, POPUP_VERTICAL|FRAME_RAISED|FRAME_THICK);
  m_cmdAddContentBox=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add Content Box"), NULL, this, ID_BOX_ADDCONTENTBOX);
  m_cmdAddHeaderBox=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add Header Box"), NULL, this, ID_BOX_ADDHEADERBOX);
  m_cmdAddFooterBox=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add Footer Box"), NULL, this, ID_BOX_ADDFOOTERBOX);
  m_cmdAddSenderAddressBox=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add Sender Address Box"), NULL, this, ID_BOX_ADDSENDERADDRBOX);
  m_cmdAddRecipientAddressBox=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add Recipient Address Box"), NULL, this, ID_BOX_ADDRECIPIENTADDRBOX);
  m_cmdAddLogoBox=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add Logo Box"), NULL, this, ID_BOX_ADDLOGOBOX);
  m_cmdAddUserBox1=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add User Box 1"), NULL, this, ID_BOX_ADDUSERBOX1);
  m_cmdAddUserBox2=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add User Box 2"), NULL, this, ID_BOX_ADDUSERBOX2);
  m_cmdAddUserBox3=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add User Box 3"), NULL, this, ID_BOX_ADDUSERBOX3);
  m_cmdAddUserBox4=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add User Box 4"), NULL, this, ID_BOX_ADDUSERBOX4);
  m_cmdAddUserBox5=new FXMenuCommand(m_menuAddBox, I18N("FF_EditPrintDoc|Add User Box 5"), NULL, this, ID_BOX_ADDUSERBOX5);
  new THEMEMENUBUTTON(vf3, I18N("Add Box"), NULL, m_menuAddBox, JUSTIFY_NORMAL|ICON_AFTER_TEXT | MENUBUTTON_DOWN|FRAME_RAISED);

  gb=new FXGroupBox(vf2, I18N("Info"), FRAME_LINE | LAYOUT_FILL_X | LAYOUT_FILL_Y);
  vf3=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  new FOX16_HtmlText(vf3,
		     I18N("Every document template can contain multiple <b>page templates</b>:"
			  "<ul>"
			  "<li>a template for a cover page</li>"
			  "<li>a template for the first text page</li>"
			  "<li>a template for following pages</li>"
			  "</ul>"
			  "<p>Every page contains at least a single <b>text box</b>.</p>"
			  "<p>The most important box is the <i>content box</i> which contains "
			  "the documents text.</p>"
			  "<p>There are also other box types like <i>header box, footer box</i> "
			  "etc.</p>"
			  "<p>Standard dimensions (page sizes, font sizes etc) are all based on a print page "
			  "resolution of 300 DPI. All dimensions are given in points.</p>"),
		     LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* =================== Dialog Controls ==================== */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  m_abortButton=new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
				BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
			     BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_EditPrintDoc::FF_EditPrintDoc()
:FXDialogBox()
,m_moveMode(MoveMode_None)
,m_leftRightCursor(NULL)
,m_upDownCursor(NULL)
,m_moveCursor(NULL)
,m_arrowCursor(NULL)
,m_menuAddPage(NULL)
,m_menuAddBox(NULL)
{
}



FF_EditPrintDoc::~FF_EditPrintDoc() {
  if (m_leftRightCursor)
    delete m_leftRightCursor;
  if (m_upDownCursor)
    delete m_upDownCursor;
  if (m_moveCursor)
    delete m_moveCursor;
  if (m_arrowCursor)
    delete m_arrowCursor;
  if (m_menuAddPage)
    delete m_menuAddPage;
  if (m_menuAddBox)
    delete m_menuAddBox;
}



void FF_EditPrintDoc::create() {
  FXDialogBox::create();
  if (m_leftRightCursor)
    m_leftRightCursor->create();
  if (m_upDownCursor)
    m_upDownCursor->create();
  if (m_moveCursor)
    m_moveCursor->create();
  if (m_arrowCursor)
    m_arrowCursor->create();

  if (m_menuAddPage)
    m_menuAddPage->create();
  if (m_menuAddBox)
    m_menuAddBox->create();
}



void FF_EditPrintDoc::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditPrintDoc");
  if (db==NULL) {
    db=GWEN_DB_Group_new("config");
    DBG_ERROR(0, "Got no config, creating one");
  }
  else {
    DBG_ERROR(0, "Got a config");
  }

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getDefaultWidth()+400, getDefaultHeight());

  GWEN_DB_Group_free(db);
}



void FF_EditPrintDoc::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditPrintDoc", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



int FF_EditPrintDoc::checkGui(FXbool quiet) {
  FXString str;

  str=m_docCategoryEdit->getText().simplify().trim();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please enter a category name."));
      m_docCategoryEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }

  str=m_docNameEdit->getText().simplify().trim();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please enter a template name."));
      m_docNameEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}



void FF_EditPrintDoc::updateDocInfo(const AF_PRINT_DOC *doc) {
  if (doc) {
    const char *s;
    AF_PRINT_PAGE_LIST *pgl;

    pgl=AF_PrintDoc_GetPrintPages(doc);

    s=AF_PrintDoc_GetCategory(doc);
    if (s && *s)
      m_docCategoryEdit->setText(s);
    else
      m_docCategoryEdit->setText(I18N("Unnamed Template Category"));

    s=AF_PrintDoc_GetName(doc);
    if (s && *s)
      m_docNameEdit->setText(s);
    else
      m_docNameEdit->setText(I18N("Unnamed Template Document"));

#define ACTIVATE_MENU(macro_name) \
    if (m_cmdAdd##macro_name) {                                          \
      if (AF_PrintPage_List_GetById(pgl, AF_PrintPage_Id_##macro_name))  \
	m_cmdAdd##macro_name->disable();                                 \
      else                                                               \
	m_cmdAdd##macro_name->enable();                                  \
    }
    ACTIVATE_MENU(CoverPage);
    ACTIVATE_MENU(FirstPage);
    ACTIVATE_MENU(FollowingPages);
#undef ACTIVATE_MENU

  }
}



void FF_EditPrintDoc::updatePageInfo(const AF_PRINT_PAGE *pg) {
  if (pg) {
    const char *s;
    AF_PRINT_BOX_LIST *pbl;

    m_pageWidthEdit->setText(FXStringVal(AF_PrintPage_GetWidth(pg), 10));
    m_pageHeightEdit->setText(FXStringVal(AF_PrintPage_GetHeight(pg), 10));
    switch(AF_PrintPage_GetId(pg)) {
    case AF_PrintPage_Id_CoverPage:      s=I18N("Cover Page"); break;
    case AF_PrintPage_Id_FirstPage:      s=I18N("First Page"); break;
    case AF_PrintPage_Id_FollowingPages: s=I18N("Following Pages"); break;
    default:                             s=I18N("Unknown Page Type"); break;
    }
    m_pageTypeLabel->setText(s);

    pbl=AF_PrintPage_GetPrintBoxes(pg);

#define ACTIVATE_MENU(macro_name) \
    if (m_cmdAdd##macro_name) {                                          \
      if (AF_PrintBox_List_GetById(pbl, AF_PrintBox_Id_##macro_name))    \
	m_cmdAdd##macro_name->disable();                                 \
      else                                                               \
	m_cmdAdd##macro_name->enable();                                  \
    }
    ACTIVATE_MENU(ContentBox);
    ACTIVATE_MENU(HeaderBox);
    ACTIVATE_MENU(FooterBox);
    ACTIVATE_MENU(SenderAddressBox);
    ACTIVATE_MENU(RecipientAddressBox);
    ACTIVATE_MENU(LogoBox);
#undef ACTIVATE_MENU

#define ACTIVATE_MENU(macro_name, macro_num) \
    if (m_cmdAdd##macro_name) {                                               \
      if (AF_PrintBox_List_GetById(pbl, AF_PrintBox_Id_UserBox+macro_num))    \
	m_cmdAdd##macro_name->disable();                                      \
      else                                                                    \
	m_cmdAdd##macro_name->enable();                                       \
    }
    ACTIVATE_MENU(UserBox1, 0);
    ACTIVATE_MENU(UserBox2, 1);
    ACTIVATE_MENU(UserBox3, 2);
    ACTIVATE_MENU(UserBox4, 3);
    ACTIVATE_MENU(UserBox5, 4);
  }
}



void FF_EditPrintDoc::selectPage(int idx) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));
    i=idx;
    while(pg && i) {
      i--;
      pg=AF_PrintPage_List_Next(pg);
    }
    if (pg) {
      m_currentPage=idx;
      m_pageSpinner->setValue(idx);
      updatePageInfo(pg);
    }
  }
}



AF_PRINT_PAGE *FF_EditPrintDoc::getDocPage(int idx) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));
    i=idx;
    while(pg && i) {
      i--;
      pg=AF_PrintPage_List_Next(pg);
    }
    return pg;
  }

  return NULL;
}



AF_PRINT_BOX *FF_EditPrintDoc::getPageBox(int pageNum, int boxNum) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=getDocPage(pageNum);
    if (pg) {
      AF_PRINT_BOX *box;

      box=AF_PrintBox_List_First(AF_PrintPage_GetPrintBoxes(pg));
      i=boxNum;
      while(box && i) {
	i--;
	box=AF_PrintBox_List_Next(box);
      }
      return box;
    }
  }
  return NULL;
}



void FF_EditPrintDoc::updateBoxInfo(const AF_PRINT_BOX *box) {
  if (box) {
    const char *s;

    m_boxXEdit->setText(FXStringVal(AF_PrintBox_GetX(box), 10));
    m_boxYEdit->setText(FXStringVal(AF_PrintBox_GetY(box), 10));
    m_boxWidthEdit->setText(FXStringVal(AF_PrintBox_GetWidth(box), 10));
    m_boxHeightEdit->setText(FXStringVal(AF_PrintBox_GetHeight(box), 10));
    m_boxFontSizeEdit->setText(FXStringVal(AF_PrintBox_GetFontSize(box), 10));
    switch(AF_PrintBox_GetId(box)) {
    case AF_PrintBox_Id_ContentBox:          s=I18N("Content Box"); break;
    case AF_PrintBox_Id_HeaderBox:           s=I18N("Header Box"); break;
    case AF_PrintBox_Id_FooterBox:           s=I18N("Footer Box"); break;
    case AF_PrintBox_Id_SenderAddressBox:    s=I18N("Sender Address Box"); break;
    case AF_PrintBox_Id_RecipientAddressBox: s=I18N("Recipient Address Box"); break;
    case AF_PrintBox_Id_LogoBox:             s=I18N("Logo Box"); break;
    case AF_PrintBox_Id_UserBox:             s=I18N("User Box 1"); break;
    case (AF_PrintBox_Id_UserBox+1):         s=I18N("User Box 2"); break;
    case (AF_PrintBox_Id_UserBox+2):         s=I18N("User Box 3"); break;
    case (AF_PrintBox_Id_UserBox+3):         s=I18N("User Box 4"); break;
    case (AF_PrintBox_Id_UserBox+4):         s=I18N("User Box 5"); break;
    case (AF_PrintBox_Id_UserBox+5):         s=I18N("User Box 6"); break;
    default:                                 s=I18N("User Box 7"); break;
    }
    m_boxTypeLabel->setText(s);

    s=AF_PrintBox_GetText(box);
    if (s && *s)
      m_boxContentEdit->setText(s);
    else
      m_boxContentEdit->setText("");
  }
}



void FF_EditPrintDoc::selectBox(int idx) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *box;

      box=AF_PrintBox_List_First(AF_PrintPage_GetPrintBoxes(pg));
      i=idx;
      while(box && i) {
	i--;
	box=AF_PrintBox_List_Next(box);
      }
      if (box) {
	m_currentBox=idx;
        updateBoxInfo(box);
      }
    }
  }
}



int FF_EditPrintDoc::toGui(AF_PRINT_DOC *doc) {
  int i;

  AF_PrintDoc_free(m_workDoc);
  m_workDoc=AF_PrintDoc_dup(doc);
  i=AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(m_workDoc))-1;
  m_pageSpinner->setRange(0, i);

  updateDocInfo(doc);

  selectPage(0);
  selectBox(0);

  update();

  return 0;
}



int FF_EditPrintDoc::fromGui(AF_PRINT_DOC *doc) {
  if (m_workDoc) {
    int rv;
    GWEN_DB_NODE *db;
    GWEN_BUFFER *tbuf=NULL;
    const char *s;
    FXString str;

    rv=checkGui(true);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }

    s=AF_PrintDoc_GetFileName(doc);
    if (s && *s) {
      tbuf=GWEN_Buffer_new(0, strlen(s), 0, 1);
      GWEN_Buffer_AppendString(tbuf, s);
    }

    db=GWEN_DB_Group_new("doc");
    AF_PrintDoc_toDb(m_workDoc, db);
    AF_PrintDoc_ReadDb(doc, db);
    if (tbuf) {
      AF_PrintDoc_SetFileName(doc, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
    GWEN_DB_Group_free(db);

    str=m_docCategoryEdit->getText().simplify().trim();
    AF_PrintDoc_SetCategory(doc, str.text());

    str=m_docNameEdit->getText().simplify().trim();
    AF_PrintDoc_SetName(doc, str.text());

    return 0;
  }

  return GWEN_ERROR_NO_DATA;
}



long FF_EditPrintDoc::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditPrintDoc::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



long FF_EditPrintDoc::onPaintCanvas(FXObject *sender, FXSelector sel, void *ptr) {
  FXEvent*ev=(FXEvent*)ptr;
  FXDCWindow dc(m_canvas, ev);

  dc.setForeground(FXRGB(255, 255, 255));
  dc.fillRectangle(0, 0, m_canvas->getWidth(), m_canvas->getHeight());

  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));
    i=m_currentPage;
    while(pg && i) {
      i--;
      pg=AF_PrintPage_List_Next(pg);
    }
    if (pg) {
      AF_PRINT_BOX *box;

      box=AF_PrintBox_List_First(AF_PrintPage_GetPrintBoxes(pg));
      i=0;
      while(box) {
	double bx;
	double by;
	double bw;
        double bh;

	bx=m_xFactor*(double)AF_PrintBox_GetX(box);
	by=m_yFactor*(double)AF_PrintBox_GetY(box);
	bw=m_xFactor*(double)AF_PrintBox_GetWidth(box);
	bh=m_yFactor*(double)AF_PrintBox_GetHeight(box);

	if (i==m_currentBox) {
	  dc.setForeground(FXRGB(255, 0, 0));
          dc.fillRectangle(bx, by, bw, bh);
	  dc.setForeground(FXRGB(0, 0, 0));
	  dc.drawRectangle(bx, by, bw, bh);
	}
	else {
	  if (AF_PrintBox_GetId(box)==AF_PrintBox_Id_ContentBox) {
	    dc.setForeground(FXRGB(255, 236, 184));
	    dc.fillRectangle(bx, by, bw, bh);
	    dc.setForeground(FXRGB(0, 0, 0));
	    dc.drawRectangle(bx, by, bw, bh);
	  }
	  else {
	    dc.setForeground(FXRGB(0, 0, 0));
	    dc.drawRectangle(bx, by, bw, bh);
	  }
	}

	i++;
	box=AF_PrintBox_List_Next(box);
      }
    }
  }

  return 1;
}



long FF_EditPrintDoc::onMotionCanvas(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;
  if (m_moveMode!=MoveMode_None) {
    AF_PRINT_BOX *box;

    box=getPageBox(m_currentPage, m_currentBox);
    if (box) {
      double pageX;
      double pageY;
      double bx;
      double by;
      double bw;
      double bh;
      double t;
      bool needUpdate=false;

      pageX=(event->win_x)/m_xFactor;
      pageY=(event->win_y)/m_yFactor;

      bx=(double)m_currentBoxDims.x;
      by=(double)m_currentBoxDims.y;
      bw=(double)m_currentBoxDims.w;
      bh=(double)m_currentBoxDims.h;

      if (m_moveMode & MoveMode_ResizeXLeft) {
	t=bx+bw; /* right */
	bx=pageX;
	bw=t-bx;
	AF_PrintBox_SetX(box, ((int)bx) & ~MOVE_RASTER);
	AF_PrintBox_SetWidth(box, ((int)bw) & ~MOVE_RASTER);
        needUpdate=true;
      }

      if (m_moveMode & MoveMode_ResizeXRight) {
        bw=pageX-bx;
	AF_PrintBox_SetWidth(box, ((int)bw) & ~MOVE_RASTER);
	needUpdate=true;
      }

      if (m_moveMode & MoveMode_ResizeYTop) {
	t=by+bh; /* bottom */
	by=pageY;
	bh=t-by;
	AF_PrintBox_SetY(box, ((int)by) & ~MOVE_RASTER);
	AF_PrintBox_SetHeight(box, ((int)bh) & ~MOVE_RASTER);
	needUpdate=true;
      }

      if (m_moveMode & MoveMode_ResizeYBottom) {
	bh=pageY-by;
	AF_PrintBox_SetHeight(box, ((int)bh) & ~MOVE_RASTER);
	needUpdate=true;
      }

      if (m_moveMode & MoveMode_Move) {
	bx=(event->win_x-m_boxHitRelX)/m_xFactor;
	by=(event->win_y-m_boxHitRelY)/m_yFactor;
	AF_PrintBox_SetX(box, ((int)bx) & ~MOVE_RASTER);
	AF_PrintBox_SetY(box, ((int)by) & ~MOVE_RASTER);
        needUpdate=true;
      }

      if (needUpdate) {
	m_canvas->update();
        updateBoxInfo(box);
      }
    }
  }

  return 1;
}



void FF_EditPrintDoc::setMoveMode(int mm) {
  if (mm!=m_moveMode) {
    FXCursor *cu=m_arrowCursor;

    DBG_ERROR(0, "Setting move mode %d", mm);
    if ((mm & MoveMode_ResizeXLeft) || (mm & MoveMode_ResizeXRight)) {
      DBG_ERROR(0, "LeftRight");
      cu=m_leftRightCursor;
    }

    if ((mm & MoveMode_ResizeYTop) || (mm & MoveMode_ResizeYBottom)) {
      DBG_ERROR(0, "UpDown");
      cu=m_upDownCursor;
    }

    if (mm==MoveMode_Move) {
      DBG_ERROR(0, "Move");
      cu=m_moveCursor;
    }

    if (cu) {
      m_canvas->setDefaultCursor(cu);
    }
    m_moveMode=mm;
  }
}



long FF_EditPrintDoc::onLeftBtnPressCanvas(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;

  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));
    i=m_currentPage;
    while(pg && i) {
      i--;
      pg=AF_PrintPage_List_Next(pg);
    }
    if (pg) {
      AF_PRINT_BOX *box;

      box=AF_PrintBox_List_First(AF_PrintPage_GetPrintBoxes(pg));
      i=0;
      while(box) {
	double bx;
	double by;
	double bw;
        double bh;

	bx=m_xFactor*(double)AF_PrintBox_GetX(box);
	by=m_yFactor*(double)AF_PrintBox_GetY(box);
	bw=m_xFactor*(double)AF_PrintBox_GetWidth(box);
	bh=m_yFactor*(double)AF_PrintBox_GetHeight(box);

        FXRectangle rect(bx-RESIZE_GRIP_SIZE, by-RESIZE_GRIP_SIZE, bw+(2.0*RESIZE_GRIP_SIZE), bh+(2.0*RESIZE_GRIP_SIZE));
	if (rect.contains(event->win_x, event->win_y)) {
	  double relPosX;
	  double relPosY;
	  int newMoveMode=MoveMode_None;

	  selectBox(i);

	  relPosX=event->win_x-bx;
	  relPosY=event->win_y-by;

	  if (relPosX>-RESIZE_GRIP_SIZE && relPosX<RESIZE_GRIP_SIZE)
	    newMoveMode|=MoveMode_ResizeXLeft;
	  if (relPosX>(bw-RESIZE_GRIP_SIZE) && relPosX<(bw+RESIZE_GRIP_SIZE))
	    newMoveMode|=MoveMode_ResizeXRight;

	  if (relPosY>-RESIZE_GRIP_SIZE && relPosY<RESIZE_GRIP_SIZE)
	    newMoveMode|=MoveMode_ResizeYTop;
	  if (relPosY>(bh-RESIZE_GRIP_SIZE) && relPosY<(bh+RESIZE_GRIP_SIZE))
	    newMoveMode|=MoveMode_ResizeYBottom;

	  if (newMoveMode==0)
            newMoveMode=MoveMode_Move;

	  m_boxHitRelX=relPosX;
	  m_boxHitRelY=relPosY;
	  m_currentBoxDims=FXRectangle(AF_PrintBox_GetX(box),
				       AF_PrintBox_GetY(box),
				       AF_PrintBox_GetWidth(box),
                                       AF_PrintBox_GetHeight(box));
	  setMoveMode(newMoveMode);

	  m_canvas->update();
	  break;
	}

	i++;
	box=AF_PrintBox_List_Next(box);
      }

    }
  }

  return 1;
}



long FF_EditPrintDoc::onLeftBtnReleaseCanvas(FXObject*, FXSelector, void *ptr) {
  //FXEvent* event=(FXEvent*)ptr;

  if (m_workDoc) {
    /* nothing to do here ATM */
  }

  setMoveMode(MoveMode_None);
  return 1;
}



long FF_EditPrintDoc::onRightBtnReleaseCanvas(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditPrintDoc::onCmdPageTextField(FXObject*, FXSelector, void *ptr) {
  AF_PRINT_PAGE *pg;

  pg=getDocPage(m_currentPage);
  if (pg) {
    FXlong n;

    n=FXLongVal(m_pageWidthEdit->getText(), 10);
    AF_PrintPage_SetWidth(pg, n);

    n=FXLongVal(m_pageHeightEdit->getText(), 10);
    AF_PrintPage_SetHeight(pg, n);

    layout();
    m_canvas->update();
  }

  return 1;
}



long FF_EditPrintDoc::onCmdPageSpinner(FXObject*, FXSelector, void *ptr) {
  selectPage(m_pageSpinner->getValue());
  selectBox(0);
  m_canvas->update();
  return 1;
}



long FF_EditPrintDoc::onCmdPageAddEmpty(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *fpg;
    AF_PRINT_PAGE *pg;
    AF_PRINT_BOX *pb;
    int i;

    fpg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));

    pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(m_workDoc), AF_PrintPage_Id_FirstPage);
    if (pg) {
      pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(m_workDoc), AF_PrintPage_Id_FollowingPages);
      if (pg) {
	pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(m_workDoc), AF_PrintPage_Id_CoverPage);
	if (pg) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("This document template already contains all possible pages."));
          return 1;
	}
	else {
	  /* create cover page */
	  pg=AF_PrintPage_new();
	  if (fpg) {
	    AF_PrintPage_SetWidth(pg, AF_PrintPage_GetWidth(fpg));
	    AF_PrintPage_SetHeight(pg, AF_PrintPage_GetHeight(fpg));
	  }
	  else {
	    AF_PrintPage_SetWidth(pg, PAGE_DEFAULT_WIDTH);
	    AF_PrintPage_SetHeight(pg, PAGE_DEFAULT_HEIGHT);
	  }
	  AF_PrintPage_SetId(pg, AF_PrintPage_Id_CoverPage);

	  /* create cover box */
	  pb=AF_PrintBox_new();
	  AF_PrintBox_SetId(pb, AF_PrintBox_Id_HeaderBox);
	  AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
	  AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
	  AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
	  AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
	  AF_PrintBox_SetFontSize(pb, 56);
	  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
	}
      }
      else {
	/* create following page */
	pg=AF_PrintPage_new();
	if (fpg) {
	  AF_PrintPage_SetWidth(pg, AF_PrintPage_GetWidth(fpg));
	  AF_PrintPage_SetHeight(pg, AF_PrintPage_GetHeight(fpg));
	}
	else {
	  AF_PrintPage_SetWidth(pg, PAGE_DEFAULT_WIDTH);
	  AF_PrintPage_SetHeight(pg, PAGE_DEFAULT_HEIGHT);
	}
	AF_PrintPage_SetId(pg, AF_PrintPage_Id_FollowingPages);

	/* create content box */
	pb=AF_PrintBox_new();
	AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
	AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
	AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
	AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
	AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
	AF_PrintBox_SetFontSize(pb, 56);
	AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      }
    }
    else {
      /* create first page */
      pg=AF_PrintPage_new();
      if (fpg) {
	AF_PrintPage_SetWidth(pg, AF_PrintPage_GetWidth(fpg));
	AF_PrintPage_SetHeight(pg, AF_PrintPage_GetHeight(fpg));
      }
      else {
	AF_PrintPage_SetWidth(pg, PAGE_DEFAULT_WIDTH);
	AF_PrintPage_SetHeight(pg, PAGE_DEFAULT_HEIGHT);
      }
      AF_PrintPage_SetId(pg, AF_PrintPage_Id_FirstPage);

      /* create content box */
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
      AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
      AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
      AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
      AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
      AF_PrintBox_SetFontSize(pb, 56);
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      AF_PrintPage_SetId(pg, AF_PrintPage_Id_FollowingPages);
    }

    if (pg) {
      AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(m_workDoc));
      i=AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(m_workDoc))-1;
      m_pageSpinner->setRange(0, i);
      selectPage(i);
      selectBox(0);
      m_canvas->update();
    }
  }

  return 1;
}



long FF_EditPrintDoc::onCmdPageAddCoverPage(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *fpg;
    AF_PRINT_PAGE *pg;
    AF_PRINT_BOX *pb;
    int i;

    fpg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));

    /* create cover page */
    pg=AF_PrintPage_new();
    if (fpg) {
      AF_PrintPage_SetWidth(pg, AF_PrintPage_GetWidth(fpg));
      AF_PrintPage_SetHeight(pg, AF_PrintPage_GetHeight(fpg));
    }
    else {
      AF_PrintPage_SetWidth(pg, PAGE_DEFAULT_WIDTH);
      AF_PrintPage_SetHeight(pg, PAGE_DEFAULT_HEIGHT);
    }
    AF_PrintPage_SetId(pg, AF_PrintPage_Id_CoverPage);

    /* create cover box */
    pb=AF_PrintBox_new();
    AF_PrintBox_SetId(pb, AF_PrintBox_Id_HeaderBox);
    AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
    AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
    AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
    AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
    AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.25);
    AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

    AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(m_workDoc));
    i=AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(m_workDoc))-1;
    m_pageSpinner->setRange(0, i);
    selectPage(i);
    selectBox(0);
    updateDocInfo(m_workDoc);
    m_canvas->update();
  }

  return 1;
}



long FF_EditPrintDoc::onCmdPageAddFirstPage(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *fpg;
    AF_PRINT_PAGE *pg;
    AF_PRINT_BOX *pb;
    int i;

    fpg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));

    /* create cover page */
    pg=AF_PrintPage_new();
    if (fpg) {
      AF_PrintPage_SetWidth(pg, AF_PrintPage_GetWidth(fpg));
      AF_PrintPage_SetHeight(pg, AF_PrintPage_GetHeight(fpg));
    }
    else {
      AF_PrintPage_SetWidth(pg, PAGE_DEFAULT_WIDTH);
      AF_PrintPage_SetHeight(pg, PAGE_DEFAULT_HEIGHT);
    }
    AF_PrintPage_SetId(pg, AF_PrintPage_Id_FirstPage);

    /* create content box */
    pb=AF_PrintBox_new();
    AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
    AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
    AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
    AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
    AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
    AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.25);
    AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

    AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(m_workDoc));
    i=AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(m_workDoc))-1;
    m_pageSpinner->setRange(0, i);
    selectPage(i);
    selectBox(0);
    updateDocInfo(m_workDoc);
    m_canvas->update();
  }

  return 1;
}



long FF_EditPrintDoc::onCmdPageAddFollowingPage(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *fpg;
    AF_PRINT_PAGE *pg;
    AF_PRINT_BOX *pb;
    int i;

    fpg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));

    /* create cover page */
    pg=AF_PrintPage_new();
    if (fpg) {
      AF_PrintPage_SetWidth(pg, AF_PrintPage_GetWidth(fpg));
      AF_PrintPage_SetHeight(pg, AF_PrintPage_GetHeight(fpg));
    }
    else {
      AF_PrintPage_SetWidth(pg, PAGE_DEFAULT_WIDTH);
      AF_PrintPage_SetHeight(pg, PAGE_DEFAULT_HEIGHT);
    }
    AF_PrintPage_SetId(pg, AF_PrintPage_Id_FollowingPages);

    /* create content box */
    pb=AF_PrintBox_new();
    AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
    AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
    AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
    AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
    AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
    AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.25);
    AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

    AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(m_workDoc));
    i=AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(m_workDoc))-1;
    m_pageSpinner->setRange(0, i);
    selectPage(i);
    selectBox(0);
    updateDocInfo(m_workDoc);
    m_canvas->update();
  }

  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddContentBox(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
      AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
      AF_PrintBox_SetY(pb, BOX_TOP_BORDER);
      AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
      AF_PrintBox_SetHeight(pb, AF_PrintPage_GetHeight(pg)-BOX_TOP_BORDER-BOX_BOTTOM_BORDER);
      AF_PrintBox_SetFontSize(pb, 48);
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddHeaderBox(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_HeaderBox);
      AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
      AF_PrintBox_SetY(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
      AF_PrintBox_SetHeight(pb, BOX_TOP_BORDER);
      AF_PrintBox_SetFontSize(pb, 32);
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddFooterBox(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_FooterBox);
      AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
      AF_PrintBox_SetY(pb, AF_PrintPage_GetHeight(pg)-BOX_BOTTOM_BORDER-RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
      AF_PrintBox_SetHeight(pb, BOX_BOTTOM_BORDER);
      AF_PrintBox_SetFontSize(pb, 32);
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddSenderAddressBox(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_SenderAddressBox);
      AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
      AF_PrintBox_SetY(pb, 1.969*PAGE_DPI);
      AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
      AF_PrintBox_SetHeight(pb, 0.394*PAGE_DPI);
      AF_PrintBox_SetFontSize(pb, PAGE_DPI/10.0);  /* 30 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddRecipientAddressBox(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_RecipientAddressBox);
      AF_PrintBox_SetX(pb, BOX_LEFT_BORDER);
      AF_PrintBox_SetY(pb, (1.969+0.394)*PAGE_DPI);
      AF_PrintBox_SetWidth(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-BOX_RIGHT_BORDER);
      AF_PrintBox_SetHeight(pb, 1.181*PAGE_DPI);
      AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.250); /* 48 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddLogoBox(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_LogoBox);
      AF_PrintBox_SetX(pb, AF_PrintPage_GetWidth(pg)-BOX_LEFT_BORDER-320);
      AF_PrintBox_SetY(pb, 0.394*PAGE_DPI);
      AF_PrintBox_SetWidth(pb, 320);
      AF_PrintBox_SetHeight(pb, 320);
      AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.5); /* 46 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddUserBox1(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_UserBox);
      AF_PrintBox_SetX(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetY(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, BOX_LEFT_BORDER-RESIZE_GRIP_SIZE);
      AF_PrintBox_SetHeight(pb, PAGE_DPI); /* one inch */
      AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.5); /* 46 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddUserBox2(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_UserBox+1);
      AF_PrintBox_SetX(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetY(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, BOX_LEFT_BORDER-RESIZE_GRIP_SIZE);
      AF_PrintBox_SetHeight(pb, PAGE_DPI); /* one inch */
      AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.5); /* 46 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddUserBox3(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_UserBox+2);
      AF_PrintBox_SetX(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetY(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, BOX_LEFT_BORDER-RESIZE_GRIP_SIZE);
      AF_PrintBox_SetHeight(pb, PAGE_DPI); /* one inch */
      AF_PrintBox_SetFontSize(pb, PAGE_DPI/6.5); /* 46 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddUserBox4(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_UserBox+3);
      AF_PrintBox_SetX(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetY(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, BOX_LEFT_BORDER-RESIZE_GRIP_SIZE);
      AF_PrintBox_SetHeight(pb, (int) PAGE_DPI); /* one inch */
      AF_PrintBox_SetFontSize(pb, (int)(PAGE_DPI/6.5)); /* 46 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxAddUserBox5(FXObject*, FXSelector, void *ptr) {
  if (m_workDoc) {
    AF_PRINT_PAGE *pg;

    pg=getDocPage(m_currentPage);
    if (pg) {
      AF_PRINT_BOX *pb;
      AF_PRINT_BOX_LIST *pbl;

      pbl=AF_PrintPage_GetPrintBoxes(pg);
      pb=AF_PrintBox_new();
      AF_PrintBox_SetId(pb, AF_PrintBox_Id_UserBox+4);
      AF_PrintBox_SetX(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetY(pb, RESIZE_GRIP_SIZE);
      AF_PrintBox_SetWidth(pb, BOX_LEFT_BORDER-RESIZE_GRIP_SIZE);
      AF_PrintBox_SetHeight(pb, (int) PAGE_DPI); /* one inch */
      AF_PrintBox_SetFontSize(pb, (int)(PAGE_DPI/6.5)); /* 46 px at 300 dpi */
      AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
      selectBox(AF_PrintBox_List_GetCount(AF_PrintPage_GetPrintBoxes(pg))-1);
      updatePageInfo(pg);
      m_canvas->update();
    }
  }
  return 1;
}



long FF_EditPrintDoc::onCmdBoxTextField(FXObject*, FXSelector, void *ptr) {
  AF_PRINT_BOX *box;

  box=getPageBox(m_currentPage, m_currentBox);
  if (box) {
    FXlong n;
    FXString str;

    n=FXLongVal(m_boxXEdit->getText(), 10);
    AF_PrintBox_SetX(box, n);

    n=FXLongVal(m_boxYEdit->getText(), 10);
    AF_PrintBox_SetY(box, n);

    n=FXLongVal(m_boxWidthEdit->getText(), 10);
    AF_PrintBox_SetWidth(box, n);

    n=FXLongVal(m_boxHeightEdit->getText(), 10);
    AF_PrintBox_SetHeight(box, n);

    n=FXLongVal(m_boxFontSizeEdit->getText(), 10);
    AF_PrintBox_SetFontSize(box, n);

    str=m_boxContentEdit->getText().simplify().trim();
    if (str.empty())
      AF_PrintBox_SetText(box, NULL);
    else
      AF_PrintBox_SetText(box, str.text());

    m_canvas->update();
  }

  return 1;
}



void FF_EditPrintDoc::layout() {
  FXDialogBox::layout();

  if (m_workDoc) {
    AF_PRINT_PAGE *pg;
    int i;

    pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(m_workDoc));
    i=m_currentPage;
    while(pg && i) {
      i--;
      pg=AF_PrintPage_List_Next(pg);
    }
    if (pg) {
      double w, h;
      double dw, dh;

      w=m_canvas->getWidth();
      h=m_canvas->getHeight();
      dw=AF_PrintPage_GetWidth(pg);
      dh=AF_PrintPage_GetHeight(pg);
      if (dw!=0.0)
	m_xFactor=w/dw;
      else
	m_xFactor=1.0;
      if (dh!=0.0)
	m_yFactor=h/dh;
      else
	m_yFactor=1.0;

      m_canvas->update();
    }

  }
}



bool FF_EditPrintDoc::edit(FF_App* app,
			   FXComposite *parent,
			   const FXString &title,
			   AF_PRINT_DOC *doc) {
  FF_EditPrintDoc dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.create();
  dlg.init();
  dlg.toGui(doc);

  dlg.show();
  dlg.layout();

  dlgres=dlg.execute();
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  dlg.fromGui(doc);
  return true;
}


