/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDIT_PRINT_DOC_HPP
#define FF_EDIT_PRINT_DOC_HPP


#include <aqfinance/print/af_print_doc.h>

#include <aqfinance/engine/book/ae_utils.h>

#include <fx.h>


class FF_App;


class FF_EditPrintDoc: public FXDialogBox {
  FXDECLARE(FF_EditPrintDoc)

public:

  enum {
    ID_CANVAS=FXDialogBox::ID_LAST,

    ID_DOC_CATEGORY,
    ID_DOC_NAME,

    ID_PAGE_WIDTH,
    ID_PAGE_HEIGHT,
    ID_PAGE_SPINNER,

    ID_PAGE_ADDCOVERPAGE,
    ID_PAGE_ADDFIRSTPAGE,
    ID_PAGE_ADDFOLLOWINGPAGE,

    ID_BOX_X,
    ID_BOX_Y,
    ID_BOX_WIDTH,
    ID_BOX_HEIGHT,
    ID_BOX_FONTSIZE,
    ID_BOX_CONTENT,
    ID_BOX_CONTENT_BUTTON,

    ID_BOX_ADDCONTENTBOX,
    ID_BOX_ADDHEADERBOX,
    ID_BOX_ADDFOOTERBOX,
    ID_BOX_ADDSENDERADDRBOX,
    ID_BOX_ADDRECIPIENTADDRBOX,
    ID_BOX_ADDLOGOBOX,
    ID_BOX_ADDUSERBOX1,
    ID_BOX_ADDUSERBOX2,
    ID_BOX_ADDUSERBOX3,
    ID_BOX_ADDUSERBOX4,
    ID_BOX_ADDUSERBOX5,

    ID_LAST
  };


  FF_EditPrintDoc(FF_App* a,
		  FXComposite *parent,
		  const FXString& title,
		  FXuint opts=DECOR_TITLE|DECOR_BORDER,
		  FXint x=0, FXint y=0, FXint w=0, FXint h=0,
		  FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
		  FXint hs=0, FXint vs=0);
  virtual ~FF_EditPrintDoc();


  static bool edit(FF_App* app,
		   FXComposite *parent,
		   const FXString &title,
		   AF_PRINT_DOC *doc);

  void init();
  void fini();

  int checkGui(FXbool quiet);
  int toGui(AF_PRINT_DOC *doc);
  int fromGui(AF_PRINT_DOC *doc);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

  long onPaintCanvas(FXObject*, FXSelector, void *ptr);
  long onMotionCanvas(FXObject*, FXSelector, void *ptr);
  long onLeftBtnPressCanvas(FXObject*, FXSelector, void *ptr);
  long onLeftBtnReleaseCanvas(FXObject*, FXSelector, void *ptr);
  long onRightBtnReleaseCanvas(FXObject*, FXSelector, void *ptr);

  long onCmdPageTextField(FXObject*, FXSelector, void *ptr);
  long onCmdPageSpinner(FXObject*, FXSelector, void *ptr);
  long onCmdPageAddEmpty(FXObject*, FXSelector, void *ptr);

  long onCmdBoxTextField(FXObject*, FXSelector, void *ptr);

  long onCmdPageAddCoverPage(FXObject*, FXSelector, void *ptr);
  long onCmdPageAddFirstPage(FXObject*, FXSelector, void *ptr);
  long onCmdPageAddFollowingPage(FXObject*, FXSelector, void *ptr);

  long onCmdBoxAddContentBox(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddHeaderBox(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddFooterBox(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddSenderAddressBox(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddRecipientAddressBox(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddLogoBox(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddUserBox1(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddUserBox2(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddUserBox3(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddUserBox4(FXObject*, FXSelector, void *ptr);
  long onCmdBoxAddUserBox5(FXObject*, FXSelector, void *ptr);

  void layout();
  void create();

protected:

  enum {
    MoveMode_None=0,
    MoveMode_ResizeXLeft  =1,
    MoveMode_ResizeXRight =2,
    MoveMode_ResizeYTop   =4,
    MoveMode_ResizeYBottom=8,
    MoveMode_Move         =16
  };

  FF_EditPrintDoc();

  AF_PRINT_PAGE *getDocPage(int idx);
  AF_PRINT_BOX *getPageBox(int pageNum, int boxNum);

  void selectPage(int idx);
  void selectBox(int idx);
  void setMoveMode(int mm);

  void updateDocInfo(const AF_PRINT_DOC *doc);
  void updatePageInfo(const AF_PRINT_PAGE *pg);
  void updateBoxInfo(const AF_PRINT_BOX *box);

  FF_App *m_app;
  AF_PRINT_DOC *m_workDoc;
  int m_currentPage;
  int m_currentBox;
  double m_xFactor;
  double m_yFactor;

  int m_moveMode;
  double m_boxHitRelX;
  double m_boxHitRelY;
  FXRectangle m_currentBoxDims;

  FXCursor *m_leftRightCursor;
  FXCursor *m_upDownCursor;
  FXCursor *m_moveCursor;
  FXCursor *m_arrowCursor;

  FXCanvas *m_canvas;

  FXTextField *m_docCategoryEdit;
  FXTextField *m_docNameEdit;

  FXLabel *m_pageTypeLabel;
  FXTextField *m_pageWidthEdit;
  FXTextField *m_pageHeightEdit;
  FXSpinner  *m_pageSpinner;

  FXLabel *m_boxTypeLabel;
  FXTextField *m_boxXEdit;
  FXTextField *m_boxYEdit;
  FXTextField *m_boxWidthEdit;
  FXTextField *m_boxHeightEdit;
  FXTextField *m_boxFontSizeEdit;
  FXTextField *m_boxContentEdit;

  FXButton *m_okButton;
  FXButton *m_abortButton;

  FXMenuPane *m_menuAddPage;
  FXMenuCommand *m_cmdAddCoverPage;
  FXMenuCommand *m_cmdAddFirstPage;
  FXMenuCommand *m_cmdAddFollowingPages;

  FXMenuPane *m_menuAddBox;
  FXMenuCommand *m_cmdAddContentBox;
  FXMenuCommand *m_cmdAddHeaderBox;
  FXMenuCommand *m_cmdAddFooterBox;
  FXMenuCommand *m_cmdAddSenderAddressBox;
  FXMenuCommand *m_cmdAddRecipientAddressBox;
  FXMenuCommand *m_cmdAddLogoBox;
  FXMenuCommand *m_cmdAddUserBox1;
  FXMenuCommand *m_cmdAddUserBox2;
  FXMenuCommand *m_cmdAddUserBox3;
  FXMenuCommand *m_cmdAddUserBox4;
  FXMenuCommand *m_cmdAddUserBox5;

};




#endif

