/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_edit_printtabledescr.hpp"

#include "ff_app.hpp"
#include "ff_printdocview.hpp"

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/directory.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg



FXDEFMAP(FF_EditPrintTableDescr) FF_EditPrintTableDescrMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditPrintTableDescr::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditPrintTableDescr::onCmdCancel),
};


FXIMPLEMENT(FF_EditPrintTableDescr, FXDialogBox,
            FF_EditPrintTableDescrMap, ARRAYNUMBER(FF_EditPrintTableDescrMap))



FF_EditPrintTableDescr::FF_EditPrintTableDescr(int maxColumns,
					       FF_App* a,
					       FXComposite *parent,
					       const FXString& title,
					       FXuint opts,
					       FXint x, FXint y, FXint w, FXint h,
					       FXint pl, FXint pr, FXint pt, FXint pb,
					       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_fieldDescriptions(NULL)
{
  FXVerticalFrame *vf;
  FXVerticalFrame *vf2;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXMatrix *m;
  int i;

  m_verticalFrame=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_tabBook=new FXTabBook(m_verticalFrame, this, ID_TABBOOK, LAYOUT_FILL_X|LAYOUT_FILL_Y);

  /* =================== Document Template Tab ==================== */

  new THEMETABITEM(m_tabBook, I18N("FF_EditPrintTableDescr|Template"), NULL,
		   JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  vf=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_docView=new FF_PrintDocView(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);


  /* =================== Prologue Tab ==================== */

  new THEMETABITEM(m_tabBook, I18N("FF_EditPrintTableDescr|Prologue"), NULL,
		   JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  vf=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_prologueLabel=new FOX16_HtmlLabel(vf,
				      I18N("Please enter the text to be inserted before the data table"),
				      LAYOUT_FILL_X);
  gb=new FXGroupBox(vf, I18N("Text"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_prologueEdit=new FXText(gb, this, ID_EDIT_PROLOGUE, LAYOUT_FILL_X | LAYOUT_FILL_Y);


  /* =================== Columns Tab ==================== */

  new THEMETABITEM(m_tabBook, I18N("FF_EditPrintTableDescr|Data Table Columns"), NULL,
		   JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  vf=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_scrollWindow=new FXScrollWindow(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  vf2=new FXVerticalFrame(m_scrollWindow, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* handle columns */
  m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  for (i=0; i<maxColumns; i++) {
    FXComboBox *cb;
    char numbuf[64];

    snprintf(numbuf, sizeof(numbuf)-1, I18N("FF_EditPrintTableDescr|Column %d"), i+1);
    new FXLabel(m, numbuf, NULL, LABEL_NORMAL);

    cb=new THEMECOMBOBOX(m, 32, this, ID_FIRST_COLUMN_COMBO+i, COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK);
    m_columns.push_back(cb);
  }


  /* =================== Epilogue tab ==================== */

  new THEMETABITEM(m_tabBook, I18N("FF_EditPrintTableDescr|Epilogue"), NULL,
		   JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  vf=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_epilogueLabel=new FOX16_HtmlLabel(vf,
				      I18N("Please enter the text to be inserted after the data table"),
				      LAYOUT_FILL_X);
  gb=new FXGroupBox(vf, I18N("Text"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_epilogueEdit=new FXText(gb, this, ID_EDIT_PROLOGUE, LAYOUT_FILL_X | LAYOUT_FILL_Y);


  /* =================== Email tab ==================== */

  new THEMETABITEM(m_tabBook, I18N("FF_EditPrintTableDescr|Email"), NULL,
		   JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  vf=new FXVerticalFrame(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_emailLabel=new FOX16_HtmlLabel(vf,
				   I18N("Please enter the text to be used as body when sending as email"),
				   LAYOUT_FILL_X);
  gb=new FXGroupBox(vf, I18N("Text"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_emailEdit=new FXText(gb, this, ID_EDIT_EMAIL, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* =================== Dialog Controls ==================== */
  new FXHorizontalSeparator(m_verticalFrame, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(m_verticalFrame, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  m_abortButton=new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
				BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
			     BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_EditPrintTableDescr::FF_EditPrintTableDescr()
:FXDialogBox()
,m_app(NULL)
,m_fieldDescriptions(NULL)
{
}



FF_EditPrintTableDescr::~FF_EditPrintTableDescr() {
}



void FF_EditPrintTableDescr::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;
  const AE_FIELD_DESCRIPTION *fd;

  db=m_app->getDialogConfig("FF_EditPrintTableDescr");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  fd=getFieldDescriptions();
  if (fd) {
    std::list<FXComboBox*>::iterator it;
    const AE_FIELD_DESCRIPTION *d;
    int i=1;

    for (it=m_columns.begin(); it!=m_columns.end(); it++) {
      (*it)->appendItem(I18N("(select)"));
      d=fd;
      while(d && d->name) {
	(*it)->appendItem(I18N(d->text));
        i++;
	d++;
      }
      if (i<2) i=2;
      if (i>20) i=20;
      (*it)->setNumVisible(i);
    }
  }

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "docView");
  m_docView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_EditPrintTableDescr::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "docView");
  m_docView->fini(dbT);

  rv=m_app->setDialogConfig("FF_EditPrintTableDescr", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



int FF_EditPrintTableDescr::checkGui(FXbool quiet) {
  FF_PrintDocTree *tree;
  AF_PRINT_DOC *currentDoc;
  std::list<FXComboBox*>::iterator it;

  tree=m_docView->getDocTree();
  currentDoc=tree->getCurrentDoc();
  if (currentDoc==NULL) {
    if (!quiet) {
      m_tabBook->setCurrent(0);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please select a page template."));
      return GWEN_ERROR_BAD_DATA;
    }
  }

  for (it=m_columns.begin(); it!=m_columns.end(); it++) {
    if ((*it)->getCurrentItem()!=0)
      break;
  }
  if (it==m_columns.end()) {
    if (!quiet) {
      m_tabBook->setCurrent(2);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please set at least one column."));
      return GWEN_ERROR_BAD_DATA;
    }
  }

  return 0;
}



int FF_EditPrintTableDescr::toGui(AF_PRINT_TABLE_DESCR *descr) {
  const char *s;
  const AE_FIELD_DESCRIPTION *fd;
  const char *name;

  reloadDocs();
  name=AF_PrintTableDescr_GetDocTemplateName(descr);
  if (name) {
    FF_PrintDocTree *tree;
    FF_PrintDocTreeItem *item;

    tree=m_docView->getDocTree();
    item=tree->findDocItem(name?name:"");
    if (item) {
      tree->unselectAll();
      tree->setCurrentItem(item);
      tree->selectItem(item);
      tree->makeItemVisible(item);
    }
  }

  fd=getFieldDescriptions();
  if (fd) {
    s=AF_PrintTableDescr_GetColumnList(descr);
    if (s && *s) {
      FXString str;
      FXString t;
      FXint n=0;
      std::list<FXComboBox*>::iterator it;

      str=FXString(s);
      it=m_columns.begin();
      while(!(t=str.section('\t',n)).empty()){
	const AE_FIELD_DESCRIPTION *d;
	int i=0;

	if (it==m_columns.end())
          break;

	d=fd;
	while(d->name && strcasecmp(d->name, t.text())!=0) {
	  i++;
	  d++;
	}
	if (d->name)
	  (*it)->setCurrentItem(i+1);
	it++;
	n++;
      }
    }
  }

  s=AF_PrintTableDescr_GetPrologue(descr);
  if (s && *s)
    m_prologueEdit->setText(s);

  s=AF_PrintTableDescr_GetEpilogue(descr);
  if (s && *s)
    m_epilogueEdit->setText(s);

  s=AF_PrintTableDescr_GetEmailBody(descr);
  if (s && *s)
    m_emailEdit->setText(s);

  return 0;
}



int FF_EditPrintTableDescr::fromGui(AF_PRINT_TABLE_DESCR *descr) {
  FF_PrintDocTree *tree;
  AF_PRINT_DOC *currentDoc;
  FXString str;
  const char *s;
  const AE_FIELD_DESCRIPTION *fd;

  tree=m_docView->getDocTree();
  currentDoc=tree->getCurrentDoc();
  if (currentDoc==NULL)
    return GWEN_ERROR_BAD_DATA;

  s=AF_PrintDoc_GetName(currentDoc);
  AF_PrintTableDescr_SetDocTemplateName(descr, s);

  AF_PrintTableDescr_SetColumnList(descr, NULL);
  fd=getFieldDescriptions();
  if (fd) {
    std::list<FXComboBox*>::iterator it;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    for (it=m_columns.begin(); it!=m_columns.end(); it++) {
      int currentItem;

      currentItem=(*it)->getCurrentItem();
      if (currentItem!=0) {
        if (GWEN_Buffer_GetUsedBytes(tbuf))
	  GWEN_Buffer_AppendByte(tbuf, '\t');
	GWEN_Buffer_AppendString(tbuf, fd[currentItem-1].name);
      }
    }
    AF_PrintTableDescr_SetColumnList(descr, GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }

  str=m_prologueEdit->getText();
  if (str.empty())
    AF_PrintTableDescr_SetPrologue(descr, NULL);
  else
    AF_PrintTableDescr_SetPrologue(descr, str.text());

  str=m_epilogueEdit->getText();
  if (str.empty())
    AF_PrintTableDescr_SetEpilogue(descr, NULL);
  else
    AF_PrintTableDescr_SetEpilogue(descr, str.text());

  str=m_emailEdit->getText();
  if (str.empty())
    AF_PrintTableDescr_SetEmailBody(descr, NULL);
  else
    AF_PrintTableDescr_SetEmailBody(descr, str.text());

  return 0;
}




long FF_EditPrintTableDescr::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditPrintTableDescr::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



void FF_EditPrintTableDescr::reloadDocs() {
  FXString dataFolder;
  GWEN_STRINGLIST *sl;
  AF_PRINT_DOC *currentDoc;
  FF_PrintDocTree *tree;

  tree=m_docView->getDocTree();
  currentDoc=tree->getCurrentDoc();
  if (currentDoc)
    AF_PrintDoc_Attach(currentDoc);

  tree->clearItems();
  dataFolder=AE_GetUserDataFolder();
  dataFolder+=GWEN_DIR_SEPARATOR_S "templates" GWEN_DIR_SEPARATOR_S "printdocs";
  DBG_ERROR(0, "Loading from [%s]", dataFolder.text());
  sl=GWEN_StringList_new();
  GWEN_Directory_GetMatchingFilesRecursively(dataFolder.text(), sl, "*.db");
  if (GWEN_StringList_Count(sl)) {
    GWEN_STRINGLISTENTRY *se;

    se=GWEN_StringList_FirstEntry(sl);
    while(se) {
      const char *s;

      s=GWEN_StringListEntry_Data(se);
      if (s && *s) {
	GWEN_DB_NODE *db;
        int rv;

	db=GWEN_DB_Group_new("doc");
        DBG_ERROR(0, "Reading file %s", s);
        rv=GWEN_DB_ReadFile(db, s, GWEN_DB_FLAGS_DEFAULT | GWEN_PATH_FLAGS_CREATE_GROUP);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	}
	else {
	  AF_PRINT_DOC *doc;

          DBG_ERROR(0, "Reading from file %s", s);
	  doc=AF_PrintDoc_fromDb(db);
	  if (doc) {
	    AF_PrintDoc_SetFileName(doc, s);
	    tree->addDoc(doc, TRUE);
	    AF_PrintDoc_free(doc);
	  }
	}
	GWEN_DB_Group_free(db);
      }
      se=GWEN_StringListEntry_Next(se);
    }
  }

  GWEN_StringList_free(sl);

  if (currentDoc) {
    const char *cat;
    const char *name;
    FF_PrintDocTreeItem *item;

    cat=AF_PrintDoc_GetCategory(currentDoc);
    name=AF_PrintDoc_GetName(currentDoc);
    item=tree->findDocItem(cat?cat:"", name?name:"");
    if (item) {
      tree->unselectAll();
      tree->setCurrentItem(item);
      tree->selectItem(item);
      tree->makeItemVisible(item);
    }
    AF_PrintDoc_free(currentDoc);
  }
}




bool FF_EditPrintTableDescr::edit(FF_App* app,
				  FXComposite *parent,
				  const FXString &title,
				  AF_PRINT_TABLE_DESCR *descr,
				  int maxColumns,
				  const AE_FIELD_DESCRIPTION *fd) {
  FF_EditPrintTableDescr dlg(maxColumns, app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.m_fieldDescriptions=fd;
  dlg.create();
  dlg.init();
  dlg.toGui(descr);

  dlg.show();
  dlg.layout();

  dlgres=dlg.execute();
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  dlg.fromGui(descr);
  return true;
}



const AE_FIELD_DESCRIPTION *FF_EditPrintTableDescr::getFieldDescriptions() const {
  return m_fieldDescriptions;
}





bool FF_EditPrintTableDescr::editForJob(FF_App* app,
					FXComposite *parent,
					const FXString &title,
					AF_PRINT_TABLE_DESCR *descr) {
  const AE_FIELD_DESCRIPTION *fd;
  int count=0;

  fd=AE_Utils_GetItemFieldDescriptions();
  if (fd) {
    const AE_FIELD_DESCRIPTION *d;

    d=fd;
    while(d->name) {
      count++;
      d++;
    }
  }

  return edit(app, parent, title, descr, count, fd);
}






