/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_EDIT_PRINT_JOB_HPP
#define FF_EDIT_PRINT_JOB_HPP


#include <aqfinance/print/af_print_table_descr.h>

#include <aqfinance/engine/book/ae_address.h>
#include <aqfinance/engine/book/ae_utils.h>

#include <fx.h>

#include <list>


class FF_App;
class FOX16_HtmlLabel;
class FF_PrintDocView;


class FF_EditPrintTableDescr: public FXDialogBox {
  FXDECLARE(FF_EditPrintTableDescr)

public:

  enum {
    ID_TABBOOK=FXDialogBox::ID_LAST,
    ID_EDIT_PROLOGUE,
    ID_EDIT_EPILOGUE,
    ID_EDIT_EMAIL,
    ID_EDIT_DOCTEMPLATE,
    ID_ADD_DOCTEMPLATE,
    ID_DEL_DOCTEMPLATE,
    ID_FIRST_COLUMN_COMBO,
    ID_LAST_COLUMN_COMBO=ID_FIRST_COLUMN_COMBO+100,

    ID_LAST
  };


  FF_EditPrintTableDescr(int maxColumns,
			 FF_App* a,
			 FXComposite *parent,
			 const FXString& title,
			 FXuint opts=DECOR_TITLE|DECOR_BORDER,
			 FXint x=0, FXint y=0, FXint w=0, FXint h=0,
			 FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
			 FXint hs=0, FXint vs=0);
  virtual ~FF_EditPrintTableDescr();


  static bool editForJob(FF_App* app,
			 FXComposite *parent,
			 const FXString &title,
			 AF_PRINT_TABLE_DESCR *descr);

  void init();
  void fini();

  int checkGui(FXbool quiet);
  int toGui(AF_PRINT_TABLE_DESCR *descr);
  int fromGui(AF_PRINT_TABLE_DESCR *descr);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

protected:

  FF_EditPrintTableDescr();

  virtual const AE_FIELD_DESCRIPTION *getFieldDescriptions() const;
  void reloadDocs();

  static bool edit(FF_App* app,
		   FXComposite *parent,
		   const FXString &title,
		   AF_PRINT_TABLE_DESCR *descr,
		   int maxColumns,
		   const AE_FIELD_DESCRIPTION *fd);

  FF_App *m_app;
  const AE_FIELD_DESCRIPTION *m_fieldDescriptions;

  FXTabBook *m_tabBook;
  FXVerticalFrame *m_verticalFrame;

  FF_PrintDocView *m_docView;

  FOX16_HtmlLabel *m_prologueLabel;
  FXText *m_prologueEdit;

  FOX16_HtmlLabel *m_epilogueLabel;
  FXText *m_epilogueEdit;

  FOX16_HtmlLabel *m_emailLabel;
  FXText *m_emailEdit;

  FXScrollWindow *m_scrollWindow;
  std::list<FXComboBox*> m_columns;


  FXButton *m_okButton;
  FXButton *m_abortButton;

};




#endif




