/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_JOBPRINTER_HPP
#define FF_JOBPRINTER_HPP


#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_utils.h>
#include <aqfinance/print/af_print_table_descr.h>

#include <gwenhywfar/db.h>


#include <fx.h>

#include <list>


class FF_App;
class FOX16_HtmlLabel;


class FF_JobPrinter: public FXDialogBox {
  FXDECLARE(FF_JobPrinter)

public:

  enum {
    ID_TABBOOK=FXDialogBox::ID_LAST,
    ID_PRINTED_CHECK,
    ID_MAILED_CHECK,
    ID_VIEW_BUTTON,
    ID_SAVE_BUTTON,
    ID_EMAIL_BUTTON,
    ID_LAST
  };


  FF_JobPrinter(FF_App* a,
		FXComposite *parent,
		const FXString& title,
		FXuint opts=DECOR_TITLE|DECOR_BORDER,
		FXint x=0, FXint y=0, FXint w=0, FXint h=0,
		FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
		FXint hs=0, FXint vs=0);
  virtual ~FF_JobPrinter();

  static bool run(FF_App* app,
		  FXComposite *parent,
		  const FXString &title,
		  AE_JOB *job,
		  const AF_PRINT_TABLE_DESCR *descr,
		  const char *tmpFileName,
		  const char *saveFileName);


  void init();
  void fini();

  int toGui(const AE_JOB *j,
            const AF_PRINT_TABLE_DESCR *descr,
	    const char *tmpFileName, const char *saveFileName);
  int fromGui(AE_JOB *j);

  long onCmdAccept(FXObject*, FXSelector, void *ptr);
  long onCmdCancel(FXObject*, FXSelector, void *ptr);

  long onCmdView(FXObject*, FXSelector, void *ptr);
  long onCmdSave(FXObject*, FXSelector, void *ptr);
  long onCmdEmail(FXObject*, FXSelector, void *ptr);

protected:

  FF_JobPrinter();

  FF_App *m_app;

  FOX16_HtmlLabel *m_label;
  FXCheckButton *m_markAsPrintedCheck;
  FXCheckButton *m_markAsMailedCheck;
  FXButton *m_printButton;
  FXButton *m_saveButton;
  FXButton *m_emailButton;

  FXString m_emailBody;
  FXString m_tmpFileName;
  FXString m_saveFileName;

  GWEN_DB_NODE *m_dbJob;
};




#endif




