/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_edit_report2_settings.hpp"
#include "ff_view_report2_settings.hpp"
#include "ff_view_report2_graph.hpp"
#include "ff_view_report2_db.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>



#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_Edit_Report2Settings) FF_Edit_Report2SettingsMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_Edit_Report2Settings::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_Edit_Report2Settings::onCmdCancel)
};


FXIMPLEMENT(FF_Edit_Report2Settings, FXDialogBox, FF_Edit_Report2SettingsMap, ARRAYNUMBER(FF_Edit_Report2SettingsMap))




FF_Edit_Report2Settings::FF_Edit_Report2Settings(FXComposite *parent,
                                                 const FXString& title,
                                                 FXuint opts,
                                                 FXint x, FXint y, FXint w, FXint h,
                                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                                 FXint hs, FXint vs)
  :FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXTabItem *tab;
  FF_View_Report2Graph *gpage;
  FXHorizontalFrame *hf;

  m_tabBook=new FXTabBook(vf, this, ID_TABBOOK, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  tab=new THEMETABITEM(m_tabBook, I18N("Admin"), NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  m_dbPage=new FF_View_Report2Db(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);


  tab=new THEMETABITEM(m_tabBook, I18N("General"), NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  tab->setTarget(this);
  tab->setSelector(ID_TABITEM);

  m_generalPage=new FF_View_Report2Settings(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  tab=new THEMETABITEM(m_tabBook, I18N("Graph 1"), NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  gpage=new FF_View_Report2Graph(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_graphPageList.push_back(gpage);

  tab=new THEMETABITEM(m_tabBook, I18N("Graph 2"), NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  gpage=new FF_View_Report2Graph(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_graphPageList.push_back(gpage);

  tab=new THEMETABITEM(m_tabBook, I18N("Graph 3"), NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  gpage=new FF_View_Report2Graph(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_graphPageList.push_back(gpage);

  tab=new THEMETABITEM(m_tabBook, I18N("Graph 4"), NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  gpage=new FF_View_Report2Graph(m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_graphPageList.push_back(gpage);

  /* last line (OK / Cancel) */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
                  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
                  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_Edit_Report2Settings::FF_Edit_Report2Settings()
:FXDialogBox()
{
}



FF_Edit_Report2Settings::~FF_Edit_Report2Settings() {
}



void FF_Edit_Report2Settings::init() {
  GWEN_DB_NODE *db;
  int w, h;
  std::list<FF_View_Report2Graph*>::iterator it;
  int i;
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());
  assert(app);

  db=app->getDialogConfig("FF_Edit_Report2Settings");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0) {
    if (w<getDefaultWidth())
      w=getDefaultWidth();
    if (h<getDefaultHeight())
      h=getDefaultHeight();
    resize(w, h);
  }

  if (m_dbPage) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, 0, "dbPage");
    m_dbPage->init(dbT);
  }

  if (m_generalPage) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, 0, "mainPage");
    m_generalPage->init(dbT);
  }

  for (it=m_graphPageList.begin(), i=1; it!=m_graphPageList.end(); it++, i++) {
    char numbuf[32];
    GWEN_DB_NODE *dbT;

    snprintf(numbuf, sizeof(numbuf)-1, "graphPage-%d", i);
    numbuf[sizeof(numbuf)-1]=0;

    dbT=GWEN_DB_GetGroup(db, 0, numbuf);
    (*it)->init(dbT);
  }

  GWEN_DB_Group_free(db);
}



void FF_Edit_Report2Settings::fini() {
  GWEN_DB_NODE *db;
  int rv;
  std::list<FF_View_Report2Graph*>::iterator it;
  int i;
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());
  assert(app);

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());
  if (m_dbPage) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "dbPage");
    m_dbPage->fini(dbT);
  }

  if (m_generalPage) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "mainPage");
    m_generalPage->fini(dbT);
  }

  for (it=m_graphPageList.begin(), i=1; it!=m_graphPageList.end(); it++, i++) {
    char numbuf[32];
    GWEN_DB_NODE *dbT;

    snprintf(numbuf, sizeof(numbuf)-1, "graphPage-%d", i);
    numbuf[sizeof(numbuf)-1]=0;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, numbuf);
    (*it)->fini(dbT);
  }

  rv=app->setDialogConfig("FF_Edit_Report2Settings", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



void FF_Edit_Report2Settings::toGui(const AF_REPORT2_SETTINGS *settings) {
  const AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;

  if (m_dbPage)
    m_dbPage->toGui(settings);

  if (m_generalPage)
    m_generalPage->toGui(settings);

  sgList=AF_Report2_Settings_GetGraphList(settings);
  if (sgList) {
    const AF_REPORT2_SETTINGS_GRAPH *sg;
    std::list<FF_View_Report2Graph*>::iterator it;

    sg=AF_Report2_SettingsGraph_List_First(sgList);
    it=m_graphPageList.begin();
    while(sg && it!=m_graphPageList.end()) {
      (*it)->toGui(sg);
      it++;
      sg=AF_Report2_SettingsGraph_List_Next(sg);
    }
  }
}



bool FF_Edit_Report2Settings::fromGui(AF_REPORT2_SETTINGS *settings) {
  AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
  std::list<FF_View_Report2Graph*>::iterator it;
  int rv;

  if (m_dbPage) {
    rv=m_dbPage->fromGui(settings);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      return false;
    }
  }

  if (m_generalPage) {
    rv=m_generalPage->fromGui(settings);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      return false;
    }
  }

  sgList=AF_Report2_SettingsGraph_List_new();
  AF_Report2_Settings_SetGraphList(settings, sgList);

  for (it=m_graphPageList.begin(); it!=m_graphPageList.end(); it++) {
    AF_REPORT2_SETTINGS_GRAPH *sg;

    sg=AF_Report2_SettingsGraph_new();
    rv=(*it)->fromGui(sg);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Report2_SettingsGraph_free(sg);
      return false;
    }
    AF_Report2_SettingsGraph_List_Add(sg, sgList);
  }

  DBG_ERROR(0, "fromGui(): Ok");
  return true;
}



long FF_Edit_Report2Settings::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  AF_REPORT2_SETTINGS *settings;
  bool rv;

  DBG_ERROR(0, "Accept");
  settings=AF_Report2_Settings_new();
  rv=fromGui(settings);
  AF_Report2_Settings_free(settings);
  if (rv) {
    DBG_ERROR(0, "Data ok, closing");
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_Edit_Report2Settings::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  /*hide();*/
  return 1;
}




bool FF_Edit_Report2Settings::editSettings(FXComposite *parent,
                                           const FXString &title,
                                           AF_REPORT2_SETTINGS *settings) {
  FF_Edit_Report2Settings dlg(parent, title, DECOR_ALL);
  FXuint dlgres;
  bool res;

  dlg.create();
  dlg.init();
  dlg.toGui(settings);
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(settings);
  if (!res) {
    DBG_ERROR(0, "Error on fromGui()");
    return false;
  }
  return true;
}



