/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_view_report2_db.hpp"
#include "ff_groupview.hpp"

#include <aqfinance/report2/af_report2_settings.h>
#include <aqfinance/report2/af_report2.h>


#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_View_Report2Db) FF_View_Report2DbMap[]={

};


FXIMPLEMENT(FF_View_Report2Db, FXHorizontalFrame, FF_View_Report2DbMap, ARRAYNUMBER(FF_View_Report2DbMap))




FF_View_Report2Db::FF_View_Report2Db(FXComposite *p, FXuint opts, FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;
  FXMatrix *m;
  FXGroupBox *gb;

  /* settings name */
  gb=new FXGroupBox(vf, I18N("FF_View_Report2Db|Name"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 1, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Description"), NULL, LABEL_NORMAL);
  m_descrEdit=new FXTextField(m, 1, this, ID_EDIT_DESCR,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  gb=new FXGroupBox(vf, I18N("FF_View_Report2Db|Group"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  hf=new FXHorizontalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_groupView=new FF_GroupView(AE_Book_TableType_GraphicReportGroup, hf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_groupView->setTarget(this);
  m_groupView->setSelector(ID_VIEW_GROUP);

}



FF_View_Report2Db::FF_View_Report2Db()
:FXHorizontalFrame()
{
}



FF_View_Report2Db::~FF_View_Report2Db() {
}



void FF_View_Report2Db::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "groupView");
  m_groupView->init(dbT);
}



void FF_View_Report2Db::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "groupView");
  m_groupView->fini(dbT);
}



void FF_View_Report2Db::toGui(const AF_REPORT2_SETTINGS *settings) {
  const char *s;
  AQDB_ID id;

  m_groupView->reload();

  s=AF_Report2_Settings_GetName(settings);
  if (s)
    m_nameEdit->setText(s);

  s=AF_Report2_Settings_GetDescription(settings);
  if (s)
    m_descrEdit->setText(s);

  id=AF_Report2_Settings_GetGroupId(settings);
  if (id!=0) {
    m_groupView->selectGroup(id, true);
    m_groupView->makeGroupVisible(id);
  }
}



bool FF_View_Report2Db::fromGui(AF_REPORT2_SETTINGS *settings) {
  FXString str;
  AE_GROUP *gr;

  str=m_nameEdit->getText().simplify();
  if (str.empty())
    AF_Report2_Settings_SetName(settings, NULL);
  else
    AF_Report2_Settings_SetName(settings, str.text());

  str=m_descrEdit->getText().simplify();
  if (str.empty())
    AF_Report2_Settings_SetDescription(settings, NULL);
  else
    AF_Report2_Settings_SetDescription(settings, str.text());

  gr=m_groupView->getFirstSelectedGroup();
  if (gr)
    AF_Report2_Settings_SetGroupId(settings, AE_Group_GetId(gr));
  else
    AF_Report2_Settings_SetGroupId(settings, 0);

  return true;
}




