/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_view_report2_graph.hpp"
#include "ff_accountedit.hpp"

#include <aqfinance/report2/af_report2_settings.h>
#include <aqfinance/report2/af_report2.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_View_Report2Graph) FF_View_Report2GraphMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_View_Report2Graph::ID_BUTTON_ENABLED, FF_View_Report2Graph::onCmdEnable)
};


FXIMPLEMENT(FF_View_Report2Graph, FXHorizontalFrame, FF_View_Report2GraphMap, ARRAYNUMBER(FF_View_Report2GraphMap))




FF_View_Report2Graph::FF_View_Report2Graph(FXComposite *p, FXuint opts, FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hfRow;
  FXGroupBox *gb;

  m_enableButton=new FXCheckButton(vf, I18N("Enabled"), this, ID_BUTTON_ENABLED,
                                   LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
  hfRow=new FXHorizontalFrame(vf, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

  /* dimensions */
  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2Graph|Data"), LAYOUT_FILL_X | FRAME_LINE);
  if (1) {
    FXVerticalFrame *vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Label"), NULL, LABEL_NORMAL);
    m_labelEdit=new FXTextField(m, 1, this, ID_EDIT_LABEL,
                                TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Data Mode"), NULL, LABEL_NORMAL);
    m_dataModeCombo=new FXComboBox(m, 1, this, ID_COMBO_DATAMODE,
                                   COMBOBOX_STATIC |
                                   FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_dataModeCombo->appendItem(I18N("-- select type --"));
    m_dataModeCombo->appendItem(I18N("AF_Report2_DataMode|direct"));
    m_dataModeCombo->appendItem(I18N("AF_Report2_DataMode|average"));
    m_dataModeCombo->appendItem(I18N("AF_Report2_DataMode|floating average"));
    m_dataModeCombo->setNumVisible(m_dataModeCombo->getNumItems());

    new FXLabel(m, I18N("Sum Type"), NULL, LABEL_NORMAL);
    m_sumTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_DATAMODE,
                                  COMBOBOX_STATIC |
                                  FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_sumTypeCombo->appendItem(I18N("-- select type --"));
    m_sumTypeCombo->appendItem(I18N("AF_REPORT2_FLAGS_SUM|total"));
    m_sumTypeCombo->appendItem(I18N("AF_REPORT2_FLAGS_SUM|positive sums only"));
    m_sumTypeCombo->appendItem(I18N("AF_REPORT2_FLAGS_SUM|negative sums only"));
    m_sumTypeCombo->setNumVisible(m_sumTypeCombo->getNumItems());

    m_reverseSignButton=new FXCheckButton(vf2, I18N("Reverse sign"), this, ID_BUTTON_REVERSE_SIGN,
                                          LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
    m_accumulateButton=new FXCheckButton(vf2, I18N("Accumulate"), this, ID_BUTTON_ACCUMULATE,
                                         LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
  }


  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2Graph|Graph"), LAYOUT_FILL_X | FRAME_LINE);
  if (1) {
    FXVerticalFrame *vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Graph Type"), NULL, LABEL_NORMAL);
    m_graphTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_GRAPHTYPE,
                                    COMBOBOX_STATIC |
                                    FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_graphTypeCombo->appendItem(I18N("-- select type --"));
    m_graphTypeCombo->appendItem(I18N("AF_GRAPH_TYPE|line"));
    m_graphTypeCombo->appendItem(I18N("AF_GRAPH_TYPE|points"));
    m_graphTypeCombo->appendItem(I18N("AF_GRAPH_TYPE|bars"));
    m_graphTypeCombo->setNumVisible(m_graphTypeCombo->getNumItems());

    new FXLabel(m, I18N("Colour"), NULL, LABEL_NORMAL);
    m_colourWell=new FXColorWell(m, 0, this, ID_COLOURWELL_COLOUR,
                                 COLORWELL_NORMAL | COLORWELL_OPAQUEONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Line Width"), NULL, LABEL_NORMAL);
    m_lineWidthSpinner=new FXSpinner(m, 1, this, ID_SPINNER_LINEWIDTH, SPIN_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Dash Type"), NULL, LABEL_NORMAL);
    m_dashTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_DASHTYPE,
                                   COMBOBOX_STATIC |
                                   FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_dashTypeCombo->appendItem(I18N("-- select type --"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|none"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|1"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|2"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|3"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|4"));
    m_dashTypeCombo->setNumVisible(m_dashTypeCombo->getNumItems());
  }



  /* account selection */
  gb=new FXGroupBox(vf, I18N("FF_View_Report2Graph|Account Selection"), LAYOUT_FILL_X | FRAME_LINE);
  if (1) {
    FXVerticalFrame *vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Account 1"), NULL, LABEL_NORMAL);
    m_accountEdit1=new FF_AccountEdit(m, this, ID_ACCEDIT_1, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

    new FXLabel(m, I18N("Account 2"), NULL, LABEL_NORMAL);
    m_accountEdit2=new FF_AccountEdit(m, this, ID_ACCEDIT_2, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
  }


}



FF_View_Report2Graph::FF_View_Report2Graph()
:FXHorizontalFrame()
{
}



FF_View_Report2Graph::~FF_View_Report2Graph() {
}



void FF_View_Report2Graph::init(GWEN_DB_NODE *db) {
  if (m_accountEdit1)
    m_accountEdit1->reload();
  if (m_accountEdit2)
    m_accountEdit2->reload();

  m_enableButton->setCheck(FALSE);
  m_graphTypeCombo->setCurrentItem(1, TRUE);
  m_sumTypeCombo->setCurrentItem(1, TRUE);
  m_dataModeCombo->setCurrentItem(1, TRUE);
  m_lineWidthSpinner->setValue(1);
  m_colourWell->setRGBA(FXRGBA(255, 0, 0, 0));
  m_dashTypeCombo->setCurrentItem(1, TRUE);
  disableInputs();
}



void FF_View_Report2Graph::fini(GWEN_DB_NODE *db) {
}



void FF_View_Report2Graph::toGui(const AF_REPORT2_SETTINGS_GRAPH *settings) {
  const char *s;
  int i;
  uint32_t scol;
  FXColor col;

  m_enableButton->setCheck((AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_ENABLED)?TRUE:FALSE);
  if (m_enableButton->getCheck())
    enableInputs();
  else
    disableInputs();

  s=AF_Report2_SettingsGraph_GetLabel(settings);
  m_labelEdit->setText(s?s:"");

  i=0;
  switch(AF_Report2_SettingsGraph_GetGraphType(settings)) {
    case AF_GRAPH_TYPE_NONE:   i=0; break;
    default:
    case AF_GRAPH_TYPE_LINE:   i=1; break;
    case AF_GRAPH_TYPE_POINTS: i=2; break;
    case AF_GRAPH_TYPE_BARS:   i=3; break;
  }
  m_graphTypeCombo->setCurrentItem(i);

  switch(AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_SUM_MASK) {
    default:
    case AF_REPORT2_FLAGS_SUM_TOTAL:    i=1; break;
    case AF_REPORT2_FLAGS_SUM_POSITIVE: i=2; break;
    case AF_REPORT2_FLAGS_SUM_NEGATIVE: i=3; break;
  }
  m_sumTypeCombo->setCurrentItem(i);

  i=0;
  switch(AF_Report2_SettingsGraph_GetDataMode(settings)) {
    default:
    case AF_Report2_DataMode_Direct:          i=1; break;
    case AF_Report2_DataMode_Average:         i=2; break;
    case AF_Report2_DataMode_FloatingAverage: i=3; break;
  }
  m_dataModeCombo->setCurrentItem(i);

  scol=AF_Report2_SettingsGraph_GetColour(settings);
  col=FXRGB((scol>>24) & 0xff, (scol>>16) & 0xff, (scol>>8) & 0xff);
  m_colourWell->setRGBA(col);

  i=AF_Report2_SettingsGraph_GetLineWidth(settings);
  if (i==0)
    i=1;
  m_lineWidthSpinner->setValue(i, TRUE);

  i=0;
  switch(AF_Report2_SettingsGraph_GetDashType(settings)) {
    default:
    case 0: i=1; break;
    case 1: i=2; break;
    case 2: i=3; break;
    case 3: i=4; break;
    case 4: i=5; break;
  }
  m_dashTypeCombo->setCurrentItem(i, TRUE);

  s=AF_Report2_SettingsGraph_GetCategoryNameAt(settings, 0);
  if (s)
    m_accountEdit1->setText(s);

  s=AF_Report2_SettingsGraph_GetCategoryNameAt(settings, 1);
  if (s)
    m_accountEdit2->setText(s);

  if (AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_REVERSE_SIGN)
    m_reverseSignButton->setCheck(TRUE);
  else
    m_reverseSignButton->setCheck(FALSE);

  if (AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_ACCUMULATE)
    m_accumulateButton->setCheck(TRUE);
  else
    m_accumulateButton->setCheck(FALSE);
}



bool FF_View_Report2Graph::fromGui(AF_REPORT2_SETTINGS_GRAPH *settings) {
  FXString str;
  uint32_t flags=0;
  int i;
  FXColor col;
  uint32_t scol;

  if (m_enableButton->getCheck())
    flags|=AF_REPORT2_FLAGS_ENABLED;
  str=m_labelEdit->getText().simplify();
  if (str.empty())
    AF_Report2_SettingsGraph_SetLabel(settings, NULL);
  else
    AF_Report2_SettingsGraph_SetLabel(settings, str.text());

  switch(m_graphTypeCombo->getCurrentItem()) {
    default:
    case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select graph type");
      m_graphTypeCombo->setFocus();
      return FALSE;
    case 1: i=AF_GRAPH_TYPE_LINE;   break;
    case 2: i=AF_GRAPH_TYPE_POINTS; break;
    case 3: i=AF_GRAPH_TYPE_BARS;   break;
  }
  AF_Report2_SettingsGraph_SetGraphType(settings, i);

  switch(m_sumTypeCombo->getCurrentItem()) {
    default:
    case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select sum type");
      m_sumTypeCombo->setFocus();
      return FALSE;
    case 1: i=AF_REPORT2_FLAGS_SUM_TOTAL;    break;
    case 2: i=AF_REPORT2_FLAGS_SUM_POSITIVE; break;
    case 3: i=AF_REPORT2_FLAGS_SUM_NEGATIVE; break;
  }
  flags|=i;


  switch(m_dataModeCombo->getCurrentItem()) {
  case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select data mode");
      m_dataModeCombo->setFocus();
      return FALSE;
  case 1: i=AF_Report2_DataMode_Direct;          break;
  case 2: i=AF_Report2_DataMode_Average;         break;
  case 3: i=AF_Report2_DataMode_FloatingAverage; break;
  }
  AF_Report2_SettingsGraph_SetDataMode(settings, i);


  col=m_colourWell->getRGBA();
  scol=
    (FXREDVAL(col)<<24) +
    (FXGREENVAL(col)<<16)+
    (FXBLUEVAL(col)<<8)+
    0xff;

  AF_Report2_SettingsGraph_SetColour(settings, scol);
  AF_Report2_SettingsGraph_SetLineWidth(settings, m_lineWidthSpinner->getValue());

  i=0;
  switch(m_dashTypeCombo->getCurrentItem()) {
    default:
    case 1: i=0; break;
    case 2: i=1; break;
    case 3: i=2; break;
    case 4: i=3; break;
    case 5: i=4; break;
  }
  AF_Report2_SettingsGraph_SetDashType(settings, i);

  if (m_reverseSignButton->getCheck())
    flags|=AF_REPORT2_FLAGS_REVERSE_SIGN;

  if (m_accumulateButton->getCheck())
    flags|=AF_REPORT2_FLAGS_ACCUMULATE;

  AF_Report2_SettingsGraph_SetFlags(settings, flags);


  str=m_accountEdit1->getText().simplify();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(settings, str.text());
  str=m_accountEdit2->getText().simplify();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(settings, str.text());


  DBG_ERROR(0, "fromGui(): ok");
  return true;
}



void FF_View_Report2Graph::enableInputs() {
  m_labelEdit->enable();
  m_graphTypeCombo->enable();
  m_sumTypeCombo->enable();
  m_dataModeCombo->enable();

  m_colourWell->enable();
  m_lineWidthSpinner->enable();
  m_dashTypeCombo->enable();

  m_reverseSignButton->enable();
  m_accumulateButton->enable();

  m_accountEdit1->enable();
  m_accountEdit2->enable();
}



void FF_View_Report2Graph::disableInputs() {
  m_labelEdit->disable();
  m_graphTypeCombo->disable();
  m_sumTypeCombo->disable();
  m_dataModeCombo->disable();

  m_colourWell->disable();
  m_lineWidthSpinner->disable();
  m_dashTypeCombo->disable();

  m_reverseSignButton->disable();
  m_accumulateButton->disable();

  m_accountEdit1->disable();
  m_accountEdit2->disable();
}



long FF_View_Report2Graph::onCmdEnable(FXObject*, FXSelector, void *ptr) {
  if (m_enableButton->getCheck())
    enableInputs();
  else
    disableInputs();
  return 1;
}








