/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_view_report2_settings.hpp"

#include "ff_dateedit.hpp"
#include "ff_datepicker.hpp"
#include "ff_commodityedit.hpp"

#include <aqfinance/report2/af_report2_settings.h>
#include <aqfinance/report2/af_report2.h>


#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_View_Report2Settings) FF_View_Report2SettingsMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_View_Report2Settings::ID_COMBO_DATESPAN, FF_View_Report2Settings::onCmdDateSpan),
  FXMAPFUNC(SEL_COMMAND, FF_View_Report2Settings::ID_BUTTON_FIRSTDATE, FF_View_Report2Settings::onCmdFirstDate),
  FXMAPFUNC(SEL_COMMAND, FF_View_Report2Settings::ID_BUTTON_LASTDATE, FF_View_Report2Settings::onCmdLastDate),

};


FXIMPLEMENT(FF_View_Report2Settings, FXHorizontalFrame, FF_View_Report2SettingsMap, ARRAYNUMBER(FF_View_Report2SettingsMap))




FF_View_Report2Settings::FF_View_Report2Settings(FXComposite *p, FXuint opts, FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hfRow;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;

  hfRow=new FXHorizontalFrame(vf, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

  /* dimensions */
  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2Settings|Dimensions"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Width"), NULL, LABEL_NORMAL);
  m_widthEdit=new FXTextField(m, 1, this, ID_EDIT_WIDTH,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Height"), NULL, LABEL_NORMAL);
  m_heightEdit=new FXTextField(m, 1, this, ID_EDIT_HEIGHT,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* data output */
  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2Settings|Data Output"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Y-Precision"), NULL, LABEL_NORMAL);
  m_yPrecisionEdit=new FXTextField(m, 1, this, ID_EDIT_YLABEL,
                                   TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Date units"), NULL, LABEL_NORMAL);
  m_dateUnitsCombo=new FXComboBox(m, 1, this, ID_COMBO_DATEUNIT,
                                  COMBOBOX_STATIC |
                                  FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_dateUnitsCombo->appendItem(I18N("-- select type --"));
  m_dateUnitsCombo->appendItem(I18N("AF_Report2_DateUnits|days"));
  m_dateUnitsCombo->appendItem(I18N("AF_Report2_DateUnits|months"));
  m_dateUnitsCombo->appendItem(I18N("AF_Report2_DateUnits|years"));
  m_dateUnitsCombo->setNumVisible(m_dateUnitsCombo->getNumItems());



  hfRow=new FXHorizontalFrame(vf, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

  /* graph settings */
  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2Settings|Graph Settings"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Title"), NULL, LABEL_NORMAL);
  m_titleEdit=new FXTextField(m, 20, this, ID_EDIT_TITLE,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Subtitle"), NULL, LABEL_NORMAL);
  m_subTitleEdit=new FXTextField(m, 20, this, ID_EDIT_SUBTITLE,
                                 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("X-Label"), NULL, LABEL_NORMAL);
  m_xLabelEdit=new FXTextField(m, 20, this, ID_EDIT_XLABEL,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Y-Label"), NULL, LABEL_NORMAL);
  m_yLabelEdit=new FXTextField(m, 20, this, ID_EDIT_YLABEL,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* data selection */
  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2Settings|Data Selection"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Datespan"), NULL, LABEL_NORMAL);
  m_dateSpanTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_DATESPAN,
                                     LAYOUT_FILL_X | COMBOBOX_STATIC |
                                     FRAME_SUNKEN | FRAME_THICK);
  m_dateSpanTypeCombo->appendItem(I18N("-- select type --"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|none"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|userdefined"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Month"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Quarter Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Half Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Month"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Quarter Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Half Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Year"));
  m_dateSpanTypeCombo->setNumVisible(m_dateSpanTypeCombo->getNumItems());

  new FXLabel(m, I18N("First Date"), NULL, LABEL_NORMAL);
  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_firstDateEdit=new FF_DateEdit(hf, this, ID_DATEEDIT_FIRST,
                                  I18N("YYYY/MM/DD"),
                                  TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_firstDateButton=new THEMEBUTTON(hf, I18N("Pick"), NULL, this, ID_BUTTON_FIRSTDATE, BUTTON_NORMAL);

  new FXLabel(m, I18N("Last Date"), NULL, LABEL_NORMAL);
  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_lastDateEdit=new FF_DateEdit(hf, this, ID_DATEEDIT_LAST,
                                 I18N("YYYY/MM/DD"),
                                 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_lastDateButton=new THEMEBUTTON(hf, I18N("Pick"), NULL, this, ID_BUTTON_LASTDATE, BUTTON_NORMAL);

  new FXLabel(m, I18N("Commodity"), NULL, LABEL_NORMAL);
  m_commodityEdit=new FF_CommodityEdit(m, this, ID_EDIT_COMMODITY,
                                       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
}



FF_View_Report2Settings::FF_View_Report2Settings()
:FXHorizontalFrame()
{
}



FF_View_Report2Settings::~FF_View_Report2Settings() {
}



void FF_View_Report2Settings::init(GWEN_DB_NODE *db) {
  m_commodityEdit->reload();
}



void FF_View_Report2Settings::fini(GWEN_DB_NODE *db) {
}



void FF_View_Report2Settings::toGui(const AF_REPORT2_SETTINGS *settings) {
  const char *s;
  int i;
  const GWEN_DATE *dt;

  i=AF_Report2_Settings_GetWidth(settings);
  if (i==0)
    i=1024;
  m_widthEdit->setText(FXStringVal(i, 10));
  i=AF_Report2_Settings_GetHeight(settings);
  if (i==0)
    i=800;
  m_heightEdit->setText(FXStringVal(i, 10));

  i=0;
  switch(AF_Report2_Settings_GetDateSpanType(settings)) {
  default:
  case AF_Report2_DateSpanType_None:              i=1;  break;
  case AF_Report2_DateSpanType_UserDefined:       i=2;  break;

  case AF_Report2_DateSpanType_ThisMonth:         i=3;  break;
  case AF_Report2_DateSpanType_ThisQuarterYear:   i=4;  break;
  case AF_Report2_DateSpanType_ThisHalfYear:      i=5;  break;
  case AF_Report2_DateSpanType_ThisYear:          i=6;  break;

  case AF_Report2_DateSpanType_LastMonth:         i=7;  break;
  case AF_Report2_DateSpanType_LastQuarterYear:   i=8;  break;
  case AF_Report2_DateSpanType_LastHalfYear:      i=9;  break;
  case AF_Report2_DateSpanType_LastYear:          i=10; break;
  }
  m_dateSpanTypeCombo->setCurrentItem(i, TRUE);

  dt=AF_Report2_Settings_GetFirstDate(settings);
  if (dt)
    m_firstDateEdit->setDate(dt);

  dt=AF_Report2_Settings_GetLastDate(settings);
  if (dt)
    m_lastDateEdit->setDate(dt);

  s=AF_Report2_Settings_GetCommodity(settings);
  if (s)
    m_commodityEdit->setCurrency(s);
  else
    m_commodityEdit->setCurrency("EUR");

  i=0;
  switch(AF_Report2_Settings_GetDateUnits(settings)) {
    case AF_Report2_DateUnits_Days:    i=1; break;
    default:
    case AF_Report2_DateUnits_Months:  i=2; break;
    case AF_Report2_DateUnits_Years:   i=3; break;
  }
  m_dateUnitsCombo->setCurrentItem(i, TRUE);

  s=AF_Report2_Settings_GetTitle(settings);
  if (s)
    m_titleEdit->setText(s);

  s=AF_Report2_Settings_GetSubTitle(settings);
  if (s)
    m_subTitleEdit->setText(s);

  s=AF_Report2_Settings_GetLabelX(settings);
  if (s)
    m_xLabelEdit->setText(s);
  else
    m_xLabelEdit->setText(I18N("Date"));

  s=AF_Report2_Settings_GetLabelY(settings);
  if (s)
    m_yLabelEdit->setText(s);
  else
    m_yLabelEdit->setText("EUR");

  m_yPrecisionEdit->setText(FXStringVal(AF_Report2_Settings_GetPrecisionY(settings), 10));

}



bool FF_View_Report2Settings::fromGui(AF_REPORT2_SETTINGS *settings) {
  FXString str;
  int i;
  GWEN_DATE *dt;

  i=FXIntVal(m_widthEdit->getText().simplify());
  AF_Report2_Settings_SetWidth(settings, i);

  i=FXIntVal(m_heightEdit->getText().simplify());
  AF_Report2_Settings_SetHeight(settings, i);

  switch(m_dateSpanTypeCombo->getCurrentItem()) {
    default:
    case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select graph type");
      m_dateSpanTypeCombo->setFocus();
      return FALSE;
    case 1:  i=AF_Report2_DateSpanType_None;            break;
    case 2:  i=AF_Report2_DateSpanType_UserDefined;     break;
    case 3:  i=AF_Report2_DateSpanType_ThisMonth;       break;
    case 4:  i=AF_Report2_DateSpanType_ThisQuarterYear; break;
    case 5:  i=AF_Report2_DateSpanType_ThisHalfYear;    break;
    case 6:  i=AF_Report2_DateSpanType_ThisYear;        break;
    case 7:  i=AF_Report2_DateSpanType_LastMonth;       break;
    case 8:  i=AF_Report2_DateSpanType_LastQuarterYear; break;
    case 9:  i=AF_Report2_DateSpanType_LastHalfYear;    break;
    case 10: i=AF_Report2_DateSpanType_LastYear;        break;
  }
  AF_Report2_Settings_SetDateSpanType(settings, i);

  dt=NULL;
  if (m_firstDateEdit->getDate(&dt)) {
    AF_Report2_Settings_SetFirstDate(settings, dt);
    GWEN_Date_free(dt);
  }

  dt=NULL;
  if (m_lastDateEdit->getDate(&dt)) {
    AF_Report2_Settings_SetLastDate(settings, dt);
    GWEN_Date_free(dt);
  }

  str=m_commodityEdit->getCurrency().simplify();
  if (str.empty())
    AF_Report2_Settings_SetCommodity(settings, NULL);
  else
    AF_Report2_Settings_SetCommodity(settings, str.text());

  switch(m_dateUnitsCombo->getCurrentItem()) {
    default:
    case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select date units");
      m_dateUnitsCombo->setFocus();
      return FALSE;
    case 1:  i=AF_Report2_DateUnits_Days;   break;
    case 2:  i=AF_Report2_DateUnits_Months; break;
    case 3:  i=AF_Report2_DateUnits_Years; break;
  }
  AF_Report2_Settings_SetDateUnits(settings, i);

  str=m_titleEdit->getText().simplify();
  if (str.empty())
    AF_Report2_Settings_SetTitle(settings, NULL);
  else
    AF_Report2_Settings_SetTitle(settings, str.text());

  str=m_subTitleEdit->getText().simplify();
  if (str.empty())
    AF_Report2_Settings_SetSubTitle(settings, NULL);
  else
    AF_Report2_Settings_SetSubTitle(settings, str.text());

  str=m_xLabelEdit->getText().simplify();
  if (str.empty())
    AF_Report2_Settings_SetLabelX(settings, NULL);
  else
    AF_Report2_Settings_SetLabelX(settings, str.text());

  str=m_yLabelEdit->getText().simplify();
  if (str.empty())
    AF_Report2_Settings_SetLabelY(settings, NULL);
  else
    AF_Report2_Settings_SetLabelY(settings, str.text());

  return true;
}




long FF_View_Report2Settings::onCmdDateSpan(FXObject*, FXSelector, void *ptr) {
  int i;

  i=m_dateSpanTypeCombo->getCurrentItem();

  if (i==2) {
    m_firstDateEdit->enable();
    m_firstDateButton->enable();

    m_lastDateEdit->enable();
    m_lastDateButton->enable();
  }
  else {
    m_firstDateEdit->disable();
    m_firstDateButton->disable();

    m_lastDateEdit->disable();
    m_lastDateButton->disable();
  }
  return 1;
}



long FF_View_Report2Settings::onCmdFirstDate(FXObject*, FXSelector, void *ptr) {
  // TODO
  return 1;
}



long FF_View_Report2Settings::onCmdLastDate(FXObject*, FXSelector, void *ptr) {
  // TODO
  return 1;
}






