/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_greportwiz.hpp"
#include "ff_greportwiz_intro.hpp"
#include "ff_greportwiz_admin.hpp"
#include "ff_greportwiz_timespan.hpp"
#include "ff_greportwiz_accounts.hpp"
#include "ff_greportwiz_graph.hpp"
#include "ff_greportwiz_extro.hpp"


#include <gwen-gui-fox16/theme.h>




FXDEFMAP(FF_GraphReportWizPage) FF_GraphReportWizPageMap[]={
  //FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_WizardPage::onCmdAccept),
};


FXIMPLEMENT(FF_GraphReportWizPage, FF_WizardPage, FF_GraphReportWizPageMap, ARRAYNUMBER(FF_GraphReportWizPageMap))




FF_GraphReportWizPage::FF_GraphReportWizPage(FF_GraphReportWiz *p,
                                             const FXString &name,
                                             FXuint opts,
                                             FXint x, FXint y, FXint w, FXint h,
                                             FXint pl, FXint pr,
                                             FXint pt, FXint pb,
                                             FXint hs, FXint vs)
:FF_WizardPage(p, name, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_wizard(p)
{
}



FF_GraphReportWizPage::FF_GraphReportWizPage()
:FF_WizardPage()
,m_wizard(NULL)
{
}



FF_GraphReportWizPage::~FF_GraphReportWizPage() {
}








FF_GraphReportWiz::FF_GraphReportWiz(FXWindow *owner, const FXString &title,
                                     FXImage *image,
                                     FXuint opts,
                                     FXint x, FXint y, FXint w, FXint h,
                                     FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_Wizard(owner, title, "FF_GraphReportWiz", image, (opts & 0x0ffffff), x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_introPage(NULL)
,m_adminPage(NULL)
,m_timeSpanPage(NULL)
,m_accountsPage(NULL)
,m_graphPage(NULL)
,m_extroPage(NULL)
{
  m_introPage=new FF_GraphReportWiz_Intro(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  if (opts & WITH_ADMIN_PAGE)
    m_adminPage=new FF_GraphReportWiz_Admin(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_timeSpanPage=new FF_GraphReportWiz_TimeSpan(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_accountsPage=new FF_GraphReportWiz_Accounts(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_graphPage=new FF_GraphReportWiz_Graph(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_extroPage=new FF_GraphReportWiz_Extro(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
}



FF_GraphReportWiz::FF_GraphReportWiz()
:FF_Wizard()
,m_introPage(NULL)
,m_adminPage(NULL)
,m_timeSpanPage(NULL)
,m_accountsPage(NULL)
,m_extroPage(NULL)
{
}



FF_GraphReportWiz::~FF_GraphReportWiz() {
}



FXint FF_GraphReportWiz::getDefaultWidth() {
  FXint w;

  w=FF_Wizard::getDefaultWidth();
  if (w<500)
    w=500;
  return w;
}



FXint FF_GraphReportWiz::getDefaultHeight() {
  FXint h;

  h=FF_Wizard::getDefaultHeight();
  if (h<400)
    h=400;
  return h;
}



void FF_GraphReportWiz::fromGui(AF_REPORT2_SETTINGS *settings) {
  FXString str;
  GWEN_DATE *dt;
  AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
  AF_REPORT2_SETTINGS_GRAPH *sg;

  AF_Report2_Settings_SetDefaults(settings);
  AF_Report2_Settings_SetDateUnits(settings, AF_Report2_DateUnits_Months);

  if (m_adminPage) {
    str=m_adminPage->getName();
    if (str.empty())
      AF_Report2_Settings_SetName(settings, NULL);
    else
      AF_Report2_Settings_SetName(settings, str.text());

    str=m_adminPage->getDescription();
    if (str.empty())
      AF_Report2_Settings_SetDescription(settings, NULL);
    else
      AF_Report2_Settings_SetDescription(settings, str.text());

    AF_Report2_Settings_SetGroupId(settings, m_adminPage->getGroupId());
  }

  AF_Report2_Settings_SetDateSpanType(settings, m_timeSpanPage->getTimeSpanType());
  dt=m_timeSpanPage->getFirstDate();
  if (dt) {
    AF_Report2_Settings_SetFirstDate(settings, dt);
    GWEN_Date_free(dt);
  }
  else
    AF_Report2_Settings_SetFirstDate(settings, NULL);

  dt=m_timeSpanPage->getLastDate();
  if (dt) {
    AF_Report2_Settings_SetLastDate(settings, dt);
    GWEN_Date_free(dt);
  }
  else
    AF_Report2_Settings_SetLastDate(settings, NULL);

  AF_Report2_Settings_SetLabelX(settings, I18N("Date"));
  str=m_accountsPage->getCommodity().simplify();
  if (str.empty())
    AF_Report2_Settings_SetCommodity(settings, NULL);
  else {
    AF_Report2_Settings_SetCommodity(settings, str.text());
    AF_Report2_Settings_SetLabelY(settings, str.text());
  }

  AF_Report2_Settings_SetWidth(settings, m_graphPage->getWidth());
  AF_Report2_Settings_SetHeight(settings, m_graphPage->getHeight());

  str=m_graphPage->getTitle();
  if (str.empty())
    AF_Report2_Settings_SetTitle(settings, NULL);
  else
    AF_Report2_Settings_SetTitle(settings, str.text());

  str=m_graphPage->getSubTitle();
  if (str.empty())
    AF_Report2_Settings_SetSubTitle(settings, NULL);
  else
    AF_Report2_Settings_SetSubTitle(settings, str.text());


  /* create graphs */
  sgList=AF_Report2_SettingsGraph_List_new();
  AF_Report2_Settings_SetGraphList(settings, sgList);

  /* main data graph */
  sg=AF_Report2_SettingsGraph_new();
  AF_Report2_SettingsGraph_SetFlags(sg, AF_REPORT2_FLAGS_SUM_TOTAL | AF_REPORT2_FLAGS_ENABLED);
  AF_Report2_SettingsGraph_SetDataMode(sg, AF_Report2_DataMode_Direct);
  AF_Report2_SettingsGraph_SetLabel(sg, I18N("Amounts"));
  AF_Report2_SettingsGraph_SetGraphType(sg, AF_GRAPH_TYPE_BARS);
  AF_Report2_SettingsGraph_SetColour(sg, 0xff000000); /* red */
  AF_Report2_SettingsGraph_SetLineWidth(sg, 1);
  AF_Report2_SettingsGraph_SetDashType(sg, 0);
  str=m_accountsPage->getAccount1();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(sg, str.text());
  str=m_accountsPage->getAccount2();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(sg, str.text());
  AF_Report2_SettingsGraph_List_Add(sg, sgList);


  /* average graph */
  sg=AF_Report2_SettingsGraph_new();
  AF_Report2_SettingsGraph_SetFlags(sg, AF_REPORT2_FLAGS_SUM_TOTAL);
  if (m_graphPage->getWithAverage())
    AF_Report2_SettingsGraph_AddFlags(sg, AF_REPORT2_FLAGS_ENABLED);

  AF_Report2_SettingsGraph_SetDataMode(sg, AF_Report2_DataMode_Average);
  AF_Report2_SettingsGraph_SetLabel(sg, I18N("Average"));
  AF_Report2_SettingsGraph_SetGraphType(sg, AF_GRAPH_TYPE_LINE);
  AF_Report2_SettingsGraph_SetColour(sg, 0x00000000); /* black */
  AF_Report2_SettingsGraph_SetLineWidth(sg, 1);
  AF_Report2_SettingsGraph_SetDashType(sg, 0);
  str=m_accountsPage->getAccount1();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(sg, str.text());
  str=m_accountsPage->getAccount2();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(sg, str.text());
  AF_Report2_SettingsGraph_List_Add(sg, sgList);

  /* floating average graph */
  sg=AF_Report2_SettingsGraph_new();
  AF_Report2_SettingsGraph_SetFlags(sg, AF_REPORT2_FLAGS_SUM_TOTAL);
  if (m_graphPage->getWithFloatingAverage())
    AF_Report2_SettingsGraph_AddFlags(sg, AF_REPORT2_FLAGS_ENABLED);
  AF_Report2_SettingsGraph_SetDataMode(sg, AF_Report2_DataMode_FloatingAverage);
  AF_Report2_SettingsGraph_SetLabel(sg, I18N("Floating Average"));
  AF_Report2_SettingsGraph_SetGraphType(sg, AF_GRAPH_TYPE_LINE);
  AF_Report2_SettingsGraph_SetColour(sg, 0x0000ff00); /* blue */
  AF_Report2_SettingsGraph_SetLineWidth(sg, 1);
  AF_Report2_SettingsGraph_SetDashType(sg, 1);
  str=m_accountsPage->getAccount1();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(sg, str.text());
  str=m_accountsPage->getAccount2();
  if (!str.empty())
    AF_Report2_SettingsGraph_AddCategoryName(sg, str.text());
  AF_Report2_SettingsGraph_List_Add(sg, sgList);
}



bool FF_GraphReportWiz::createProfile(FXComposite *parent,
                                      const FXString &title,
                                      AF_REPORT2_SETTINGS *settings,
                                      FXuint opts) {
  FF_GraphReportWiz dlg(parent, title, NULL, opts);

  FXuint dlgres;

  dlg.create();
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  dlg.fromGui(settings);
  return true;
}






