/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_greportwiz_accounts.hpp"
#include "ff_accountedit.hpp"
#include "ff_commodityedit.hpp"

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_GraphReportWiz_Accounts) FF_GraphReportWiz_AccountsMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_Accounts::ID_EDIT_COMMODITY, FF_GraphReportWiz_Accounts::onCmdComboBox),
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_Accounts::ID_ACCEDIT_1, FF_GraphReportWiz_Accounts::onCmdEdit),
  FXMAPFUNC(SEL_CHANGED, FF_GraphReportWiz_Accounts::ID_ACCEDIT_1, FF_GraphReportWiz_Accounts::onCmdEdit),
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_Accounts::ID_ACCEDIT_2, FF_GraphReportWiz_Accounts::onCmdEdit),
  FXMAPFUNC(SEL_CHANGED, FF_GraphReportWiz_Accounts::ID_ACCEDIT_2, FF_GraphReportWiz_Accounts::onCmdEdit)
};


FXIMPLEMENT(FF_GraphReportWiz_Accounts, FF_GraphReportWizPage, FF_GraphReportWiz_AccountsMap, ARRAYNUMBER(FF_GraphReportWiz_AccountsMap))




FF_GraphReportWiz_Accounts::FF_GraphReportWiz_Accounts(FF_GraphReportWiz *p,
                                                       FXuint opts,
                                                       FXint x, FXint y, FXint w, FXint h,
                                                       FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_GraphReportWizPage(p, "timespan", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXMatrix *m;

  new FXLabel(this, I18N("Account Selection"), NULL, LABEL_NORMAL);
  new FXSeparator(this, SEPARATOR_GROOVE | LAYOUT_FILL_X);

  m_label=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_FILL_Y | FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  m_label->setMinimumWidth(150);
  new FXSpring(this, LAYOUT_FILL_Y);

  m=new FXMatrix(this, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Currency"), NULL, LABEL_NORMAL);
  m_commodityEdit=new FF_CommodityEdit(m, this, ID_EDIT_COMMODITY,
                                       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Account 1"), NULL, LABEL_NORMAL);
  m_accountEdit1=new FF_AccountEdit(m, this, ID_ACCEDIT_1, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  new FXLabel(m, I18N("Account 2"), NULL, LABEL_NORMAL);
  m_accountEdit2=new FF_AccountEdit(m, this, ID_ACCEDIT_2, LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
}



FF_GraphReportWiz_Accounts::FF_GraphReportWiz_Accounts()
:FF_GraphReportWizPage()
{
}



FF_GraphReportWiz_Accounts::~FF_GraphReportWiz_Accounts() {
}



void FF_GraphReportWiz_Accounts::init(GWEN_DB_NODE *db) {
  m_label->setText(I18N("Please specify the accounts from which data is to be collected for the report."));

  m_accountEdit1->reload();
  m_accountEdit2->reload();
  m_commodityEdit->reload();

  m_commodityEdit->setCurrency("EUR");
}



void FF_GraphReportWiz_Accounts::fini(GWEN_DB_NODE *db) {
}



void FF_GraphReportWiz_Accounts::enterPageForwards() {
  FXString str;

  if ((m_accountEdit1->getText().simplify().empty() && m_accountEdit2->getText().simplify().empty()) ||
      m_commodityEdit->getCurrency().simplify().empty())
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
}



void FF_GraphReportWiz_Accounts::enterPageBackwards() {
  enterPageForwards();
}



FXString FF_GraphReportWiz_Accounts::getAccount1() {
  return m_accountEdit1->getText().simplify();
}



FXString FF_GraphReportWiz_Accounts::getAccount2() {
  return m_accountEdit2->getText().simplify();
}


FXString FF_GraphReportWiz_Accounts::getCommodity() {
  return m_commodityEdit->getCurrency();
}



long FF_GraphReportWiz_Accounts::onCmdComboBox(FXObject*, FXSelector, void *ptr) {
  enterPageForwards();
  return 1;
}



long FF_GraphReportWiz_Accounts::onCmdEdit(FXObject*, FXSelector, void *ptr) {
  enterPageForwards();
  return 1;
}



