/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_greportwiz_admin.hpp"
#include "ff_groupview.hpp"

#include <gwen-gui-fox16/fox16_htmllabel.hpp>



FXDEFMAP(FF_GraphReportWiz_Admin) FF_GraphReportWiz_AdminMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_Admin::ID_EDIT_NAME, FF_GraphReportWiz_Admin::onCmdEdit),
  FXMAPFUNC(SEL_CHANGED, FF_GraphReportWiz_Admin::ID_EDIT_NAME, FF_GraphReportWiz_Admin::onCmdEdit)
};


FXIMPLEMENT(FF_GraphReportWiz_Admin, FF_GraphReportWizPage, FF_GraphReportWiz_AdminMap, ARRAYNUMBER(FF_GraphReportWiz_AdminMap))




FF_GraphReportWiz_Admin::FF_GraphReportWiz_Admin(FF_GraphReportWiz *p,
                                                 FXuint opts,
                                                 FXint x, FXint y, FXint w, FXint h,
                                                 FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_GraphReportWizPage(p, "admin", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXMatrix *m;

  new FXLabel(this, I18N("Administration"), NULL, LABEL_NORMAL);
  new FXSeparator(this, SEPARATOR_GROOVE | LAYOUT_FILL_X);

  m_label=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_FILL_Y | FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  m_label->setMinimumWidth(150);
  new FXSpring(this, LAYOUT_FILL_Y);

  m=new FXMatrix(this, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 1, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Description"), NULL, LABEL_NORMAL);
  m_descrEdit=new FXTextField(m, 1, this, ID_EDIT_DESCR,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  m_groupView=new FF_GroupView(AE_Book_TableType_GraphicReportGroup, this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_groupView->setTarget(this);
  m_groupView->setSelector(ID_VIEW_GROUP);

}



FF_GraphReportWiz_Admin::FF_GraphReportWiz_Admin()
:FF_GraphReportWizPage()
{
}



FF_GraphReportWiz_Admin::~FF_GraphReportWiz_Admin() {
}



void FF_GraphReportWiz_Admin::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  m_label->setText(I18N("<html>"
                        "<p>You must now specify a name for new graphic report profile."
                        "This name should be unique.</p>"
                        "<p>Report profiles can be organized into groups.</p>"
                        "</html>"));

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "groupView");
  m_groupView->init(dbT);
  m_groupView->reload();
}



void FF_GraphReportWiz_Admin::fini(GWEN_DB_NODE *db) {
}



void FF_GraphReportWiz_Admin::enterPageForwards() {
  FXString str;

  str=m_nameEdit->getText().simplify();
  if (str.empty())
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
}



void FF_GraphReportWiz_Admin::enterPageBackwards() {
  enterPageForwards();
}



FXString FF_GraphReportWiz_Admin::getName() {
  return m_nameEdit->getText().simplify();
}



FXString FF_GraphReportWiz_Admin::getDescription() {
  return m_descrEdit->getText().simplify();
}



AQDB_ID FF_GraphReportWiz_Admin::getGroupId() {
  AE_GROUP *gr;

  gr=m_groupView->getFirstSelectedGroup();
  if (gr)
    return AE_Group_GetId(gr);
  return 0;
}



long FF_GraphReportWiz_Admin::onCmdEdit(FXObject*, FXSelector, void *ptr) {
  enterPageForwards();
  return 1;
}




