/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_greportwiz_graph.hpp"

#include <gwen-gui-fox16/fox16_htmllabel.hpp>



FXDEFMAP(FF_GraphReportWiz_Graph) FF_GraphReportWiz_GraphMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_Graph::ID_EDIT_WIDTH, FF_GraphReportWiz_Graph::onCmdEdit),
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_Graph::ID_EDIT_HEIGHT, FF_GraphReportWiz_Graph::onCmdEdit)
};


FXIMPLEMENT(FF_GraphReportWiz_Graph, FF_GraphReportWizPage, FF_GraphReportWiz_GraphMap, ARRAYNUMBER(FF_GraphReportWiz_GraphMap))




FF_GraphReportWiz_Graph::FF_GraphReportWiz_Graph(FF_GraphReportWiz *p,
                                                 FXuint opts,
                                                 FXint x, FXint y, FXint w, FXint h,
                                                 FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_GraphReportWizPage(p, "graph", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXMatrix *m;

  new FXLabel(this, I18N("Graph Settings"), NULL, LABEL_NORMAL);
  new FXSeparator(this, SEPARATOR_GROOVE | LAYOUT_FILL_X);

  m_label=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label->setMinimumWidth(150);
  new FXSpring(this, LAYOUT_FILL_Y);

  m=new FXMatrix(this, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Title"), NULL, LABEL_NORMAL);
  m_titleEdit=new FXTextField(m, 20, this, ID_EDIT_TITLE,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Subtitle"), NULL, LABEL_NORMAL);
  m_subTitleEdit=new FXTextField(m, 20, this, ID_EDIT_SUBTITLE,
                                 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Width"), NULL, LABEL_NORMAL);
  m_widthEdit=new FXTextField(m, 1, this, ID_EDIT_WIDTH,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Height"), NULL, LABEL_NORMAL);
  m_heightEdit=new FXTextField(m, 1, this, ID_EDIT_HEIGHT,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  m_averageButton=new FXCheckButton(this, I18N("With average graph"), this, ID_BUTTON_AVERAGE,
                                    LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
  m_floatingAverageButton=new FXCheckButton(this, I18N("With floating average graph"), this, ID_BUTTON_FLOATING_AVERAGE,
                                            LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
}



FF_GraphReportWiz_Graph::FF_GraphReportWiz_Graph()
:FF_GraphReportWizPage()
{
}



FF_GraphReportWiz_Graph::~FF_GraphReportWiz_Graph() {
}



void FF_GraphReportWiz_Graph::init(GWEN_DB_NODE *db) {
  m_label->setText(I18N("<html>"
                        "<p>Please specify title, sub-title and dimensions of the graphics report.</p>"
                        "<p>Graphics reports may contain an additional graph showing the data <b>average</b> over "
                        "the input data.</p>"
                        "<p>Also, there is the possibilty to include a graph showing a <b>floating average</b> "
                        "(i.e. the average over the last 20 values).</p>"
                        "</html>"));
  m_widthEdit->setText("900");
  m_heightEdit->setText("700");
}



void FF_GraphReportWiz_Graph::fini(GWEN_DB_NODE *db) {
}



void FF_GraphReportWiz_Graph::enterPageForwards() {

  if ((FXIntVal(m_widthEdit->getText().simplify())<10) ||
      (FXIntVal(m_heightEdit->getText().simplify())<10))
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
}



void FF_GraphReportWiz_Graph::enterPageBackwards() {
  enterPageForwards();
}



int FF_GraphReportWiz_Graph::getWidth() {
  return FXIntVal(m_widthEdit->getText().simplify());
}



int FF_GraphReportWiz_Graph::getHeight() {
  return FXIntVal(m_heightEdit->getText().simplify());
}



FXString FF_GraphReportWiz_Graph::getTitle() {
  return m_titleEdit->getText().simplify();
}



FXString FF_GraphReportWiz_Graph::getSubTitle() {
  return m_subTitleEdit->getText().simplify();
}



bool FF_GraphReportWiz_Graph::getWithAverage() {
  return (m_averageButton->getCheck()==TRUE)?true:false;
}



bool FF_GraphReportWiz_Graph::getWithFloatingAverage() {
  return (m_floatingAverageButton->getCheck()==TRUE)?true:false;
}



long FF_GraphReportWiz_Graph::onCmdEdit(FXObject*, FXSelector, void *ptr) {
  enterPageForwards();
  return 1;
}






