/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_greportwiz_timespan.hpp"
#include "ff_dateedit.hpp"
#include "ff_datepicker.hpp"

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_GraphReportWiz_TimeSpan) FF_GraphReportWiz_TimeSpanMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_GraphReportWiz_TimeSpan::ID_COMBO_DATESPAN, FF_GraphReportWiz_TimeSpan::onCmdComboBox)
};


FXIMPLEMENT(FF_GraphReportWiz_TimeSpan, FF_GraphReportWizPage, FF_GraphReportWiz_TimeSpanMap, ARRAYNUMBER(FF_GraphReportWiz_TimeSpanMap))




FF_GraphReportWiz_TimeSpan::FF_GraphReportWiz_TimeSpan(FF_GraphReportWiz *p,
                                                       FXuint opts,
                                                       FXint x, FXint y, FXint w, FXint h,
                                                       FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_GraphReportWizPage(p, "timespan", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXMatrix *m;
  FXHorizontalFrame *hf;

  new FXLabel(this, I18N("Date Span"), NULL, LABEL_NORMAL);
  new FXSeparator(this, SEPARATOR_GROOVE | LAYOUT_FILL_X);

  m_label=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_FILL_Y | FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  m_label->setMinimumWidth(150);
  new FXSpring(this, LAYOUT_FILL_Y);

  m=new FXMatrix(this, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Datespan"), NULL, LABEL_NORMAL);
  m_dateSpanTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_DATESPAN,
                                     LAYOUT_FILL_X | COMBOBOX_STATIC |
                                     FRAME_SUNKEN | FRAME_THICK);
  m_dateSpanTypeCombo->appendItem(I18N("-- select type --"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|none"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|userdefined"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Month"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Quarter Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Half Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|This Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Month"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Quarter Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Half Year"));
  m_dateSpanTypeCombo->appendItem(I18N("AF_Report2_DateSpanType|Last Year"));
  m_dateSpanTypeCombo->setNumVisible(m_dateSpanTypeCombo->getNumItems());

  new FXLabel(m, I18N("First Date"), NULL, LABEL_NORMAL);
  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_firstDateEdit=new FF_DateEdit(hf, this, ID_DATEEDIT_FIRST,
                                  I18N("YYYY/MM/DD"),
                                  TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_firstDateButton=new THEMEBUTTON(hf, I18N("Pick"), NULL, this, ID_BUTTON_FIRSTDATE, BUTTON_NORMAL);

  new FXLabel(m, I18N("Last Date"), NULL, LABEL_NORMAL);
  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_lastDateEdit=new FF_DateEdit(hf, this, ID_DATEEDIT_LAST,
                                 I18N("YYYY/MM/DD"),
                                 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_lastDateButton=new THEMEBUTTON(hf, I18N("Pick"), NULL, this, ID_BUTTON_LASTDATE, BUTTON_NORMAL);
}



FF_GraphReportWiz_TimeSpan::FF_GraphReportWiz_TimeSpan()
:FF_GraphReportWizPage()
{
}



FF_GraphReportWiz_TimeSpan::~FF_GraphReportWiz_TimeSpan() {
}



void FF_GraphReportWiz_TimeSpan::init(GWEN_DB_NODE *db) {
  m_label->setText(I18N("<html>"
                        "Please specify the time span for which the report is to be created."
                        "</html>"));
  m_dateSpanTypeCombo->setCurrentItem(1);
}



void FF_GraphReportWiz_TimeSpan::fini(GWEN_DB_NODE *db) {
}



void FF_GraphReportWiz_TimeSpan::enterPageForwards() {
  if (m_dateSpanTypeCombo->getCurrentItem()==0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
}



void FF_GraphReportWiz_TimeSpan::enterPageBackwards() {
  enterPageForwards();
}



int FF_GraphReportWiz_TimeSpan::getTimeSpanType() {
  int i;

  switch(m_dateSpanTypeCombo->getCurrentItem()) {
    default:
    case 0:
    case 1:  i=AF_Report2_DateSpanType_None;            break;
    case 2:  i=AF_Report2_DateSpanType_UserDefined;     break;
    case 3:  i=AF_Report2_DateSpanType_ThisMonth;       break;
    case 4:  i=AF_Report2_DateSpanType_ThisQuarterYear; break;
    case 5:  i=AF_Report2_DateSpanType_ThisHalfYear;    break;
    case 6:  i=AF_Report2_DateSpanType_ThisYear;        break;
    case 7:  i=AF_Report2_DateSpanType_LastMonth;       break;
    case 8:  i=AF_Report2_DateSpanType_LastQuarterYear; break;
    case 9:  i=AF_Report2_DateSpanType_LastHalfYear;    break;
    case 10: i=AF_Report2_DateSpanType_LastYear;        break;
  }

  return i;
}



GWEN_DATE *FF_GraphReportWiz_TimeSpan::getFirstDate() {
  GWEN_DATE *dt;

  if (m_firstDateEdit->getDate(&dt))
    return dt;
  return NULL;
}



GWEN_DATE *FF_GraphReportWiz_TimeSpan::getLastDate() {
  GWEN_DATE *dt;

  if (m_lastDateEdit->getDate(&dt))
    return dt;
  return NULL;
}






long FF_GraphReportWiz_TimeSpan::onCmdComboBox(FXObject*, FXSelector, void *ptr) {
  int i;

  enterPageForwards();

  i=m_dateSpanTypeCombo->getCurrentItem();

  if (i==2) {
    m_firstDateEdit->enable();
    m_firstDateButton->enable();

    m_lastDateEdit->enable();
    m_lastDateButton->enable();
  }
  else {
    m_firstDateEdit->disable();
    m_firstDateButton->disable();

    m_lastDateEdit->disable();
    m_lastDateButton->disable();
  }
  return 1;
}



long FF_GraphReportWiz_TimeSpan::onCmdEdit(FXObject*, FXSelector, void *ptr) {
  enterPageForwards();
  return 1;
}

