/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_SETUPWIZ_HPP
#define FF_SETUPWIZ_HPP


#include "ff_wizard.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_book.h>
#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_bankaccount.h>
#include <aqfinance/engine/book/ae_commodity.h>


class FF_SetupWiz;


class FF_SetupWizPage: public FF_WizardPage {
  FXDECLARE(FF_SetupWizPage)
public:
  FF_SetupWizPage(FF_SetupWiz *p,
		  const FXString &name,
		  FXuint opts=0,
		  FXint x=0, FXint y=0, FXint w=0, FXint h=0,
		  FXint pl=DEFAULT_SPACING, FXint pr=DEFAULT_SPACING,
		  FXint pt=DEFAULT_SPACING, FXint pb=DEFAULT_SPACING,
		  FXint hs=DEFAULT_SPACING, FXint vs=DEFAULT_SPACING);
  FF_SetupWizPage();

  virtual ~FF_SetupWizPage();

protected:
  FF_SetupWiz *m_wizard;
};



class FF_SetupWiz: public FF_Wizard {
public:
  FF_SetupWiz(FXWindow *owner, const FXString &title,
	      FXImage *image,
	      FXuint opts=DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,
	      FXint x=0, FXint y=0, FXint w=0, FXint h=0,
	      FXint pl=10, FXint pr=10, FXint pt=10, FXint pb=10, FXint hs=10, FXint vs=10);

  ~FF_SetupWiz();


  const FXString &getDbPath() const { return m_dbPath;};
  void setDbPath(const FXString &s) { m_dbPath=s;};

  const FXString &getCurrencyCode() const { return m_currencyCode;};
  void setCurrencyCode(const FXString &str) { m_currencyCode=str;};

  AE_COMMODITY_LIST *getCommodityList() const { return m_commodities;};

  AE_ACCOUNT_TREE *getAccountTree() const { return m_accountTree;};

  AE_BANKACCOUNT_LIST *getBankAccountList() const { return m_bankAccountList;};

  FXint getDefaultWidth();
  FXint getDefaultHeight();


protected:
  FF_SetupWiz();

  FXString m_dbPath;
  AE_COMMODITY_LIST *m_commodities;
  AE_ACCOUNT_TREE *m_accountTree;
  AE_BANKACCOUNT_LIST *m_bankAccountList;

  FXString m_currencyCode;
};



#endif

