/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_setupwiz_accounts.hpp"
#include "ff_accounttree.hpp"

#include <aqfoxext/fxx_checktree.hpp>

#include <gwen-gui-fox16/fox16_htmllabel.hpp>
#include <gwenhywfar/pathmanager.h>
#include <gwenhywfar/directory.h>




FXDEFMAP(FF_SetupWiz_Accounts) FF_SetupWiz_AccountsMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, FF_SetupWiz_Accounts::ID_FILE_TREE, FF_SetupWiz_Accounts::onFileLeftButtonPress),
};


FXIMPLEMENT(FF_SetupWiz_Accounts, FF_SetupWizPage, FF_SetupWiz_AccountsMap, ARRAYNUMBER(FF_SetupWiz_AccountsMap))




FF_SetupWiz_Accounts::FF_SetupWiz_Accounts(FF_SetupWiz *p,
					   FXuint opts,
					   FXint x, FXint y, FXint w, FXint h,
					   FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_SetupWizPage(p, "accounts", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_xmlNodes(NULL)
{
  FXSplitter *hf;
  FXGroupBox *gb;

  m_label=new FOX16_HtmlLabel(this,
			      I18N("Select the accounts to be used."),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  hf=new FXSplitter(this, SPLITTER_HORIZONTAL | LAYOUT_FILL_X | LAYOUT_FILL_Y);
  gb=new FXGroupBox(hf, I18N("Account Selections"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_checkTree=new FXXCheckTree(gb, this, ID_FILE_TREE,
			       FXXCheckTree::OPTIONS_CHECKMARKS |
			       FOLDINGLIST_NORMAL | FOLDINGLIST_BROWSESELECT |
			       LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_checkTree->appendHeader(I18N("Group"));
  m_checkTree->setHeaderSize(0, 200);

  gb=new FXGroupBox(hf, I18N("Accounts"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_accountTree=new FF_AccountTree(gb, this, ID_ACCOUNT_TREE,
				   FOLDINGLIST_NORMAL | FOLDINGLIST_BROWSESELECT |
				   FOLDINGLIST_SHOWS_LINES |
				   FOLDINGLIST_SHOWS_BOXES |
				   FOLDINGLIST_ROOT_BOXES |
                                   FRAME_SUNKEN|FRAME_THICK |
				   LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_xmlNodes=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "root");

}



FF_SetupWiz_Accounts::FF_SetupWiz_Accounts()
:FF_SetupWizPage()
,m_xmlNodes(NULL)
{
}



FF_SetupWiz_Accounts::~FF_SetupWiz_Accounts() {
  GWEN_XMLNode_free(m_xmlNodes);
}



void FF_SetupWiz_Accounts::enterPageForwards() {
  layout();
  updateAccountTree();
}



void FF_SetupWiz_Accounts::enterPageBackwards() {
  m_wizard->enableNext();
}



void FF_SetupWiz_Accounts::leavePageForwards() {

}



void FF_SetupWiz_Accounts::init(GWEN_DB_NODE *db) {
  FXString str;
  int i;
  int sortByColumn;
  int sortUp;

  /* setup GUI */
  sortByColumn=GWEN_DB_GetIntValue(db, "fileSortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "fileSortUp", 0, 0);

  for (i=0; i<m_checkTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "fileColumns", i, m_checkTree->getHeaderSize(i));
    m_checkTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_checkTree->setHeaderArrowDir(i, TRUE);
      else
	m_checkTree->setHeaderArrowDir(i, FALSE);
    }
  }

  sortByColumn=GWEN_DB_GetIntValue(db, "accountSortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "accountSortUp", 0, 0);

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "accountColumns", i, m_accountTree->getHeaderSize(i));
    m_accountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_accountTree->setHeaderArrowDir(i, TRUE);
      else
	m_accountTree->setHeaderArrowDir(i, FALSE);
    }
  }

  m_checkTree->clearItems();
  str=getAccountFolder();
  if (!str.empty()) {
    GWEN_STRINGLIST *fileList;
    GWEN_STRINGLISTENTRY *se;
    int rv;

    /* read file list */
    fileList=GWEN_StringList_new();
    rv=GWEN_Directory_GetFileEntries(str.text(), fileList, "*.xml");
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "No accounts found");
      GWEN_StringList_free(fileList);
      return;
    }

    GWEN_StringList_Sort(fileList, 0, GWEN_StringList_SortModeNoCase);

    se=GWEN_StringList_FirstEntry(fileList);
    while(se) {
      GWEN_XMLNODE *ndoc;
      FXString fname;

      fname=str+GWEN_DIR_SEPARATOR_S+GWEN_StringListEntry_Data(se);
      ndoc=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "root");
      rv=GWEN_XML_ReadFile(ndoc, fname.text(),
                           GWEN_XML_FLAGS_DEFAULT |
			   GWEN_XML_FLAGS_HANDLE_HEADERS);
      if (rv>=0) {
	GWEN_XMLNODE *n;

	n=GWEN_XMLNode_FindFirstTag(ndoc, "accounts", NULL, NULL);
	if (n) {
	  const char *s;

	  s=GWEN_XMLNode_GetProperty(n, "name", NULL);
	  if (s) {
	    FXXCheckTreeItem *item;

	    /* unlink <accounts> node */
	    GWEN_XMLNode_UnlinkChild(ndoc, n);
            GWEN_XMLNode_AddChild(m_xmlNodes, n);
	    item=new FXXCheckTreeItem(s, NULL, NULL, (void*) n);
	    m_checkTree->appendItem(NULL, item);
	  }
	  else {
	    DBG_ERROR(AE_LOGDOMAIN, "Missing \"name\" property in file [%s]", fname.text());
	  }
	}
	else {
	  DBG_ERROR(AE_LOGDOMAIN, "No accounts in file [%s]", fname.text());
	}
      }
      GWEN_XMLNode_free(ndoc);

      se=GWEN_StringListEntry_Next(se);
    }

  }
  else {
    DBG_ERROR(0, "Folder not found");
  }

  layout();
  update();
}



void FF_SetupWiz_Accounts::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_checkTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "fileColumns",
			m_checkTree->getHeaderSize(i));
    b=m_checkTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "fileSortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "accountColumns",
			m_accountTree->getHeaderSize(i));
    b=m_accountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "accountSortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



FXString FF_SetupWiz_Accounts::getAccountFolder() {
  GWEN_STRINGLIST *pathList;
  FXString res;

  pathList=GWEN_PathManager_GetPaths(AE_PM_LIBNAME, AE_PM_DATADIR);
  if (pathList) {
    GWEN_STRINGLISTENTRY *sePath;
    GWEN_STRINGLIST *localeList;
    GWEN_STRINGLIST *newLocaleList=NULL;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    localeList=GWEN_I18N_GetCurrentLocaleList();
    if (localeList==NULL || GWEN_StringList_Count(localeList)==0) {
      newLocaleList=GWEN_StringList_new();
      GWEN_StringList_AppendString(localeList, "c", 0, 0);
      localeList=newLocaleList;
    }

    sePath=GWEN_StringList_FirstEntry(pathList);
    while(sePath) {
      GWEN_STRINGLISTENTRY *seLocale;

      seLocale=GWEN_StringList_FirstEntry(localeList);
      while(seLocale) {
	FILE *f;
        uint32_t pos;

	GWEN_Buffer_AppendString(tbuf, GWEN_StringListEntry_Data(sePath));
	GWEN_Buffer_AppendString(tbuf,GWEN_DIR_SEPARATOR_S);
	GWEN_Buffer_AppendString(tbuf, "accounts");
	GWEN_Buffer_AppendString(tbuf,GWEN_DIR_SEPARATOR_S);
	GWEN_Buffer_AppendString(tbuf, GWEN_StringListEntry_Data(seLocale));
        pos=GWEN_Buffer_GetPos(tbuf);
	GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);
	GWEN_Buffer_AppendString(tbuf, "00-basic_accounts.xml");
        DBG_ERROR(0, "Trying [%s]", GWEN_Buffer_GetStart(tbuf));
	f=fopen(GWEN_Buffer_GetStart(tbuf), "r");
	if (f) {
	  DBG_ERROR(0, "Found [%s]", GWEN_Buffer_GetStart(tbuf));
	  fclose(f);
	  GWEN_Buffer_Crop(tbuf, 0, pos);
          break;
	}
	GWEN_Buffer_Reset(tbuf);

	seLocale=GWEN_StringListEntry_Next(seLocale);
      }

      if (GWEN_Buffer_GetUsedBytes(tbuf))
	/* there is something in the buffer, so we found the folder */
	break;

      sePath=GWEN_StringListEntry_Next(sePath);
    }

    GWEN_StringList_free(newLocaleList);
    GWEN_StringList_free(pathList);

    if (GWEN_Buffer_GetUsedBytes(tbuf)) {
      res=FXString(GWEN_Buffer_GetStart(tbuf));
    }
    GWEN_Buffer_free(tbuf);
  }

  return res;
}



long FF_SetupWiz_Accounts::onFileLeftButtonPress(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;
  FXFoldingItem *item;
  FXint code;

  item=m_checkTree->getItemAt(event->win_x, event->win_y);
  if (item) {
    code=m_checkTree->hitItem(item,event->win_x,event->win_y);
    if (code==4) {
      FXXCheckTreeItem *aitem=dynamic_cast<FXXCheckTreeItem*>(item);

      if (aitem) {
	/* checkbox clicked */
	aitem->setChecked(!(aitem->isChecked()));
	m_checkTree->updateItem(aitem);
        updateAccountTree();
        return 1;
      }
    }
  }

  return 0;
}



void FF_SetupWiz_Accounts::updateAccountTree() {
  FXFoldingItem *item;
  AE_ACCOUNT_LIST *al;
  AE_ACCOUNT_TREE *at;

  m_accountTree->clearItems();

  al=AE_Account_List_new();
  item=m_checkTree->getFirstItem();
  while(item) {
    FXXCheckTreeItem *aitem=dynamic_cast<FXXCheckTreeItem*>(item);

    if (aitem && aitem->isChecked()) {
      GWEN_XMLNODE *n;

      n=(GWEN_XMLNODE*)(aitem->getData());
      if (n) {
	GWEN_XMLNODE *nn;

	nn=GWEN_XMLNode_FindFirstTag(n, "account", NULL, NULL);
	while(nn) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_fromXml(nn);
	  if (acc) {
	    if (AE_Account_List_GetById(al, AE_Account_GetId(acc)))
	      AE_Account_free(acc);
	    else
	      AE_Account_List_Add(acc, al);
	  }

	  nn=GWEN_XMLNode_FindNextTag(nn, "account", NULL, NULL);
	}
      }
    }

    item=item->getBelow();
  }

  at=m_wizard->getAccountTree();
  AE_Account_Tree_Clear(at);

  if (AE_Account_List_GetCount(al)) {
    int rv;

    rv=AE_Book_SortAccountsIntoTree(at, NULL, al);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      m_accountTree->addAccounts(at);
      m_accountTree->expandItems();
    }
  }

  if (AE_Account_Tree_GetCount(at))
    m_wizard->enableNext();
  else
    m_wizard->disableNext();

  AE_Account_List_free(al);
}



