/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_setupwiz_bank.hpp"
#include "ff_bankaccounttree.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/fox16_htmllabel.hpp>


FXDEFMAP(FF_SetupWiz_Bank) FF_SetupWiz_BankMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, FF_SetupWiz_Bank::ID_BANKACC_TREE, FF_SetupWiz_Bank::onLeftButtonPress),
};


FXIMPLEMENT(FF_SetupWiz_Bank, FF_SetupWizPage, FF_SetupWiz_BankMap, ARRAYNUMBER(FF_SetupWiz_BankMap))




FF_SetupWiz_Bank::FF_SetupWiz_Bank(FF_SetupWiz *p,
				   FXuint opts,
				   FXint x, FXint y, FXint w, FXint h,
				   FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_SetupWizPage(p, "intro", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXGroupBox *gb;

  m_label=new FOX16_HtmlLabel(this,
			      I18N("<p>This is the list of bank accounts known to <b>AqBanking</b>, the "
				   "online banking library.</p>"
				   "<p>This list is for informational purposes ony. "
				   "You can edit this list later.</p>"),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  gb=new FXGroupBox(this, I18N("Bank Accounts"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_bankAccountTree=new FF_BankAccountTree(gb, this, ID_BANKACC_TREE,
					   LAYOUT_FILL_X | LAYOUT_FILL_Y |
					   FXXCheckTree::OPTIONS_CHECKMARKS);
}



FF_SetupWiz_Bank::FF_SetupWiz_Bank()
:FF_SetupWizPage()
{
}



FF_SetupWiz_Bank::~FF_SetupWiz_Bank() {
}



void FF_SetupWiz_Bank::enterPageForwards() {
  layout();
  m_wizard->enableNext();
}



void FF_SetupWiz_Bank::enterPageBackwards() {
  m_wizard->enableNext();
}



void FF_SetupWiz_Bank::leavePageForwards() {
  AE_BANKACCOUNT_LIST *bal;
  FXFoldingItem *item;

  bal=m_wizard->getBankAccountList();
  AE_BankAccount_List_Clear(bal);

  item=m_bankAccountTree->getFirstItem();
  while (item) {
    FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(item);
    if (baItem) {
      AE_BANKACCOUNT *ba;

      ba=baItem->getBankAccount();
      /* only add checked items */
      if (ba && baItem->isChecked())
        AE_BankAccount_List_Add(AE_BankAccount_dup(ba), bal);
    }

    item=item->getBelow();
  }
}



void FF_SetupWiz_Bank::init(GWEN_DB_NODE *db) {
  FF_App *app;
  AB_BANKING *ab;
  AE_BANKACCOUNT_LIST *bal;
  AB_ACCOUNT_LIST2 *al2;
  int i;
  int sortByColumn;
  int sortUp;

  /* setup GUI */
  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_bankAccountTree->getHeaderSize(i));
    m_bankAccountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_bankAccountTree->setHeaderArrowDir(i, TRUE);
      else
	m_bankAccountTree->setHeaderArrowDir(i, FALSE);
    }
  }

  /* read banking accounts */
  app=dynamic_cast<FF_App*>(getApp());
  ab=app->getBanking();
  assert(ab);

  m_bankAccountTree->clearItems();

  /* get bank account list from AqBanking */
  bal=m_wizard->getBankAccountList();
  AE_BankAccount_List_Clear(bal);

  al2=AB_Banking_GetAccounts(ab);
  if (al2) {
    AB_ACCOUNT_LIST2_ITERATOR *it;

    it=AB_Account_List2_First(al2);
    if (it) {
      const AB_ACCOUNT *a;

      a=AB_Account_List2Iterator_Data(it);
      while(a) {
	AE_BANKACCOUNT *ba;
	const char *s;

	ba=AE_BankAccount_new();

	s=AB_Account_GetAccountName(a);
	AE_BankAccount_SetAccountName(ba, s);

	s=AB_Account_GetAccountNumber(a);
	AE_BankAccount_SetAccountNumber(ba, s);

	s=AB_Account_GetBankName(a);
	AE_BankAccount_SetBankName(ba, s);

	s=AB_Account_GetBankCode(a);
	AE_BankAccount_SetBankCode(ba, s);

	s=AB_Account_GetBIC(a);
	AE_BankAccount_SetBic(ba, s);

	s=AB_Account_GetIBAN(a);
	AE_BankAccount_SetIban(ba, s);

	AE_BankAccount_SetBankingId(ba, AB_Account_GetUniqueId(a));

        AE_BankAccount_List_Add(ba, bal);

	a=AB_Account_List2Iterator_Next(it);
      }
      AB_Account_List2Iterator_free(it);
    }

    AB_Account_List2_free(al2);
  }
  else {
    DBG_ERROR(0, "No bank accounts");
  }

  if (AE_BankAccount_List_GetCount(bal)) {
    AE_BANKACCOUNT *ba;

    ba=AE_BankAccount_List_First(bal);
    while(ba) {
      m_bankAccountTree->addAccount(AE_BankAccount_dup(ba), NULL, NULL);
      ba=AE_BankAccount_List_Next(ba);
    }
  }
  m_bankAccountTree->sort();
}



void FF_SetupWiz_Bank::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_bankAccountTree->getHeaderSize(i));
    b=m_bankAccountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



long FF_SetupWiz_Bank::onLeftButtonPress(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;
  FXFoldingItem *item;
  FXint code;

  item=m_bankAccountTree->getItemAt(event->win_x,event->win_y);
  if (item) {
    code=m_bankAccountTree->hitItem(item,event->win_x,event->win_y);
    if (code==4) {
      FF_BankAccountTreeItem *aitem=dynamic_cast<FF_BankAccountTreeItem*>(item);

      if (aitem) {
	/* checkbox clicked */
	aitem->setChecked(!(aitem->isChecked()));
	m_bankAccountTree->updateItem(aitem);
        return 1;
      }
    }
  }

  return 0;
}




