/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_setupwiz_currency.hpp"
#include "ff_app.hpp"

#include <aqbanking/banking_info.h>
#include <aqbanking/country.h>

#include <gwen-gui-fox16/fox16_htmllabel.hpp>


FXDEFMAP(FF_SetupWiz_Currency) FF_SetupWiz_CurrencyMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_SetupWiz_Currency::ID_CURRENCY_COMBO, FF_SetupWiz_Currency::onCmdComboBox),
};


FXIMPLEMENT(FF_SetupWiz_Currency, FF_SetupWizPage, FF_SetupWiz_CurrencyMap, ARRAYNUMBER(FF_SetupWiz_CurrencyMap))




FF_SetupWiz_Currency::FF_SetupWiz_Currency(FF_SetupWiz *p,
					   FXuint opts,
					   FXint x, FXint y, FXint w, FXint h,
					   FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_SetupWizPage(p, "currency", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXMatrix *m;

  m_label=new FOX16_HtmlLabel(this,
			      I18N("Please select the default currency for the new database."),
			      LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label->setMinimumWidth(150);
  new FXSpring(this, LAYOUT_FILL_Y);

  m=new FXMatrix(this, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Currency"), NULL, LABEL_NORMAL);
  m_currencyCombo=new FXComboBox(m, 1, this, ID_CURRENCY_COMBO, COMBOBOX_STATIC);
}



FF_SetupWiz_Currency::FF_SetupWiz_Currency()
:FF_SetupWizPage()
{
}



FF_SetupWiz_Currency::~FF_SetupWiz_Currency() {
}



void FF_SetupWiz_Currency::init(GWEN_DB_NODE *db) {
  FF_App *app;
  AB_BANKING *ab;
  AB_COUNTRY_CONSTLIST2 *cnl;
  AE_COMMODITY_LIST *col;

  app=dynamic_cast<FF_App*>(getApp());
  ab=app->getBanking();
  assert(ab);

  m_currencyCombo->clearItems();
  m_currencyCombo->appendItem(I18N("(select)"));

  /* get currency list from AqBanking */
  col=m_wizard->getCommodityList();
  AE_Commodity_List_Clear(col);

  /* add most important currencies */
  if (1) {
    AE_COMMODITY *co;
    AQDB_VALUE *v;
  
    co=AE_Commodity_new();
    AE_Commodity_SetCode(co, "EUR");
    AE_Commodity_SetName(co, I18N("Euro"));

    v=AQDB_Value_fromString("0.01");
    AE_Commodity_SetSmallestUnit(co, v);
    AQDB_Value_free(v);
    AE_Commodity_SetDescription(co, I18N("Currency of the European Union"));
    AE_Commodity_SetNameSpace(co, AE_CURRENCY_NAMESPACE_ISO_4217_CODE);
    AE_Commodity_SetSymbol(co, "\xe2\x82\xac");
    AE_Commodity_List_Add(co, col);
  }

  if (1) {
    AE_COMMODITY *co;
    AQDB_VALUE *v;
  
    co=AE_Commodity_new();
    AE_Commodity_SetCode(co, "USD");
    AE_Commodity_SetName(co, I18N("US Dollar"));

    v=AQDB_Value_fromString("0.01");
    AE_Commodity_SetSmallestUnit(co, v);
    AQDB_Value_free(v);
    AE_Commodity_SetDescription(co, I18N("Currency of the United States of America"));
    AE_Commodity_SetNameSpace(co, AE_CURRENCY_NAMESPACE_ISO_4217_CODE);
    AE_Commodity_SetSymbol(co, "$");
    AE_Commodity_List_Add(co, col);
  }

  cnl=AB_Banking_ListCountriesByName(ab, "*");
  if (cnl) {
    AB_COUNTRY_CONSTLIST2_ITERATOR *it;

    it=AB_Country_ConstList2_First(cnl);
    if (it) {
      const AB_COUNTRY *c;

      c=AB_Country_ConstList2Iterator_Data(it);
      while(c) {
	const char *s;

	s=AB_Country_GetCurrencyCode(c);
	if (s && *s) {
	  AE_COMMODITY *co;

	  co=AE_Commodity_List_GetByCode(col, s);
	  if (co==NULL) {
	    AQDB_VALUE *v;

	    co=AE_Commodity_new();
	    AE_Commodity_SetCode(co, s);
	    s=AB_Country_GetLocalCurrencyName(c);
	    AE_Commodity_SetName(co, s);

	    v=AQDB_Value_fromString("0.01");
	    AE_Commodity_SetSmallestUnit(co, v);
	    AQDB_Value_free(v);
	    s=AB_Country_GetLocalName(c);
	    if (s && *s)
	      AE_Commodity_SetDescription(co, s);

            AE_Commodity_SetNameSpace(co, AE_CURRENCY_NAMESPACE_ISO_4217_CODE);
	    AE_Commodity_List_Add(co, col);
	  }
	}
	c=AB_Country_ConstList2Iterator_Next(it);
      }
      AB_Country_ConstList2Iterator_free(it);
    }
    AB_Country_ConstList2_free(cnl);
  }

  if (AE_Commodity_List_GetCount(col)) {
    AE_COMMODITY *co;
    int i;

    co=AE_Commodity_List_First(col);
    while(co) {
      const char *s;

      s=AE_Commodity_GetCode(co);
      if (s && *s)
	m_currencyCombo->appendItem(s);
      co=AE_Commodity_List_Next(co);
    }

    m_currencyCombo->setSortFunc(FXList::ascending);
    m_currencyCombo->sortItems();
    i=AE_Commodity_List_GetCount(col);
    if (i>10)
      i=10;
    m_currencyCombo->setNumVisible(i);
  }
}



FXint FF_SetupWiz_Currency::checkGui(FXbool quiet) {
  if (m_currencyCombo->getCurrentItem()==0) {
    DBG_ERROR(0, "No currency selected");
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}



FXint FF_SetupWiz_Currency::toGui(const FXString &currString) {
  int i, j;

  j=m_currencyCombo->getNumItems();
  for (i=0; i<j; i++) {
    FXString str=m_currencyCombo->getItem(i);
    if (strcasecmp(str.text(), currString.text())==0) {
      m_currencyCombo->setCurrentItem(i);
      break;
    }
  }

  return 0;
}



FXint FF_SetupWiz_Currency::fromGui(FXString &currString) {
  AE_COMMODITY_LIST *col;
  FXString str=m_currencyCombo->getText();

  /* get currency list from AqBanking */
  col=m_wizard->getCommodityList();
  if (AE_Commodity_List_GetCount(col)) {
    AE_COMMODITY *co;

    co=AE_Commodity_List_First(col);
    while(co) {
      const char *s;

      s=AE_Commodity_GetCode(co);
      if (s && *s && strcasecmp(str.text(), s)==0) {
	currString=str;
        return 0;
      }
      co=AE_Commodity_List_Next(co);
    }
  }

  DBG_ERROR(0, "No currency");
  return GWEN_ERROR_BAD_DATA;
}



void FF_SetupWiz_Currency::enterPageForwards() {
  layout();
  if (checkGui(FALSE)<0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
}



void FF_SetupWiz_Currency::enterPageBackwards() {
  enterPageForwards();
}



void FF_SetupWiz_Currency::leavePageForwards() {
  FXString str;

  fromGui(str);
  m_wizard->setCurrencyCode(str);
}



long FF_SetupWiz_Currency::onCmdComboBox(FXObject*, FXSelector, void *ptr) {
  if (checkGui(FALSE)<0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
  return 1;
}




