/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_setupwiz_db.hpp"
#include "ff_opendb.hpp"

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>



FXDEFMAP(FF_SetupWiz_Db) FF_SetupWiz_DbMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_SetupWiz_Db::ID_FILENAME_BUTTON, FF_SetupWiz_Db::onCmdButton),
  FXMAPFUNC(SEL_COMMAND, FF_SetupWiz_Db::ID_FILENAME_EDIT, FF_SetupWiz_Db::onCmdEdit),
  FXMAPFUNC(SEL_CHANGED, FF_SetupWiz_Db::ID_FILENAME_EDIT, FF_SetupWiz_Db::onChangedEdit),
  FXMAPFUNC(SEL_COMMAND, FF_SetupWiz_Db::ID_COMBOBOX, FF_SetupWiz_Db::onCmdComboBox),
};


FXIMPLEMENT(FF_SetupWiz_Db, FF_SetupWizPage, FF_SetupWiz_DbMap, ARRAYNUMBER(FF_SetupWiz_DbMap))




FF_SetupWiz_Db::FF_SetupWiz_Db(FF_SetupWiz *p,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h,
			       FXint pl, FXint pr, FXint pt, FXint pb, FXint hs, FXint vs)
:FF_SetupWizPage(p, "db", opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  FXHorizontalFrame *hf;
  FXMatrix *m;

  m_label=new FOX16_HtmlLabel(this, I18N("Please select the path for the database to be created."),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);
  new FXSpring(this, LAYOUT_FILL_Y);
  m=new FXMatrix(this, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Module"), NULL, LABEL_NORMAL);
  m_moduleCombo=new FXComboBox(m, 1, this, ID_COMBOBOX, COMBOBOX_STATIC);
  m_moduleCombo->appendItem(I18N("Folder"));
  m_moduleCombo->setNumVisible(1);

  m_pathLabel=new FXLabel(m, I18N("Path"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);

  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X,
                           0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0);
  m_pathEdit=new FXTextField(hf, 32, this, ID_FILENAME_EDIT, TEXTFIELD_NORMAL);
  m_pathButton=new THEMEBUTTON(hf, "...", NULL, this, ID_FILENAME_BUTTON, BUTTON_NORMAL);

  m_moduleLabel=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_moduleLabel->setMinimumWidth(m->getDefaultWidth());
}



FF_SetupWiz_Db::FF_SetupWiz_Db()
:FF_SetupWizPage()
{
}



FF_SetupWiz_Db::~FF_SetupWiz_Db() {
}



void FF_SetupWiz_Db::enterPageForwards() {
  layout();
  if (checkGui(TRUE)<0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
}



void FF_SetupWiz_Db::enterPageBackwards() {
  enterPageForwards();
}



void FF_SetupWiz_Db::leavePageForwards() {
  FXString str;

  fromGui(str);
  m_wizard->setDbPath(str);
}



void FF_SetupWiz_Db::init(GWEN_DB_NODE *db) {
  toGui("");
}



void FF_SetupWiz_Db::selectModule(int i) {
  switch(i) {
  case 0:
    DBG_ERROR(0, "Entry 0 selected");
    m_pathLabel->setText(I18N("Path"));
    m_moduleLabel->setText(I18N("<p>This plugin uses a folder to store the "
                                "database.</p>"
                                "<p><b>Path</b> must point to that folder.</p>"));
    m_pathButton->enable();
    break;

  default:
    DBG_ERROR(0, "Unknown entry %d selected", i);
    m_moduleLabel->setText("");
    break;
  }
}



FXint FF_SetupWiz_Db::checkGui(FXbool quiet) {
  FXint i=m_moduleCombo->getCurrentItem();
  FXString path;
  GWEN_URL *url;

  switch(i) {
  case 0:
    break;
  default:
    DBG_ERROR(0, "Unknown module selected");
    if (!quiet) {
    }
    return GWEN_ERROR_INVALID;
  }

  path=m_pathEdit->getText().simplify();
  if (path.empty()) {
    DBG_ERROR(0, "Empty path");
    if (!quiet) {
    }
    return GWEN_ERROR_INVALID;
  }

  path.insert(0,"dir://");
  url=GWEN_Url_fromString(path.text());
  if (url==NULL) {
    DBG_ERROR(0, "Bad URL");
    if (!quiet) {
    }
    return GWEN_ERROR_INVALID;
  }
  GWEN_Url_free(url);
  return 0;
}



FXint FF_SetupWiz_Db::toGui(const FXString &urlString) {
  GWEN_URL *url;
  const char *s;
  FXString path;

  url=GWEN_Url_fromString(urlString.text());
  if (url==NULL) {
    DBG_ERROR(0, "Bad URL, assuming default");
    m_moduleCombo->setCurrentItem(0);
    m_pathEdit->setText("");
    selectModule(0);
    return 0;
  }

  /* select protocol (module) */
  s=GWEN_Url_GetProtocol(url);
  if (s) {
    if (strcasecmp(s, "dir")!=0) {
      DBG_ERROR(0, "Bad protocol [%s]", s);
      GWEN_Url_free(url);
      return GWEN_ERROR_INVALID;
    }
    m_moduleCombo->setCurrentItem(0);
  }
  else
    m_moduleCombo->setCurrentItem(0);

  s=GWEN_Url_GetServer(url);
  if (s)
    path=FXString(s);

  s=GWEN_Url_GetPath(url);
  if (s) {
    if (!path.empty())
      path+=GWEN_DIR_SEPARATOR_S;
    path+=FXString(s);
  }
  m_pathEdit->setText(path);

  return 0;
}



FXint FF_SetupWiz_Db::fromGui(FXString &urlString) {
  FXint i=m_moduleCombo->getCurrentItem();
  FXString path;
  GWEN_URL *url;

  switch(i) {
  case 0:
    break;
  default:
    DBG_ERROR(0, "Unknown module selected");
    return GWEN_ERROR_INVALID;
  }

  path=m_pathEdit->getText().simplify();
  if (path.empty()) {
    DBG_ERROR(0, "Empty path");
    return GWEN_ERROR_INVALID;
  }

  path.insert(0,"dir://");
  url=GWEN_Url_fromString(path.text());
  if (url==NULL) {
    DBG_ERROR(0, "Bad URL");
    return GWEN_ERROR_INVALID;
  }
  GWEN_Url_free(url);

  urlString=path;

  return 0;
}



long FF_SetupWiz_Db::onCmdButton(FXObject*, FXSelector, void *ptr) {
  FXString dirName=FXFileDialog::getOpenDirectory(this,
						  I18N("Select Folder"),
						  "");
  if (!dirName.empty())
    m_pathEdit->setText(dirName, TRUE);

  if (checkGui(TRUE)<0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
  return 1;
}



long FF_SetupWiz_Db::onCmdEdit(FXObject*, FXSelector, void *ptr) {
  if (checkGui(TRUE)<0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
  return 1;
}



long FF_SetupWiz_Db::onChangedEdit(FXObject*, FXSelector, void *ptr) {
  if (checkGui(TRUE)<0)
    m_wizard->disableNext();
  else
    m_wizard->enableNext();
  return 1;
}



long FF_SetupWiz_Db::onCmdComboBox(FXObject*, FXSelector, void *ptr) {
  FXint i=m_moduleCombo->getCurrentItem();

  selectModule(i);

  return 1;
}




