/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_APP_HPP
#define FF_APP_HPP



#include <fx.h>

#include <aqfoxext/fxx_app.hpp>

#include <aqfinance/engine/book/ae_book.h>

#include <aqbanking/banking.h>
#include <aqbanking/jobloadcellphone.h>


class FF_ModuleManager;

class FXXIconSet;


#include "ff_appspec.h"


#define FF_APP_CACHE_MAX_ENTRIES ((uint32_t)(10000))
#define FF_APP_CACHE_MAX_SIZE    ((uint32_t)(10*1024*1024))


#define FF_APP_YELLOW1 (FXRGB(255, 234, 84))
#define FF_APP_YELLOW2 (FXRGB(255, 244, 164))




class FF_App: public FXXApp {
public:
  FF_App(const FXString& name="AqFinance",
         const FXString& vendor="Aquamaniac");
  ~FF_App();

  void init(int& argc, char** argv, bool connect=true);
  void fini();

  int loadSettings();
  int saveSettings();

  void setBook(AE_BOOK *b) { m_book=b;};
  AE_BOOK *getBook() const { return m_book;};

  FF_ModuleManager *getModuleManager() const { return m_moduleManager;};
  void setModuleManager(FF_ModuleManager *mm) { m_moduleManager=mm;};

  AB_BANKING *getBanking() const { return m_banking;};
  void setBanking(AB_BANKING *ab) { m_banking=ab;};

  FXXIconSet *getIconSet(const FXString &name);

  void signalDbOpened(AE_BOOK *bk);
  void signalDbClosing();
  void signalDbClosed();
  void signalDbReload(AE_BOOK_TABLE_TYPE tt);

  void signalJobListUpdated();
  void signalAboutToSendJobs();

  bool dbCheckDeleteObject(FXComposite *parent,
                           AE_BOOK_TABLE_TYPE tt,
                           AQDB_ID id);

  AB_JOB_LIST2 *getJobList() const { return m_jobList;};
  void addJobToList(AB_JOB *j);
  int cancelJob(AB_JOB *j, bool doLock);


  int importContext(FXComposite *p, AB_IMEXPORTER_CONTEXT *ctx);
  int importAsTransactions(FXComposite *p, AE_STATEMENT_LIST *sl);

  AE_ACCOUNT *getPartnerReceivableAccount(FXComposite *p,
					  AE_ACCOUNT_TREE *at,
					  const AE_PARTNER *pt,
					  int doLock);
  AE_ACCOUNT *getPartnerPayableAccount(FXComposite *p,
				       AE_ACCOUNT_TREE *at,
				       const AE_PARTNER *pt,
				       int doLock);

  AE_ACCOUNT *getAutoEquityAccount(FXComposite *p,
				   AE_ACCOUNT_TREE *at,
				   const AE_COMMODITY *co,
				   int doLock);

  AE_ACCOUNT *getCashAccount(FXComposite *p,
			     AE_ACCOUNT_TREE *at,
			     const AE_COMMODITY *co,
			     int doLock);

  AE_ACCOUNT *getAccountForBankAccount(FXComposite *p,
                                       AE_ACCOUNT_TREE *at,
                                       AE_COMMODITY_LIST *cl,
				       const AE_BANKACCOUNT *ba,
                                       AQDB_ID commodityId,
				       int doLock);

  int addAccount(FXComposite *p,
		 AE_ACCOUNT_TREE *at,
		 AE_BOOK_TABLE_TYPE tt,
		 AE_ACCOUNT *acc,
		 int doLock);

  //int addAccount(AE_BOOK_TABLE_TYPE tt, AE_ACCOUINT *acc, int doLock);


  bool checkTransaction(FXComposite *p,
			AE_ACCOUNT_TREE *at,
			AE_COMMODITY_LIST *cl,
			const AE_TRANSACTION *t,
			bool quiet);

  bool checkTransaction(FXComposite *p,
			const AE_TRANSACTION *t,
			bool quiet);

  FXColor getAccountColorAsset() const { return m_accountColor_asset;};
  FXColor getAccountColorLiability() const { return m_accountColor_liability;};
  FXColor getAccountColorEquity() const { return m_accountColor_equity;};
  FXColor getAccountColorIncome() const { return m_accountColor_income;};
  FXColor getAccountColorExpense() const { return m_accountColor_expense;};
  FXColor getAccountColor(AE_ACCOUNT_TYPE t) const;

  bool getUseAccountingTerms() const { return m_useAccountingTerms;};
  void setUseAccountingTerms(bool b) { m_useAccountingTerms=b;};

  bool getShowBankAccountGraphs() const { return m_showBankAccountGraphs;};
  void setShowBankAccountGraphs(bool b) { m_showBankAccountGraphs=b;};

  uint32_t getCacheMaxEntries() const { return m_cacheMaxEntries;};
  void setCacheMaxEntries(uint32_t i) { m_cacheMaxEntries=i;};

  uint32_t getCacheMaxSize() const { return m_cacheMaxSize;};
  void setCacheMaxSize(uint32_t i) { m_cacheMaxSize=i;};

  const FF_APPSPEC_LIST *getAppSpecsPdfViewer() const { return m_appSpecsPdfViewer;};
  const FXString &getPdfViewerName() const { return m_pdfViewerName;};
  void setPdfViewerName(const FXString &s) { m_pdfViewerName=s;};

  const FF_APPSPEC_LIST *getAppSpecsEmailClient() const { return m_appSpecsEmailClient;};
  const FXString &getEmailClientName() const { return m_emailClientName;};
  void setEmailClientName(const FXString &s) { m_emailClientName=s;};

  bool deleteAccount(AQDB_ID id);
  bool deleteBankAccount(AQDB_ID id);
  bool deleteTransaction(AQDB_ID id);
  bool deleteJob(AQDB_ID id);
  bool deleteTask(AQDB_ID id);
  bool deletePartner(AQDB_ID id);
  bool deletePartnerGroup(AQDB_ID id);

  int presentConflictList(const FXString &title,
			  const FXString &text,
			  AE_DBCONFLICT_LIST *tmpList);

  int viewPdfFile(const char *fname, GWEN_DB_NODE *db);
  int emailPdfFile(const char *fname, GWEN_DB_NODE *db);

  int createLoadCellPhoneJob(const AE_LOAD_CELLPHONE *lc, AB_JOB **pJob);

  GWEN_CONFIGMGR *getConfigManager();

protected:
  int _importContext(FXComposite *p, AB_IMEXPORTER_CONTEXT *ctx,
                     AE_STATEMENT_LIST *statementsToImport);
  int _importSetFirstSplit(FXComposite *p,
                           AE_ACCOUNT_TREE *at,
                           AE_COMMODITY_LIST *cl,
                           AE_BANKACCOUNT_LIST *bal,
                           AE_TRANSACTION *t);
  int _importSetMissingSplit(FXComposite *p,
                             AE_ACCOUNT_TREE *at,
                             AE_COMMODITY_LIST *cl,
                             AE_TRANSACTION *t);

  int _handleConflicts(const FXString &title,
		       const FXString &text,
		       AQDB_ACTION_TYPE action,
		       AE_BOOK_TABLE_TYPE tt,
		       AQDB_ID oid);

  int _loadAppSpecs(const char *typeName, FF_APPSPEC_LIST *spList);
  void _genPdfAppSpecs();
  void _genEmailAppSpecs();

  int _execWithArgs(const FF_APPSPEC_LIST *asl, const FXString &an,
		    const char *fname, GWEN_DB_NODE *db);

  int _cancelSingleTransferJob(AB_JOB *j, bool doLock);
  int _cancelCreateStoJob(AB_JOB *j, bool doLock);
  int _cancelModifyStoJob(AB_JOB *j, bool doLock);


  AE_BOOK *m_book;
  FF_ModuleManager *m_moduleManager;
  AB_BANKING *m_banking;

  FXColor m_accountColor_asset;
  FXColor m_accountColor_liability;
  FXColor m_accountColor_equity;
  FXColor m_accountColor_income;
  FXColor m_accountColor_expense;

  bool m_useAccountingTerms;
  bool m_showBankAccountGraphs;
  FF_APPSPEC_LIST *m_appSpecsPdfViewer;
  FXString m_pdfViewerName;

  FF_APPSPEC_LIST *m_appSpecsEmailClient;
  FXString m_emailClientName;

  uint32_t m_cacheMaxEntries;
  uint32_t m_cacheMaxSize;

  AB_JOB_LIST2 *m_jobList;
};



#endif

