/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_MODULEMANAGER_HPP
#define FF_MODULEMANAGER_HPP


#include "ff_module.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/configmgr.h>


class FF_App;
class FF_Module;


class FF_ModuleManager: public FXComposite {
  FXDECLARE(FF_ModuleManager)

public:
  enum {
    ID_SPLITTER=FXComposite::ID_LAST,
    ID_LISTBOX
  };

public:
  FF_ModuleManager(FF_App *app,
                   FXComposite *p,
                   FXuint opts=0);
  virtual ~FF_ModuleManager();

  void init();
  void fini();

  FF_App *getApp() const { return m_app;};

  void addModule(FF_Module *m);

  bool enterModule(int id);

  void signalReset();

  void signalDbOpened(AE_BOOK *bk);
  void signalDbClosing();
  void signalDbClosed();
  void signalDbReload(AE_BOOK_TABLE_TYPE tt);

  bool dbCheckDeleteObject(FXComposite *parent,
                           AE_BOOK_TABLE_TYPE tt,
                           AQDB_ID id);

  void signalJobListUpdated();
  void signalAboutToSendJobs();

  void create();

  long onListSelected(FXObject*, FXSelector, void*);

  void dump(int indent);

  void layout();

protected:
  FF_ModuleManager();

  FF_App *m_app;
  FF_ModulePtrList m_modules;
  FF_Module *m_currentModule;

  FXSplitter *m_splitter;
  FXList *m_listBox;
  FXSwitcher *m_switcher;

  GWEN_DB_NODE *m_mainConfig;

};



#endif

