/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_accaccpage.hpp"
#include "fm_accpage.hpp"
#include "ff_accounttree.hpp"
#include "ff_app.hpp"
#include "ff_editaccount.hpp"
#include "ff_selectbankacc.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_AccountAccPage) FM_AccountAccPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_AccountAccPage::ID_BUTTON_ADDACCOUNT,
            FM_AccountAccPage::onCmdAddAccount),
  FXMAPFUNC(SEL_COMMAND, FM_AccountAccPage::ID_BUTTON_EDITACCOUNT,
            FM_AccountAccPage::onCmdEditAccount),
  FXMAPFUNC(SEL_COMMAND, FM_AccountAccPage::ID_BUTTON_DELACCOUNT,
            FM_AccountAccPage::onCmdDelAccount),
  FXMAPFUNC(SEL_COMMAND, FM_AccountAccPage::ID_BUTTON_WRITEXML,
            FM_AccountAccPage::onCmdWriteXml),

  FXMAPFUNC(SEL_COMMAND, FM_AccountAccPage::ID_BUTTON_RECALC,
            FM_AccountAccPage::onCmdRecalc),

  FXMAPFUNC(SEL_CHANGED, FM_AccountAccPage::ID_ACCOUNT_TREE,
            FM_AccountAccPage::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_AccountAccPage::ID_ACCOUNT_TREE,
            FM_AccountAccPage::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_AccountAccPage::ID_ACCOUNT_TREE,
            FM_AccountAccPage::onTreeRightBtnPress),
  FXMAPFUNC(SEL_CHANGED, FM_AccountAccPage::ID_MATCHER,
            FM_AccountAccPage::onChgMatcher),

  FXMAPFUNC(SEL_COMMAND, FM_AccountAccPage::ID_ASSIGN_BANK_ACCOUNT,
            FM_AccountAccPage::onCmdAssignBankingAccount),
};



FXIMPLEMENT(FM_AccountAccPage, FXVerticalFrame,
            FM_AccountAccPageMap, ARRAYNUMBER(FM_AccountAccPageMap))






FM_AccountAccPage::FM_AccountAccPage(FM_AccountPage *mainPage,
                                     FXComposite *p,
                                     FXuint opts,
                                     FXint x, FXint y, FXint w, FXint h)
:FXVerticalFrame(p, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_accountTree(NULL)
,m_accounts(NULL)
{
  FXHorizontalFrame *hf;

  flags|=FLAG_ENABLED|FLAG_SHOWN;

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_accountTree=new FF_AccountTree(hf, this, ID_ACCOUNT_TREE,
                                   LAYOUT_SIDE_LEFT |
                                   LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                   FOLDINGLIST_EXTENDEDSELECT |
                                   FOLDINGLIST_SHOWS_LINES |
                                   FOLDINGLIST_SHOWS_BOXES |
				   FOLDINGLIST_ROOT_BOXES |
				   FRAME_SUNKEN|FRAME_THICK);
}




FM_AccountAccPage::FM_AccountAccPage()
:FXVerticalFrame()
,m_mainPage(NULL)
,m_accountTree(NULL)
,m_accounts(NULL)
{
}



FM_AccountAccPage::~FM_AccountAccPage() {
  AE_Account_Tree_free(m_accounts);
}



void FM_AccountAccPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "accountPage");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_accountTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_accountTree->getHeaderSize(i));
      m_accountTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_accountTree->setHeaderArrowDir(i, TRUE);
        else
          m_accountTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_AccountAccPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "accountPage");
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_accountTree->getHeaderSize(i));
    b=m_accountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_AccountAccPage::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_AccountView: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_accountTree->dump(indent+2);
}



long FM_AccountAccPage::onCmdAddAccount(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ACCOUNT *pacc;
    AE_ACCOUNT *acc;

    acc=AE_Account_new();
    pacc=m_accountTree->getCurrentAccount();
    if (pacc)
      AE_Account_SetParentId(acc, AE_Account_GetId(pacc));
    else
      AE_Account_SetParentId(acc, 0);
    if (FF_EditAccount::editAccount(app, this, I18N("Add Account"), acc)) {
      int rv;

      rv=AE_Book_AddAccount(b, AE_Book_TableType_Account, acc, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add account (%d)", rv);
      }
      else {
        AE_RULE_LIST *rules;

        rules=AE_Account_GetRules(acc);
        if (rules) {
          AE_RULE *r;

          r=AE_Rule_List_First(rules);
          while(r) {
            /* rule is new, create */
            AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
            AE_Rule_SetOwnerId(r, AE_Account_GetId(acc));
            rv=AE_Book_AddRule(b, AE_Book_TableType_Rule, r, 1);
            if (rv<0) {
              DBG_ERROR(0, "here (%d)", rv);
            }
            r=AE_Rule_List_Next(r);
          }
        }
      }
      app->signalDbReload(AE_Book_TableType_Account);
    }
    AE_Account_free(acc);
  }

  return 1;
}



long FM_AccountAccPage::onCmdEditAccount(FXObject*, FXSelector, void*) {
  AE_ACCOUNT *acc;

  acc=m_accountTree->getCurrentAccount();
  if (acc) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditAccount::editAccount(app, this, I18N("Edit Account"), acc)) {
        int rv;

        rv=AE_Book_WriteAccount(b, AE_Book_TableType_Account, acc, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to write account (%d)", rv);
        }
        else {
          AE_RULE_LIST *rules;

          rules=AE_Account_GetRules(acc);
          if (rules) {
            AE_RULE *r;
  
            r=AE_Rule_List_First(rules);
            while(r) {
              if (AE_Rule_GetId(r)) {
                /* rule exists, check whether is has been deleted or modified */
                if (AE_Rule_GetRuntimeFlags(r) & AE_RULE_RTFLAGS_DELETED) {
                  rv=AE_Book_DeleteRule(b, AE_Book_TableType_Rule, r, 1);
                  if (rv<0) {
                    DBG_ERROR(0, "here (%d)", rv);
                  }
                }
		else if (AE_Rule_GetRuntimeFlags(r) & AE_RULE_RTFLAGS_MODIFIED) {
                  rv=AE_Book_WriteRule(b, AE_Book_TableType_Rule, r, 1);
                  if (rv<0) {
                    DBG_ERROR(0, "here (%d)", rv);
                  }
		}
	      }
              else {
                /* rule is new, create */
                AE_Rule_SetOwnerType(r, AE_Book_TableType_Account);
                AE_Rule_SetOwnerId(r, AE_Account_GetId(acc));
                rv=AE_Book_AddRule(b, AE_Book_TableType_Rule, r, 1);
                if (rv<0) {
                  DBG_ERROR(0, "here (%d)", rv);
                }
              }
              r=AE_Rule_List_Next(r);
            }
          }
        }
        app->signalDbReload(AE_Book_TableType_Account);
      }
    }
  }
  return 1;
}



long FM_AccountAccPage::onCmdDelAccount(FXObject*, FXSelector, void*) {
  AE_ACCOUNT *acc;

  acc=m_accountTree->getCurrentAccount();
  if (acc) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint res;

      res=FXMessageBox::question(this, MBOX_YES_NO,
                                 I18N("Delete Account"),
				 "%s",
                                 I18N("Are you sure you want to delete the selected "
				      "account?"));
      if (res!=MBOX_CLICKED_YES) {
        DBG_ERROR(0, "Aborted");
      }
      else {
	app->deleteAccount(AE_Account_GetId(acc));
	app->signalDbReload(AE_Book_TableType_Account);
      }
    }
  }
  return 1;
}



long FM_AccountAccPage::onCmdWriteXml(FXObject*, FXSelector, void*) {
  AE_ACCOUNT *acc;

  acc=m_accountTree->getCurrentAccount();
  if (acc) {
    FF_App *app;
    AE_BOOK *b;
    AQDB_ID aid;

    aid=AE_Account_GetId(acc);
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXString fname;
    
      fname=FXFileDialog::getSaveFilename(this,
					  I18N("Select Name of XML File"),
					  "",
					  I18N("All Files (*)\nXML Files (*.xml)"));
      if (!fname.empty()) {
	AE_ACCOUNT_TREE *at;
	int rv;
  
	at=AE_Account_Tree_new();
	rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
          AE_Account_Tree_free(at);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Error reading account tree from database (%d)"), rv);
	  return 1;
	}

	acc=AE_Account_Tree_GetById(at, aid);
	if (acc) {
	  GWEN_XMLNODE *root;
	  GWEN_XMLNODE *nh;

	  root=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "accounts");
	  nh=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "xml");
	  GWEN_XMLNode_AddHeader(root, nh);

	  AE_Book_WriteAccountTreeXml(b, at, acc, root);

	  rv=GWEN_XMLNode_WriteFile(root, fname.text(),
				    GWEN_XML_FLAGS_DEFAULT |
                                    GWEN_XML_FLAGS_SIMPLE |
				    GWEN_XML_FLAGS_HANDLE_HEADERS);
	  if (rv<0) {
	    GWEN_XMLNode_free(root);
	    AE_Account_Tree_free(at);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("File Error"),
				I18N("Error writing XML file: %d"), rv);
	    return 1;
	  }
	  GWEN_XMLNode_free(root);
	}
	else {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
                              "%s",
			      I18N("Account not found in database. "
				   "Maybe the account was deleted by another process?"));
	}
	AE_Account_Tree_free(at);
      }
    }
  }

  return 1;
}



long FM_AccountAccPage::onCmdRecalc(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ACCOUNT_TREE *at;
    int rv;

    at=AE_Account_Tree_new();
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Database Error"),
			  I18N("Error reading accounts from database (%d)"),
			  rv);
      AE_Account_Tree_free(at);
    }
    else {
      rv=AE_Book_ReadAccountTreeBalances(b, at);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Error reading account balances from database (%d)"),
			    rv);
	AE_Account_Tree_free(at);
      }
      else {
	AE_Account_Tree_ClearBalances(at);
	rv=AE_Book_CalcAccountTreeBalances(b, at);
	if (rv<0) {
	  DBG_ERROR(0, "Error calculating account balances");
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Error calculating accounts balances (%d)"),
			      rv);
	}
	else {
	  rv=AE_Book_WriteAccountTreeBalances(b, at, 1);
	  if (rv<0) {
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Error writing account balances to database (%d)"),
				rv);
	  }
	}
	AE_Account_Tree_free(at);
  
	/* update account tree */
	dbReload(AE_Book_TableType_Account);
      }
    }
  }
  return 1;
}




long FM_AccountAccPage::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  return 1;
}



long FM_AccountAccPage::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_AccountTreeItem *item=dynamic_cast<FF_AccountTreeItem*>((FXFoldingItem*) ptr);
    if (item)
      m_mainPage->openAccountPage(item->getAccount());
  }

  return 1;
}



long FM_AccountAccPage::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  rebuild();
  return 1;
}



long FM_AccountAccPage::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_ACCOUNT *acc;

  acc=m_accountTree->getCurrentAccount();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    new FXMenuCommand(&pmenu, I18N("Add Account"), NULL, this, ID_BUTTON_ADDACCOUNT);

    c=new FXMenuCommand(&pmenu, I18N("Edit Account"), NULL, this, ID_BUTTON_EDITACCOUNT);
    if (!acc) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Account"), NULL, this, ID_BUTTON_DELACCOUNT);
    if (!acc) c->disable();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Export Account Branch to XML"), NULL, this, ID_BUTTON_WRITEXML);
    if (!acc) c->disable();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Assign Online Account"), NULL,
			this, ID_ASSIGN_BANK_ACCOUNT);
    if (!acc) c->disable();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Recalculate"), NULL, this, ID_BUTTON_RECALC);

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FM_AccountAccPage::onCmdAssignBankingAccount(FXObject*, FXSelector, void* ptr) {
  AE_ACCOUNT *acc;

  acc=m_accountTree->getCurrentAccount();
  if (acc) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AQDB_ID id;

      id=AE_Account_GetBankAccountId(acc);
      if (FF_SelectBankAccount::selectAccount(app,
					      this,
					      I18N("Assign Bank Account"),
                                              NULL,
					      id)) {
	int rv;

        AE_Account_SetBankAccountId(acc, id);
        rv=AE_Book_WriteAccount(b, AE_Book_TableType_Account, acc, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add bank account (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_Account);
      }
    }
  }
  return 1;
}



void FM_AccountAccPage::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_AccountAccPage::dbClosing() {
  m_accountTree->clearItems();
}



void FM_AccountAccPage::dbClosed() {
}



void FM_AccountAccPage::rebuild() {
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());
  if (m_accounts) {
    AE_ACCOUNT *acc;
    AQDB_ID currentAccId=0;

    acc=m_accountTree->getCurrentAccount();
    if (acc)
      currentAccId=AE_Account_GetId(acc);

    m_accountTree->clearItems();
    m_accountTree->reload();

    FXString str=m_matcher->getText().simplify();

    /* select matching accounts */
    AE_Account_Tree_SelectMatching(m_accounts, str.text());
    m_accountTree->addSelectedAccounts(m_accounts, 0);
  
    /* expand and sort */
    m_accountTree->expandItems();
    m_accountTree->sort();

    if (currentAccId!=0) {
      FF_AccountTreeItem *item=m_accountTree->findAccountItem(currentAccId);
      if (item) {
	item->setSelected(true);
	m_accountTree->makeItemVisible(item);
	m_accountTree->setCurrentItem(item);
      }
    }

    if (1) {
      FXFoldingItem *fi;

      /* colorize */
      fi=m_accountTree->getFirstItem();
      while(fi) {
	FF_AccountTreeItem *item;

	item=dynamic_cast<FF_AccountTreeItem*>(fi);
	if (item) {
	  AE_ACCOUNT *a;

	  a=item->getAccount();
	  if (a) {
	    AE_ACCOUNT_GROUP gr;

	    gr=AE_AccountType_GetGroup(AE_Account_GetType(a));
	    switch(gr) {
	    case AE_AccountGroup_Asset:
	      item->setBackColor(app->getAccountColorAsset());
	      item->setTextColor(m_accountTree->getTextColor());
	      item->setUseOwnColors(TRUE);
	      m_accountTree->updateItem(item);
              break;
	    case AE_AccountGroup_Liability:
	      item->setBackColor(app->getAccountColorLiability());
	      item->setTextColor(m_accountTree->getTextColor());
	      item->setUseOwnColors(TRUE);
	      m_accountTree->updateItem(item);
              break;
	    case AE_AccountGroup_Equity:
	      item->setBackColor(app->getAccountColorEquity());
	      item->setTextColor(m_accountTree->getTextColor());
	      item->setUseOwnColors(TRUE);
	      m_accountTree->updateItem(item);
              break;
	    case AE_AccountGroup_Income:
	      item->setBackColor(app->getAccountColorIncome());
	      item->setTextColor(m_accountTree->getTextColor());
	      item->setUseOwnColors(TRUE);
	      m_accountTree->updateItem(item);
              break;
	    case AE_AccountGroup_Expense:
	      item->setBackColor(app->getAccountColorExpense());
	      item->setTextColor(m_accountTree->getTextColor());
	      item->setUseOwnColors(TRUE);
	      m_accountTree->updateItem(item);
              break;
	    default:
              break;
	    }
	  }
	}

        fi=fi->getBelow();
      }
    }
  }
}



void FM_AccountAccPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_Account ||
      tt==AE_Book_TableType_Split ||
      tt==AE_Book_TableType_AccountBalance ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      int rv;

      if (m_accounts)
	AE_Account_Tree_Clear(m_accounts);
      else
        m_accounts=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, m_accounts);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	AE_Account_Tree_ClearBranchBalances(m_accounts);

	/* read account balances */
	rv=AE_Book_ReadAccountTreeBalances(b, m_accounts);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	}

	/* calculate branch balances */
	AE_Account_Tree_CalcBranchBalances(m_accounts);
      }
    }
    rebuild();
  }
}



void FM_AccountAccPage::aboutToSendJobs() {
}



void FM_AccountAccPage::enter() {
}



void FM_AccountAccPage::leave() {
}



void FM_AccountAccPage::reset() {
}



