/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_BANKACCACCPAGE_HPP
#define FM_BANKACCACCPAGE_HPP


#include "ff_accounttree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_AccountTree;
class FM_AccountPage;


class FM_AccountAccPage: public FXVerticalFrame {
  FXDECLARE(FM_AccountAccPage)

public:
  enum {
    ID_ACCOUNT_TREE=FXPacker::ID_LAST,
    ID_MATCHER,

    ID_BUTTON_ADDACCOUNT,
    ID_BUTTON_EDITACCOUNT,
    ID_BUTTON_DELACCOUNT,
    ID_BUTTON_RECALC,
    ID_BUTTON_WRITEXML,

    ID_ASSIGN_BANK_ACCOUNT,

    ID_LAST
  };

public:
  FM_AccountAccPage(FM_AccountPage *mainPage,
                    FXComposite *p,
                    FXuint opts=0,
                    FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_AccountAccPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void aboutToSendJobs();

  void enter();
  void leave();

  void reset();

  long onCmdAddAccount(FXObject*, FXSelector, void*);
  long onCmdEditAccount(FXObject*, FXSelector, void*);
  long onCmdDelAccount(FXObject*, FXSelector, void*);
  long onCmdWriteXml(FXObject*, FXSelector, void*);

  long onCmdRecalc(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeCommand(FXObject*, FXSelector, void* ptr);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  long onChgMatcher(FXObject*, FXSelector, void* ptr);

  long onCmdAssignBankingAccount(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

protected:
  FM_AccountAccPage();
  void rebuild();


  FM_AccountPage *m_mainPage;

  FF_AccountTree *m_accountTree;

  FXTextField *m_matcher;

  AE_ACCOUNT_TREE *m_accounts;
};




#endif

