/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_app.hpp"
#include "fm_accpage.hpp"
#include "fm_accaccpage.hpp"
#include "fm_accstmpage.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_AccountPage) FM_AccountPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_AccountPage::ID_TABBOOK,
	    FM_AccountPage::onTabClicked),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, FM_AccountPage::ID_TABITEM,
	    FM_AccountPage::onTabLeftButtonPressed),
};


            
FXIMPLEMENT(FM_AccountPage, FXHorizontalFrame,
            FM_AccountPageMap, ARRAYNUMBER(FM_AccountPageMap))





FM_AccountPage::FM_AccountPage(FXComposite *p,
				       FXuint opts,
				       FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_tabBook(NULL)
,m_accountView(NULL)
,m_lastEnteredTab(-1)
,m_config(NULL)
,m_transactionMarkId(0)
,m_closeTabIcon(NULL)
{
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());

  m_closeTabIcon=app->getIcon("closetab");
  m_tabBook=new FXTabBook(this, this, ID_TABBOOK,
			  LAYOUT_FILL_X|LAYOUT_FILL_Y);
  /* create tab 1 */
  new THEMETABITEM(m_tabBook, I18N("Accounts"), NULL,
                JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  m_accountView=new FM_AccountAccPage(this, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

}



FM_AccountPage::FM_AccountPage()
:FXHorizontalFrame()
,m_tabBook(NULL)
,m_accountView(NULL)
,m_lastEnteredTab(-1)
,m_config(NULL)
,m_transactionMarkId(0)
,m_closeTabIcon(NULL)
{
}



FM_AccountPage::~FM_AccountPage() {
}



void FM_AccountPage::create() {
  FXHorizontalFrame::create();
  if (m_closeTabIcon)
    m_closeTabIcon->create();
}



void FM_AccountPage::init(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Init");
  m_config=GWEN_DB_Group_dup(db);
  std::list<FM_AccountStmPage*>::iterator it;

  m_accountView->init(m_config);
  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++)
    (*it)->init(m_config);

}



void FM_AccountPage::fini(GWEN_DB_NODE *db) {
  std::list<FM_AccountStmPage*>::iterator it;

  DBG_INFO(0, "Fini");

  /* list open accounts */
  GWEN_DB_DeleteVar(m_config, "openAccounts");
  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++)
    GWEN_DB_SetIntValue(m_config, GWEN_DB_FLAGS_DEFAULT, "openAccounts",
                        (*it)->getBankAccountId());

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++)
    (*it)->fini(m_config);
  m_accountView->fini(m_config);

  GWEN_DB_ClearGroup(db, NULL);
  GWEN_DB_AddGroupChildren(db, m_config);

  GWEN_DB_Group_free(m_config);
  m_config=NULL;
}



void FM_AccountPage::dbOpened(AE_BOOK *bk) {
  std::list<FM_AccountStmPage*>::iterator it;

  m_accountView->dbOpened(bk);

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    (*it)->dbOpened(bk);
  }
}



void FM_AccountPage::dbClosing() {
  std::list<FM_AccountStmPage*>::iterator it;

  m_accountView->dbClosing();

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    (*it)->dbClosing();
  }
}



void FM_AccountPage::dbClosed() {
  std::list<FM_AccountStmPage*>::iterator it;

  m_accountView->dbClosed();

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    (*it)->dbClosed();
  }
}



void FM_AccountPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  std::list<FM_AccountStmPage*>::iterator it;

  m_accountView->dbReload(tt);

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    (*it)->dbReload(tt);
  }
}



void FM_AccountPage::aboutToSendJobs() {
  FF_App *app;
  AE_BOOK *b;
  std::list<FM_AccountStmPage*>::iterator it;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AQDB_ID id;

    rv=AE_Book_GetLastId(b, AE_Book_TableType_Transaction, &id);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      m_transactionMarkId=id;
    }
  }

  m_accountView->aboutToSendJobs();
  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    (*it)->aboutToSendJobs();
  }
}



long FM_AccountPage::onTabClicked(FXObject*, FXSelector, void* ptr) {
  FXint idx=(FXint)(FXival)ptr;
  FM_AccountStmPage *ap;

  if (m_lastEnteredTab!=-1) {
    if (m_lastEnteredTab==0)
      m_accountView->leave();
    else {
      ap=findPageAt(m_lastEnteredTab-1);
      if (ap)
        ap->leave();
    }
  }

  if (idx==0) {
    m_accountView->enter();
  }
  else {
    ap=findPageAt(idx-1);
    if (ap)
      ap->enter();
  }

  m_lastEnteredTab=idx;


  return 1;
}



long FM_AccountPage::onTabLeftButtonPressed(FXObject *obj, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*) ptr;
  FXTabItem *item=(FXTabItem*) obj;

  if (item->isEnabled()){
    if (m_closeTabIcon) {
      int ix;

      ix=item->getWidth()-m_closeTabIcon->getWidth();
      if (event->win_x>ix) {
        FM_AccountStmPage *page;

        page=(FM_AccountStmPage*)(item->getNext());
	page->fini(m_config);
        m_accountPages.remove(page);
        delete item;
        delete page;
        return 1;
      }
    }
    else {
      DBG_ERROR(0, "no icon");
    }
  }
  else {
    DBG_ERROR(0, "Not enabled");
  }

  return 0;
}



FM_AccountStmPage *FM_AccountPage::findPageForAccount(AQDB_ID aid) {
  std::list<FM_AccountStmPage*>::iterator it;

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    if ((*it)->getBankAccountId()==aid)
      return *it;
  }

  return NULL;
}



int FM_AccountPage::findIndexForAccount(AQDB_ID aid) {
  std::list<FM_AccountStmPage*>::iterator it;
  int idx=0;

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    if ((*it)->getBankAccountId()==aid)
      return idx;
    idx++;
  }

  return -1;
}



FM_AccountStmPage *FM_AccountPage::findPageAt(int idx) {
  std::list<FM_AccountStmPage*>::iterator it;

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    if (idx==0)
      return *it;
    idx--;
  }

  return NULL;
}



void FM_AccountPage::openAccountPage(const AE_ACCOUNT *acc) {
  FM_AccountStmPage *ap;
  AQDB_ID aid;

  aid=AE_Account_GetId(acc);
  ap=findPageForAccount(aid);
  if (ap) {
    int idx=findIndexForAccount(aid);
    if (idx!=-1)
      m_tabBook->setCurrent(idx+1, true);
  }
  else {
    const char *s;
    FXString title;
    FXTabItem *tab;

    /* no page open for account, create one */
    s=AE_Account_GetName(acc);
    if (s==NULL || *s==0)
      s=AE_Account_GetCode(acc);
    if (s==NULL || *s==0)
      s=I18N("<unknown>");
    title=FXString(s);

    /* create tab entry */
    tab=new THEMETABITEM(m_tabBook, title, m_closeTabIcon,
			 JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
    tab->setTarget(this);
    tab->setSelector(ID_TABITEM);

    /* create page */
    ap=new FM_AccountStmPage(this, acc, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    tab->create();
    ap->create();
    m_tabBook->layout();
    m_accountPages.push_back(ap);
    ap->init(m_config);

    /* we correctly use .size() here, not size()-1, because the first tab is the account tree */
    m_tabBook->setCurrent(m_accountPages.size(), true);
  }
}



void FM_AccountPage::enter() {
  if (m_lastEnteredTab!=-1) {
    FM_AccountStmPage *ap;

    if (m_lastEnteredTab==0)
      m_accountView->enter();
    else {
      ap=findPageAt(m_lastEnteredTab-1);
      if (ap)
        ap->enter();
    }
  }
}



void FM_AccountPage::leave() {
  if (m_lastEnteredTab!=-1) {
    FM_AccountStmPage *ap;

    if (m_lastEnteredTab==0)
      m_accountView->leave();
    else {
      ap=findPageAt(m_lastEnteredTab-1);
      if (ap)
        ap->leave();
    }
  }
}


void FM_AccountPage::reset() {
  std::list<FM_AccountStmPage*>::iterator it;

  m_accountView->reset();

  for (it=m_accountPages.begin(); it!=m_accountPages.end(); it++) {
    (*it)->reset();
  }
}





