/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_ACCPAGE_HPP
#define FM_ACCPAGE_HPP



#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>



class FM_AccountAccPage;
class FM_AccountStmPage;


class FM_AccountPage: public FXHorizontalFrame {
  FXDECLARE(FM_AccountPage)
public:
  enum {
    ID_TABBOOK=FXHorizontalFrame::ID_LAST,
    ID_TABITEM,
    ID_LAST
  };

  FM_AccountPage(FXComposite *p,
		     FXuint opts=0,
		     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_AccountPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void aboutToSendJobs();

  AQDB_ID getTransactionMarkId() const { return m_transactionMarkId;};

  void enter();
  void leave();

  void reset();

  void create();

  void openAccountPage(const AE_ACCOUNT *acc);

  long onTabClicked(FXObject*, FXSelector, void*);
  long onTabLeftButtonPressed(FXObject*, FXSelector, void *ptr);

protected:
  FM_AccountPage();

  FM_AccountStmPage *findPageForAccount(AQDB_ID aid);
  int findIndexForAccount(AQDB_ID aid);
  FM_AccountStmPage *findPageAt(int idx);

  FXTabBook *m_tabBook;
  FM_AccountAccPage *m_accountView;
  int m_lastEnteredTab;
  std::list<FM_AccountStmPage*> m_accountPages;

  GWEN_DB_NODE *m_config;

  AQDB_ID m_transactionMarkId;
  FXIcon *m_closeTabIcon;
};



#endif
