/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_accstmpage.hpp"
#include "fm_accpage.hpp"
#include "ff_transactiontable.hpp"
#include "ff_app.hpp"
#include "ff_edittransaction.hpp"
#include "ff_selectjob.hpp"
#include "ff_editpayment.hpp"
#include "ff_selectaccount.hpp"
#include "ff_editsimpletransaction.hpp"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_AccountStmPage) FM_AccountStmPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_SPLIT_TABLE,
            FM_AccountStmPage::onCmdTable),
  FXMAPFUNC(SEL_REPLACED, FM_AccountStmPage::ID_SPLIT_TABLE,
            FM_AccountStmPage::onTableContentReplaced),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FM_AccountStmPage::ID_SPLIT_TABLE,
            FM_AccountStmPage::onTableRightButtonRelease),
  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_DELSPLIT,
            FM_AccountStmPage::onCmdDelStm),
  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_ADDTRANS,
            FM_AccountStmPage::onCmdAddStm),

  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_ASSIGN_TO_BUY_PAYMENT,
	    FM_AccountStmPage::onCmdAssignToOutgoingPayment),
  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_ASSIGN_TO_SELL_PAYMENT,
	    FM_AccountStmPage::onCmdAssignToIncomingPayment),
  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_CREATE_BUY_PAYMENT,
	    FM_AccountStmPage::onCmdCreateOutgoingPayment),
  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_CREATE_SELL_PAYMENT,
	    FM_AccountStmPage::onCmdCreateIncomingPayment),

  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_CMD_MOVE_TO_ACCOUNT,
	    FM_AccountStmPage::onCmdMoveToAccount),

  FXMAPFUNC(SEL_COMMAND, FM_AccountStmPage::ID_EDIT_SPLITS,
	    FM_AccountStmPage::onCmdSplitEditor),
  FXMAPFUNC(SEL_CHANGED, FM_AccountStmPage::ID_MATCHER,
            FM_AccountStmPage::onChgMatcher),
};


            
FXIMPLEMENT(FM_AccountStmPage, FXVerticalFrame,
            FM_AccountStmPageMap, ARRAYNUMBER(FM_AccountStmPageMap))



FM_AccountStmPage::FM_AccountStmPage(FM_AccountPage *page,
                                     const AE_ACCOUNT *acc,
                                     FXComposite *p,
                                     FXuint opts,
                                     FXint x, FXint y, FXint w, FXint h)
:FXVerticalFrame(p, opts, x, y, w, h)
,m_parent(page)
,m_accountId(AE_Account_GetId(acc))
,m_hLabel(NULL)
,m_fLabel(NULL)
,m_transTable(NULL)
,m_account(NULL)
,m_accountCommodity(NULL)
,m_needReload(true)
,m_isVisible(false)
,m_blockReload(false)
,m_transactions(NULL)
{
  FXHorizontalFrame *hf;
  FF_App *app;

  flags|=FLAG_SHOWN | FLAG_ENABLED;

  app=dynamic_cast<FF_App*>(getApp());

  new FXHorizontalSeparator(this, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  m_hLabel=new FOX16_HtmlLabel(this, "",
                               LAYOUT_FILL_X | LAYOUT_MIN_HEIGHT |
                               FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  m_hLabel->setMinimumWidth(150);
  m_hLabel->setMaxDefaultWidth(600);
  new FXHorizontalSeparator(this, LAYOUT_FILL_X | SEPARATOR_GROOVE);

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  m_transTable=new FF_TransactionTable(this, this, ID_SPLIT_TABLE,
                                       FXXTable::TABLE_TRACK_SCROLLBARS |
				       LAYOUT_FILL_X | LAYOUT_FILL_Y|
				       FRAME_SUNKEN | FRAME_THICK);
  if (AE_AccountType_GetGroup(AE_Account_GetType(acc))==AE_AccountGroup_Equity)
    m_transTable->setColorizeCounterAccounts(true);
  m_transTable->setShowGrid(false);
  m_transTable->setCellBackColor(0, 0, FXRGB(183, 220, 159));
  m_transTable->setCellBackColor(0, 1, FXRGB(183, 220, 159));
  m_transTable->setCellBackColor(1, 0, FXRGB(255, 255, 255));
  m_transTable->setCellBackColor(1, 1, FXRGB(255, 255, 255));

  m_transTable->setMinRowHeight(20);
  m_transTable->setAccountId(m_accountId);

  m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_DATE, 100);
  m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_MEMO, 100);
  m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_COUNTER_ACCOUNT, 100);
  if (app->getUseAccountingTerms()) {
    m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_DEBIT_VALUE, 100);
    m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_CREDIT_VALUE, 100);
  }
  else {
    m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_DEPOSIT_VALUE, 100);
    m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_WITHDRAWAL_VALUE, 100);
  }
  m_transTable->addColumn(FF_TransactionTable::COLUMN_TYPE_FLAGS, 100);

  new FXHorizontalSeparator(this, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  m_fLabel=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_MIN_HEIGHT | FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  m_fLabel->setMinimumWidth(150);
}



FM_AccountStmPage::FM_AccountStmPage()
:FXVerticalFrame()
,m_accountId(0)
,m_hLabel(NULL)
,m_fLabel(NULL)
,m_transTable(NULL)
,m_account(NULL)
,m_accountCommodity(NULL)
{
}



FM_AccountStmPage::~FM_AccountStmPage() {
  AE_Transaction_List_free(m_transactions);
  AE_Commodity_free(m_accountCommodity);
  AE_Account_free(m_account);
}



void FM_AccountStmPage::create() {
  FXVerticalFrame::create();
  m_hLabel->create();
  m_transTable->create();
}




void FM_AccountStmPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbSplitPage;
  GWEN_DB_NODE *dbT=NULL;
  char numbuf[32];
  int rv;

  DBG_INFO(0, "Init");
  if (m_accountId) {
    FF_App *app;
    AE_BOOK *b;
    AE_ACCOUNT_TREE *at;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    at=AE_Account_Tree_new();
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
    if (rv>=0) {
      AE_ACCOUNT *a;

      a=AE_Account_Tree_GetById(at, m_accountId);
      if (a) {
	GWEN_BUFFER *tbuf;
	FXString str;

	tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	AE_Account_PathToBuffer(a, tbuf);
	str=FXStringFormat(I18N("<h2>%s</h2>"), GWEN_Buffer_GetStart(tbuf));
        GWEN_Buffer_free(tbuf);
	m_hLabel->setText(str);
      }
    }
    AE_Account_Tree_free(at);
  }

  snprintf(numbuf, sizeof(numbuf)-1, "acc-%x", m_accountId);
  numbuf[sizeof(numbuf)-1]=0;
  dbSplitPage=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "splitPage");
  if (dbSplitPage) {
    dbT=GWEN_DB_GetGroup(dbSplitPage, GWEN_PATH_FLAGS_NAMEMUSTEXIST, numbuf);
    if (dbT==NULL)
      dbT=GWEN_DB_GetGroup(dbSplitPage, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "last");
  }
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_transTable->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_transTable->getHeaderSize(i));
      m_transTable->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_transTable->setHeaderArrowDir(i, TRUE);
        else
          m_transTable->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_AccountStmPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbSplitPage;
  GWEN_DB_NODE *dbT;
  GWEN_DB_NODE *dbLast;
  int i;
  char numbuf[32];

  DBG_INFO(0, "Fini");
  snprintf(numbuf, sizeof(numbuf)-1, "acc-%x", m_accountId);
  numbuf[sizeof(numbuf)-1]=0;
  dbSplitPage=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "splitPage");
  dbT=GWEN_DB_GetGroup(dbSplitPage, GWEN_DB_FLAGS_OVERWRITE_GROUPS, numbuf);

  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_transTable->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_transTable->getHeaderSize(i));
    b=m_transTable->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  dbLast=GWEN_DB_GetGroup(dbSplitPage, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "last");
  GWEN_DB_AddGroupChildren(dbLast, dbT);
}



void FM_AccountStmPage::enter() {
  m_isVisible=true;

  layout();
  if (m_needReload) {
    reload();
    m_needReload=false;
  }
}



void FM_AccountStmPage::leave() {
  m_isVisible=false;
}



void FM_AccountStmPage::dbOpened(AE_BOOK *bk) {
  if (m_isVisible) {
    reload();
    m_needReload=false;
  }
  else
    m_needReload=true;
}



void FM_AccountStmPage::dbClosing() {
  AE_Account_free(m_account);
  m_account=NULL;

  AE_Commodity_free(m_accountCommodity);
  m_accountCommodity=NULL;

  m_transTable->clearItems();
}



void FM_AccountStmPage::dbClosed() {
}



void FM_AccountStmPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (!m_blockReload) {
    if (tt==AE_Book_TableType_Split ||
	tt==AE_Book_TableType_Transaction ||
	tt==AE_Book_TableType_None) {
      if (m_isVisible) {
	reload();
	m_needReload=false;
      }
      else
	m_needReload=true;
    }
  }
}



void FM_AccountStmPage::aboutToSendJobs() {
}



long FM_AccountStmPage::onCmdTable(FXObject*, FXSelector, void* ptr) {
#if 0
  int r=m_transTable->getCurrentRow();
  if (r>=0) {
    AE_STATEMENT *st=m_transTable->getStatementByRow(r);
    AE_Statement_Dump(st, stderr, 2);
  }
#endif
  return 1;
}



long FM_AccountStmPage::onTableContentReplaced(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  int r;
  AE_TRANSACTION *t;
  FXXTable::ColumnInfo *ci=(FXXTable::ColumnInfo*) ptr;

  app=dynamic_cast<FF_App*>(getApp());

  r=ci->getRow();

  t=m_transTable->getTransactionByRow(r);
  if (t) {
    bool saveBlockRebuild;

    if ((AE_Transaction_GetRuntimeFlags(t) & (AE_TRANSACTION_RTFLAGS_MODIFIED |
					      AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED))) {
      int rv;
      AE_BOOK *b;
      AE_TRANSACTION *tOld=NULL;
      AQDB_ID tid;

      b=app->getBook();

      tid=AE_Transaction_GetId(t);
      if (tid!=0) {
	/* read original transaction */
	rv=AE_Book_ReadTransaction(b, AE_Book_TableType_Transaction, tid, &tOld);
	if (rv<0) {
	  FXMessageBox::error(this, FX::MBOX_OK,
			      I18N("Database Error"),
			      I18N("The original transaction could not be read (error: %d)"),
			      rv);
	  return 1;
	}

	rv=AE_Book_ReadTransactionSubObjects(b, tOld);
	if (rv<0) {
	  FXMessageBox::error(this, FX::MBOX_OK,
			      I18N("Database Error"),
			      I18N("The original transaction could not be read (error: %d)"),
			      rv);
	  AE_Transaction_free(tOld);
	  return 1;
	}
      }

      /* apply transaction */
      rv=AE_Book_ApplyTransaction(b, t, tOld, 1);
      if (rv<0) {
	FXMessageBox::error(this, FX::MBOX_OK,
			    I18N("Database Error"),
			    I18N("Could not write transaction and splits to database (error: %d)"),
			    rv);
	AE_Transaction_free(tOld);
	return 1;
      }

      AE_Transaction_free(tOld);
    }
    else {
      DBG_ERROR(0, "Transaction not modified");
    }

    saveBlockRebuild=m_blockReload;
    m_blockReload=true;
    app->signalDbReload(AE_Book_TableType_Transaction);
    m_blockReload=saveBlockRebuild;
  }
  else {
    DBG_ERROR(0, "No transaction");
  }
  return 1;
}



long FM_AccountStmPage::onTableRightButtonRelease(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;

  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuPane paneBuy(this);
    FXMenuPane paneSell(this);
    FXMenuPane paneSelected(this);

    new FXMenuCommand(&pmenu, I18N("Add Transaction(s)"), NULL, this, ID_CMD_ADDTRANS);
    new FXMenuCommand(&pmenu, I18N("Split Editor"), NULL, this, ID_EDIT_SPLITS);
    new FXMenuSeparator(&pmenu);

    /* create submenu for incoming payment */
    new FXMenuCommand(&paneSell, I18N("Assign to Incoming Payment (Sell)"), NULL, this, ID_CMD_ASSIGN_TO_SELL_PAYMENT);
    new FXMenuCommand(&paneSell, I18N("Create Incoming Payment (Sell)"), NULL, this, ID_CMD_CREATE_SELL_PAYMENT);
    new FXMenuCascade(&pmenu, I18N("Handle As Incoming Payment (Sell)..."), NULL, &paneSell);

    /* create submenu for outgoing payment */
    new FXMenuCommand(&paneBuy, I18N("Assign to Outgoing Payment (Buy)"), NULL, this, ID_CMD_ASSIGN_TO_BUY_PAYMENT);
    new FXMenuCommand(&paneBuy, I18N("Create Outgoing Payment (Buy)"), NULL, this, ID_CMD_CREATE_BUY_PAYMENT);
    new FXMenuCascade(&pmenu, I18N("Handle As Outgoing Payment (Buy)..."), NULL, &paneBuy);

    new FXMenuSeparator(&pmenu);

    /* create submenu for selected transactions */
    new FXMenuCommand(&paneSelected, I18N("Move to Another Account"), NULL, this, ID_CMD_MOVE_TO_ACCOUNT);
    new FXMenuCommand(&paneSelected, I18N("Delete Transaction(s)"), NULL, this, ID_CMD_DELSPLIT);
    new FXMenuCascade(&pmenu, I18N("Selected Transactions..."), NULL, &paneSelected);

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FM_AccountStmPage::onCmdDelStm(FXObject*, FXSelector, void* ptr) {
  AE_TRANSACTION_LIST2 *tl;
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  tl=AE_Transaction_List2_new();
  m_transTable->getSelectedTransactions(tl, FF_TransactionTable::COPYMODE_NONE);
  if (AE_Transaction_List2_GetSize(tl)) {
    FXuint res;

    res=FXMessageBox::question(this, MBOX_YES_NO,
			       I18N("Delete Transactions"),
			       "%s",
			       I18N("Are you sure you want to delete the selected "
				    "transaction(s)?"));
    if (res!=MBOX_CLICKED_YES) {
      DBG_ERROR(0, "Aborted");
      AE_Transaction_List2_free(tl);
      return 1;
    }
    else {
      AE_DBCONFLICT_LIST *clist;
      AE_TRANSACTION_LIST2_ITERATOR *it;
      int rv;
  
      clist=AE_DbConflict_List_new();
  
      rv=AE_Book_BeginEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "Error on begin edit: %d", rv);
	FXMessageBox::error(this, FX::MBOX_OK,
			    I18N("Database Error"),
			    I18N("Could not lock database for write access (error: %d)"),
			    rv);
	AE_Transaction_List2_free(tl);
	AE_DbConflict_List_free(clist);
	return 1;
      }
  
      it=AE_Transaction_List2_First(tl);
      if (it) {
	AE_TRANSACTION *t;
  
	t=AE_Transaction_List2Iterator_Data(it);
	while(t) {
	  rv=AE_Book_GetConflicts(b,
				  AQDB_ActionType_Remove,
				  AE_Book_TableType_Transaction,
				  AE_Transaction_GetId(t),
				  clist);
	  if (rv<0) {
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Database Error"),
				I18N("Error getting conflicts for transaction %08x (error: %d)"),
				AE_Transaction_GetId(t), rv);
	    AE_Transaction_List2Iterator_free(it);
	    AE_Transaction_List2_free(tl);
	    AE_DbConflict_List_free(clist);
	    return 1;
	  }
	  t=AE_Transaction_List2Iterator_Next(it);
	}
	AE_Transaction_List2Iterator_free(it);
      }
  
      /* takes over clist in any case */
      rv=app->presentConflictList(I18N("Data Conflicts"),
				  I18N("<p>Deleting these transactions leads to the following conflicts. "
				       "Please select appropriate solutions.</p>"
				       "<p>Click <i>Ok</i> when finished or <i>Abort</i> to abort the whole operation.</p>"),
				  clist);
      if (rv<0) {
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	DBG_ERROR(0, "here (%d)", rv);
	AE_Transaction_List2_free(tl);
	return 1;
      }


      it=AE_Transaction_List2_First(tl);
      if (it) {
	AE_TRANSACTION *t;
  
	t=AE_Transaction_List2Iterator_Data(it);
	while(t) {
	  rv=AE_Book_ApplyTransaction(b, NULL, t, 0);
	  if (rv<0) {
	    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    DBG_ERROR(0, "here (%d)", rv);
	    AE_Transaction_List2Iterator_free(it);
	    AE_Transaction_List2_free(tl);
	    return 1;
	  }

	  t=AE_Transaction_List2Iterator_Next(it);
	}
	AE_Transaction_List2Iterator_free(it);
      }

      rv=AE_Book_EndEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	FXMessageBox::error(this, FX::MBOX_OK,
			    I18N("Database Error"),
			    I18N("Could not apply changes to the database (error: %d)"),
			    rv);
	AE_Transaction_List2_free(tl);
	return 1;
      }
    }
  }
  AE_Transaction_List2_free(tl);

  app->signalDbReload(AE_Book_TableType_Transaction);
  app->signalDbReload(AE_Book_TableType_AccountBalance);

  return 1;
}



long FM_AccountStmPage::onCmdAddStm(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  AE_TRANSACTION *t;

  app=dynamic_cast<FF_App*>(getApp());
  t=AE_Transaction_new();
  if (m_account) {
    AE_SPLIT_LIST *sl;
    AE_SPLIT *sp;

    sl=AE_Transaction_GetSplitList(t);
    if (sl==NULL) {
      sl=AE_Split_List_new();
      AE_Transaction_SetSplitList(t, sl);
    }
    else
      AE_Split_List_Clear(sl);
    sp=AE_Split_new();
    AE_Split_SetAccountId(sp, AE_Account_GetId(m_account));
    AE_Split_SetCommodityId(sp, AE_Account_GetPrimaryCommodityId(m_account));
    AE_Split_List_Add(sp, sl);
  }
  FF_EditSimpleTransaction::createTransactions(app, this, I18N("Enter Transaction(s)"), t);
  AE_Transaction_free(t);
  return 1;
}



long FM_AccountStmPage::onCmdSplitEditor(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;
  AE_TRANSACTION *t;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    t=m_transTable->getCurrentTransaction();
    if (t) {
      AE_TRANSACTION *tcpy;
      int rv;

      tcpy=AE_Transaction_dup(t);

#if 0
	{
	  AE_SPLIT *sp;

	  DBG_ERROR(0, "Split List for transaction %p", t);
	  sp=AE_Transaction_GetFirstSplit(t);
	  while(sp) {
	    DBG_ERROR(0, "- split %p", sp);
	    sp=AE_Split_List_Next(sp);
	  }

	}
#endif

#if 0
      /* read original transaction */
      rv=AE_Book_ReadTransactionSubObjects(b, tcpy);
      if (rv<0) {
	FXMessageBox::error(this, FX::MBOX_OK,
			    I18N("Database Error"),
			    I18N("The original transaction could not be read (error: %d)"),
			    rv);
	AE_Transaction_free(tcpy);
	return 1;
      }
#endif

      if (FF_EditTransaction::editTransaction(app, this,
					      I18N("Edit Splits"),
					      t)) {
	m_transTable->rebuildTransactionflags(t);
	rv=AE_Book_ApplyTransaction(b, t, tcpy, 1);
	if (rv<0) {
	  FXMessageBox::error(this, FX::MBOX_OK,
			      I18N("Database Error"),
			      I18N("The original transaction could not be applied (error: %d)"),
			      rv);
	}
	app->signalDbReload(AE_Book_TableType_Split);
	app->signalDbReload(AE_Book_TableType_AccountBalance);
      }
      AE_Transaction_free(tcpy);
#if 0
      m_transTable->reload();
      m_transTable->rebuild();
#endif
    }
  }

  return 1;
}



long FM_AccountStmPage::onCmdAssignToOutgoingPayment(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;
  AE_TRANSACTION *t;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    t=m_transTable->getCurrentTransaction();
    if (t) {
      int rv;
      AQDB_ID payId=0;
      AE_JOB *j=NULL;

      while(1) {
	if (FF_SelectJob::selectJob(app, this,
				    I18N("Select Outgoing Payment"),
				    I18N("Please select the outgoing payment to which you want to assign "
					 "this transaction"),
				    payId,
				    FF_SelectJob::JobSelection_Buys)) {
	  if (payId!=0) {
	    rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, payId, &j);
	    if (rv<0) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("The selected job could not be read (error: %d)"),
				  rv);
	      return 1;
	    }
	    if (AE_Job_GetType(j)!=AE_Job_TypePayment) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Input Error"),
                                  "%s",
				  I18N("The selected job is not a payment. "
				       "Please select a payment."));
	    }
	    else {
	      AQDB_ID oldId;

	      oldId=AE_Job_GetTransactionId(j);
	      if (oldId!=0) {
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Input Error"),
				    "%s",
				    I18N("The selected job already has a transaction assigned to it. "
					 "Please select another payment."));
	      }
	      else {
		/* set transaction, write job back */
		AE_Job_SetTransactionId(j, AE_Transaction_GetId(t));
		rv=AE_Book_WriteJob(b, AE_Book_TableType_Job, j, 1);
		if (rv<0) {
		  FXMessageBox::error(this, FX::MBOX_OK,
				      I18N("Database Error"),
				      I18N("The selected job could not be written (error: %d)"),
				      rv);
		  return 1;
		}
		app->signalDbReload(AE_Book_TableType_Job);
		return 1;
	      }
	    }
	  }
	}
	else {
          /* aborted */
	  return 1;
	}
      }
    }
  }

  return 1;
}



long FM_AccountStmPage::onCmdAssignToIncomingPayment(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;
  AE_TRANSACTION *t;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    t=m_transTable->getCurrentTransaction();
    if (t) {
      int rv;
      AQDB_ID payId=0;
      AE_JOB *j=NULL;

      while(1) {
	if (FF_SelectJob::selectJob(app, this,
				    I18N("Select Incoming Payment"),
				    I18N("Please select the incoming payment to which you want to assign "
					 "this transaction"),
				    payId,
				    FF_SelectJob::JobSelection_Sells)) {
	  if (payId!=0) {
	    rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, payId, &j);
	    if (rv<0) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("The selected job could not be read (error: %d)"),
				  rv);
	      return 1;
	    }
	    if (AE_Job_GetType(j)!=AE_Job_TypePayment) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Input Error"),
                                  "%s",
				  I18N("The selected job is not a payment. "
				       "Please select a payment."));
	    }
	    else {
	      AQDB_ID oldId;

	      oldId=AE_Job_GetTransactionId(j);
	      if (oldId!=0) {
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Input Error"),
				    "%s",
				    I18N("The selected job already has a transaction assigned to it. "
					 "Please select another payment."));
	      }
	      else {
		/* set transaction, write job back */
		AE_Job_SetTransactionId(j, AE_Transaction_GetId(t));
		rv=AE_Book_WriteJob(b, AE_Book_TableType_Job, j, 1);
		if (rv<0) {
		  FXMessageBox::error(this, FX::MBOX_OK,
				      I18N("Database Error"),
				      I18N("The selected job could not be written (error: %d)"),
				      rv);
		  return 1;
		}
		app->signalDbReload(AE_Book_TableType_Job);
		return 1;
	      }
	    }
	  }
	}
	else {
          /* aborted */
	  return 1;
	}
      }
    }
  }

  return 1;
}



long FM_AccountStmPage::onCmdCreateIncomingPayment(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TRANSACTION *t;

    t=m_transTable->getCurrentTransaction();
    if (t) {
      int rv;
      AQDB_ID invoiceId=0;
      AE_JOB *j=NULL;

      while(1) {
	if (FF_SelectJob::selectJob(app, this,
				    I18N("Select Outgoing Invoice"),
				    I18N("Please select the outgoing invoice for which you want to "
					 "create a payment."),
				    invoiceId,
				    FF_SelectJob::JobSelection_Sells)) {
	  if (invoiceId==0) {
            DBG_ERROR(0, "No invoice selected");
	    return 1;
	  }

	  rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, invoiceId, &j);
	  if (rv<0) {
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Database Error"),
				I18N("The selected job could not be read (error: %d)"),
				rv);
	    return 1;
	  }

	  if (AE_Job_GetType(j)!=AE_Job_TypeInvoice) {
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Input Error"),
				"%s",
				I18N("The selected job is not an invoice. "
				     "Please select an invoice."));
	  }
	  else {
	    AQDB_ID tid=0;
	    AE_TASK *tk=NULL;
	    AE_JOB *jPayment;
	    GWEN_DATE *dt;
	    AQDB_VALUE *v;

	    tid=AE_Job_GetTaskId(j);
	    assert(tid);

	    rv=AE_Book_ReadTask(b, AE_Book_TableType_Task, tid, &tk);
	    if (rv<0) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error while reading selected task (error: %d)"),
				  rv);
	      AE_Job_free(j);
	      return 1;
	    }

	    rv=AE_Book_ReadTaskSubObjects(b, tk, 1); /* outgoing */
	    if (rv<0) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error while reading sub-objects of the selected task (error: %d)"),
				  rv);
	      AE_Task_free(tk);
	      return 1;
	    }
	    j=AE_Job_List_GetById(AE_Task_GetJobs(tk), invoiceId);
	    if (j==NULL) {
	      DBG_ERROR(0, "Internal error");
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  "%s",
				  I18N("An internal error occurred: The task doesn't contain the given job."));
	      AE_Task_free(tk);
	      return 1;
	    }

	    jPayment=AE_Job_new();
            AE_Job_SetTaskId(jPayment, tid);
	    AE_Job_SetReferenceId(jPayment, invoiceId);
	    AE_Job_SetType(jPayment, AE_Job_TypePayment);
	    AE_Job_SetId(jPayment, 0);
	    AE_Job_SetDbId(jPayment, 0);
	    AE_Job_SetPartnerId(jPayment, AE_Job_GetPartnerId(j));
	    AE_Job_SetCommodityId(jPayment, AE_Job_GetCommodityId(j));

	    v=AE_Transaction_CalculateAmount(t, AE_Job_GetCommodityId(j));
	    if (v) {
	      AE_ITEM *item;

	      /* create an item for the open amont */
	      item=AE_Item_new();
	      AE_Item_SetSellPriceValue(item, v);
	      AE_Item_SetCommodityId(item, AE_Job_GetCommodityId(j));
	      AE_Item_List_Add(item, AE_Job_GetItems(jPayment));
              AQDB_Value_free(v);
	    }

	    dt=GWEN_Date_CurrentDate();
	    AE_Job_SetEnterDate(jPayment, dt);
	    AE_Job_SetDueDate(jPayment, dt);
	    GWEN_Date_free(dt);
	    AE_Job_SetType(jPayment, AE_Job_TypePayment);

	    if (FF_EditPayment::editPayment(app, this,
					    I18N("Create Payment for Outgoing Invoice"),
					    jPayment,
					    0)) { /* !isBuy */
	      rv=AE_Book_BeginEdit(b, 0);
	      if (rv<0) {
		DBG_ERROR(0, "Error on begin edit: %d", rv);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Could not lock database for write access (error: %d)"),
				    rv);
		AE_Task_free(tk);
		return 1;
	      }

	      AE_Task_SetWaitingFor(tk, AE_Job_TypeNone);

	      DBG_ERROR(0, "Writing task");
	      rv=AE_Book_WriteTask(b, AE_Book_TableType_Task, tk, 0);
	      if (rv<0) {
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Error while writing task (error: %d)"),
				    rv);
		AE_Task_free(tk);
		return 1;
	      }

	      AE_Job_SetTransactionId(jPayment, AE_Transaction_GetId(t));
	      rv=AE_Book_ApplyJob(b, jPayment, NULL, 0);
	      if (rv<0) {
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("The selected job could not be written (error: %d)"),
				    rv);
		AE_Task_free(tk);
		return 1;
	      }
	      AE_Task_free(tk);

	      rv=AE_Book_EndEdit(b, 0);
	      if (rv<0) {
		DBG_ERROR(0, "here (%d)", rv);
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Could not apply changes to the database (error: %d)"),
				    rv);
		return 1;
	      }

	      app->signalDbReload(AE_Book_TableType_Job);
	      app->signalDbReload(AE_Book_TableType_Task);
	      return 1;
	    }
	  }
	}
	else {
	  /* aborted */
	  return 1;
	}
      }
    }
  }

  return 1;
}



long FM_AccountStmPage::onCmdCreateOutgoingPayment(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TRANSACTION *t;

    t=m_transTable->getCurrentTransaction();
    if (t) {
      int rv;
      AQDB_ID invoiceId=0;
      AE_JOB *j=NULL;

      while(1) {
	if (FF_SelectJob::selectJob(app, this,
				    I18N("Select Outgoing Invoice"),
				    I18N("Please select the outgoing invoice for which you want to "
					 "create a payment."),
				    invoiceId,
				    FF_SelectJob::JobSelection_Buys)) {
	  if (invoiceId==0) {
	  }

	  rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, invoiceId, &j);
	  if (rv<0) {
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Database Error"),
				I18N("The selected job could not be read (error: %d)"),
				rv);
	    return 1;
	  }

	  if (AE_Job_GetType(j)!=AE_Job_TypeInvoice) {
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Input Error"),
				"%s",
				I18N("The selected job is not an invoice. "
				     "Please select an invoice."));
	  }
	  else {
	    AQDB_ID tid=0;
	    AE_TASK *tk=NULL;
	    AE_JOB *jPayment;
	    GWEN_DATE *dt;
	    AQDB_VALUE *v;

	    tid=AE_Job_GetTaskId(j);
	    assert(tid);

	    rv=AE_Book_ReadTask(b, AE_Book_TableType_Task, tid, &tk);
	    if (rv<0) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error while reading selected task (error: %d)"),
				  rv);
	      AE_Job_free(j);
	      return 1;
	    }

	    rv=AE_Book_ReadTaskSubObjects(b, tk, 1); /* outgoing */
	    if (rv<0) {
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error while reading sub-objects of the selected task (error: %d)"),
				  rv);
	      AE_Task_free(tk);
	      return 1;
	    }
	    j=AE_Job_List_GetById(AE_Task_GetJobs(tk), invoiceId);
	    if (j==NULL) {
	      DBG_ERROR(0, "Internal error");
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  "%s",
				  I18N("An internal error occurred: The task doesn't contain the given job."));
	      AE_Task_free(tk);
	      return 1;
	    }

	    jPayment=AE_Job_new();
            AE_Job_SetTaskId(jPayment, tid);
	    AE_Job_SetReferenceId(jPayment, invoiceId);
	    AE_Job_SetType(jPayment, AE_Job_TypePayment);
	    AE_Job_SetId(jPayment, 0);
	    AE_Job_SetDbId(jPayment, 0);
	    AE_Job_SetPartnerId(jPayment, AE_Job_GetPartnerId(j));
	    AE_Job_SetCommodityId(jPayment, AE_Job_GetCommodityId(j));

	    v=AE_Transaction_CalculateAmount(t, AE_Job_GetCommodityId(j));
	    if (v) {
	      AE_ITEM *item;

	      /* create an item for the open amont */
	      item=AE_Item_new();
	      AE_Item_SetBuyPriceValue(item, v);
	      AE_Item_SetCommodityId(item, AE_Job_GetCommodityId(j));
	      AE_Item_List_Add(item, AE_Job_GetItems(jPayment));
              AQDB_Value_free(v);
	    }

	    dt=GWEN_Date_CurrentDate();
	    AE_Job_SetEnterDate(jPayment, dt);
	    AE_Job_SetDueDate(jPayment, dt);
	    GWEN_Date_free(dt);
	    AE_Job_SetType(jPayment, AE_Job_TypePayment);

	    if (FF_EditPayment::editPayment(app, this,
					    I18N("Create Payment for Incoming Invoice"),
					    jPayment,
					    1)) { /* isBuy */
	      rv=AE_Book_BeginEdit(b, 0);
	      if (rv<0) {
		DBG_ERROR(0, "Error on begin edit: %d", rv);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Could not lock database for write access (error: %d)"),
				    rv);
		AE_Task_free(tk);
		return 1;
	      }

	      AE_Task_SetWaitingFor(tk, AE_Job_TypeNone);

	      DBG_ERROR(0, "Writing task");
	      rv=AE_Book_WriteTask(b, AE_Book_TableType_Task, tk, 0);
	      if (rv<0) {
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Error while writing task (error: %d)"),
				    rv);
		AE_Task_free(tk);
		return 1;
	      }

	      AE_Job_SetTransactionId(jPayment, AE_Transaction_GetId(t));
	      rv=AE_Book_ApplyJob(b, jPayment, NULL, 0);
	      if (rv<0) {
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("The selected job could not be written (error: %d)"),
				    rv);
		AE_Task_free(tk);
		return 1;
	      }
	      AE_Task_free(tk);

	      rv=AE_Book_EndEdit(b, 0);
	      if (rv<0) {
		DBG_ERROR(0, "here (%d)", rv);
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Could not apply changes to the database (error: %d)"),
				    rv);
		return 1;
	      }

	      app->signalDbReload(AE_Book_TableType_Job);
	      app->signalDbReload(AE_Book_TableType_Task);
	      return 1;
	    }
	  }
	}
	else {
	  /* aborted */
	  return 1;
	}
      }
    }
  }

  return 1;
}



long FM_AccountStmPage::onCmdMoveToAccount(FXObject*, FXSelector, void* ptr) {
  AE_TRANSACTION_LIST2 *tl;
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  tl=AE_Transaction_List2_new();
  m_transTable->getSelectedTransactions(tl, FF_TransactionTable::COPYMODE_NONE);
  if (AE_Transaction_List2_GetSize(tl)) {
    AQDB_ID aid=0;

    if (FF_SelectAccount::selectAccount(app, this,
					I18N("Select Target Account"),
					aid)) {
      if (aid) {
	AE_TRANSACTION_LIST2_ITERATOR *it;
	int rv;
  
	rv=AE_Book_BeginEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "Error on begin edit: %d", rv);
	  FXMessageBox::error(this, FX::MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not lock database for write access (error: %d)"),
			      rv);
	  AE_Transaction_List2_free(tl);
	  return 1;
	}
  
	it=AE_Transaction_List2_First(tl);
	if (it) {
	  AE_TRANSACTION *t;
  
	  t=AE_Transaction_List2Iterator_Data(it);
	  while(t) {
	    AE_SPLIT *sp;
	    AE_TRANSACTION *cpy;
  
	    cpy=AE_Transaction_dup(t);
	    sp=AE_Split_List_First(AE_Transaction_GetSplitList(cpy));
	    while(sp) {
	      if (AE_Split_GetAccountId(sp)==m_accountId) {
		AE_Split_SetAccountId(sp, aid);
		AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MODIFIED);
		AE_Transaction_AddRuntimeFlags(cpy,
					       AE_TRANSACTION_RTFLAGS_MODIFIED |
					       AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
	      }
	      sp=AE_Split_List_Next(sp);
	    }
  
	    if (AE_Transaction_GetRuntimeFlags(cpy) & AE_TRANSACTION_RTFLAGS_MODIFIED) {
	      rv=AE_Book_ApplyTransaction(b, cpy, t, 0);
	      if (rv<0) {
		DBG_ERROR(0, "here (%d)", rv);
		AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
		AE_Transaction_free(cpy);
		AE_Transaction_List2Iterator_free(it);
		AE_Transaction_List2_free(tl);
		FXMessageBox::error(this, FX::MBOX_OK,
				    I18N("Database Error"),
				    I18N("Could not apply changes to the database (error: %d)"),
				    rv);
		return 1;
	      }
	    }
	    AE_Transaction_free(cpy);
  
	    t=AE_Transaction_List2Iterator_Next(it);
	  }
	  AE_Transaction_List2Iterator_free(it);
	}
  
	rv=AE_Book_EndEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, FX::MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not apply changes to the database (error: %d)"),
			      rv);
	  AE_Transaction_List2_free(tl);
	  return 1;
	}
      } /* if aid */
    } /* if selectAccount */
  }
  AE_Transaction_List2_free(tl);

  app->signalDbReload(AE_Book_TableType_Transaction);
  app->signalDbReload(AE_Book_TableType_AccountBalance);

  return 1;
}



long FM_AccountStmPage::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  rebuild();
  return 1;
}



void FM_AccountStmPage::reset() {
}



int FM_AccountStmPage::readAccountSplits(AE_SPLIT_LIST *splits) {
  FF_App *app;
  int rv;
  char qexpr[256];

  app=dynamic_cast<FF_App*>(getApp());

  /* read splits for this account */
  snprintf(qexpr, sizeof(qexpr)-1,
           "$accountId==%d",
           m_accountId);
  rv=AE_Book_QuerySplitList(app->getBook(), AE_Book_TableType_Split, 0, qexpr, splits);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int FM_AccountStmPage::readAccountTransactions(AE_SPLIT_IDMAP *sMap,
                                               AE_TRANSACTION_LIST *transactions,
                                               AE_TRANSACTION_IDMAP *tMap) {
  FF_App *app;
  AE_BOOK *b;
  AQDB_ID qid=0;
  AQDB_DB *db;
  AQDB_ID tid;
  int rv;
  AQDB_ID oids[16];
  int len;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  db=AE_Book_GetDataBase(b);
  tid=AE_Book_GetTableId(b, AE_Book_TableType_Transaction);

  /* query every transaction */
  rv=AQDB_DB_QuerySubmit(db, tid, 0, NULL, 0, &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* browse all objects */
  len=16;
  rv=AQDB_DB_QueryGetFirst(db, qid, oids, &len);
  while(rv==0) {
    int i;

    for (i=0; i<len; i++) {
      AQDB_OBJECT *o=NULL;
      AE_TRANSACTION *t=NULL;
      AQDB_ID transId;
      AE_SPLIT *sp;

      rv=AQDB_DB_ReadObject(db, tid, oids[i], &o);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_DB_QueryClose(db, qid);
        return rv;
      }

      rv=AE_Transaction_fromObject(o, &t);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_Object_free(o);
        AQDB_DB_QueryClose(db, qid);
        return rv;
      }
      AE_Transaction_SetDbId(t, AQDB_Object_GetObjectId(o));

      /* find split to which the transaction belongs */
      transId=AE_Transaction_GetId(t);
      sp=AE_Split_IdMap_Find(sMap, transId);
      if (sp==NULL)
        /* no split is part of that transaction, free it */
        AE_Transaction_free(t);
      else {
	/* otherwise add to the list */
	AE_Transaction_List_Add(t, transactions);
	AE_Transaction_IdMap_Insert(tMap, transId, t);
      }
      AQDB_Object_free(o);
    }

    len=16;
    rv=AQDB_DB_QueryGetNext(db, qid, oids, &len);
  }
  AQDB_DB_QueryClose(db, qid);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



int FM_AccountStmPage::readTransactionSplits(AE_TRANSACTION_IDMAP *tMap) {
  FF_App *app;
  AE_BOOK *b;
  AQDB_ID qid=0;
  AQDB_DB *db;
  AQDB_ID tid;
  int rv;
  AQDB_ID oids[16];
  int len;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  db=AE_Book_GetDataBase(b);
  tid=AE_Book_GetTableId(b, AE_Book_TableType_Split);

  /* query */
  rv=AQDB_DB_QuerySubmit(db, tid, 0, NULL, 0, &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* browse all objects */
  len=16;
  rv=AQDB_DB_QueryGetFirst(db, qid, oids, &len);
  while(rv==0) {
    int i;

    for (i=0; i<len; i++) {
      AQDB_OBJECT *o=NULL;
      AE_SPLIT *sp=NULL;
      AQDB_ID transId;
      AE_TRANSACTION *t;

      rv=AQDB_DB_ReadObject(db, tid, oids[i], &o);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_DB_QueryClose(db, qid);
        return rv;
      }

      rv=AE_Split_fromObject(o, &sp);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_Object_free(o);
        AQDB_DB_QueryClose(db, qid);
        return rv;
      }
      AE_Split_SetDbId(sp, AQDB_Object_GetObjectId(o));

      /* find transaction to which the split belongs */
      transId=AE_Split_GetTransactionId(sp);
      t=AE_Transaction_IdMap_Find(tMap, transId);
      if (t) {
        AE_Transaction_AddSplit(t, sp);
        if (AE_Split_List_GetCount(AE_Transaction_GetSplitList(t))>2)
          AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTISPLIT);
      }
      else
        AE_Split_free(sp);

      AQDB_Object_free(o);
    }

    len=16;
    rv=AQDB_DB_QueryGetNext(db, qid, oids, &len);
  }
  AQDB_DB_QueryClose(db, qid);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



void FM_AccountStmPage::reload() {
  FF_App *app;
  AE_BOOK *b;
  AE_SPLIT_LIST *splits;
  AE_SPLIT_IDMAP *sMap;
  AE_SPLIT *sp;
  AE_TRANSACTION_LIST *transactions;
  AE_TRANSACTION_IDMAP *tMap;
  int rv;
  FXString str;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  AE_Account_free(m_account);
  m_account=NULL;

  AE_Commodity_free(m_accountCommodity);
  m_accountCommodity=NULL;

  m_transTable->clearItems();
  m_transTable->setMarkBeyondId(m_parent->getTransactionMarkId());
  m_transTable->reload();

  /* read account */
  if (m_accountId) {
    AQDB_ID id;

    rv=AE_Book_ReadAccount(b, AE_Book_TableType_Account, m_accountId, &m_account);
    if (rv<0) {
      FXMessageBox::error(this, FX::MBOX_OK,
			  I18N("Database Error"),
			  I18N("Could not read account from database (error: %d)"),
			  rv);
      return;
    }

    id=AE_Account_GetPrimaryCommodityId(m_account);
    if (id) {
      /* read primary commodity */
      rv=AE_Book_ReadCommodity(b, AE_Book_TableType_Commodity, id, &m_accountCommodity);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	FXMessageBox::error(this, FX::MBOX_OK,
			    I18N("Database Error"),
			    I18N("Could not read commodity (error: %d)"),
			    rv);
	return;
      }
    }
  }

  /* read splits of this account */
  splits=AE_Split_List_new();
  rv=readAccountSplits(splits);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AE_Split_List_free(splits);
    return;
  }

  /* store split by transactionId in map for faster access */
  sMap=AE_Split_IdMap_new(GWEN_IdMapAlgo_Hex4);
  sp=AE_Split_List_First(splits);
  while(sp) {
    AE_Split_IdMap_Insert(sMap, AE_Split_GetTransactionId(sp), sp);
    sp=AE_Split_List_Next(sp);
  }

  /* read transactions which have a split in this account */
  transactions=AE_Transaction_List_new();
  tMap=AE_Transaction_IdMap_new(GWEN_IdMapAlgo_Hex4);
  rv=readAccountTransactions(sMap, transactions, tMap);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AE_Transaction_List_free(transactions);
    AE_Split_List_free(splits);
    return;
  }
  AE_Split_IdMap_free(sMap);
  AE_Split_List_free(splits);

  /* read splits for every transaction */
  rv=readTransactionSplits(tMap);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AE_Transaction_List_free(transactions);
    AE_Transaction_IdMap_free(tMap);
    return;
  }

  AE_Transaction_List_free(m_transactions);
  m_transactions=transactions;
  AE_Transaction_IdMap_free(tMap);

  rebuild();
}



void FM_AccountStmPage::rebuild() {
  m_transTable->clearItems();
  if (m_transactions) {
    AE_TRANSACTION *t;
    FXString str;
    int i;

    str=m_matcher->getText().simplify().trim();
    AE_Transaction_List_SelectMatching(m_transactions, str.text());

    i=0;
    t=AE_Transaction_List_First(m_transactions);
    while(t) {
      if (AE_Transaction_GetRuntimeFlags(t) & AE_TRANSACTION_RTFLAGS_SELECTED) {
        AE_Transaction_Attach(t);
	m_transTable->addTransaction(t);
	i++;
      }
      t=AE_Transaction_List_Next(t);
    }

    str=FXStringFormat(I18N("%d transactions (of %d)"), i, AE_Transaction_List_GetCount(m_transactions));
    m_fLabel->setText(str);

    /* sort table */
    m_transTable->sort();

    if (m_account) {

    }

    layout();
  }
}




