/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_BANKACCSPLITPAGE_HPP
#define FM_BANKACCSPLITPAGE_HPP



#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>


class FF_TransactionTable;
class FOX16_HtmlLabel;
class FM_AccountPage;


class FM_AccountStmPage: public FXVerticalFrame {
  FXDECLARE(FM_AccountStmPage)
public:
  enum {
    ID_SPLIT_TABLE=FXPacker::ID_LAST,
    ID_CMD_DELSPLIT,
    ID_CMD_ASSIGN_TO_BUY_PAYMENT,
    ID_CMD_ASSIGN_TO_SELL_PAYMENT,
    ID_CMD_CREATE_BUY_PAYMENT,
    ID_CMD_CREATE_SELL_PAYMENT,
    ID_CMD_MOVE_TO_ACCOUNT,
    ID_EDIT_SPLITS,
    ID_MATCHER,
    ID_CMD_ADDTRANS,
    ID_LAST
  };


  FM_AccountStmPage(FM_AccountPage *page,
                    const AE_ACCOUNT *acc,
                    FXComposite *p,
                    FXuint opts=0,
                    FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_AccountStmPage();

  AQDB_ID getBankAccountId() const { return m_accountId;};

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void aboutToSendJobs();

  void enter();
  void leave();
  void reset();

  void create();

  long onCmdTable(FXObject*, FXSelector, void* ptr);
  long onTableRightButtonRelease(FXObject*, FXSelector, void* ptr);
  long onTableContentReplaced(FXObject*, FXSelector, void* ptr);
  long onCmdAddStm(FXObject*, FXSelector, void* ptr);
  long onCmdDelStm(FXObject*, FXSelector, void* ptr);
  long onCmdSplitEditor(FXObject*, FXSelector, void *ptr);

  long onCmdAssignToIncomingPayment(FXObject*, FXSelector, void *ptr);
  long onCmdAssignToOutgoingPayment(FXObject*, FXSelector, void *ptr);

  long onCmdCreateIncomingPayment(FXObject*, FXSelector, void *ptr);
  long onCmdCreateOutgoingPayment(FXObject*, FXSelector, void *ptr);

  long onCmdMoveToAccount(FXObject*, FXSelector, void* ptr);

  long onChgMatcher(FXObject*, FXSelector, void* ptr);

protected:
  FM_AccountStmPage();
  void rebuild();
  void reload();

  int readAccountSplits(AE_SPLIT_LIST *splits);
  int readAccountTransactions(AE_SPLIT_IDMAP *sMap,
                              AE_TRANSACTION_LIST *transactions,
                              AE_TRANSACTION_IDMAP *tMap);
  int readTransactionSplits(AE_TRANSACTION_IDMAP *tMap);

  FM_AccountPage *m_parent;

  AQDB_ID m_accountId;
  FOX16_HtmlLabel *m_hLabel;
  FOX16_HtmlLabel *m_fLabel;
  FF_TransactionTable *m_transTable;

  AE_ACCOUNT *m_account;
  AE_COMMODITY *m_accountCommodity;

  bool m_needReload;
  bool m_isVisible;
  bool m_blockReload;

  FXTextField *m_matcher;
  FXString m_tempMatcherString;

  AE_TRANSACTION_LIST *m_transactions;

};



#endif
