/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_BANKACCACCPAGE_HPP
#define FM_BANKACCACCPAGE_HPP


#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <aqbanking/job.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_BankAccountTree;
class FM_BankAccountPage;


class FM_BankAccountAccPage: public FXHorizontalFrame {
  FXDECLARE(FM_BankAccountAccPage)

public:
  enum {
    ID_ACCOUNT_TREE=FXPacker::ID_LAST,
    ID_BUTTON_ONLINE1,
    ID_BUTTON_ONLINE2,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_BUTTON_ADDACCOUNT,
    ID_BUTTON_EDITACCOUNT,
    ID_BUTTON_DELACCOUNT,

    ID_ASSIGN_BANKING_ACCOUNT,
    ID_IMPORT_BANKING_ACCOUNTS,

    ID_LAST
  };

public:
  FM_BankAccountAccPage(FM_BankAccountPage *mainPage,
                        FXComposite *p,
                        FXuint opts=0,
                        FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_BankAccountAccPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void aboutToSendJobs();

  void jobListUpdated();

  void enter();
  void leave();

  void reset();

  void create();

  long onCmdOnlineUpdate1(FXObject*, FXSelector, void*);
  long onCmdOnlineUpdate2(FXObject*, FXSelector, void*);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onCmdAddAccount(FXObject*, FXSelector, void*);
  long onCmdEditAccount(FXObject*, FXSelector, void*);
  long onCmdDelAccount(FXObject*, FXSelector, void*);

  long onCmdAssignBankingAccount(FXObject*, FXSelector, void* ptr);
  long onCmdImportBankingAccounts(FXObject*, FXSelector, void* ptr);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeCommand(FXObject*, FXSelector, void* ptr);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

protected:
  FM_BankAccountAccPage();

  bool getLatestStatementDate(AQDB_ID aid, FXDate &dt);
  bool queueHasJobForAccount(uint32_t uid, AB_JOB_TYPE jt);

  FM_BankAccountPage *m_mainPage;

  FF_BankAccountTree *m_bankAccountTree;
  FXButton *m_onlineUpdateButton;
};




#endif

