/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_BANKACCPAGE_HPP
#define FM_BANKACCPAGE_HPP



#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>



class FM_BankAccountAccPage;
class FM_BankAccountStmPage;


class FM_BankAccountPage: public FXHorizontalFrame {
  FXDECLARE(FM_BankAccountPage)
public:
  enum {
    ID_TABBOOK=FXHorizontalFrame::ID_LAST,
    ID_TABITEM,
    ID_LAST
  };

  FM_BankAccountPage(FXComposite *p,
		     FXuint opts=0,
		     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_BankAccountPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void aboutToSendJobs();

  void jobListUpdated();

  AQDB_ID getStatementMarkId() const { return m_statementMarkId;};

  void enter();
  void leave();

  void reset();

  void create();

  void openAccountPage(const AE_BANKACCOUNT *ba);

  long onTabClicked(FXObject*, FXSelector, void*);
  long onTabLeftButtonPressed(FXObject *obj, FXSelector, void *ptr);

protected:
  FM_BankAccountPage();

  FM_BankAccountStmPage *findPageForAccount(AQDB_ID aid);
  int findIndexForAccount(AQDB_ID aid);
  FM_BankAccountStmPage *findPageAt(int idx);

  FXTabBook *m_tabBook;
  FM_BankAccountAccPage *m_bankAccountView;
  int m_lastEnteredTab;
  std::list<FM_BankAccountStmPage*> m_accountPages;

  GWEN_DB_NODE *m_config;

  AQDB_ID m_statementMarkId;
  FXIcon *m_closeTabIcon;
};



#endif
