/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_bankaccstmpage.hpp"
#include "fm_bankaccpage.hpp"
#include "ff_statementtable.hpp"
#include "ff_app.hpp"
#include "ff_editpartner.hpp"

#include <aqfinance/engine/modules/ae_statementimport.h>

#include <aqfinance/graphics/af_graphics_cairo.h>
#include <aqfinance/graphs/af_graph_dataset.h>
#include <aqfinance/graphs/af_graph.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/directory.h>

#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define GRAPH_HEIGHT 150



FXDEFMAP(FM_BankAccountStmPage) FM_BankAccountStmPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountStmPage::ID_STM_TABLE,
            FM_BankAccountStmPage::onCmdTable),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FM_BankAccountStmPage::ID_STM_TABLE,
            FM_BankAccountStmPage::onTableRightButtonRelease),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountStmPage::ID_CMD_DELSTM,
            FM_BankAccountStmPage::onCmdDelStm),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountStmPage::ID_CMD_MKPARTNER,
            FM_BankAccountStmPage::onCmdMkPartner),
  FXMAPFUNC(SEL_REPLACED, FM_BankAccountStmPage::ID_STM_TABLE,
            FM_BankAccountStmPage::onTableContentReplaced),
  FXMAPFUNC(SEL_SELECTED, FM_BankAccountStmPage::ID_STM_TABLE,
            FM_BankAccountStmPage::onTableSelectionChanged),

  FXMAPFUNC(SEL_DOCKED, FM_BankAccountStmPage::ID_STM_TABLE,
            FM_BankAccountStmPage::onTableDocked),

  FXMAPFUNC(SEL_CHANGED, FM_BankAccountStmPage::ID_MATCHER,
            FM_BankAccountStmPage::onChgMatcher),
};


            
FXIMPLEMENT(FM_BankAccountStmPage, FXVerticalFrame,
            FM_BankAccountStmPageMap, ARRAYNUMBER(FM_BankAccountStmPageMap))



FM_BankAccountStmPage::FM_BankAccountStmPage(FM_BankAccountPage *page,
                                             const AE_BANKACCOUNT *ba,
                                             FXComposite *p,
                                             FXuint opts,
                                             FXint x, FXint y, FXint w, FXint h)
:FXVerticalFrame(p, opts, x, y, w, h)
,m_parent(page)
,m_bankAccountId(AE_BankAccount_GetId(ba))
,m_hLabel(NULL)
,m_statementTable(NULL)
,m_needRebuild(true)
,m_isVisible(false)
,m_imageFrame(NULL)
,m_image(NULL)
{
  FF_App *app;
  FXString txt;
  const char *s1;
  const char *s2;
  FXHorizontalFrame *hf;

  app=dynamic_cast<FF_App*>(getApp());

  flags|=FLAG_SHOWN | FLAG_ENABLED;

  s1=AE_BankAccount_GetBankName(ba);
  s2=AE_BankAccount_GetAccountName(ba);
  if (s1 && s2) {
    char tbuf[256];

    snprintf(tbuf, sizeof(tbuf)-1, I18N("<font size=+1><b>%s at %s</b></font>"), s2, s1);
    tbuf[sizeof(tbuf)-1]=0;
    txt=FXString(tbuf);
  }
  else if (s1) {
    char tbuf[256];

    snprintf(tbuf, sizeof(tbuf)-1, I18N("<font size=+1><b>Account at %s</b></font>"), s1);
    tbuf[sizeof(tbuf)-1]=0;
    txt=FXString(tbuf);
  }
  else if (s2) {
    char tbuf[256];

    snprintf(tbuf, sizeof(tbuf)-1, I18N("<font size=+1><b>Account %s</b></font>"), s2);
    tbuf[sizeof(tbuf)-1]=0;
    txt=FXString(tbuf);
  }
#if 0
  s1=AE_BankAccount_GetBankCode(ba);
  if (s1) {
    txt+=I18N(" Bank Code:<i>");
    txt+=s1;
    txt+="</i>";
  }
  s1=AE_BankAccount_GetAccountNumber(ba);
  if (s1) {
    txt+=I18N(" Account Number:<i>");
    txt+=s1;
    txt+="</i>";
  }
  s1=AE_BankAccount_GetBic(ba);
  if (s1) {
    txt+=I18N(" SWIFT BIC:<i>");
    txt+=s1;
    txt+="</i>";
  }
  s1=AE_BankAccount_GetIban(ba);
  if (s1) {
    txt+=I18N(" IBAN:<i>");
    txt+=s1;
    txt+="</i>";
  }
  txt+="<br>";
#endif
  m_accountHeader=txt;

  m_hLabel=new FOX16_HtmlLabel(this, "",
			       LAYOUT_FILL_X | LAYOUT_MIN_HEIGHT |
			       FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  //m_hLabel->setMaxDefaultWidth(600);

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  m_statementTable=new FF_StatementTable(this, this, ID_STM_TABLE,
					 FXXTable::TABLE_TRACK_SCROLLBARS |
                                         LAYOUT_FILL_X | LAYOUT_FILL_Y|
                                         FRAME_SUNKEN | FRAME_THICK);
  m_statementTable->setShowGrid(false);
  m_statementTable->setCellBackColor(0, 0, FXRGB(183, 220, 159));
  m_statementTable->setCellBackColor(0, 1, FXRGB(183, 220, 159));
  m_statementTable->setCellBackColor(1, 0, FXRGB(255, 255, 255));
  m_statementTable->setCellBackColor(1, 1, FXRGB(255, 255, 255));

  m_statementTable->addColumn(FF_StatementTable::COLUMN_TYPE_DATE, 100);
  m_statementTable->addColumn(FF_StatementTable::COLUMN_TYPE_MEMO, 100);
  m_statementTable->addColumn(FF_StatementTable::COLUMN_TYPE_TYPE, 100);
  m_statementTable->addColumn(FF_StatementTable::COLUMN_TYPE_VALUE, 100);

  new FXHorizontalSeparator(this, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  m_fLabel=new FOX16_HtmlLabel(this, "", LAYOUT_FILL_X | LAYOUT_MIN_HEIGHT | FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);
  m_fLabel->setMaxDefaultWidth(600);

  if (app->getShowBankAccountGraphs())
    m_imageFrame=new FXImageFrame(this, NULL,
                                  FRAME_SUNKEN|FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FIX_HEIGHT,
                                  0, 0, 0, GRAPH_HEIGHT);
}



FM_BankAccountStmPage::FM_BankAccountStmPage()
:FXVerticalFrame()
,m_bankAccountId(0)
,m_hLabel(NULL)
,m_statementTable(NULL)
,m_imageFrame(NULL)
,m_image(NULL)
{
}



FM_BankAccountStmPage::~FM_BankAccountStmPage() {
  if (m_imageFrame) {
    delete m_imageFrame;
    m_imageFrame=NULL;
  }

  if (m_image) {
    delete m_image;
    m_image=NULL;
  }

}



void FM_BankAccountStmPage::create() {
  FXVerticalFrame::create();
  m_hLabel->create();
  m_statementTable->create();
}




void FM_BankAccountStmPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbStmPage;
  GWEN_DB_NODE *dbT=NULL;
  char numbuf[32];

  DBG_INFO(0, "Init");
  m_hLabel->setText(m_accountHeader);

  snprintf(numbuf, sizeof(numbuf)-1, "acc-%x", m_bankAccountId);
  numbuf[sizeof(numbuf)-1]=0;
  dbStmPage=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "statementPage");
  if (dbStmPage) {
    dbT=GWEN_DB_GetGroup(dbStmPage, GWEN_PATH_FLAGS_NAMEMUSTEXIST, numbuf);
    if (dbT==NULL)
      dbT=GWEN_DB_GetGroup(dbStmPage, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "last");
  }
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_statementTable->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_statementTable->getHeaderSize(i));
      m_statementTable->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_statementTable->setHeaderArrowDir(i, TRUE);
        else
          m_statementTable->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_BankAccountStmPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbStmPage;
  GWEN_DB_NODE *dbT;
  GWEN_DB_NODE *dbLast;
  int i;
  char numbuf[32];

  DBG_INFO(0, "Fini");
  snprintf(numbuf, sizeof(numbuf)-1, "acc-%x", m_bankAccountId);
  numbuf[sizeof(numbuf)-1]=0;
  dbStmPage=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "statementPage");
  dbT=GWEN_DB_GetGroup(dbStmPage, GWEN_DB_FLAGS_OVERWRITE_GROUPS, numbuf);

  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_statementTable->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_statementTable->getHeaderSize(i));
    b=m_statementTable->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  dbLast=GWEN_DB_GetGroup(dbStmPage, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "last");
  GWEN_DB_AddGroupChildren(dbLast, dbT);
}



void FM_BankAccountStmPage::enter() {
  m_isVisible=true;

  layout();
  if (m_needRebuild) {
    rebuild();
    m_needRebuild=false;
  }
}



void FM_BankAccountStmPage::leave() {
  m_isVisible=false;
}



void FM_BankAccountStmPage::dbOpened(AE_BOOK *bk) {
  if (m_isVisible) {
    rebuild();
    m_needRebuild=false;
  }
  else
    m_needRebuild=true;
}



void FM_BankAccountStmPage::dbClosing() {
  m_statementTable->clearItems();
}



void FM_BankAccountStmPage::dbClosed() {
}



void FM_BankAccountStmPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_BankStatement ||
      tt==AE_Book_TableType_Partner ||
      tt==AE_Book_TableType_None) {
    if (m_isVisible) {
      rebuild();
      m_needRebuild=false;
    }
    else
      m_needRebuild=true;
  }
}



void FM_BankAccountStmPage::aboutToSendJobs() {
}



long FM_BankAccountStmPage::onCmdTable(FXObject*, FXSelector, void* ptr) {
#if 0
  int r=m_statementTable->getCurrentRow();
  if (r>=0) {
    AE_STATEMENT *st=m_statementTable->getStatementByRow(r);
    AE_Statement_Dump(st, stderr, 2);
  }
#endif
  return 1;
}



long FM_BankAccountStmPage::onTableRightButtonRelease(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  int r;
  AE_STATEMENT *st=NULL;

  r=m_statementTable->getCurrentRow();
  if (r>=0)
    st=m_statementTable->getCurrentStatement();

  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    c=new FXMenuCommand(&pmenu, I18N("Delete Statement"), NULL, this, ID_CMD_DELSTM);
    if (st==NULL)
      c->disable();
    c=new FXMenuCommand(&pmenu, I18N("Create Partner for Statement"), NULL, this, ID_CMD_MKPARTNER);
    if (st==NULL)
      c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FM_BankAccountStmPage::onCmdDelStm(FXObject*, FXSelector, void* ptr) {
  int r;

  r=m_statementTable->getCurrentRow();
  if (r>=0) {
    AE_STATEMENT *st;

    st=m_statementTable->getCurrentStatement();
    if (st) {
      FF_App *app;
      AE_BOOK *b;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
        FXuint res;

        res=FXMessageBox::question(this, MBOX_YES_NO,
                                   I18N("Delete Statement"),
				   "%s",
                                   I18N("Are you sure you want to delete the selected "
					"statement?"));
        if (res!=MBOX_CLICKED_YES) {
          DBG_ERROR(0, "Aborted");
        }
        else {
          int rv;

          rv=AE_Book_DeleteStatement(b, AE_Book_TableType_BankStatement, st, 1);
          if (rv<0) {
            DBG_ERROR(0, "Unable to delete bank statement (%d)", rv);
          }
          else {
            m_statementTable->delRow(r);
            app->signalDbReload(AE_Book_TableType_BankStatement);
          }
        }
      }
    }
  }

  return 1;
}



long FM_BankAccountStmPage::onCmdMkPartner(FXObject*, FXSelector, void* ptr) {
  int r;

  r=m_statementTable->getCurrentRow();
  if (r>=0) {
    AE_STATEMENT *st;

    st=m_statementTable->getCurrentStatement();
    if (st) {
      FF_App *app;
      AE_BOOK *b;
      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
	AE_PARTNER *pt=NULL;
	int rv;

	if (AE_Statement_GetPartnerId(st)!=0) {
	  FXMessageBox::information(this, FX::MBOX_OK,
				    I18N("Partner Exists"),
				    "%s",
				    I18N("There already is a partner assigned to the given statement"));
	  return 1;
	}

	rv=AE_StatementImport_GetPartnerForStatementSimple(b,
                                                           app->getBanking(),
                                                           AE_Book_TableType_Partner,
							   st,
							   &pt,
							   1);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  if (rv==GWEN_ERROR_BAD_DATA) {
	    DBG_INFO(AE_LOGDOMAIN, "Not enough data to create a partner");
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Data Error"),
				"%s",
				I18N("There is not enough information in the given statement "
				     "to create a partner"));
            return 1;
	  }
	  else {
	    FXMessageBox::error(this, FX::MBOX_OK,
				I18N("Database Error"),
				I18N("Could not create a partner for this statement (error: %d)"),
				rv);
	    return 1;
	  }
	}

	if (FF_EditPartner::editPartner(app, this, I18N("Edit Partner"), pt)) {
	  if (AE_Partner_GetId(pt)==0) {
	    /* create partner in DB */
	    rv=AE_Book_ApplyPartner(b, pt, NULL, 1);
	    if (rv<0) {
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	      AE_Partner_free(pt);
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      return rv;
	    }
	  }
	  else if (AE_Partner_GetRuntimeFlags(pt) & AE_PARTNER_RTFLAGS_MODIFIED) {
	    AE_PARTNER *ptOrig=NULL;

            /* lock database */
	    rv=AE_Book_BeginEdit(b, 0);
	    if (rv<0) {
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Could not lock database (error: %d)"),
				  rv);
	      return 1;
	    }

	    /* existing partner modified, apply to database */
	    rv=AE_Book_ReadPartner(b,
				   AE_Book_TableType_Partner,
				   AE_Partner_GetId(pt),
				   &ptOrig);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	      AE_Partner_free(pt);
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error reading from database (error: %d)"),
				  rv);
	      return rv;
	    }

	    rv=AE_Book_ReadPartnerSubObjects(b, ptOrig);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	      AE_Partner_free(ptOrig);
	      AE_Partner_free(pt);
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error reading from database (error: %d)"),
				  rv);
	      return rv;
	    }

	    rv=AE_Book_ApplyPartner(b, pt, ptOrig, 0);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	      AE_Partner_free(ptOrig);
	      AE_Partner_free(pt);
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error writing to database (error: %d)"),
				  rv);
	      return rv;
	    }
	    AE_Partner_free(ptOrig);

	    rv=AE_Book_EndEdit(b, 0);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	      AE_Partner_free(pt);
	      FXMessageBox::error(this, FX::MBOX_OK,
				  I18N("Database Error"),
				  I18N("Error applying changes to database (error: %d)"),
				  rv);
	      return rv;
	    }
	    app->signalDbReload(AE_Book_TableType_Partner);
	  } /* if modified */
	} /* if editPartner */
	AE_Partner_free(pt);
      } /* if b */
    }
  }

  return 1;
}



void FM_BankAccountStmPage::reset() {
}



void GWENHYWFAR_CB *FM_BankAccountStmPage::_readStatementCallback(AE_BOOK *bk,
								  AE_BOOK_TABLE_TYPE tt,
								  AQDB_OBJECT *o,
								  void *data) {
  FM_BankAccountStmPage *p;
  AE_STATEMENT *st=NULL;
  int rv;

  p=(FM_BankAccountStmPage*) data;

  rv=AE_Statement_fromObject(o, &st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AE_Statement_SetDbId(st, AQDB_Object_GetObjectId(o));
    if (AE_Statement_Matches(st, p->m_tempMatcherString.text()))
      p->m_statementTable->addStatement(st);
    else
      AE_Statement_free(st);
  }

  return NULL;
}



int FM_BankAccountStmPage::readStatements() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    void *dummy=NULL;
    char qexpr[256];

    snprintf(qexpr, sizeof(qexpr)-1,
             "$bankAccountId==%d",
             m_bankAccountId);
    qexpr[sizeof(qexpr)-1]=0;

    m_tempMatcherString=m_matcher->getText().simplify().trim();
    rv=AE_Book_ForEveryQueryObject(b,
                                   AE_Book_TableType_BankStatement,
                                   0,
                                   qexpr, /* empty query string */
                                   _readStatementCallback,
                                   (void*) this,
                                   &dummy);
    if (rv<0) {
      if (rv!=GWEN_ERROR_NOT_FOUND) {
        DBG_ERROR(0, "here (%d)", rv);
        return rv;
      }
    }
  }

  return 0;
}



void FM_BankAccountStmPage::rebuild() {
  m_statementTable->clearItems();
  m_statementTable->setMarkBeyondId(m_parent->getStatementMarkId());
  m_statementTable->reload();
  readStatements();

  m_statementTable->sort();
  m_statementTable->layout();

  updateGraph();
}



long FM_BankAccountStmPage::onTableContentReplaced(FXObject*, FXSelector, void* ptr) {
  FXXTable::TableCoords *pos=(FXXTable::TableCoords*) ptr;
  int r, c;
  AE_STATEMENT *st;

  r=pos->row;
  c=pos->column;

  st=m_statementTable->getStatementByRow(r);
  if (st && (AE_Statement_GetRuntimeFlags(st) & AE_STATEMENT_RTFLAGS_MODIFIED)) {
    int rv;
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();

    AE_Book_BeginEdit(b, 0);
    if (AE_Statement_GetRuntimeFlags(st) & AE_STATEMENT_RTFLAGS_MODIFIED) {
      if (AE_Statement_GetId(st)==0) {
        /* create transaction */
        rv=AE_Book_AddStatement(b, AE_Book_TableType_BankStatement, st, 0);
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
        }
      }
      else {
        /* write transaction */
	rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankStatement, st, 0);
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
        }
      }
      if (rv<0) {
        AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
        FXMessageBox::error(this, FX::MBOX_OK,
                            I18N("Database Error"),
			    I18N("The statement could not be written (error: %d)"),
			    rv);
	return 1;
      }
      AE_Statement_SubRuntimeFlags(st, AE_STATEMENT_RTFLAGS_MODIFIED);
    }

    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      FXMessageBox::error(this, FX::MBOX_OK,
                          I18N("Database Error"),
                          I18N("The edit session could not be closed (error: %d)"),
                          rv);
      return 1;
    }
  }
  return 1;
}



long FM_BankAccountStmPage::onTableSelectionChanged(FXObject*, FXSelector, void* ptr) {
  AE_STATEMENT_LIST2 *sl;
  AE_STATEMENT_LIST2_ITERATOR *it;

  DBG_ERROR(0, "Selection changed");

  sl=AE_Statement_List2_new();
  m_statementTable->getSelectedStatements(sl, FXXTable::COPYMODE_NONE);
  it=AE_Statement_List2_First(sl);
  if (it) {
    const AQDB_VALUE *cvMin=NULL;
    const AQDB_VALUE *cvMax=NULL;
    AQDB_VALUE *vTotal=NULL;
    AQDB_VALUE *vAvg=NULL;
    const char *currency=NULL;
    AE_STATEMENT *st;
    GWEN_BUFFER *buf;
    char nbuf[32];
    FXString str;

    st=AE_Statement_List2Iterator_Data(it);
    while(st) {
      const AQDB_VALUE *cv;

      cv=AE_Statement_GetValue(st);
      if (cv) {
	if (currency==NULL)
	  currency=AE_Statement_GetCurrency(st);

	if (cvMin) {
	  if (AQDB_Value_Compare(cvMin, cv)>0) {
	    cvMin=cv;
	  }
	}
	else
	  cvMin=cv;

	if (cvMax) {
	  if (AQDB_Value_Compare(cvMax, cv)<0) {
	    cvMax=cv;
	  }
	}
	else
	  cvMax=cv;

	if (vTotal==NULL)
	  vTotal=AQDB_Value_new();
	AQDB_Value_AddValue(vTotal, cv);

	if (vAvg==NULL)
	  vAvg=AQDB_Value_new();
	AQDB_Value_AddValue(vAvg, cv);
      }
      st=AE_Statement_List2Iterator_Next(it);
    }

    if (vAvg) {
      AQDB_VALUE *vdiv;

      vdiv=AQDB_Value_fromDouble((int)(AE_Statement_List2_GetSize(sl)));
      AQDB_Value_DivValue(vAvg, vdiv);
      AQDB_Value_free(vdiv);
    }

    buf=GWEN_Buffer_new(0, 256, 0, 1);
    if (cvMin) {
      if (GWEN_Buffer_GetUsedBytes(buf))
	GWEN_Buffer_AppendString(buf, " ");
      GWEN_Buffer_AppendString(buf, I18N("FM_BankAccountStmPage|min="));
      AQDB_Value_toLocalHumanReadableString(cvMin, buf, 2);
      if (currency) {
	GWEN_Buffer_AppendString(buf, " ");
	GWEN_Buffer_AppendString(buf, currency);
      }
    }

    if (cvMax) {
      if (GWEN_Buffer_GetUsedBytes(buf))
	GWEN_Buffer_AppendString(buf, " ");
      GWEN_Buffer_AppendString(buf, I18N("FM_BankAccountStmPage|max="));
      AQDB_Value_toLocalHumanReadableString(cvMax, buf, 2);
      if (currency) {
	GWEN_Buffer_AppendString(buf, " ");
	GWEN_Buffer_AppendString(buf, currency);
      }
    }

    if (vAvg) {
      if (GWEN_Buffer_GetUsedBytes(buf))
	GWEN_Buffer_AppendString(buf, " ");
      GWEN_Buffer_AppendString(buf, I18N("FM_BankAccountStmPage|avg="));
      AQDB_Value_toLocalHumanReadableString(vAvg, buf, 2);
      if (currency) {
	GWEN_Buffer_AppendString(buf, " ");
	GWEN_Buffer_AppendString(buf, currency);
      }
    }

    if (vTotal) {
      if (GWEN_Buffer_GetUsedBytes(buf))
	GWEN_Buffer_AppendString(buf, " ");
      GWEN_Buffer_AppendString(buf, I18N("FM_BankAccountStmPage|sum="));
      AQDB_Value_toLocalHumanReadableString(vTotal, buf, 2);
      if (currency) {
	GWEN_Buffer_AppendString(buf, " ");
	GWEN_Buffer_AppendString(buf, currency);
      }
    }

    if (GWEN_Buffer_GetUsedBytes(buf))
      GWEN_Buffer_AppendString(buf, " ");
    GWEN_Buffer_AppendString(buf, I18N("FM_BankAccountStmPage|cnt="));
    snprintf(nbuf, sizeof(nbuf)-1, "%d", AE_Statement_List2_GetSize(sl));
    GWEN_Buffer_AppendString(buf, nbuf);

    str=FXString(GWEN_Buffer_GetStart(buf));
    GWEN_Buffer_free(buf);
    m_fLabel->setText(str);

    AQDB_Value_free(vAvg);
    AQDB_Value_free(vTotal);

    AE_Statement_List2Iterator_free(it);
  }
  AE_Statement_List2_free(sl);

  return 1;
}



long FM_BankAccountStmPage::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  rebuild();
  return 1;
}



int FM_BankAccountStmPage::createGraph(int w, int h, const char *fname) {
  AF_GRAPH_DATASET *ds;
  int i;
  uint32_t col;
  int cnt=0;

  /* prepare data set */
  ds=AF_Graph_DataSet_new();
  AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Integer);
  AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
  AF_Graph_DataSet_SetPrecisionY(ds, 2);

  col=(255<<24); /* red */

  AF_Graph_DataSet_AddSubGraph(ds, "", 1, AF_GRAPH_TYPE_BARS, AF_Report2_DataMode_Direct, col, 1, 0);

  /* sample data */
  for (i=m_statementTable->getFirstVisibleRow(); i<=m_statementTable->getLastVisibleRow(); i++) {
    AE_STATEMENT *st;

    st=m_statementTable->getStatementByRow(i);
    if (st) {
      AF_GRAPH_DATAPAIR *dp;
      const AQDB_VALUE *v;
      double dv=0.0;

      v=AE_Statement_GetValue(st);
      if (v)
        dv=AQDB_Value_GetValueAsDouble(v);

      dp=AF_Graph_DataPair_new();
      AF_Graph_DataPair_SetLevel(dp, 1);
      AF_Graph_DataPair_SetValueX(dp, cnt+1);

      AF_Graph_DataPair_SetValueY(dp, dv);
      AF_Graph_DataSet_AddDataPair(ds, dp);
      cnt++;
    }
  }

  if (cnt) {
    AF_Graph_DataSet_AdjustXRange(ds);
    AF_Graph_DataSet_AdjustYRange(ds);
    AF_Graph_DataSet_AddTicks(ds);

    /* actually draw */
      {
        AF_GRAPHICS *g;
        AF_GRAPH *gr;
        AF_GRAPH_SETTINGS *gs;
        int rv;

        g=AF_GraphicsCairo_Png_new(fname, w, h);
        AF_Graphics_Attach(g);
        gs=AF_Graph_Settings_new();
        AF_Graph_Settings_SetDefaults(gs);

        gr=AF_Graph_new(ds, g, w, h);
        rv=AF_Graph_Init(gr, gs);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AF_Graphics_free(g);
          AF_Graph_Settings_free(gs);
          AF_Graph_free(gr);
          return rv;
        }

        rv=AF_Graph_Draw(gr);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AF_Graphics_free(g);
          AF_Graph_Settings_free(gs);
          AF_Graph_free(gr);
          return rv;
        }

        AF_Graph_Fini(gr);
        AF_Graph_free(gr);

        AF_Graphics_Finish(g);
        AF_Graph_Settings_free(gs);
        AF_Graphics_free(g);
      }
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "No visible statements");
    return GWEN_ERROR_NO_DATA;
  }

  return 0;
}



int FM_BankAccountStmPage::updateGraph() {
  if (m_imageFrame) {
    int w;
    int h;
    int rv;
    const char *s;
    GWEN_BUFFER *tbuf;

    w=m_imageFrame->getWidth();
    h=m_imageFrame->getHeight();

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    s=AE_GetUserDataFolder();
    assert(s);
    GWEN_Buffer_AppendString(tbuf, s);
    GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S "tmp" GWEN_DIR_SEPARATOR_S "bankaccountgraphs");

    rv=GWEN_Directory_GetPath(GWEN_Buffer_GetStart(tbuf), GWEN_PATH_FLAGS_CHECKROOT);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "Unable to acces/create folder [%s]. ",
		GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
    else {
      FXString fname;

      /* create file name */
      GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);
      GWEN_Buffer_AppendString(tbuf, "graph-");
      fname=GWEN_Buffer_GetStart(tbuf);
      GWEN_Buffer_free(tbuf);
      fname+=FXStringVal(m_bankAccountId);
      fname+=".png";

      DBG_ERROR(0, "Creating graph [%s]", fname.text());
      rv=createGraph(w, h, fname.text());
      if (rv<0) {
        DBG_ERROR(0, "Error creating graph (%d)", rv);
      }
      else {
        FXIconSource iconSource(getApp());
        FXImage *image;

        DBG_ERROR(0, "Loading report image [%s]", fname.text());
        image=iconSource.loadImageFile(fname);
        if (image) {
          DBG_ERROR(0, "Setting graph image [%s]", fname.text());
          image->create();
          m_imageFrame->setImage(image);
          if (m_image)
            delete m_image;
          m_image=image;
        }
      }
    }

    return 0;
  }

  return GWEN_ERROR_NO_DATA;
}



long FM_BankAccountStmPage::onTableDocked(FXObject*, FXSelector, void* ptr) {
  updateGraph();
  return 1;
}



