/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_BANKACCSTMPAGE_HPP
#define FM_BANKACCSTMPAGE_HPP



#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>


class FF_StatementTable;
class FOX16_HtmlLabel;
class FM_BankAccountPage;


class FM_BankAccountStmPage: public FXVerticalFrame {
  FXDECLARE(FM_BankAccountStmPage)
public:
  enum {
    ID_STM_TABLE=FXPacker::ID_LAST,
    ID_CMD_DELSTM,
    ID_CMD_MKPARTNER,
    ID_MATCHER,
    ID_LAST
  };


  FM_BankAccountStmPage(FM_BankAccountPage *page,
                        const AE_BANKACCOUNT *ba,
                        FXComposite *p,
                        FXuint opts=0,
                        FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_BankAccountStmPage();

  AQDB_ID getBankAccountId() const { return m_bankAccountId;};

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void aboutToSendJobs();

  void enter();
  void leave();
  void reset();

  void create();

  long onCmdTable(FXObject*, FXSelector, void* ptr);
  long onTableRightButtonRelease(FXObject*, FXSelector, void* ptr);
  long onTableContentReplaced(FXObject*, FXSelector, void* ptr);
  long onTableSelectionChanged(FXObject*, FXSelector, void* ptr);
  long onTableDocked(FXObject*, FXSelector, void* ptr);

  long onCmdDelStm(FXObject*, FXSelector, void* ptr);
  long onCmdMkPartner(FXObject*, FXSelector, void* ptr);
  long onChgMatcher(FXObject*, FXSelector, void* ptr);

protected:
  FM_BankAccountStmPage();
  void rebuild();

  static void GWENHYWFAR_CB *_readStatementCallback(AE_BOOK *bk,
						    AE_BOOK_TABLE_TYPE tt,
						    AQDB_OBJECT *o,
						    void *data);
  int readStatements();
  int createGraph(int w, int h, const char *fname);
  int updateGraph();

  FM_BankAccountPage *m_parent;

  AQDB_ID m_bankAccountId;
  FOX16_HtmlLabel *m_hLabel;
  FOX16_HtmlLabel *m_fLabel;
  FF_StatementTable *m_statementTable;

  FXTextField *m_matcher;
  FXString m_tempMatcherString;

  bool m_needRebuild;
  bool m_isVisible;

  FXImageFrame *m_imageFrame;
  FXImage *m_image;

  FXString m_accountHeader;
};



#endif
