/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_businesspage.hpp"
#include "fm_bus_taskpage.hpp"
#include "ff_jobtree.hpp"
#include "ff_app.hpp"
#include "ff_editjob.hpp"
#include "ff_editpayment.hpp"
#include "ff_edittransaction.hpp"
#include "ff_editxfer.hpp"

#ifdef HAVE_CAIRO
# include <aqfinance/print/af_print.h>
# include <aqfinance/print/af_print_box.h>
# include "ff_edit_printtabledescr.hpp"
# include "ff_jobprinter.hpp"
#endif

#include <aqfinance/engine/modules/ae_openbills.h>
#include <aqfinance/engine/modules/ae_opentasks.h>
#include <aqfinance/engine/modules/ae_trans4job.h>
#include <aqfinance/engine/book/ae_utils.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwendate.h>
#include <gwenhywfar/directory.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_BusinessTaskPage) FM_BusinessTaskPageMap[]={
  FXMAPFUNC(SEL_CHANGED, FM_BusinessTaskPage::ID_JOB_TREE,
            FM_BusinessTaskPage::onTreeChanged),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_BusinessTaskPage::ID_JOB_TREE,
            FM_BusinessTaskPage::onTreeRightBtnPress),
  FXMAPFUNC(SEL_EXPANDED, FM_BusinessTaskPage::ID_JOB_TREE,
            FM_BusinessTaskPage::onTreeOpened),

  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_NEW_REQUEST,
            FM_BusinessTaskPage::onCmdNewRequest),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_NEW_OFFER,
            FM_BusinessTaskPage::onCmdNewOffer),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_NEW_ORDER,
            FM_BusinessTaskPage::onCmdNewOrder),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_NEW_DELIVERY,
            FM_BusinessTaskPage::onCmdNewDelivery),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_NEW_INVOICE,
            FM_BusinessTaskPage::onCmdNewInvoice),

  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_REQUEST,
            FM_BusinessTaskPage::onCmdAddRequest),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_OFFER,
            FM_BusinessTaskPage::onCmdAddOffer),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_ORDER,
            FM_BusinessTaskPage::onCmdAddOrder),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_CONFIRMATION,
            FM_BusinessTaskPage::onCmdAddConfirmation),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_RELEASE,
            FM_BusinessTaskPage::onCmdAddRelease),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_DELIVERY,
            FM_BusinessTaskPage::onCmdAddDelivery),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_INVOICE,
            FM_BusinessTaskPage::onCmdAddInvoice),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_PAYMENT,
            FM_BusinessTaskPage::onCmdAddPayment),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_REMINDER,
            FM_BusinessTaskPage::onCmdAddReminder),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_ADD_CANCELLATION,
            FM_BusinessTaskPage::onCmdAddCancellation),

  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_EDIT_JOB,
            FM_BusinessTaskPage::onCmdEditJob),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_PRINT_JOB,
            FM_BusinessTaskPage::onCmdPrintJob),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_VIEWPDF,
	    FM_BusinessTaskPage::onCmdViewPdf),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_EMAILPDF,
	    FM_BusinessTaskPage::onCmdEmailPdf),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_MARKASPRINTED,
	    FM_BusinessTaskPage::onCmdMarkedAsPrinted),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_MARKASEMAILED,
            FM_BusinessTaskPage::onCmdMarkedAsEmailed),

  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_DEL_JOB,
            FM_BusinessTaskPage::onCmdDeleteJob),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_DEL_TASK,
            FM_BusinessTaskPage::onCmdDeleteTask),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_CLOSE_TASK,
            FM_BusinessTaskPage::onCmdCloseTask),

  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_NEW_TRANS,
            FM_BusinessTaskPage::onCmdCreateTransaction),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_EDIT_TRANS,
            FM_BusinessTaskPage::onCmdEditTransaction),
  FXMAPFUNC(SEL_COMMAND, FM_BusinessTaskPage::ID_BUTTON_CREATE_TRANSFER,
            FM_BusinessTaskPage::onCmdCreateTransfer),
};



FXIMPLEMENT(FM_BusinessTaskPage, FXHorizontalFrame,
            FM_BusinessTaskPageMap, ARRAYNUMBER(FM_BusinessTaskPageMap))






FM_BusinessTaskPage::FM_BusinessTaskPage(bool isBuy,
					 FM_BusinessPage *mainPage,
					 FXComposite *p,
					 FXuint opts,
					 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_isBuy(isBuy)
,m_mainPage(mainPage)
,m_needRebuild(true)
,m_isVisible(false)
{
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_taskTree=new FF_JobTree(this, this, ID_JOB_TREE,
			    LAYOUT_SIDE_LEFT |
			    LAYOUT_FILL_X | LAYOUT_FILL_Y |
			    FOLDINGLIST_BROWSESELECT |
			    FOLDINGLIST_SHOWS_LINES |
			    FOLDINGLIST_SHOWS_BOXES |
			    FOLDINGLIST_ROOT_BOXES |
			    FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_taskTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_taskTree->setIconClosed(ic);
}




FM_BusinessTaskPage::FM_BusinessTaskPage()
:FXHorizontalFrame()
,m_isBuy(true)
,m_mainPage(NULL)
,m_taskTree(NULL)
,m_needRebuild(false)
{
}



FM_BusinessTaskPage::~FM_BusinessTaskPage() {
}



void FM_BusinessTaskPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "invoicePage");
  if (dbT)
    dbT=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
			 m_isBuy?"incoming":"outgoing");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_taskTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_taskTree->getHeaderSize(i));
      m_taskTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_taskTree->setHeaderArrowDir(i, TRUE);
        else
          m_taskTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_BusinessTaskPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "invoicePage");
  dbT=GWEN_DB_GetGroup(dbT, GWEN_DB_FLAGS_OVERWRITE_GROUPS, m_isBuy?"incoming":"outgoing");
  for (i=0; i<m_taskTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_taskTree->getHeaderSize(i));
    b=m_taskTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}


long FM_BusinessTaskPage::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  return 1;
}



int FM_BusinessTaskPage::createTask(int createJobType, const char *title) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;
    GWEN_DATE *dt;
    AQDB_ID nid=0;

    j=AE_Job_new();
    dt=GWEN_Date_CurrentDate();
    AE_Job_SetEnterDate(j, dt);
    if (1) {
      GWEN_DATE *dd;
      int jul;

      jul=GWEN_Date_GetJulian(dt);
      dd=GWEN_Date_fromJulian(jul+7);
      AE_Job_SetDueDate(j, dd);
      GWEN_Date_free(dd);
    }
    AE_Job_SetType(j, createJobType);
    AE_Job_SetPriority(j, AE_Job_PriorityMedium);
  
    if (FF_EditJob::editJob(app, this,
                            title,
			    j,
			    m_isBuy)) {
      int rv;
      AE_TASK *tk;
      FF_JobTreeItem *jItem;

      tk=AE_Task_new();
      if (m_isBuy)
	AE_Task_AddFlags(tk, AE_TASK_FLAGS_INCOMING);
      AE_Task_AddFlags(tk, AE_TASK_FLAGS_ACTIVE);

      AE_Task_SetEnterDate(tk, dt);
      AE_Task_SetPartnerId(tk, AE_Job_GetPartnerId(j));

      rv=AE_Book_BeginEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "Error on begin edit: %d", rv);
	AE_Task_free(tk);
        GWEN_Date_free(dt);
	AE_Job_free(j);
	return 1;
      }

      DBG_ERROR(0, "Adding task");
      rv=AE_Book_AddTask(b, AE_Book_TableType_Task, tk, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Task_free(tk);
        GWEN_Date_free(dt);
	AE_Job_free(j);
	return 1;
      }

      DBG_ERROR(0, "Applying job");
      AE_Job_SetTaskId(j, AE_Task_GetId(tk));
      rv=AE_Book_ApplyJob(b, j, NULL, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Task_free(tk);
        GWEN_Date_free(dt);
	AE_Job_free(j);
	return 1;
      }

      nid=AE_Job_GetId(j);

      rv=AE_Book_EndEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Task_free(tk);
	GWEN_Date_free(dt);
	AE_Job_free(j);
	return 1;
      }
      AE_Task_free(tk);

      dbReload(AE_Book_TableType_Job);

      jItem=m_taskTree->findJobItem(nid);
      if (jItem) {
	m_taskTree->unselectAll();
	m_taskTree->makeItemVisible(jItem);
	jItem->setSelected(TRUE);
	m_taskTree->setCurrentItem(jItem);
      }
    }
    else {
      DBG_ERROR(0, "Aborted");
    }
    GWEN_Date_free(dt);
    AE_Job_free(j);
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdNewRequest(FXObject*, FXSelector, void*) {
  return createTask(AE_Job_TypeRequest, I18N("Start New Task With A Request"));
}



long FM_BusinessTaskPage::onCmdNewOffer(FXObject*, FXSelector, void*) {
  return createTask(AE_Job_TypeOffer, I18N("Start New Task With An Offer"));
}



long FM_BusinessTaskPage::onCmdNewOrder(FXObject*, FXSelector, void*) {
  return createTask(AE_Job_TypeOrder, I18N("Start New Task With An Order"));
}



long FM_BusinessTaskPage::onCmdNewDelivery(FXObject*, FXSelector, void*) {
  return createTask(AE_Job_TypeDelivery, I18N("Start New Task With A Delivery"));
}



long FM_BusinessTaskPage::onCmdNewInvoice(FXObject*, FXSelector, void*) {
  return createTask(AE_Job_TypeInvoice, I18N("Start New Task With An Invoice"));
}



int FM_BusinessTaskPage::addJob(int createJobType, const char *title, bool doCopyItems) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;
    AE_JOB *jNew;
    GWEN_DATE *dt;
    AE_TASK *tk=NULL;
    AQDB_ID tid=0;
    AQDB_ID jid=0;
    AQDB_ID nid=0;
    int jobType=AE_Job_TypeNone;
    int rv;

    j=m_taskTree->getCurrentJob();
    if (j) {
      tid=AE_Job_GetTaskId(j);
      jid=AE_Job_GetId(j);
      jobType=AE_Job_GetType(j);
    }
    if (tid==0) {
      DBG_ERROR(0, "No task selected, SNH");
      return 1;
    }

    /* read task */
    tk=AE_Task_new();
    rv=AE_Book_ReadTask(b, AE_Book_TableType_Task, tid, &tk);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Task_free(tk);
      return 1;
    }

    rv=AE_Book_ReadTaskSubObjects(b, tk, m_isBuy);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Task_free(tk);
      return 1;
    }

    /* create new job */
    if (jid) {
      /* copy from existing */
      jNew=AE_Job_dup(j);
      AE_Job_SetReferenceId(jNew, jid);
      AE_Job_SetId(jNew, 0);
      AE_Job_SetDbId(jNew, 0);
      AE_Job_SetCode(jNew, NULL);

      if (doCopyItems) {
	AE_ITEM *item;

	/* clear item ids in new job */
	item=AE_Item_List_First(AE_Job_GetItems(jNew));
	while(item) {
	  AE_Item_SetId(item, 0);
	  AE_Item_SetDbId(item, 0);
	  item=AE_Item_List_Next(item);
	}
      }
      else
	AE_Item_List_Clear(AE_Job_GetItems(jNew));
    }
    else {
      /* create a completely new job */
      jNew=AE_Job_new();
      AE_Job_SetPriority(jNew, AE_Job_PriorityMedium);
      AE_Job_SetTaskId(jNew, tid);
      AE_Job_SetPartnerId(jNew, AE_Task_GetPartnerId(tk));
    }
    AE_Job_SetType(jNew, createJobType);
    dt=GWEN_Date_CurrentDate();
    AE_Job_SetEnterDate(jNew, dt);
    if (1) {
      GWEN_DATE *dd;
      int jul;

      jul=GWEN_Date_GetJulian(dt);
      dd=GWEN_Date_fromJulian(jul+7);
      AE_Job_SetDueDate(jNew, dd);
      GWEN_Date_free(dd);
    }

    if (FF_EditJob::editJob(app, this,
                            title,
			    jNew,
			    m_isBuy)) {
      rv=AE_Book_BeginEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "Error on begin edit: %d", rv);
	AE_Task_free(tk);
	GWEN_Date_free(dt);
	AE_Job_free(jNew);
	return 1;
      }
  
      /* modify task */

      /* write task */
      rv=AE_Book_WriteTask(b, AE_Book_TableType_Task, tk, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Task_free(tk);
	GWEN_Date_free(dt);
	AE_Job_free(jNew);
	return 1;
      }
  
      AE_Job_SetTaskId(jNew, AE_Task_GetId(tk));
      rv=AE_Book_ApplyJob(b, jNew, NULL, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Task_free(tk);
	GWEN_Date_free(dt);
	AE_Job_free(jNew);
	return 1;
      }
      nid=AE_Job_GetId(jNew);
  
      rv=AE_Book_EndEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Task_free(tk);
	GWEN_Date_free(dt);
	AE_Job_free(jNew);
	return 1;
      }
    }
    AE_Task_free(tk);
    GWEN_Date_free(dt);
    AE_Job_free(jNew);

    /* signal changes */
    app->signalDbReload(AE_Book_TableType_Job);

    m_taskTree->unselectAll();
    m_taskTree->makeJobVisible(nid);
    m_taskTree->setCurrentJob(nid, TRUE);
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdAddRequest(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeRequest, I18N("Add Request To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddOffer(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeOffer, I18N("Add Offer To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddOrder(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeOrder, I18N("Add Order To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddConfirmation(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeConfirmation, I18N("Add Confirmation To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddRelease(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeRelease, I18N("Add Release To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddDelivery(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeDelivery, I18N("Add Delivery To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddInvoice(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeInvoice, I18N("Add Invoice To Task"), 1);
}



long FM_BusinessTaskPage::onCmdAddPayment(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j) {
      int rv;
      AE_JOB *jPayment;
      GWEN_DATE *dt;
      AE_TASK *tk=NULL;
      AQDB_ID tid=0;
      AQDB_ID jid;
      const AQDB_VALUE *v;

      tid=AE_Job_GetTaskId(j);
      if (tid==0) {
	DBG_ERROR(0, "No task selected, SNH");
	return 1;
      }

      jid=AE_Job_GetId(j);
      if (AE_Job_GetType(j)!=AE_Job_TypeInvoice || jid==0) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Not an Invoice"),
			    "%s",
			    I18N("Please select an invoice to add a payment to."));
        return 1;
      }

      /* read task */
      tk=AE_Task_new();
      rv=AE_Book_ReadTask(b, AE_Book_TableType_Task, tid, &tk);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Task_free(tk);
	return 1;
      }
  
      rv=AE_Book_ReadTaskSubObjects(b, tk, m_isBuy);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Task_free(tk);
	return 1;
      }

      dt=GWEN_Date_CurrentDate();

      jPayment=AE_Job_new();
      AE_Job_SetReferenceId(jPayment, jid);
      AE_Job_SetType(jPayment, AE_Job_TypePayment);
      AE_Job_SetId(jPayment, 0);
      AE_Job_SetDbId(jPayment, 0);
      AE_Job_SetPartnerId(jPayment, AE_Job_GetPartnerId(j));
      AE_Job_SetCommodityId(jPayment, AE_Job_GetCommodityId(j));
      v=AE_Job_GetOpenSum(j);
      if (v) {
	AE_ITEM *item;

        /* create an item for the open amont */
	item=AE_Item_new();
        if (m_isBuy)
	  AE_Item_SetBuyPriceValue(item, v);
        else
	  AE_Item_SetSellPriceValue(item, v);
	AE_Item_SetCommodityId(item, AE_Job_GetCommodityId(j));
        AE_Item_List_Add(item, AE_Job_GetItems(jPayment));
      }

      AE_Job_SetEnterDate(jPayment, dt);
      AE_Job_SetDueDate(jPayment, dt);
      AE_Job_SetType(jPayment, AE_Job_TypePayment);

      if (FF_EditPayment::editPayment(app, this,
				      I18N("Create Payment for Invoice"),
				      jPayment,
				      m_isBuy)) {
	AQDB_ID tid;
        AQDB_ID nid=0;

	tid=AE_Job_GetTaskId(j);

	rv=AE_Book_BeginEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "Error on begin edit: %d", rv);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}

	/* modify task */
	AE_Task_SetWaitingFor(tk, AE_Job_TypeNone);

	rv=AE_Book_WriteTask(b, AE_Book_TableType_Task, tk, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}
  
	AE_Job_SetTaskId(jPayment, AE_Task_GetId(tk));
	rv=AE_Book_ApplyJob(b, jPayment, NULL, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}

	nid=AE_Job_GetId(jPayment);
  
	rv=AE_Book_EndEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}
	AE_Task_free(tk);

	dbReload(AE_Book_TableType_Job);

	m_taskTree->unselectAll();
	m_taskTree->makeJobVisible(nid);
	m_taskTree->setCurrentJob(nid, TRUE);
      }

      AE_Job_free(jPayment);
      GWEN_Date_free(dt);
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdAddReminder(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j) {
      int rv;
      AE_JOB *jReminder;
      GWEN_DATE *dt;
      AE_TASK *tk=NULL;
      AQDB_ID tid=0;
      AQDB_ID jid;
      const AQDB_VALUE *v;

      tid=AE_Job_GetTaskId(j);
      if (tid==0) {
	DBG_ERROR(0, "No task selected, SNH");
	return 1;
      }

      jid=AE_Job_GetId(j);
      if (AE_Job_GetType(j)!=AE_Job_TypeInvoice || jid==0) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Not an Invoice"),
			    "%s",
			    I18N("Please select an invoice to add a payment to."));
        return 1;
      }

      /* read task */
      tk=AE_Task_new();
      rv=AE_Book_ReadTask(b, AE_Book_TableType_Task, tid, &tk);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Task_free(tk);
	return 1;
      }
  
      rv=AE_Book_ReadTaskSubObjects(b, tk, m_isBuy);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Task_free(tk);
	return 1;
      }

      dt=GWEN_Date_CurrentDate();

      jReminder=AE_Job_new();
      AE_Job_SetReferenceId(jReminder, jid);
      AE_Job_SetType(jReminder, AE_Job_TypeReminder);
      AE_Job_SetId(jReminder, 0);
      AE_Job_SetDbId(jReminder, 0);
      AE_Job_SetPartnerId(jReminder, AE_Job_GetPartnerId(j));
      AE_Job_SetCommodityId(jReminder, AE_Job_GetCommodityId(j));
      v=AE_Job_GetOpenSum(j);
      if (v) {
	AE_ITEM *item;

        /* create an item for the open amont */
	item=AE_Item_new();
        if (m_isBuy)
	  AE_Item_SetBuyPriceValue(item, v);
        else
	  AE_Item_SetSellPriceValue(item, v);
	AE_Item_SetCommodityId(item, AE_Job_GetCommodityId(j));
        AE_Item_List_Add(item, AE_Job_GetItems(jReminder));
      }

      AE_Job_SetEnterDate(jReminder, dt);
      AE_Job_SetDueDate(jReminder, dt);
      AE_Job_SetType(jReminder, AE_Job_TypeReminder);

      if (FF_EditJob::editJob(app, this,
			      I18N("Create Reminder for Invoice"),
			      jReminder,
			      m_isBuy /* incoming means: buy */)) {
	AQDB_ID tid;

	tid=AE_Job_GetTaskId(j);

	rv=AE_Book_BeginEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "Error on begin edit: %d", rv);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}

	/* modify task */

	/* write task */
	rv=AE_Book_WriteTask(b, AE_Book_TableType_Task, tk, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}
  
	AE_Job_SetTaskId(jReminder, AE_Task_GetId(tk));
	rv=AE_Book_ApplyJob(b, jReminder, NULL, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}
  
	rv=AE_Book_EndEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  GWEN_Date_free(dt);
	  AE_Job_free(j);
	  return 1;
	}
	AE_Task_free(tk);

        dbReload(AE_Book_TableType_Job);
      }

      AE_Job_free(jReminder);
      GWEN_Date_free(dt);
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdAddCancellation(FXObject*, FXSelector, void*) {
  return addJob(AE_Job_TypeCancellation, I18N("Add Cancellation To Task"), 1);
}



long FM_BusinessTaskPage::onCmdCreateTransaction(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j && AE_Job_GetId(j)>0) {
      int jt;
      AQDB_ID jid;

      jid=AE_Job_GetId(j);

      jt=AE_Job_GetType(j);
      if (jt!=AE_Job_TypeInvoice &&
	  jt!=AE_Job_TypePayment) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Selection Error"),
			    "%s",
			    I18N("Please select an invoice or a payment to create a "
				 "booking transaction for."));
	return 1;
      }
      else {
	AE_TRANSACTION *t;
	AE_ACCOUNT_TREE *at;
	AE_PARTNER *pt=NULL;
	AE_COMMODITY_LIST *cl=NULL;
	AE_JOB *jRead=NULL;
        const char *s;
        int rv;

	at=AE_Account_Tree_new();
	rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
	if (rv<0) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      "%s",
			      I18N("Could not read account tree"));
	  AE_Account_Tree_free(at);
	  return 1;
	}

	cl=AE_Commodity_List_new();
	rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
	if (rv<0) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      "%s",
			      I18N("Could not read commodity list"));
          AE_Commodity_List_free(cl);
	  AE_Account_Tree_free(at);
	  return 1;
	}

	/* determine the partner's account */
	if (AE_Job_GetPartnerId(j)>0) {
	  rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner,
				 AE_Job_GetPartnerId(j), &pt);
	  if (rv<0) {
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				"%s",
				I18N("Could not read the partner assigned to the job"));
	    AE_Commodity_List_free(cl);
	    AE_Account_Tree_free(at);
	    return 1;
	  }
	}

	t=AE_Transaction_new();
	AE_Transaction_SetDate(t, AE_Job_GetEnterDate(j));
	s=AE_Job_GetMemo(j);
	if (s && *s)
	  AE_Transaction_SetMemo(t, s);
	if (jt==AE_Job_TypeInvoice) {
	  s=AE_Transaction_GetMemo(t);
	  if (s==NULL || *s==0) {
	    FXString str;
  
	    str=I18N("Invoice");
	    s=AE_Job_GetCode(j);
	    if (s && *s) {
	      str+=" ";
	      str+=s;
	    }
	    AE_Transaction_SetMemo(t, str.text());
	  }

	  if (m_isBuy) {
	    AQDB_ID aid=0;

	    /* incoming invoice, get payable account */
	    if (pt) {
	      aid=AE_Partner_GetPayableAccountId(pt);
	      if (aid==0) {
		AE_ACCOUNT *acc;

		/* determine payable account, create if necessary */
		acc=app->getPartnerPayableAccount(this, at, pt, 1);
		if (acc==NULL) {
		  DBG_ERROR(AE_LOGDOMAIN, "here");
                  AE_Transaction_free(t);
		  AE_Commodity_List_free(cl);
		  AE_Account_Tree_free(at);
		  return 1;
		}
		aid=AE_Account_GetId(acc);
	      }
	    }

	    rv=AE_Transaction4BuyInvoice(b, j, t, aid);
	    if (rv<0) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Transaction Error"),
				  I18N("Could not create a transaction for this invoice (%d)"),
				  rv);
              AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }

	    /* edit transaction */
	    if (!FF_EditTransaction::editTransaction(app, this,
						     I18N("Create Transaction for Received Invoice"),
						     t)) {
	      DBG_ERROR(0, "Aborted");
	      AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }
	  }
	  else {
	    AQDB_ID aid=0;

	    /* outgoing invoice, get receivable account */
	    if (pt) {
	      aid=AE_Partner_GetReceivableAccountId(pt);
	      if (aid==0) {
		AE_ACCOUNT *acc;

		/* determine payable account, create if necessary */
		acc=app->getPartnerReceivableAccount(this, at, pt, 1);
		if (acc==NULL) {
		  DBG_ERROR(AE_LOGDOMAIN, "here");
                  AE_Transaction_free(t);
		  AE_Commodity_List_free(cl);
		  AE_Account_Tree_free(at);
		  return 1;
		}
		aid=AE_Account_GetId(acc);
	      }
	    }

	    rv=AE_Transaction4SellInvoice(b, j, t, aid);
	    if (rv<0) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Transaction Error"),
				  I18N("Could not create a transaction for this invoice (%d)"),
				  rv);
              AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }

	    /* edit transaction */
	    if (!FF_EditTransaction::editTransaction(app, this,
						     I18N("Create Transaction for Sent Invoice"),
						     t)) {
	      DBG_ERROR(0, "Aborted");
	      AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }
	  }
	}
	else if (jt==AE_Job_TypePayment) {
	  s=AE_Transaction_GetMemo(t);
	  if (s==NULL || *s==0) {
	    FXString str;
  
	    str=I18N("Payment");
	    s=AE_Job_GetCode(j);
	    if (s && *s) {
	      str+=" ";
	      str+=s;
	    }
	    AE_Transaction_SetMemo(t, str.text());
	  }

	  if (m_isBuy) {
	    AQDB_ID aid1=0;
	    AQDB_ID aid2=0;
	    AE_ACCOUNT *acc;
	    AE_COMMODITY *co=NULL;

	    co=AE_Commodity_List_GetById(cl, AE_Job_GetCommodityId(j));
	    if (co==NULL) {
	      DBG_ERROR(AE_LOGDOMAIN, "here");
	    }

	    /* incoming payment, get payable account */
	    if (pt) {
	      aid1=AE_Partner_GetPayableAccountId(pt);
	      if (aid1==0) {
		/* determine payable account, create if necessary */
		acc=app->getPartnerPayableAccount(this, at, pt, 1);
		if (acc==NULL) {
		  DBG_ERROR(AE_LOGDOMAIN, "here");
                  AE_Transaction_free(t);
		  AE_Commodity_List_free(cl);
		  AE_Account_Tree_free(at);
		  return 1;
		}
		aid1=AE_Account_GetId(acc);
	      }
	    }

	    /* get cash account (as a preset) */
	    if (co) {
	      acc=app->getCashAccount(this, at, co, 1);
	      if (acc==NULL) {
		DBG_ERROR(AE_LOGDOMAIN, "here");
		AE_Transaction_free(t);
		AE_Commodity_List_free(cl);
		AE_Account_Tree_free(at);
		return 1;
	      }
	      aid2=AE_Account_GetId(acc);
	    }

	    /* create payment */
	    rv=AE_Transaction4BuyPayment(b, j, t, aid1, aid2);
	    if (rv<0) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Transaction Error"),
				  I18N("Could not create a transaction for this payment (%d)"),
				  rv);
              AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }

	    /* edit transaction */
	    if (!FF_EditTransaction::editTransaction(app, this,
						     I18N("Create Transaction for Outgoing Payment"),
						     t)) {
	      DBG_ERROR(0, "Aborted");
	      AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }
	  } /* if buy */
	  else {
	    AQDB_ID aid1=0;
	    AQDB_ID aid2=0;
	    AE_ACCOUNT *acc;
	    AE_COMMODITY *co=NULL;

	    co=AE_Commodity_List_GetById(cl, AE_Job_GetCommodityId(j));
	    if (co==NULL) {
	      DBG_ERROR(AE_LOGDOMAIN, "here");
	    }

	    /* outgoing payment, get receivable account */
	    if (pt) {
	      aid1=AE_Partner_GetReceivableAccountId(pt);
	      if (aid1==0) {
		/* determine receivable account, create if necessary */
		acc=app->getPartnerReceivableAccount(this, at, pt, 1);
		if (acc==NULL) {
		  DBG_ERROR(AE_LOGDOMAIN, "here");
                  AE_Transaction_free(t);
		  AE_Commodity_List_free(cl);
		  AE_Account_Tree_free(at);
		  return 1;
		}
		aid1=AE_Account_GetId(acc);
	      }
	    }

	    /* get cash account (as a preset) */
	    if (co) {
	      acc=app->getCashAccount(this, at, co, 1);
	      if (acc==NULL) {
		DBG_ERROR(AE_LOGDOMAIN, "here");
		AE_Transaction_free(t);
		AE_Commodity_List_free(cl);
		AE_Account_Tree_free(at);
		return 1;
	      }
	      aid2=AE_Account_GetId(acc);
	    }

	    /* create payment */
	    rv=AE_Transaction4SellPayment(b, j, t, aid1, aid2);
	    if (rv<0) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Transaction Error"),
				  I18N("Could not create a transaction for this payment (%d)"),
				  rv);
              AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }

	    /* edit transaction */
	    if (FF_EditTransaction::editTransaction(app, this,
						    I18N("Create Transaction for Incoming Payment"),
						    t)) {
              DBG_ERROR(0, "Accepted transaction");
	    }
	    else {
	      DBG_ERROR(0, "Aborted");
	      AE_Transaction_free(t);
	      AE_Commodity_List_free(cl);
	      AE_Account_Tree_free(at);
	      return 1;
	    }
	  } /* if sell */
	} /* if payment */

	/* we will now lock the database, re-read the job, store the transaction
	 * and write back the job with the transaction id set.
	 */
	rv=AE_Book_BeginEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "Error on begin edit: %d", rv);
	  AE_Transaction_free(t);
	  AE_Commodity_List_free(cl);
	  AE_Account_Tree_free(at);
	  return 1;
	}

	rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, AE_Job_GetId(j), &jRead);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not read job from database (%d)"),
			      rv);
	  AE_Transaction_free(t);
	  AE_Commodity_List_free(cl);
	  AE_Account_Tree_free(at);
	  return 1;
	}

	rv=AE_Book_ApplyTransaction(b, t, NULL, 0);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not store transaction (%d)"),
			      rv);
	  AE_Job_free(jRead);
	  AE_Transaction_free(t);
	  AE_Commodity_List_free(cl);
	  AE_Account_Tree_free(at);
	  return 1;
	}

	/* store transaction id (both in list job and in  newly read job) */
	AE_Job_SetTransactionId(j, AE_Transaction_GetId(t));
	AE_Job_SetTransactionId(jRead, AE_Transaction_GetId(t));

	/* write back new job */
	rv=AE_Book_WriteJob(b, AE_Book_TableType_Job, jRead, 0);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not store job (%d)"),
			      rv);
	  AE_Job_free(jRead);
	  AE_Transaction_free(t);
	  AE_Commodity_List_free(cl);
	  AE_Account_Tree_free(at);
	  return 1;
	}
	AE_Job_free(jRead);

	/* unlock database */
	rv=AE_Book_EndEdit(b, 0);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not apply changes to database (%d)"),
			      rv);
	  AE_Transaction_free(t);
	  AE_Commodity_List_free(cl);
	  AE_Account_Tree_free(at);
	  return 1;
	}

	AE_Transaction_free(t);
	AE_Commodity_List_free(cl);
	AE_Account_Tree_free(at);

	/* signal changes */
	app->signalDbReload(AE_Book_TableType_Transaction);
	app->signalDbReload(AE_Book_TableType_Split);
	app->signalDbReload(AE_Book_TableType_AccountBalance);
	app->signalDbReload(AE_Book_TableType_Task);

	m_taskTree->unselectAll();
	m_taskTree->makeJobVisible(jid);
	m_taskTree->setCurrentJob(jid, TRUE);

      } /* if payment or invoice */
    } /* if job && jobId */
  }
  return 1;
}



long FM_BusinessTaskPage::onCmdEditTransaction(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j && AE_Job_GetId(j)>0) {
      int jt;
      AQDB_ID jid;

      jid=AE_Job_GetId(j);
      jt=AE_Job_GetType(j);
      if (jt!=AE_Job_TypeInvoice &&
	  jt!=AE_Job_TypePayment) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Selection Error"),
			    "%s",
			    I18N("Please select an invoice or a payment to create a "
				 "booking transaction for."));
	return 1;
      }
      else {
	AQDB_ID tid;

	tid=AE_Job_GetTransactionId(j);
	if (tid==0) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Internal Error"),
			      "%s",
			      I18N("No transaction stored for this job."));
	  return 1;
	}
	else {
	  AE_TRANSACTION *t;
	  int rv;

	  rv=AE_Book_ReadTransaction(b, AE_Book_TableType_Transaction, tid, &t);
	  if (rv<0) {
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Could not read transaction (%d)"), rv);
	    return 1;
	  }

          rv=AE_Book_ReadTransactionSubObjects(b, t);
	  if (rv<0) {
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Could not read transaction (%d)"), rv);
	    AE_Transaction_free(t);
	    return 1;
	  }

	  /* edit transaction */
	  if (!FF_EditTransaction::editTransaction(app, this,
						   I18N("Edit Transaction"),
						   t)) {
	    DBG_ERROR(0, "Aborted");
	    AE_Transaction_free(t);
	    return 1;
	  }
	  else {
	    AE_TRANSACTION *tOld;

	    /* we will now lock the database, re-read the job, store the transaction
	     * and write back the job with the transaction id set.
	     */
	    rv=AE_Book_BeginEdit(b, 0);
	    if (rv<0) {
	      DBG_ERROR(0, "Error on begin edit: %d", rv);
	      AE_Transaction_free(t);
	      return 1;
	    }

	    rv=AE_Book_ReadTransaction(b, AE_Book_TableType_Transaction, tid, &tOld);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Database Error"),
				  I18N("Could not read job from database (%d)"),
				  rv);
	      AE_Transaction_free(tOld);
	      AE_Transaction_free(t);
	      return 1;
	    }

	    rv=AE_Book_ReadTransactionSubObjects(b, t);
	    if (rv<0) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Database Error"),
				  I18N("Could not read transaction (%d)"), rv);
	      AE_Transaction_free(tOld);
	      AE_Transaction_free(t);
	      return 1;
	    }

	    rv=AE_Book_ApplyTransaction(b, t, tOld, 0);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Database Error"),
				  I18N("Could not store transaction (%d)"),
				  rv);
	      AE_Transaction_free(tOld);
	      AE_Transaction_free(t);
	      return 1;
	    }

	    /* unlock database */
	    rv=AE_Book_EndEdit(b, 0);
	    if (rv<0) {
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Database Error"),
				  I18N("Could not apply changes to database (%d)"),
				  rv);
	      AE_Transaction_free(tOld);
	      AE_Transaction_free(t);
	      return 1;
	    }

	    AE_Transaction_free(tOld);
	    AE_Transaction_free(t);

	    /* signal changes */
	    app->signalDbReload(AE_Book_TableType_Transaction);
	    app->signalDbReload(AE_Book_TableType_Split);
	    app->signalDbReload(AE_Book_TableType_AccountBalance);
	    app->signalDbReload(AE_Book_TableType_Task);

	    m_taskTree->unselectAll();
	    m_taskTree->makeJobVisible(jid);
	    m_taskTree->setCurrentJob(jid, TRUE);
	  } /* if editTransaction */
	} /* if transaction id */
      } /* if invoice or payment */
    } /* if job and job id */
  } /* if book */
  return 1;
}



long FM_BusinessTaskPage::onCmdEditJob(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j && AE_Job_GetId(j)>0) {
      bool res=false;
      int rv;
      AE_JOB *oldJ=NULL;
      AQDB_ID jid;

      jid=AE_Job_GetId(j);
      rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, jid, &oldJ);
      if (rv<0) {
	DBG_ERROR(0, "Error reading job from database (%d)", rv);
        return 1;
      }

      rv=AE_Book_ReadJobSubObjects(b, oldJ, m_isBuy);
      if (rv<0) {
	DBG_ERROR(0, "Error reading job from database (%d)", rv);
        AE_Job_free(oldJ);
        return 1;
      }

      switch(AE_Job_GetType(j)) {
      case AE_Job_TypeUnknown:
      case AE_Job_TypeNone:
	break;
      case AE_Job_TypeRequest:
	break;
      case AE_Job_TypeOffer:
	break;
      case AE_Job_TypeOrder:
	res=FF_EditJob::editJob(app, this, I18N("Edit Order"), j, m_isBuy /* incoming means: buy */);
	break;
      case AE_Job_TypeConfirmation:
	break;
      case AE_Job_TypeRelease:
	break;
      case AE_Job_TypeDelivery:
	res=FF_EditJob::editJob(app, this, I18N("Edit Delivery"), j, m_isBuy /* incoming means: buy */);
	break;
      case AE_Job_TypeInvoice:
	res=FF_EditJob::editJob(app, this, I18N("Edit Invoice"), j, m_isBuy /* incoming means: buy */);
	break;
      case AE_Job_TypePayment:
	res=FF_EditPayment::editPayment(app, this, I18N("Edit Payment"), j, m_isBuy);
	break;
      case AE_Job_TypeReminder:
	break;
      case AE_Job_TypeCancellation:
	break;
      }

      if (res) {
	int rv;

	rv=AE_Book_ApplyJob(b, j, oldJ, 1);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Job_free(oldJ);
	}
	AE_Job_free(oldJ);

	rebuild();
        m_taskTree->unselectAll();
	m_taskTree->makeJobVisible(jid);
	m_taskTree->setCurrentJob(jid, TRUE);
      }
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdCreateTransfer(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;
    AE_COMMODITY_LIST *cl;
    const char *s;
    int rv;

    cl=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
    if (rv<0) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Database Error"),
			  I18N("Could not load commodity list (error: %d)"),
			  rv);
      AE_Commodity_List_free(cl);
      return 1;
    }

    j=m_taskTree->getCurrentJob();
    if (j && AE_Job_GetId(j)>0) {
      if (AE_Job_GetType(j)!=AE_Job_TypeInvoice) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Not an Invoice"),
			    "%s",
			    I18N("Please select an invoice to create a transfer for."));
	AE_Commodity_List_free(cl);
	return 1;
      }
      else {
	AE_STATEMENT *st;
	const AQDB_VALUE *v;

	st=AE_Statement_new();
	AE_Statement_SetTextKey(st, 51);
	v=AE_Job_GetOpenSum(j);
	if (v) {
	  AQDB_ID comId;

	  AE_Statement_SetValue(st, v);
	  comId=AE_Job_GetCommodityId(j);
	  if (comId) {
	    AE_COMMODITY *co;

	    co=AE_Commodity_List_GetById(cl, comId);
	    if (co) {
	      s=AE_Commodity_GetCode(co);
	      if (s && *s)
                AE_Statement_SetCurrency(st, s);
	    }
	  }
	}

	AE_Statement_SetPartnerId(st, AE_Job_GetPartnerId(j));
	s=AE_Job_GetPurpose(j);
	if (!(s && *s))
          s=AE_Job_GetMemo(j);
	if (s && *s)
	  AE_Statement_SetPurpose(st, s);

	if (FF_EditTransfer::editTransfer(app,
					  FF_EditTransfer::EDITMODE_STRICT,
					  this,
					  I18N("Create Bank Transfer"), st)) {
	  int rv;

	  rv=AE_Book_AddStatement(b, AE_Book_TableType_BankTransferPlanned, st, 1);
	  if (rv<0) {
	    DBG_ERROR(0, "Unable to add transfer (%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Could not add transfer (error: %d)"),
				rv);
	  }
	  app->signalDbReload(AE_Book_TableType_BankTransferPlanned);
	}
	AE_Statement_free(st);
	AE_Commodity_List_free(cl);
      }
    }
  }

  return 1;
}



int FM_BusinessTaskPage::_modifyJobFlags(AE_JOB *j, uint32_t flags, bool checked) {
  FF_App *app;
  AE_BOOK *b;
  int rv;
  AE_JOB *jRead=NULL;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  rv=AE_Book_BeginEdit(b, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error on begin edit: %d", rv);
    FXMessageBox::error(this, MBOX_OK,
			I18N("Database Error"),
			I18N("Could not lock database (%d)"),
			rv);
    return rv;
  }

  rv=AE_Book_ReadJob(b, AE_Book_TableType_Job, AE_Job_GetId(j), &jRead);
  if (rv<0) {
    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    FXMessageBox::error(this, MBOX_OK,
			I18N("Database Error"),
			I18N("Could not read job from database (%d)"),
			rv);
    return rv;
  }

  if (checked) {
    AE_Job_AddFlags(j, flags);
    AE_Job_AddFlags(jRead, flags);
  }
  else {
    AE_Job_SubFlags(j, flags);
    AE_Job_SubFlags(jRead, flags);
  }

  /* write back new job */
  rv=AE_Book_WriteJob(b, AE_Book_TableType_Job, jRead, 0);
  if (rv<0) {
    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    FXMessageBox::error(this, MBOX_OK,
			I18N("Database Error"),
			I18N("Could not store job (%d)"),
			rv);
    AE_Job_free(jRead);
    return rv;
  }
  AE_Job_free(jRead);

  /* unlock database */
  rv=AE_Book_EndEdit(b, 0);
  if (rv<0) {
    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    FXMessageBox::error(this, MBOX_OK,
			I18N("Database Error"),
			I18N("Could not apply changes to database (%d)"),
			rv);
    return rv;
  }

  rebuild();

  return 0;
}



long FM_BusinessTaskPage::onCmdMarkedAsPrinted(FXObject*, FXSelector, void *ptr) {
  int checked=(FXint)(FXival)ptr;
  AE_JOB *j;

  j=m_taskTree->getCurrentJob();
  if (j && AE_Job_GetId(j)>0) {
    int rv;

    rv=_modifyJobFlags(j, AE_JOB_FLAGS_PRINTED, checked);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdMarkedAsEmailed(FXObject*, FXSelector, void *ptr) {
  int checked=(FXint)(FXival)ptr;
  AE_JOB *j;

  j=m_taskTree->getCurrentJob();
  if (j && AE_Job_GetId(j)>0) {
    int rv;

    rv=_modifyJobFlags(j, AE_JOB_FLAGS_EMAILED, checked);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdCloseTask(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j) {
      AQDB_ID tid;

      tid=AE_Job_GetTaskId(j);
      if (tid==0) {
	DBG_ERROR(0, "No task selected, SNH");
	return 1;
      }


      if (FXMessageBox::question(this, MBOX_YES_NO,
				 I18N("Close Task"),
				 "%s",
				 I18N("Please note that closing the task makes it disappear from\n"
				      "this view but will not delete it from the database.\n"
				      "\n"
				      "Are you sure you want to close the selected task?\n"))!=MBOX_CLICKED_YES) {
	DBG_ERROR(0, "Aborted");
      }
      else {
	AE_TASK *tk=NULL;
        int rv;

	rv=AE_Book_BeginEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "Error on begin edit: %d", rv);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not lock database: %d"), rv);
	  return 1;
	}

	/* read task */
	rv=AE_Book_ReadTask(b, AE_Book_TableType_Task, tid, &tk);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Task_free(tk);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not read task from database: %d"), rv);
	  return 1;
	}

	/* modify task */
	AE_Task_SubFlags(tk, AE_TASK_FLAGS_ACTIVE);

	/* write task */
	rv=AE_Book_WriteTask(b, AE_Book_TableType_Task, tk, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not write task to database: %d"), rv);
	  return 1;
	}

	rv=AE_Book_EndEdit(b, 0);
	if (rv<0) {
	  DBG_ERROR(0, "here (%d)", rv);
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  AE_Task_free(tk);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not unlock database: %d"), rv);
	  return 1;
	}
	AE_Task_free(tk);

	/* signal changes */
	app->signalDbReload(AE_Book_TableType_Job);
      }
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdDeleteJob(FXObject*, FXSelector, void *ptr) {
  AE_JOB *j;

  j=m_taskTree->getCurrentJob();
  if (j && AE_Job_GetId(j)>0) {
    FF_App *app;

    app=dynamic_cast<FF_App*>(getApp());
    if (!app->deleteJob(AE_Job_GetId(j))) {
      DBG_INFO(AE_LOGDOMAIN, "here");
    }
    rebuild();
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdDeleteTask(FXObject*, FXSelector, void *ptr) {
  AE_JOB *j;

  j=m_taskTree->getCurrentJob();
  if (j) {
    AQDB_ID currentTaskId;

    currentTaskId=AE_Job_GetTaskId(j);
    if (currentTaskId) {
      FF_App *app;

      app=dynamic_cast<FF_App*>(getApp());
      if (!app->deleteTask(currentTaskId)) {
	DBG_INFO(AE_LOGDOMAIN, "here");
      }
      rebuild();
    }
  }
  else {
    DBG_ERROR(0, "No current job");
  }

  return 1;
}



long FM_BusinessTaskPage::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuPane paneCreate(this);
    FXMenuPane paneAdd(this);
    FXMenuPane paneJob(this);
    FXMenuCascade *mc;
    FXMenuCommand *c;
    FXMenuCheck *ck;
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();

    /* create "New Task" menu */
    new FXMenuCommand(&paneCreate, I18N("With An Order"), NULL, this, ID_BUTTON_NEW_ORDER);
    new FXMenuCommand(&paneCreate, I18N("With An Invoice"), NULL, this, ID_BUTTON_NEW_INVOICE);
    new FXMenuSeparator(&paneCreate);
    new FXMenuCommand(&paneCreate, I18N("With A Delivery"), NULL, this, ID_BUTTON_NEW_DELIVERY);
    new FXMenuCommand(&paneCreate, I18N("With A Request"), NULL, this, ID_BUTTON_NEW_REQUEST);
    new FXMenuCommand(&paneCreate, I18N("With An Offer"), NULL, this, ID_BUTTON_NEW_OFFER);
    new FXMenuCascade(&pmenu, I18N("Start a New Task..."), NULL, &paneCreate);
    new FXMenuSeparator(&pmenu);

    /* create "Add Job" menu */
    new FXMenuCommand(&paneAdd, I18N("Add Delivery"), NULL, this, ID_BUTTON_ADD_DELIVERY);
    new FXMenuCommand(&paneAdd, I18N("Add Invoice"), NULL, this, ID_BUTTON_ADD_INVOICE);
    new FXMenuCommand(&paneAdd, I18N("Add Payment"), NULL, this, ID_BUTTON_ADD_PAYMENT);
    new FXMenuSeparator(&paneAdd);
    new FXMenuCommand(&paneAdd, I18N("Add Request"), NULL, this, ID_BUTTON_ADD_REQUEST);
    new FXMenuCommand(&paneAdd, I18N("Add Offer"), NULL, this, ID_BUTTON_ADD_OFFER);
    new FXMenuCommand(&paneAdd, I18N("Add Order"), NULL, this, ID_BUTTON_ADD_ORDER);
    new FXMenuCommand(&paneAdd, I18N("Add Confirmation"), NULL, this, ID_BUTTON_ADD_CONFIRMATION);
    new FXMenuCommand(&paneAdd, I18N("Add Release"), NULL, this, ID_BUTTON_ADD_RELEASE);
    new FXMenuCommand(&paneAdd, I18N("Add Reminder"), NULL, this, ID_BUTTON_ADD_REMINDER);
    new FXMenuCommand(&paneAdd, I18N("Add Cancellation"), NULL, this, ID_BUTTON_ADD_CANCELLATION);
    new FXMenuSeparator(&paneAdd);
    new FXMenuCommand(&paneAdd, I18N("Close Task"), NULL, this, ID_BUTTON_CLOSE_TASK);
    new FXMenuSeparator(&paneAdd);
    c=new FXMenuCommand(&paneAdd, I18N("Delete Task"), NULL, this, ID_BUTTON_DEL_TASK);
    mc=new FXMenuCascade(&pmenu, I18N("Current Task"), NULL, &paneAdd);
    if (!(j && AE_Job_GetTaskId(j)))
      mc->disable();
    new FXMenuSeparator(&pmenu);

    new FXMenuCommand(&paneJob, I18N("Edit Job"), NULL, this, ID_BUTTON_EDIT_JOB);
    new FXMenuSeparator(&paneJob);
    c=new FXMenuCommand(&paneJob, I18N("Create PDF"), NULL, this, ID_BUTTON_PRINT_JOB);
#ifdef HAVE_CAIRO
    if (j && AE_Job_GetDocName(j) && *(AE_Job_GetDocName(j)))
      c->disable();
#else
    c->disable();
#endif
    c=new FXMenuCommand(&paneJob, I18N("View PDF"), NULL, this, ID_BUTTON_VIEWPDF);
    if (!(j && AE_Job_GetDocName(j) && *(AE_Job_GetDocName(j))))
      c->disable();
    c=new FXMenuCommand(&paneJob, I18N("Email PDF"), NULL, this, ID_BUTTON_EMAILPDF);
    if (!(j && AE_Job_GetDocName(j) && *(AE_Job_GetDocName(j))))
      c->disable();

    ck=new FXMenuCheck(&paneJob, I18N("Marked as Printed"), this, ID_BUTTON_MARKASPRINTED);
    if (j && (AE_Job_GetFlags(j) & AE_JOB_FLAGS_PRINTED))
      ck->setCheck(TRUE);
    else
      ck->setCheck(FALSE);

    ck=new FXMenuCheck(&paneJob, I18N("Marked as Mailed"), this, ID_BUTTON_MARKASEMAILED);
    if (j && (AE_Job_GetFlags(j) & AE_JOB_FLAGS_EMAILED))
      ck->setCheck(TRUE);
    else
      ck->setCheck(FALSE);

    new FXMenuSeparator(&paneJob);
    c=new FXMenuCommand(&paneJob, I18N("Create Booking Transaction"), NULL, this, ID_BUTTON_NEW_TRANS);
    if (j==NULL || AE_Job_GetTransactionId(j)!=0 ||
	(AE_Job_GetType(j)!=AE_Job_TypeInvoice &&
	 AE_Job_GetType(j)!=AE_Job_TypePayment))
      c->disable();
    c=new FXMenuCommand(&paneJob, I18N("Edit Booking Transaction"), NULL, this, ID_BUTTON_EDIT_TRANS);
    if (j==NULL || AE_Job_GetTransactionId(j)==0 ||
	(AE_Job_GetType(j)!=AE_Job_TypeInvoice &&
	 AE_Job_GetType(j)!=AE_Job_TypePayment))
      c->disable();
    new FXMenuSeparator(&paneJob);
    c=new FXMenuCommand(&paneJob, I18N("Create Transfer"), NULL, this, ID_BUTTON_CREATE_TRANSFER);
    if (j==NULL || AE_Job_GetType(j)!=AE_Job_TypeInvoice || !m_isBuy)
      c->disable();
    new FXMenuSeparator(&paneJob);
    c=new FXMenuCommand(&paneJob, I18N("Delete Job"), NULL, this, ID_BUTTON_DEL_JOB);

    mc=new FXMenuCascade(&pmenu, I18N("Current Job"), NULL, &paneJob);
    if (!(j && AE_Job_GetId(j)))
      mc->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FM_BusinessTaskPage::onTreeOpened(FXObject*, FXSelector, void* ptr) {
  FXFoldingItem *item=(FXFoldingItem*) ptr;

  if (item) {
    item=item->getFirst();
    while(item) {
      m_taskTree->expandTree(item, false);
      item=item->getNext();
    }
  }

  return 1;
}



bool FM_BusinessTaskPage::printJob(AE_JOB *j, AF_PRINT *pr) {
#ifdef HAVE_CAIRO
#endif
  return false;
}



/* TODO: move this to FF_APP */
void FM_BusinessTaskPage::getJobDocumentFolder(int jt, GWEN_BUFFER *sbuf) {
  const char *s;

  /* create name for save file */
  s=AE_GetUserDataFolder();
  assert(s);
  GWEN_Buffer_AppendString(sbuf, s);
  GWEN_Buffer_AppendString(sbuf,
			   GWEN_DIR_SEPARATOR_S "documents"
			   GWEN_DIR_SEPARATOR_S "saved"
			   GWEN_DIR_SEPARATOR_S);
  s=AE_Job_Type_toString(jt);
  GWEN_Buffer_AppendString(sbuf, s);
}



long FM_BusinessTaskPage::onCmdPrintJob(FXObject*, FXSelector, void*) {
#ifdef HAVE_CAIRO
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_JOB *j;

    j=m_taskTree->getCurrentJob();
    if (j && AE_Job_GetId(j)>0) {
      AE_COMMODITY_LIST *cl=NULL;
      int rv;

      cl=AE_Commodity_List_new();
      rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
      if (rv<0) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    "%s",
			    I18N("Could not read commodity list"));
	AE_Commodity_List_free(cl);
	return 1;
      }

      switch(AE_Job_GetType(j)) {
      case AE_Job_TypeUnknown:
      case AE_Job_TypeNone:
	break;
      case AE_Job_TypeRequest:
	break;
      case AE_Job_TypeOffer:
	break;
      case AE_Job_TypeOrder:
	break;
      case AE_Job_TypeConfirmation:
	break;
      case AE_Job_TypeInvoice: {
	AF_PRINT_TABLE_DESCR *descr=NULL;
	AQDB_ID partnerId;

	partnerId=AE_Job_GetPartnerId(j);
	if (partnerId==0) {
	  const AF_PRINT_TABLE_DESCR *cd;

	  cd=AE_GetPsInvoice();
          if (cd)
	    descr=AF_PrintTableDescr_dup(cd);
	}
	else
	  descr=AE_ReadPsInvoiceForPartner(partnerId);

	if (descr && AF_PrintTableDescr_GetDocTemplateName(descr)) {
	  AF_PRINT *pr;
	  AF_PRINT_DOC *tmpl=NULL;
	  int rv;
	  GWEN_BUFFER *tbuf;
	  GWEN_BUFFER *sbuf;
	  const char *s;
          GWEN_TIME *ti;

	  rv=AF_Print_LoadDocTemplate(AF_PrintTableDescr_GetDocTemplateName(descr), &tmpl);
	  if (rv<0) {
	    DBG_ERROR(0, "Template [%s] not found", AF_PrintTableDescr_GetDocTemplateName(descr));
	    AE_Commodity_List_free(cl);
            return 1;
	  }

	  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	  s=AE_GetUserDataFolder();
	  assert(s);
	  GWEN_Buffer_AppendString(tbuf, s);
	  GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S "tmp");
	  /* create path if necessary */
	  rv=GWEN_Directory_GetPath(GWEN_Buffer_GetStart(tbuf), GWEN_PATH_FLAGS_CHECKROOT);
	  if (rv<0) {
	    DBG_ERROR(AE_LOGDOMAIN, "Could not access path [%s]", GWEN_Buffer_GetStart(tbuf));
            GWEN_Buffer_free(tbuf);
	    AE_Commodity_List_free(cl);
	    return 1;
	  }

	  /* create name for save file */
	  sbuf=GWEN_Buffer_new(0, 256, 0, 1);
	  getJobDocumentFolder(AE_Job_GetType(j), sbuf);

	  /* create path if necessary */
	  rv=GWEN_Directory_GetPath(GWEN_Buffer_GetStart(sbuf), GWEN_PATH_FLAGS_CHECKROOT);
	  if (rv<0) {
	    DBG_ERROR(AE_LOGDOMAIN, "Could not access path [%s]", GWEN_Buffer_GetStart(sbuf));
            GWEN_Buffer_free(sbuf);
            GWEN_Buffer_free(tbuf);
	    AE_Commodity_List_free(cl);
	    return 1;
	  }

	  /* add filenname to both buffers */
	  GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S "INV");
	  GWEN_Buffer_AppendString(sbuf, GWEN_DIR_SEPARATOR_S);
	  ti=GWEN_CurrentTime();
	  assert(ti);
	  GWEN_Time_toString(ti, "YYYYMMDDhhmmss", tbuf);
	  GWEN_Time_toString(ti, "YYYYMMDDhhmmss", sbuf);
	  GWEN_Time_free(ti);
	  GWEN_Buffer_AppendString(tbuf, ".pdf");
	  GWEN_Buffer_AppendString(sbuf, ".pdf");

	  pr=AF_Print_newForPdf(GWEN_Buffer_GetStart(tbuf), tmpl);
	  if (pr==NULL) {
	    DBG_ERROR(0, "No PRINT object created.");
	    GWEN_Buffer_free(sbuf);
	    GWEN_Buffer_free(tbuf);
	    AE_Commodity_List_free(cl);
            return 1;
	  }
	  rv=AF_Print_Job(pr, descr, cl, NULL, j);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here(%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Print Error"),
				I18N("Could not print job: %d"), rv);
	    GWEN_Buffer_free(sbuf);
	    GWEN_Buffer_free(tbuf);
	    AE_Commodity_List_free(cl);
	    return 1;
	  }

	  /* show to user, allow preview/save */
	  FF_JobPrinter::run(app, this, I18N("PDF Dialog"), j, descr,
			     GWEN_Buffer_GetStart(tbuf),
			     GWEN_Buffer_GetStart(sbuf));
	  /* write changed job (flags, docname etc may have changed) */
	  rv=AE_Book_WriteJob(b, AE_Book_TableType_Job, j, 1);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here(%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Could not write job information: %d"), rv);
	    GWEN_Buffer_free(sbuf);
	    GWEN_Buffer_free(tbuf);
	    AE_Commodity_List_free(cl);
	    return 1;
	  }

	  GWEN_Buffer_free(sbuf);
	  GWEN_Buffer_free(tbuf);
          rebuild();
	}
	else {
          DBG_ERROR(AE_LOGDOMAIN, "No print table description");
	}
	AE_Commodity_List_free(cl);
	break;
      }
      case AE_Job_TypeReminder:
	break;
      case AE_Job_TypeCancellation:
	break;
      }
    }
  }
#endif

  return 1;
}



long FM_BusinessTaskPage::onCmdViewPdf(FXObject*, FXSelector, void*) {
  AE_JOB *j;

  j=m_taskTree->getCurrentJob();
  if (j && AE_Job_GetId(j)>0) {
    const char *s;

    s=AE_Job_GetDocName(j);
    if (s && *s) {
      FF_App *app;
      GWEN_BUFFER *tbuf;
      GWEN_DB_NODE *db;
      GWEN_DB_NODE *dbJob;
      int rv;
      const char *t;

      app=dynamic_cast<FF_App*>(getApp());
      assert(app);
      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      getJobDocumentFolder(AE_Job_GetType(j), tbuf);
      GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);

      t=strrchr(s, GWEN_DIR_SEPARATOR);
      if (t)
	s=t+1;
      GWEN_Buffer_AppendString(tbuf, s);

      /* prepare DB for args */
      db=GWEN_DB_Group_new("args");
      GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "fileName", GWEN_Buffer_GetStart(tbuf));

      dbJob=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "job");
      AE_Job_toDb(j, dbJob);

      s=AE_GetUserDataFolder();
      if (s && *s)
	GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "userdatadir", s);

      rv=app->viewPdfFile(GWEN_Buffer_GetStart(tbuf), db);
      GWEN_DB_Group_free(db);
      GWEN_Buffer_free(tbuf);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      }
    }
  }

  return 1;
}



long FM_BusinessTaskPage::onCmdEmailPdf(FXObject*, FXSelector, void*) {
  AE_JOB *j;

  j=m_taskTree->getCurrentJob();
  if (j && AE_Job_GetId(j)>0) {
    const char *s;

    s=AE_Job_GetDocName(j);
    if (s && *s) {
      FF_App *app;
      AE_BOOK *b;
      GWEN_BUFFER *tbuf;
      GWEN_DB_NODE *db;
      GWEN_DB_NODE *dbJob;
      int rv;
      AF_PRINT_TABLE_DESCR *ps;
      const char *t;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      assert(b);

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      getJobDocumentFolder(AE_Job_GetType(j), tbuf);
      GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);
      t=strrchr(s, GWEN_DIR_SEPARATOR);
      if (t)
	s=t+1;
      GWEN_Buffer_AppendString(tbuf, s);

      /* prepare DB for args */
      db=GWEN_DB_Group_new("args");
      GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "fileName", GWEN_Buffer_GetStart(tbuf));

      dbJob=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "job");
      AE_Job_toDb(j, dbJob);

      s=AE_GetUserDataFolder();
      if (s && *s)
	GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "userdatadir", s);

      ps=AE_ReadPsInvoiceForPartner(AE_Job_GetPartnerId(j));
      assert(ps);

      s=AF_PrintTableDescr_GetEmailBody(ps);
      if (s && *s) {
	GWEN_BUFFER *bbuf;
    
	bbuf=GWEN_Buffer_new(0, 256, 0, 1);
	rv=AE_Utils_Replace_Vars(s, bbuf, db);
	if (rv<0) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Setup  Error"),
			      "%s",
			      I18N("There seems to be an error in the email body settings in the job templates."));
	  GWEN_Buffer_free(bbuf);
	  AF_PrintTableDescr_free(ps);
	  GWEN_DB_Group_free(db);
	  return 1;
	}
	GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "body", GWEN_Buffer_GetStart(bbuf));
	GWEN_Buffer_free(bbuf);
      }
      AF_PrintTableDescr_free(ps);

      rv=app->emailPdfFile(GWEN_Buffer_GetStart(tbuf), db);
      GWEN_DB_Group_free(db);
      GWEN_Buffer_free(tbuf);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      }
    }
  }

  return 1;
}






void FM_BusinessTaskPage::dbOpened(AE_BOOK *b) {
  if (m_isVisible) {
    rebuild();
    m_needRebuild=false;
  }
  else {
    m_needRebuild=true;
  }
}



void FM_BusinessTaskPage::dbClosing() {
  m_taskTree->clearItems();
}



void FM_BusinessTaskPage::dbClosed() {
}



void FM_BusinessTaskPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_Job ||
      tt==AE_Book_TableType_Task ||
      tt==AE_Book_TableType_None) {
    if (m_isVisible) {
      rebuild();
      m_needRebuild=false;
    }
    else
      m_needRebuild=true;
  }
}



static uint32_t calcTaskFlags(const AE_TASK *tk) {
  AE_JOB *j;
  uint32_t flags=0;
  AQDB_VALUE *v;
  int booked=0;
  int toBook=0;

  v=AQDB_Value_new();

  j=AE_Job_List_First(AE_Task_GetJobs(tk));
  while(j) {
    const AQDB_VALUE *vt;

    switch(AE_Job_GetType(j)) {
    case AE_Job_TypeInvoice:
      vt=AE_Job_GetOpenSum(j);
      if (vt) {
	AQDB_Value_AddValue(v, vt);
	if (!AQDB_Value_IsZero(vt)) {
	  const GWEN_DATE *dd;

	  dd=AE_Job_GetDueDate(j);
	  if (dd) {
	    GWEN_DATE *cd;
            int diff;

	    cd=GWEN_Date_CurrentDate();
	    diff=GWEN_Date_Diff(cd, dd);
	    if (diff>=-2)
	      flags|=FF_JobTreeItem::FLAGS_URGENT;
            GWEN_Date_free(cd);
	  }
	}
      }
      // TODO: determine if the billing is complete or partial
      flags|=FF_JobTreeItem::FLAGS_BILLED_FULL;
      toBook++;
      if (AE_Job_GetTransactionId(j))
        booked++;
      break;

    case AE_Job_TypePayment:
      flags|=FF_JobTreeItem::FLAGS_PAID_PARTIAL;
      toBook++;
      if (AE_Job_GetTransactionId(j))
	booked++;
      break;

    case AE_Job_TypeReminder:
      flags|=FF_JobTreeItem::FLAGS_REMINDER;
      break;

    case AE_Job_TypeDelivery:
      // TODO: determine whether the delivery is full or partial
      flags|=FF_JobTreeItem::FLAGS_DELIVERED_PARTIAL;
      break;

    default:
      break;
    }
    j=AE_Job_List_Next(j);
  }

  if (flags & FF_JobTreeItem::FLAGS_PAID_PARTIAL) {
    if (AQDB_Value_IsZero(v)) {
      flags&=~FF_JobTreeItem::FLAGS_PAID_PARTIAL;
      flags|=FF_JobTreeItem::FLAGS_PAID_FULL;
    }
  }

  if (flags & FF_JobTreeItem::FLAGS_DELIVERED_PARTIAL) {
    if (AE_Item_List_GetCount(AE_Task_GetOpenOrderedItems(tk))==0) {
      /* no open deliveries */
      flags&=~FF_JobTreeItem::FLAGS_DELIVERED_PARTIAL;
      flags|=FF_JobTreeItem::FLAGS_DELIVERED_FULL;
    }
  }

  if (toBook) {
    if (booked==toBook)
      flags|=FF_JobTreeItem::FLAGS_BOOKED_FULL;
    else if (booked)
      flags|=FF_JobTreeItem::FLAGS_BOOKED_PARTIAL;
  }


  AQDB_Value_free(v);

  return flags;
}



static uint32_t calcJobFlags(const AE_JOB *j) {
  uint32_t flags=0;
  const AQDB_VALUE *vt;
  uint32_t jobFlags;
  const char *s;

  jobFlags=AE_Job_GetFlags(j);
  switch(AE_Job_GetType(j)) {
  case AE_Job_TypeInvoice:
    vt=AE_Job_GetOpenSum(j);
    if (vt) {
      if (AQDB_Value_IsZero(vt)) {
	flags|=FF_JobTreeItem::FLAGS_PAID_FULL;
      }
      else {
	const GWEN_DATE *dd;

	dd=AE_Job_GetDueDate(j);
	if (dd) {
	  GWEN_DATE *cd;
	  int diff;

	  cd=GWEN_Date_CurrentDate();
	  diff=GWEN_Date_Diff(cd, dd);
	  if (diff>=-2)
	    flags|=FF_JobTreeItem::FLAGS_URGENT;
	  GWEN_Date_free(cd);
	}
      }
    }

    if (AE_Job_GetTransactionId(j))
      flags|=FF_JobTreeItem::FLAGS_BOOKED_FULL;
    break;

  case AE_Job_TypePayment:
    if (AE_Job_GetTransactionId(j))
      flags|=FF_JobTreeItem::FLAGS_BOOKED_FULL;
    break;

  case AE_Job_TypeReminder:
    flags|=FF_JobTreeItem::FLAGS_REMINDER;
    break;

  case AE_Job_TypeDelivery:
    // TODO: determine whether the delivery is full or partial
    flags|=FF_JobTreeItem::FLAGS_DELIVERED_PARTIAL;
    break;

  default:
    break;
  }

  if (jobFlags & AE_JOB_FLAGS_PRINTED)
    flags|=FF_JobTreeItem::FLAGS_PRINTED;
  if (jobFlags & AE_JOB_FLAGS_EMAILED)
    flags|=FF_JobTreeItem::FLAGS_MAILED;

  s=AE_Job_GetDocName(j);
  if (s && *s)
    flags|=FF_JobTreeItem::FLAGS_SAVED;

  return flags;
}






void FM_BusinessTaskPage::rebuild() {
  FF_App *app;
  AE_BOOK *b;
  AE_JOB *j;
  AQDB_ID currentJobId=0;
  AQDB_ID currentTaskId=0;

  j=m_taskTree->getCurrentJob();
  if (j) {
    currentJobId=AE_Job_GetId(j);
    currentTaskId=AE_Job_GetTaskId(j);
  }
  else {
    DBG_ERROR(0, "No current job");
  }
  m_taskTree->clearItems();
  m_taskTree->reload();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TASK_LIST *tl;
    int rv;

    tl=AE_Task_List_new();
    rv=AE_Book_GetOpenTasks(b, m_isBuy?1:0, tl);
    if (rv<0) {
      DBG_ERROR(0, "Error getting open bills (%d)", rv);
    }
    else {
      AE_TASK *tk;

      tk=AE_Task_List_First(tl);
      while(tk) {
	AE_JOB *j;
        uint32_t flags;

        flags=calcTaskFlags(tk);
	m_taskTree->addTask(tk, flags);
	while( (j=AE_Job_List_First(AE_Task_GetJobs(tk))) ) {
	  AE_Job_List_Del(j);
	  m_taskTree->addJob(j, calcJobFlags(j));
	}

	tk=AE_Task_List_Next(tk);
      }
    }
    AE_Task_List_free(tl);

    /* sort list */
    m_taskTree->sort();
    //m_taskTree->expandItems();
  }
  else {
    DBG_ERROR(0, "No book (%s)", m_isBuy?"incoming":"outgoing");
  }

  if (currentJobId) {
    FF_JobTreeItem *item=m_taskTree->findJobItem(currentJobId);
    if (item) {
      m_taskTree->unselectAll();
      m_taskTree->makeItemVisible(item);
      m_taskTree->setCurrentItem(item);
      item->setSelected(true);
    }
  }
  else if (currentTaskId) {
    FF_JobTreeItem *item=m_taskTree->findTaskItem(currentTaskId);
    if (item) {
      m_taskTree->unselectAll();
      m_taskTree->makeItemVisible(item);
      m_taskTree->setCurrentItem(item);
      item->setSelected(true);
    }
  }
}



void FM_BusinessTaskPage::enter() {
  m_isVisible=true;

  if (m_needRebuild) {
    rebuild();
    m_needRebuild=false;
  }
}



void FM_BusinessTaskPage::leave() {
  m_isVisible=false;
}



void FM_BusinessTaskPage::reset() {
  m_taskTree->clearItems();
}





