/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_BUS_TASKPAGE_HPP
#define FM_BUS_TASKPAGE_HPP


#include "ff_jobtree.hpp"
#include <aqfinance/print/af_print.h>

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_InvoiceTree;
class FM_BusinessPage;


class FM_BusinessTaskPage: public FXHorizontalFrame {
  FXDECLARE(FM_BusinessTaskPage)

public:
  enum {
    ID_JOB_TREE=FXPacker::ID_LAST,
    ID_BUTTON_NEW_REQUEST,
    ID_BUTTON_NEW_OFFER,
    ID_BUTTON_NEW_ORDER,
    ID_BUTTON_NEW_DELIVERY,
    ID_BUTTON_NEW_INVOICE,

    ID_BUTTON_ADD_REQUEST,
    ID_BUTTON_ADD_OFFER,
    ID_BUTTON_ADD_ORDER,
    ID_BUTTON_ADD_CONFIRMATION,
    ID_BUTTON_ADD_RELEASE,
    ID_BUTTON_ADD_DELIVERY,
    ID_BUTTON_ADD_INVOICE,
    ID_BUTTON_ADD_PAYMENT,
    ID_BUTTON_ADD_REMINDER,
    ID_BUTTON_ADD_CANCELLATION,

    ID_BUTTON_EDIT_JOB,
    ID_BUTTON_PRINT_JOB,
    ID_BUTTON_VIEWPDF,
    ID_BUTTON_EMAILPDF,
    ID_BUTTON_MARKASPRINTED,
    ID_BUTTON_MARKASEMAILED,
    ID_BUTTON_NEW_TRANS,
    ID_BUTTON_EDIT_TRANS,
    ID_BUTTON_CREATE_TRANSFER,
    ID_BUTTON_DEL_JOB,
    ID_BUTTON_DEL_TASK,
    ID_BUTTON_CLOSE_TASK,

    ID_LAST
  };

public:
  FM_BusinessTaskPage(bool isBuy,
		      FM_BusinessPage *mainPage,
		      FXComposite *p,
		      FXuint opts=0,
		      FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_BusinessTaskPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void enter();
  void leave();
  void reset();

  long onCmdNewRequest(FXObject*, FXSelector, void*);
  long onCmdNewOffer(FXObject*, FXSelector, void*);
  long onCmdNewOrder(FXObject*, FXSelector, void*);
  long onCmdNewDelivery(FXObject*, FXSelector, void*);
  long onCmdNewInvoice(FXObject*, FXSelector, void*);
  long onCmdNewPayment(FXObject*, FXSelector, void*);

  long onCmdAddRequest(FXObject*, FXSelector, void*);
  long onCmdAddOffer(FXObject*, FXSelector, void*);
  long onCmdAddOrder(FXObject*, FXSelector, void*);
  long onCmdAddConfirmation(FXObject*, FXSelector, void*);
  long onCmdAddRelease(FXObject*, FXSelector, void*);
  long onCmdAddDelivery(FXObject*, FXSelector, void*);
  long onCmdAddInvoice(FXObject*, FXSelector, void*);
  long onCmdAddPayment(FXObject*, FXSelector, void*);
  long onCmdAddReminder(FXObject*, FXSelector, void*);
  long onCmdAddCancellation(FXObject*, FXSelector, void*);

  long onCmdEditJob(FXObject*, FXSelector, void*);
  long onCmdCreateTransaction(FXObject*, FXSelector, void*);
  long onCmdEditTransaction(FXObject*, FXSelector, void*);
  long onCmdCreateTransfer(FXObject*, FXSelector, void*);

  long onCmdPrintJob(FXObject*, FXSelector, void*);
  long onCmdViewPdf(FXObject*, FXSelector, void*);
  long onCmdEmailPdf(FXObject*, FXSelector, void*);

  long onCmdMarkedAsPrinted(FXObject*, FXSelector, void*);
  long onCmdMarkedAsEmailed(FXObject*, FXSelector, void*);

  long onCmdDeleteJob(FXObject*, FXSelector, void *ptr);
  long onCmdDeleteTask(FXObject*, FXSelector, void *ptr);
  long onCmdCloseTask(FXObject*, FXSelector, void *ptr);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeRightBtnPress(FXObject*, FXSelector, void*);
  long onTreeOpened(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

protected:
  FM_BusinessTaskPage();

  int addJob(int createJobType, const char *title, bool doCopyItems);
  int createTask(int createJobType, const char *title);

  bool printJob(AE_JOB *j, AF_PRINT *pr);
  void getJobDocumentFolder(int jt, GWEN_BUFFER *sbuf);

  int _modifyJobFlags(AE_JOB *j, uint32_t flags, bool checked);

  void rebuild();

  bool m_isBuy;
  FM_BusinessPage *m_mainPage;

  FF_JobTree *m_taskTree;

  bool m_needRebuild;
  bool m_isVisible;
};




#endif

