/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_businesspage.hpp"
#include "fm_bus_taskpage.hpp"
#include "ff_app.hpp"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_BusinessPage) FM_BusinessPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_BusinessPage::ID_TABBOOK,
	    FM_BusinessPage::onTabClicked)
};


            
FXIMPLEMENT(FM_BusinessPage, FXHorizontalFrame,
            FM_BusinessPageMap, ARRAYNUMBER(FM_BusinessPageMap))





FM_BusinessPage::FM_BusinessPage(FXComposite *p,
				 FXuint opts,
				 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_tabBook(NULL)
,m_lastEnteredTab(-1)
,m_incomingTaskPage(NULL)
,m_outgoingTaskPage(NULL)
,m_openTasks(NULL)
{

  m_tabBook=new FXTabBook(this, this, ID_TABBOOK,
			  LAYOUT_FILL_X|LAYOUT_FILL_Y);

  /* create tab 1 */
  new FXTabItem(m_tabBook, I18N("Buys"), NULL);
  m_incomingTaskPage=new FM_BusinessTaskPage(true, this, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* create tab 2 */
  new FXTabItem(m_tabBook, I18N("Sells"), NULL);
  m_outgoingTaskPage=new FM_BusinessTaskPage(false, this, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
}



FM_BusinessPage::FM_BusinessPage()
:FXHorizontalFrame()
,m_tabBook(NULL)
,m_lastEnteredTab(-1)
,m_incomingTaskPage(NULL)
,m_outgoingTaskPage(NULL)
,m_openTasks(NULL)
{
}



FM_BusinessPage::~FM_BusinessPage() {
}



void FM_BusinessPage::init(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Init");
  if (m_incomingTaskPage)
    m_incomingTaskPage->init(db);
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->init(db);
}



void FM_BusinessPage::fini(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Fini");
  if (m_incomingTaskPage)
    m_incomingTaskPage->fini(db);
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->fini(db);
}



void FM_BusinessPage::dbOpened(AE_BOOK *bk) {
  if (m_incomingTaskPage)
    m_incomingTaskPage->dbOpened(bk);
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->dbOpened(bk);
}



void FM_BusinessPage::dbClosing() {
  if (m_incomingTaskPage)
    m_incomingTaskPage->dbClosing();
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->dbClosing();
}



void FM_BusinessPage::dbClosed() {
  if (m_incomingTaskPage)
    m_incomingTaskPage->dbClosed();
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->dbClosed();
}



void FM_BusinessPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_Job ||
      tt==AE_Book_TableType_Partner ||
      tt==AE_Book_TableType_Commodity ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AE_TASK_LIST *tl;
      char qexpr[64];
      int rv;

      /* only load active tasks */
      snprintf(qexpr, sizeof(qexpr)-1, "$flags ?isset %d", AE_TASK_FLAGS_ACTIVE);
      qexpr[sizeof(qexpr)-1]=0;

      tl=AE_Task_List_new();
      rv=AE_Book_QueryTaskList(b, AE_Book_TableType_Task, 0, qexpr, tl);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Task_List_free(tl);
      }
      else {
	AE_Task_List_free(m_openTasks);
	m_openTasks=tl;
      }
    }
  }

  if (m_incomingTaskPage)
    m_incomingTaskPage->dbReload(tt);
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->dbReload(tt);
}



long FM_BusinessPage::onTabClicked(FXObject*, FXSelector, void* ptr) {
  FXint idx=(FXint)(FXival)ptr;

  if (m_lastEnteredTab!=-1) {
    switch(m_lastEnteredTab) {
    case 0:
      if (m_incomingTaskPage)
	m_incomingTaskPage->leave();
      break;
    case 1: 
      if (m_outgoingTaskPage)
	m_outgoingTaskPage->leave();
      break;
    default:
      break;
    }
  }

  switch(idx) {
  case 0:
    if (m_incomingTaskPage)
      m_incomingTaskPage->enter();
    break;
  case 1: 
    if (m_outgoingTaskPage)
      m_outgoingTaskPage->enter();
    break;
  default:
    break;
  }

  m_lastEnteredTab=idx;

  return 1;
}



void FM_BusinessPage::enter() {
  switch(m_tabBook->getCurrent()) {
  case 0:
    if (m_incomingTaskPage)
      m_incomingTaskPage->enter();
    break;
  case 1: 
    if (m_outgoingTaskPage)
      m_outgoingTaskPage->enter();
    break;
  default:
    break;
  }
}



void FM_BusinessPage::leave() {
  switch(m_tabBook->getCurrent()) {
  case 0:
    if (m_incomingTaskPage)
      m_incomingTaskPage->leave();
    break;
  case 1: 
    if (m_outgoingTaskPage)
      m_outgoingTaskPage->leave();
    break;
  default:
    break;
  }
}



void FM_BusinessPage::reset() {
  if (m_incomingTaskPage)
    m_incomingTaskPage->reset();
  if (m_outgoingTaskPage)
    m_outgoingTaskPage->reset();
}




