/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_partners.hpp"
#include "fm_partnerpage.hpp"
#include "ff_app.hpp"


#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FM_Partners::FM_Partners(FF_ModuleManager *mm)
:FF_Module(mm, FF_Module::MODULE_PARTNERS, I18N("Partners"))
,m_page(NULL)
,m_config(NULL){
  m_icon=getApp()->getIcon("partners");
  if (m_icon)
    m_icon->create();
}



FM_Partners::~FM_Partners() {
}



void FM_Partners::dump(int indent) {
}



void FM_Partners::init(FXComposite *parentForView) {
  int rv;

  DBG_INFO(0, "Init");
  rv=GWEN_ConfigMgr_GetGroup(AE_GetConfigManager(),
                             "modules",
                             "partners",
                             &m_config);
  if (rv<0) {
    DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
    return;
  }


  m_page=new FM_PartnerPage(parentForView, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_page->create();
  m_page->init(m_config);
  m_page->show();
}



void FM_Partners::fini() {
  DBG_INFO(0, "Fini");
  if (m_config) {
    if (m_page) {
      int rv;

      rv=GWEN_ConfigMgr_LockGroup(AE_GetConfigManager(),
                                  "modules",
                                  "partners");
      if (rv<0) {
        DBG_ERROR(0, "Unable to lock configuration group (%d)", rv);
      }
      else {
	GWEN_DB_ClearGroup(m_config, NULL);
        m_page->fini(m_config);

        rv=GWEN_ConfigMgr_SetGroup(AE_GetConfigManager(),
                                   "modules",
                                   "partners",
                                   m_config);
        if (rv<0) {
          DBG_ERROR(0, "Error getting configuration group (%d)", rv);
        }

        rv=GWEN_ConfigMgr_UnlockGroup(AE_GetConfigManager(),
                                      "modules",
                                      "partners");
      }
    } /* if page */
    GWEN_DB_Group_free(m_config);
    m_config=NULL;
  }

  if (m_page) {
    delete m_page;
    m_page=NULL;
  }
}



void FM_Partners::enter() {
  m_page->enter();
}



void FM_Partners::leave() {
  m_page->leave();
}



void FM_Partners::reset() {
  m_page->reset();
}



void FM_Partners::dbOpened(AE_BOOK *bk) {
  m_page->dbOpened(bk);
}



void FM_Partners::dbClosing() {
  m_page->dbClosing();
}



void FM_Partners::dbClosed() {
  m_page->dbClosed();
}



void FM_Partners::dbReload(AE_BOOK_TABLE_TYPE tt) {
  m_page->dbReload(tt);
}






