/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_greportstab.hpp"
#include "fm_reportspage.hpp"
#include "ff_vehicletree.hpp"
#include "ff_report2settingstree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_edit_report2_settings.hpp"
#include "ff_greportwiz.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_GraphicsReportsTab) FM_GraphicsReportsTabMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_ADDGROUP,
            FM_GraphicsReportsTab::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_EDITGROUP,
            FM_GraphicsReportsTab::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_DELGROUP,
            FM_GraphicsReportsTab::onCmdDelGroup),

  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_ADDSETTINGS,
            FM_GraphicsReportsTab::onCmdAddSettings),
  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_EDITSETTINGS,
            FM_GraphicsReportsTab::onCmdEditSettings),
  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_DELSETTINGS,
            FM_GraphicsReportsTab::onCmdDelSettings),

  FXMAPFUNC(SEL_COMMAND, FM_GraphicsReportsTab::ID_BUTTON_MKREPORT,
            FM_GraphicsReportsTab::onCmdMkReport),

  FXMAPFUNC(SEL_CHANGED, FM_GraphicsReportsTab::ID_SETTINGS_TREE,
            FM_GraphicsReportsTab::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_GraphicsReportsTab::ID_SETTINGS_TREE,
            FM_GraphicsReportsTab::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_GraphicsReportsTab::ID_SETTINGS_TREE,
            FM_GraphicsReportsTab::onTreeRightBtnPress),

};



FXIMPLEMENT(FM_GraphicsReportsTab, FF_ModTabPage,
            FM_GraphicsReportsTabMap, ARRAYNUMBER(FM_GraphicsReportsTabMap))






FM_GraphicsReportsTab::FM_GraphicsReportsTab(const FXString &pageId,
                                             FM_ReportsPage *mainPage,
                                             FXuint opts,
                                             FXint x, FXint y, FXint w, FXint h)
:FF_ModTabPage(pageId, mainPage, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_reportSettingsTree(NULL)
,m_isEditing(false)
{
  FXIcon *ic;
  FF_App *app;

  m_closeable=false;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_reportSettingsTree=new FF_Report2SettingsTree(this, this, ID_SETTINGS_TREE,
                                                  LAYOUT_SIDE_LEFT |
                                                  LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                                  FOLDINGLIST_EXTENDEDSELECT |
                                                  FOLDINGLIST_SHOWS_LINES |
                                                  FOLDINGLIST_SHOWS_BOXES |
                                                  FOLDINGLIST_ROOT_BOXES |
                                                  FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_reportSettingsTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_reportSettingsTree->setIconClosed(ic);

  // TODO: add buttons

}




FM_GraphicsReportsTab::FM_GraphicsReportsTab()
:FF_ModTabPage()
,m_mainPage(NULL)
,m_reportSettingsTree(NULL)
,m_isEditing(false)
{
}



FM_GraphicsReportsTab::~FM_GraphicsReportsTab() {
}



void FM_GraphicsReportsTab::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");

  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "graphReportsPage");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_reportSettingsTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_reportSettingsTree->getHeaderSize(i));
      m_reportSettingsTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_reportSettingsTree->setHeaderArrowDir(i, TRUE);
        else
          m_reportSettingsTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_GraphicsReportsTab::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "graphReportsPage");
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_reportSettingsTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_reportSettingsTree->getHeaderSize(i));
    b=m_reportSettingsTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_GraphicsReportsTab::create(){
  FF_ModTabPage::create();
  m_reportSettingsTree->create();
}



void FM_GraphicsReportsTab::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_GraphReportsTab: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_reportSettingsTree->dump(indent+2);
}



long FM_GraphicsReportsTab::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_reportSettingsTree->getFirstSelectedGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    m_isEditing=true;
    if (FF_EditGroup::editGroup(app,
				this,
                                I18N("Add Graphics Report Group"),
                                AE_Book_TableType_GraphicReportGroup,
                                gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_GraphicReportGroup, gr, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_reportSettingsTree->addGroup(gr);
      item=m_reportSettingsTree->findGroupItem(gr);
      assert(item);
      m_reportSettingsTree->makeItemVisible(item);
      m_reportSettingsTree->unselectAll();
      item->setSelected(TRUE);
      m_reportSettingsTree->setCurrentItem(item, TRUE);
      m_isEditing=false;
      app->signalDbReload(AE_Book_TableType_GraphicReportGroup);
    }
    else
      AE_Group_free(gr);
    m_isEditing=false;
  }

  return 1;
}



long FM_GraphicsReportsTab::onCmdEditGroup(FXObject*, FXSelector, void*) {
  AE_GROUP *gr;

  gr=m_reportSettingsTree->getFirstSelectedGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      m_isEditing=true;
      if (FF_EditGroup::editGroup(app,
                                  this,
                                  I18N("Edit Graphics Report Group"),
                                  AE_Book_TableType_GraphicReportGroup,
                                  gr)) {
        int rv;
        FXString t;

        rv=AE_Book_WriteGroup(b, AE_Book_TableType_GraphicReportGroup, gr, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to edit group (%d)", rv);
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Database Error"),
                              I18N("Unable to write object to database: %d"), rv);
        }

        m_isEditing=false;
        app->signalDbReload(AE_Book_TableType_GraphicReportGroup);
      }
      m_isEditing=false;
    }
  }
  return 1;
}



long FM_GraphicsReportsTab::onCmdDelGroup(FXObject*, FXSelector, void*) {
#if 0
  AE_GROUP *gr;

  gr=m_reportSettingsTree->getFirstSelectedGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint res;

      res=FXMessageBox::question(this, MBOX_YES_NO,
                                 I18N("Delete Group"),
                                 "%s",
                                 I18N("Are you sure you want to delete the selected "
                                      "group?"));
      if (res!=MBOX_CLICKED_YES) {
        DBG_ERROR(0, "Aborted");
      }
      else {
        app->deleteGraphicReportGroup(AE_Group_GetId(gr));
        app->signalDbReload(AE_Book_TableType_GraphicReportGroup);
      }
    }
  }
#else
# warning Need to implemente this!
  FXMessageBox::error(this, MBOX_OK,
                      I18N("Not Implemented"),
                      "%s",
                      I18N("The function your are calling is not yet implemented."));
#endif

  return 1;
}



long FM_GraphicsReportsTab::onCmdAddSettings(FXObject*, FXSelector, void*) {
#if 0
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AF_REPORT2_SETTINGS *settings;
    AE_GROUP *gr;

    settings=AF_Report2_Settings_new();
    AF_Report2_Settings_SetDefaults(settings);
    gr=m_reportSettingsTree->getFirstSelectedGroup();
    if (gr)
      AF_Report2_Settings_SetGroupId(settings, AE_Group_GetId(gr));
    else
      AF_Report2_Settings_SetGroupId(settings, 0);
    if (FF_Edit_Report2Settings::editSettings(this, I18N("Add Report Profile"), settings)) {
      int rv;
      AE_DATA_10K *data;

      data=AE_Data10k_new();
      rv=AF_Report2_Settings_toData10k(settings, data);
      if (rv<0) {
        DBG_ERROR(0, "Unable to translate settings to data10k object (%d)", rv);
      }
      else {
        rv=AE_Book_AddData10k(b, AE_Book_TableType_GraphicReportSettings, data, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add settings (%d)", rv);
        }
      }
      AE_Data10k_free(data);
      app->signalDbReload(AE_Book_TableType_GraphicReportSettings);
    }
    AF_Report2_Settings_free(settings);
  }
#else
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AF_REPORT2_SETTINGS *settings;
    AE_GROUP *gr;

    settings=AF_Report2_Settings_new();
    AF_Report2_Settings_SetDefaults(settings);
    gr=m_reportSettingsTree->getFirstSelectedGroup();
    if (gr)
      AF_Report2_Settings_SetGroupId(settings, AE_Group_GetId(gr));
    else
      AF_Report2_Settings_SetGroupId(settings, 0);
    if (FF_GraphReportWiz::createProfile(this,
                                         I18N("Add Report Profile"),
                                         settings,
                                         DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|FF_GraphReportWiz::WITH_ADMIN_PAGE)) {
      int rv;
      AE_DATA_10K *data;

      DBG_INFO(AE_LOGDOMAIN, "Adding report profile");
      data=AE_Data10k_new();
      rv=AF_Report2_Settings_toData10k(settings, data);
      if (rv<0) {
        DBG_ERROR(0, "Unable to translate settings to data10k object (%d)", rv);
      }
      else {
        rv=AE_Book_AddData10k(b, AE_Book_TableType_GraphicReportSettings, data, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add settings (%d)", rv);
        }
      }
      AE_Data10k_free(data);
      app->signalDbReload(AE_Book_TableType_GraphicReportSettings);
    }
    AF_Report2_Settings_free(settings);
  }

#endif
  return 1;
}



long FM_GraphicsReportsTab::onCmdEditSettings(FXObject*, FXSelector, void*) {
  AF_REPORT2_SETTINGS *settings;

  settings=m_reportSettingsTree->getFirstSelectedSettings();
  if (settings) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_Edit_Report2Settings::editSettings(this, I18N("Edit Report Profile"), settings)) {
        int rv;
        AE_DATA_10K *data;
  
        data=AE_Data10k_new();
        rv=AF_Report2_Settings_toData10k(settings, data);
        if (rv<0) {
          DBG_ERROR(0, "Unable to translate settings to data10k object (%d)", rv);
        }
        else {
          rv=AE_Book_WriteData10k(b, AE_Book_TableType_GraphicReportSettings, data, 1);
          if (rv<0) {
            DBG_ERROR(0, "Unable to write settings back (%d)", rv);
          }
        }
        AE_Data10k_free(data);
        app->signalDbReload(AE_Book_TableType_GraphicReportSettings);
      }
    }
    //AF_Report2_Settings_free(settings);
  }

  return 1;
}



long FM_GraphicsReportsTab::onCmdDelSettings(FXObject*, FXSelector, void*) {
#if 0
  AE_VEHICLE *veh;

  veh=m_reportSettingsTree->getFirstSelectedVehicle();
  if (veh) {
    FF_App *app;
    AQDB_ID id;

    app=dynamic_cast<FF_App*>(getApp());
    id=AE_Vehicle_GetId(veh);
    if (id)
      app->deleteVehicle(id);
    app->signalDbReload(AE_Book_TableType_Vehicle);
  }

#else
# warning Need to implemente this!
#endif

  return 1;
}



long FM_GraphicsReportsTab::onCmdMkReport(FXObject*, FXSelector, void*) {
  AF_REPORT2_SETTINGS *settings;

  settings=m_reportSettingsTree->getFirstSelectedSettings();
  if (settings) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_Edit_Report2Settings::editSettings(this, I18N("Create Report from Profile"), settings)) {
        m_mainPage->openGraphicsReportPage(settings);
      }
    }
    //AF_Report2_Settings_free(settings);
  }

  return 1;
}



long FM_GraphicsReportsTab::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        //FF_VehicleTreeItem *baItem=dynamic_cast<FF_VehicleTreeItem*>(item);
        //AE_VEHICLE *veh=baItem->getVehicle();

      }
    }
  }
  return 1;
}



long FM_GraphicsReportsTab::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item && item->isDataItem()) {
      FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(item);
      if (baItem) {
        AF_REPORT2_SETTINGS *settings;

        settings=baItem->getSettings();
        if (settings) {
          AF_REPORT2_SETTINGS *execSettings;

          execSettings=AF_Report2_Settings_dup(settings);
          if (FF_Edit_Report2Settings::editSettings(this, I18N("Create Report from Profile"), execSettings)) {
            DBG_ERROR(AE_LOGDOMAIN, "Opening graphics report tab");
            m_mainPage->openGraphicsReportPage(execSettings);
          }
          else {
            DBG_ERROR(AE_LOGDOMAIN, "Aborted");
          }
        }
      }
    }
  }
  return 1;
}



long FM_GraphicsReportsTab::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_GROUP *gr;
  AF_REPORT2_SETTINGS *settings;

  gr=m_reportSettingsTree->getCurrentGroup();
  settings=m_reportSettingsTree->getCurrentSettings();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    c=new FXMenuCommand(&pmenu, I18N("Create Report from Profile"), NULL, this, ID_BUTTON_MKREPORT);
    if (!settings) c->disable();
    new FXMenuSeparator(&pmenu);

    new FXMenuCommand(&pmenu, I18N("Add Group"), NULL, this, ID_BUTTON_ADDGROUP);
    new FXMenuCommand(&pmenu, I18N("Add Settings"), NULL, this, ID_BUTTON_ADDSETTINGS);

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Group"), NULL, this, ID_BUTTON_EDITGROUP);
    if (!gr) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Group"), NULL, this, ID_BUTTON_DELGROUP);
    if (!gr) c->disable();

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Settings"), NULL, this, ID_BUTTON_EDITSETTINGS);
    if (!settings) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Settings"), NULL, this, ID_BUTTON_DELSETTINGS);
    if (!settings) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_GraphicsReportsTab::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_GraphicsReportsTab::dbClosing() {
  m_reportSettingsTree->clearItems();
}



void FM_GraphicsReportsTab::dbClosed() {
}



void FM_GraphicsReportsTab::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_GraphicReportGroup ||
      tt==AE_Book_TableType_GraphicReportSettings ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
    AE_BOOK *b;
    AE_GROUP *gr;
    AF_REPORT2_SETTINGS *settings;
    AQDB_ID currentSettingsId=0;
    AQDB_ID currentGrId=0;
  
    gr=m_reportSettingsTree->getCurrentGroup();
    if (gr)
      currentGrId=AE_Group_GetId(gr);
    settings=m_reportSettingsTree->getCurrentSettings();
    if (settings)
      currentSettingsId=AF_Report2_Settings_GetId(settings);

    m_reportSettingsTree->clearItems();
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AE_GROUP_TREE *gt;
      int rv;
      AE_DATA_10K_LIST *tkList;
      int i;

      gt=AE_Group_Tree_new();
      rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_GraphicReportGroup, gt);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_reportSettingsTree->addGroups(gt);
      }
      AE_Group_Tree_free(gt);

      /* read vehicles */
      tkList=AE_Data10k_List_new();
      rv=AE_Book_ReadData10kList(b, AE_Book_TableType_GraphicReportSettings, tkList);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
        AE_DATA_10K *tk;

        tk=AE_Data10k_List_First(tkList);
        while (tk) {
          AF_REPORT2_SETTINGS *settings=NULL;

          rv=AF_Report2_Settings_fromData10k(tk, &settings);
          if (rv<0) {
            DBG_ERROR(0, "here (%d)", rv);
          }
          else {
            /* add settings */
            m_reportSettingsTree->addSettings(settings);
          }
          tk=AE_Data10k_List_Next(tk);
        }
      }
      AE_Data10k_List_free(tkList);

      for (i=0; i<m_reportSettingsTree->getNumHeaders(); i++) {
	FXbool b;
    
	b=m_reportSettingsTree->getHeaderArrowDir(i);
	if (b!=MAYBE) {
	  if (b==TRUE)
	    m_reportSettingsTree->sortByColumn(i, true);
	  else
	    m_reportSettingsTree->sortByColumn(i, false);
	  break;
	}
      }

      m_reportSettingsTree->expandItems();
    }

    if (currentSettingsId!=0) {
      FF_Report2SettingsTreeItem *item=m_reportSettingsTree->findSettingsItem(currentSettingsId);
      if (item) {
        item->setSelected(true);
        m_reportSettingsTree->makeItemVisible(item);
        m_reportSettingsTree->setCurrentItem(item);
      }
      else {
        DBG_ERROR(0, "Item not found");
      }
    }
    else if (currentGrId!=0) {
      FF_GroupTreeItem *item=m_reportSettingsTree->findGroupItem(currentGrId);
      if (item) {
        item->setSelected(true);
        m_reportSettingsTree->makeItemVisible(item);
        m_reportSettingsTree->setCurrentItem(item);
      }
      else {
        DBG_ERROR(0, "Item not found");
      }
    }
  }
}



void FM_GraphicsReportsTab::reset() {
}






