/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_greporttab.hpp"
#include "fm_reportspage.hpp"
#include "ff_vehicletree.hpp"
#include "ff_report2settingstree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_edit_report2_settings.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/directory.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_GraphicsReportTab) FM_GraphicsReportTabMap[]={
};



FXIMPLEMENT(FM_GraphicsReportTab, FF_ModTabPage,
            FM_GraphicsReportTabMap, ARRAYNUMBER(FM_GraphicsReportTabMap))






FM_GraphicsReportTab::FM_GraphicsReportTab(const AF_REPORT2_SETTINGS *settings,
                                           const FXString &pageId,
                                           FM_ReportsPage *mainPage,
                                           FXuint opts,
                                           FXint x, FXint y, FXint w, FXint h)
:FF_ModTabPage(pageId, mainPage, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_settings(NULL)
,m_imageView(NULL)
,m_image(NULL)
{
  assert(settings);
  m_closeable=true;

  flags|=FLAG_ENABLED|FLAG_SHOWN;

  m_settings=AF_Report2_Settings_dup(settings);

  m_imageView=new FXImageView(this, NULL, NULL, 0,
                              LAYOUT_FILL_X | LAYOUT_FILL_Y |
                              FRAME_SUNKEN | FRAME_THICK |
                              IMAGEVIEW_CENTER_X | IMAGEVIEW_CENTER_Y);

}




FM_GraphicsReportTab::FM_GraphicsReportTab()
:FF_ModTabPage()
,m_mainPage(NULL)
,m_settings(NULL)
,m_imageView(NULL)
,m_image(NULL)
{
}



FM_GraphicsReportTab::~FM_GraphicsReportTab() {
  if (m_settings)
    AF_Report2_Settings_free(m_settings);
  m_settings=NULL;

  if (m_imageView)
    delete m_imageView;
  if (m_image)
    delete m_image;

  if (!(m_fileName.empty()))
    FXFile::remove(m_fileName);
}



void FM_GraphicsReportTab::init(GWEN_DB_NODE *db) {
}



void FM_GraphicsReportTab::fini(GWEN_DB_NODE *db) {
}



void FM_GraphicsReportTab::create(){
  FF_ModTabPage::create();
}



void FM_GraphicsReportTab::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_GraphReportTab: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());
}



void FM_GraphicsReportTab::dbClosing() {
}



void FM_GraphicsReportTab::dbClosed() {
}



void FM_GraphicsReportTab::dbReload(AE_BOOK_TABLE_TYPE tt) {
#if 0
  if (tt==AE_Book_TableType_GraphicReportGroup ||
      tt==AE_Book_TableType_GraphicReportSettings ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
    AE_BOOK *b;
    AE_GROUP *gr;
    AF_REPORT2_SETTINGS *settings;
    AQDB_ID currentSettingsId=0;
    AQDB_ID currentGrId=0;
  
    gr=m_reportSettingsTree->getCurrentGroup();
    if (gr)
      currentGrId=AE_Group_GetId(gr);
    settings=m_reportSettingsTree->getCurrentSettings();
    if (settings)
      currentSettingsId=AF_Report2_Settings_GetId(settings);

    m_reportSettingsTree->clearItems();
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AE_GROUP_TREE *gt;
      int rv;
      AE_DATA_10K_LIST *tkList;
      int i;

      gt=AE_Group_Tree_new();
      rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_GraphicReportGroup, gt);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_reportSettingsTree->addGroups(gt);
      }
      AE_Group_Tree_free(gt);

      /* read vehicles */
      tkList=AE_Data10k_List_new();
      rv=AE_Book_ReadData10kList(b, AE_Book_TableType_GraphicReportSettings, tkList);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
        AE_DATA_10K *tk;

        tk=AE_Data10k_List_First(tkList);
        while (tk) {
          AF_REPORT2_SETTINGS *settings=NULL;

          rv=AF_Report2_Settings_fromData10k(tk, &settings);
          if (rv<0) {
            DBG_ERROR(0, "here (%d)", rv);
          }
          else {
            /* add settings */
            m_reportSettingsTree->addSettings(settings);
          }
          tk=AE_Data10k_List_Next(tk);
        }
      }
      AE_Data10k_List_free(tkList);

      for (i=0; i<m_reportSettingsTree->getNumHeaders(); i++) {
	FXbool b;
    
	b=m_reportSettingsTree->getHeaderArrowDir(i);
	if (b!=MAYBE) {
	  if (b==TRUE)
	    m_reportSettingsTree->sortByColumn(i, true);
	  else
	    m_reportSettingsTree->sortByColumn(i, false);
	  break;
	}
      }

      m_reportSettingsTree->expandItems();
    }

    if (currentSettingsId!=0) {
      FF_Report2SettingsTreeItem *item=m_reportSettingsTree->findSettingsItem(currentSettingsId);
      if (item) {
        item->setSelected(true);
        m_reportSettingsTree->makeItemVisible(item);
        m_reportSettingsTree->setCurrentItem(item);
      }
      else {
        DBG_ERROR(0, "Item not found");
      }
    }
    else if (currentGrId!=0) {
      FF_GroupTreeItem *item=m_reportSettingsTree->findGroupItem(currentGrId);
      if (item) {
        item->setSelected(true);
        m_reportSettingsTree->makeItemVisible(item);
        m_reportSettingsTree->setCurrentItem(item);
      }
      else {
        DBG_ERROR(0, "Item not found");
      }
    }
  }
#endif
}



void FM_GraphicsReportTab::reset() {
}




void FM_GraphicsReportTab::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    const char *s;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    s=AE_GetUserDataFolder();
    assert(s);
    GWEN_Buffer_AppendString(tbuf, s);
    GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S "tmp" GWEN_DIR_SEPARATOR_S "graphreports");

    rv=GWEN_Directory_GetPath(GWEN_Buffer_GetStart(tbuf), GWEN_PATH_FLAGS_CHECKROOT);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "Unable to acces/create folder [%s]. ",
		GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
    else {
      GWEN_TIME *ti;
      FXString fname;

      /* create file name */
      GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);
      ti=GWEN_CurrentTime();
      assert(ti);
      GWEN_Time_toString(ti, "YYYYMMDD-hhmmss", tbuf);
      GWEN_Time_free(ti);
      GWEN_Buffer_AppendString(tbuf, ".png");
      fname=GWEN_Buffer_GetStart(tbuf);
      GWEN_Buffer_free(tbuf);

      DBG_ERROR(0, "Creating report image [%s]", fname.text());
      rv=AF_Report2_CreateReport(b, m_settings, fname.text());
      if (rv<0) {
        DBG_ERROR(0, "Error creating report (%d)", rv);
        if (rv==GWEN_ERROR_NO_DATA) {
          FXMessageBox::error(this, FX::MBOX_OK,
                              I18N("Report Error"),
                              "%s",
                              I18N("No data for the given report profile."));
        }
        else {
          FXMessageBox::error(this, FX::MBOX_OK,
                              I18N("Report Error"),
                              I18N("Could not create report (error: %d)"),
                              rv);
        }
      }
      else {
	FXIconSource iconSource(app);
	FXImage *image;

	DBG_ERROR(0, "Loading report image [%s]", fname.text());
	image=iconSource.loadImageFile(fname);
	if (image) {
          DBG_ERROR(0, "Setting report image [%s]", fname.text());
          image->create();
	  m_imageView->setImage(image);
          if (m_image)
            delete m_image;
          if (!(m_fileName.empty()))
            FXFile::remove(m_fileName);

          m_image=image;
          m_fileName=fname;
	}
      }
    }
  }
}


