/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_transferpage.hpp"
#include "fm_transferstmpage.hpp"
#include "fm_transferstopage.hpp"
#include "fm_transferplannedpage.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_TransferPage) FM_TransferPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_TransferPage::ID_TABBOOK,
	    FM_TransferPage::onTabClicked)
};


            
FXIMPLEMENT(FM_TransferPage, FXHorizontalFrame,
            FM_TransferPageMap, ARRAYNUMBER(FM_TransferPageMap))





FM_TransferPage::FM_TransferPage(FXComposite *p,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_tabBook(NULL)
,m_lastEnteredTab(-1)
,m_transfersPage(NULL)
,m_stoPage(NULL)
,m_plannedPage(NULL)
{

  m_tabBook=new FXTabBook(this, this, ID_TABBOOK,
			  LAYOUT_FILL_X|LAYOUT_FILL_Y);
  /* create tab 1 (reemmber to also modify getTransferPageAt() if you change the order here) */
  new THEMETABITEM(m_tabBook, I18N("Transfers|Planned"), NULL);
  m_plannedPage=new FM_TransferPlannedPage(this, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* create tab 2 (reemmber to also modify getTransferPageAt() if you change the order here) */
  new THEMETABITEM(m_tabBook, I18N("Transfers|Transfers"), NULL);
  m_transfersPage=new FM_TransferStmPage(this, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* create tab 3 (reemmber to also modify getTransferPageAt() if you change the order here) */
  new THEMETABITEM(m_tabBook, I18N("Transfers|Standing Orders"), NULL);
  m_stoPage=new FM_TransferStoPage(this, m_tabBook, LAYOUT_FILL_X | LAYOUT_FILL_Y);
}



FM_TransferPage::FM_TransferPage()
:FXHorizontalFrame()
,m_tabBook(NULL)
,m_lastEnteredTab(-1)
,m_transfersPage(NULL)
,m_stoPage(NULL)
,m_plannedPage(NULL)
{
}



FM_TransferPage::~FM_TransferPage() {
}



void FM_TransferPage::init(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Init");
  m_plannedPage->init(db);
  m_transfersPage->init(db);
  m_stoPage->init(db);
}



void FM_TransferPage::fini(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Fini");
  m_plannedPage->fini(db);
  m_transfersPage->fini(db);
  m_stoPage->fini(db);
}



void FM_TransferPage::dbOpened(AE_BOOK *bk) {
  m_plannedPage->dbOpened(bk);
  m_transfersPage->dbOpened(bk);
  m_stoPage->dbOpened(bk);
}



void FM_TransferPage::dbClosing() {
  m_plannedPage->dbClosing();
  m_transfersPage->dbClosing();
  m_stoPage->dbClosing();
}



void FM_TransferPage::dbClosed() {
  m_plannedPage->dbClosed();
  m_transfersPage->dbClosed();
  m_stoPage->dbClosed();
}



void FM_TransferPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  m_plannedPage->dbReload(tt);
  m_transfersPage->dbReload(tt);
  m_stoPage->dbReload(tt);
}



long FM_TransferPage::onTabClicked(FXObject*, FXSelector, void* ptr) {
  FXint idx=(FXint)(FXival)ptr;

  if (m_lastEnteredTab!=-1) {
    switch(m_lastEnteredTab) {
      case 0:
        if (m_plannedPage)
          m_plannedPage->leave();
        break;
      case 1:
        if (m_transfersPage)
          m_transfersPage->leave();
        break;
      case 2:
        if (m_stoPage)
          m_stoPage->leave();
        break;
    }
  }

  switch(idx) {
  case 0:
    if (m_plannedPage)
      m_plannedPage->enter();
    break;
  case 1:
    if (m_transfersPage)
      m_transfersPage->enter();
    break;
  case 2:
    if (m_stoPage)
      m_stoPage->enter();
    break;
  }

  m_lastEnteredTab=idx;

  return 1;
}



void FM_TransferPage::enter() {
  int idx;

  idx=m_tabBook->getCurrent();
  switch(idx) {
  case 0:
    if (m_plannedPage)
      m_plannedPage->enter();
    break;
  case 1:
    if (m_transfersPage)
      m_transfersPage->enter();
    break;
  case 2:
    if (m_stoPage)
      m_stoPage->enter();
    break;
  }
}



void FM_TransferPage::leave() {
  int idx;

  idx=m_tabBook->getCurrent();
  switch(idx) {
  case 0:
    if (m_plannedPage)
      m_plannedPage->leave();
    break;
  case 1:
    if (m_transfersPage)
      m_transfersPage->leave();
    break;
  case 2:
    if (m_stoPage)
      m_stoPage->leave();
    break;
  }
}



void FM_TransferPage::reset() {
  if (m_plannedPage)
    m_plannedPage->reset();
  if (m_transfersPage)
    m_transfersPage->reset();
  if (m_stoPage)
    m_stoPage->reset();
}




