/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_TRANSFERPLANNEDPAGE_HPP
#define FM_TRANSFERPLANNEDPAGE_HPP


#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_TransferTree;
class FM_TransferPage;


class FM_TransferPlannedPage: public FXVerticalFrame {
  FXDECLARE(FM_TransferPlannedPage)

public:
  enum {
    ID_TRANSFER_TREE=FXPacker::ID_LAST,

    ID_BUTTON_NEW_STD_TRANSFER,
    ID_BUTTON_NEW_INT_TRANSFER,
    ID_BUTTON_DEL_TRANSFER,
    ID_BUTTON_EDIT_TRANSFER,
    ID_BUTTON_EXEC_TRANSFER,

    ID_BUTTON_COPYTO_TEMPLATES,

    ID_MATCHER,

    ID_LAST
  };

public:
  FM_TransferPlannedPage(FM_TransferPage *mainPage,
                     FXComposite *p,
                     FXuint opts=0,
                     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_TransferPlannedPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void enter();
  void leave();
  void reset();

  void create();

  long onCmdAddTransfer(FXObject*, FXSelector, void*);

  long onCmdCopyToTemplates(FXObject*, FXSelector, void*);
  long onCmdNewStdTransfer(FXObject*, FXSelector, void*);
  long onCmdNewInternalTransfer(FXObject*, FXSelector, void*);
  long onCmdEditTransfer(FXObject*, FXSelector, void*);
  long onCmdExecTransfer(FXObject*, FXSelector, void*);
  long onCmdDeleteTransfer(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  long onChgMatcher(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

protected:
  FM_TransferPlannedPage();

  void rebuild();
  int copyTransfer(AE_STATEMENT *st, AE_BOOK_TABLE_TYPE tt);
  int execStatement(AE_STATEMENT *st, AQDB_ID tid);

  int newTransfer(bool isInternal);
  int deleteSelectedTransfers();

  FM_TransferPage *m_mainPage;

  FF_TransferTree *m_transferTree;

  FXButton *m_newStdTransferButton;
  FXButton *m_newIntTransferButton;
  FXButton *m_editTransferButton;
  FXButton *m_execTransferButton;

  FXTextField *m_matcher;

  bool m_needRebuild;
  bool m_isVisible;
};




#endif

