/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_transferstmpage.hpp"
#include "fm_transferpage.hpp"
#include "ff_transfertree.hpp"
#include "ff_edittransfergroup.hpp"
#include "ff_edittransfer.hpp"
#include "ff_transferedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/modules/ae_statementexport.h>
#include <aqfinance/engine/modules/ae_recontransfers.h>

#include <aqbanking/jobsingletransfer.h>
#include <aqbanking/jobinternaltransfer.h>
#include <aqbanking/jobsepatransfer.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_TransferStmPage) FM_TransferStmPageMap[]={
  FXMAPFUNC(SEL_CHANGED, FM_TransferStmPage::ID_TRANSFER_TREE,
            FM_TransferStmPage::onTreeChanged),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FM_TransferStmPage::ID_TRANSFER_TREE,
            FM_TransferStmPage::onTreeRightBtnRelease),

  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_COPYTO_TEMPLATES,
            FM_TransferStmPage::onCmdCopyToTemplates),

  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_REPEAT_TRANSFER,
            FM_TransferStmPage::onCmdRepeatTransfer),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_NEW_STD_TRANSFER,
            FM_TransferStmPage::onCmdNewStdTransfer),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_NEW_INT_TRANSFER,
            FM_TransferStmPage::onCmdNewInternalTransfer),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_NEW_SEPA_TRANSFER,
            FM_TransferStmPage::onCmdNewSepaTransfer),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_DEL_TRANSFER,
            FM_TransferStmPage::onCmdDeleteTransfer),

  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_AUTORECON,
            FM_TransferStmPage::onCmdAutoRecon),

  FXMAPFUNC(SEL_CHANGED, FM_TransferStmPage::ID_MATCHER,
            FM_TransferStmPage::onChgMatcher),

  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_STATUS_ACCEPTED,
            FM_TransferStmPage::onCmdStatusAccepted),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_STATUS_REJECTED,
            FM_TransferStmPage::onCmdStatusRejected),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_STATUS_ABORTED,
            FM_TransferStmPage::onCmdStatusAborted),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_STATUS_REVOKED,
            FM_TransferStmPage::onCmdStatusRevoked),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_STATUS_PENDING,
            FM_TransferStmPage::onCmdStatusPending),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStmPage::ID_BUTTON_STATUS_RECONCILED,
            FM_TransferStmPage::onCmdStatusReconciled),

};



FXIMPLEMENT(FM_TransferStmPage, FXVerticalFrame,
            FM_TransferStmPageMap, ARRAYNUMBER(FM_TransferStmPageMap))






FM_TransferStmPage::FM_TransferStmPage(FM_TransferPage *mainPage,
                                       FXComposite *p,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h)
:FXVerticalFrame(p, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_transferTree(NULL)
,m_newStdTransferButton(NULL)
,m_newIntTransferButton(NULL)
,m_newSepaTransferButton(NULL)
,m_repeatTransferButton(NULL)
,m_needRebuild(true)
,m_isVisible(false)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_transferTree=new FF_TransferTree(hf, this, ID_TRANSFER_TREE,
				     LAYOUT_SIDE_LEFT |
				     LAYOUT_FILL_X | LAYOUT_FILL_Y |
				     FOLDINGLIST_EXTENDEDSELECT |
				     FOLDINGLIST_SHOWS_LINES |
				     FOLDINGLIST_SHOWS_BOXES |
				     FOLDINGLIST_ROOT_BOXES |
                                     FRAME_SUNKEN|FRAME_THICK);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_DATE, 100);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_REMOTE_NAME, 200);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_PURPOSE, 200);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_TYPE, 100);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_AMOUNT, 100);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_STATUS, 100);

  ic=app->getIcon("minifolderopen");
  if (ic)
    m_transferTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_transferTree->setIconClosed(ic);

  vf=new FXVerticalFrame(hf, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_newStdTransferButton=new THEMEBUTTON(vf,
					 I18N("New Standard Transfer"),
					 NULL,
					 this, ID_BUTTON_NEW_STD_TRANSFER,
					 BUTTON_NORMAL);

  m_newIntTransferButton=new THEMEBUTTON(vf,
					 I18N("New Internal Transfer"),
					 NULL,
					 this, ID_BUTTON_NEW_INT_TRANSFER,
					 BUTTON_NORMAL);

  m_newSepaTransferButton=new THEMEBUTTON(vf,
                                          I18N("New SEPA Transfer"),
                                          NULL,
                                          this, ID_BUTTON_NEW_SEPA_TRANSFER,
                                          BUTTON_NORMAL);

  m_repeatTransferButton=new THEMEBUTTON(vf,
                                         I18N("Repeat Transfer"),
                                         NULL,
                                         this, ID_BUTTON_REPEAT_TRANSFER,
                                         BUTTON_NORMAL);

}




FM_TransferStmPage::FM_TransferStmPage()
:FXVerticalFrame()
,m_mainPage(NULL)
,m_transferTree(NULL)
,m_newStdTransferButton(NULL)
,m_newIntTransferButton(NULL)
,m_newSepaTransferButton(NULL)
,m_repeatTransferButton(NULL)
,m_needRebuild(false)
{
}



FM_TransferStmPage::~FM_TransferStmPage() {
}



void FM_TransferStmPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "transferPage");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_transferTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_transferTree->getHeaderSize(i));
      m_transferTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_transferTree->setHeaderArrowDir(i, TRUE);
        else
          m_transferTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_TransferStmPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "transferPage");
  for (i=0; i<m_transferTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_transferTree->getHeaderSize(i));
    b=m_transferTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_TransferStmPage::create(){
  FXVerticalFrame::create();
}



void FM_TransferStmPage::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_TransferView: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_transferTree->dump(indent+2);
}



int FM_TransferStmPage::copyTransfer(AE_STATEMENT *st, AE_BOOK_TABLE_TYPE tt) {
  FF_App *app;
  AE_BOOK *b;
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_STATEMENT *cpy;

    cpy=AE_Statement_dup(st);
    AE_Statement_SetDbId(cpy, 0);
    AE_Statement_SetId(cpy, 0);
    rv=AE_Book_AddStatement(b, tt, cpy, 1);
    if (rv<0) {
      DBG_ERROR(0, "Unable to add statement (%d)", rv);
    }
    else {
      app->signalDbReload(tt);
    }
    AE_Statement_free(cpy);
    return rv;
  }
  else
    return GWEN_ERROR_NOT_OPEN;
}



long FM_TransferStmPage::onCmdCopyToTemplates(FXObject*, FXSelector, void*) {
  AE_STATEMENT *st;

  st=m_transferTree->getCurrentTransfer();
  if (st)
    copyTransfer(st, AE_Book_TableType_BankTransferTemplate);
  return 1;
}



int FM_TransferStmPage::execStatement(AE_STATEMENT *st, AQDB_ID tid) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;
  
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    AE_BANKACCOUNT *ba=NULL;
    AB_ACCOUNT *a;
    AB_JOB *j;
    AB_TRANSACTION *t;
    int rv;
    uint32_t baId=0;

    rv=AE_Book_ReadBankAccount(b, AE_Book_TableType_BankAccount, AE_Statement_GetBankAccountId(st), &ba);
    if (rv<0) {
      DBG_ERROR(0, "Error reading bank account %lu",
                (unsigned long int) AE_Statement_GetBankAccountId(st));
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Database Error"),
                          I18N("Error reading bank account %lu from database (%d)"),
                          (unsigned long int) AE_Statement_GetBankAccountId(st),
                          rv);
      return GWEN_ERROR_NOT_FOUND;
    }
    baId=AE_BankAccount_GetBankingId(ba);
    AE_BankAccount_free(ba);

    a=AB_Banking_GetAccount(ab, baId);
    if (a==NULL) {
      DBG_ERROR(0, "Account %d not found", AE_Statement_GetBankAccountId(st));
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Banking Error"),
			  "%s",
                          I18N("AqBanking account not found"));
      return GWEN_ERROR_NOT_FOUND;
    }

    switch(AE_Statement_GetFlags(st) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK) {
    case AE_STATEMENT_FLAGS_XFER_TYPE_STD_XFER:
      j=AB_JobSingleTransfer_new(a);
      rv=AB_Job_CheckAvailability(j);
      if (rv<0) {
	DBG_ERROR(0, "Job not available (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
			    I18N("Transfer job is not available (%d)"), rv);
	AB_Job_free(j);
	return rv;
      }
  
      /* transform to banking transaction */
      t=AB_Transaction_new();
      rv=AE_StatementExport_StatementToTransaction(st, t);
      if (rv<0) {
	DBG_ERROR(0, "Error transforming statement to AB_TRANSACTION (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
			    I18N("Unable to create a banking transaction (%d)"), rv);
	AB_Transaction_free(t);
	AB_Job_free(j);
	return rv;
      }
      /* always set data to NULL for non-dated transfers */
      AB_Transaction_SetDate(t, NULL);
      AB_Transaction_SetIdForApplication(t, tid);
      AB_Job_SetTransaction(j, t);
  
      app->addJobToList(j);
      app->signalJobListUpdated();
      break;

    case AE_STATEMENT_FLAGS_XFER_TYPE_INT_XFER:
      j=AB_JobInternalTransfer_new(a);
      rv=AB_Job_CheckAvailability(j);
      if (rv<0) {
	DBG_ERROR(0, "Job not available (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
			    I18N("Internal transfer job is not available (%d)"), rv);
	AB_Job_free(j);
	return rv;
      }
  
      /* transform to banking transaction */
      t=AB_Transaction_new();
      rv=AE_StatementExport_StatementToTransaction(st, t);
      if (rv<0) {
	DBG_ERROR(0, "Error transforming statement to AB_TRANSACTION (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
			    I18N("Unable to create a banking transaction (%d)"), rv);
	AB_Transaction_free(t);
	AB_Job_free(j);
	return rv;
      }
      /* always set data to NULL for non-dated transfers */
      AB_Transaction_SetDate(t, NULL);
      AB_Transaction_SetIdForApplication(t, tid);
      AB_Job_SetTransaction(j, t);

      app->addJobToList(j);
      app->signalJobListUpdated();
      break;

    case AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_XFER:
      j=AB_JobSepaTransfer_new(a);
      rv=AB_Job_CheckAvailability(j);
      if (rv<0) {
	DBG_ERROR(0, "Job not available (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
			    I18N("SEPA Transfer job is not available (%d)"), rv);
	AB_Job_free(j);
	return rv;
      }
  
      /* transform to banking transaction */
      t=AB_Transaction_new();
      rv=AE_StatementExport_StatementToTransaction(st, t);
      if (rv<0) {
	DBG_ERROR(0, "Error transforming statement to AB_TRANSACTION (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
			    I18N("Unable to create a banking transaction (%d)"), rv);
	AB_Transaction_free(t);
	AB_Job_free(j);
	return rv;
      }
      /* always set data to NULL for non-dated transfers */
      AB_Transaction_SetDate(t, NULL);
      AB_Transaction_SetIdForApplication(t, tid);
      AB_Job_SetTransaction(j, t);
  
      app->addJobToList(j);
      app->signalJobListUpdated();
      break;

    default:
      DBG_ERROR(0, "Unexpected transfer type (%d)",
                AE_Statement_GetFlags(st) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK);
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Internal Error"),
                          I18N("Unexpected transfer type (%d)"),
                          AE_Statement_GetFlags(st) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK);
      return GWEN_ERROR_INTERNAL;
    }

    return 0;
  }
  else {
    DBG_ERROR(0, "Aborted by user");
    return GWEN_ERROR_USER_ABORTED;
  }
}



long FM_TransferStmPage::onCmdRepeatTransfer(FXObject*, FXSelector, void*) {
  AE_STATEMENT *st;

  st=m_transferTree->getCurrentTransfer();
  if (st) {
    FF_App *app;
    AE_BOOK *b;
    AB_BANKING *ab;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    ab=app->getBanking();
    if (b && ab) {
      AE_STATEMENT *cpy;
      int rv;
      GWEN_DATE *ti;
      bool rb;

      /* always work on copy */
      cpy=AE_Statement_dup(st);
      ti=GWEN_Date_CurrentDate();
      AE_Statement_SetDate(cpy, ti);
      GWEN_Date_free(ti);


      switch(AE_Statement_GetFlags(cpy) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK) {
      case AE_STATEMENT_FLAGS_XFER_TYPE_STD_XFER:
        rb=FF_EditTransfer2::editSingleTransfer(app, FF_TransferEdit::EDITMODE_STRICT,
                                                this,
                                                I18N("Repeat Bank Transfer"), cpy,
                                                DECOR_ALL |
                                                FF_EditTransfer2::OPTS_SHOWLOADTMPL |
                                                FF_EditTransfer2::OPTS_SHOWSAVETMPL);
        break;
      case AE_STATEMENT_FLAGS_XFER_TYPE_INT_XFER:
        rb=FF_EditTransfer2::editInternalTransfer(app, FF_TransferEdit::EDITMODE_STRICT,
                                                  this,
                                                  I18N("Repeat Internal Bank Transfer"), cpy,
                                                  DECOR_ALL |
                                                  FF_EditTransfer2::OPTS_SHOWLOADTMPL |
                                                  FF_EditTransfer2::OPTS_SHOWSAVETMPL);
        break;
      case AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_XFER:
        rb=FF_EditTransfer2::editSepaTransfer(app, FF_TransferEdit::EDITMODE_STRICT,
                                              this,
                                              I18N("Repeat SEPA  Transfer"), cpy,
                                              DECOR_ALL |
                                              FF_EditTransfer2::OPTS_SHOWLOADTMPL |
                                              FF_EditTransfer2::OPTS_SHOWSAVETMPL);
        break;

      default:
        DBG_ERROR(0, "Unexpected transfer type (%d)",
                  AE_Statement_GetFlags(cpy) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Internal Error"),
                            I18N("Unexpected transfer type (%d)"),
                            AE_Statement_GetFlags(cpy) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK);
        rb=false;
        break;
      }


      if (rb) {
        /* store in database before actually enqueing it, so that we get an id to set
	 * in AB_Transaction_SetIdForApplication() */
	/* create a copy of the initial statement */
	rv=AE_Book_AddStatement(b, AE_Book_TableType_BankTransfer, cpy, 1);
	if (rv<0) {
	  DBG_ERROR(0, "Unable to add statement (%d)", rv);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Error adding transfer (%d)"), rv);
	}
	else {
	  app->signalDbReload(AE_Book_TableType_BankTransfer);
	}

        /* CAVEAT: don't use st here since it might have been destryoed by
         * signalDbReload */

        /* create job for transfer */
        DBG_ERROR(0, "Id for transaction: %d", AE_Statement_GetId(cpy));
        rv=execStatement(cpy, AE_Statement_GetId(cpy));
        if (rv==0) {
          /* ok, change status to "changed" */
          AE_Statement_SetStatus(cpy, AE_Statement_StatusEnqueued);
          rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, cpy, 1);
          if (rv<0) {
            DBG_ERROR(0, "Unable write transfer (%d)", rv);
            FXMessageBox::error(this, MBOX_OK,
                                I18N("Database Error"),
                                I18N("Error writing transfer (%d)"), rv);
          }
        }
        else {
          /* set status to "aborted" */
          AE_Statement_SetStatus(cpy, AE_Statement_StatusAborted);
	  rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, cpy, 1);
          if (rv<0) {
            DBG_ERROR(0, "Unable to write transfer (%d)", rv);
            FXMessageBox::error(this, MBOX_OK,
                                I18N("Database Error"),
				I18N("Error writing transfer (%d)"), rv);
	  }
	}
	app->signalDbReload(AE_Book_TableType_BankTransfer);
      }
      AE_Statement_free(cpy);
    }
    else {
      DBG_ERROR(0, "No book/AqBanking");
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Banking Error"),
                          "%s",
                          I18N("Database not open or AqBanking not available"));
    }
  }
  return 1;
}



int FM_TransferStmPage::deleteSelectedTransfers() {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    FXuint res;

    res=FXMessageBox::question(this, MBOX_YES_NO,
			       I18N("Delete Transfer(s)"),
			       "%s",
			       I18N("Are you sure you want to delete the selected "
				    "transfer(s)?"));
    if (res!=MBOX_CLICKED_YES) {
      DBG_ERROR(0, "Aborted");
    }
    else {
      int rv;
      AE_STATEMENT_LIST2 *sl;
      AE_STATEMENT_LIST2_ITERATOR *it;
      int changed=0;
  
      sl=AE_Statement_List2_new();
      m_transferTree->getSelectedTransfers(sl, FF_TransferTree::COPYMODE_NONE);
      it=AE_Statement_List2_First(sl);
      if (it) {
	AE_STATEMENT *st;
  
	/* lock db */
	rv=AE_Book_BeginEdit(b, 0);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  return rv;
	}
  
	st=AE_Statement_List2Iterator_Data(it);
	while(st) {
	  /* delete transfer */
	  rv=AE_Book_DeleteStatement(b, AE_Book_TableType_BankTransfer, st, 0);
	  if (rv<0) {
	    DBG_ERROR(0, "Unable to delete bank transfer (%d)", rv);
	    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Could not delete transfer from database (%d)"),
				rv);
	    AE_Statement_List2Iterator_free(it);
	    AE_Statement_List2_free(sl);
	    return rv;
	  }
    
	  changed++;
  
	  st=AE_Statement_List2Iterator_Next(it);
	}
	AE_Statement_List2Iterator_free(it);
      }
      AE_Statement_List2_free(sl);
  
      /* unlock */
      rv=AE_Book_EndEdit(b, 0);
      if (rv<0) {
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Could not apply changes to database (%d)"),
			    rv);
	return rv;
      }
  
      if (changed) {
	app->signalDbReload(AE_Book_TableType_BankTransfer);
      }
    }
  }
  else {
    DBG_ERROR(0, "No book/AqBanking");
    FXMessageBox::error(this, MBOX_OK,
			I18N("Banking Error"),
			"%s",
			I18N("Database not open or AqBanking not available"));
    return GWEN_ERROR_NOT_OPEN;
  }
  return 0;
}



long FM_TransferStmPage::onCmdDeleteTransfer(FXObject*, FXSelector, void*) {
  deleteSelectedTransfers();
  return 1;
}




int FM_TransferStmPage::changeStatusOfCurrentTransfer(AE_STATEMENT_STATUS sst) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    int rv;
    AE_STATEMENT_LIST2 *sl;
    AE_STATEMENT_LIST2_ITERATOR *it;
    int changed=0;

    sl=AE_Statement_List2_new();
    m_transferTree->getSelectedTransfers(sl, FF_TransferTree::COPYMODE_NONE);
    it=AE_Statement_List2_First(sl);
    if (it) {
      AE_STATEMENT *st;

      /* lock db */
      rv=AE_Book_BeginEdit(b, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	return rv;
      }

      st=AE_Statement_List2Iterator_Data(it);
      while(st) {
	AE_STATEMENT *stRead=NULL;

	/* read transfer */
	rv=AE_Book_ReadStatement(b, AE_Book_TableType_BankTransfer, AE_Statement_GetId(st), &stRead);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not read transfer from database (%d)"),
			      rv);
	  AE_Statement_List2Iterator_free(it);
	  AE_Statement_List2_free(sl);
	  return rv;
	}
  
	/* modify */
	AE_Statement_SetStatus(stRead, sst);
  
	/* write transfer */
	rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, stRead, 0);
	if (rv<0) {
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Database Error"),
			      I18N("Could not write transfer to database (%d)"),
			      rv);
	  AE_Statement_free(stRead);
	  AE_Statement_List2Iterator_free(it);
	  AE_Statement_List2_free(sl);
	  return rv;
	}
	AE_Statement_free(stRead);
	changed++;

	st=AE_Statement_List2Iterator_Next(it);
      }
      AE_Statement_List2Iterator_free(it);
    }
    AE_Statement_List2_free(sl);

    /* unlock */
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Database Error"),
			  I18N("Could not apply changes to database (%d)"),
			  rv);
      return rv;
    }

    if (changed) {
      app->signalDbReload(AE_Book_TableType_BankTransfer);
    }
  }
  else {
    DBG_ERROR(0, "No book/AqBanking");
    FXMessageBox::error(this, MBOX_OK,
			I18N("Banking Error"),
			"%s",
			I18N("Database not open or AqBanking not available"));
    return GWEN_ERROR_NOT_OPEN;
  }
  return 0;
}



long FM_TransferStmPage::onCmdStatusAccepted(FXObject*, FXSelector, void*) {
  changeStatusOfCurrentTransfer(AE_Statement_StatusAccepted);
  return 1;
}



long FM_TransferStmPage::onCmdStatusRejected(FXObject*, FXSelector, void*) {
  changeStatusOfCurrentTransfer(AE_Statement_StatusRejected);
  return 1;
}



long FM_TransferStmPage::onCmdStatusAborted(FXObject*, FXSelector, void*) {
  changeStatusOfCurrentTransfer(AE_Statement_StatusAborted);
  return 1;
}



long FM_TransferStmPage::onCmdStatusRevoked(FXObject*, FXSelector, void*) {
  changeStatusOfCurrentTransfer(AE_Statement_StatusRevoked);
  return 1;
}



long FM_TransferStmPage::onCmdStatusPending(FXObject*, FXSelector, void*) {
  changeStatusOfCurrentTransfer(AE_Statement_StatusPending);
  return 1;
}



long FM_TransferStmPage::onCmdStatusReconciled(FXObject*, FXSelector, void*) {
  changeStatusOfCurrentTransfer(AE_Statement_StatusManuallyReconciled);
  return 1;
}



int FM_TransferStmPage::newTransfer(int tt) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    AE_STATEMENT *st;
    int rv;
    bool rb;
    GWEN_DATE *ti;

    st=AE_Statement_new();
    ti=GWEN_Date_CurrentDate();
    AE_Statement_SetDate(st, ti);
    GWEN_Date_free(ti);
    AE_Statement_SetGroupId(st, 0);
    AE_Statement_SetTextKey(st, 51);
    AE_Statement_SetFlags(st, tt);

    switch(tt) {
    case AE_STATEMENT_FLAGS_XFER_TYPE_STD_XFER:
      rb=FF_EditTransfer2::editSingleTransfer(app, FF_TransferEdit::EDITMODE_STRICT,
                                              this,
                                              I18N("Send Bank Transfer"), st,
                                              DECOR_ALL |
                                              FF_EditTransfer2::OPTS_SHOWLOADTMPL |
                                              FF_EditTransfer2::OPTS_SHOWSAVETMPL);
      break;
    case AE_STATEMENT_FLAGS_XFER_TYPE_INT_XFER:
      rb=FF_EditTransfer2::editInternalTransfer(app, FF_TransferEdit::EDITMODE_STRICT,
                                                this,
                                                I18N("Send Internal Bank Transfer"), st,
                                                DECOR_ALL |
                                                FF_EditTransfer2::OPTS_SHOWLOADTMPL |
                                                FF_EditTransfer2::OPTS_SHOWSAVETMPL);
      break;
    case AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_XFER:
      rb=FF_EditTransfer2::editSepaTransfer(app, FF_TransferEdit::EDITMODE_STRICT,
                                            this,
                                            I18N("Send SEPA Transfer"), st,
                                            DECOR_ALL |
                                            FF_EditTransfer2::OPTS_SHOWLOADTMPL |
                                            FF_EditTransfer2::OPTS_SHOWSAVETMPL);
      break;
    }

    if (rb) {
      /* store in database before actually enqueing it, so that we get an id to set
       * in AB_Transaction_SetIdForApplication() */
      /* create a copy of the initial statement */
      rv=AE_Book_AddStatement(b, AE_Book_TableType_BankTransfer, st, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add statement (%d)", rv);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Database Error"),
                            I18N("Error adding transfer (%d)"), rv);
	AE_Statement_free(st);
	return rv;
      }
      else {
        app->signalDbReload(AE_Book_TableType_BankTransfer);
      }

      /* CAVEAT: don't use st here since it might have been destryoed by
       * signalDbReload */

      /* create job for transfer */
      DBG_ERROR(0, "Id for transaction: %d", AE_Statement_GetId(st));
      rv=execStatement(st, AE_Statement_GetId(st));
      if (rv==0) {
        /* ok, change status to "enqueued" */
        AE_Statement_SetStatus(st, AE_Statement_StatusEnqueued);
        rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, st, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable write transfer (%d)", rv);
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Database Error"),
                              I18N("Error writing transfer (%d)"), rv);
	  AE_Statement_free(st);
	  return rv;
	}
      }
      else {
        /* set status to "aborted" */
        AE_Statement_SetStatus(st, AE_Statement_StatusAborted);
        rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, st, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to write transfer (%d)", rv);
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Database Error"),
                              I18N("Error writing transfer (%d)"), rv);
	  AE_Statement_free(st);
	  return rv;
	}
      }
      app->signalDbReload(AE_Book_TableType_BankTransfer);
    }
    AE_Statement_free(st);
  }
  else {
    DBG_ERROR(0, "No book/AqBanking");
    FXMessageBox::error(this, MBOX_OK,
                        I18N("Banking Error"),
                        "%s",
			I18N("Database not open or AqBanking not available"));
    return GWEN_ERROR_NOT_OPEN;
  }
  return 0;
}



long FM_TransferStmPage::onCmdNewStdTransfer(FXObject*, FXSelector, void*) {
  newTransfer(AE_STATEMENT_FLAGS_XFER_TYPE_STD_XFER);
  return 1;
}



long FM_TransferStmPage::onCmdNewInternalTransfer(FXObject*, FXSelector, void*) {
  newTransfer(AE_STATEMENT_FLAGS_XFER_TYPE_INT_XFER);
  return 1;
}



long FM_TransferStmPage::onCmdNewSepaTransfer(FXObject*, FXSelector, void*) {
  newTransfer(AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_XFER);
  return 1;
}



long FM_TransferStmPage::onCmdAutoRecon(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv=AE_ReconTransfers_ReconTransfers(b, 1);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Database Error"),
			  I18N("Error automatically reconciling transfers (%d)"), rv);
    }
    app->signalDbReload(AE_Book_TableType_BankTransfer);
  }

  return 1;
}



long FM_TransferStmPage::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
	m_repeatTransferButton->enable();
      }
      else {
	m_repeatTransferButton->disable();
      }
    }
    else {
      m_repeatTransferButton->disable();
    }
  }
  return 1;
}



long FM_TransferStmPage::onTreeRightBtnRelease(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuPane paneStatus(this);
    FXMenuCommand *c;
    FXMenuCascade *mc;
    FXMenuCheck *ck;
    AE_GROUP *gr;
    AE_STATEMENT *st;
    AE_STATEMENT_STATUS sst=AE_Statement_StatusUnknown;

    gr=m_transferTree->getCurrentGroup();
    st=m_transferTree->getCurrentTransfer();
    if (st)
      sst=AE_Statement_GetStatus(st);


    new FXMenuCommand(&pmenu, I18N("New Standard Transfer"), NULL, this, ID_BUTTON_NEW_STD_TRANSFER);

    new FXMenuCommand(&pmenu, I18N("New Internal Transfer"), NULL, this, ID_BUTTON_NEW_INT_TRANSFER);

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Repeat transfer"), NULL,
			this, ID_BUTTON_REPEAT_TRANSFER);
    if (!st) c->disable();

    /* status pane */
    ck=new FXMenuCheck(&paneStatus, I18N("Accepted"), this, ID_BUTTON_STATUS_ACCEPTED);
    if (sst==AE_Statement_StatusAccepted)
      ck->setCheck(TRUE);
    ck=new FXMenuCheck(&paneStatus, I18N("Rejected"), this, ID_BUTTON_STATUS_REJECTED);
    if (sst==AE_Statement_StatusRejected)
      ck->setCheck(TRUE);
    ck=new FXMenuCheck(&paneStatus, I18N("Aborted"), this, ID_BUTTON_STATUS_ABORTED);
    if (sst==AE_Statement_StatusAborted)
      ck->setCheck(TRUE);
    ck=new FXMenuCheck(&paneStatus, I18N("Revoked"), this, ID_BUTTON_STATUS_REVOKED);
    if (sst==AE_Statement_StatusRevoked)
      ck->setCheck(TRUE);
    ck=new FXMenuCheck(&paneStatus, I18N("Pending"), this, ID_BUTTON_STATUS_PENDING);
    if (sst==AE_Statement_StatusPending)
      ck->setCheck(TRUE);
    ck=new FXMenuCheck(&paneStatus, I18N("Reconciled"), this, ID_BUTTON_STATUS_RECONCILED);
    if (sst==AE_Statement_StatusManuallyReconciled)
      ck->setCheck(TRUE);
    mc=new FXMenuCascade(&pmenu, I18N("Change Status"), NULL, &paneStatus);
    if (!st) mc->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete transfer(s)"), NULL,
			this, ID_BUTTON_DEL_TRANSFER);
    if (!st) c->disable();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Save as template"), NULL,
			this, ID_BUTTON_COPYTO_TEMPLATES);
    if (!st) c->disable();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Automatically reconcile"), NULL,
			this, ID_BUTTON_AUTORECON);

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FM_TransferStmPage::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  rebuild();
  m_transferTree->expandItems();
  return 1;
}



void FM_TransferStmPage::dbOpened(AE_BOOK *b) {
  if (m_isVisible) {
    rebuild();
    m_needRebuild=false;
  }
  else {
    m_needRebuild=true;
  }
}



void FM_TransferStmPage::dbClosing() {
  m_transferTree->clearItems();
}



void FM_TransferStmPage::dbClosed() {
}



void FM_TransferStmPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_BankTransfer ||
      tt==AE_Book_TableType_BankTransferGroup ||
      tt==AE_Book_TableType_None) {
    if (m_isVisible) {
      rebuild();
      m_needRebuild=false;
    }
    else
      m_needRebuild=true;
  }
}



FF_GroupTreeItem *FM_TransferStmPage::getStatusGroupItem(AE_STATEMENT_STATUS st, bool createIfMissing) {
  int nst=STATUS_GROUP_NONE;
  const char *grName=I18N("TransferGroupStatus|None");;
  FF_GroupTreeItem *item;

  switch(st) {
  case AE_Statement_StatusUnknown:
  case AE_Statement_StatusNone:
    nst=STATUS_GROUP_NONE;
    grName=I18N("TransferGroupStatus|None");
    break;
  case AE_Statement_StatusEnqueued:
    nst=STATUS_GROUP_ENQUEUED;
    grName=I18N("TransferGroupStatus|En route");
    break;
  case AE_Statement_StatusAccepted:
    nst=STATUS_GROUP_ACCEPTED;
    grName=I18N("TransferGroupStatus|Accepted");
    break;
  case AE_Statement_StatusRevoked:
  case AE_Statement_StatusAborted:
  case AE_Statement_StatusRejected:
    nst=STATUS_GROUP_FAILED;
    grName=I18N("TransferGroupStatus|Failed");
    break;
  case AE_Statement_StatusSending:
  case AE_Statement_StatusPending:
    nst=STATUS_GROUP_PENDING;
    grName=I18N("TransferGroupStatus|Pending");
    break;
  case AE_Statement_StatusAutoReconciled:
  case AE_Statement_StatusManuallyReconciled:
    nst=STATUS_GROUP_RECONCILED;
    grName=I18N("TransferGroupStatus|Reconciled");
    break;
  }

  DBG_ERROR(0, "Transfer status matches transfer group \"%s\"", grName);


  item=m_transferTree->findGroupItem(nst);
  if (item==NULL && createIfMissing) {
    AE_GROUP *gr;

    /* create group */
    DBG_ERROR(0, "Adding transfer group \"%s\"", grName);
    gr=AE_Group_new();
    AE_Group_SetName(gr, grName);
    AE_Group_SetId(gr, nst);
    m_transferTree->addGroup(gr);
    item=m_transferTree->findGroupItem(nst);
    assert(item);
  }

  return item;
}



FF_GroupTreeItem *FM_TransferStmPage::getMonthGroupItem(const GWEN_DATE *dt, bool createIfMissing) {
  int nid=0;
  const char *grName=NULL;
  FF_GroupTreeItem *item;
  GWEN_BUFFER *tbuf;
  char cbuf[64];

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  if (dt) {
    int m, y;
    const char *monthNames[12]= {
      I18S("January"), I18S("February"), I18S("March"), I18S("April"),
      I18S("May"), I18S("June"), I18S("July"), I18S("August"),
      I18S("September"), I18S("October"), I18S("November"), I18S("December") };

    m=GWEN_Date_GetMonth(dt);
    y=GWEN_Date_GetYear(dt);
    snprintf(cbuf, sizeof(cbuf)-1, "%04d/%02d", GWEN_Date_GetYear(dt), m);
    cbuf[sizeof(cbuf)-1]=0;
    GWEN_Buffer_AppendString(tbuf, cbuf);
    if (m>=1 && m<=12) {
      GWEN_Buffer_AppendString(tbuf, " - ");
      GWEN_Buffer_AppendString(tbuf, I18N(monthNames[m-1]));
      GWEN_Buffer_AppendString(tbuf, " ");
      snprintf(cbuf, sizeof(cbuf)-1, "%04d", y);
      cbuf[sizeof(cbuf)-1]=0;
      GWEN_Buffer_AppendString(tbuf, cbuf);
    }
    grName=GWEN_Buffer_GetStart(tbuf);
    nid=(y<<8)+m;
  }

  DBG_ERROR(0, "Transfer status matches transfer group \"%s\"", grName);


  item=m_transferTree->findGroupItem(nid);
  if (item==NULL && createIfMissing) {
    AE_GROUP *gr;

    /* create group */
    DBG_ERROR(0, "Adding transfer group \"%s\"", grName);
    gr=AE_Group_new();
    AE_Group_SetName(gr, grName);
    AE_Group_SetId(gr, nid);
    m_transferTree->addGroup(gr);
    item=m_transferTree->findGroupItem(nid);
    assert(item);
  }

  return item;
}



void FM_TransferStmPage::rebuild() {
  FF_App *app;
  AE_BOOK *b;
  AE_GROUP *gr;
  AE_STATEMENT *st;
  AQDB_ID currentStmId=0;
  AQDB_ID currentGrId=0;
  FXString str;

  str=m_matcher->getText().simplify().trim();

  gr=m_transferTree->getCurrentGroup();
  if (gr)
    currentGrId=AE_Group_GetId(gr);
  st=m_transferTree->getCurrentTransfer();
  if (st)
    currentStmId=AE_Statement_GetId(st);

  m_transferTree->clearItems();

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_STATEMENT_LIST *sl;

    /* read transfers */
    sl=AE_Statement_List_new();
    rv=AE_Book_ReadStatementList(b, AE_Book_TableType_BankTransfer, sl);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      AE_STATEMENT *st;

      if (!str.empty()) {
	AE_Statement_List_SelectMatching(sl, str.text());
	/* remove non-matching statements */
	st=AE_Statement_List_First(sl);
	while(st) {
	  AE_STATEMENT *next;

	  next=AE_Statement_List_Next(st);
	  if (!(AE_Statement_GetRuntimeFlags(st) & AE_STATEMENT_RTFLAGS_SELECTED)) {
	    AE_Statement_List_Del(st);
	    AE_Statement_free(st);
	  }

	  st=next;
	}
      }

#if 0
      if (m_tableType==AE_Book_TableType_BankTransfer) {
        /* TODO: get groups from transfers */
	st=AE_Statement_List_First(sl);
	while(st) {

	  st=AE_Statement_List_Next(st);
	}
      }
#endif

      /* add transfers */
      while ( (st=AE_Statement_List_First(sl)) ) {
	FF_GroupTreeItem *itemFather;
	const GWEN_DATE *dt;

	dt=AE_Statement_GetDate(st);
	//itemFather=getStatusGroupItem(AE_Statement_GetStatus(st), true);
	itemFather=getMonthGroupItem(dt, true);
	AE_Statement_List_Del(st);
	m_transferTree->addTransferToItem(st, itemFather, FF_TransferTree::USECOLOR_NORMAL);
      }
    }
    AE_Statement_List_free(sl);

    m_transferTree->sort();
    /*m_transferTree->expandItems();*/
  }
  else {
    DBG_ERROR(0, "No book");
  }

  if (currentStmId!=0) {
    FF_TransferTreeItem *item=m_transferTree->findTransferItem(currentStmId);
    if (item) {
      m_transferTree->unselectAll();
      item->setSelected(true);
      m_transferTree->makeItemVisible(item);
      m_transferTree->setCurrentItem(item);
    }
    else {
      DBG_ERROR(0, "Item not found");
    }
  }
  else if (currentGrId!=0) {
    FF_GroupTreeItem *item=m_transferTree->findGroupItem(currentGrId);
    if (item) {
      item->setSelected(true);
      m_transferTree->makeItemVisible(item);
      m_transferTree->setCurrentItem(item);
    }
    else {
      DBG_ERROR(0, "Item not found");
    }
  }
}


void FM_TransferStmPage::enter() {
  m_isVisible=true;

  if (m_needRebuild) {
    rebuild();
    m_needRebuild=false;
  }
}



void FM_TransferStmPage::leave() {
  m_isVisible=false;
}



void FM_TransferStmPage::reset() {
  m_transferTree->clearItems();
}









