/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_TRANSFERSTMPAGE_HPP
#define FM_TRANSFERSTMPAGE_HPP


#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_TransferTree;
class FM_TransferPage;


class FM_TransferStmPage: public FXVerticalFrame {
  FXDECLARE(FM_TransferStmPage)

public:
  enum {
    ID_TRANSFER_TREE=FXPacker::ID_LAST,

    ID_BUTTON_NEW_STD_TRANSFER,
    ID_BUTTON_NEW_INT_TRANSFER,
    ID_BUTTON_NEW_SEPA_TRANSFER,
    ID_BUTTON_DEL_TRANSFER,
    ID_BUTTON_REPEAT_TRANSFER,

    ID_BUTTON_COPYTO_TEMPLATES,

    ID_BUTTON_EXECUTE,
    ID_BUTTON_AUTORECON,

    ID_MATCHER,

    ID_BUTTON_STATUS_ACCEPTED,
    ID_BUTTON_STATUS_REJECTED,
    ID_BUTTON_STATUS_ABORTED,
    ID_BUTTON_STATUS_REVOKED,
    ID_BUTTON_STATUS_PENDING,
    ID_BUTTON_STATUS_RECONCILED,

    ID_LAST
  };

  enum {
    STATUS_GROUP_NONE=0,
    STATUS_GROUP_ENQUEUED=1,
    STATUS_GROUP_ACCEPTED,
    STATUS_GROUP_REJECTED,
    STATUS_GROUP_PENDING,
    STATUS_GROUP_RECONCILED,
    STATUS_GROUP_FAILED
  };

public:
  FM_TransferStmPage(FM_TransferPage *mainPage,
                     FXComposite *p,
                     FXuint opts=0,
                     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_TransferStmPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void enter();
  void leave();
  void reset();

  void create();

  long onCmdAddTransfer(FXObject*, FXSelector, void*);
  long onCmdDeleteTransfer(FXObject*, FXSelector, void*);

  long onCmdCopyToTemplates(FXObject*, FXSelector, void*);
  long onCmdRepeatTransfer(FXObject*, FXSelector, void*);

  long onCmdNewStdTransfer(FXObject*, FXSelector, void*);
  long onCmdNewInternalTransfer(FXObject*, FXSelector, void*);
  long onCmdNewSepaTransfer(FXObject*, FXSelector, void*);

  long onCmdAutoRecon(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeRightBtnRelease(FXObject*, FXSelector, void* ptr);

  long onChgMatcher(FXObject*, FXSelector, void* ptr);

  long onCmdStatusAccepted(FXObject*, FXSelector, void*);
  long onCmdStatusRejected(FXObject*, FXSelector, void*);
  long onCmdStatusAborted(FXObject*, FXSelector, void*);
  long onCmdStatusRevoked(FXObject*, FXSelector, void*);
  long onCmdStatusPending(FXObject*, FXSelector, void*);
  long onCmdStatusReconciled(FXObject*, FXSelector, void*);


  void dump(int indent);

protected:
  FM_TransferStmPage();

  FF_GroupTreeItem *getStatusGroupItem(AE_STATEMENT_STATUS st, bool createIfMissing);
  FF_GroupTreeItem *getMonthGroupItem(const GWEN_DATE *dt, bool createIfMissing);

  void rebuild();
  int copyTransfer(AE_STATEMENT *st, AE_BOOK_TABLE_TYPE tt);
  int execStatement(AE_STATEMENT *st, AQDB_ID tid);

  int newTransfer(int tt);
  int changeStatusOfCurrentTransfer(AE_STATEMENT_STATUS sst);
  int deleteSelectedTransfers();

  FM_TransferPage *m_mainPage;

  FF_TransferTree *m_transferTree;

  FXButton *m_newStdTransferButton;
  FXButton *m_newIntTransferButton;
  FXButton *m_newSepaTransferButton;
  FXButton *m_repeatTransferButton;

  FXTextField *m_matcher;

  bool m_needRebuild;
  bool m_isVisible;
};




#endif

