/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_TRANSFERSTOPAGE_HPP
#define FM_TRANSFERSTOPAGE_HPP


#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_TransferTree;
class FM_TransferPage;


class FM_TransferStoPage: public FXVerticalFrame {
  FXDECLARE(FM_TransferStoPage)

public:
  enum {
    ID_TRANSFER_TREE=FXPacker::ID_LAST,

    ID_BUTTON_CREATE_STO,
    ID_BUTTON_MODIFY_STO,
    ID_BUTTON_DELETE_STO,

    ID_MATCHER,

    ID_LAST
  };

public:
  FM_TransferStoPage(FM_TransferPage *mainPage,
                     FXComposite *p,
                     FXuint opts=0,
                     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_TransferStoPage();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void enter();
  void leave();
  void reset();

  void create();

  long onCmdCreateSto(FXObject*, FXSelector, void*);
  long onCmdModifySto(FXObject*, FXSelector, void*);
  long onCmdDeleteSto(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  long onChgMatcher(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

protected:
  FM_TransferStoPage();

  void rebuild();
  int createSto(void);
  int deleteSto(const AE_STATEMENT *st);
  int modifySto(const AE_STATEMENT *st);

  FM_TransferPage *m_mainPage;

  FF_TransferTree *m_transferTree;

  FXButton *m_createStoButton;
  FXButton *m_modifyStoButton;
  FXButton *m_deleteStoButton;

  FXTextField *m_matcher;

  bool m_needRebuild;
  bool m_isVisible;
};




#endif

