/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_app.hpp"
#include "fm_vehiclespage.hpp"
#include "fm_vehiclesvtab.hpp"
#include "fm_vehiclesrtab.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FM_VehiclesPage::FM_VehiclesPage(FXComposite *p,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FF_ModTabBook(p, opts, x, y, w, h)
,m_vehicleTab(NULL)
{
  /* create tab 1 */
  addTabHeader(I18N("Vehicles"), false);
  m_vehicleTab=new FM_VehiclesVehicleTab("vehiclePage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(m_vehicleTab);
}



FM_VehiclesPage::FM_VehiclesPage()
:FF_ModTabBook()
,m_vehicleTab(NULL)
{
}



FM_VehiclesPage::~FM_VehiclesPage() {
}



void FM_VehiclesPage::openVehiclePage(const AE_VEHICLE *veh) {
  char nbuf[64];
  FXString pageId;
  FF_ModTabPage *p;
  AQDB_ID vid;

  vid=AE_Vehicle_GetId(veh);
  /* TODO: prepend book id */
  snprintf(nbuf, sizeof(nbuf)-1, "%lld", (unsigned long long int) vid);
  nbuf[sizeof(nbuf)-1]=0;

  pageId=FXString(nbuf);
  p=findPageById(pageId);
  if (p==NULL) {
    const char *s;
    FXString title;
    FXTabItem *tab;

    /* create page */
    s=AE_Vehicle_GetName(veh);
    if (s==NULL || *s==0)
      s=AE_Vehicle_GetLicensePlate(veh);
    if (s==NULL || *s==0)
      s=AE_Vehicle_GetIdentificationNumber(veh);
    title=FXString(s);

    DBG_DEBUG(0, "Creating page %s (%s)", title.text(), pageId.text());
    tab=addTabHeader(title, true);
    p=new FM_VehiclesRefuelingTab(veh, pageId, this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    tab->create();
    p->create();
    addTabPage(p);
    p->init(m_config);
  }
  else {
    DBG_DEBUG(0, "Page %s already open, changing to it", pageId.text());
  }

  if (p)
    setCurrentPage(p);
}

