/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_vehiclesrtab.hpp"
#include "fm_vehiclespage.hpp"
#include "ff_refuelingtree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_editrefueling.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_VehiclesRefuelingTab) FM_VehiclesRefuelingTabMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesRefuelingTab::ID_BUTTON_ADDREFUELING,
            FM_VehiclesRefuelingTab::onCmdAddRefueling),
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesRefuelingTab::ID_BUTTON_EDITREFUELING,
            FM_VehiclesRefuelingTab::onCmdEditRefueling),
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesRefuelingTab::ID_BUTTON_DELREFUELING,
            FM_VehiclesRefuelingTab::onCmdDelRefueling),

  FXMAPFUNC(SEL_CHANGED, FM_VehiclesRefuelingTab::ID_REFUELING_TREE,
            FM_VehiclesRefuelingTab::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_VehiclesRefuelingTab::ID_REFUELING_TREE,
            FM_VehiclesRefuelingTab::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_VehiclesRefuelingTab::ID_REFUELING_TREE,
            FM_VehiclesRefuelingTab::onTreeRightBtnPress),

};



FXIMPLEMENT(FM_VehiclesRefuelingTab, FF_ModTabPage,
            FM_VehiclesRefuelingTabMap, ARRAYNUMBER(FM_VehiclesRefuelingTabMap))






FM_VehiclesRefuelingTab::FM_VehiclesRefuelingTab(const AE_VEHICLE *vehicle,
                                                 const FXString &pageId,
                                                 FM_VehiclesPage *mainPage,
                                                 FXuint opts,
                                                 FXint x, FXint y, FXint w, FXint h)
:FF_ModTabPage(pageId, mainPage, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_vehicle(NULL)
,m_refuelingTree(NULL)
,m_isEditing(false)
{
  FXIcon *ic;
  FF_App *app;

  m_closeable=true;

  assert(vehicle);
  m_vehicle=AE_Vehicle_dup(vehicle);

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_refuelingTree=new FF_RefuelingTree(this, this, ID_REFUELING_TREE,
                                       LAYOUT_SIDE_LEFT |
                                       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                       FOLDINGLIST_EXTENDEDSELECT |
                                       FOLDINGLIST_SHOWS_LINES |
                                       FOLDINGLIST_SHOWS_BOXES |
                                       FOLDINGLIST_ROOT_BOXES |
                                       FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_refuelingTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_refuelingTree->setIconClosed(ic);
}




FM_VehiclesRefuelingTab::FM_VehiclesRefuelingTab()
:FF_ModTabPage()
,m_mainPage(NULL)
,m_vehicle(NULL)
,m_refuelingTree(NULL)
,m_isEditing(false)
{
}



FM_VehiclesRefuelingTab::~FM_VehiclesRefuelingTab() {
  AE_Vehicle_free(m_vehicle);
}



void FM_VehiclesRefuelingTab::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  FXString pgId;

  DBG_INFO(0, "Init");

  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_DATE, 100);
  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_MILEAGE, 100);
  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_FUELAMOUNT, 100);
  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_FUELPRICE, 100);
  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_FLAGS, 100);
  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_PETROLSTATION, 100);
  m_refuelingTree->addColumn(FF_RefuelingTree::COLUMN_MEMO, 100);

  pgId="refuelingPage";
  pgId+=getPageId();
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, pgId.text());
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_refuelingTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_refuelingTree->getHeaderSize(i));
      m_refuelingTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_refuelingTree->setHeaderArrowDir(i, TRUE);
        else
          m_refuelingTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
  reload();
}



void FM_VehiclesRefuelingTab::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  FXString pgId;
  int i;

  DBG_INFO(0, "Fini");

  pgId="refuelingPage";
  pgId+=getPageId();
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, pgId.text());
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_refuelingTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_refuelingTree->getHeaderSize(i));
    b=m_refuelingTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_VehiclesRefuelingTab::create(){
  FF_ModTabPage::create();
  m_refuelingTree->create();
}



long FM_VehiclesRefuelingTab::onCmdAddRefueling(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_REFUELING *tmpl;

    tmpl=AE_Refueling_new();
    AE_Refueling_SetVehicleId(tmpl, AE_Vehicle_GetId(m_vehicle));
    AE_Refueling_SetFuelType(tmpl, AE_Vehicle_GetFuelType(m_vehicle));

    if (FF_EditRefueling::createRefuelings(app, this,
                                           I18N("Add Refueling"),
                                           tmpl)) {
      DBG_ERROR(0, "Added refueling(s)");
    }
    AE_Refueling_free(tmpl);
  }

  return 1;
}



long FM_VehiclesRefuelingTab::onCmdEditRefueling(FXObject*, FXSelector, void*) {
  AE_REFUELING *re;

  re=m_refuelingTree->getCurrentRefueling();
  if (re) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditRefueling::editRefueling(app, this,
                                          I18N("Edit Refueling"),
                                          re)) {
        int rv;

        rv=AE_Book_WriteRefueling(b, AE_Book_TableType_Refueling, re, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to write refueling back (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_Refueling);
      }
    }
  }

  return 1;
}



long FM_VehiclesRefuelingTab::onCmdDelRefueling(FXObject*, FXSelector, void*) {
#if 0
  AE_REFUELING *re;

  re=m_refuelingTree->getCurrentRefueling();
  if (veh) {
    FF_App *app;
    AQDB_ID id;

    app=dynamic_cast<FF_App*>(getApp());
    id=AE_Refueling_GetId(re);
    if (id)
      app->deleteRefueling(id);
    app->signalDbReload(AE_Book_TableType_Refueling);
  }
#else
# warning Need to implemente this!
#endif

  return 1;
}



long FM_VehiclesRefuelingTab::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        //FF_RefuelingTreeItem *baItem=dynamic_cast<FF_RefuelingTreeItem*>(item);
        //AE_VEHICLE *veh=baItem->getVehicle();

      }
    }
  }
  return 1;
}



long FM_VehiclesRefuelingTab::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item && item->isDataItem()) {
      FF_RefuelingTreeItem *baItem=dynamic_cast<FF_RefuelingTreeItem*>(item);
      if (baItem) {
        AE_REFUELING *re;

        re=baItem->getRefueling();
        if (re) {
          FF_App *app;
          AE_BOOK *b;
      
          app=dynamic_cast<FF_App*>(getApp());
          b=app->getBook();
          if (b) {
            if (FF_EditRefueling::editRefueling(app, this,
                                                I18N("Edit Refueling"),
                                                re)) {
              int rv;

              rv=AE_Book_WriteRefueling(b, AE_Book_TableType_Refueling, re, 1);
              if (rv<0) {
                DBG_ERROR(0, "Unable to write refueling back (%d)", rv);
              }
              app->signalDbReload(AE_Book_TableType_Refueling);
            }
          }
        }
      }
    }
  }
  return 1;
}



long FM_VehiclesRefuelingTab::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_REFUELING *re;

  re=m_refuelingTree->getCurrentRefueling();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    new FXMenuCommand(&pmenu, I18N("Add Refueling"), NULL, this, ID_BUTTON_ADDREFUELING);

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Refueling"), NULL, this, ID_BUTTON_EDITREFUELING);
    if (!re) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Refueling"), NULL, this, ID_BUTTON_DELREFUELING);
    if (!re) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_VehiclesRefuelingTab::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_VehiclesRefuelingTab::dbClosing() {
  m_refuelingTree->clearItems();
}



void FM_VehiclesRefuelingTab::dbClosed() {
}



void FM_VehiclesRefuelingTab::reload() {
  FF_App *app;
  AE_BOOK *b;
  AE_REFUELING *re;
  AQDB_ID currentRefuelingId=0;

  re=m_refuelingTree->getCurrentRefueling();
  if (re)
    currentRefuelingId=AE_Refueling_GetId(re);

  m_refuelingTree->clearItems();
  m_refuelingTree->reload();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_VEHICLE_LIST *rel;
    char qexpr[64];

    snprintf(qexpr, sizeof(qexpr)-1, "$vehicleId==%lld", (unsigned long long int) AE_Vehicle_GetId(m_vehicle));
    qexpr[sizeof(qexpr)-1]=0;

    /* read refuelings */
    rel=AE_Vehicle_List_new();
    rv=AE_Book_QueryRefuelingList(b, AE_Book_TableType_Refueling, 0, qexpr, rel);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      AE_REFUELING *re;

      while ( (re=AE_Refueling_List_First(rel)) ) {
        AE_Refueling_List_Del(re);

        /* add refueling */
        m_refuelingTree->addRefueling(re, false, 0);
      }
    }
    AE_Refueling_List_free(rel);

    m_refuelingTree->sort();
#if 0
    for (i=0; i<m_refuelingTree->getNumHeaders(); i++) {
      FXbool b;

      b=m_refuelingTree->getHeaderArrowDir(i);
      if (b!=MAYBE) {
        if (b==TRUE)
          m_refuelingTree->sortByColumn(i, true);
        else
          m_refuelingTree->sortByColumn(i, false);
        break;
      }
    }
#endif

    m_refuelingTree->expandItems();
  }

  if (currentRefuelingId!=0) {
    FF_RefuelingTreeItem *item=m_refuelingTree->findRefuelingItem(currentRefuelingId);
    if (item) {
      item->setSelected(true);
      m_refuelingTree->makeItemVisible(item);
      m_refuelingTree->setCurrentItem(item);
    }
    else {
      DBG_ERROR(0, "Item not found");
    }
  }
}



void FM_VehiclesRefuelingTab::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_Refueling ||
      tt==AE_Book_TableType_Commodity ||
      tt==AE_Book_TableType_None) {
    reload();
  }
  else if (tt==AE_Book_TableType_Vehicle) {
    /* TODO: reload vehicle */
  }
}



void FM_VehiclesRefuelingTab::reset() {
}



