/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_VEHICLESRTAB_HPP
#define FM_VEHICLESRTAB_HPP


#include "ff_refuelingtree.hpp"
#include "ff_modtabpage.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_RefuelingTree;
class FM_VehiclesPage;


class FM_VehiclesRefuelingTab: public FF_ModTabPage {
  FXDECLARE(FM_VehiclesRefuelingTab)

public:
  enum {
    ID_REFUELING_TREE=FXPacker::ID_LAST,

    ID_BUTTON_ADDREFUELING,
    ID_BUTTON_EDITREFUELING,
    ID_BUTTON_DELREFUELING,

    ID_LAST
  };

public:
  FM_VehiclesRefuelingTab(const AE_VEHICLE *vehicle,
                          const FXString &pageId,
                          FM_VehiclesPage *mainPage,
                          FXuint opts=0,
                          FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_VehiclesRefuelingTab();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void reset();

  void create();

  long onCmdAddRefueling(FXObject*, FXSelector, void*);
  long onCmdEditRefueling(FXObject*, FXSelector, void*);
  long onCmdDelRefueling(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeCommand(FXObject*, FXSelector, void* ptr);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  bool isEditing() const { return m_isEditing;};

protected:
  FM_VehiclesRefuelingTab();
  void reload();

  FM_VehiclesPage *m_mainPage;
  AE_VEHICLE *m_vehicle;

  FF_RefuelingTree *m_refuelingTree;

  bool m_isEditing;
};




#endif

