/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_vehiclesvtab.hpp"
#include "fm_vehiclespage.hpp"
#include "ff_vehicletree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_editvehicle.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_VehiclesVehicleTab) FM_VehiclesVehicleTabMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesVehicleTab::ID_BUTTON_ADDGROUP,
            FM_VehiclesVehicleTab::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesVehicleTab::ID_BUTTON_EDITGROUP,
            FM_VehiclesVehicleTab::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesVehicleTab::ID_BUTTON_DELGROUP,
            FM_VehiclesVehicleTab::onCmdDelGroup),

  FXMAPFUNC(SEL_COMMAND, FM_VehiclesVehicleTab::ID_BUTTON_ADDVEHICLE,
            FM_VehiclesVehicleTab::onCmdAddVehicle),
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesVehicleTab::ID_BUTTON_EDITVEHICLE,
            FM_VehiclesVehicleTab::onCmdEditVehicle),
  FXMAPFUNC(SEL_COMMAND, FM_VehiclesVehicleTab::ID_BUTTON_DELVEHICLE,
            FM_VehiclesVehicleTab::onCmdDelVehicle),

  FXMAPFUNC(SEL_CHANGED, FM_VehiclesVehicleTab::ID_VEHICLE_TREE,
            FM_VehiclesVehicleTab::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_VehiclesVehicleTab::ID_VEHICLE_TREE,
            FM_VehiclesVehicleTab::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_VehiclesVehicleTab::ID_VEHICLE_TREE,
            FM_VehiclesVehicleTab::onTreeRightBtnPress),

};



FXIMPLEMENT(FM_VehiclesVehicleTab, FF_ModTabPage,
            FM_VehiclesVehicleTabMap, ARRAYNUMBER(FM_VehiclesVehicleTabMap))






FM_VehiclesVehicleTab::FM_VehiclesVehicleTab(const FXString &pageId,
                                             FM_VehiclesPage *mainPage,
                                             FXuint opts,
                                             FXint x, FXint y, FXint w, FXint h)
:FF_ModTabPage(pageId, mainPage, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_vehicleTree(NULL)
,m_isEditing(false)
{
  FXIcon *ic;
  FF_App *app;

  m_closeable=false;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_vehicleTree=new FF_VehicleTree(this, this, ID_VEHICLE_TREE,
                                   LAYOUT_SIDE_LEFT |
                                   LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                   FOLDINGLIST_EXTENDEDSELECT |
                                   FOLDINGLIST_SHOWS_LINES |
                                   FOLDINGLIST_SHOWS_BOXES |
                                   FOLDINGLIST_ROOT_BOXES |
                                   FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_vehicleTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_vehicleTree->setIconClosed(ic);

}




FM_VehiclesVehicleTab::FM_VehiclesVehicleTab()
:FF_ModTabPage()
,m_mainPage(NULL)
,m_vehicleTree(NULL)
,m_isEditing(false)
{
}



FM_VehiclesVehicleTab::~FM_VehiclesVehicleTab() {
}



void FM_VehiclesVehicleTab::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");

  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_NAME, 100);
  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_MAKE, 100);
  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_MODEL, 100);
  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_LICENSEPLATE, 100);
  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_MEMO, 100);
  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_FUELCAPACITY, 100);
  m_vehicleTree->addColumn(FF_VehicleTree::COLUMN_FUELTYPE, 100);

  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "vehiclePage");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_vehicleTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_vehicleTree->getHeaderSize(i));
      m_vehicleTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_vehicleTree->setHeaderArrowDir(i, TRUE);
        else
          m_vehicleTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_VehiclesVehicleTab::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "vehiclePage");
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_vehicleTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_vehicleTree->getHeaderSize(i));
    b=m_vehicleTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_VehiclesVehicleTab::create(){
  FF_ModTabPage::create();
  m_vehicleTree->create();
}



void FM_VehiclesVehicleTab::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_VehicleView: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_vehicleTree->dump(indent+2);
}



long FM_VehiclesVehicleTab::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_vehicleTree->getFirstSelectedGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    m_isEditing=true;
    if (FF_EditGroup::editGroup(app,
				this,
                                I18N("Add Vehicle Group"),
                                AE_Book_TableType_VehicleGroup,
                                gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_VehicleGroup, gr, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_vehicleTree->addGroup(gr);
      item=m_vehicleTree->findGroupItem(gr);
      assert(item);
      m_vehicleTree->makeItemVisible(item);
      m_vehicleTree->unselectAll();
      item->setSelected(TRUE);
      m_vehicleTree->setCurrentItem(item, TRUE);
      m_isEditing=false;
      app->signalDbReload(AE_Book_TableType_VehicleGroup);
    }
    else
      AE_Group_free(gr);
    m_isEditing=false;
  }

  return 1;
}



long FM_VehiclesVehicleTab::onCmdEditGroup(FXObject*, FXSelector, void*) {
  AE_GROUP *gr;

  gr=m_vehicleTree->getFirstSelectedGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      m_isEditing=true;
      if (FF_EditGroup::editGroup(app,
                                  this,
                                  I18N("Edit Vehicle Group"),
                                  AE_Book_TableType_VehicleGroup,
                                  gr)) {
        int rv;
        FXString t;

        rv=AE_Book_WriteGroup(b, AE_Book_TableType_VehicleGroup, gr, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to edit group (%d)", rv);
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Database Error"),
                              I18N("Unable to write object to database: %d"), rv);
        }

        m_isEditing=false;
        app->signalDbReload(AE_Book_TableType_VehicleGroup);
      }
      m_isEditing=false;
    }
  }
  return 1;
}



long FM_VehiclesVehicleTab::onCmdDelGroup(FXObject*, FXSelector, void*) {
#if 0
  AE_GROUP *gr;

  gr=m_vehicleTree->getFirstSelectedGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint res;

      res=FXMessageBox::question(this, MBOX_YES_NO,
                                 I18N("Delete Group"),
                                 "%s",
                                 I18N("Are you sure you want to delete the selected "
                                      "group?"));
      if (res!=MBOX_CLICKED_YES) {
        DBG_ERROR(0, "Aborted");
      }
      else {
        app->deleteVehicleGroup(AE_Group_GetId(gr));
        app->signalDbReload(AE_Book_TableType_VehicleGroup);
        app->signalDbReload(AE_Book_TableType_Vehicle);
      }
    }
  }
#else
# warning Need to implemente this!
  FXMessageBox::error(this, MBOX_OK,
                      I18N("Not Implemented"),
                      "%s",
                      I18N("The function your are calling is not yet implemented."));
#endif

  return 1;
}



long FM_VehiclesVehicleTab::onCmdAddVehicle(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_VEHICLE *veh;
    AE_GROUP *gr;

    veh=AE_Vehicle_new();
    AE_Vehicle_SetFuelCapacityUnits(veh, AE_Units_Liters);
    gr=m_vehicleTree->getFirstSelectedGroup();
    if (gr)
      AE_Vehicle_SetGroupId(veh, AE_Group_GetId(gr));
    else
      AE_Vehicle_SetGroupId(veh, 0);
    if (FF_EditVehicle::editVehicle(app, this,
                                    I18N("Add Vehicle"),
                                    veh)) {
      int rv;

      rv=AE_Book_AddVehicle(b, AE_Book_TableType_Vehicle, veh, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add vehicle (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_Vehicle);
    }
    AE_Vehicle_free(veh);
  }

  return 1;
}



long FM_VehiclesVehicleTab::onCmdEditVehicle(FXObject*, FXSelector, void*) {
  AE_VEHICLE *veh;

  veh=m_vehicleTree->getFirstSelectedVehicle();
  if (veh) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditVehicle::editVehicle(app, this,
                                      I18N("Edit Vehicle"),
                                      veh)) {
        int rv;

        rv=AE_Book_WriteVehicle(b, AE_Book_TableType_Vehicle, veh, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to write vehicle back (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_Vehicle);
      }
    }
  }

  return 1;
}



long FM_VehiclesVehicleTab::onCmdDelVehicle(FXObject*, FXSelector, void*) {
#if 0
  AE_VEHICLE *veh;

  veh=m_vehicleTree->getFirstSelectedVehicle();
  if (veh) {
    FF_App *app;
    AQDB_ID id;

    app=dynamic_cast<FF_App*>(getApp());
    id=AE_Vehicle_GetId(veh);
    if (id)
      app->deleteVehicle(id);
    app->signalDbReload(AE_Book_TableType_Vehicle);
  }

#else
# warning Need to implemente this!
#endif

  return 1;
}



long FM_VehiclesVehicleTab::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        //FF_VehicleTreeItem *baItem=dynamic_cast<FF_VehicleTreeItem*>(item);
        //AE_VEHICLE *veh=baItem->getVehicle();

      }
    }
  }
  return 1;
}



long FM_VehiclesVehicleTab::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item && item->isDataItem()) {
      FF_VehicleTreeItem *baItem=dynamic_cast<FF_VehicleTreeItem*>(item);
      if (baItem) {
        m_mainPage->openVehiclePage(baItem->getVehicle());
      }
    }
  }
  return 1;
}



long FM_VehiclesVehicleTab::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_GROUP *gr;
  AE_VEHICLE *veh;

  gr=m_vehicleTree->getCurrentGroup();
  veh=m_vehicleTree->getCurrentVehicle();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    new FXMenuCommand(&pmenu, I18N("Add Group"), NULL, this, ID_BUTTON_ADDGROUP);
    new FXMenuCommand(&pmenu, I18N("Add Vehicle"), NULL, this, ID_BUTTON_ADDVEHICLE);

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Group"), NULL, this, ID_BUTTON_EDITGROUP);
    if (!gr) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Edit Group"), NULL, this, ID_BUTTON_EDITGROUP);
    if (!gr) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Group"), NULL, this, ID_BUTTON_DELGROUP);
    if (!gr) c->disable();

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Vehicle"), NULL, this, ID_BUTTON_EDITVEHICLE);
    if (!veh) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Vehicle"), NULL, this, ID_BUTTON_DELVEHICLE);
    if (!veh) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_VehiclesVehicleTab::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_VehiclesVehicleTab::dbClosing() {
  m_vehicleTree->clearItems();
}



void FM_VehiclesVehicleTab::dbClosed() {
}



void FM_VehiclesVehicleTab::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_VehicleGroup ||
      tt==AE_Book_TableType_Vehicle ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
    AE_BOOK *b;
    AE_GROUP *gr;
    AE_VEHICLE *ve;
    AQDB_ID currentVehicleId=0;
    AQDB_ID currentGrId=0;
  
    gr=m_vehicleTree->getCurrentGroup();
    if (gr)
      currentGrId=AE_Group_GetId(gr);
    ve=m_vehicleTree->getCurrentVehicle();
    if (ve)
      currentVehicleId=AE_Vehicle_GetId(ve);
  
    m_vehicleTree->clearItems();
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AE_GROUP_TREE *gt;
      int rv;
      AE_VEHICLE_LIST *bal;
      int i;

      gt=AE_Group_Tree_new();
      rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_VehicleGroup, gt);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_vehicleTree->addGroups(gt);
      }
      AE_Group_Tree_free(gt);

      /* read vehicles */
      bal=AE_Vehicle_List_new();
      rv=AE_Book_ReadVehicleList(b, AE_Book_TableType_Vehicle, bal);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	AE_VEHICLE *ve;
  
	while ( (ve=AE_Vehicle_List_First(bal)) ) {
	  AE_Vehicle_List_Del(ve);

          /* add vehicle */
          m_vehicleTree->addVehicle(ve, false, 0);
        }
      }
      AE_Vehicle_List_free(bal);

      for (i=0; i<m_vehicleTree->getNumHeaders(); i++) {
	FXbool b;
    
	b=m_vehicleTree->getHeaderArrowDir(i);
	if (b!=MAYBE) {
	  if (b==TRUE)
	    m_vehicleTree->sortByColumn(i, true);
	  else
	    m_vehicleTree->sortByColumn(i, false);
	  break;
	}
      }

      m_vehicleTree->expandItems();
    }

    if (currentVehicleId!=0) {
      FF_VehicleTreeItem *item=m_vehicleTree->findVehicleItem(currentVehicleId);
      if (item) {
	item->setSelected(true);
	m_vehicleTree->makeItemVisible(item);
	m_vehicleTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
    else if (currentGrId!=0) {
      FF_GroupTreeItem *item=m_vehicleTree->findGroupItem(currentGrId);
      if (item) {
	item->setSelected(true);
	m_vehicleTree->makeItemVisible(item);
	m_vehicleTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
  }
}



void FM_VehiclesVehicleTab::reset() {
}






