/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_VEHICLESVTAB_HPP
#define FM_VEHICLESVTAB_HPP


#include "ff_vehicletree.hpp"
#include "ff_modtabpage.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_VehicleTree;
class FM_VehiclesPage;


class FM_VehiclesVehicleTab: public FF_ModTabPage {
  FXDECLARE(FM_VehiclesVehicleTab)

public:
  enum {
    ID_VEHICLE_TREE=FXPacker::ID_LAST,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_BUTTON_ADDVEHICLE,
    ID_BUTTON_EDITVEHICLE,
    ID_BUTTON_DELVEHICLE,

    ID_LAST
  };

public:
  FM_VehiclesVehicleTab(const FXString &pageId,
                        FM_VehiclesPage *mainPage,
                        FXuint opts=0,
                        FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_VehiclesVehicleTab();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void reset();

  void create();

  void openVehiclePage(const AE_VEHICLE *veh);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onCmdAddVehicle(FXObject*, FXSelector, void*);
  long onCmdEditVehicle(FXObject*, FXSelector, void*);
  long onCmdDelVehicle(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeCommand(FXObject*, FXSelector, void* ptr);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

  bool isEditing() const { return m_isEditing;};

protected:
  FM_VehiclesVehicleTab();

  FM_VehiclesPage *m_mainPage;

  FF_VehicleTree *m_vehicleTree;

  bool m_isEditing;
};




#endif

