/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_BANKACCOUNTVIEW_HPP
#define FF_BANKACCOUNTVIEW_HPP


#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>

#include <list>


class FF_BankAccountTree;


class FF_BankAccountView: public FXHorizontalFrame {
  FXDECLARE(FF_BankAccountView)

public:
  enum {
    ID_ACCOUNT_TREE=FXPacker::ID_LAST,
    ID_BUTTON_ADDACCOUNT,
    ID_BUTTON_EDITACCOUNT,
    ID_BUTTON_DELACCOUNT,

    ID_LAST
  };

  enum {
    FF_BANKACCOUNTVIEW_OPTS_SHOWGROUP = 0x80000000
  };

public:
  FF_BankAccountView(FXComposite *p,
		     FXuint opts=0,
		     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FF_BankAccountView();

  void enter();
  void leave();

  void create();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void toGui(const AE_BANKACCOUNT_LIST *bankAccounts);
  void fromGui(AE_BANKACCOUNT_LIST *bankAccounts);

  void jobListUpdated();

  bool isEditing() const { return m_isEditing;};

  void clear();
  void sortAccounts();

  AE_BANKACCOUNT *getCurrentBankAccount();
  AE_BANKACCOUNT *getFirstSelectedBankAccount();
  void getSelectedBankAccounts(AE_BANKACCOUNT_LIST2 *al);


  long onCmdAddAccount(FXObject*, FXSelector, void*);
  long onCmdEditAccount(FXObject*, FXSelector, void*);
  long onCmdDelAccount(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);

  void dump(int indent);

protected:
  FF_BankAccountView();

  FF_BankAccountTree *m_bankAccountTree;

  FXButton *m_addAccountButton;
  FXButton *m_editAccountButton;
  FXButton *m_delAccountButton;

  AE_BOOK_TABLE_TYPE m_type;
  bool m_isEditing;
};




#endif

