/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_currencyview.hpp"
#include "ff_app.hpp"
#include "ff_currencytree.hpp"
#include "ff_editcurrency.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_CurrencyView) FF_CurrencyViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_CurrencyView::ID_BUTTON_ADD,
            FF_CurrencyView::onCmdCurrencyAdd),
  FXMAPFUNC(SEL_COMMAND, FF_CurrencyView::ID_BUTTON_EDIT,
            FF_CurrencyView::onCmdCurrencyEdit),
  FXMAPFUNC(SEL_COMMAND, FF_CurrencyView::ID_BUTTON_DEL,
            FF_CurrencyView::onCmdCurrencyDel)
};


            
FXIMPLEMENT(FF_CurrencyView, FXHorizontalFrame,
            FF_CurrencyViewMap, ARRAYNUMBER(FF_CurrencyViewMap))




FF_CurrencyView::FF_CurrencyView(FXComposite *p,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_currencyTree(NULL)
,m_addCurrencyButton(NULL)
,m_editCurrencyButton(NULL)
,m_delCurrencyButton(NULL) {

  FXVerticalFrame *vf;

  m_currencyTree=new FF_CurrencyTree(this, this, ID_CURRENCY_TREE,
                                     LAYOUT_FILL_X | LAYOUT_FILL_Y);
  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_addCurrencyButton=new THEMEBUTTON(vf, I18N("Add Currency"), NULL, this, ID_BUTTON_ADD);
  m_editCurrencyButton=new THEMEBUTTON(vf, I18N("Edit Currency"), NULL, this, ID_BUTTON_EDIT);
  m_delCurrencyButton=new THEMEBUTTON(vf, I18N("Delete Currency"), NULL, this, ID_BUTTON_DEL);
}



FF_CurrencyView::FF_CurrencyView()
:FXHorizontalFrame()
,m_currencyTree(NULL)
,m_addCurrencyButton(NULL)
,m_editCurrencyButton(NULL)
,m_delCurrencyButton(NULL) {
}



FF_CurrencyView::~FF_CurrencyView() {
}



void FF_CurrencyView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_currencyTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_currencyTree->getHeaderSize(i));
    m_currencyTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_currencyTree->setHeaderArrowDir(i, TRUE);
      else
        m_currencyTree->setHeaderArrowDir(i, FALSE);
    }
  }

  rebuild();
}



void FF_CurrencyView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_currencyTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_currencyTree->getHeaderSize(i));
    b=m_currencyTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_CurrencyView::rebuild() {
  FF_App *app;
  AE_BOOK *b;

  m_currencyTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_COMMODITY_LIST *cl;
    int rv;
    int i;

    cl=AE_Commodity_List_new();
    /* only list currencies */
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Commodity_List_free(cl);
    }
    else {
      AE_COMMODITY *co;

      while ( (co=AE_Commodity_List_First(cl)) ) {
        AE_Commodity_List_Del(co);
        m_currencyTree->addCurrency(co);
      }
    }

    for (i=0; i<m_currencyTree->getNumHeaders(); i++) {
      FXbool b;
  
      b=m_currencyTree->getHeaderArrowDir(i);
      if (b!=MAYBE) {
        if (b==TRUE)
          m_currencyTree->sortByColumn(i, true);
        else
          m_currencyTree->sortByColumn(i, false);
        break;
      }
    }
  }
}



void FF_CurrencyView::create() {
  FXHorizontalFrame::create();
  m_currencyTree->create();
}



long FF_CurrencyView::onCmdCurrencyAdd(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_COMMODITY *co;
    AQDB_VALUE *v;

    co=AE_Commodity_new();
    v=AQDB_Value_fromString("0.01");
    AE_Commodity_SetNameSpace(co, AE_CURRENCY_NAMESPACE_ISO_4217_CODE);
    AE_Commodity_SetSmallestUnit(co, v);
    AQDB_Value_free(v);

    if (FF_EditCurrency::editCurrency(app, this, I18N("Add Currency"), co)) {
      int rv;

      rv=AE_Book_AddCommodity(b, AE_Book_TableType_Commodity, co, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add commodity (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_Commodity);
      rebuild();
    }
    AE_Commodity_free(co);
  }

  return 1;
}



long FF_CurrencyView::onCmdCurrencyEdit(FXObject*, FXSelector, void* ptr) {
  AE_COMMODITY *co;

  co=m_currencyTree->getCurrentCurrency();
  if (co) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditCurrency::editCurrency(app, this, I18N("Edit Currency"), co)) {
        int rv;

        rv=AE_Book_WriteCommodity(b, AE_Book_TableType_Commodity, co, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add currency (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_Commodity);
      }
    }
  }
  return 1;
}



long FF_CurrencyView::onCmdCurrencyDel(FXObject*, FXSelector, void* ptr) {
  AE_COMMODITY *co;

  co=m_currencyTree->getCurrentCurrency();
  if (co) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint res;

      res=FXMessageBox::question(this, MBOX_YES_NO,
				 I18N("Delete Currency"),
                                 "%s",
				 I18N("Are you sure you want to delete the selected "
				      "currency?"));
      if (res!=MBOX_CLICKED_YES) {
	DBG_ERROR(0, "Aborted");
      }
      else {
	int rv;

        rv=AE_Book_DeleteCommodity(b, AE_Book_TableType_Commodity, co, 1);
	if (rv<0) {
          DBG_ERROR(0, "Unable to delete commodity (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_Commodity);
      }
    }
  }
  return 1;
}









