/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_itemview.hpp"
#include "ff_itemtree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_edititem.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ItemView) FF_ItemViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_ItemView::ID_BUTTON_ADDTMPL,
            FF_ItemView::onCmdAddItem),
  FXMAPFUNC(SEL_COMMAND, FF_ItemView::ID_BUTTON_EDITTMPL,
            FF_ItemView::onCmdEditItem),
  FXMAPFUNC(SEL_COMMAND, FF_ItemView::ID_BUTTON_DELTMPL,
            FF_ItemView::onCmdDelItem),

  FXMAPFUNC(SEL_COMMAND, FF_ItemView::ID_BUTTON_ADDGROUP,
            FF_ItemView::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FF_ItemView::ID_BUTTON_EDITGROUP,
            FF_ItemView::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FF_ItemView::ID_BUTTON_DELGROUP,
            FF_ItemView::onCmdDelGroup),

  FXMAPFUNC(SEL_CHANGED, FF_ItemView::ID_ITEM_TREE,
            FF_ItemView::onTreeChanged),

};



FXIMPLEMENT(FF_ItemView, FXHorizontalFrame,
            FF_ItemViewMap, ARRAYNUMBER(FF_ItemViewMap))






FF_ItemView::FF_ItemView(AE_BOOK_TABLE_TYPE tt,
			 FXComposite *p,
			 FXuint opts,
			 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_tableType(tt)
,m_itemTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addItemButton(NULL)
,m_editItemButton(NULL)
,m_delItemButton(NULL)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_itemTree=new FF_ItemTree(this, this, ID_ITEM_TREE,
				   LAYOUT_SIDE_LEFT |
				   LAYOUT_FILL_X | LAYOUT_FILL_Y |
				   FOLDINGLIST_EXTENDEDSELECT |
				   FOLDINGLIST_SHOWS_LINES |
				   FOLDINGLIST_SHOWS_BOXES |
				   FOLDINGLIST_ROOT_BOXES |
				   FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_itemTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_itemTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_addGroupButton=new THEMEBUTTON(vf,
				   I18N("Add Group"),
				   NULL,
				   this, ID_BUTTON_ADDGROUP,
				   BUTTON_NORMAL);
  m_editGroupButton=new THEMEBUTTON(vf,
				    I18N("Edit Group"),
				    NULL,
				    this, ID_BUTTON_EDITGROUP,
				    BUTTON_NORMAL);
  m_delGroupButton=new THEMEBUTTON(vf,
				   I18N("Delete Group"),
				   NULL,
				   this, ID_BUTTON_DELGROUP,
				   BUTTON_NORMAL);

  new FXLabel(vf, "");
  new FXHorizontalSeparator(vf, SEPARATOR_GROOVE|LAYOUT_FILL_X);
  new FXLabel(vf, "");

  m_addItemButton=new THEMEBUTTON(vf,
				  I18N("Add Item"),
				  NULL,
				  this, ID_BUTTON_ADDTMPL,
				  BUTTON_NORMAL);
  m_editItemButton=new THEMEBUTTON(vf,
				   I18N("Edit Item"),
				   NULL,
				   this, ID_BUTTON_EDITTMPL,
				   BUTTON_NORMAL);
  m_delItemButton=new THEMEBUTTON(vf,
				  I18N("Delete Item"),
                                  NULL,
                                  this, ID_BUTTON_DELTMPL,
                                  BUTTON_NORMAL);
}



FF_ItemView::FF_ItemView()
:FXHorizontalFrame()
,m_itemTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addItemButton(NULL)
,m_editItemButton(NULL)
,m_delItemButton(NULL)
{
}



FF_ItemView::~FF_ItemView() {
}



void FF_ItemView::create(){
  FXHorizontalFrame::create();
  m_itemTree->create();

  m_addItemButton->create();
  m_editItemButton->create();
  m_delItemButton->create();
}



void FF_ItemView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_itemTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_itemTree->getHeaderSize(i));
    m_itemTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      m_itemTree->setHeaderArrowDir(i, sortUp?TRUE:FALSE);
    }
  }
}



void FF_ItemView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_itemTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_itemTree->getHeaderSize(i));
    b=m_itemTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_ItemView::toGui(const AE_ITEM_LIST *items) {
  FF_App *app;
  AE_BOOK *b;

  m_itemTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    const AE_ITEM *item;
    AE_GROUP_TREE *gt;
    int rv;
    FXFoldingItem *firstItem;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_ItemTemplateGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      m_itemTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);

    item=AE_Item_List_First(items);
    while(item) {
      m_itemTree->addEngineItem(AE_Item_dup(item));
      item=AE_Item_List_Next(item);
    }
    m_itemTree->expandItems();

    firstItem=m_itemTree->getFirstItem();
    onTreeChanged(NULL, FXSEL(SEL_CHANGED, ID_ITEM_TREE), firstItem);
  }
}



void FF_ItemView::fromGui(AE_ITEM_LIST *items) {
  FXFoldingItem *item;

  item=m_itemTree->getFirstItem();
  while (item) {
    FF_ItemTreeItem *grItem;

    grItem=dynamic_cast<FF_ItemTreeItem*>(item);
    if (grItem) {
      AE_ITEM *item=grItem->getItem();
      if (item)
	AE_Item_List_Add(AE_Item_dup(item), items);
    }
    item=item->getBelow();
  }
}



long FF_ItemView::onCmdAddItem(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ITEM *it;

    it=AE_Item_new();
    AE_Item_SetGroupId(it, 0);
    if (FF_EditItem::editItem(app,
			      this,
			      I18N("Add Item"),
                              m_tableType,
			      it)) {
      FF_ItemTreeItem *item;

      if (target)
	target->tryHandle(this, FXSEL(SEL_INSERTED, message), (void*) it);
      m_itemTree->addEngineItem(it);
      item=m_itemTree->findItem(it);
      assert(item);
      m_itemTree->makeItemVisible(item);
      m_itemTree->unselectAll();
      item->setSelected(TRUE);
      m_itemTree->setCurrentItem(item, TRUE);
      m_itemTree->sort();
   }
    else {
      AE_Item_free(it);
    }
  }

  return 1;
}



long FF_ItemView::onCmdEditItem(FXObject*, FXSelector, void*) {
  FXFoldingItem *item;

  item=m_itemTree->getCurrentItem();
  if (item) {
    FF_ItemTreeItem *ptitem;

    ptitem=dynamic_cast<FF_ItemTreeItem*>(item);
    if (ptitem) {
      AE_ITEM *item;

      item=ptitem->getItem();
      if (item) {
	FF_App *app;
	AE_BOOK *b;

	app=dynamic_cast<FF_App*>(getApp());
	b=app->getBook();
	if (b) {
	  AQDB_ID oldGroupId;

	  if (target)
	    target->tryHandle(this, FXSEL(SEL_OPENED, message), (void*) item);

	  oldGroupId=AE_Item_GetGroupId(item);

	  if (FF_EditItem::editItem(app,
				    this,
				    I18N("Edit Item"),
                                    m_tableType,
				    item)) {
	    if (target)
	      target->tryHandle(this, FXSEL(SEL_REPLACED, message), (void*) item);
	    DBG_ERROR(0, "Modified item");
	    ptitem->rebuild();
	    m_itemTree->updateItem(ptitem);
	    if (oldGroupId!=AE_Item_GetGroupId(item)) {
	      FF_GroupTreeItem *gitem=NULL;

	      /* re-group */
	      gitem=m_itemTree->findGroupItem(AE_Item_GetGroupId(item));
	      m_itemTree->updateItem(ptitem);
	      m_itemTree->moveItem(NULL, gitem, ptitem);
	      m_itemTree->sort();
	      m_itemTree->makeItemVisible(ptitem);
	      m_itemTree->selectEngineItem(item, true);
	    }
	  }
	}
      }
    }
  }
  return 1;
}



long FF_ItemView::onCmdDelItem(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_ItemView::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_itemTree->getCurrentGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    if (FF_EditGroup::editGroup(app,
				this,
				I18N("Add Item Group"),
				AE_Book_TableType_ItemTemplateGroup,
				gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_ItemTemplateGroup, gr, 1);
      if (rv<0) {
	DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_itemTree->addGroup(gr);
      item=m_itemTree->findGroupItem(gr);
      assert(item);
      m_itemTree->makeItemVisible(item);
      m_itemTree->unselectAll();
      item->setSelected(TRUE);
      m_itemTree->setCurrentItem(item, TRUE);
      app->signalDbReload(AE_Book_TableType_ItemTemplateGroup);
    }
    else
      AE_Group_free(gr);
  }

  return 1;
}



long FF_ItemView::onCmdEditGroup(FXObject*, FXSelector, void*) {
  FF_GroupTreeItem *grItem;

  grItem=m_itemTree->getCurrentGroupItem();
  if (grItem) {
    AE_GROUP *gr;

    gr=grItem->getGroup();
    if (gr) {
      FF_App *app;
      AE_BOOK *b;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
	if (FF_EditGroup::editGroup(app,
				    this,
				    I18N("Edit Item Group"),
				    AE_Book_TableType_ItemTemplateGroup,
				    gr)) {
	  int rv;
	  FXString t;

	  rv=AE_Book_WriteGroup(b, AE_Book_TableType_ItemTemplateGroup, gr, 1);
	  if (rv<0) {
	    DBG_ERROR(0, "Unable to write group (%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Unable to write object to database: %d"), rv);
	  }

	  t=FXString(AE_Group_GetName(gr));
	  grItem->setText(t);
	  m_itemTree->updateItem(grItem);
	  app->signalDbReload(AE_Book_TableType_ItemTemplateGroup);
	}
      }
    }
  }
  return 1;
}



long FF_ItemView::onCmdDelGroup(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_ItemView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editItemButton->enable();
        m_delItemButton->enable();

	m_editGroupButton->disable();
        m_delGroupButton->disable();
      }
      else {
        m_editItemButton->disable();
        m_delItemButton->disable();

        m_editGroupButton->enable();
        m_delGroupButton->enable();
      }
    }
    else {
      m_editItemButton->disable();
      m_delItemButton->disable();

      m_editGroupButton->disable();
      m_delGroupButton->disable();
    }
  }
  return 1;
}



void FF_ItemView::clear() {
  m_itemTree->clearItems();
}



void FF_ItemView::sortItems() {
  m_itemTree->sort();
}




