/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_ITEMVIEW_HPP
#define FF_ITEMVIEW_HPP


#include "ff_itemtree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>

#include <list>


class FF_ItemTree;

/**
 * Emits signal SEL_INSERTED when a new item is added and SEL_REPLACED when an existing
 * item was edited. Also, just before using a item from the list for editing
 * the signal SEL_OPENED is emitted to allow for the target to fill the to-edit item
 * with data from the database (->on-demand loading)
 */
class FF_ItemView: public FXHorizontalFrame {
  FXDECLARE(FF_ItemView)

public:
  enum {
    ID_ITEM_TREE=FXPacker::ID_LAST,
    ID_BUTTON_ADDTMPL,
    ID_BUTTON_EDITTMPL,
    ID_BUTTON_DELTMPL,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_LAST
  };

public:
  FF_ItemView(AE_BOOK_TABLE_TYPE tt,
	      FXComposite *p,
	      FXuint opts=0,
	      FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FF_ItemView();

  void enter();
  void leave();

  void create();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void toGui(const AE_ITEM_LIST *items);
  void fromGui(AE_ITEM_LIST *items);

  void clear();
  void sortItems();

  long onCmdAddItem(FXObject*, FXSelector, void*);
  long onCmdEditItem(FXObject*, FXSelector, void*);
  long onCmdDelItem(FXObject*, FXSelector, void*);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);

  FF_ItemTree *getItemTree() const { return m_itemTree;};

protected:
  FF_ItemView();

  AE_BOOK_TABLE_TYPE m_tableType;

  FF_ItemTree *m_itemTree;

  FXButton *m_addGroupButton;
  FXButton *m_editGroupButton;
  FXButton *m_delGroupButton;

  FXButton *m_addItemButton;
  FXButton *m_editItemButton;
  FXButton *m_delItemButton;
};




#endif

