/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_partnerview.hpp"
#include "ff_partnertree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_editpartner.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_PartnerView) FF_PartnerViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_PartnerView::ID_BUTTON_ADDPARTNER,
            FF_PartnerView::onCmdAddPartner),
  FXMAPFUNC(SEL_COMMAND, FF_PartnerView::ID_BUTTON_EDITPARTNER,
            FF_PartnerView::onCmdEditPartner),
  FXMAPFUNC(SEL_COMMAND, FF_PartnerView::ID_BUTTON_DELPARTNER,
            FF_PartnerView::onCmdDelPartner),

  FXMAPFUNC(SEL_COMMAND, FF_PartnerView::ID_BUTTON_ADDGROUP,
            FF_PartnerView::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FF_PartnerView::ID_BUTTON_EDITGROUP,
            FF_PartnerView::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FF_PartnerView::ID_BUTTON_DELGROUP,
            FF_PartnerView::onCmdDelGroup),

  FXMAPFUNC(SEL_CHANGED, FF_PartnerView::ID_PARTNER_TREE,
            FF_PartnerView::onTreeChanged),

};



FXIMPLEMENT(FF_PartnerView, FXHorizontalFrame,
            FF_PartnerViewMap, ARRAYNUMBER(FF_PartnerViewMap))






FF_PartnerView::FF_PartnerView(FXComposite *p,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_partnerTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addPartnerButton(NULL)
,m_editPartnerButton(NULL)
,m_delPartnerButton(NULL)
,m_isEditing(false)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_partnerTree=new FF_PartnerTree(this, this, ID_PARTNER_TREE,
				   LAYOUT_SIDE_LEFT |
				   LAYOUT_FILL_X | LAYOUT_FILL_Y |
				   FOLDINGLIST_EXTENDEDSELECT |
				   FOLDINGLIST_SHOWS_LINES |
				   FOLDINGLIST_SHOWS_BOXES |
				   FOLDINGLIST_ROOT_BOXES |
				   FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_partnerTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_partnerTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_addGroupButton=new THEMEBUTTON(vf,
				   I18N("Add Group"),
				   NULL,
				   this, ID_BUTTON_ADDGROUP,
				   BUTTON_NORMAL);
  m_editGroupButton=new THEMEBUTTON(vf,
				    I18N("Edit Group"),
				    NULL,
				    this, ID_BUTTON_EDITGROUP,
				    BUTTON_NORMAL);
  m_delGroupButton=new THEMEBUTTON(vf,
				   I18N("Delete Group"),
				   NULL,
				   this, ID_BUTTON_DELGROUP,
				   BUTTON_NORMAL);

  new FXLabel(vf, "");
  new FXHorizontalSeparator(vf, SEPARATOR_GROOVE|LAYOUT_FILL_X);
  new FXLabel(vf, "");

  m_addPartnerButton=new THEMEBUTTON(vf,
				     I18N("Add Partner"),
				     NULL,
				     this, ID_BUTTON_ADDPARTNER,
				     BUTTON_NORMAL);
  m_editPartnerButton=new THEMEBUTTON(vf,
				      I18N("Edit Partner"),
				      NULL,
				      this, ID_BUTTON_EDITPARTNER,
				      BUTTON_NORMAL);
  m_delPartnerButton=new THEMEBUTTON(vf,
				     I18N("Delete Partner"),
				     NULL,
				     this, ID_BUTTON_DELPARTNER,
				     BUTTON_NORMAL);
}



FF_PartnerView::FF_PartnerView()
:FXHorizontalFrame()
,m_partnerTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addPartnerButton(NULL)
,m_editPartnerButton(NULL)
,m_delPartnerButton(NULL)
{
}



FF_PartnerView::~FF_PartnerView() {
}



void FF_PartnerView::create(){
  FXHorizontalFrame::create();
  m_partnerTree->create();

  m_addPartnerButton->create();
  m_editPartnerButton->create();
  m_delPartnerButton->create();
}



void FF_PartnerView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_partnerTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_partnerTree->getHeaderSize(i));
    m_partnerTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      m_partnerTree->setHeaderArrowDir(i, sortUp?TRUE:FALSE);
    }
  }
}



void FF_PartnerView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_partnerTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_partnerTree->getHeaderSize(i));
    b=m_partnerTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_PartnerView::toGui(const AE_PARTNER_LIST *partners) {
  FF_App *app;
  AE_BOOK *b;

  m_partnerTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    const AE_PARTNER *pt;
    AE_GROUP_TREE *gt;
    int rv;
    FXFoldingItem *firstItem;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_PartnerGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      m_partnerTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);

    pt=AE_Partner_List_First(partners);
    while(pt) {
      m_partnerTree->addPartner(AE_Partner_dup(pt));
      pt=AE_Partner_List_Next(pt);
    }
    m_partnerTree->expandItems();

    firstItem=m_partnerTree->getFirstItem();
    onTreeChanged(NULL, FXSEL(SEL_CHANGED, ID_PARTNER_TREE), firstItem);
  }
}



void FF_PartnerView::fromGui(AE_PARTNER_LIST *partners) {
  FXFoldingItem *item;

  item=m_partnerTree->getFirstItem();
  while (item) {
    FF_PartnerTreeItem *grItem;

    grItem=dynamic_cast<FF_PartnerTreeItem*>(item);
    if (grItem) {
      AE_PARTNER *pt=grItem->getPartner();
      if (pt)
	AE_Partner_List_Add(AE_Partner_dup(pt), partners);
    }
    item=item->getBelow();
  }
}



long FF_PartnerView::onCmdAddPartner(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_PARTNER *pt;
    AE_GROUP *parent;

    pt=AE_Partner_new();
    parent=m_partnerTree->getCurrentGroup();
    if (parent)
      AE_Partner_SetGroupId(pt, AE_Group_GetId(parent));
    else
      AE_Partner_SetGroupId(pt, 0);

    m_isEditing=true;
    if (FF_EditPartner::editPartner(app, this,
				    I18N("Add Partner"), pt)) {
      FF_PartnerTreeItem *item;

      if (target)
	target->tryHandle(this, FXSEL(SEL_INSERTED, message), (void*) pt);
      m_partnerTree->addPartner(pt);
      item=m_partnerTree->findPartnerItem(pt);
      assert(item);
      m_partnerTree->makeItemVisible(item);
      m_partnerTree->unselectAll();
      item->setSelected(TRUE);
      m_partnerTree->setCurrentItem(item, TRUE);
      m_partnerTree->sort();
      m_isEditing=false;
      app->signalDbReload(AE_Book_TableType_Partner);
    }
    else {
      AE_Partner_free(pt);
    }
    m_isEditing=false;
  }

  return 1;
}



long FF_PartnerView::onCmdEditPartner(FXObject*, FXSelector, void*) {
  FXFoldingItem *item;

  item=m_partnerTree->getCurrentItem();
  if (item) {
    FF_PartnerTreeItem *ptitem;

    ptitem=dynamic_cast<FF_PartnerTreeItem*>(item);
    if (ptitem) {
      AE_PARTNER *pt;

      pt=ptitem->getPartner();
      if (pt) {
	FF_App *app;
	AE_BOOK *b;
 
	app=dynamic_cast<FF_App*>(getApp());
	b=app->getBook();
	if (b) {
	  AQDB_ID oldGroupId;

	  if (target)
	    target->tryHandle(this, FXSEL(SEL_OPENED, message), (void*) pt);

	  oldGroupId=AE_Partner_GetGroupId(pt);

	  m_isEditing=true;
	  if (FF_EditPartner::editPartner(app, this, I18N("Edit Partner"), pt)) {
	    if (target)
	      target->tryHandle(this, FXSEL(SEL_REPLACED, message), (void*) pt);
	    DBG_ERROR(0, "Modified partner");
	    ptitem->rebuild();
	    m_partnerTree->updateItem(ptitem);
	    if (oldGroupId!=AE_Partner_GetGroupId(pt)) {
	      FF_GroupTreeItem *gitem=NULL;

	      /* re-group */
	      gitem=m_partnerTree->findGroupItem(AE_Partner_GetGroupId(pt));
	      m_partnerTree->updateItem(ptitem);
	      m_partnerTree->moveItem(NULL, gitem, ptitem);
	      m_partnerTree->sort();
	      m_partnerTree->makeItemVisible(ptitem);
	      m_partnerTree->selectPartner(pt, true);
	    }
	    m_isEditing=false;
	    app->signalDbReload(AE_Book_TableType_Partner);
	  }
	  m_isEditing=false;
	}
      }
    }
  }
  return 1;
}



long FF_PartnerView::onCmdDelPartner(FXObject*, FXSelector, void*) {
  FXFoldingItem *item;

  item=m_partnerTree->getCurrentItem();
  if (item) {
    FF_PartnerTreeItem *ptitem;

    ptitem=dynamic_cast<FF_PartnerTreeItem*>(item);
    if (ptitem) {
      AE_PARTNER *pt;

      pt=ptitem->getPartner();
      if (pt) {
	FF_App *app;
	AE_BOOK *b;
 
	app=dynamic_cast<FF_App*>(getApp());
	b=app->getBook();
	if (b) {
          FXuint res;

          res=FXMessageBox::question(this, MBOX_YES_NO,
                                     I18N("Delete Partner"),
                                     "%s",
                                     I18N("Are you sure you want to delete the selected "
                                          "partner?"));
          if (res!=MBOX_CLICKED_YES) {
            DBG_ERROR(0, "Aborted");
          }
          else {
            app->deletePartner(AE_Partner_GetId(pt));
            app->signalDbReload(AE_Book_TableType_Partner);
            app->signalDbReload(AE_Book_TableType_Task);
            app->signalDbReload(AE_Book_TableType_Account);
          }
        }
      }
    }
  }
  return 1;
}



long FF_PartnerView::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_partnerTree->getCurrentGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    m_isEditing=true;
    if (FF_EditGroup::editGroup(app,
				this,
				I18N("Add Partner Group"),
				AE_Book_TableType_PartnerGroup,
				gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_PartnerGroup, gr, 1);
      if (rv<0) {
	DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_partnerTree->addGroup(gr);
      item=m_partnerTree->findGroupItem(gr);
      assert(item);
      m_partnerTree->makeItemVisible(item);
      m_partnerTree->unselectAll();
      item->setSelected(TRUE);
      m_partnerTree->setCurrentItem(item, TRUE);
      m_isEditing=false;
      app->signalDbReload(AE_Book_TableType_PartnerGroup);
    }
    else
      AE_Group_free(gr);
    m_isEditing=false;
  }

  return 1;
}



long FF_PartnerView::onCmdEditGroup(FXObject*, FXSelector, void*) {
  FF_GroupTreeItem *grItem;

  grItem=m_partnerTree->getCurrentGroupItem();
  if (grItem) {
    AE_GROUP *gr;

    gr=grItem->getGroup();
    if (gr) {
      FF_App *app;
      AE_BOOK *b;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
	m_isEditing=true;
	if (FF_EditGroup::editGroup(app,
				    this,
				    I18N("Edit Partner Group"),
				    AE_Book_TableType_PartnerGroup,
				    gr)) {
	  int rv;
	  FXString t;

	  rv=AE_Book_WriteGroup(b, AE_Book_TableType_PartnerGroup, gr, 1);
	  if (rv<0) {
	    DBG_ERROR(0, "Unable to edit group (%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Unable to write object to database: %d"), rv);
	  }

	  t=FXString(AE_Group_GetName(gr));
	  grItem->setText(t);
	  m_partnerTree->updateItem(grItem);
	  m_isEditing=false;
	  app->signalDbReload(AE_Book_TableType_PartnerGroup);
	}
	m_isEditing=false;
      }
    }
  }
  return 1;
}



long FF_PartnerView::onCmdDelGroup(FXObject*, FXSelector, void*) {
  FF_GroupTreeItem *grItem;

  grItem=m_partnerTree->getCurrentGroupItem();
  if (grItem) {
    AE_GROUP *gr;

    gr=grItem->getGroup();
    if (gr) {
      FF_App *app;
      AE_BOOK *b;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
        FXuint res;

        res=FXMessageBox::question(this, MBOX_YES_NO,
                                   I18N("Delete Group"),
                                   "%s",
                                   I18N("Are you sure you want to delete the selected "
                                        "group?"));
        if (res!=MBOX_CLICKED_YES) {
          DBG_ERROR(0, "Aborted");
        }
        else {
          app->deletePartnerGroup(AE_Group_GetId(gr));
          app->signalDbReload(AE_Book_TableType_PartnerGroup);
          app->signalDbReload(AE_Book_TableType_Partner);
          app->signalDbReload(AE_Book_TableType_Task);
          app->signalDbReload(AE_Book_TableType_Account);
        }
      }
    }
  }
  return 1;
}



long FF_PartnerView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editPartnerButton->enable();
        m_delPartnerButton->enable();

	m_editGroupButton->disable();
        m_delGroupButton->disable();
      }
      else {
        m_editPartnerButton->disable();
        m_delPartnerButton->disable();

        m_editGroupButton->enable();
        m_delGroupButton->enable();
      }
    }
    else {
      m_editPartnerButton->disable();
      m_delPartnerButton->disable();

      m_editGroupButton->disable();
      m_delGroupButton->disable();
    }
  }
  return 1;
}



void FF_PartnerView::clear() {
  m_partnerTree->clearItems();
}



void FF_PartnerView::sortPartners() {
  m_partnerTree->sort();
}




