/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_PARTNERVIEW_HPP
#define FF_PARTNERVIEW_HPP


#include "ff_partnertree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>

#include <list>


class FF_PartnerTree;

/**
 * Emits signal SEL_INSERTED when a new partner is added and SEL_REPLACED when an existing
 * partner was edited. Also, just before using a partner from the list for editing
 * the signal SEL_OPENED is emitted to allow for the target to fill the to-edit partner
 * with data from the database (->on-demand loading)
 */
class FF_PartnerView: public FXHorizontalFrame {
  FXDECLARE(FF_PartnerView)

public:
  enum {
    ID_PARTNER_TREE=FXPacker::ID_LAST,
    ID_BUTTON_ADDPARTNER,
    ID_BUTTON_EDITPARTNER,
    ID_BUTTON_DELPARTNER,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_LAST
  };

public:
  FF_PartnerView(FXComposite *p,
		 FXuint opts=0,
		 FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FF_PartnerView();

  void enter();
  void leave();

  void create();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void toGui(const AE_PARTNER_LIST *partners);
  void fromGui(AE_PARTNER_LIST *partners);

  bool isEditing() const { return m_isEditing;};

  void clear();
  void sortPartners();

  long onCmdAddPartner(FXObject*, FXSelector, void*);
  long onCmdEditPartner(FXObject*, FXSelector, void*);
  long onCmdDelPartner(FXObject*, FXSelector, void*);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);

  void dump(int indent);

  FF_PartnerTree *getPartnerTree() const { return m_partnerTree;};

protected:
  FF_PartnerView();

  FF_PartnerTree *m_partnerTree;

  FXButton *m_addGroupButton;
  FXButton *m_editGroupButton;
  FXButton *m_delGroupButton;

  FXButton *m_addPartnerButton;
  FXButton *m_editPartnerButton;
  FXButton *m_delPartnerButton;

  AE_BOOK_TABLE_TYPE m_type;
  bool m_isEditing;
};




#endif

