/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_ruleview.hpp"
#include "ff_ruletree.hpp"
#include "ff_editstmrule.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_RuleView) FF_RuleViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_RuleView::ID_BUTTON_ADD, FF_RuleView::onCmdRuleAdd),
  FXMAPFUNC(SEL_COMMAND, FF_RuleView::ID_BUTTON_EDIT, FF_RuleView::onCmdRuleEdit),
  FXMAPFUNC(SEL_COMMAND, FF_RuleView::ID_BUTTON_DEL, FF_RuleView::onCmdRuleDel),
  FXMAPFUNC(SEL_COMMAND, FF_RuleView::ID_BUTTON_TEST, FF_RuleView::onCmdRuleTest),
};


FXIMPLEMENT(FF_RuleView, FXHorizontalFrame, FF_RuleViewMap, ARRAYNUMBER(FF_RuleViewMap))




FF_RuleView::FF_RuleView(FXComposite *p,
                         FXObject* tgt,
                         FXSelector sel,
                         FXuint opts, FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts)
,m_ruleTree(NULL)
,m_addRuleButton(NULL)
,m_editRuleButton(NULL)
,m_delRuleButton(NULL)
,m_testRuleButton(NULL)
,m_ownerType(AE_Book_TableType_Unknown)
,m_ownerId(0) {
  FXVerticalFrame *vf;

  target=tgt;
  message=sel;

  m_ruleTree=new FF_RuleTree(this, this, ID_RULE_TREE,
			     LAYOUT_FILL_X | LAYOUT_FILL_Y |
			     FOLDINGLIST_SINGLESELECT |
			     FOLDINGLIST_SHOWS_BOXES);
  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_addRuleButton=new THEMEBUTTON(vf, I18N("Add Rule"), NULL, this, ID_BUTTON_ADD, BUTTON_NORMAL);
  m_editRuleButton=new THEMEBUTTON(vf, I18N("Edit Rule"), NULL, this, ID_BUTTON_EDIT, BUTTON_NORMAL);
  m_delRuleButton=new THEMEBUTTON(vf, I18N("Delete Rule"), NULL, this, ID_BUTTON_DEL, BUTTON_NORMAL);
  m_testRuleButton=new THEMEBUTTON(vf, I18N("Test Rules"), NULL, this, ID_BUTTON_TEST, BUTTON_NORMAL);
}



FF_RuleView::FF_RuleView()
:FXHorizontalFrame()
,m_ruleTree(NULL)
,m_addRuleButton(NULL)
,m_editRuleButton(NULL)
,m_delRuleButton(NULL)
,m_ownerType(AE_Book_TableType_Unknown)
,m_ownerId(0) {

}



FF_RuleView::~FF_RuleView() {
}



void FF_RuleView::init(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_ruleTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_ruleTree->getHeaderSize(i));
    m_ruleTree->setHeaderSize(i, j);
  }

}



void FF_RuleView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_ruleTree->getNumHeaders(); i++)
    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_ruleTree->getHeaderSize(i));
}



void FF_RuleView::toGui(const AE_RULE_LIST *rules) {
  m_ruleTree->clearItems();
  m_ruleTree->addRules(rules);
}



void FF_RuleView::fromGui(AE_RULE_LIST *rules) {
  FXFoldingItem *item;

  item=m_ruleTree->getFirstItem();
  while (item) {
    FF_RuleTreeItem *grItem;

    grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      AE_RULE *r=grItem->getRule();
      if (r)
	AE_Rule_List_Add(AE_Rule_dup(r), rules);
    }
    item=item->getBelow();
  }
}



void FF_RuleView::create() {
  FXHorizontalFrame::create();
}



long FF_RuleView::onCmdRuleAdd(FXObject*, FXSelector, void* ptr) {
  FF_App *app=dynamic_cast<FF_App*>(getApp());
  AE_RULE *r;

  r=AE_Rule_new();
  AE_Rule_SetOwnerType(r, m_ownerType);
  AE_Rule_SetOwnerId(r, m_ownerId);
  AE_Rule_SetTypeId(r, AE_Book_TableType_BankStatement);
  if (FF_EditStatementRule::editRule(app, this, I18N("Create A Matcher Rule"), r)) {
    if (target)
      target->tryHandle(this, FXSEL(SEL_INSERTED, message), (void*) r);
    m_ruleTree->addRule(r);
  }
  AE_Rule_free(r);
  return 1;
}



long FF_RuleView::onCmdRuleEdit(FXObject*, FXSelector, void* ptr) {
  FF_App *app=dynamic_cast<FF_App*>(getApp());
  FXFoldingItem *item;

  item=m_ruleTree->getCurrentItem();

  while (item) {
    FF_RuleTreeItem *grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      AE_RULE *r=grItem->getRule();
      if (r) {
	if (target)
	  target->tryHandle(this, FXSEL(SEL_OPENED, message), (void*) r);

	if (FF_EditStatementRule::editRule(app, this, I18N("Edit Matcher Rule"), r)) {
	  AE_Rule_AddRuntimeFlags(r, AE_RULE_RTFLAGS_MODIFIED);
	  if (target)
	    target->tryHandle(this, FXSEL(SEL_REPLACED, message), (void*) r);
	  /* update item */
	  grItem->updateRule();
#if 1
          if (1) {
            GWEN_BUFFER *tbuf;
            int rv;

            tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            rv=AE_Rule_toQueryString(r, tbuf);
            if (rv<0) {
              DBG_ERROR(0, "here (%d)", rv);
            }
            DBG_ERROR(0, "QueryString: [%s]", GWEN_Buffer_GetStart(tbuf));
            GWEN_Buffer_free(tbuf);
          }
#endif
        }
      }
      break;
    }
    item=item->getParent();
  }

  return 1;
}



long FF_RuleView::onCmdRuleDel(FXObject*, FXSelector, void* ptr) {
  FXFoldingItem *item;

  item=m_ruleTree->getCurrentItem();

  while (item) {
    FF_RuleTreeItem *grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      AE_RULE *r=grItem->getRule();
      if (r) {
        FXuint res;

        res=FXMessageBox::question(this, MBOX_YES_NO,
				   I18N("Delete Rule"),
                                   "%s",
                                   I18N("Are you sure you want to delete the current "
					"rule?"));
	if (res==MBOX_CLICKED_YES) {
	  /* remove item */
	  m_ruleTree->removeItem(item, false);
	}
      }
      break;
    }
    item=item->getParent();
  }

  return 1;
}



long FF_RuleView::onCmdRuleTest(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Test");
  return 1;
}







