/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_TAXVIEW_HPP
#define FF_TAXVIEW_HPP


#include "ff_taxtree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>

#include <list>


class FF_TaxTree;

/**
 * Emits signal SEL_INSERTED when a new tax is added and SEL_REPLACED when an existing
 * tax was edited. Also, just before using a tax from the list for editing
 * the signal SEL_OPENED is emitted to allow for the target to fill the to-edit tax
 * with data from the database (->on-demand loading)
 */
class FF_TaxView: public FXHorizontalFrame {
  FXDECLARE(FF_TaxView)

public:
  enum {
    ID_TAX_TREE=FXPacker::ID_LAST,
    ID_BUTTON_ADDTAX,
    ID_BUTTON_EDITTAX,
    ID_BUTTON_DELTAX,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_LAST
  };

public:
  FF_TaxView(FXComposite *p,
	     FXuint opts=0,
	     FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FF_TaxView();

  void enter();
  void leave();

  void create();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void toGui(const AE_TAX_LIST *taxes);
  void fromGui(AE_TAX_LIST *taxes);

  void clear();
  void sortTaxes();

  long onCmdAddTax(FXObject*, FXSelector, void*);
  long onCmdEditTax(FXObject*, FXSelector, void*);
  long onCmdDelTax(FXObject*, FXSelector, void*);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);

  FF_TaxTree *getTaxTree() const { return m_taxTree;};

protected:
  FF_TaxView();

  FF_TaxTree *m_taxTree;

  FXButton *m_addGroupButton;
  FXButton *m_editGroupButton;
  FXButton *m_delGroupButton;

  FXButton *m_addTaxButton;
  FXButton *m_editTaxButton;
  FXButton *m_delTaxButton;

  AE_BOOK_TABLE_TYPE m_type;
};




#endif

