/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_transferview.hpp"
#include "ff_app.hpp"
#include "ff_transfertree.hpp"
#include "ff_edittransfergroup.hpp"
#include "ff_editxfer.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_TransferView) FF_TransferViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_TransferView::ID_BUTTON_ADDSTATEMENT,
            FF_TransferView::onCmdAddTransfer),
  FXMAPFUNC(SEL_COMMAND, FF_TransferView::ID_BUTTON_EDITSTATEMENT,
            FF_TransferView::onCmdEditTransfer),
  FXMAPFUNC(SEL_COMMAND, FF_TransferView::ID_BUTTON_DELSTATEMENT,
            FF_TransferView::onCmdDelTransfer),

  FXMAPFUNC(SEL_COMMAND, FF_TransferView::ID_BUTTON_ADDGROUP,
            FF_TransferView::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FF_TransferView::ID_BUTTON_EDITGROUP,
            FF_TransferView::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FF_TransferView::ID_BUTTON_DELGROUP,
            FF_TransferView::onCmdDelGroup),

  FXMAPFUNC(SEL_CHANGED, FF_TransferView::ID_STATEMENT_TREE,
            FF_TransferView::onTreeChanged),

};



FXIMPLEMENT(FF_TransferView, FXHorizontalFrame,
            FF_TransferViewMap, ARRAYNUMBER(FF_TransferViewMap))






FF_TransferView::FF_TransferView(FXComposite *p,
                                 AE_BOOK_TABLE_TYPE tt,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_transferTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addTransferButton(NULL)
,m_editTransferButton(NULL)
,m_delTransferButton(NULL)
,m_type(tt)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_transferTree=new FF_TransferTree(this, this, ID_STATEMENT_TREE,
                                     LAYOUT_SIDE_LEFT |
                                     LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                     FOLDINGLIST_EXTENDEDSELECT |
                                     FOLDINGLIST_SHOWS_LINES |
                                     FOLDINGLIST_SHOWS_BOXES |
                                     FOLDINGLIST_ROOT_BOXES |
                                     FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_transferTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_transferTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_addGroupButton=new THEMEBUTTON(vf,
				   I18N("Add Group"),
				   NULL,
				   this, ID_BUTTON_ADDGROUP,
				   BUTTON_NORMAL);
  m_editGroupButton=new THEMEBUTTON(vf,
				    I18N("Edit Group"),
				    NULL,
				    this, ID_BUTTON_EDITGROUP,
				    BUTTON_NORMAL);
  m_delGroupButton=new THEMEBUTTON(vf,
				   I18N("Delete Group"),
				   NULL,
				   this, ID_BUTTON_DELGROUP,
				   BUTTON_NORMAL);

  new FXLabel(vf, "");
  new FXHorizontalSeparator(vf, SEPARATOR_GROOVE|LAYOUT_FILL_X);
  new FXLabel(vf, "");

  m_addTransferButton=new THEMEBUTTON(vf,
                                      I18N("Add Transfer"),
                                      NULL,
                                      this, ID_BUTTON_ADDSTATEMENT,
                                      BUTTON_NORMAL);
  m_editTransferButton=new THEMEBUTTON(vf,
                                       I18N("Edit Transfer"),
                                       NULL,
                                       this, ID_BUTTON_EDITSTATEMENT,
                                       BUTTON_NORMAL);
  m_delTransferButton=new THEMEBUTTON(vf,
                                      I18N("Delete Transfer"),
                                      NULL,
                                      this, ID_BUTTON_DELSTATEMENT,
                                      BUTTON_NORMAL);
}



FF_TransferView::FF_TransferView()
:FXHorizontalFrame()
,m_transferTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addTransferButton(NULL)
,m_editTransferButton(NULL)
,m_delTransferButton(NULL)
{
}



FF_TransferView::~FF_TransferView() {
}



void FF_TransferView::create(){
  FXHorizontalFrame::create();
  m_transferTree->create();

  m_addTransferButton->create();
  m_editTransferButton->create();
  m_delTransferButton->create();
}



void FF_TransferView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_transferTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_transferTree->getHeaderSize(i));
    m_transferTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      m_transferTree->setHeaderArrowDir(i, sortUp?TRUE:FALSE);
    }
  }
}



void FF_TransferView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_transferTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_transferTree->getHeaderSize(i));
    b=m_transferTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_TransferView::toGui(const AE_STATEMENT_LIST *transfers) {
  FF_App *app;
  AE_BOOK *b;

  m_transferTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    const AE_STATEMENT *st;
    AE_GROUP_TREE *gt;
    int rv;
    FXFoldingItem *firstItem;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_BankTransferGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      m_transferTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);

    if (m_type==AE_Book_TableType_BankTransferPlanned) {
      GWEN_DATE *now;

      now=GWEN_Date_CurrentDate();
      /* add transfers */
      st=AE_Statement_List_First(transfers);
      while (st) {
        const GWEN_DATE *ti;
        int col=FF_TransferTree::USECOLOR_NORMAL;

        ti=AE_Statement_GetDate(st);
        if (ti) {
          int d=GWEN_Date_Diff(ti, now);

          if (d<3)
            col=FF_TransferTree::USECOLOR_WARNING;
          else if (d<7)
            col=FF_TransferTree::USECOLOR_NOTICE;
        }
        m_transferTree->addTransfer(AE_Statement_dup(st), false, col);
        st=AE_Statement_List_Next(st);
      }
      GWEN_Date_free(now);
    }
    else {
      bool useDirect;

      useDirect=(m_type==AE_Book_TableType_BankTransferTemplate);

      /* add transfers */
      st=AE_Statement_List_First(transfers);
      while (st) {
        m_transferTree->addTransfer(AE_Statement_dup(st), useDirect, FF_TransferTree::USECOLOR_NORMAL);
        st=AE_Statement_List_Next(st);
      }
    }

    m_transferTree->sort();
    m_transferTree->expandItems();

    firstItem=m_transferTree->getFirstItem();
    onTreeChanged(NULL, FXSEL(SEL_CHANGED, ID_STATEMENT_TREE), firstItem);
  }
}



void FF_TransferView::fromGui(AE_STATEMENT_LIST *transfers) {
  FXFoldingItem *item;

  item=m_transferTree->getFirstItem();
  while (item) {
    FF_TransferTreeItem *grItem;

    grItem=dynamic_cast<FF_TransferTreeItem*>(item);
    if (grItem) {
      AE_STATEMENT *transfer=grItem->getTransfer();
      if (transfer)
	AE_Statement_List_Add(AE_Statement_dup(transfer), transfers);
    }
    item=item->getBelow();
  }
}



long FF_TransferView::onCmdAddTransfer(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_STATEMENT *transfer;
    int editMode;

    switch(m_type) {
    case AE_Book_TableType_BankTransferPlanned:
      editMode=FF_EditTransfer::EDITMODE_EASY;
      break;
    case AE_Book_TableType_BankTransferTemplate:
      editMode=FF_EditTransfer::EDITMODE_TEMPLATE;
      break;
    case AE_Book_TableType_BankTransfer:
    default:
      editMode=FF_EditTransfer::EDITMODE_STRICT;
      break;
    }

    transfer=AE_Statement_new();
    gr=m_transferTree->getCurrentGroup();
    if (gr)
      AE_Statement_SetGroupId(transfer, AE_Group_GetId(gr));
    else
      AE_Statement_SetGroupId(transfer, 0);
    if (FF_EditTransfer::editTransfer(app, editMode, this, I18N("Add Transfer"), transfer)) {
      FF_TransferTreeItem *item;

      if (target)
	target->tryHandle(this, FXSEL(SEL_INSERTED, message), (void*) transfer);
      m_transferTree->addTransfer(transfer, false, FF_TransferTree::USECOLOR_NORMAL);
      item=m_transferTree->findTransferItem(transfer);
      assert(item);
      m_transferTree->makeItemVisible(item);
      m_transferTree->unselectAll();
      item->setSelected(TRUE);
      m_transferTree->setCurrentItem(item, TRUE);
      m_transferTree->sort();
   }
    else {
      AE_Statement_free(transfer);
    }
  }

  return 1;
}



long FF_TransferView::onCmdEditTransfer(FXObject*, FXSelector, void*) {
  FXFoldingItem *item;

  item=m_transferTree->getCurrentItem();
  if (item) {
    FF_TransferTreeItem *ptitem;

    ptitem=dynamic_cast<FF_TransferTreeItem*>(item);
    if (ptitem) {
      AE_STATEMENT *transfer;

      transfer=ptitem->getTransfer();
      if (transfer) {
	FF_App *app;
	AE_BOOK *b;

	app=dynamic_cast<FF_App*>(getApp());
	b=app->getBook();
	if (b) {
	  AQDB_ID oldGroupId;
	  int editMode;

	  if (target)
	    target->tryHandle(this, FXSEL(SEL_OPENED, message), (void*) transfer);

	  oldGroupId=AE_Statement_GetGroupId(transfer);

	  switch(m_type) {
	  case AE_Book_TableType_BankTransferPlanned:
	    editMode=FF_EditTransfer::EDITMODE_EASY;
	    break;
	  case AE_Book_TableType_BankTransferTemplate:
	    editMode=FF_EditTransfer::EDITMODE_TEMPLATE;
	    break;
	  case AE_Book_TableType_BankTransfer:
	  default:
	    editMode=FF_EditTransfer::EDITMODE_STRICT;
	    break;
	  }

	  if (FF_EditTransfer::editTransfer(app, editMode, this, I18N("Edit Transfer"), transfer)) {
	    if (target)
	      target->tryHandle(this, FXSEL(SEL_REPLACED, message), (void*) transfer);
	    DBG_ERROR(0, "Modified transfer");
	    m_transferTree->updateItem(ptitem);
	    if (oldGroupId!=AE_Statement_GetGroupId(transfer)) {
	      FF_GroupTreeItem *gitem=NULL;

	      /* re-group */
	      gitem=m_transferTree->findGroupItem(AE_Statement_GetGroupId(transfer));
	      m_transferTree->updateItem(ptitem);
	      m_transferTree->moveItem(NULL, gitem, ptitem);
	      m_transferTree->sort();
	      m_transferTree->makeItemVisible(ptitem);
	      m_transferTree->selectTransfer(transfer, true);
	    }
	  }
	}
      }
    }
  }
  return 1;
}



long FF_TransferView::onCmdDelTransfer(FXObject*, FXSelector, void*) {
  FXFoldingItem *item;

  item=m_transferTree->getCurrentItem();
  if (item) {
    FF_TransferTreeItem *ptitem;

    ptitem=dynamic_cast<FF_TransferTreeItem*>(item);
    if (ptitem) {
      AE_STATEMENT *transfer;

      transfer=ptitem->getTransfer();
      if (transfer) {
	FF_App *app;
	AE_BOOK *b;

	app=dynamic_cast<FF_App*>(getApp());
	b=app->getBook();
	if (b) {
          FXuint res;

          res=FXMessageBox::question(this, MBOX_YES_NO,
                                     I18N("Delete Transfer"),
                                     "%s",
                                     I18N("Are you sure you want to delete the selected "
                                          "transfer?"));
          if (res!=MBOX_CLICKED_YES) {
            DBG_ERROR(0, "Aborted");
          }
          else {
            /* attach to transfer because "removeItem" will free it */
            AE_Statement_Attach(transfer);
            m_transferTree->removeItem(item, FALSE);

            if (target)
              target->tryHandle(this, FXSEL(SEL_DELETED, message), (void*) transfer);
            AE_Statement_free(transfer);
          }
        }
      }
    }
  }
  return 1;
}



long FF_TransferView::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_transferTree->getCurrentGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    if (FF_EditTransferGroup::editGroup(app,
                                        this,
                                        I18N("Add Transfer Group"),
                                        gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_BankTransferGroup, gr, 1);
      if (rv<0) {
	DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_transferTree->addGroup(gr);
      item=m_transferTree->findGroupItem(gr);
      assert(item);
      m_transferTree->makeItemVisible(item);
      m_transferTree->unselectAll();
      item->setSelected(TRUE);
      m_transferTree->setCurrentItem(item, TRUE);
      app->signalDbReload(AE_Book_TableType_BankTransferGroup);
    }
    else
      AE_Group_free(gr);
  }

  return 1;
}



long FF_TransferView::onCmdEditGroup(FXObject*, FXSelector, void*) {
  FF_GroupTreeItem *grItem;

  grItem=m_transferTree->getCurrentGroupItem();
  if (grItem) {
    AE_GROUP *gr;

    gr=grItem->getGroup();
    if (gr) {
      FF_App *app;
      AE_BOOK *b;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
	if (FF_EditTransferGroup::editGroup(app,
                                            this,
                                            I18N("Edit Transfer Group"),
                                            gr)) {
	  int rv;
	  FXString t;

	  rv=AE_Book_WriteGroup(b, AE_Book_TableType_BankTransferGroup, gr, 1);
	  if (rv<0) {
	    DBG_ERROR(0, "Unable to write group (%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Unable to write object to database: %d"), rv);
	  }

	  t=FXString(AE_Group_GetName(gr));
	  grItem->setText(t);
	  m_transferTree->updateItem(grItem);
	  app->signalDbReload(AE_Book_TableType_BankTransferGroup);
	}
      }
    }
  }
  return 1;
}



long FF_TransferView::onCmdDelGroup(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_TransferView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editTransferButton->enable();
        m_delTransferButton->enable();

	m_editGroupButton->disable();
        m_delGroupButton->disable();
      }
      else {
        m_editTransferButton->disable();
        m_delTransferButton->disable();

        m_editGroupButton->enable();
        m_delGroupButton->enable();
      }
    }
    else {
      m_editTransferButton->disable();
      m_delTransferButton->disable();

      m_editGroupButton->disable();
      m_delGroupButton->disable();
    }
  }
  return 1;
}



void FF_TransferView::clear() {
  m_transferTree->clearItems();
}



void FF_TransferView::sortTransfers() {
  m_transferTree->sort();
}




