/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_STATEMENTVIEW_HPP
#define FF_STATEMENTVIEW_HPP


#include "ff_transfertree.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <fx.h>

#include <list>


class FF_TransferTree;

/**
 * Emits signal SEL_INSERTED when a new transfer is added and SEL_REPLACED when an existing
 * transfer was edited. Also, just before using a transfer from the list for editing
 * the signal SEL_OPENED is emitted to allow for the target to fill the to-edit transfer
 * with data from the database (->on-demand loading)
 */
class FF_TransferView: public FXHorizontalFrame {
  FXDECLARE(FF_TransferView)

public:
  enum {
    ID_STATEMENT_TREE=FXPacker::ID_LAST,
    ID_BUTTON_ADDSTATEMENT,
    ID_BUTTON_EDITSTATEMENT,
    ID_BUTTON_DELSTATEMENT,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_LAST
  };

public:
  FF_TransferView(FXComposite *p,
                  AE_BOOK_TABLE_TYPE tt,
                  FXuint opts=0,
                  FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FF_TransferView();

  void enter();
  void leave();

  void create();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void toGui(const AE_STATEMENT_LIST *transfers);
  void fromGui(AE_STATEMENT_LIST *transfers);

  void clear();
  void sortTransfers();

  long onCmdAddTransfer(FXObject*, FXSelector, void*);
  long onCmdEditTransfer(FXObject*, FXSelector, void*);
  long onCmdDelTransfer(FXObject*, FXSelector, void*);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);

  FF_TransferTree *getTransferTree() const { return m_transferTree;};

protected:
  FF_TransferView();

  FF_TransferTree *m_transferTree;

  FXButton *m_addGroupButton;
  FXButton *m_editGroupButton;
  FXButton *m_delGroupButton;

  FXButton *m_addTransferButton;
  FXButton *m_editTransferButton;
  FXButton *m_delTransferButton;

  AE_BOOK_TABLE_TYPE m_type;
};




#endif

