/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_ACCOUNTTREE_HPP
#define FF_ACCOUNTTREE_HPP


#include <fx.h>

#include <aqfoxext/fxx_checktree.hpp>

#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_commodity.h>

#include <list>


class FF_AccountTree;



class FF_AccountTreeItem: public FXXCheckTreeItem {
public:
  /**
   *
   */
  FF_AccountTreeItem(FF_AccountTree *tree, AE_ACCOUNT *acc, void* ptr=NULL);

  virtual ~FF_AccountTreeItem();

  AE_ACCOUNT *getAccount() const { return m_account;};
  void rebuild();

protected:
  void drawColumn(const FXXCheckTree* list,
		  FXDC &dc,
		  FXint col,
		  FXint x, FXint y, FXint w, FXint h,
		  const FXchar* string, FXuint length) const;


  FF_AccountTree *m_tree;
  AE_ACCOUNT *m_account;
};



class FF_AccountTree: public FXXCheckTree {
  FXDECLARE(FF_AccountTree)
public:
  enum {
    COPYMODE_NONE=0,
    COPYMODE_COPY,
    COPYMODE_ATTACH
  };

public:
  FF_AccountTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                 FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_AccountTree();

  FF_AccountTreeItem *findAccountItem(AQDB_ID id);
  FF_AccountTreeItem *findAccountItem(FF_AccountTreeItem *grItem, AQDB_ID id);

  AE_ACCOUNT *getCurrentAccount();
  AE_ACCOUNT *getFirstSelectedAccount();

  void expandItems();

  void addAccounts(AE_ACCOUNT_TREE *gt, AQDB_ID exceptId=0);
  void addAccountWithChildren(FF_AccountTreeItem *itemFather,
                              const AE_ACCOUNT *acc,
                              AQDB_ID exceptId=0);

  void addSelectedAccountWithChildren(FF_AccountTreeItem *itemFather,
				      const AE_ACCOUNT *acc,
				      AQDB_ID exceptId);

  void addSelectedAccounts(const AE_ACCOUNT_TREE *gt, AQDB_ID exceptId);

  void selectAccount(AQDB_ID id, bool b);
  void makeAccountVisible(AQDB_ID id);
  void setCurrentAccount(AQDB_ID id, bool b);

  void create();

  void dump(int indent);

  virtual void sortByColumn(int c, bool up);

  FXbool getHeaderArrowDir(int c) const;
  void setHeaderArrowDir(int c, FXbool b);

  long onHeaderClicked(FXObject*, FXSelector, void* ptr);

  AE_COMMODITY_LIST *getCommodities() const { return m_commodities;};

  void reload();

protected:
  FF_AccountTree();

  AE_COMMODITY_LIST *m_commodities;
};



#endif




