/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_ADDRESSTREE_HPP
#define FF_ADDRESSTREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_address.h>



class FF_AddressTreeItem: public FF_GroupTreeItem {
public:
  FF_AddressTreeItem(AE_ADDRESS *addr, FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_AddressTreeItem();

  AE_ADDRESS *getAddress() const { return m_address;};
  void rebuild();

protected:

  AE_ADDRESS *m_address;
};



class FF_AddressTree: public FF_GroupTree {

public:
  FF_AddressTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                  FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_AddressTree();

  FF_AddressTreeItem *findAddressItem(AQDB_ID id);
  FF_AddressTreeItem *findAddressItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_AddressTreeItem *findAddressItem(const AE_ADDRESS *addr);

  void addAddress(AE_ADDRESS *addr, AQDB_ID gid=0);

  AE_ADDRESS *getCurrentAddress();
  AE_ADDRESS *getFirstSelectedAddress();
  void getSelectedCurrencies(AE_ADDRESS_LIST2 *al);

  void selectAddress(AQDB_ID id, bool b);
  void makeAddressVisible(AQDB_ID id);

  void sortByColumn(int c, bool up);

  void create();

protected:
  FF_AddressTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp2(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown2(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp3(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown3(const FXFoldingItem *it1, const FXFoldingItem *it2);

};



#endif




