/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_BANKACCOUNT_HPP
#define FF_BANKACCOUNT_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_bankaccount.h>
#include <aqfinance/engine/book/ae_bankbalance.h>



class FF_BankAccountTreeItem: public FF_GroupTreeItem {
public:
  enum {
    FLAGS_PAYMENTS_ENQUEUED   = 0x00000001,
    FLAGS_RETRIEVAL_ENQUEUED  = 0x00000002,
  };


  FF_BankAccountTreeItem(AE_BANKACCOUNT *ba,
                         AE_BANKBALANCE *booked,
                         AE_BANKBALANCE *noted,
                         FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_BankAccountTreeItem();

  AE_BANKACCOUNT *getBankAccount() const { return m_bankAccount;};

  AE_BANKBALANCE *getNotedBalance() const { return m_notedBalance;};
  AE_BANKBALANCE *getBookedBalance() const { return m_bookedBalance;};

  uint32_t getFlags() const { return m_flags;};
  void setFlags(uint32_t flags) { m_flags=flags;};
  void addFlags(uint32_t flags) { m_flags|=flags;};
  void subFlags(uint32_t flags) { m_flags&=~flags;};

protected:
  void drawColumn(const FXXCheckTree* list, FXDC &dc, FXint col,
		  FXint xx, FXint y, FXint w, FXint h,
		  const FXchar* string, FXuint length) const;

  AE_BANKACCOUNT *m_bankAccount;
  AE_BANKBALANCE *m_notedBalance;
  AE_BANKBALANCE *m_bookedBalance;
  uint32_t m_flags;
};



class FF_BankAccountTree: public FF_GroupTree {
  friend class FF_BankAccountTreeItem;
public:
  FF_BankAccountTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                     FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_BankAccountTree();

  FF_BankAccountTreeItem *findBankAccountItem(AQDB_ID id);
  FF_BankAccountTreeItem *findBankAccountItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_BankAccountTreeItem *findBankAccountItem(const AE_BANKACCOUNT *qba);

  void addAccount(const AE_BANKACCOUNT *ba,
                  const AE_BANKBALANCE *booked,
                  const AE_BANKBALANCE *noted);

  void addAccount(AE_BANKACCOUNT *ba,
                  AE_BANKBALANCE *booked,
                  AE_BANKBALANCE *noted);

  AE_BANKACCOUNT *getCurrentBankAccount();
  AE_BANKACCOUNT *getFirstSelectedBankAccount();
  void getSelectedBankAccounts(AE_BANKACCOUNT_LIST2 *al);

  void getBankAccounts(AE_BANKACCOUNT_LIST2 *al, int copyMode);

  void selectBankAccount(AQDB_ID id, bool b);
  void makeBankAccountVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);

protected:
  FF_BankAccountTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp2(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown2(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp3(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown3(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp4(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown4(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp5(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown5(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp6(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown6(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp7(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown7(const FXFoldingItem *it1, const FXFoldingItem *it2);


  FXIcon *m_icon_paymnt;
  FXIcon *m_icon_paymnt_not;

  FXIcon *m_icon_retr;
  FXIcon *m_icon_retr_not;

};



#endif




