/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_commodityedit.hpp"
#include "ff_app.hpp"
#include "ff_selectcurrency.hpp"

#include <aqfinance/engine/book/ae_commodity.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_CommodityEdit) FF_CommodityEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_CommodityEdit::ID_COMBOEDIT, FF_CommodityEdit::onCmdCombo),
  FXMAPFUNC(SEL_COMMAND, FF_CommodityEdit::ID_BUTTON, FF_CommodityEdit::onCmdButton),
  FXMAPFUNC(SEL_DESELECTED, FF_CommodityEdit::ID_COMBOEDIT, FF_CommodityEdit::onCmdCancel),
  FXMAPFUNC(SEL_FOCUSIN, 0, FF_CommodityEdit::onFocusIn),
};


// Object implementation
FXIMPLEMENT(FF_CommodityEdit, FXHorizontalFrame, FF_CommodityEditMap, ARRAYNUMBER(FF_CommodityEditMap))



FF_CommodityEdit::FF_CommodityEdit(FXComposite *p,
				   FXObject* tgt,
				   FXSelector sel,
				   FXuint opts,
				   FXint x, FXint y, FXint w, FXint h,
				   FXint pl,
				   FXint pr,
				   FXint pt,
				   FXint pb)
:FXHorizontalFrame(p, opts, x, y, w, h, pl, pr, pt, pb)
,m_commodityList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
  FF_App *app;
  target=tgt;
  message=sel;

  app=dynamic_cast<FF_App*>(getApp());

  m_comboEdit=new FXXComboEdit(this, 1,
                               this, ID_COMBOEDIT,
                               LAYOUT_FILL_X);
  m_comboEdit->setSortFunc(FXList::ascendingCase);
  m_iconButton=app->getIcon("minifolderopen");
  if (m_iconButton) {
    m_button=new THEMEBUTTON(this, "", m_iconButton, this, ID_BUTTON);
  }
  else
    m_button=new THEMEBUTTON(this, "...", NULL, this, ID_BUTTON);
}



FF_CommodityEdit::FF_CommodityEdit()
:FXHorizontalFrame()
,m_commodityList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
}



FF_CommodityEdit::~FF_CommodityEdit() {
  AE_Commodity_List_free(m_commodityList);
}



void FF_CommodityEdit::create() {
  FXHorizontalFrame::create();
  if (m_iconButton)
    m_iconButton->create();
  m_comboEdit->create();
  m_button->create();
}



void FF_CommodityEdit::setFont(FXFont* fnt) {
  m_comboEdit->setFont(fnt);
  m_button->setFont(fnt);
}



void FF_CommodityEdit::setBackColor(FXColor clr) {
  FXHorizontalFrame::setBackColor(clr);
  m_comboEdit->setBackColor(clr);
  m_button->setBackColor(clr);
}



void FF_CommodityEdit::setTextColor(FXColor clr) {
  m_comboEdit->setTextColor(clr);
  m_button->setTextColor(clr);
}



void FF_CommodityEdit::setSelBackColor(FXColor clr) {
  m_comboEdit->setSelBackColor(clr);
}



void FF_CommodityEdit::setSelTextColor(FXColor clr) {
  m_comboEdit->setSelTextColor(clr);
}



void FF_CommodityEdit::setText(const FXString& text) {
  m_comboEdit->setText(text);
}



FXString FF_CommodityEdit::getText() const {
  return m_comboEdit->getText();
}



void FF_CommodityEdit::selectAll() {
  m_comboEdit->selectAll();
}



void FF_CommodityEdit::setCommodityId(AQDB_ID id) {
  AE_COMMODITY *co;

  co=AE_Commodity_List_GetById(m_commodityList, id);
  if (co) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AE_Commodity_PathToBuffer(co, tbuf);
    m_comboEdit->setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }
}



AQDB_ID FF_CommodityEdit::getCommodityId() {
  FXString str;

  str=m_comboEdit->getText();
  if (!str.empty()) {
    AE_COMMODITY *co;

    co=AE_Commodity_List_GetByPath(m_commodityList, str.text());
    if (co)
      return AE_Commodity_GetId(co);
  }

  return 0;
}



void FF_CommodityEdit::setCurrency(const char *s) {
  AE_COMMODITY *co;

  co=AE_Commodity_List_GetByCode(m_commodityList, s);
  if (co) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AE_Commodity_PathToBuffer(co, tbuf);
    m_comboEdit->setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }
}



FXString FF_CommodityEdit::getCurrency() {
  FXString str;

  str=m_comboEdit->getText();
  if (!str.empty()) {
    AE_COMMODITY *co;

    co=AE_Commodity_List_GetByPath(m_commodityList, str.text());
    if (co) {
      const char *s;

      s=AE_Commodity_GetCode(co);
      if (s && *s)
	return FXString(s);
    }
  }

  return "";
}



long FF_CommodityEdit::onCmdCombo(FXObject*, FXSelector, void*) {
  if (target && target->handle(this, FXSEL(SEL_COMMAND, message), NULL))
    return 1;
  return 1;
}



long FF_CommodityEdit::onCmdButton(FXObject*, FXSelector, void*) {
  AQDB_ID id;

  /* TODO: get currently selected currency */
  id=0;

  if (FF_SelectCurrency::selectCurrency(dynamic_cast<FF_App*>(getApp()),
                                        this,
					I18N("Select a Currency"),
					id)) {
    AE_COMMODITY *co;

    reload();

    co=AE_Commodity_List_GetById(m_commodityList, id);
    if (co) {
      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

      AE_Commodity_PathToBuffer(co, tbuf);
      setText(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
      if (target)
	target->handle(this, FXSEL(SEL_COMMAND, message), NULL);
    }
  }

  return 1;
}



long FF_CommodityEdit::onFocusIn(FXObject *sender, FXSelector sel, void *ptr){
  FXHorizontalFrame::onFocusIn(sender, sel, ptr);
  m_comboEdit->setFocus();
  return 1;
}



long FF_CommodityEdit::onCmdCancel(FXObject *sender, FXSelector sel, void *ptr){
  if (target && target->handle(this, FXSEL(SEL_DESELECTED, message), NULL))
    return 1;

  return 0;
}



void FF_CommodityEdit::reload() {
  FF_App *app;
  int rv;

  m_comboEdit->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  AE_Commodity_List_free(m_commodityList);
  m_commodityList=AE_Commodity_List_new();
  rv=AE_Book_ReadCommodityList(app->getBook(), AE_Book_TableType_Commodity, m_commodityList);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else {
    AE_COMMODITY *co;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    co=AE_Commodity_List_First(m_commodityList);
    while(co) {
      AE_Commodity_PathToBuffer(co, tbuf);
      m_comboEdit->addEntry(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
      co=AE_Commodity_List_Next(co);
    }
    GWEN_Buffer_free(tbuf);
    m_comboEdit->sortItems();
    m_comboEdit->reloadList();
  }
}






