/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_conflictwidget.hpp"
#include "ff_app.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ConflictWidget) FF_ConflictWidgetMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_ConflictWidget::ID_RADIO, FF_ConflictWidget::onCmdRadio),
};

FXIMPLEMENT(FF_ConflictWidget, FXVerticalFrame, FF_ConflictWidgetMap, ARRAYNUMBER(FF_ConflictWidgetMap))



FF_ConflictWidget::FF_ConflictWidget()
:FXVerticalFrame() {
}



FF_ConflictWidget::FF_ConflictWidget(FXComposite *p,
				     AE_DBCONFLICT *conflict,
				     FXuint opts)
:FXVerticalFrame(p, opts)
,m_conflict(conflict)
,m_radioValue(1)
,m_radiotarget(m_radioValue, this, ID_RADIO)
{
  AE_DBCONFLICTSOLUTION *sol;
  const char *s;
  FXString str;
  FOX16_HtmlLabel *label;
  int id;

  s=AE_DbConflict_GetText(conflict);
  if (s && *s)
    str=FXString(s);
  label=new FOX16_HtmlLabel(this, str, LAYOUT_FILL_X);
  label->setBackColor(FXRGB(255,166,166));
  /* select first solution */
  AE_DbConflict_SetSelectedSolutionIndex(conflict, 1);

  id=1;
  sol=AE_DbConflictSolution_List_First(AE_DbConflict_GetSolutionList(conflict));
  while(sol) {
    FXString str;

    s=AE_DbConflictSolution_GetText(sol);
    if (s && *s)
      str=FXString(s);
    new FXRadioButton(this, str, &m_radiotarget, FXDataTarget::ID_OPTION+(id++), JUSTIFY_LEFT|ICON_BEFORE_TEXT|LAYOUT_FILL_X);

    sol=AE_DbConflictSolution_List_Next(sol);
  }
}



FF_ConflictWidget::~FF_ConflictWidget() {
}



long FF_ConflictWidget::onCmdRadio(FXObject *obj, FXSelector sel, void *ptr) {
  int idx;

  idx=m_radioValue;
  AE_DbConflict_SetSelectedSolutionIndex(m_conflict, idx);
  return 1;
}


