/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_CURRENCYTREE_HPP
#define FF_CURRENCYTREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_commodity.h>



class FF_CurrencyTreeItem: public FF_GroupTreeItem {
public:
  FF_CurrencyTreeItem(AE_COMMODITY *co, FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_CurrencyTreeItem();

  AE_COMMODITY *getCurrency() const { return m_currency;};

protected:

  AE_COMMODITY *m_currency;
};



class FF_CurrencyTree: public FF_GroupTree {

public:
  FF_CurrencyTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                  FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_CurrencyTree();

  FF_CurrencyTreeItem *findCurrencyItem(AQDB_ID id);
  FF_CurrencyTreeItem *findCurrencyItem(FF_GroupTreeItem *grItem, AQDB_ID id);

  void addCurrency(AE_COMMODITY *co, AQDB_ID gid=0);

  AE_COMMODITY *getCurrentCurrency();
  AE_COMMODITY *getFirstSelectedCurrency();
  void getSelectedCurrencies(AE_COMMODITY_LIST2 *al);

  void selectCurrency(AQDB_ID id, bool b);
  void makeCurrencyVisible(AQDB_ID id);

  void sortByColumn(int c, bool up);

  void create();

protected:
  FF_CurrencyTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp2(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown2(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp3(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown3(const FXFoldingItem *it1, const FXFoldingItem *it2);

};



#endif




