/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_DATEEDIT_HPP
#define FF_DATEEDIT_HPP


#include <fx.h>

#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/gwendate.h>



class FF_DateEdit: public FXTextField {
  FXDECLARE(FF_DateEdit)

protected:
  FXString formatString;

protected:
  FF_DateEdit(): FXTextField() {};

public:
  long onKeyPress(FXObject* sender,FXSelector sel,void* ptr);
  long onVerify(FXObject* sender,FXSelector sel,void* ptr);

public:
  /**
   * @param formatString this is a max 5 byte long string containing the
   *  edit-format of the date (e.g. for DE "d.m.y", or for US "m/d/y")
   */
  FF_DateEdit(FXComposite *p,
	      FXObject* tgt=NULL,
	      FXSelector sel=0,
	      const FXString &formatString="YYYY/MM/DD",
	      FXuint opts=TEXTFIELD_NORMAL,
	      FXint x=0,FXint y=0,FXint w=0,FXint h=0);
  ~FF_DateEdit();

  FXbool getDate(FXDate &rdate);
  FXbool setDate(const FXDate &date, FXbool notify=FALSE);

  FXbool getDate(GWEN_TIME **ti);
  FXbool setDate(const GWEN_TIME *pTi, FXbool notify=FALSE);

  FXbool getDate(GWEN_DATE **ti);
  FXbool setDate(const GWEN_DATE *pTi, FXbool notify=FALSE);

  static FXbool dateToString(const FXDate &date,
                             const FXString &fmt,
                             FXString &str);
  static FXbool dateToString(const GWEN_TIME *ti,
                             const FXString &fmt,
                             FXString &str);
  static FXbool dateToString(const GWEN_DATE *ti,
                             const FXString &fmt,
                             FXString &str);

  static FXbool dateFromString(const FXString &str,
                               const FXString &fmt,
                               FXDate &date);
  static FXbool dateFromString(const FXString &str,
                               const FXString &fmt,
                               GWEN_TIME **pTi);
  static FXbool dateFromString(const FXString &str,
                               const FXString &fmt,
                               GWEN_DATE **pTi);

  static FXbool convertDate(const FXDate d, GWEN_TIME **pTi);
  static FXbool convertDate(const GWEN_TIME *ti, FXDate &res);

  static FXbool convertDate(const FXDate d, GWEN_DATE **pTi);
  static FXbool convertDate(const GWEN_DATE *ti, FXDate &res);

  static FXDate today();
};


#endif


