/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_flickerwidget.hpp"

#include <aqfinance/engine/ae_api.h>

#include <gwenhywfar/debug.h>

#include <ctype.h>




FXDEFMAP(FF_FlickerWidget) FF_FlickerWidgetMap[]={
  FXMAPFUNC(SEL_PAINT, 0, FF_FlickerWidget::onPaint),
  FXMAPFUNC(SEL_TIMEOUT, FF_FlickerWidget::ID_TIMER, FF_FlickerWidget::onTimer),
};


FXIMPLEMENT(FF_FlickerWidget, FXFrame, FF_FlickerWidgetMap, ARRAYNUMBER(FF_FlickerWidgetMap))





FF_FlickerWidget::FF_FlickerWidget(FXComposite *p,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h)
:FXFrame(p,opts,x,y,w,h)
,m_phaseLength(50)
,m_isRunning(false)
,m_currentPos(-1)
,m_clock(0){
}



FF_FlickerWidget::FF_FlickerWidget()
:FXFrame()
,m_phaseLength(50)
,m_isRunning(false)
,m_currentPos(-1)
,m_clock(0) {
}



FF_FlickerWidget::~FF_FlickerWidget() {
}



void FF_FlickerWidget::setPhaseLength(int i) {
  m_phaseLength=i;
}



void FF_FlickerWidget::setFlickerCode(const char *code) {
  if (code) {
    int len;

    len=strlen(code);
    if (len % 1) {
      DBG_ERROR(AE_LOGDOMAIN, "Invalid length of code (%d)", len);
    }
    else {
      int i;
      FXString str;
      FXbool wasRunning=m_isRunning;

      if (wasRunning)
        stopFlicker();

      str="F0FF";

      /* add code, switch 1st and 2nd char */
      for (i=0; i<len; i+=2) {
        str+=code[i+1];
        str+=code[i];
      }

      m_code=str;
      m_clock=0;
      m_currentPos=0;

      if (wasRunning)
        startFlicker();
    }
  }
}



void FF_FlickerWidget::startFlicker() {
  if (!m_code.empty()) {
    m_currentPos=0;
    m_clock=0;
    m_isRunning=true;
    getApp()->addTimeout(this, ID_TIMER, m_phaseLength);
  }

}



void FF_FlickerWidget::stopFlicker() {
  getApp()->removeTimeout(this, ID_TIMER);
  m_isRunning=false;
}



long FF_FlickerWidget::onPaint(FXObject*,FXSelector, void *ptr) {
  FXEvent *ev=(FXEvent*)ptr;
  FXDCWindow dc(this,ev);
  int margin=7;
  int barWidth=width/5;

  dc.setForeground(FXRGB(255, 255, 255));
  dc.fillRectangle(0, 0, width, height);

  if (m_currentPos<m_code.length()) {
    uint8_t c;
    int i;

    c=toupper(m_code[m_currentPos]);
    c-='0';
    if (c>9)
      c-=7;
    c<<=1;
    if (m_clock==0)
      c|=1;
    for (i=0; i<5; i++) {
      if (c & 1)
        dc.setForeground(FXRGB(255, 255, 255));
      else
        dc.setForeground(FXRGB(0, 0, 0));

      dc.fillRectangle(i*barWidth+margin, margin, barWidth-2*margin, height-2*margin);

      c>>=1;
    }
  }
  return 1;
}



long FF_FlickerWidget::onTimer(FXObject*,FXSelector,void*) {
  if (m_currentPos>=m_code.length())
    m_currentPos=0;

  m_clock++;
  if (m_clock>1) {
    m_clock=0;
    m_currentPos++;
  }
  update();
  getApp()->addTimeout(this, ID_TIMER, m_phaseLength);
  return 1;
}






