/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_GROUPTREE_HPP
#define FF_GROUPTREE_HPP


#include <fx.h>

#include <aqfoxext/fxx_htmltreeitem.hpp>

#include <aqfinance/engine/book/ae_group.h>

#include <list>



class FF_GroupTreeItem: public FXXHtmlTreeItem {
public:
  /**
   *
   */
  FF_GroupTreeItem(AE_GROUP *gr, FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_GroupTreeItem();

  bool isDataItem() const { return m_isDataItem;};
  AE_GROUP *getGroup() const { return m_group;};

protected:
  /**
   * This constructor should be used for data items.
   */
  FF_GroupTreeItem(FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  /**
   * This implementation calls FXFoldingItem::draw for data items and draws
   * the item itself for group items.
   * Group items always use the full width of the tree regardless of the number
   * and width of the header columns.
   */
  void draw(const FXFoldingList* list,
            FXDC& dc,
            FXint x, FXint y, FXint, FXint h) const;

  bool m_isDataItem;
  AE_GROUP *m_group;
};



class FF_GroupTree: public FXXCheckTree {
  FXDECLARE(FF_GroupTree)
public:
  enum {
    COPYMODE_NONE=0,
    COPYMODE_COPY,
    COPYMODE_ATTACH
  };

public:
  FF_GroupTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
	       FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_GroupTree();

  FF_GroupTreeItem *findGroupItem(AQDB_ID id);
  FF_GroupTreeItem *findGroupItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_GroupTreeItem *findGroupItem(const AE_GROUP *gr);

  FF_GroupTreeItem *findGroupItemByName(FF_GroupTreeItem *grItem, const char *name);
  FF_GroupTreeItem *findGroupItemByName(const char *name);

  AE_GROUP *getCurrentGroup();
  AE_GROUP *getFirstSelectedGroup();

  FF_GroupTreeItem *getCurrentGroupItem();

  void expandItems();

  void addGroup(AE_GROUP *gr);
  void addGroups(AE_GROUP_TREE *gt, AQDB_ID exceptId=0);
  void addGroupWithChildren(FF_GroupTreeItem *itemFather,
                            const AE_GROUP *gr,
                            AQDB_ID exceptId=0);

  void selectGroup(AQDB_ID id, bool b);
  void makeGroupVisible(AQDB_ID id);

  void unselectAll();


  FXIcon *getIconOpen() const { return m_iconOpen;};
  void setIconOpen(FXIcon *ic) { m_iconOpen=ic;};

  FXIcon *getIconClosed() const { return m_iconClosed;};
  void setIconClosed(FXIcon *ic) { m_iconClosed=ic;};

  void create();

  void dump(int indent);

  virtual void sortByColumn(int c, bool up);

  FXbool getHeaderArrowDir(int c) const;
  void setHeaderArrowDir(int c, FXbool b);

protected:
  FF_GroupTree();

  static FXint sortFunctionUp(const FXFoldingItem*, const FXFoldingItem*);
  static FXint sortFunctionDown(const FXFoldingItem*, const FXFoldingItem*);

  FXIcon *m_iconOpen;
  FXIcon *m_iconClosed;

};




/* some macros for sorting */
#define GROUPTREE_SORT_FN_UP_TXT(macro_class, macro_itemtype, macro_getmemberfn,             \
                                 macro_membertype, macro_memberprefix, macro_name)           \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                   \
      const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                   \
      const macro_membertype *ba1=tti1->macro_getmemberfn();                                 \
      const macro_membertype *ba2=tti2->macro_getmemberfn();                                 \
      const char *s1=macro_memberprefix##_Get##macro_name(ba1);                              \
      const char *s2=macro_memberprefix##_Get##macro_name(ba2);                              \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}




#define GROUPTREE_SORT_FN_UP_VALUE(macro_class, macro_itemtype, macro_getmemberfn,           \
                                   macro_membertype, macro_memberprefix, macro_name)         \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                   \
      const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                   \
      const macro_membertype *ba1=tti1->macro_getmemberfn();                                 \
      const macro_membertype *ba2=tti2->macro_getmemberfn();                                 \
      const AQDB_VALUE *v1=macro_memberprefix##_Get##macro_name(ba1);                        \
      const AQDB_VALUE *v2=macro_memberprefix##_Get##macro_name(ba2);                        \
                                                                                             \
      /* both are data items */                                                              \
      if (v1 && v2)                                                                          \
        return AQDB_Value_Compare(v1, v2);                                                   \
      else if (v1)                                                                           \
        return 1;                                                                            \
      else if (v2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define GROUPTREE_SORT_FN_UP_DATE(macro_class, macro_itemtype, macro_getmemberfn,            \
                                  macro_membertype, macro_memberprefix, macro_name)          \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                   \
      const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                   \
      const macro_membertype *ba1=tti1->macro_getmemberfn();                                 \
      const macro_membertype *ba2=tti2->macro_getmemberfn();                                 \
      const GWEN_DATE *t1=macro_memberprefix##_Get##macro_name(ba1);                         \
      const GWEN_DATE *t2=macro_memberprefix##_Get##macro_name(ba2);                         \
                                                                                             \
      /* both are data items */                                                              \
      if (t1 && t2) {                                                                        \
        int d=GWEN_Date_Diff(t1, t2);                                                        \
                                                                                             \
        if (d<0)                                                                             \
          return -1;                                                                         \
        else if (d>0)                                                                        \
          return 1;                                                                          \
      }                                                                                      \
      else if (t1)                                                                           \
        return 1;                                                                            \
      else if (t2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define GROUPTREE_SORT_FN_UP_INT(macro_class, macro_itemtype, macro_getmemberfn,             \
                                 macro_membertype, macro_memberprefix, macro_name)           \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                   \
      const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                   \
      const macro_membertype *ba1=tti1->macro_getmemberfn();                                 \
      const macro_membertype *ba2=tti2->macro_getmemberfn();                                 \
      int v1=macro_memberprefix##_Get##macro_name(ba1);                                      \
      int v2=macro_memberprefix##_Get##macro_name(ba2);                                      \
                                                                                             \
      /* both are data items */                                                              \
      if (v2<v1)                                                                             \
        return 1;                                                                            \
      else if (v1<v2)                                                                        \
        return -1;                                                                           \
      else                                                                                   \
        return 0;                                                                            \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}





#define GROUPTREE_SORT_FN_DOWN(macro_class, macro_itemtype, macro_getmemberfn,               \
                               macro_membertype, macro_memberprefix, macro_name)             \
FXint macro_class::sortFunctionDown##macro_name(const FXFoldingItem *it1,                    \
                                                const FXFoldingItem *it2){                   \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_name(it1, it2);                                                   \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



#endif




