/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_itemedit.hpp"
#include "ff_app.hpp"
#include "ff_selectitem.hpp"

#include <aqfinance/engine/book/ae_item.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ItemEdit) FF_ItemEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_ItemEdit::ID_COMBOEDIT, FF_ItemEdit::onCmdCombo),
  FXMAPFUNC(SEL_COMMAND, FF_ItemEdit::ID_BUTTON, FF_ItemEdit::onCmdButton),
  FXMAPFUNC(SEL_DESELECTED, FF_ItemEdit::ID_COMBOEDIT, FF_ItemEdit::onCmdCancel),
  FXMAPFUNC(SEL_FOCUSIN, 0, FF_ItemEdit::onFocusIn),
};


// Object implementation
FXIMPLEMENT(FF_ItemEdit, FXHorizontalFrame,
	    FF_ItemEditMap, ARRAYNUMBER(FF_ItemEditMap))



FF_ItemEdit::FF_ItemEdit(AE_BOOK_TABLE_TYPE tt,
			 FXComposite *p,
			 FXObject* tgt,
			 FXSelector sel,
			 FXuint opts,
			 FXint x, FXint y, FXint w, FXint h,
			 FXint pl,
			 FXint pr,
			 FXint pt,
			 FXint pb)
:FXHorizontalFrame(p, opts, x, y, w, h, pl, pr, pt, pb)
,m_itemList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL)
,m_tableType(tt)
{
  FF_App *app;
  target=tgt;
  message=sel;

  app=dynamic_cast<FF_App*>(getApp());

  m_comboEdit=new FXXComboEdit(this, 1,
                               this, ID_COMBOEDIT,
                               LAYOUT_FILL_X);
  m_comboEdit->setSortFunc(FXList::ascendingCase);
  m_iconButton=app->getIcon("minifolderopen");
  if (m_iconButton) {
    m_button=new THEMEBUTTON(this, "", m_iconButton, this, ID_BUTTON);
  }
  else
    m_button=new THEMEBUTTON(this, "...", NULL, this, ID_BUTTON);
}



FF_ItemEdit::FF_ItemEdit()
:FXHorizontalFrame()
,m_itemList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
}



FF_ItemEdit::~FF_ItemEdit() {
  AE_Item_List_free(m_itemList);
}



void FF_ItemEdit::create() {
  FXHorizontalFrame::create();
  if (m_iconButton)
    m_iconButton->create();
  m_comboEdit->create();
  m_button->create();
}



void FF_ItemEdit::setFont(FXFont* fnt) {
  m_comboEdit->setFont(fnt);
  m_button->setFont(fnt);
}



void FF_ItemEdit::setBackColor(FXColor clr) {
  FXHorizontalFrame::setBackColor(clr);
  m_comboEdit->setBackColor(clr);
  m_button->setBackColor(clr);
}



void FF_ItemEdit::setTextColor(FXColor clr) {
  m_comboEdit->setTextColor(clr);
  m_button->setTextColor(clr);
}



void FF_ItemEdit::setSelBackColor(FXColor clr) {
  m_comboEdit->setSelBackColor(clr);
}



void FF_ItemEdit::setSelTextColor(FXColor clr) {
  m_comboEdit->setSelTextColor(clr);
}



void FF_ItemEdit::setText(const FXString& text) {
  m_comboEdit->setText(text);
}



FXString FF_ItemEdit::getText() const {
  return m_comboEdit->getText();
}



void FF_ItemEdit::selectAll() {
  m_comboEdit->selectAll();
}



void FF_ItemEdit::setItemId(AQDB_ID id) {
  AE_ITEM *it;

  it=AE_Item_List_GetById(m_itemList, id);
  if (it) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AE_Item_PathToBuffer(it, tbuf);
    m_comboEdit->setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }
}



AQDB_ID FF_ItemEdit::getItemId() {
  FXString str;

  str=m_comboEdit->getText();
  if (!str.empty()) {
    AE_ITEM *it;

    it=AE_Item_List_GetByPath(m_itemList, str.text());
    if (it)
      return AE_Item_GetId(it);
  }

  return 0;
}



long FF_ItemEdit::onCmdCombo(FXObject*, FXSelector, void*) {
  if (target && target->handle(this, FXSEL(SEL_COMMAND, message), NULL))
    return 1;
  return 1;
}



long FF_ItemEdit::onCmdButton(FXObject*, FXSelector, void*) {
  AQDB_ID id;
  AE_ITEM *it;

  /* TODO: get currently selected partner */
  id=0;

  it=FF_SelectItem::selectEngineItem(dynamic_cast<FF_App*>(getApp()),
				     this,
				     I18N("Select a Item"));
  reload();
  if (it) {
    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

    AE_Item_PathToBuffer(it, tbuf);
    setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
    if (target)
      target->handle(this, FXSEL(SEL_COMMAND, message), NULL);
  }

  return 1;
}



long FF_ItemEdit::onFocusIn(FXObject *sender, FXSelector sel, void *ptr){
  FXHorizontalFrame::onFocusIn(sender, sel, ptr);
  m_comboEdit->setFocus();
  return 1;
}



long FF_ItemEdit::onCmdCancel(FXObject *sender, FXSelector sel, void *ptr){
  if (target && target->handle(this, FXSEL(SEL_DESELECTED, message), NULL))
    return 1;

  return 0;
}



void FF_ItemEdit::reload() {
  FF_App *app;
  int rv;

  m_comboEdit->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  AE_Item_List_free(m_itemList);
  m_itemList=AE_Item_List_new();
  rv=AE_Book_ReadItemList(app->getBook(), m_tableType, m_itemList);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else {
    AE_ITEM *it;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    it=AE_Item_List_First(m_itemList);
    while(it) {
      AE_Item_PathToBuffer(it, tbuf);
      m_comboEdit->addEntry(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
      it=AE_Item_List_Next(it);
    }
    GWEN_Buffer_free(tbuf);
    m_comboEdit->sortItems();
    m_comboEdit->reloadList();
  }
}






