/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_ITEMTABLE_HPP
#define FF_ITEMTABLE_HPP


#include <aqfoxext/fxx_table.hpp>

#include <fx.h>

#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_item.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_tax.h>

#include <list>



class FF_ItemTable: public FXXTable {
  FXDECLARE(FF_ItemTable)

public:
  enum {
    MAX_COLUMNS=32
  };

  typedef enum {
    COLUMN_TYPE_NONE=0,
    COLUMN_TYPE_CODE=1,
    COLUMN_TYPE_NAME,
    COLUMN_TYPE_MEMO,
    COLUMN_TYPE_AMOUNT,

    COLUMN_TYPE_BUY_TAX,
    COLUMN_TYPE_BUY_NETSUM,
    COLUMN_TYPE_BUY_TAXSUM,
    COLUMN_TYPE_BUY_TOTALSUM,

    COLUMN_TYPE_SELL_TAX,
    COLUMN_TYPE_SELL_NETSUM,
    COLUMN_TYPE_SELL_TAXSUM,
    COLUMN_TYPE_SELL_TOTALSUM,

  } COLUMN_TYPE;

  enum {
    ID_TEXTFIELD=FXXTable::ID_LAST,
    ID_AMOUNT_EDIT,
    ID_TAX_EDIT,
    ID_CODE_EDIT,
    ID_LAST
  };

  enum {
    ITEMTABLE_USE_ITEM_TEMPLATES = 0x00800000,    /// use item templates
    };

public:
  FF_ItemTable(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
	       FXint x=0, FXint y=0, FXint w=0, FXint h=0,
	       FXint pl=DEFAULT_MARGIN, FXint pr=DEFAULT_MARGIN,
	       FXint pt=DEFAULT_MARGIN, FXint pb=DEFAULT_MARGIN);

  virtual ~FF_ItemTable();

  void addColumn(COLUMN_TYPE coltype, int size=DEFAULT_COLUMN_WIDTH);

  void addItem(AE_ITEM *st);
  void addItems(AE_ITEM_LIST2 *sl, int copyMode);

  void getSelectedItems(AE_ITEM_LIST2 *sl, int copyMode);

  void delRow(int r);


  AE_ITEM *getItemByRow(int r);

  AE_ITEM *getCurrentItem();

  AQDB_ID getMarkBeyondId() const { return m_markBeyondId;};
  void setMarkBeyondId(AQDB_ID id) { m_markBeyondId=id;};

  virtual void setNumberOfRows(int i);

  virtual void clearItems();

  int reload();

  long onCmdAmount(FXObject*, FXSelector, void* ptr);
  long onCancelAmount(FXObject*, FXSelector, void* ptr);

  long onCmdTax(FXObject*, FXSelector, void* ptr);
  long onCancelTax(FXObject*, FXSelector, void* ptr);

  long onCmdCode(FXObject*, FXSelector, void* ptr);
  long onCancelCode(FXObject*, FXSelector, void* ptr);

  long onCmdTextField(FXObject*, FXSelector, void* ptr);
  long onCancelTextField(FXObject*, FXSelector, void* ptr);

protected:

  class ItemContext {
  public:
    ItemContext();
    ~ItemContext();

    AE_ITEM *item;
    //AE_TAX *tax;

    FXString sAmount;
    FXString sTax;
    FXString sBuyTax;
    FXString sBuyNetSum;
    FXString sBuyTotal;

    FXString sSellTax;
    FXString sSellNetSum;
    FXString sBuyTaxSum;
    FXString sSellTaxSum;
    FXString sSellTotal;
  };


  FF_ItemTable();

  ItemContext *getItemContextByRow(int r);

  virtual ColumnInfo *getColumnInfo(int r, int c);
  virtual void releaseColumnInfo(ColumnInfo *ci);

  virtual FXWindow *getEditor(int r, int c);
  virtual void setFromEditor(ColumnInfo *ci, FXWindow *ed);
  virtual void releaseEditor(int r, int c, FXWindow *ed);

  void addSpaceForItems(int n);

  int readItems();

  int calculateAmountsByNetSum(AQDB_ID taxId,
			       AQDB_ID commodityId,
			       AQDB_VALUE *vNetSum,
			       AQDB_VALUE *vTaxSum,
			       AQDB_VALUE *vTotal);

  int calculateAmountsByTaxSum(AQDB_ID taxId,
			       AQDB_ID commodityId,
			       AQDB_VALUE *vNetSum,
			       AQDB_VALUE *vTaxSum,
			       AQDB_VALUE *vTotal);

  int calculateAmountsByTotal(AQDB_ID taxId,
			      AQDB_ID commodityId,
			      AQDB_VALUE *vNetSum,
			      AQDB_VALUE *vTaxSum,
			      AQDB_VALUE *vTotal);

  void recalcValues(ItemContext *sctx);

  ItemContext **m_itemArray;
  int m_itemArraySize;
  int m_itemCount;

  AQDB_ID m_markBeyondId;

  AE_COMMODITY_LIST *m_commodities;
  AE_TAX_LIST *m_taxes;

  COLUMN_TYPE m_columnTypes[MAX_COLUMNS];
  bool m_useTemplateEditor;
};



#endif


