/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_ITEMTREE_HPP
#define FF_ITEMTREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_item.h>



class FF_ItemTreeItem: public FF_GroupTreeItem {
public:
  FF_ItemTreeItem(AE_ITEM *ba,
		  FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_ItemTreeItem();

  AE_ITEM *getItem() const { return m_itemTemplate;};
  void rebuild();

protected:
  AE_ITEM *m_itemTemplate;
};



class FF_ItemTree: public FF_GroupTree {

public:
  FF_ItemTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
	      FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_ItemTree();

  FF_ItemTreeItem *findItem(AQDB_ID id);
  FF_ItemTreeItem *findItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_ItemTreeItem *findItem(const AE_ITEM *itl);

  void addEngineItem(const AE_ITEM *itl);
  void addEngineItem(AE_ITEM *itl);

  AE_ITEM *getCurrentEngineItem();
  AE_ITEM *getFirstSelectedEngineItem();
  void getSelectedEngineItems(AE_ITEM_LIST2 *pl);

  void getEngineItems(AE_ITEM_LIST2 *pl, int copyMode);

  void selectEngineItem(AQDB_ID id, bool b);
  void selectEngineItem(const AE_ITEM *itl, bool b);
  void makeEngineItemVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);

protected:
  FF_ItemTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp2(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown2(const FXFoldingItem *it1, const FXFoldingItem *it2);

};



#endif




