/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_JOBTREE_HPP
#define FF_JOBTREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_partner.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_task.h>



class FF_JobTreeItem: public FF_GroupTreeItem {
  friend class FF_JobTree;
public:
  enum {
    FLAGS_URGENT            = 0x00000001,
    FLAGS_REMINDER          = 0x00000002,
    FLAGS_PAID_FULL         = 0x00000004,
    FLAGS_PAID_PARTIAL      = 0x00000008,
    FLAGS_DELIVERED_FULL    = 0x00000010,
    FLAGS_DELIVERED_PARTIAL = 0x00000020,
    FLAGS_BILLED_FULL       = 0x00000040,
    FLAGS_BILLED_PARTIAL    = 0x00000080,
    FLAGS_BOOKED_FULL       = 0x00000100,
    FLAGS_BOOKED_PARTIAL    = 0x00000200,
    FLAGS_PRINTED           = 0x00000400,
    FLAGS_SAVED             = 0x00000800,
    FLAGS_MAILED            = 0x00001000
  };

  FF_JobTreeItem(const FXString &txt, uint32_t flags,
		 AE_JOB *j,
		 FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_JobTreeItem();

  AE_JOB *getJob() const { return m_job;};

  void drawColumn(const FXXCheckTree* list,
		  FXDC &dc,
		  FXint col,
		  FXint x, FXint y, FXint w, FXint h,
		  const FXchar* string, FXuint length) const;

  virtual FXString getColumnTooltipText(FXint col);

protected:
  AE_JOB *m_job;
  FXString m_preparedToolTip;
  uint32_t m_flags;
};



class FF_JobTree: public FF_GroupTree {
  friend class FF_JobTreeItem;
public:
  FF_JobTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
	     FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_JobTree();

  FF_JobTreeItem *findJobItem(AQDB_ID id);
  FF_JobTreeItem *findJobItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_JobTreeItem *findJobItem(const AE_JOB *j);
  FF_JobTreeItem *findTaskItem(AQDB_ID id);

  void addJob(const AE_JOB *j, uint32_t flags);
  void addJob(AE_JOB *j, uint32_t flags);

  void addTask(const AE_TASK *tk, uint32_t flags);

  void openTaskItem(AQDB_ID id);

  AE_JOB *getCurrentJob();
  AE_JOB *getFirstSelectedJob();
  void getSelectedJobs(AE_JOB_LIST2 *jl);

  void setCurrentJob(const AE_JOB *j, bool b);
  void setCurrentJob(AQDB_ID id, bool b);

  void getJobs(AE_JOB_LIST2 *jl, int copyMode);

  void selectJob(AQDB_ID id, bool b);
  void selectJob(const AE_JOB *j, bool b);
  void makeJobVisible(AQDB_ID id);

  void sortByColumn(int c, bool up);

  void reload();

  void create();

protected:
  FF_JobTree();

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp2(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown2(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp3(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown3(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp4(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown4(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp6(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown6(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp7(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown7(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp8(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown8(const FXFoldingItem *it1, const FXFoldingItem *it2);


  AE_PARTNER_LIST *m_partners;
  AE_COMMODITY_LIST *m_commodities;

  FXIcon *m_icon_billed_full;
  FXIcon *m_icon_billed_part;
  FXIcon *m_icon_billed_not;

  FXIcon *m_icon_delivered_full;
  FXIcon *m_icon_delivered_part;
  FXIcon *m_icon_delivered_not;

  FXIcon *m_icon_paid_full;
  FXIcon *m_icon_paid_part;
  FXIcon *m_icon_paid_not;

  FXIcon *m_icon_reminded;
  FXIcon *m_icon_reminded_not;

  FXIcon *m_icon_urgent;
  FXIcon *m_icon_urgent_not;

  FXIcon *m_icon_doc;
  FXIcon *m_icon_van;
  FXIcon *m_icon_coin;
  FXIcon *m_icon_bill;
  FXIcon *m_icon_payment;

  FXIcon *m_icon_booked_full;
  FXIcon *m_icon_booked_part;
  FXIcon *m_icon_booked_not;

  FXIcon *m_icon_mailed;
  FXIcon *m_icon_mailed_not;
  FXIcon *m_icon_printed;
  FXIcon *m_icon_printed_not;
  FXIcon *m_icon_saved;
  FXIcon *m_icon_saved_not;

};



#endif




