/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_listitem.hpp"


#include <gwenhywfar/debug.h>




#define ICON_SPACING 4 // Spacing between icon and label
#define SIDE_SPACING 6 // Left or right spacing between items



FF_ListItem::FF_ListItem(const FXString& text,
                       FXIcon* ic,
                       void* ptr)
:FXListItem(text, ic, ptr) {

}



FF_ListItem::~FF_ListItem() {
}



void FF_ListItem::draw(const FXList* list, FXDC& dc,
                       FXint xx, FXint yy, FXint ww, FXint hh) {
  register FXFont *font=list->getFont();
  register FXint ih=0, iw=0, th=0, tw=0, fh=0;

  if (icon) {
    ih=icon->getHeight();
    iw=icon->getHeight();
    fh+=ih;
  }

  if (!label.empty()) {
    th=font->getFontHeight();
    tw=font->getTextWidth(label);
    if (fh) fh+=ICON_SPACING;
    fh+=th;
  }

  if (isSelected())
    dc.setForeground(list->getSelBackColor());
  else
    dc.setForeground(list->getBackColor());
  dc.fillRectangle(xx,yy,ww,hh);

  if (hasFocus()){
    dc.drawFocusRectangle(xx+1,yy+1,ww-2,hh-2);
  }

  xx+=SIDE_SPACING/2;
  yy+=(hh-fh)/2;

  // draw icon
  if (icon){
    dc.drawIcon(icon,xx+(ww-iw)/2,yy);
    yy+=ICON_SPACING+icon->getHeight();
  }

  // draw text
  if (!label.empty()){
    dc.setFont(font);
    if(!isEnabled())
      dc.setForeground(makeShadowColor(list->getBackColor()));
    else if(isSelected())
      dc.setForeground(list->getSelTextColor());
    else
      dc.setForeground(list->getTextColor());
    dc.drawText(xx+(ww-tw)/2,yy+font->getFontAscent(), label);
  }
}



FXint FF_ListItem::getWidth(const FXList* list) const {
  register FXFont *font=list->getFont();
  register FXint iw=0, tw=0, fw=0;

  if (icon) {
    iw=icon->getHeight();
    if (fw<iw) fw=iw;
  }

  if (!label.empty()) {
    tw=font->getTextWidth(label);
    if (fw<tw) fw=tw;
  }

  return fw;
}



FXint FF_ListItem::getHeight(const FXList* list) const {
  register FXFont *font=list->getFont();
  register FXint ih=0, th=0, fh=0;

  if (icon) {
    ih=icon->getHeight();
    fh+=ih;
  }

  if (!label.empty()) {
    FXbool fhWasNull=(fh==0)?TRUE:FALSE;

    th=font->getFontHeight();
    fh+=th;
    if (!fhWasNull)
      fh+=ICON_SPACING;
  }

  return fh;
}

