/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_partneredit.hpp"
#include "ff_app.hpp"
#include "ff_selectpartner.hpp"

#include <aqfinance/engine/book/ae_partner.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_PartnerEdit) FF_PartnerEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_PartnerEdit::ID_COMBOEDIT, FF_PartnerEdit::onCmdCombo),
  FXMAPFUNC(SEL_COMMAND, FF_PartnerEdit::ID_BUTTON, FF_PartnerEdit::onCmdButton),
  FXMAPFUNC(SEL_DESELECTED, FF_PartnerEdit::ID_COMBOEDIT, FF_PartnerEdit::onCmdCancel),
  FXMAPFUNC(SEL_FOCUSIN, 0, FF_PartnerEdit::onFocusIn),
};


// Object implementation
FXIMPLEMENT(FF_PartnerEdit, FXHorizontalFrame, FF_PartnerEditMap, ARRAYNUMBER(FF_PartnerEditMap))



FF_PartnerEdit::FF_PartnerEdit(FXComposite *p,
                               FXObject* tgt,
                               FXSelector sel,
                               FXuint opts,
                               FXint x, FXint y, FXint w, FXint h,
                               FXint pl,
                               FXint pr,
                               FXint pt,
                               FXint pb)
:FXHorizontalFrame(p, opts, x, y, w, h, pl, pr, pt, pb)
,m_partnerList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
  FF_App *app;
  target=tgt;
  message=sel;

  app=dynamic_cast<FF_App*>(getApp());

  m_comboEdit=new FXXComboEdit(this, 1,
                               this, ID_COMBOEDIT,
                               LAYOUT_FILL_X);
  m_comboEdit->setSortFunc(FXList::ascendingCase);
  m_iconButton=app->getIcon("minifolderopen");
  if (m_iconButton) {
    m_button=new THEMEBUTTON(this, "", m_iconButton, this, ID_BUTTON);
  }
  else
    m_button=new THEMEBUTTON(this, "...", NULL, this, ID_BUTTON);
}



FF_PartnerEdit::FF_PartnerEdit()
:FXHorizontalFrame()
,m_partnerList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
}



FF_PartnerEdit::~FF_PartnerEdit() {
  AE_Partner_List_free(m_partnerList);
}



void FF_PartnerEdit::create() {
  FXHorizontalFrame::create();
  if (m_iconButton)
    m_iconButton->create();
  m_comboEdit->create();
  m_button->create();
}



void FF_PartnerEdit::setFont(FXFont* fnt) {
  m_comboEdit->setFont(fnt);
  m_button->setFont(fnt);
}



void FF_PartnerEdit::setBackColor(FXColor clr) {
  FXHorizontalFrame::setBackColor(clr);
  m_comboEdit->setBackColor(clr);
  m_button->setBackColor(clr);
}



void FF_PartnerEdit::setTextColor(FXColor clr) {
  m_comboEdit->setTextColor(clr);
  m_button->setTextColor(clr);
}



void FF_PartnerEdit::setSelBackColor(FXColor clr) {
  m_comboEdit->setSelBackColor(clr);
}



void FF_PartnerEdit::setSelTextColor(FXColor clr) {
  m_comboEdit->setSelTextColor(clr);
}



void FF_PartnerEdit::setText(const FXString& text) {
  m_comboEdit->setText(text);
}



FXString FF_PartnerEdit::getText() const {
  return m_comboEdit->getText();
}



void FF_PartnerEdit::selectAll() {
  m_comboEdit->selectAll();
}



void FF_PartnerEdit::setPartnerId(AQDB_ID id) {
  AE_PARTNER *pt;

  pt=AE_Partner_List_GetById(m_partnerList, id);
  if (pt) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AE_Partner_PathToBuffer(pt, tbuf);
    m_comboEdit->setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }
}



AQDB_ID FF_PartnerEdit::getPartnerId() {
  FXString str;

  str=m_comboEdit->getText();
  if (!str.empty()) {
    AE_PARTNER *pt;

    pt=AE_Partner_List_GetByPath(m_partnerList, str.text());
    if (pt)
      return AE_Partner_GetId(pt);
  }

  return 0;
}



long FF_PartnerEdit::onCmdCombo(FXObject*, FXSelector, void*) {
  if (target && target->handle(this, FXSEL(SEL_COMMAND, message), NULL))
    return 1;
  return 1;
}



long FF_PartnerEdit::onCmdButton(FXObject*, FXSelector, void*) {
  AQDB_ID id;
  AE_PARTNER *pt;

  /* TODO: get currently selected partner */
  id=0;

  pt=FF_SelectPartner::selectPartner(dynamic_cast<FF_App*>(getApp()),
				     this,
				     I18N("Select a Partner"));
  reload();
  if (pt) {
    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

    AE_Partner_PathToBuffer(pt, tbuf);
    setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
    if (target)
      target->handle(this, FXSEL(SEL_COMMAND, message), NULL);
  }

  return 1;
}



long FF_PartnerEdit::onFocusIn(FXObject *sender, FXSelector sel, void *ptr){
  FXHorizontalFrame::onFocusIn(sender, sel, ptr);
  m_comboEdit->setFocus();
  return 1;
}



long FF_PartnerEdit::onCmdCancel(FXObject *sender, FXSelector sel, void *ptr){
  if (target && target->handle(this, FXSEL(SEL_DESELECTED, message), NULL))
    return 1;

  return 0;
}



void FF_PartnerEdit::reload() {
  FF_App *app;
  int rv;

  m_comboEdit->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  AE_Partner_List_free(m_partnerList);
  m_partnerList=AE_Partner_List_new();
  rv=AE_Book_ReadPartnerList(app->getBook(), AE_Book_TableType_Partner, m_partnerList);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else {
    AE_PARTNER *pt;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    pt=AE_Partner_List_First(m_partnerList);
    while(pt) {
      AE_Partner_PathToBuffer(pt, tbuf);
      m_comboEdit->addEntry(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
      pt=AE_Partner_List_Next(pt);
    }
    GWEN_Buffer_free(tbuf);
    m_comboEdit->sortItems();
    m_comboEdit->reloadList();
  }
}






