/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_PARTNER_HPP
#define FF_PARTNER_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_partner.h>



class FF_PartnerTreeItem: public FF_GroupTreeItem {
public:
  FF_PartnerTreeItem(AE_PARTNER *ba,
		     FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_PartnerTreeItem();

  AE_PARTNER *getPartner() const { return m_partner;};
  void rebuild();

protected:
  AE_PARTNER *m_partner;
};



class FF_PartnerTree: public FF_GroupTree {

public:
  FF_PartnerTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
		 FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_PartnerTree();

  FF_PartnerTreeItem *findPartnerItem(AQDB_ID id);
  FF_PartnerTreeItem *findPartnerItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_PartnerTreeItem *findPartnerItem(const AE_PARTNER *pt);

  void addPartner(const AE_PARTNER *pt);
  void addPartner(AE_PARTNER *pt);

  AE_PARTNER *getCurrentPartner();
  AE_PARTNER *getFirstSelectedPartner();
  void getSelectedPartners(AE_PARTNER_LIST2 *pl);

  void getPartners(AE_PARTNER_LIST2 *pl, int copyMode);

  void selectPartner(AQDB_ID id, bool b);
  void selectPartner(const AE_PARTNER *pt, bool b);
  void makePartnerVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);

protected:
  FF_PartnerTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

};



#endif




