/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_printdoctree.hpp"

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/text.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FF_PrintDocTreeItem::FF_PrintDocTreeItem(AF_PRINT_DOC *doc,
					 FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_doc(doc) {
  AF_PrintDoc_Attach(doc);

  rebuild();
}



FF_PrintDocTreeItem::~FF_PrintDocTreeItem() {
  if (m_isDataItem && m_doc) {
    AF_PrintDoc_free(m_doc);
    m_doc=(AF_PRINT_DOC*) -1;
  }
}



void FF_PrintDocTreeItem::rebuild() {
  FXString t;
  const char *s;

  /* name */
  s=AF_PrintDoc_GetName(m_doc);
  if (s)
    t+=s;
  //t+="\t";

  setText(t);
}



FF_PrintDocTree::FF_PrintDocTree(FXComposite *p,
				 FXObject* tgt, FXSelector sel,
				 FXuint opts,
				 FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Name"));
  setHeaderSize(idx++, 100);
}



FF_PrintDocTree::~FF_PrintDocTree() {
}



FF_PrintDocTreeItem *FF_PrintDocTree::findDocItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_PrintDocTreeItem *res=findDocItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_PrintDocTreeItem *FF_PrintDocTree::findDocItem(FF_GroupTreeItem *grItem,
						  AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_PrintDocTreeItem *jItem=dynamic_cast<FF_PrintDocTreeItem*>(grItem);
    if (jItem) {
      AF_PRINT_DOC *doc=jItem->getDoc();
      if (AF_PrintDoc_GetId(doc)==id)
	return jItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_PrintDocTreeItem *res=findDocItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_PrintDocTreeItem *FF_PrintDocTree::findDocItem(const char *category, const char *name) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      if (grItem->isDataItem()) {
	FF_PrintDocTreeItem *jItem=dynamic_cast<FF_PrintDocTreeItem*>(grItem);
	if (jItem) {
	  AF_PRINT_DOC *doc;
	  const char *s1;
	  const char *s2;

	  doc=jItem->getDoc();
	  s1=AF_PrintDoc_GetCategory(doc);
	  s2=AF_PrintDoc_GetName(doc);
	  if (s1 && s2 && strcasecmp(s1, category)==0 && strcasecmp(s2, name)==0)
	    return jItem;
	}
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



FF_PrintDocTreeItem *FF_PrintDocTree::findDocItem(const char *name) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      if (grItem->isDataItem()) {
	FF_PrintDocTreeItem *jItem=dynamic_cast<FF_PrintDocTreeItem*>(grItem);
	if (jItem) {
	  AF_PRINT_DOC *doc;
	  const char *s2;

	  doc=jItem->getDoc();
	  s2=AF_PrintDoc_GetName(doc);
	  if (s2 && strcasecmp(s2, name)==0)
	    return jItem;
	}
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_PrintDocTree::addDoc(AF_PRINT_DOC *doc, FXbool notify) {
  FF_GroupTreeItem *itemFather=NULL;
  FF_PrintDocTreeItem *item;
  const char *s;

  s=AF_PrintDoc_GetCategory(doc);
  if (s && *s) {
    itemFather=findGroupItemByName(s);
    if (itemFather==NULL) {
      AE_GROUP *gr;

      /* no group for gategory, create it */
      gr=AE_Group_new();
      AE_Group_SetName(gr, s);
      itemFather=new FF_GroupTreeItem(gr, m_iconOpen, m_iconClosed);
      appendItem(NULL, itemFather, false);
    }
  }

  item=new FF_PrintDocTreeItem(doc,
			       /*m_iconOpen, m_iconClosed*/
			       NULL, NULL); /* no icons here */
  appendItem(itemFather, item, notify);
}



FF_PrintDocTree::FF_PrintDocTree(): FF_GroupTree() {
}



AF_PRINT_DOC *FF_PrintDocTree::getCurrentDoc() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_PrintDocTreeItem *jItem;

      jItem=dynamic_cast<FF_PrintDocTreeItem*>(grItem);
      if (jItem)
        return jItem->getDoc();
    }
  }

  return NULL;
}



AF_PRINT_DOC *FF_PrintDocTree::getFirstSelectedDoc() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_PrintDocTreeItem *jItem;

        jItem=dynamic_cast<FF_PrintDocTreeItem*>(grItem);
        if (jItem)
          return jItem->getDoc();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_PrintDocTree::getSelectedDocs(AF_PRINT_DOC_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_PrintDocTreeItem *jItem=dynamic_cast<FF_PrintDocTreeItem*>(grItem);
        if (jItem)
          AF_PrintDoc_List2_PushBack(al, jItem->getDoc());
      }
    }
    item=item->getBelow();
  }
}



void FF_PrintDocTree::selectDoc(AQDB_ID id, bool b) {
  FF_PrintDocTreeItem *item;

  item=findDocItem(id);
  if (item)
    item->setSelected(b);
}



void FF_PrintDocTree::selectDoc(const char *category, const char *name, bool b) {
  FF_PrintDocTreeItem *item;

  item=findDocItem(category, name);
  if (item)
    item->setSelected(b);
}



void FF_PrintDocTree::makeDocVisible(AQDB_ID id) {
  FF_PrintDocTreeItem *item;

  item=findDocItem(id);
  if (item)
    makeItemVisible(item);
}



void FF_PrintDocTree::makeDocVisible(const char *category, const char *name) {
  FF_PrintDocTreeItem *item;

  item=findDocItem(category, name);
  if (item)
    makeItemVisible(item);
}






