/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_recurrenceedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_recurrence.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_RecurrenceEdit) FF_RecurrenceEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_RecurrenceEdit::ID_PERIOD_COMBO, FF_RecurrenceEdit::onCmdPeriodCombo)
};


// Object implementation
FXIMPLEMENT(FF_RecurrenceEdit, FXHorizontalFrame, FF_RecurrenceEditMap, ARRAYNUMBER(FF_RecurrenceEditMap))



FF_RecurrenceEdit::FF_RecurrenceEdit(FXComposite *p,
                                     FXObject* tgt,
                                     FXSelector sel,
                                     FXuint opts,
                                     FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h, 0l, 0, 0, 0)
,m_everyCombo(NULL)
,m_periodCombo(NULL)
,m_onCombo(NULL)
,m_periodLabel(NULL)
,m_comboPosPeriodDay(0)
,m_comboPosPeriodWeek(0)
,m_comboPosPeriodMonth(0)
,m_comboPosPeriodYear(0)
,m_comboPosOnLastOfMonth(0)
,m_bankingLimits(NULL)
{
  target=tgt;
  message=sel;

  new FXLabel(this, I18N("FF_RecurrenceEdit|Period:"), NULL, LABEL_NORMAL);
  m_periodCombo=new THEMECOMBOBOX(this, 1,
                                  this, ID_PERIOD_COMBO,
                                  COMBOBOX_STATIC | LAYOUT_FILL_X);

  new FXLabel(this, I18N("FF_RecurrenceEdit|Every"), NULL, LABEL_NORMAL);

  m_everyCombo=new THEMECOMBOBOX(this, 4,
				 this, ID_EVERY_COMBO,
                                 COMBOBOX_STATIC);

  m_periodLabel=new FXLabel(this, "", NULL, LABEL_NORMAL);


  new FXLabel(this, I18N("FF_RecurrenceEdit|on"), NULL, LABEL_NORMAL);
  m_onCombo=new THEMECOMBOBOX(this, 1,
			      this, ID_ON_COMBO,
                              COMBOBOX_STATIC | LAYOUT_FILL_X);

}



FF_RecurrenceEdit::FF_RecurrenceEdit()
:FXHorizontalFrame()
,m_everyCombo(NULL)
,m_periodCombo(NULL)
,m_onCombo(NULL)
,m_periodLabel(NULL)
,m_comboPosPeriodDay(0)
,m_comboPosPeriodWeek(0)
,m_comboPosPeriodMonth(0)
,m_comboPosPeriodYear(0)
,m_comboPosOnLastOfMonth(0)
,m_bankingLimits(NULL)
{
}



FF_RecurrenceEdit::~FF_RecurrenceEdit() {
  AB_TransactionLimits_free(m_bankingLimits);
  m_bankingLimits=(AB_TRANSACTION_LIMITS*) -1;
}



void FF_RecurrenceEdit::create() {
  FXHorizontalFrame::create();
}



void FF_RecurrenceEdit::init(GWEN_DB_NODE *db) {
}



void FF_RecurrenceEdit::fini(GWEN_DB_NODE *db) {
}



void FF_RecurrenceEdit::toGui(const AE_RECURRENCE *rec) {
  updatePeriodCombo();
  if (rec) {
    FXString str;
    int i;

    switch(AE_Recurrence_GetPeriod(rec)) {
    case AE_Recurrence_PeriodDays:
      if (m_comboPosPeriodDay>0)
        m_periodCombo->setCurrentItem(m_comboPosPeriodDay, true);
      break;
    case AE_Recurrence_PeriodWeeks:
      if (m_comboPosPeriodWeek>0)
        m_periodCombo->setCurrentItem(m_comboPosPeriodWeek, true);
      break;
    case AE_Recurrence_PeriodMonths:
      if (m_comboPosPeriodMonth>0)
        m_periodCombo->setCurrentItem(m_comboPosPeriodMonth, true);
      break;
    case AE_Recurrence_PeriodYears:
      if (m_comboPosPeriodYear>0)
        m_periodCombo->setCurrentItem(m_comboPosPeriodYear, true);
      break;

    default:
      m_periodCombo->setCurrentItem(0, true);
      break;
    }

    updateOnCombo();
    updateEveryCombo();

    str=FXStringFormat("%02d", AE_Recurrence_GetEvery(rec));
    i=m_everyCombo->findItem(str);
    if (i)
      m_everyCombo->setCurrentItem(i);

    if (AE_Recurrence_GetPeriod(rec)==AE_Recurrence_PeriodWeeks) {
      /* 1=Monday, 2=Tuesday, ..., 7=Sunday */
      i=AE_Recurrence_GetOn(rec);
      if (i>0)
        m_onCombo->setCurrentItem(i);
    }
    else if (AE_Recurrence_GetPeriod(rec)==AE_Recurrence_PeriodMonths) {
      str=FXStringFormat("%02d", AE_Recurrence_GetOn(rec));
      i=m_onCombo->findItem(str);
      if (i)
        m_onCombo->setCurrentItem(i);
    }
  }
  else {
    updateOnCombo();
    updateEveryCombo();
  }
}



int FF_RecurrenceEdit::checkGui(bool quiet) {
  FXint ci;
  int period;
  FXString str;

  ci=m_periodCombo->getCurrentItem();
  if (ci<1) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please select the period type"));
      m_periodCombo->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }

  switch(ci) {
  case 1:  period=AE_Recurrence_PeriodDays; break;
  case 2:  period=AE_Recurrence_PeriodWeeks; break;
  case 3:  period=AE_Recurrence_PeriodMonths; break;
  case 4:  period=AE_Recurrence_PeriodYears; break;
  default: period=AE_Recurrence_PeriodUnknown; break;
  }

  ci=m_everyCombo->getCurrentItem();
  if (ci<1) {
    str=m_everyCombo->getText().simplify();
    if (str.empty())
      ci=FXIntVal(str, 10);
  }
  if (ci<1) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
			  I18N("Missing interval"));
      m_everyCombo->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (period==AE_Recurrence_PeriodWeeks ||
      period==AE_Recurrence_PeriodMonths) {
    ci=m_onCombo->getCurrentItem();
    if (ci<1) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
                            "%s",
                            I18N("Please select the execution day"));
        m_onCombo->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
  }

  return 0;
}



void FF_RecurrenceEdit::fromGui(AE_RECURRENCE *rec) {
  FXint ci;
  int period;
  int ev;
  FXString str;

  ci=m_periodCombo->getCurrentItem();

  if (m_comboPosPeriodDay!=-1 && ci==m_comboPosPeriodDay)
    period=AE_Recurrence_PeriodDays;
  else if (m_comboPosPeriodWeek!=-1 && ci==m_comboPosPeriodWeek)
    period=AE_Recurrence_PeriodWeeks;
  else if (m_comboPosPeriodMonth!=-1 && ci==m_comboPosPeriodMonth)
    period=AE_Recurrence_PeriodMonths;
  else if (m_comboPosPeriodYear!=-1 && ci==m_comboPosPeriodYear)
    period=AE_Recurrence_PeriodYears;
  else
    period=AE_Recurrence_PeriodUnknown;

  AE_Recurrence_SetPeriod(rec, period);

  ci=m_everyCombo->getCurrentItem();

  ev=FXIntVal(m_everyCombo->getText());
  AE_Recurrence_SetEvery(rec, ev);

  if (period==AE_Recurrence_PeriodWeeks) {
    AE_Recurrence_SetOn(rec, m_onCombo->getCurrentItem());
  }
  else if (period==AE_Recurrence_PeriodMonths) {
    int o;

    ci=m_onCombo->getCurrentItem();
    assert(ci>0);

    o=FXIntVal(m_onCombo->getText());
    AE_Recurrence_SetOn(rec, o);
  }
}



void FF_RecurrenceEdit::updateEveryCombo() {
  FXString prevVal;

  prevVal=m_everyCombo->getText();
  _updateEveryCombo();
  if (!prevVal.empty()) {
    FXint ci;

    ci=m_everyCombo->findItem(prevVal);
    if (ci>0)
      m_everyCombo->setCurrentItem(ci, false);
  }
}



void FF_RecurrenceEdit::_updateEveryCombo() {
  FXint ci;

  m_everyCombo->clearItems();
  m_everyCombo->appendItem(I18N("FF_RecurrenceEdit|(select)"));

  if (m_bankingLimits) {
    ci=m_periodCombo->getCurrentItem();
    if (m_comboPosPeriodWeek>0 && ci==m_comboPosPeriodWeek &&
	AB_TransactionLimits_GetAllowWeekly(m_bankingLimits)!=-1) {
      const GWEN_STRINGLIST *csl;

      csl=AB_TransactionLimits_GetValuesCycleWeek(m_bankingLimits);
      if (csl) {
	GWEN_STRINGLISTENTRY *se;
	GWEN_STRINGLIST *sl;

	sl=GWEN_StringList_dup(csl);
	GWEN_StringList_Sort(sl, 1, GWEN_StringList_SortModeInt);
	se=GWEN_StringList_FirstEntry(sl);
	while(se) {
	  const char *s;

	  s=GWEN_StringListEntry_Data(se);
	  if (s && *s) {
	    FXint i;
	    FXString str;

	    i=FXIntVal(s, 10);
	    str=FXStringFormat("%02d", i);
	    m_everyCombo->appendItem(str);
	  }
	  se=GWEN_StringListEntry_Next(se);
	}
	GWEN_StringList_free(sl);

	m_everyCombo->setCurrentItem(0);
	ci=m_everyCombo->getNumItems();
	if (ci>15)
	  ci=15;
	m_everyCombo->setNumVisible(ci);
        m_everyCombo->enable();
	return;
      }
    }
    else if (m_comboPosPeriodMonth>0 && ci==m_comboPosPeriodMonth &&
	     AB_TransactionLimits_GetAllowMonthly(m_bankingLimits)!=-1) {
      const GWEN_STRINGLIST *csl;

      csl=AB_TransactionLimits_GetValuesCycleMonth(m_bankingLimits);
      if (csl) {
	GWEN_STRINGLISTENTRY *se;
	GWEN_STRINGLIST *sl;

	sl=GWEN_StringList_dup(csl);
	GWEN_StringList_Sort(sl, 1, GWEN_StringList_SortModeInt);
	se=GWEN_StringList_FirstEntry(sl);
	while(se) {
	  const char *s;

	  s=GWEN_StringListEntry_Data(se);
	  if (s && *s) {
	    FXint i;
	    FXString str;

	    i=FXIntVal(s, 10);
	    str=FXStringFormat("%02d", i);
	    m_everyCombo->appendItem(str);
	  }
	  se=GWEN_StringListEntry_Next(se);
	}
	GWEN_StringList_free(sl);

	m_everyCombo->setCurrentItem(0);
	ci=m_everyCombo->getNumItems();
	if (ci>15)
	  ci=15;
	m_everyCombo->setNumVisible(ci);
        m_everyCombo->enable();
	return;
      }
    }
  }

  /* fallback */
  ci=m_periodCombo->getCurrentItem();
  if (m_comboPosPeriodWeek>0 && ci==m_comboPosPeriodWeek) {
    FXint i;

    /* weeks */
    m_everyCombo->enable();
    for (i=1; i<53; i++) {
      FXString str;

      str=FXStringFormat("%02d", i);
      m_everyCombo->appendItem(str);
    }
    m_everyCombo->setCurrentItem(0);
  }
  else if (m_comboPosPeriodMonth>0 && ci==m_comboPosPeriodMonth) {
    FXint i;

    /* months */
    m_everyCombo->enable();
    for (i=1; i<12; i++) {
      FXString str;

      str=FXStringFormat("%02d", i);
      m_everyCombo->appendItem(str);
    }
    m_everyCombo->setCurrentItem(0);
  }
  else if (m_comboPosPeriodDay>0 && ci==m_comboPosPeriodDay) {
    FXint i;

    /* days */
    m_everyCombo->enable();
    for (i=1; i<14; i++) {
      FXString str;

      str=FXStringFormat("%02d", i);
      m_everyCombo->appendItem(str);
    }
    m_everyCombo->setCurrentItem(0);
  }
  else {
    m_everyCombo->disable();
  }

  m_everyCombo->setCurrentItem(0);
  ci=m_everyCombo->getNumItems();
  if (ci>15)
    ci=15;
  m_everyCombo->setNumVisible(ci);
}



void FF_RecurrenceEdit::updateOnCombo() {
  FXString prevVal;

  prevVal=m_onCombo->getText();
  _updateOnCombo();
  if (!prevVal.empty()) {
    FXint ci;

    ci=m_onCombo->findItem(prevVal);
    if (ci>0)
      m_onCombo->setCurrentItem(ci, false);
  }
}



void FF_RecurrenceEdit::_updateOnCombo() {
  FXint ci;

  m_onCombo->clearItems();
  m_onCombo->appendItem(I18N("FF_RecurrenceEdit|(select)"));
  m_comboPosOnLastOfMonth=0;

  if (m_bankingLimits) {
    ci=m_periodCombo->getCurrentItem();
    DBG_ERROR(0, "PeriodCombo: %d", ci);
    if (m_comboPosPeriodWeek>0 && ci==m_comboPosPeriodWeek &&
        AB_TransactionLimits_GetAllowWeekly(m_bankingLimits)!=-1) {
      const GWEN_STRINGLIST *csl;

      DBG_ERROR(0, "Weekly, allowed");
      csl=AB_TransactionLimits_GetValuesExecutionDayWeek(m_bankingLimits);
      if (csl) {
	GWEN_STRINGLISTENTRY *se;
	GWEN_STRINGLIST *sl;

	sl=GWEN_StringList_dup(csl);
	GWEN_StringList_Sort(sl, 1, GWEN_StringList_SortModeInt);
        se=GWEN_StringList_FirstEntry(sl);
	while(se) {
	  const char *s;

	  s=GWEN_StringListEntry_Data(se);
	  if (s && *s) {
	    FXint i;

            DBG_ERROR(0, " WeekDay: %s", s);
	    i=FXIntVal(s, 10);
	    if (i==0) {
	      m_onCombo->appendItem(I18N("Monday"));
	      m_onCombo->appendItem(I18N("Tuesday"));
	      m_onCombo->appendItem(I18N("Wednesday"));
	      m_onCombo->appendItem(I18N("Thursday"));
	      m_onCombo->appendItem(I18N("Friday"));
	      m_onCombo->appendItem(I18N("Saturday"));
	      m_onCombo->appendItem(I18N("Sunday"));
	    }
	    else {
	      switch(i) {
	      case 1:
		m_onCombo->appendItem(I18N("Monday"));
		break;
	      case 2:
		m_onCombo->appendItem(I18N("Tuesday"));
		break;
	      case 3:
		m_onCombo->appendItem(I18N("Wednesday"));
		break;
	      case 4:
		m_onCombo->appendItem(I18N("Thursday"));
		break;
	      case 5:
		m_onCombo->appendItem(I18N("Friday"));
		break;
	      case 6:
		m_onCombo->appendItem(I18N("Saturday"));
		break;
	      case 7:
		m_onCombo->appendItem(I18N("Sunday"));
		break;
	      }
	    }
	  }
	  se=GWEN_StringListEntry_Next(se);
	}
	GWEN_StringList_free(sl);

	m_onCombo->setCurrentItem(0);
	ci=m_onCombo->getNumItems();
	if (ci>15)
	  ci=15;
	m_onCombo->setNumVisible(ci);
        m_onCombo->enable();
	return;
      }
    }
    else if (m_comboPosPeriodMonth>0 && ci==m_comboPosPeriodMonth &&
             AB_TransactionLimits_GetAllowMonthly(m_bankingLimits)!=-1) {
      const GWEN_STRINGLIST *csl;

      csl=AB_TransactionLimits_GetValuesExecutionDayMonth(m_bankingLimits);
      if (csl) {
	GWEN_STRINGLISTENTRY *se;
        GWEN_STRINGLIST *sl;

	sl=GWEN_StringList_dup(csl);
	GWEN_StringList_Sort(sl, 1, GWEN_StringList_SortModeInt);
	se=GWEN_StringList_FirstEntry(sl);
	while(se) {
	  const char *s;

	  s=GWEN_StringListEntry_Data(se);
	  if (s && *s) {
	    FXint i;
	    FXString str;

            i=FXIntVal(s, 10);
            DBG_ERROR(0, "  Adding value %d [%s]", i, s);
            if (i==99) {
#if 0
              m_comboPosOnLastOfMonth=m_onCombo->getNumItems();
              m_onCombo->appendItem(I18N("FF_RecurrenceEdit|Last Day of Month"));
#endif
            }
	    else if (i==98) {
	    }
	    else {
	      str=FXStringFormat("%02d", i);
	      m_onCombo->appendItem(str);
            }
          }
	  se=GWEN_StringListEntry_Next(se);
	}
	GWEN_StringList_free(sl);

	m_onCombo->setCurrentItem(0);
	ci=m_onCombo->getNumItems();
	if (ci>15)
	  ci=15;
	m_onCombo->setNumVisible(ci);
        m_onCombo->enable();
	return;
      }
    }
    else {
      m_onCombo->disable();
    }
  }
  else {
    DBG_ERROR(0, "No banking limits");
  }

  /* fallback */
  ci=m_periodCombo->getCurrentItem();
  if (m_comboPosPeriodWeek>0 && ci==m_comboPosPeriodWeek) {
    /* weeks */
    m_onCombo->enable();
    m_onCombo->appendItem(I18N("Monday"));
    m_onCombo->appendItem(I18N("Tuesday"));
    m_onCombo->appendItem(I18N("Wednesday"));
    m_onCombo->appendItem(I18N("Thursday"));
    m_onCombo->appendItem(I18N("Friday"));
    m_onCombo->appendItem(I18N("Saturday"));
    m_onCombo->appendItem(I18N("Sunday"));
    m_onCombo->setCurrentItem(0);
  }
  else if (m_comboPosPeriodMonth>0 && ci==m_comboPosPeriodMonth) {
    FXint i;

    /* months */
    m_onCombo->enable();
    for (i=1; i<31; i++) {
      FXString str;

      str=FXStringFormat("%02d", i);
      m_onCombo->appendItem(str);
    }
    m_onCombo->appendItem(I18N("FF_RecurrenceEdit|Last Day of Month"));
    m_onCombo->setCurrentItem(0);
  }
  else {
    m_onCombo->disable();
  }

  m_onCombo->setCurrentItem(0);
  ci=m_onCombo->getNumItems();
  if (ci>15)
    ci=15;
  m_onCombo->setNumVisible(ci);
}



void FF_RecurrenceEdit::updatePeriodCombo() {
  FXString prevVal;

  prevVal=m_periodCombo->getText();
  _updatePeriodCombo();
  if (!prevVal.empty()) {
    FXint ci;

    ci=m_periodCombo->findItem(prevVal);
    if (ci>0)
      m_periodCombo->setCurrentItem(ci, false);
  }
}



void FF_RecurrenceEdit::_updatePeriodCombo() {
  m_periodCombo->clearItems();
  m_comboPosPeriodDay=0;
  m_comboPosPeriodWeek=0;
  m_comboPosPeriodMonth=0;
  m_comboPosPeriodYear=0;
  m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|(select)"));

  if (m_bankingLimits) {
    int idx=1;

    if (AB_TransactionLimits_GetAllowWeekly(m_bankingLimits)!=-1) {
      m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|week(s)"));
      m_comboPosPeriodWeek=idx++;
    }
    if (AB_TransactionLimits_GetAllowMonthly(m_bankingLimits)!=-1) {
      m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|month(s)"));
      m_comboPosPeriodMonth=idx++;
    }
  }
  else {
    int idx=1;

    m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|day(s)"));
    m_comboPosPeriodDay=idx++;
    m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|week(s)"));
    m_comboPosPeriodWeek=idx++;
    m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|month(s)"));
    m_comboPosPeriodMonth=idx++;
    m_periodCombo->appendItem(I18N("FF_RecurrenceEdit|year(s)"));
    m_comboPosPeriodYear=idx++;
  }

  m_periodCombo->setNumVisible(m_periodCombo->getNumItems());
  m_periodCombo->setCurrentItem(0);
}



long FF_RecurrenceEdit::onCmdPeriodCombo(FXObject*, FXSelector, void*) {
  FXint ci;

  DBG_ERROR(0, "PeriodCombo changed, updating onCombo and everyCombo");
  ci=m_periodCombo->getCurrentItem();
  if (m_comboPosPeriodDay>0 && ci==m_comboPosPeriodDay)
    m_periodLabel->setText(I18N("FF_RecurrenceEdit|day(s)"));
  else if (m_comboPosPeriodWeek>0 && ci==m_comboPosPeriodWeek)
    m_periodLabel->setText(I18N("FF_RecurrenceEdit|week(s)"));
  else if (m_comboPosPeriodMonth>0 && ci==m_comboPosPeriodMonth)
    m_periodLabel->setText(I18N("FF_RecurrenceEdit|month(s)"));
  else if (m_comboPosPeriodYear>0 && ci==m_comboPosPeriodYear)
    m_periodLabel->setText(I18N("FF_RecurrenceEdit|year(s)"));

  updateOnCombo();
  updateEveryCombo();
  return 1;
}



void FF_RecurrenceEdit::setBankingLimits(const AB_TRANSACTION_LIMITS *tl, bool withChangeLimits) {
  AB_TransactionLimits_free(m_bankingLimits);
  m_bankingLimits=AB_TransactionLimits_dup(tl);

  updatePeriodCombo();
  updateOnCombo();
  updateEveryCombo();

  if (withChangeLimits) {
    if (AB_TransactionLimits_GetAllowChangeCycle(tl)==-1)
      m_everyCombo->disable();
    else
      m_everyCombo->enable();

    if (AB_TransactionLimits_GetAllowChangePeriod(tl)==-1)
      m_periodCombo->disable();
    else
      m_periodCombo->enable();

    if (AB_TransactionLimits_GetAllowChangeExecutionDay(tl)==-1)
      m_onCombo->disable();
    else
      m_onCombo->enable();
  }
}






