/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_REFUELINGTREE_HPP
#define FF_REFUELINGTREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_refueling.h>
#include <aqfinance/engine/book/ae_commodity.h>

#include <aqbanking/job.h>



class FF_RefuelingTreeItem: public FF_GroupTreeItem {
public:
  FF_RefuelingTreeItem(AE_REFUELING *st, void* ptr=NULL);

  virtual ~FF_RefuelingTreeItem();

  AE_REFUELING *getRefueling() const { return m_refueling;};

  virtual FXString getItemText(const FXXCheckTree* list) const;

protected:
  AE_REFUELING *m_refueling;
};



class FF_RefuelingTree: public FF_GroupTree {
  friend class FF_RefuelingTreeItem;
public:
  enum {
    USECOLOR_NORMAL=0,
    USECOLOR_NOTICE,
    USECOLOR_WARNING
  };

  enum {
    COLUMN_NONE=0,
    COLUMN_PETROLSTATION=1,
    COLUMN_FUELAMOUNT,
    COLUMN_FUELPRICE,
    COLUMN_MILEAGE,
    COLUMN_DATE,
    COLUMN_MEMO,
    COLUMN_FLAGS,
    COLUMN_MAX=100
  };

  FF_RefuelingTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                   FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_RefuelingTree();

  void addColumn(int c, int w=100);

  FF_RefuelingTreeItem *findRefuelingItem(AQDB_ID id);
  FF_RefuelingTreeItem *findRefuelingItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_RefuelingTreeItem *findRefuelingItem(const AE_REFUELING *qba);

  void addRefueling(AE_REFUELING *st, bool direct, int useColor);
  void addRefuelings(AE_REFUELING_LIST2 *sl, int copyMode, bool direct);

  AE_REFUELING *getCurrentRefueling();
  AE_REFUELING *getFirstSelectedRefueling();
  void getSelectedRefuelings(AE_REFUELING_LIST2 *sl, int copyMode);

  void selectRefueling(AQDB_ID id, bool b);
  void selectRefueling(const AE_REFUELING *st, bool b);
  void makeRefuelingVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);
  void addColumn(int c);

  void reload();

protected:
  FF_RefuelingTree();

  static FXint sortFunctionUpPetrolStation(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownPetrolStation(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpMemo(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownMemo(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpFuelAmount(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownFuelAmount(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpFuelPrice(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownFuelPrice(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpMileage(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownMileage(const FXFoldingItem *it1, const FXFoldingItem *it2);

  int m_columns;
  int m_columnTypes[COLUMN_MAX];

  AE_COMMODITY_LIST *m_commodityList;

};



#endif




