/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_REPORT2_SETTINGS_HPP
#define FF_REPORT2_SETTINGS_HPP


#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/report2/af_report2_settings.h>



class FF_Report2SettingsTreeItem: public FF_GroupTreeItem {
public:
  FF_Report2SettingsTreeItem(AF_REPORT2_SETTINGS *ba,
		     FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_Report2SettingsTreeItem();

  AF_REPORT2_SETTINGS *getSettings() const { return m_settings;};
  void rebuild();

protected:
  AF_REPORT2_SETTINGS *m_settings;
};



class FF_Report2SettingsTree: public FF_GroupTree {

public:
  FF_Report2SettingsTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                         FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_Report2SettingsTree();

  FF_Report2SettingsTreeItem *findSettingsItem(AQDB_ID id);
  FF_Report2SettingsTreeItem *findSettingsItem(const char *id);
  FF_Report2SettingsTreeItem *findSettingsItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_Report2SettingsTreeItem *findSettingsItem(const AF_REPORT2_SETTINGS *settings);

  void addSettings(const AF_REPORT2_SETTINGS *settings);
  void addSettings(AF_REPORT2_SETTINGS *settings);

  AF_REPORT2_SETTINGS *getCurrentSettings();
  AF_REPORT2_SETTINGS *getFirstSelectedSettings();
  void getSelectedSettings(AF_REPORT2_SETTINGS_LIST2 *pl);

  void getSettings(AF_REPORT2_SETTINGS_LIST2 *pl, int copyMode);

  void selectSettings(const char *name, bool b);
  void selectSettings(const AF_REPORT2_SETTINGS *settings, bool b);
  void makeSettingsVisible(const char *name);

  void create();

  void sortByColumn(int c, bool up);

protected:
  FF_Report2SettingsTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

};



#endif




