/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_SPLITTABLE_HPP
#define FF_SPLITTABLE_HPP


#include <aqfoxext/fxx_table.hpp>

#include <fx.h>

#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_split.h>
#include <aqfinance/engine/book/ae_partner.h>
#include <aqfinance/engine/book/ae_commodity.h>

#include <list>



class FF_SplitTable: public FXXTable {
  FXDECLARE(FF_SplitTable)

public:
  enum {
    MAX_COLUMNS=32
  };

  enum {
    COLUMN_TYPE_NONE=0,
    COLUMN_TYPE_MEMO,
    COLUMN_TYPE_ACCOUNT,
    COLUMN_TYPE_VALUE,
    COLUMN_TYPE_CREDIT_VALUE,
    COLUMN_TYPE_DEBIT_VALUE
  };

  enum {
    ID_TEXTFIELD=FXXTable::ID_LAST,
    ID_ACCOUNT_EDIT,
    ID_AMOUNT_EDIT,
    ID_LAST
  };


public:
  FF_SplitTable(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
		FXint x=0, FXint y=0, FXint w=0, FXint h=0,
		FXint pl=DEFAULT_MARGIN, FXint pr=DEFAULT_MARGIN,
		FXint pt=DEFAULT_MARGIN, FXint pb=DEFAULT_MARGIN);

  virtual ~FF_SplitTable();

  void addColumn(int coltype, int size=DEFAULT_COLUMN_WIDTH);

  void addSplit(AE_SPLIT *st);
  void addSplits(AE_SPLIT_LIST2 *sl, int copyMode);

  void getSelectedSplits(AE_SPLIT_LIST2 *sl, int copyMode);

  void delRow(int r);


  AE_SPLIT *getSplitByRow(int r);

  AE_SPLIT *getCurrentSplit();

  AQDB_ID getMarkBeyondId() const { return m_markBeyondId;};
  void setMarkBeyondId(AQDB_ID id) { m_markBeyondId=id;};

  bool getColorizeMatches() const { return m_colorizeMatches;};
  void setColorizeMatches(bool b);

  virtual void setNumberOfRows(int i);

  virtual void clearItems();

  virtual void sortByColumn(int idx, bool up);

  void reload();

  long onCmdAccount(FXObject*, FXSelector, void* ptr);
  long onCancelAccount(FXObject*, FXSelector, void* ptr);

  long onCmdAmount(FXObject*, FXSelector, void* ptr);
  long onCancelAmount(FXObject*, FXSelector, void* ptr);

  long onCmdTextField(FXObject*, FXSelector, void* ptr);
  long onCancelTextField(FXObject*, FXSelector, void* ptr);

protected:
  FF_SplitTable();

  virtual ColumnInfo *getColumnInfo(int r, int c);
  virtual void releaseColumnInfo(ColumnInfo *ci);

  virtual FXWindow *getEditor(int r, int c);
  virtual void setFromEditor(ColumnInfo *ci, FXWindow *ed);
  virtual void releaseEditor(int r, int c, FXWindow *ed);

  void addSpaceForSplits(int n);

  static int compareDateUp(const void *a, const void *b);
  static int compareDateDown(const void *a, const void *b);


  AE_SPLIT **m_splitArray;
  int m_splitArraySize;
  int m_splitCount;

  AQDB_ID m_markBeyondId;
  bool m_colorizeMatches;

  AE_ACCOUNT_TREE *m_accountTree;
  AE_COMMODITY_LIST *m_commodityList;

  int m_columnTypes[MAX_COLUMNS];
};



#endif


