/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_STATEMENTTABLE_HPP
#define FF_STATEMENTTABLE_HPP


#include <aqfoxext/fxx_table.hpp>

#include <fx.h>

#include <aqfinance/engine/book/ae_statement.h>
#include <aqfinance/engine/book/ae_partner.h>

#include <list>



class FF_StatementTable: public FXXTable {
  FXDECLARE(FF_StatementTable)

public:
  enum {
    MAX_COLUMNS=32
  };

  enum {
    COLUMN_TYPE_NONE=0,
    COLUMN_TYPE_DATE=1,
    COLUMN_TYPE_MEMO,
    COLUMN_TYPE_TYPE,
    COLUMN_TYPE_VALUE,
    COLUMN_TYPE_PARTNER
  };

  enum {
    ID_PARTNER_EDIT=FXXTable::ID_LAST,
    ID_DATE_EDIT,
    ID_TEXTFIELD,
    ID_LAST
  };


public:
  FF_StatementTable(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                    FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                    FXint pl=DEFAULT_MARGIN, FXint pr=DEFAULT_MARGIN,
                    FXint pt=DEFAULT_MARGIN, FXint pb=DEFAULT_MARGIN);

  virtual ~FF_StatementTable();

  void addColumn(int coltype, int size=DEFAULT_COLUMN_WIDTH);

  void addStatement(AE_STATEMENT *st);
  void addStatements(AE_STATEMENT_LIST2 *sl, int copyMode);

  void getSelectedStatements(AE_STATEMENT_LIST2 *sl, int copyMode);

  void delRow(int r);


  AE_STATEMENT *getStatementByRow(int r);

  AE_STATEMENT *getCurrentStatement();

  AQDB_ID getMarkBeyondId() const { return m_markBeyondId;};
  void setMarkBeyondId(AQDB_ID id) { m_markBeyondId=id;};

  bool getColorizeMatches() const { return m_colorizeMatches;};
  void setColorizeMatches(bool b);

  virtual void setNumberOfRows(int i);

  virtual void clearItems();

  virtual void sortByColumn(int idx, bool up);

  void reload();

  long onCmdPartner(FXObject*, FXSelector, void* ptr);
  long onCancelPartner(FXObject*, FXSelector, void* ptr);

  long onCmdTextField(FXObject*, FXSelector, void* ptr);
  long onCancelTextField(FXObject*, FXSelector, void* ptr);

protected:
  FF_StatementTable();

  virtual ColumnInfo *getColumnInfo(int r, int c);
  virtual void releaseColumnInfo(ColumnInfo *ci);

  virtual FXWindow *getEditor(int r, int c);
  virtual void setFromEditor(ColumnInfo *ci, FXWindow *ed);
  virtual void releaseEditor(int r, int c, FXWindow *ed);

  void addSpaceForStatements(int n);

  static int compareDateUp(const void *a, const void *b);
  static int compareDateDown(const void *a, const void *b);


  AE_STATEMENT **m_stmArray;
  int m_stmArraySize;
  int m_stmCount;

  AE_PARTNER_LIST *m_partners;

  AQDB_ID m_markBeyondId;
  bool m_colorizeMatches;

  int m_columnTypes[MAX_COLUMNS];
};



#endif


