/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_TAX_HPP
#define FF_TAX_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_tax.h>



class FF_TaxTreeItem: public FF_GroupTreeItem {
public:
  FF_TaxTreeItem(AE_TAX *ba,
		     FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);

  virtual ~FF_TaxTreeItem();

  AE_TAX *getTax() const { return m_tax;};
  void rebuild();

protected:
  AE_TAX *m_tax;
};



class FF_TaxTree: public FF_GroupTree {

public:
  FF_TaxTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
		 FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_TaxTree();

  FF_TaxTreeItem *findTaxItem(AQDB_ID id);
  FF_TaxTreeItem *findTaxItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_TaxTreeItem *findTaxItem(const AE_TAX *pt);

  void addTax(const AE_TAX *pt);
  void addTax(AE_TAX *pt);

  AE_TAX *getCurrentTax();
  AE_TAX *getFirstSelectedTax();
  void getSelectedTaxes(AE_TAX_LIST2 *pl);

  void getTaxes(AE_TAX_LIST2 *pl, int copyMode);

  void selectTax(AQDB_ID id, bool b);
  void selectTax(const AE_TAX *pt, bool b);
  void makeTaxVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);

protected:
  FF_TaxTree();

  static FXint sortFunctionUp0(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown0(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUp1(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDown1(const FXFoldingItem *it1, const FXFoldingItem *it2);

};



#endif




