/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_TRANSACTIONTABLE_HPP
#define FF_TRANSACTIONTABLE_HPP


#include <aqfoxext/fxx_table.hpp>

#include <fx.h>

#include <aqfinance/engine/book/ae_transaction.h>
#include <aqfinance/engine/book/ae_split.h>
#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_partner.h>

#include <list>



class FF_TransactionTable: public FXXTable {
  FXDECLARE(FF_TransactionTable)
public:
  enum {
    MAX_COLUMNS=32
  };

  enum {
    COLUMN_TYPE_NONE=0,
    COLUMN_TYPE_DATE=1,
    COLUMN_TYPE_MEMO,
    COLUMN_TYPE_ACCOUNT,
    COLUMN_TYPE_COUNTER_ACCOUNT,
    COLUMN_TYPE_CREDIT_ACCOUNT,
    COLUMN_TYPE_DEBIT_ACCOUNT,
    COLUMN_TYPE_VALUE,
    COLUMN_TYPE_CREDIT_VALUE,
    COLUMN_TYPE_DEBIT_VALUE,
    COLUMN_TYPE_DEPOSIT_VALUE,
    COLUMN_TYPE_WITHDRAWAL_VALUE,
    COLUMN_TYPE_IMPORT,
    COLUMN_TYPE_FLAGS,
    COLUMN_TYPE_PARTNER
  };

  enum {
    ID_ACCOUNT_EDIT=FXXTable::ID_LAST,
    ID_PARTNER_EDIT,
    ID_VALUE_EDIT,
    ID_TEXT_EDIT,
    ID_DATE_EDIT,
    ID_LAST
  };

public:
  FF_TransactionTable(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                      FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                      FXint pl=DEFAULT_MARGIN, FXint pr=DEFAULT_MARGIN,
                      FXint pt=DEFAULT_MARGIN, FXint pb=DEFAULT_MARGIN);

  virtual ~FF_TransactionTable();

  void addColumn(int coltype, int size=DEFAULT_COLUMN_WIDTH);

  void addTransaction(AE_TRANSACTION *st);
  void addTransactions(AE_TRANSACTION_LIST2 *sl, int copyMode);

  void getSelectedTransactions(AE_TRANSACTION_LIST2 *sl, int copyMode);

  void delRow(int r);

  void create();

  int reload();

  AE_TRANSACTION *getTransactionByRow(int r);

  AE_TRANSACTION *getCurrentTransaction();

  AQDB_ID getMarkBeyondId() const { return m_markBeyondId;};
  void setMarkBeyondId(AQDB_ID id) { m_markBeyondId=id;};

  bool getColorizeMatches() const { return m_colorizeMatches;};
  void setColorizeMatches(bool b);

  bool getColorizeCounterAccounts() const { return m_colorizeCounterAccounts;};
  void setColorizeCounterAccounts(bool b);

  AQDB_ID getAccountId() const { return m_accountId;};
  void setAccountId(AQDB_ID id) { m_accountId=id;};

  virtual void setNumberOfRows(int i);

  virtual void clearItems();

  virtual void sortByColumn(int idx, bool up);

  void rebuildTransactionflags(AE_TRANSACTION *t);

  long onCmdAccount(FXObject*, FXSelector, void* ptr);
  long onCancelAccount(FXObject*, FXSelector, void* ptr);

  long onCmdPartner(FXObject*, FXSelector, void* ptr);
  long onCancelPartner(FXObject*, FXSelector, void* ptr);

  long onCmdValue(FXObject*, FXSelector, void* ptr);
  long onCancelValue(FXObject*, FXSelector, void* ptr);

  long onCmdDate(FXObject*, FXSelector, void* ptr);
  long onCancelDate(FXObject*, FXSelector, void* ptr);

  long onCmdTextField(FXObject*, FXSelector, void* ptr);
  long onCancelTextField(FXObject*, FXSelector, void* ptr);

protected:
  FF_TransactionTable();

  virtual ColumnInfo *getColumnInfo(int r, int c);
  virtual void releaseColumnInfo(ColumnInfo *ci);

  virtual FXWindow *getEditor(int r, int c);
  virtual void setFromEditor(ColumnInfo *ci, FXWindow *ed);
  virtual void releaseEditor(int r, int c, FXWindow *ed);

  void addSpaceForTransactions(int n);

  static int compareDateUp(const void *a, const void *b);
  static int compareDateDown(const void *a, const void *b);

  AE_TRANSACTION **m_stmArray;
  int m_stmArraySize;
  int m_stmCount;

  AQDB_ID m_markBeyondId;

  int m_columnTypes[MAX_COLUMNS];

  AE_COMMODITY_LIST *m_commodities;
  AE_ACCOUNT_TREE *m_accountTree;
  AE_PARTNER_LIST *m_partners;

  FXIcon *m_iconMultiSplitOn;
  FXIcon *m_iconMultiSplitOff;
  FXIcon *m_iconMultiCurrencyOn;
  FXIcon *m_iconMultiCurrencyOff;
  FXIcon *m_iconNewOn;
  FXIcon *m_iconNewOff;
  FXIcon *m_iconInWorkOn;
  FXIcon *m_iconInWorkOff;

  FXIcon *m_iconAccept;
  FXIcon *m_iconReject;

  bool m_colorizeMatches;
  bool m_colorizeCounterAccounts;
  AQDB_ID m_accountId;
};



#endif


