/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_transferedit.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_datepicker.hpp"
#include "ff_partneredit.hpp"
#include "ff_recurrenceedit.hpp"

#include <aqbanking/jobsingletransfer.h>
#include <aqbanking/jobinternaltransfer.h>
#include <aqbanking/jobcreatesto.h>
#include <aqbanking/jobmodifysto.h>
#include <aqbanking/jobsepatransfer.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)

#define MAX_PURPOSE_LINES 4



FXDEFMAP(FF_TransferEdit) FF_TransferEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_TransferEdit::ID_COMBO_LOCAL_ACC,
            FF_TransferEdit::onCmdLocalAccount),
  FXMAPFUNC(SEL_COMMAND, FF_TransferEdit::ID_COMBO_PARTNER_ACC,
            FF_TransferEdit::onCmdPartnerAccount),
  FXMAPFUNC(SEL_COMMAND, FF_TransferEdit::ID_EDIT_PARTNER, FF_TransferEdit::onCmdPartner),
  FXMAPFUNC(SEL_COMMAND, FF_TransferEdit::ID_BUTTON_DATE, FF_TransferEdit::onCmdDate),
  FXMAPFUNCS(SEL_CHANGED, FF_TransferEdit::ID_EDIT_PURPOSE1, FF_TransferEdit::ID_EDIT_PURPOSE4, FF_TransferEdit::onChgPurpose),
  FXMAPFUNC(SEL_CHANGED, FF_TransferEdit::ID_EDIT_NAME, FF_TransferEdit::onChgName),
  FXMAPFUNC(SEL_COMMAND, FF_TransferEdit::ID_EDIT_BANK_CODE, FF_TransferEdit::onCmdBankCode),
  FXMAPFUNC(SEL_COMMAND, FF_TransferEdit::ID_EDIT_ACCOUNT_NUM, FF_TransferEdit::onCmdAccountNumber),

};


FXIMPLEMENT(FF_TransferEdit, FXVerticalFrame, FF_TransferEditMap, ARRAYNUMBER(FF_TransferEditMap))




FF_TransferEdit::FF_TransferEdit(FF_App* a,
				 int editMode,
                                 int rules,
				 FXComposite *parent,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h,
                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                 FXint hs, FXint vs)
:FXVerticalFrame(parent, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_editMode(editMode)
,m_rules(rules)
,m_maxNumPurposeLines(2)
,m_maxPurposeLen(27)
,m_maxRemoteNameLen(27)
,m_localAccountCombo(NULL)
,m_nameEdit(NULL)
,m_accNumEdit(NULL)
,m_bankCodeEdit(NULL)
,m_purposeEdit1(NULL)
,m_purposeEdit2(NULL)
,m_amountEdit(NULL)
,m_currencyCombo(NULL)
,m_textKeyCombo(NULL)
,m_dateEdit(NULL)
,m_dateButton(NULL)
,m_recurrenceEdit(NULL)
,m_bankAccountList(NULL)
,m_partnerAccountList(NULL)
,m_bankingLimits(NULL)
{
  FXVerticalFrame *vf;
  FXGroupBox *gb;
  FXMatrix *m;
  FXHorizontalFrame *hf;
  FXHorizontalFrame *hf2;

  /* local account */
  gb=new FXGroupBox(this, I18N("FF_TransferEdit|Local Account"), LAYOUT_FILL_X | FRAME_LINE);

  m_localAccountCombo=new THEMECOMBOBOX(gb, 1, this, ID_COMBO_LOCAL_ACC,
                                        COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  /* business partner */
  gb=new FXGroupBox(this, I18N("FF_TransferEdit|Partner"), LAYOUT_FILL_X | FRAME_LINE);
  m=new FXMatrix(gb, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Partner"), NULL, LABEL_NORMAL);
  m_partnerEdit=new FF_PartnerEdit(m, this, ID_EDIT_PARTNER,
				   COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK |
				   LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Partner Account"), NULL, LABEL_NORMAL);
  m_partnerAccountCombo=new THEMECOMBOBOX(m, 1, this, ID_COMBO_PARTNER_ACC,
                                          COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);

  /* purpose, amount, text key */
  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X | PACK_UNIFORM_HEIGHT, 0, 0, 0, 0, 0, 0);

  /* remote account */
  gb=new FXGroupBox(hf, I18N("FF_TransferEdit|Remote Account"), LAYOUT_FILL_X | FRAME_LINE);
  vf=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0);
  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);


  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 24, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN |
                             LAYOUT_FILL_X);

  if (m_rules==RULES_SEPA_TRANSFER)
    new FXLabel(m, I18N("BIC"), NULL, LABEL_NORMAL);
  else
    new FXLabel(m, I18N("Bank Code"), NULL, LABEL_NORMAL);
  m_bankCodeEdit=new FXTextField(m, 24, this, ID_EDIT_BANK_CODE,
                                 TEXTFIELD_NORMAL |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  if (m_rules==RULES_SEPA_TRANSFER)
    new FXLabel(m, I18N("IBAN"), NULL, LABEL_NORMAL);
  else
    new FXLabel(m, I18N("Account Number"), NULL, LABEL_NORMAL);
  m_accNumEdit=new FXTextField(m, 24, this, ID_EDIT_ACCOUNT_NUM,
			       TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
                               LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  m_bankNameLabel=new FXLabel(vf, "(---)", NULL, LABEL_NORMAL|LAYOUT_FILL_X);
  new FXSpring(vf, LAYOUT_FILL_Y);


  /* purpose */
  gb=new FXGroupBox(hf, I18N("FF_TransferEdit|Purpose"), FRAME_LINE);
  vf=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_purposeEdit1=new FXTextField(vf, 27, this, ID_EDIT_PURPOSE1,
				 LAYOUT_FILL_X |
				 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | TEXTFIELD_LIMITED |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_purposeEdit2=new FXTextField(vf, 27, this, ID_EDIT_PURPOSE2,
				 LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  m_purposeEdit3=new FXTextField(vf, 27, this, ID_EDIT_PURPOSE3,
				 LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
				 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_purposeEdit4=new FXTextField(vf, 27, this, ID_EDIT_PURPOSE4,
                                 LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
                                 LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);


  /* amount, text key */
  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0);

  /* amount */
  gb=new FXGroupBox(hf, I18N("FF_TransferEdit|Amount"), FRAME_LINE);
  hf2=new FXHorizontalFrame(gb, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
  m_amountEdit=new FXTextField(hf2, 12, this, ID_EDIT_AMOUNT,
			       LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY |
			       JUSTIFY_RIGHT | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
  m_currencyCombo=new THEMECOMBOBOX(hf2, 1, this, ID_COMBO_CURRENCY,
                                    COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK |
                                    LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

  /* textkey */
  gb=new FXGroupBox(hf, I18N("FF_TransferEdit|Textkey"), FRAME_LINE);
  m_textKeyCombo=new THEMECOMBOBOX(gb, 1, this, ID_COMBO_TEXTKEY,
                                   COMBOBOX_STATIC | LAYOUT_FILL_X | FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN);


  if (opts & OPTS_SHOWDATE) {
    hf=new FXHorizontalFrame(this, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    /* date */
    gb=new FXGroupBox(hf, I18N("Date"), FRAME_LINE);
    hf2=new FXHorizontalFrame(gb, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);
    m_dateEdit=new FF_DateEdit(hf2, this, ID_EDIT_DATE,
                               I18N("YYYY/MM/DD"),
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
    m_dateButton=new THEMEBUTTON(hf2, I18N("Pick"), NULL, this, ID_BUTTON_DATE, BUTTON_NORMAL);
  }

  if (m_rules==RULES_CREATE_STO ||
      m_rules==RULES_MODIFY_STO) {
    /* recurrence */
    gb=new FXGroupBox(this, I18N("FF_TransferEdit|Recurrence"), FRAME_LINE | LAYOUT_FILL_X);
    m_recurrenceEdit=new FF_RecurrenceEdit(gb, this, LAYOUT_FILL_X);
  }
}



FF_TransferEdit::FF_TransferEdit()
:FXVerticalFrame()
,m_app(NULL)
,m_maxNumPurposeLines(2)
,m_maxPurposeLen(27)
,m_maxRemoteNameLen(27)
,m_localAccountCombo(NULL)
,m_nameEdit(NULL)
,m_accNumEdit(NULL)
,m_bankCodeEdit(NULL)
,m_purposeEdit1(NULL)
,m_purposeEdit2(NULL)
,m_amountEdit(NULL)
,m_currencyCombo(NULL)
,m_textKeyCombo(NULL)
,m_dateEdit(NULL)
,m_dateButton(NULL)
,m_recurrenceEdit(NULL)
,m_bankAccountList(NULL)
,m_partnerAccountList(NULL)
,m_bankingLimits(NULL)
{
}



FF_TransferEdit::~FF_TransferEdit() {
  AE_BankAccount_List_free(m_bankAccountList);
  m_bankAccountList=(AE_BANKACCOUNT_LIST*) -1;

  AE_BankAccount_List_free(m_partnerAccountList);
  m_partnerAccountList=(AE_BANKACCOUNT_LIST*) -1;

  AB_TransactionLimits_free(m_bankingLimits);
  m_bankingLimits=(AB_TRANSACTION_LIMITS*) -1;
}



void FF_TransferEdit::setMaxNumPurposeLines(int i) {
  m_purposeEdit1->disable();
  m_purposeEdit2->disable();
  m_purposeEdit3->disable();
  m_purposeEdit4->disable();

  if (i>3)
    m_purposeEdit4->enable();
  if (i>2)
    m_purposeEdit3->enable();
  if (i>1)
    m_purposeEdit2->enable();
  if (i>0)
    m_purposeEdit1->enable();
  m_maxNumPurposeLines=i;
}



void FF_TransferEdit::setMaxPurposeLen(int i) {
#if 0
  m_purposeEdit1->setNumColumns(i);
  m_purposeEdit2->setNumColumns(i);
  m_purposeEdit3->setNumColumns(i);
  m_purposeEdit4->setNumColumns(i);
#endif

  m_maxPurposeLen=i;
  checkPurposeLengths();
}



void FF_TransferEdit::setMaxRemoteNameLen(int i) {
#if 0
  m_nameEdit->setNumColumns(i);
#endif

  m_maxRemoteNameLen=i;
  checkRemoteNameLength();
}



void FF_TransferEdit::init(GWEN_DB_NODE *db) {
  if (m_recurrenceEdit) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "recurrenceEdit");
    m_recurrenceEdit->init(dbT);
  }
}



void FF_TransferEdit::fini(GWEN_DB_NODE *db) {
  if (m_recurrenceEdit) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "recurrenceEdit");
    m_recurrenceEdit->fini(dbT);
  }
}



int FF_TransferEdit::checkGui(FXbool quiet) {
  return checkGui(quiet, m_editMode);
}



int FF_TransferEdit::checkGui(FXbool quiet, int editMode) {
  FXString str;
  FXString str2;
  FXString str3;
  FXString str4;
  int i;
  AE_BANKACCOUNT *ba;
  AB_BANKING *ab;
  GWEN_TIME *ti;

  ab=m_app->getBanking();

  i=m_localAccountCombo->getCurrentItem();
  if (i==0) {
    if (editMode==EDITMODE_STRICT) {
      DBG_ERROR(0, "Missing account");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
			    I18N("Please select a local account"));
        m_localAccountCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    i--;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }
  
    if (ba==NULL) {
      if (editMode==EDITMODE_STRICT) {
        DBG_ERROR(0, "Missing account");
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
                              I18N("Missing local account"));
          m_localAccountCombo->setFocus();
        }
        return GWEN_ERROR_NO_DATA;
      }
    }
    else {
      if (m_rules==RULES_SEPA_TRANSFER) {
	const char *s;

	s=AE_BankAccount_GetIban(ba);
	if (!(s && *s)) {
	  if (editMode==EDITMODE_STRICT) {
	    DBG_ERROR(0, "Account missing IBAN");
	    if (!quiet) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Input Error"),
				  "%s",
				  I18N("Local account has no IBAN, please edit your bank account settings first."));
	      m_localAccountCombo->setFocus();
	    }
	    return GWEN_ERROR_NO_DATA;
	  }
	}

	s=AE_BankAccount_GetBic(ba);
	if (!(s && *s)) {
	  if (editMode==EDITMODE_STRICT) {
	    DBG_ERROR(0, "Account missing BIC");
	    if (!quiet) {
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("Input Error"),
				  "%s",
				  I18N("Local account has no BIC, please edit your bank account settings first."));
	      m_localAccountCombo->setFocus();
	    }
	    return GWEN_ERROR_NO_DATA;
	  }
	}
      }
    }
  }

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing peer name"));
        m_nameEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    if (str.count()>m_maxRemoteNameLen) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Remote name has too many characters"));
	m_nameEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  str=m_accNumEdit->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT) {
      if (!quiet) {
	if (m_rules==RULES_SEPA_TRANSFER) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Missing peer IBAN"));
	}
	else {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Missing peer account number"));
	}
        m_accNumEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  str2=m_bankCodeEdit->getText().simplify();
  if (str2.empty()) {
    if (editMode==EDITMODE_STRICT) {
      if (!quiet) {
	if (m_rules==RULES_SEPA_TRANSFER) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Missing peer BIC"));
	}
	else {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Missing peer bank code"));
	}
        m_bankCodeEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  if (ab) {
    AB_BANKINFO_CHECKRESULT res;

    if (m_rules==RULES_SEPA_TRANSFER) {
      int rv;

      /* SEPA */
      rv=AB_Banking_CheckIban(str.text());
      if (rv<0) { /* error */
	DBG_ERROR(0,
		  "Error checking IBAN (%d)", rv);
	if (!quiet) {
	  if (FXMessageBox::warning(this, MBOX_YES_NO,
				    I18N("Internal Error"),
				    "%s",
				    I18N("The given remote IBAN could not be checked.\n"
					 "Do you still want to use the information entered?"))
	      !=MBOX_CLICKED_YES) {
	    return GWEN_ERROR_BAD_DATA;
	  }
	}
      }
      else if (rv==1) { /* invalid IBAN */
	DBG_ERROR(0,
		  "Invalid IBAN [%s] for remote account", str.text());
	if (!quiet) {
	  if (FXMessageBox::warning(this, MBOX_YES_NO,
				    I18N("Input Error"),
				    "%s",
				    I18N("The given remote IBAN seems to be invalid.\n"
					 "Please verify your input.\n"
					 "Do you still want to use the information entered?"))
	      !=MBOX_CLICKED_YES) {
	    return GWEN_ERROR_BAD_DATA;
	  }
	}
      }
    }
    else {
      /* non-SEPA */
      res=AB_Banking_CheckAccount(ab, "de", 0, str2.text(), str.text());
      switch(res) {
	case AB_BankInfoCheckResult_NotOk:
	  DBG_ERROR(0,
		    "Invalid combination of bank code and account number "
		    "for remote account");
	  if (!quiet) {
	    if (FXMessageBox::warning(this, MBOX_YES_NO,
				      I18N("Input Error"),
				      "%s",
				      I18N("The given combination of bank code and account "
					   "number seems to be invalid.\n"
					   "Please verify your input.\n"
					   "Do you still want to use the information entered?"))
		!=MBOX_CLICKED_YES) {
	      return GWEN_ERROR_BAD_DATA;
	    }
	  }
	  break;
  
	case AB_BankInfoCheckResult_UnknownBank:
	  DBG_ERROR(0, "Remote bank code is unknown");
	  if (!quiet) {
	    if (FXMessageBox::warning(this, MBOX_YES_NO,
				      I18N("Input Error"),
				      "%s",
				      I18N("The given bank code is unknown.\n"
					   "Please verify your input.\n"
					   "Do you still want to use the information entered?"))
		!=MBOX_CLICKED_YES) {
	      return GWEN_ERROR_BAD_DATA;
	    }
	  }
	  break;
      
	case AB_BankInfoCheckResult_UnknownResult:
	  DBG_ERROR(0,
		    "Indifferent result for remote account check");
	  if (!quiet) {
	    if (FXMessageBox::warning(this, MBOX_YES_NO,
				      I18N("Input Error"),
				      "%s",
				      I18N("The validity of the given combination of bank code "
					   "and account couldn't be established.\n"
					   "Please verify your input.\n"
					   "Do you still want to use the information entered?"))
		!=MBOX_CLICKED_YES) {
	      return GWEN_ERROR_BAD_DATA;
	    }
	  }
	  break;
  
	case AB_BankInfoCheckResult_Ok:
	  break;
  
	default:
	  DBG_ERROR(0, "Unknown check result %d", res);
	  return 4;
      }
    }
  }

  /* extensive purpose testing */
  str=m_purposeEdit1->getText().simplify();
  str2=m_purposeEdit2->getText().simplify();
  str3=m_purposeEdit3->getText().simplify();
  str4=m_purposeEdit4->getText().simplify();

  if (str.empty() &&
      str2.empty() &&
      str3.empty() &&
      str4.empty()) {
    if (editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing purpose"));
        m_purposeEdit1->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  if (str.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit1->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (str2.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit2->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (str3.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit3->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  if (str4.count()>m_maxPurposeLen) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Purpose line has too many characters"));
      m_purposeEdit4->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }


  str=m_amountEdit->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing amount"));
        m_amountEdit->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Two types of komma detected, "
                                 "please specify the decimal point only"));
        m_amountEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad amount input"));
          m_amountEdit->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
      AQDB_Value_free(v);
    }
  }

  str=m_currencyCombo->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT) {
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Missing currency"));
        m_currencyCombo->setFocus();
      }
      return GWEN_ERROR_NO_DATA;
    }
  }

  if (m_rules!=RULES_SEPA_TRANSFER) {
    str=m_textKeyCombo->getText().simplify();
    if (str.empty()) {
      if (editMode==EDITMODE_STRICT) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
                              "%s",
                              I18N("Missing text key"));
          m_textKeyCombo->setFocus();
        }
        return GWEN_ERROR_NO_DATA;
      }
    }
    else {
      int i;

      if (1!=sscanf(str.text(), "%d", &i)) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
                              "%s",
                              I18N("Bad text key input"));
          m_textKeyCombo->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
    }
  }

  if (m_dateEdit) {
    if (!m_dateEdit->getDate(&ti)) {
      if (editMode==EDITMODE_STRICT) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
                              "%s",
                              I18N("Bad date input"));
          m_dateEdit->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
    }
    else
      GWEN_Time_free(ti);
  }

  if (m_recurrenceEdit) {
    int rv;

    rv=m_recurrenceEdit->checkGui(quiet);
    if (rv<0)
      return rv;
  }

  return 0;
}



int FF_TransferEdit::toGui(const AE_STATEMENT *stm) {
  AE_BOOK *b;
  AB_BANKING *ab;

  setMaxNumPurposeLines(4);
  setMaxPurposeLen(27);

  ab=m_app->getBanking();
  b=m_app->getBook();
  if (b && ab) {
    int rv;
    AE_BANKACCOUNT *ba;
    int i;
    int selected=0;
    const char *s;
    const AQDB_VALUE *v;
    const GWEN_DATE *ti;

    m_partnerEdit->reload();

    /* fill local account combo box */
    m_localAccountCombo->clearItems();
    m_currencyCombo->clearItems();

    m_localAccountCombo->appendItem(I18N("Select an Account"));

    if (m_bankAccountList==NULL) {
      m_bankAccountList=AE_BankAccount_List_new();
      rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, m_bankAccountList);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
        AE_BankAccount_List_free(m_bankAccountList);
	m_bankAccountList=NULL;
        return rv;
      }

      /* remove offline accounts */
      ba=AE_BankAccount_List_First(m_bankAccountList);
      while(ba) {
	AE_BANKACCOUNT *n;
	AQDB_ID oid;

	n=AE_BankAccount_List_Next(ba);
	oid=AE_BankAccount_GetBankingId(ba);
	if (oid) {
	  if (AB_Banking_GetAccount(ab, oid)==NULL) {
	    /* oid, but there is no account for it; remove from list */
	    AE_BankAccount_List_Del(ba);
	    AE_BankAccount_free(ba);
	  }
	}
	else {
	  /* offline account, remove from list and free */
	  AE_BankAccount_List_Del(ba);
	  AE_BankAccount_free(ba);
	}

	ba=n;
      }
    }

    i=1;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      const char *s1;
      const char *s2;
      FXString str;

      if (m_rules==RULES_SEPA_TRANSFER) {
	s1=AE_BankAccount_GetBankName(ba);
	if (s1==NULL || *s1==0)
	  s1=AE_BankAccount_GetBic(ba);
	s2=AE_BankAccount_GetAccountName(ba);
	if (s2==NULL || *s1==0)
	  s2=AE_BankAccount_GetIban(ba);
      }
      else {
	s1=AE_BankAccount_GetBankName(ba);
	if (s1==NULL || *s1==0)
	  s1=AE_BankAccount_GetBankCode(ba);
	s2=AE_BankAccount_GetAccountName(ba);
	if (s2==NULL || *s1==0)
	  s2=AE_BankAccount_GetAccountNumber(ba);
      }
      if (*s1==0) s1=NULL;
      if (*s2==0) s2=NULL;

      if (s1 && s2)
        str=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        str=FXString(s1);
      else if (s2)
        str=FXString(s2);
      else
        str=FXString(I18N("unnamed account"));
      m_localAccountCombo->appendItem(str);
      if (AE_Statement_GetBankAccountId(stm)==AE_BankAccount_GetId(ba))
        selected=i;

      i++;
      ba=AE_BankAccount_List_Next(ba);
    }

    i=m_localAccountCombo->getNumItems();
    if (i>10)
      m_localAccountCombo->setNumVisible(10);
    else
      m_localAccountCombo->setNumVisible(i);

    m_localAccountCombo->setCurrentItem(selected);
    if (selected)
      selectLocalAccount(selected-1);

    /* partner */
    m_partnerEdit->setPartnerId(AE_Statement_GetPartnerId(stm));
    updatePartnerWidgets();

    /* name */
    s=AE_Statement_GetRemoteName(stm);
    if (s && *s)
      m_nameEdit->setText(FXString(s));

    /* select parter account according to remote bank account */
    i=m_partnerAccountCombo->getNumItems();
    if (i<1)
      m_partnerAccountCombo->setCurrentItem(0);
    else {
      int j;
      const char *sBankCode;
      const char *sAccNum;
      const char *sIban;

      sBankCode=AE_Statement_GetRemoteBankCode(stm);
      sAccNum=AE_Statement_GetRemoteAccountNumber(stm);
      sIban=AE_Statement_GetRemoteIban(stm);

      if (sBankCode)
	while(*sBankCode=='0')
	  sBankCode++;

      if (sAccNum)
	while(*sAccNum=='0')
	  sAccNum++;

      for (j=1; j<i; j++) {
	AE_BANKACCOUNT *ba;

	ba=(AE_BANKACCOUNT*) (m_partnerAccountCombo->getItemData(j));
	if (ba) {
	  const char *xBankCode;
	  const char *xAccNum;
	  const char *xIban;

	  xBankCode=AE_BankAccount_GetBankCode(ba);
	  xAccNum=AE_BankAccount_GetAccountNumber(ba);
	  xIban=AE_BankAccount_GetIban(ba);

	  if (sBankCode && *sBankCode && sAccNum && *sAccNum &&
	      xBankCode && *xBankCode && xAccNum && *xAccNum) {
	    if (xBankCode)
	      while(*xBankCode=='0')
		xBankCode++;
	    if (xAccNum)
	      while(*xAccNum=='0')
		xAccNum++;
	    if (strcasecmp(sBankCode, xBankCode)==0 &&
		strcasecmp(sAccNum, xAccNum)==0)
              break;
	  }
	  else if (sIban && *sIban && xIban && *xIban) {
	    if (strcasecmp(sIban, xIban)==0)
              break;
	  }
	}
      }
      if (j<i)
        m_partnerAccountCombo->setCurrentItem(j);
    }

    if (m_rules==RULES_SEPA_TRANSFER) {
      /* BIC */
      s=AE_Statement_GetRemoteBic(stm);
      if (s && *s)
	m_bankCodeEdit->setText(FXString(s), TRUE);

      /* account number */
      s=AE_Statement_GetRemoteIban(stm);
      if (s && *s)
	m_accNumEdit->setText(FXString(s), TRUE);
    }
    else {
      /* bank code */
      s=AE_Statement_GetRemoteBankCode(stm);
      if (s && *s)
	m_bankCodeEdit->setText(FXString(s), TRUE);

      /* account number */
      s=AE_Statement_GetRemoteAccountNumber(stm);
      if (s && *s)
	m_accNumEdit->setText(FXString(s), TRUE);
    }

    /* purpose (split lines) */
    s=AE_Statement_GetPurpose(stm);
    if (s && *s) {
      FXString str[MAX_PURPOSE_LINES];
      int i=0;
  
      if (strchr(s, '\n')) {
	/* multi-line purpose, handle appropriately */
	while(*s) {
	  const char *ts;
  
	  ts=strchr(s, '\n');
	  if (ts) {
	    int l;
	    char *cpy;
  
	    l=ts-s;
	    cpy=(char*) malloc(l+1);
	    assert(cpy);
	    memmove(cpy, s, l);
	    cpy[l]=0;
	    if (i<MAX_PURPOSE_LINES) {
	      str[i]=FXString(cpy);
	      i++;
	    }
	    free(cpy);
	    s=ts;
	    s++;
	  }
	  else {
	    if (i<MAX_PURPOSE_LINES) {
	      str[i]=FXString(s);
	      i++;
	    }
	    s+=strlen(s);
	  }
	}
      }
      else {
	/* single-line purpose, simply add */
	if (i<MAX_PURPOSE_LINES) {
	  str[i]=FXString(s);
	  i++;
	}
      }

      if (i>0)
	m_purposeEdit1->setText(str[0]);
      if (i>1)
	m_purposeEdit2->setText(str[1]);
      if (i>2)
	m_purposeEdit3->setText(str[2]);
      if (i>3)
	m_purposeEdit4->setText(str[3]);
    }

    /* set currency and amount */
    v=AE_Statement_GetValue(stm);
    if (v) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      m_amountEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }
    s=AE_Statement_GetCurrency(stm);
    if (s)
      m_currencyCombo->appendItem(FXString(s));
    else
      m_currencyCombo->appendItem(FXString("EUR"));

    /* set textkey */
    i=AE_Statement_GetTextKey(stm);
    if (i) {
      FXString str;

      str=FXStringVal(i, 10);
      i=m_textKeyCombo->findItem(str);
      if (i>0) {
        //i=m_textKeyCombo->appendItem(str);
        m_textKeyCombo->setCurrentItem(i);
      }
    }

    if (m_dateEdit) {
      /* set date */
      ti=AE_Statement_GetDate(stm);
      if (ti)
        m_dateEdit->setDate(ti);
    }

    /* recurrence */
    if (m_recurrenceEdit)
      m_recurrenceEdit->toGui(AE_Statement_GetRecurrence(stm));
  }
  else {
    DBG_ERROR(0, "No book, no accounts");
    return GWEN_ERROR_INVALID;
  }


  return 0;
}



int FF_TransferEdit::fromGui(AE_STATEMENT *stm) {
  return fromGui(stm, m_editMode);
}



int FF_TransferEdit::fromGui(AE_STATEMENT *stm, int editMode) {
  FXString str;
  FXString str2;
  int i;
  AE_BANKACCOUNT *ba;
  GWEN_DATE *ti;

  i=m_localAccountCombo->getCurrentItem();
  if (i==0) {
    if (editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetBankAccountId(stm, 0);
    AE_Statement_SetLocalBankCode(stm, NULL);
    AE_Statement_SetLocalAccountNumber(stm, NULL);
    AE_Statement_SetLocalIban(stm, NULL);
    AE_Statement_SetLocalBic(stm, NULL);
  }
  else {
    i--;
    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }
  
    if (ba==NULL) {
      if (editMode==EDITMODE_STRICT)
        return GWEN_ERROR_NO_DATA;
      AE_Statement_SetBankAccountId(stm, 0);
      AE_Statement_SetLocalBankCode(stm, NULL);
      AE_Statement_SetLocalAccountNumber(stm, NULL);
      AE_Statement_SetLocalIban(stm, NULL);
      AE_Statement_SetLocalBic(stm, NULL);
    }
    else {
      AQDB_ID aid;
      const char *s;

      aid=AE_BankAccount_GetId(ba);
      AE_Statement_SetBankAccountId(stm, aid);
      s=AE_BankAccount_GetOwnerName(ba);
      if (s && *s)
        AE_Statement_SetLocalName(stm, s);
      else {
        if (aid) {
          AB_BANKING *ab;

          ab=m_app->getBanking();
          if (ab) {
            AB_ACCOUNT *acc;

            acc=AB_Banking_GetAccount(ab, aid);
            if (acc)
              AE_Statement_SetLocalName(stm, AB_Account_GetOwnerName(acc));
          }
        }
      }
      AE_Statement_SetLocalBankCode(stm, AE_BankAccount_GetBankCode(ba));
      AE_Statement_SetLocalAccountNumber(stm, AE_BankAccount_GetAccountNumber(ba));
      AE_Statement_SetLocalIban(stm, AE_BankAccount_GetIban(ba));
      AE_Statement_SetLocalBic(stm, AE_BankAccount_GetBic(ba));
    }
  }

  AE_Statement_SetPartnerId(stm, m_partnerEdit->getPartnerId());

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetRemoteName(stm, NULL);
  }
  else
    AE_Statement_SetRemoteName(stm, str.text());

  if (m_rules==RULES_SEPA_TRANSFER) {
    /* SEPA */
    str=m_accNumEdit->getText().simplify();
    if (str.empty()) {
      if (editMode==EDITMODE_STRICT)
	return GWEN_ERROR_NO_DATA;
      AE_Statement_SetRemoteIban(stm, NULL);
    }
    else
      AE_Statement_SetRemoteIban(stm, str.text());

    str=m_bankCodeEdit->getText().simplify();
    if (str.empty()) {
      if (editMode==EDITMODE_STRICT)
	return GWEN_ERROR_NO_DATA;
      AE_Statement_SetRemoteBic(stm, NULL);
    }
    else
      AE_Statement_SetRemoteBic(stm, str.text());
  }
  else {
    /* non-SEPA */
    str=m_accNumEdit->getText().simplify();
    if (str.empty()) {
      if (editMode==EDITMODE_STRICT)
	return GWEN_ERROR_NO_DATA;
      AE_Statement_SetRemoteAccountNumber(stm, NULL);
    }
    else
      AE_Statement_SetRemoteAccountNumber(stm, str.text());

    str=m_bankCodeEdit->getText().simplify();
    if (str.empty()) {
      if (editMode==EDITMODE_STRICT)
	return GWEN_ERROR_NO_DATA;
      AE_Statement_SetRemoteBankCode(stm, NULL);
    }
    else
      AE_Statement_SetRemoteBankCode(stm, str.text());
  }

  str=FXString("");
  i=0;

  str2=m_purposeEdit1->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  str2=m_purposeEdit2->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  str2=m_purposeEdit3->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  str2=m_purposeEdit4->getText().simplify();
  if (!str2.empty() && i<m_maxNumPurposeLines) {
    if (!str.empty())
      str+="\n";
    str+=str2;
    i++;
  }

  if (str.empty()) {
    DBG_ERROR(AE_LOGDOMAIN, "Empty purpose");
    if (editMode==EDITMODE_STRICT) {
      return GWEN_ERROR_NO_DATA;
    }
    AE_Statement_SetPurpose(stm, NULL);
  }
  else
    AE_Statement_SetPurpose(stm, str.text());

  str=m_amountEdit->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetValue(stm, NULL);
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL)
        return GWEN_ERROR_BAD_DATA;
      AE_Statement_SetValue(stm, v);
      AQDB_Value_free(v);
    }
  }

  str=m_currencyCombo->getText().simplify();
  if (str.empty()) {
    if (editMode==EDITMODE_STRICT)
      return GWEN_ERROR_NO_DATA;
    AE_Statement_SetCurrency(stm, NULL);
  }
  else
    AE_Statement_SetCurrency(stm, str.text());

  if (m_rules!=RULES_SEPA_TRANSFER) {
    str=m_textKeyCombo->getText().simplify();
    if (str.empty()) {
      if (editMode==EDITMODE_STRICT)
        return GWEN_ERROR_NO_DATA;
      AE_Statement_SetTextKey(stm, 0);
    }
    else {
      if (1!=sscanf(str.text(), "%d", &i))
        return GWEN_ERROR_BAD_DATA;
      AE_Statement_SetTextKey(stm, i);
    }
  }

  if (m_dateEdit) {
    if (!m_dateEdit->getDate(&ti)) {
      if (editMode==EDITMODE_STRICT)
        return GWEN_ERROR_BAD_DATA;
    }
    AE_Statement_SetDate(stm, ti);
    GWEN_Date_free(ti);
  }
  /* This causes an error, because other parts of AqFinance rely on transfers having a date
  else
    AE_Statement_SetDate(stm, NULL);
  */

  /* recurrence */
  if (m_recurrenceEdit) {
    AE_RECURRENCE *rec;

    rec=AE_Recurrence_new();
    m_recurrenceEdit->fromGui(rec);
    if (m_rules==RULES_CREATE_STO) {
      AE_Recurrence_SetBeginDate(rec, AE_Statement_GetDate(stm));
    }

    AE_Statement_SetRecurrence(stm, rec);
  }

  AE_Statement_SubFlags(stm, AE_STATEMENT_FLAGS_XFER_TYPE_MASK);
  switch(m_rules) {
  case RULES_SINGLE_TRANSFER:
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_XFER_TYPE_STD_XFER);
    break;

  case RULES_INTERNAL_TRANSFER:
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_XFER_TYPE_INT_XFER);
    break;

  case RULES_CREATE_STO:
  case RULES_MODIFY_STO:
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_XFER_TYPE_STO_XFER);
    break;

  case RULES_CREATE_DATED_TRANSFER:
  case RULES_MODIFY_DATED_TRANSFER:
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_XFER_TYPE_DATED_XFER);
    break;

  case RULES_SEPA_TRANSFER:
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_XFER);
    break;
  }

  if (m_rules==RULES_INTERNAL_TRANSFER)
    AE_Statement_AddFlags(stm, AE_STATEMENT_FLAGS_INTERNAL_XFER);
  else
    AE_Statement_SubFlags(stm, AE_STATEMENT_FLAGS_INTERNAL_XFER);

  return 0;
}



void FF_TransferEdit::selectLocalAccount(int i) {
  AB_BANKING *ab;
  FXString currentKey;
  int j;

  currentKey=m_textKeyCombo->getText();
  m_textKeyCombo->clearItems();
  setMaxNumPurposeLines(1);
  setMaxPurposeLen(27);

  AB_TransactionLimits_free(m_bankingLimits);
  m_bankingLimits=NULL;

  ab=m_app->getBanking();
  if (ab) {
    AE_BANKACCOUNT *ba;

    ba=AE_BankAccount_List_First(m_bankAccountList);
    while(ba) {
      if ((i--)==0)
        break;
      ba=AE_BankAccount_List_Next(ba);
    }

    if (ba) {
      AB_ACCOUNT *a;

      a=AB_Banking_GetAccount(ab, AE_BankAccount_GetBankingId(ba));
      if (a) {
        const AB_TRANSACTION_LIMITS *tl=NULL;
        AB_JOB *j=NULL;
        int rv;


        if (m_rules==RULES_SINGLE_TRANSFER) {
          j=AB_JobSingleTransfer_new(a);
          rv=AB_Job_CheckAvailability(j);
          if (rv<0) {
            DBG_ERROR(0, "SingleTransfer not available (%d) with %s/%s", rv,
                      AB_Account_GetBankCode(a),
                      AB_Account_GetAccountNumber(a));
          }
          else
            tl=AB_Job_GetFieldLimits(j);
        }
        else if (m_rules==RULES_INTERNAL_TRANSFER) {
          j=AB_JobInternalTransfer_new(a);
          rv=AB_Job_CheckAvailability(j);
          if (rv<0) {
            DBG_ERROR(0, "InternalTransfer not available (%d) with %s/%s", rv,
                      AB_Account_GetBankCode(a),
                      AB_Account_GetAccountNumber(a));
          }
          else
            tl=AB_Job_GetFieldLimits(j);
        }
        else if (m_rules==RULES_CREATE_STO) {
          DBG_ERROR(0, "Creating standing order job");
          j=AB_JobCreateStandingOrder_new(a);
          rv=AB_Job_CheckAvailability(j);
          if (rv<0) {
            DBG_ERROR(0, "CreateStandingOrder not available (%d) with %s/%s", rv,
                      AB_Account_GetBankCode(a),
                      AB_Account_GetAccountNumber(a));
          }
          else
            tl=AB_Job_GetFieldLimits(j);
        }
        else if (m_rules==RULES_MODIFY_STO) {
          DBG_ERROR(0, "Modify standing order job");
          j=AB_JobModifyStandingOrder_new(a);
          rv=AB_Job_CheckAvailability(j);
          if (rv<0) {
            DBG_ERROR(0, "CreateStandingOrder not available (%d) with %s/%s", rv,
                      AB_Account_GetBankCode(a),
                      AB_Account_GetAccountNumber(a));
          }
          else
	    tl=AB_Job_GetFieldLimits(j);
	}
	else if (m_rules==RULES_SEPA_TRANSFER) {
          DBG_ERROR(0, "SEPA transfer");
          j=AB_JobSepaTransfer_new(a);
          rv=AB_Job_CheckAvailability(j);
          if (rv<0) {
	    DBG_ERROR(0, "SEPA-Transfer not available (%d) with %s/%s", rv,
		      AB_Account_GetBankCode(a),
		      AB_Account_GetAccountNumber(a));
	  }
	  else
	    tl=AB_Job_GetFieldLimits(j);
	}


        if (tl) {
          GWEN_STRINGLIST *keys=AB_TransactionLimits_GetValuesTextKey(tl);
          GWEN_STRINGLISTENTRY *se=GWEN_StringList_FirstEntry(keys);
          int i;

          m_bankingLimits=AB_TransactionLimits_dup(tl);

          while(se) {
            const char *s;

            s=GWEN_StringListEntry_Data(se);
            if (s)
              m_textKeyCombo->appendItem(FXString(s));
            se=GWEN_StringListEntry_Next(se);
          }
          i=AB_TransactionLimits_GetMaxLinesPurpose(tl);
          if (i<1)
            i=2;
          setMaxNumPurposeLines(i);
          i=AB_TransactionLimits_GetMaxLenPurpose(tl);
          if (i<1)
            i=27;
          setMaxPurposeLen(27);

          i=AB_TransactionLimits_GetMaxLenRemoteName(tl);
          if (i<1)
            i=27;
          setMaxRemoteNameLen(27);

          /* set limits in recurrence editor */
          if (m_recurrenceEdit)
            m_recurrenceEdit->setBankingLimits(tl, (m_rules==RULES_MODIFY_STO)?true:false);

          if (0) {
            GWEN_DB_NODE *db;

            db=GWEN_DB_Group_new("test");
            AB_TransactionLimits_toDb(tl, db);
            GWEN_DB_Dump(db, 2);
            GWEN_DB_Group_free(db);
          }

        }

        if (j)
          AB_Job_free(j);
      }
    }
  }
  if (!currentKey.empty()) {
    FXint i=m_textKeyCombo->findItem(currentKey);
    if (i>=0)
      m_textKeyCombo->setCurrentItem(i);
  }

  j=m_textKeyCombo->getNumItems();
  if (j>10)
    m_textKeyCombo->setNumVisible(10);
  else
    m_textKeyCombo->setNumVisible(j);

}



long FF_TransferEdit::onCmdLocalAccount(FXObject*, FXSelector, void *ptr) {
  int i;

  i=m_localAccountCombo->getCurrentItem();
  if (i>0) {
    selectLocalAccount(i-1);
  }

  return 1;
}



void FF_TransferEdit::updatePartnerWidgets() {
  AE_BOOK *b;

  m_partnerAccountCombo->clearItems();
  if (m_partnerAccountList)
    AE_BankAccount_List_Clear(m_partnerAccountList);
  else
    m_partnerAccountList=AE_BankAccount_List_new();
  m_partnerAccountCombo->appendItem(I18N("--select--"), NULL);

  b=m_app->getBook();
  if (b) {
    AQDB_ID id;
    int i;

    // read name and account from partner
    id=m_partnerEdit->getPartnerId();
    if (id) {
      AE_PARTNER *pt=NULL;
      AE_BANKACCOUNT *ba;
      AE_ADDRESS *addr;
      const char *s;
      int rv;
      int idx;

      /* read partner */
      rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner, id, &pt);
      if (rv<0) {
	DBG_ERROR(0, "Partner %08x not found", (unsigned int) id);
	return;
      }

      /* read subobjects (like account, address etc) */
      rv=AE_Book_ReadPartnerSubObjects(b, pt);
      if (rv<0) {
	DBG_ERROR(0, "Error reading subobjects of partner %08x", (unsigned int) id);
        AE_Partner_free(pt);
	return;
      }

      ba=AE_BankAccount_List_First(AE_Partner_GetBankAccountList(pt));
      while(ba) {
        const char *s1;
        const char *s2;
        FXString str;

        if (m_rules==RULES_SEPA_TRANSFER) {
          s1=AE_BankAccount_GetBankName(ba);
          if (s1==NULL || *s1==0)
            s1=AE_BankAccount_GetBic(ba);
          s2=AE_BankAccount_GetAccountName(ba);
          if (s2==NULL || *s1==0)
            s2=AE_BankAccount_GetIban(ba);
        }
        else {
          s1=AE_BankAccount_GetBankName(ba);
          if (s1==NULL || *s1==0)
            s1=AE_BankAccount_GetBankCode(ba);
          s2=AE_BankAccount_GetAccountName(ba);
          if (s2==NULL || *s1==0)
            s2=AE_BankAccount_GetAccountNumber(ba);
        }
        if (*s1==0) s1=NULL;
        if (*s2==0) s2=NULL;
  
        if (s1 && s2)
          str=FXStringFormat(I18N("%s at %s"), s2, s1);
        else if (s1)
          str=FXString(s1);
        else if (s2)
          str=FXString(s2);
        else
          str=FXString(I18N("unnamed account"));
        if (!str.empty()) {
	  AE_BANKACCOUNT *cpy;

          cpy=AE_BankAccount_dup(ba);
	  m_partnerAccountCombo->appendItem(str, cpy);
	  AE_BankAccount_List_Add(cpy, m_partnerAccountList);
	}

	ba=AE_BankAccount_List_Next(ba);
      }
      idx=m_partnerAccountCombo->getNumItems();
      if (idx>0) {
	m_partnerAccountCombo->setCurrentItem(idx-1);
	selectPartnerAccount(idx-1);
      }

      addr=AE_Address_List_First(AE_Partner_GetAddressList(pt));
      if (addr) {
	s=AE_Address_GetName(addr);
	if (s && *s)
	  m_nameEdit->setText(FXString(s));
      }
      else {
	FXString str;

	s=AE_Partner_GetFirstName(pt);
	if (s && *s)
	  str+=FXString(s);
	s=AE_Partner_GetLastName(pt);
	if (s && *s) {
	  if (!str.empty())
	    str+=" ";
          str+=FXString(s);
	}
        m_nameEdit->setText(str);
      }

      AE_Partner_free(pt);
    }
    i=m_partnerAccountCombo->getNumItems();
    if (i>10)
      m_partnerAccountCombo->setNumVisible(10);
    else
      m_partnerAccountCombo->setNumVisible(i);
  }
}



long FF_TransferEdit::onCmdPartner(FXObject*, FXSelector, void *ptr) {
  updatePartnerWidgets();
  return 1;
}



void FF_TransferEdit::selectPartnerAccount(int i) {
  if (i>=0) {
    AE_BANKACCOUNT *ba;

    ba=(AE_BANKACCOUNT*) m_partnerAccountCombo->getItemData(i);
    if (ba) {
      const char *sBic;
      const char *sIban;
      const char *sBankCode;
      const char *sAccountNumber;
      AB_BANKING *ab;

      ab=m_app->getBanking();
      assert(ab);

      sBic=AE_BankAccount_GetBic(ba);
      sIban=AE_BankAccount_GetIban(ba);
      sBankCode=AE_BankAccount_GetBankCode(ba);
      sAccountNumber=AE_BankAccount_GetAccountNumber(ba);

      if (m_rules==RULES_SEPA_TRANSFER) {
        /* SEPA */
        if (sBic && *sBic)
          m_bankCodeEdit->setText(FXString(sBic), TRUE);
        else if (!(sBic && *sBic) && (sBankCode && *sBankCode)) {
          AB_BANKINFO *bi;

          bi=AB_Banking_GetBankInfo(ab, "de", "*", sBankCode); /* TODO: country should not be fixed here... */
          if (bi) {
            const char *s;
    
            s=AB_BankInfo_GetBic(bi);
            if (s && *s) {
              DBG_INFO(AQBANKING_LOGDOMAIN, "Setting BIC for [%s] to [%s]", sBankCode, s);
              m_bankCodeEdit->setText(FXString(s), TRUE);
            }
            AB_BankInfo_free(bi);
          }
        }

        if (sIban && *sIban)
          m_accNumEdit->setText(FXString(sIban), TRUE);
        else if (!(sIban && *sIban) && (sBankCode && *sBankCode) && (sAccountNumber && *sAccountNumber)) {
          GWEN_BUFFER *tbuf;
          int rv;
    
          tbuf=GWEN_Buffer_new(0, 32, 0, 1);
          rv=AB_Banking_MakeGermanIban(sBankCode, sAccountNumber, tbuf);
          if (rv<0) {
            DBG_INFO(AQBANKING_LOGDOMAIN, "here (%d)", rv);
          }
          else {
            DBG_INFO(AQBANKING_LOGDOMAIN, "Setting IBAN for [%s/%s] to [%s]",
                     sBankCode, sAccountNumber, GWEN_Buffer_GetStart(tbuf));
            m_accNumEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)), TRUE);
          }
          GWEN_Buffer_free(tbuf);
        }
      }
      else {
        /* non-SEPA */
        if (sBankCode && *sBankCode)
          m_bankCodeEdit->setText(FXString(sBankCode), TRUE);

        if (sAccountNumber && *sAccountNumber)
          m_accNumEdit->setText(FXString(sAccountNumber), TRUE);
      }
    }
  }
}



long FF_TransferEdit::onCmdPartnerAccount(FXObject*, FXSelector, void *ptr) {
  int i;

  i=m_partnerAccountCombo->getCurrentItem();
  if (i>0)
    selectPartnerAccount(i);

  return 1;
}



long FF_TransferEdit::onCmdSearchBank(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_TransferEdit::onCmdCurrency(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_TransferEdit::onCmdTextKey(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_TransferEdit::onCmdAmount(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_TransferEdit::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



void FF_TransferEdit::checkPurposeLengths() {
  FXTextField *tf;

  tf=m_purposeEdit1;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }

  tf=m_purposeEdit2;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }

  tf=m_purposeEdit3;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }

  tf=m_purposeEdit4;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxPurposeLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }
}



void FF_TransferEdit::checkRemoteNameLength() {
  FXTextField *tf;

  tf=m_nameEdit;
  if (tf) {
    FXString str;

    str=tf->getText().simplify();
    if (str.count()>m_maxRemoteNameLen)
      tf->setBackColor(FXRGB(255, 0, 0));
    else
      tf->setBackColor(getApp()->getBackColor());
  }
}



long FF_TransferEdit::onChgPurpose(FXObject *sender, FXSelector, void *ptr) {
  checkPurposeLengths();
  return 1;
}



long FF_TransferEdit::onChgName(FXObject *sender, FXSelector, void *ptr) {
  checkRemoteNameLength();
  return 1;
}



long FF_TransferEdit::onCmdDate(FXObject*, FXSelector, void *ptr) {
  if (m_dateEdit) {
    FXDate d;

    if (!m_dateEdit->getDate(d))
      d=FF_DateEdit::today();
    if (FF_DatePicker::pickDate(this, I18N("Date Picker"), d))
      m_dateEdit->setDate(d);
  }
  return 1;
}



void FF_TransferEdit::updateBankNameLabel() {
  AB_BANKING *ab;

  ab=m_app->getBanking();
  if (ab) {
    AB_BANKINFO *bi;
    FXString country;
    FXString str;
    FXString bcText;
    FXString bnText=I18N("(unknown)");
    FXString acText;
    FXColor col=getApp()->getForeColor();
    int i;

    str=m_bankCodeEdit->getText().simplify().trim();
    for (i=0; i<str.count(); i++) {
      if (isalnum(str[i]))
        bcText+=str[i];
    }

    str=m_accNumEdit->getText().simplify().trim();
    for (i=0; i<str.count(); i++) {
      if (isalnum(str[i]))
        acText+=str[i];
    }

    if (m_rules==RULES_SEPA_TRANSFER) {
      if (acText.length()>=2)
        country=acText.left(2);
      else if (bcText.length()>=6)
        country=bcText.mid(4, 2);
    }

    if (country.empty())
      country="de";

    if (m_rules==RULES_SEPA_TRANSFER) {
      int rv;
      FXString bankString;
      FXString ibanResultString;

      if (!(bcText.empty())) {
        AB_BANKINFO *bi;
        AB_BANKINFO_LIST2 *biList;

        bi=AB_BankInfo_new();
        AB_BankInfo_SetBic(bi, bcText.text());

        biList=AB_BankInfo_List2_new();
        rv=AB_Banking_GetBankInfoByTemplate(ab, country.text(), bi, biList);
        if (rv<0) {
          DBG_INFO(0, "here (%d)", rv);
          bankString=FXStringFormat(I18N("%s, unknown bank %s"), country.text(), bcText.text());
        }
        else {
          AB_BANKINFO *rbi;

          rbi=AB_BankInfo_List2_GetFront(biList);
          if (rbi) {
            const char *sBankName;
            const char *sBankLocation;

            sBankName=AB_BankInfo_GetBankName(rbi);
            sBankLocation=AB_BankInfo_GetLocation(rbi);
            if (sBankName && *sBankName && sBankLocation && *sBankLocation)
              bankString=FXStringFormat(I18N("%s %s (%s)"),
                                        sBankName, sBankLocation, country.text());
            else if (sBankName && *sBankName)
              bankString=FXStringFormat(I18N("%s (%s)"),
                                        sBankName, country.text());
            else
              bankString=FXStringFormat(I18N("%s, unknown bank %s"), country.text(), bcText.text());
          }
        }
        AB_BankInfo_List2_freeAll(biList);
        AB_BankInfo_free(bi);
      }
      else {
        DBG_ERROR(0, "bcText is empty");
      }


      DBG_ERROR(0, "Checking IBAN [%s]", acText.text());
      rv=AB_Banking_CheckIban(acText.text());
      if (rv<0) { /* error */
	DBG_ERROR(0, "Error checking IBAN (%d)", rv);
        col=FXRGB(255, 0, 0); /* red */
        ibanResultString=I18N("(Error checking IBAN)");
      }
      else if (rv==1) { /* invalid IBAN */
        DBG_ERROR(0, "Invalid IBAN [%s] for remote account", str.text());
        col=FXRGB(255, 0, 0); /* red */
        ibanResultString=I18N("(Invalid IBAN)");
      }
      else {
        col=FXRGB(0, 255, 0); /* green */
        ibanResultString=I18N("(IBAN is valid)");
      }

      bnText=bankString;
    }
    else {
      bi=AB_Banking_GetBankInfo(ab,
                                country.text(),
                                NULL,
                                bcText.text());
      if (bi) {
        const char *s;
  
        s=AB_BankInfo_GetBankName(bi);
        if (s && *s) {
          bnText="(";
          bnText+=s;
          bnText+=")";
        }
  
        AB_BankInfo_free(bi);
      }
  
      if (!bcText.empty() && !acText.empty()) {
        AB_BANKINFO_CHECKRESULT res;
  
        res=AB_Banking_CheckAccount(ab, country.text(), NULL,
                                    bcText.text(), acText.text());
        switch(res) {
        case AB_BankInfoCheckResult_Ok:             col=FXRGB(0, 255, 0); break; /* green */
        case AB_BankInfoCheckResult_NotOk:          col=FXRGB(255, 0, 0); break; /* red */
        case AB_BankInfoCheckResult_UnknownBank:    col=FXRGB(0, 0, 255); break; /* blue */
        case AB_BankInfoCheckResult_UnknownResult:  col=FXRGB(0, 0, 255); break; /* blue */
        default:                                    col=FXRGB(0, 0, 255); break; /* blue */
        }
      }
    }

    m_bankNameLabel->setTextColor(col);
    m_bankNameLabel->setText(bnText);

  }
}



long FF_TransferEdit::onCmdBankCode(FXObject*, FXSelector, void *ptr) {
  updateBankNameLabel();
  return 1;
}



long FF_TransferEdit::onCmdAccountNumber(FXObject*, FXSelector, void *ptr) {
  updateBankNameLabel();
  return 1;
}





