/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef FF_TRANSFEREDIT_HPP
#define FF_TRANSFEREDIT_HPP


#include <fx.h>


#include <aqfinance/engine/book/ae_statement.h>
#include <aqfinance/engine/book/ae_bankaccount.h>

#include <aqbanking/transactionlimits.h>



class FF_App;
class FF_DateEdit;
class FF_PartnerEdit;
class FF_RecurrenceEdit;



class FF_TransferEdit: public FXVerticalFrame {
  FXDECLARE(FF_TransferEdit)

public:

  enum {
    OPTS_SHOWDATE = 0x80000000
  };

  enum {
    EDITMODE_STRICT=0,
    EDITMODE_EASY,
    EDITMODE_TEMPLATE
  };

  enum {
    RULES_SINGLE_TRANSFER=0,
    RULES_INTERNAL_TRANSFER,
    RULES_CREATE_STO,
    RULES_MODIFY_STO,
    RULES_CREATE_DATED_TRANSFER,
    RULES_MODIFY_DATED_TRANSFER,
    RULES_SEPA_TRANSFER
  };

  enum {
    ID_COMBO_LOCAL_ACC=FXDialogBox::ID_LAST,
    ID_EDIT_PARTNER,
    ID_EDIT_NAME,
    ID_EDIT_ACCOUNT_NUM,
    ID_EDIT_BANK_CODE,
    ID_EDIT_PURPOSE1,
    ID_EDIT_PURPOSE2,
    ID_EDIT_PURPOSE3,
    ID_EDIT_PURPOSE4,
    ID_EDIT_AMOUNT,
    ID_COMBO_CURRENCY,
    ID_COMBO_TEXTKEY,
    ID_EDIT_DATE,
    ID_BUTTON_DATE,
    ID_BUTTON_SEARCH_PEER,
    ID_BUTTON_SEARCH_BANK,
    ID_COMBO_PARTNER_ACC,
    ID_LAST
  };


  FF_TransferEdit(FF_App* a,
                  int editMode,
                  int rules,
                  FXComposite *parent,
                  FXuint opts=DECOR_TITLE|DECOR_BORDER,
                  FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                  FXint pl=0, FXint pr=0, FXint pt=0, FXint pb=0,
                  FXint hs=0, FXint vs=0);
  virtual ~FF_TransferEdit();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);


  int checkGui(FXbool quiet, int editMode);
  int checkGui(FXbool quiet);
  int fromGui(AE_STATEMENT *stml, int editMode);
  int fromGui(AE_STATEMENT *stml);
  int toGui(const AE_STATEMENT *stm);

  long onCmdPartner(FXObject*, FXSelector, void *ptr);
  long onCmdPartnerAccount(FXObject*, FXSelector, void *ptr);

  long onCmdLocalAccount(FXObject*, FXSelector, void *ptr);
  long onCmdSearchBank(FXObject*, FXSelector, void *ptr);

  long onCmdTextField(FXObject*, FXSelector, void *ptr);
  long onCmdCurrency(FXObject*, FXSelector, void *ptr);
  long onCmdTextKey(FXObject*, FXSelector, void *ptr);

  long onChgPurpose(FXObject *sender, FXSelector, void *ptr);
  long onChgName(FXObject *sender, FXSelector, void *ptr);

  long onCmdAmount(FXObject*, FXSelector, void *ptr);

  long onCmdDate(FXObject*, FXSelector, void *ptr);
  long onCmdBankCode(FXObject*, FXSelector, void *ptr);
  long onCmdAccountNumber(FXObject*, FXSelector, void *ptr);

protected:
  FF_TransferEdit();

  virtual void selectLocalAccount(int i);
  virtual void selectPartnerAccount(int i);
  void updatePartnerWidgets();

  void setMaxNumPurposeLines(int i);
  void setMaxPurposeLen(int i);
  void setMaxRemoteNameLen(int i);

  void checkPurposeLengths();
  void checkRemoteNameLength();

  void updateBankNameLabel();

  FF_App *m_app;
  int m_editMode;
  int m_rules;

  int m_maxNumPurposeLines;
  int m_maxPurposeLen;
  int m_maxRemoteNameLen;

  FXComboBox *m_localAccountCombo;

  FF_PartnerEdit *m_partnerEdit;
  FXComboBox *m_partnerAccountCombo;

  FXTextField *m_nameEdit;
  FXTextField *m_accNumEdit;
  FXTextField *m_bankCodeEdit;

  FXTextField *m_purposeEdit1;
  FXTextField *m_purposeEdit2;
  FXTextField *m_purposeEdit3;
  FXTextField *m_purposeEdit4;

  FXTextField *m_amountEdit;

  FXComboBox *m_currencyCombo;
  FXComboBox *m_textKeyCombo;

  FF_DateEdit *m_dateEdit;
  FXButton *m_dateButton;

  FF_RecurrenceEdit *m_recurrenceEdit;

  FXLabel *m_bankNameLabel;

  AE_BANKACCOUNT_LIST *m_bankAccountList;

  AE_BANKACCOUNT_LIST *m_partnerAccountList;

  AB_TRANSACTION_LIMITS *m_bankingLimits;
};




#endif




