/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_TRANSFERTREE_HPP
#define FF_TRANSFERTREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_statement.h>

#include <aqbanking/job.h>



class FF_TransferTreeItem: public FF_GroupTreeItem {
public:
  FF_TransferTreeItem(AE_STATEMENT *st, void* ptr=NULL);

  virtual ~FF_TransferTreeItem();

  AE_STATEMENT *getTransfer() const { return m_transfer;};

  virtual FXString getItemText(const FXXCheckTree* list) const;
  virtual FXString getColumnTooltipText(FXint col);

protected:
  AE_STATEMENT *m_transfer;
};



class FF_TransferTree: public FF_GroupTree {
  friend class FF_TransferTreeItem;
public:
  enum {
    USECOLOR_NORMAL=0,
    USECOLOR_NOTICE,
    USECOLOR_WARNING
  };

  enum {
    COLUMN_NONE=0,
    COLUMN_DATE=1,
    COLUMN_REMOTE_NAME,
    COLUMN_PURPOSE,
    COLUMN_TYPE,
    COLUMN_AMOUNT,
    COLUMN_STATUS,
    COLUMN_DATE_BEGIN,
    COLUMN_DATE_END,
    COLUMN_DATE_LAST,
    COLUMN_DATE_LASTUPDATED,
    COLUMN_PERIOD,

    COLUMN_MAX=100
  };

  FF_TransferTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                  FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_TransferTree();

  void addColumn(int c, int w=100);

  FF_TransferTreeItem *findTransferItem(AQDB_ID id);
  FF_TransferTreeItem *findTransferItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_TransferTreeItem *findTransferItem(const AE_STATEMENT *qba);

  void addTransfer(AE_STATEMENT *st, bool direct, int useColor);
  void addTransfers(AE_STATEMENT_LIST2 *sl, int copyMode, bool direct);

  void addTransferToGroup(AE_STATEMENT *st, AQDB_ID gid, int useColor);
  void addTransferToItem(AE_STATEMENT *st, FF_GroupTreeItem *itemFather, int useColor);

  AE_STATEMENT *getCurrentTransfer();
  AE_STATEMENT *getFirstSelectedTransfer();
  void getSelectedTransfers(AE_STATEMENT_LIST2 *sl, int copyMode);

  void selectTransfer(AQDB_ID id, bool b);
  void selectTransfer(const AE_STATEMENT *st, bool b);
  void makeTransferVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);
  void addColumn(int c);

protected:
  FF_TransferTree();

  static FXint sortFunctionUpDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpRemoteName(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownRemoteName(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpPurpose(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownPurpose(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpInfoText(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownInfoText(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpAmount(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownAmount(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpStatus(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownStatus(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpBeginDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownBeginDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpEndDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownEndDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpLastDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownLastDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  int m_columns;
  int m_columnTypes[COLUMN_MAX];
};



#endif




