/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_valueedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ValueEdit) FF_ValueEditMap[]={
};


// Object implementation
FXIMPLEMENT(FF_ValueEdit, FXTextField, FF_ValueEditMap, ARRAYNUMBER(FF_ValueEditMap))



FF_ValueEdit::FF_ValueEdit(FXComposite *p,
                           FXint cols,
			   FXObject* tgt,
			   FXSelector sel,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl,
			   FXint pr,
			   FXint pt,
			   FXint pb)
:FXTextField(p, cols, tgt, sel, opts)
{
}



FF_ValueEdit::FF_ValueEdit()
:FXTextField()
{
}



FF_ValueEdit::~FF_ValueEdit() {
}



void FF_ValueEdit::setValue(const AQDB_VALUE *v) {
  if (v) {
    GWEN_BUFFER *vbuf;

    vbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AQDB_Value_toHumanReadableString(v, vbuf, 2);
    setText(GWEN_Buffer_GetStart(vbuf));
    GWEN_Buffer_free(vbuf);
  }
  else
    setText("");
}



void FF_ValueEdit::setValueAndUnits(const AQDB_VALUE *v, const char *units) {
  FXString txt;

  if (v) {
    GWEN_BUFFER *vbuf;

    vbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AQDB_Value_toHumanReadableString(v, vbuf, 2);
    txt=FXString(GWEN_Buffer_GetStart(vbuf));
    GWEN_Buffer_free(vbuf);

    if (units && *units) {
      txt+=" ";
      txt+=units;
    }
  }
  setText(txt);
}



AQDB_VALUE *FF_ValueEdit::getValue() {
  FXString s;

  s=getText().trim();
  if (s.empty())
    return NULL;
  else
    return AQDB_Value_fromString(s.text());
}






