/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_VEHICLETREE_HPP
#define FF_VEHICLETREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_vehicle.h>
#include <aqfinance/engine/book/ae_commodity.h>

#include <aqbanking/job.h>



class FF_VehicleTreeItem: public FF_GroupTreeItem {
public:
  FF_VehicleTreeItem(AE_VEHICLE *st, void* ptr=NULL);

  virtual ~FF_VehicleTreeItem();

  AE_VEHICLE *getVehicle() const { return m_vehicle;};

  virtual FXString getItemText(const FXXCheckTree* list) const;

protected:
  AE_VEHICLE *m_vehicle;
};



class FF_VehicleTree: public FF_GroupTree {
  friend class FF_VehicleTreeItem;
public:
  enum {
    USECOLOR_NORMAL=0,
    USECOLOR_NOTICE,
    USECOLOR_WARNING
  };

  enum {
    COLUMN_NONE=0,
    COLUMN_NAME=1,
    COLUMN_MAKE,
    COLUMN_MODEL,
    COLUMN_BUILDYEAR,
    COLUMN_BASE,
    COLUMN_LICENSEPLATE,
    COLUMN_VIN,
    COLUMN_MEMO,
    COLUMN_FUELCAPACITY,
    COLUMN_FUELTYPE,
    COLUMN_BUYDATE,

    COLUMN_MAX=100
  };

  FF_VehicleTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                 FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_VehicleTree();

  void addColumn(int c, int w=100);

  FF_VehicleTreeItem *findVehicleItem(AQDB_ID id);
  FF_VehicleTreeItem *findVehicleItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_VehicleTreeItem *findVehicleItem(const AE_VEHICLE *qba);

  void addVehicle(AE_VEHICLE *st, bool direct, int useColor);
  void addVehicles(AE_VEHICLE_LIST2 *sl, int copyMode, bool direct);

  AE_VEHICLE *getCurrentVehicle();
  AE_VEHICLE *getFirstSelectedVehicle();
  void getSelectedVehicles(AE_VEHICLE_LIST2 *sl, int copyMode);

  void selectVehicle(AQDB_ID id, bool b);
  void selectVehicle(const AE_VEHICLE *st, bool b);
  void makeVehicleVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);
  void addColumn(int c);

  void reload();

protected:
  FF_VehicleTree();

  static FXint sortFunctionUpName(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownName(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpMake(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownMake(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpModel(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownModel(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpBuildYear(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownBuildYear(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpBase(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownBase(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpLicensePlate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownLicensePlate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpIdentificationNumber(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownIdentificationNumber(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpMemo(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownMemo(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpFuelCapacity(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownFuelCapacity(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpBuyDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownBuyDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  int m_columns;
  int m_columnTypes[COLUMN_MAX];
};



#endif




