/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AE_BOOK_H
#define AE_BOOK_H

#include <aqfinance/engine/ae.h>

#include <aqdatabase/aqdb_db.h>

#include <gwenhywfar/inherit.h>
#include <gwenhywfar/xml.h>
#include <gwenhywfar/gwendate.h>


#ifdef __cplusplus
extern "C" {
#endif


/** @defgroup G_AE_ENGINE_BOOK_BOOK Book Module
 * @ingroup G_AE_ENGINE_BOOK
 *
 */
/*@{*/

typedef struct AE_BOOK AE_BOOK;
GWEN_INHERIT_FUNCTION_LIB_DEFS(AE_BOOK, AE_API)


typedef enum {
  AE_Book_TableType_Unknown=-1,
  AE_Book_TableType_None=0,
  AE_Book_TableType_IdCounter,
  AE_Book_TableType_Commodity,
  AE_Book_TableType_Account,
  AE_Book_TableType_Transaction,
  AE_Book_TableType_Split,
  AE_Book_TableType_BankAccount,
  AE_Book_TableType_Address,
  AE_Book_TableType_Partner,

  AE_Book_TableType_Project,
  AE_Book_TableType_Task,
  AE_Book_TableType_Job,

  AE_Book_TableType_Warehouse,
  AE_Book_TableType_Item,
  AE_Book_TableType_BankStatement,
  AE_Book_TableType_BankTransfer,
  AE_Book_TableType_BankDebitNote,

  AE_Book_TableType_BankAccountGroup,

  AE_Book_TableType_BankTransferPlanned,
  AE_Book_TableType_BankTransferTemplate,
  AE_Book_TableType_BankTransferGroup,
  AE_Book_TableType_BookedBalance,
  AE_Book_TableType_NotedBalance,

  AE_Book_TableType_Rule,

  AE_Book_TableType_PartnerGroup,
  AE_Book_TableType_PartnerRule,

  AE_Book_TableType_Tax,
  AE_Book_TableType_TaxGroup,

  AE_Book_TableType_ItemTemplates,
  AE_Book_TableType_ItemTemplateGroup,

  AE_Book_TableType_AccountBalance,

  AE_Book_TableType_BankStandingOrder,
  AE_Book_TableType_BankStoRecurrence,

  AE_Book_TableType_Vehicle,
  AE_Book_TableType_VehicleGroup,
  AE_Book_TableType_Refueling,

  AE_Book_TableType_GraphicReportGroup,
  AE_Book_TableType_GraphicReportSettings,

  AE_Book_TableType_Share,

  AE_Book_TableType_Last
} AE_BOOK_TABLE_TYPE;


#include <aqfinance/engine/book/ae_group.h>
#include <aqfinance/engine/book/ae_bankaccount.h>
#include <aqfinance/engine/book/ae_statement.h>
#include <aqfinance/engine/book/ae_bankbalance.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_account.h>
#include <aqfinance/engine/book/ae_accountbalance.h>
#include <aqfinance/engine/book/ae_transaction.h>
#include <aqfinance/engine/book/ae_split.h>
#include <aqfinance/engine/book/ae_rule.h>
#include <aqfinance/engine/book/ae_address.h>
#include <aqfinance/engine/book/ae_partner.h>
#include <aqfinance/engine/book/ae_task.h>
#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_item.h>
#include <aqfinance/engine/book/ae_tax.h>
#include <aqfinance/engine/book/ae_vehicle.h>
#include <aqfinance/engine/book/ae_refueling.h>
#include <aqfinance/engine/book/ae_load_cellphone.h>
#include <aqfinance/engine/book/ae_recurrence.h>
#include <aqfinance/engine/book/ae_dbconflict.h>
#include <aqfinance/engine/book/ae_data_10k.h>
#include <aqfinance/engine/book/ae_share.h>

#include <aqfinance/report/af_report_data.h>
#include <aqfinance/report/af_report_cfg.h>

#include <aqfinance/report2/af_report2.h>



/** @name Constructor, Destructor
 *
 */
/*@{*/
AE_API
AE_BOOK *AE_Book_new(AQDB_DB *db);

AE_API
void AE_Book_free(AE_BOOK *bk);

AE_API
void AE_Book_Attach(AE_BOOK *bk);
/*@}*/


/** @name Book Opening and Closing
 *
 */
/*@{*/
AE_API
int AE_Book_Open(AE_BOOK *bk, uint32_t flags);

AE_API
int AE_Book_Create(AE_BOOK *bk, uint32_t flags);

AE_API
int AE_Book_Close(AE_BOOK *bk, uint32_t flags);
/*@}*/


/** @name Table Opening and Closing
 *
 */
/*@{*/
AE_API
int AE_Book_CreateTable(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, uint32_t flags);

AE_API
int AE_Book_OpenTable(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, uint32_t flags);

AE_API
int AE_Book_IsTableOpen(AE_BOOK *bk,
                        AE_BOOK_TABLE_TYPE tt,
                        uint32_t flags);

AE_API
AQDB_COLUMN_LIST* AE_Book_GetColumnList(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt);

AE_API
int AE_Book_CloseTable(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, uint32_t flags);
/*@}*/


/** @name Direct Database Access
 *
 */
/*@{*/
AE_API
AQDB_DB *AE_Book_GetDataBase(const AE_BOOK *bk);

AE_API
AQDB_ID AE_Book_GetTableId(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt);

AE_API
const char *AE_Book_GetTableName(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt);

AE_API
AE_BOOK_TABLE_TYPE AE_Book_GetTableType(const AE_BOOK *bk, const char *s);

AE_API
int AE_Book_GetNextId(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, AQDB_ID *pId);

AE_API
int AE_Book_GetLastId(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, AQDB_ID *pId);

AE_API
int AE_Book_BeginEdit(AE_BOOK *bk, uint32_t flags);

AE_API
int AE_Book_EndEdit(AE_BOOK *bk, uint32_t flags);

/*@}*/



/** @name Browsing the Database
 *
 */
/*@{*/

typedef void* GWENHYWFAR_CB (*AE_BOOK_FOR_EVERY_QUERY_OBJECT_FN)(AE_BOOK *bk,
								 AE_BOOK_TABLE_TYPE tt,
								 AQDB_OBJECT *o,
								 void *data);

/**
 * This function calls the given callback function for every object
 * which matches the given query expression until the callback function
 * returns non-NULL.
 * @param bk pointer to book object
 * @param tt table type
 * @param refQueryId id of reference query (0 if none). This can be used to
 *   create subqueries.
 * @param qexpr use NULL to match all objects
 * @param fn callback function to call
 * @param data 4th argument to callback function
 * @param pResult pointer to a variable which receives the result returned by the
 *   last invocation of the callback function (e.g. the non-NULL value returned by that
 *   call).
 */
AE_API
int AE_Book_ForEveryQueryObject(AE_BOOK *bk,
				AE_BOOK_TABLE_TYPE tt,
				AQDB_ID refQueryId,
				const char *qexpr,
				AE_BOOK_FOR_EVERY_QUERY_OBJECT_FN fn,
				void *data,
				void **pResult);

AE_API
int AE_Book_GetFirstDbIdForQuery(AE_BOOK *bk,
				 AE_BOOK_TABLE_TYPE tt,
				 const char *qexpr,
				 AQDB_ID *pId);

AE_API
int AE_Book_GetFirstObjectForQuery(AE_BOOK *bk,
				   AE_BOOK_TABLE_TYPE tt,
				   const char *qexpr,
				   AQDB_OBJECT **pObj);

/*@}*/



AE_API
int AE_Book_GetConflicts(AE_BOOK *b,
			 AQDB_ACTION_TYPE action,
			 AE_BOOK_TABLE_TYPE tt,
			 AQDB_ID id,
			 AE_DBCONFLICT_LIST *clist);



/** @name Object Functions
 *
 */
/*@{*/

AE_API
int AE_Book_ReadGroup(AE_BOOK *b,
                      AE_BOOK_TABLE_TYPE tt,
                      AQDB_ID elemId,
                      AE_GROUP **pst);

AE_API
int AE_Book_ReadGroupList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AE_GROUP_LIST *lst);

AE_API
int AE_Book_QueryGroupList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                           AQDB_ID refQueryId,
                           const char *qexpr,
                           AE_GROUP_LIST *elist);

AE_API
int AE_Book_ReadGroupTree(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt, AE_GROUP_TREE *atree);


AE_API
int AE_Book_AddGroup(AE_BOOK *b,
                     AE_BOOK_TABLE_TYPE tt,
                     AE_GROUP *gr,
                     int doLock);

AE_API
int AE_Book_WriteGroup(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_GROUP *gr,
                       int doLock);

AE_API
int AE_Book_DeleteGroup(AE_BOOK *b,
                        AE_BOOK_TABLE_TYPE tt,
                        const AE_GROUP *gr,
                        int doLock);




AE_API
int AE_Book_ReadBankAccountList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                AE_BANKACCOUNT_LIST *balist);

AE_API
int AE_Book_QueryBankAccountList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                 AQDB_ID refQueryId,
                                 const char *qexpr,
                                 AE_BANKACCOUNT_LIST *elist);


AE_API
int AE_Book_ReadBankAccount(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                            AQDB_ID id,
                            AE_BANKACCOUNT **pba);

AE_API
int AE_Book_AddBankAccount(AE_BOOK *b,
                           AE_BOOK_TABLE_TYPE tt,
                           AE_BANKACCOUNT *ba,
                           int doLock);

AE_API
int AE_Book_WriteBankAccount(AE_BOOK *b,
                             AE_BOOK_TABLE_TYPE tt,
                             const AE_BANKACCOUNT *ba,
                             int doLock);

AE_API
int AE_Book_DeleteBankAccount(AE_BOOK *b,
                              AE_BOOK_TABLE_TYPE tt,
                              const AE_BANKACCOUNT *ba,
                              int doLock);





AE_API
int AE_Book_ReadStatement(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID stmId,
                          AE_STATEMENT **pst);

AE_API
int AE_Book_ReadStatementList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                              AE_STATEMENT_LIST *stlist);

AE_API
int AE_Book_QueryStatementList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                               AQDB_ID refQueryId,
                               const char *qexpr,
                               AE_STATEMENT_LIST *stlist);


/**
 * Returns the statement with the latest date. This is used when looking for a start date
 * for online banking requests of transaction reports.
 * The object returned (if any) must be freed.
 */
AE_API
int AE_Book_FindLatestStatement(AE_BOOK *b, const char *queryString, AE_STATEMENT **pStatement);


AE_API
int AE_Book_AddStatement(AE_BOOK *b,
                         AE_BOOK_TABLE_TYPE tt,
                         AE_STATEMENT *stm,
                         int doLock);

AE_API
int AE_Book_WriteStatement(AE_BOOK *b,
                           AE_BOOK_TABLE_TYPE tt,
                           const AE_STATEMENT *stm,
                           int doLock);

AE_API
int AE_Book_DeleteStatement(AE_BOOK *b,
                            AE_BOOK_TABLE_TYPE tt,
                            const AE_STATEMENT *stm,
                            int doLock);




AE_API
int AE_Book_ReadBankBalance(AE_BOOK *b,
			    AE_BOOK_TABLE_TYPE tt,
			    AQDB_ID stmId,
			    AE_BANKBALANCE **pbal);

AE_API
int AE_Book_ReadBankBalanceList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
				AE_BANKBALANCE_LIST *elist);

AE_API
int AE_Book_QueryBankBalanceList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
				 AQDB_ID refQueryId,
				 const char *qexpr,
				 AE_BANKBALANCE_LIST *elist);

AE_API
int AE_Book_AddBankBalance(AE_BOOK *b,
			   AE_BOOK_TABLE_TYPE tt,
			   AE_BANKBALANCE *ba,
			   int doLock);

AE_API
int AE_Book_WriteBankBalance(AE_BOOK *b,
			     AE_BOOK_TABLE_TYPE tt,
			     const AE_BANKBALANCE *ba,
			     int doLock);

AE_API
int AE_Book_DeleteBankBalance(AE_BOOK *b,
			      AE_BOOK_TABLE_TYPE tt,
			      const AE_BANKBALANCE *ba,
			      int doLock);


AE_API
int AE_Book_ReadAccountBalance(AE_BOOK *b,
			       AE_BOOK_TABLE_TYPE tt,
			       AQDB_ID stmId,
			       AE_ACCOUNTBALANCE **pbal);

AE_API
int AE_Book_ReadAccountBalanceList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
				   AE_ACCOUNTBALANCE_LIST *elist);

AE_API
int AE_Book_QueryAccountBalanceList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
				    AQDB_ID refQueryId,
				    const char *qexpr,
				    AE_ACCOUNTBALANCE_LIST *elist);

AE_API
int AE_Book_AddAccountBalance(AE_BOOK *b,
			      AE_BOOK_TABLE_TYPE tt,
			      AE_ACCOUNTBALANCE *ba,
			      int doLock);

AE_API
int AE_Book_WriteAccountBalance(AE_BOOK *b,
				AE_BOOK_TABLE_TYPE tt,
				const AE_ACCOUNTBALANCE *ba,
				int doLock);

AE_API
int AE_Book_DeleteAccountBalance(AE_BOOK *b,
				 AE_BOOK_TABLE_TYPE tt,
				 const AE_ACCOUNTBALANCE *ba,
				 int doLock);


AE_API
int AE_Book_ReadCommodity(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID elemId,
                          AE_COMMODITY **pst);

AE_API
int AE_Book_ReadCommodityList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                              AE_COMMODITY_LIST *lst);

AE_API
int AE_Book_QueryCommodityList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                               AQDB_ID refQueryId,
                               const char *qexpr,
                               AE_COMMODITY_LIST *elist);


AE_API
int AE_Book_AddCommodity(AE_BOOK *b,
                         AE_BOOK_TABLE_TYPE tt,
                         AE_COMMODITY *elem,
                         int doLock);

AE_API
int AE_Book_WriteCommodity(AE_BOOK *b,
                           AE_BOOK_TABLE_TYPE tt,
                           const AE_COMMODITY *elem,
                           int doLock);

AE_API
int AE_Book_DeleteCommodity(AE_BOOK *b,
                            AE_BOOK_TABLE_TYPE tt,
                            const AE_COMMODITY *elem,
                            int doLock);



AE_API
int AE_Book_ReadTransaction(AE_BOOK *b,
                            AE_BOOK_TABLE_TYPE tt,
                            AQDB_ID elemId,
                            AE_TRANSACTION **pst);

AE_API
int AE_Book_ReadTransactionList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                AE_TRANSACTION_LIST *lst);

AE_API
int AE_Book_QueryTransactionList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                 AQDB_ID refQueryId,
                                 const char *qexpr,
                                 AE_TRANSACTION_LIST *elist);

AE_API
int AE_Book_ReadTransactionsToListAndMap(AE_BOOK *b,
                                         AQDB_ID refQueryId,
                                         const char *qString,
                                         AE_TRANSACTION_LIST *tList,
                                         AE_TRANSACTION_IDMAP *tMap);


/**
 * This is a complex function. It reads all splits matching the given date span and commodity.
 * The splits returned have all their runtime objects containing "tmp" in their name filled
 * (like "tmpTransaction", "tmpAccountPath" etc).
 */
AE_API
int AE_Book_ReadSplitListByDateAndCommodity(AE_BOOK *b,
                                            const GWEN_DATE *fromDate,
                                            const GWEN_DATE *toDate,
                                            const char *commodity,
                                            AE_SPLIT_LIST *splitList);

/**
 * This is a complex function. It reads all splits matching the given date span
 * and account path pattern.
 * Internally calls @ref AE_Book_ReadSplitListByDateAndCommodity().
 */
AE_API
int AE_Book_ReadSplitListByDateAndAccount(AE_BOOK *b,
                                          const GWEN_DATE *fromDate,
                                          const GWEN_DATE *toDate,
                                          const char *accountPattern,
                                          const char *commodity,
                                          AE_SPLIT_LIST *splitList);

/**
 * This is a complex function. It reads all splits matching the given date span, commodity
 * and account id.
 * Internally calls @ref AE_Book_ReadSplitListByDateAndCommodity().
 */
AE_API
int AE_Book_ReadSplitListByDateAndAccountId(AE_BOOK *b,
                                            const GWEN_DATE *fromDate,
                                            const GWEN_DATE *toDate,
                                            AQDB_ID accountId,
                                            const char *commodity,
                                            AE_SPLIT_LIST *splitList);

/**
 * This is a complex function. It reads all splits matching the given date span and commodity
 * and whose account in the given AE_ACCOUNT_TREE has the runtime flag AE_ACCOUNT_RTFLAGS_SELECTED set.
 * Internally calls @ref AE_Book_ReadSplitListByDateAndCommodity().
 */
AE_API int AE_Book_ReadSplitListForSelectedAccounts(AE_BOOK *b,
                                                    const AE_ACCOUNT_TREE *at,
                                                    const GWEN_DATE *fromDate,
                                                    const GWEN_DATE *toDate,
                                                    const char *commodity,
                                                    AE_SPLIT_LIST *splitList);




/**
 * This function reads all splits belonging to the given transaction and stores
 * them in the transactions's runtime split list.
 * If there already was a split list stored with the transaction then that list is cleared
 * before reading the splits.
 * @param b book to read the splits from
 * @param t transaction whose splits are to be read
 * @param tt table type of the splits to read (most likely AE_Book_TableType_Split)
 * @return 0 if ok, <0 in case of an error
 */
AE_API
int AE_Book_ReadTransactionSplits(AE_BOOK *b,
                                  AE_TRANSACTION *t,
                                  AE_BOOK_TABLE_TYPE tt);

/**
 * This function reads all sub-objects of a transaction (splits and statement)
 */
AE_API
int AE_Book_ReadTransactionSubObjects(AE_BOOK *b, AE_TRANSACTION *t);


AE_API
int AE_Book_AddTransaction(AE_BOOK *b,
                           AE_BOOK_TABLE_TYPE tt,
                           AE_TRANSACTION *elem,
                           int doLock);

AE_API
int AE_Book_WriteTransaction(AE_BOOK *b,
                             AE_BOOK_TABLE_TYPE tt,
                             const AE_TRANSACTION *elem,
                             int doLock);

AE_API
int AE_Book_DeleteTransaction(AE_BOOK *b,
                              AE_BOOK_TABLE_TYPE tt,
                              const AE_TRANSACTION *elem,
                              int doLock);

AE_API
int AE_Book_ApplyTransaction(AE_BOOK *b,
			     AE_TRANSACTION *tNew,
			     const AE_TRANSACTION *tOld,
			     int doLock);

AE_API
int AE_Book_ApplyTransactionWithBalances(AE_BOOK *b,
					 AE_ACCOUNT_TREE *at,
					 AE_TRANSACTION *tNew,
					 const AE_TRANSACTION *tOld,
					 int doLock);


AE_API
int AE_Book_ApplyTransactionToAccountTree(AE_BOOK *b,
					  AE_TRANSACTION *tNew, const AE_TRANSACTION *tOld,
					  AE_ACCOUNT_TREE *at);




AE_API
int AE_Book_ReadSplit(AE_BOOK *b,
                      AE_BOOK_TABLE_TYPE tt,
                      AQDB_ID elemId,
                      AE_SPLIT **pst);

AE_API
int AE_Book_ReadSplitList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AE_SPLIT_LIST *lst);

AE_API
int AE_Book_QuerySplitList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                           AQDB_ID refQueryId,
                           const char *qexpr,
                           AE_SPLIT_LIST *elist);


AE_API
int AE_Book_AddSplit(AE_BOOK *b,
                     AE_BOOK_TABLE_TYPE tt,
                     AE_SPLIT *elem,
                     int doLock);

AE_API
int AE_Book_WriteSplit(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_SPLIT *elem,
                       int doLock);

AE_API
int AE_Book_DeleteSplit(AE_BOOK *b,
                        AE_BOOK_TABLE_TYPE tt,
                        const AE_SPLIT *elem,
                        int doLock);

AE_API
int AE_Book_ReadSplitListSubObjects(AE_BOOK *b, AE_SPLIT_LIST *sl);



AE_API
int AE_Book_ReadAccount(AE_BOOK *b,
                        AE_BOOK_TABLE_TYPE tt,
                        AQDB_ID elemId,
                        AE_ACCOUNT **pst);

AE_API
int AE_Book_ReadAccountList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                            AE_ACCOUNT_LIST *lst);

AE_API
int AE_Book_QueryAccountList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                             AQDB_ID refQueryId,
                             const char *qexpr,
                             AE_ACCOUNT_LIST *elist);

AE_API
int AE_Book_ReadAccountTree(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt, AE_ACCOUNT_TREE *atree);

AE_API
int AE_Book_SortAccountsIntoTree(AE_ACCOUNT_TREE *atree,
				 AE_ACCOUNT *parent,
				 AE_ACCOUNT_LIST *alist);


AE_API
int AE_Book_ReadAccountTreeBalances(AE_BOOK *b, AE_ACCOUNT_TREE *at);

AE_API
int AE_Book_WriteAccountTreeBalances(AE_BOOK *b, AE_ACCOUNT_TREE *at, int doLock);

AE_API
int AE_Book_CalcAccountTreeBalances(AE_BOOK *b, AE_ACCOUNT_TREE *at);



AE_API
int AE_Book_AddAccount(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       AE_ACCOUNT *elem,
                       int doLock);

AE_API
int AE_Book_WriteAccount(AE_BOOK *b,
                         AE_BOOK_TABLE_TYPE tt,
                         const AE_ACCOUNT *elem,
                         int doLock);

AE_API
int AE_Book_DeleteAccount(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          const AE_ACCOUNT *elem,
                          int doLock);


AE_API int AE_Book_ReadAccountTreeXml(AE_BOOK *b, AE_ACCOUNT_TREE *at, GWEN_XMLNODE *node);
AE_API void AE_Book_WriteAccountTreeXml(AE_BOOK *b,
					const AE_ACCOUNT_TREE *at,
                                        const AE_ACCOUNT *acc,
					GWEN_XMLNODE *node);


AE_API
int AE_Book_ReadRule(AE_BOOK *b,
                     AE_BOOK_TABLE_TYPE tt,
                     AQDB_ID elemId,
                     AE_RULE **pst);

AE_API
int AE_Book_ReadRuleList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                         AE_RULE_LIST *lst);

AE_API
int AE_Book_QueryRuleList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID refQueryId,
                          const char *qexpr,
                          AE_RULE_LIST *elist);


AE_API
int AE_Book_AddRule(AE_BOOK *b,
                    AE_BOOK_TABLE_TYPE tt,
                    AE_RULE *elem,
                    int doLock);

AE_API
int AE_Book_WriteRule(AE_BOOK *b,
                      AE_BOOK_TABLE_TYPE tt,
                      const AE_RULE *elem,
                      int doLock);

AE_API
int AE_Book_DeleteRule(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_RULE *elem,
                       int doLock);


AE_API
int AE_Book_ReadAddress(AE_BOOK *b,
			AE_BOOK_TABLE_TYPE tt,
			AQDB_ID elemId,
			AE_ADDRESS **pst);

AE_API
int AE_Book_ReadAddressList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
			    AE_ADDRESS_LIST *lst);

AE_API
int AE_Book_QueryAddressList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
			     AQDB_ID refQueryId,
			     const char *qexpr,
			     AE_ADDRESS_LIST *elist);


AE_API
int AE_Book_AddAddress(AE_BOOK *b,
		       AE_BOOK_TABLE_TYPE tt,
		       AE_ADDRESS *elem,
		       int doLock);

AE_API
int AE_Book_WriteAddress(AE_BOOK *b,
			 AE_BOOK_TABLE_TYPE tt,
			 const AE_ADDRESS *elem,
			 int doLock);

AE_API
int AE_Book_DeleteAddress(AE_BOOK *b,
			  AE_BOOK_TABLE_TYPE tt,
			  const AE_ADDRESS *elem,
			  int doLock);



AE_API
int AE_Book_ReadPartner(AE_BOOK *b,
			AE_BOOK_TABLE_TYPE tt,
			AQDB_ID elemId,
			AE_PARTNER **pst);

AE_API
int AE_Book_ReadPartnerList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
			    AE_PARTNER_LIST *lst);

AE_API
int AE_Book_QueryPartnerList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
			     AQDB_ID refQueryId,
			     const char *qexpr,
			     AE_PARTNER_LIST *elist);


AE_API
int AE_Book_AddPartner(AE_BOOK *b,
		       AE_BOOK_TABLE_TYPE tt,
		       AE_PARTNER *elem,
		       int doLock);

AE_API
int AE_Book_WritePartner(AE_BOOK *b,
			 AE_BOOK_TABLE_TYPE tt,
			 const AE_PARTNER *elem,
			 int doLock);

AE_API
int AE_Book_DeletePartner(AE_BOOK *b,
			  AE_BOOK_TABLE_TYPE tt,
			  const AE_PARTNER *elem,
			  int doLock);

AE_API int AE_Book_ApplyPartner(AE_BOOK *b, AE_PARTNER *ptNew, const AE_PARTNER *ptOld, int doLock);

AE_API int AE_Book_ReadPartnerSubObjects(AE_BOOK *b, AE_PARTNER *pt);



AE_API
int AE_Book_ReadTask(AE_BOOK *b,
                     AE_BOOK_TABLE_TYPE tt,
                     AQDB_ID elemId,
                     AE_TASK **pst);

AE_API
int AE_Book_ReadTaskList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                         AE_TASK_LIST *lst);

AE_API
int AE_Book_QueryTaskList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID refQueryId,
                          const char *qexpr,
                          AE_TASK_LIST *elist);


AE_API
int AE_Book_AddTask(AE_BOOK *b,
                    AE_BOOK_TABLE_TYPE tt,
                    AE_TASK *elem,
                    int doLock);

AE_API
int AE_Book_WriteTask(AE_BOOK *b,
                      AE_BOOK_TABLE_TYPE tt,
                      const AE_TASK *elem,
                      int doLock);

AE_API
int AE_Book_DeleteTask(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_TASK *elem,
                       int doLock);

AE_API
int AE_Book_ReadTaskSubObjects(AE_BOOK *b, AE_TASK *tk, int buy);



AE_API
int AE_Book_ReadJob(AE_BOOK *b,
		    AE_BOOK_TABLE_TYPE tt,
		    AQDB_ID elemId,
		    AE_JOB **pst);

AE_API
int AE_Book_ReadJobList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                         AE_JOB_LIST *lst);

AE_API
int AE_Book_QueryJobList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID refQueryId,
                          const char *qexpr,
                          AE_JOB_LIST *elist);


AE_API
int AE_Book_AddJob(AE_BOOK *b,
                    AE_BOOK_TABLE_TYPE tt,
                    AE_JOB *elem,
                    int doLock);

AE_API
int AE_Book_WriteJob(AE_BOOK *b,
		     AE_BOOK_TABLE_TYPE tt,
		     const AE_JOB *elem,
		     int doLock);

AE_API
int AE_Book_DeleteJob(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_JOB *elem,
                       int doLock);

AE_API
int AE_Book_ApplyJob(AE_BOOK *b, AE_JOB *jNew, const AE_JOB *jOld, int doLock);

AE_API
int AE_Book_ReadJobSubObjects(AE_BOOK *b, AE_JOB *j, int buy);


AE_API
int AE_Book_ReadItem(AE_BOOK *b,
                     AE_BOOK_TABLE_TYPE tt,
                     AQDB_ID elemId,
                     AE_ITEM **pst);

AE_API
int AE_Book_ReadItemList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                         AE_ITEM_LIST *lst);

AE_API
int AE_Book_QueryItemList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID refQueryId,
                          const char *qexpr,
                          AE_ITEM_LIST *elist);


AE_API
int AE_Book_AddItem(AE_BOOK *b,
                    AE_BOOK_TABLE_TYPE tt,
                    AE_ITEM *elem,
                    int doLock);

AE_API
int AE_Book_WriteItem(AE_BOOK *b,
                      AE_BOOK_TABLE_TYPE tt,
                      const AE_ITEM *elem,
                      int doLock);

AE_API
int AE_Book_DeleteItem(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_ITEM *elem,
                       int doLock);



AE_API
int AE_Book_ReadTax(AE_BOOK *b,
		    AE_BOOK_TABLE_TYPE tt,
		    AQDB_ID elemId,
		    AE_TAX **pst);

AE_API
int AE_Book_ReadTaxList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
			AE_TAX_LIST *lst);

AE_API
int AE_Book_QueryTaxList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
			 AQDB_ID refQueryId,
			 const char *qexpr,
			 AE_TAX_LIST *elist);


AE_API
int AE_Book_AddTax(AE_BOOK *b,
		   AE_BOOK_TABLE_TYPE tt,
		   AE_TAX *elem,
		   int doLock);

AE_API
int AE_Book_WriteTax(AE_BOOK *b,
		     AE_BOOK_TABLE_TYPE tt,
		     const AE_TAX *elem,
		     int doLock);

AE_API
int AE_Book_DeleteTax(AE_BOOK *b,
		      AE_BOOK_TABLE_TYPE tt,
		      const AE_TAX *elem,
		      int doLock);



AE_API
int AE_Book_ReadLoadCellPhone(AE_BOOK *b,
                              AE_BOOK_TABLE_TYPE tt,
                              AQDB_ID elemId,
                              AE_LOAD_CELLPHONE **pst);

AE_API
int AE_Book_ReadLoadCellPhoneList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                  AE_LOAD_CELLPHONE_LIST *lst);

AE_API
int AE_Book_QueryLoadCellPhoneList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                   AQDB_ID refQueryId,
                                   const char *qexpr,
                                   AE_LOAD_CELLPHONE_LIST *elist);


AE_API
int AE_Book_AddLoadCellPhone(AE_BOOK *b,
                             AE_BOOK_TABLE_TYPE tt,
                             AE_LOAD_CELLPHONE *elem,
                             int doLock);

AE_API
int AE_Book_WriteLoadCellPhone(AE_BOOK *b,
                               AE_BOOK_TABLE_TYPE tt,
                               const AE_LOAD_CELLPHONE *elem,
                               int doLock);

AE_API
int AE_Book_DeleteLoadCellPhone(AE_BOOK *b,
                                AE_BOOK_TABLE_TYPE tt,
                                const AE_LOAD_CELLPHONE *elem,
                                int doLock);




AE_API
int AE_Book_ReadRecurrence(AE_BOOK *b,
                           AE_BOOK_TABLE_TYPE tt,
                           AQDB_ID elemId,
                           AE_RECURRENCE **pst);

AE_API
int AE_Book_ReadRecurrenceList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                               AE_RECURRENCE_LIST *lst);

AE_API
int AE_Book_QueryRecurrenceList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                                AQDB_ID refQueryId,
                                const char *qexpr,
                                AE_RECURRENCE_LIST *elist);


AE_API
int AE_Book_AddRecurrence(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          AE_RECURRENCE *elem,
                          int doLock);

AE_API
int AE_Book_WriteRecurrence(AE_BOOK *b,
                            AE_BOOK_TABLE_TYPE tt,
                            const AE_RECURRENCE *elem,
                            int doLock);

AE_API
int AE_Book_DeleteRecurrence(AE_BOOK *b,
                             AE_BOOK_TABLE_TYPE tt,
                             const AE_RECURRENCE *elem,
                             int doLock);

AE_API
int AE_Book_ReadBankStandingOrderSubObjects(AE_BOOK *b, AE_STATEMENT *st);

AE_API
int AE_Book_ReadBankStandingOrderListSubObjects(AE_BOOK *b, AE_STATEMENT_LIST *sl);




AE_API
int AE_Book_ReadVehicle(AE_BOOK *b,
                        AE_BOOK_TABLE_TYPE tt,
                        AQDB_ID elemId,
                        AE_VEHICLE **pst);

AE_API
int AE_Book_ReadVehicleList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                            AE_VEHICLE_LIST *lst);

AE_API
int AE_Book_QueryVehicleList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                             AQDB_ID refQueryId,
                             const char *qexpr,
                             AE_VEHICLE_LIST *elist);


AE_API
int AE_Book_AddVehicle(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       AE_VEHICLE *elem,
                       int doLock);

AE_API
int AE_Book_WriteVehicle(AE_BOOK *b,
                         AE_BOOK_TABLE_TYPE tt,
                         const AE_VEHICLE *elem,
                         int doLock);

AE_API
int AE_Book_DeleteVehicle(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          const AE_VEHICLE *elem,
                          int doLock);



AE_API
int AE_Book_ReadRefueling(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          AQDB_ID elemId,
                          AE_REFUELING **pst);

AE_API
int AE_Book_ReadRefuelingList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                              AE_REFUELING_LIST *lst);

AE_API
int AE_Book_QueryRefuelingList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                               AQDB_ID refQueryId,
                               const char *qexpr,
                               AE_REFUELING_LIST *elist);


AE_API
int AE_Book_AddRefueling(AE_BOOK *b,
                         AE_BOOK_TABLE_TYPE tt,
                         AE_REFUELING *elem,
                         int doLock);

AE_API
int AE_Book_WriteRefueling(AE_BOOK *b,
                           AE_BOOK_TABLE_TYPE tt,
                           const AE_REFUELING *elem,
                           int doLock);

AE_API
int AE_Book_DeleteRefueling(AE_BOOK *b,
                            AE_BOOK_TABLE_TYPE tt,
                            const AE_REFUELING *elem,
                            int doLock);


AE_API
int AE_Book_ReadData10k(AE_BOOK *b,
                        AE_BOOK_TABLE_TYPE tt,
                        AQDB_ID elemId,
                        AE_DATA_10K **pst);

AE_API
int AE_Book_ReadData10kList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                            AE_DATA_10K_LIST *lst);

AE_API
int AE_Book_QueryData10kList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                             AQDB_ID refQueryId,
                             const char *qexpr,
                             AE_DATA_10K_LIST *elist);


AE_API
int AE_Book_AddData10k(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       AE_DATA_10K *elem,
                       int doLock);

AE_API
int AE_Book_WriteData10k(AE_BOOK *b,
                         AE_BOOK_TABLE_TYPE tt,
                         const AE_DATA_10K *elem,
                         int doLock);

AE_API
int AE_Book_DeleteData10k(AE_BOOK *b,
                          AE_BOOK_TABLE_TYPE tt,
                          const AE_DATA_10K *elem,
                          int doLock);



AE_API
int AE_Book_ReadShare(AE_BOOK *b,
                      AE_BOOK_TABLE_TYPE tt,
                      AQDB_ID elemId,
                      AE_SHARE **pst);

AE_API
int AE_Book_ReadShareList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                          AE_SHARE_LIST *lst);

AE_API
int AE_Book_QueryShareList(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,
                           AQDB_ID refQueryId,
                           const char *qexpr,
                           AE_SHARE_LIST *elist);


AE_API
int AE_Book_AddShare(AE_BOOK *b,
                     AE_BOOK_TABLE_TYPE tt,
                     AE_SHARE *elem,
                     int doLock);

AE_API
int AE_Book_WriteShare(AE_BOOK *b,
                       AE_BOOK_TABLE_TYPE tt,
                       const AE_SHARE *elem,
                       int doLock);

AE_API
int AE_Book_DeleteShare(AE_BOOK *b,
                        AE_BOOK_TABLE_TYPE tt,
                        const AE_SHARE *elem,
                        int doLock);


/*@}*/




/** @name Report Functions
 *
 */
/*@{*/


#define AE_BOOK_REPORT_FLAG_ADD_POS_AMT 0x00000001
#define AE_BOOK_REPORT_FLAG_ADD_NEG_AMT 0x00000002
#define AE_BOOK_REPORT_FLAG_ADD_SUM_AMT 0x00000004



AE_API
void AE_Book_Report_CreateColumns(AE_BOOK *b,
				  const AF_REPORT_CFG *cfg,
				  AF_REPORT_DATA *rd);


AE_API
int AE_Book_Report_DbToDataPair(AE_BOOK *b,
				const AF_REPORT_CFG *cfg,
                                AF_REPORT_DATAPAIR *dp,
                                GWEN_DB_NODE *dbData);



AE_API
void AE_Book_Report_AddStatementsToReportData(AE_BOOK *b,
                                              AF_REPORT_DATA *rd,
					      const AF_REPORT_CFG *cfg,
                                              const AE_STATEMENT_LIST *sl);

AE_API
void AE_Book_Report_AddSplitsToReportData(AE_BOOK *b,
                                          AF_REPORT_DATA *rd,
                                          const AF_REPORT_CFG *cfg,
                                          const AE_SPLIT_LIST *sl);

AE_API
int AE_Book_Report_SumSplitsIntoReportData(AE_BOOK *b,
                                           AF_REPORT_DATA *rd,
                                           const AF_REPORT_CFG *cfg,
                                           const AE_SPLIT_LIST *sl);


AE_API
void AE_Book_Report_ToFlatList(AE_BOOK *b, AF_REPORT_DATA *rd);


AE_API
void AE_Book_Report_ToFlatDayList(AE_BOOK *b, AF_REPORT_DATA *rd, uint32_t flags);

AE_API
void AE_Book_Report_ToFlatMonthList(AE_BOOK *b, AF_REPORT_DATA *rd, uint32_t flags);


/**
 * Creates a a flat list with the following columns:
 * <ul>
 *   <li>Category Name</li>
 *   <li>Positive Amount</li>
 *   <li>Negative Amount</li>
 *   <li>Sum Amount (balance)</li>
 * </ul>
 */
AE_API
void AE_Book_Report_ToFlatCategoryList(AE_BOOK *b, AF_REPORT_DATA *rd, uint32_t flags);


/*@}*/



/** @name Report Functions (2nd generation)
 *
 */
/*@{*/


#define AE_BOOK_REPORT2_FLAG_ADD_POS_AMT      0x00000001

AE_API int AF_Book_Report2_AccountTree2CategoryTree(AE_BOOK *bk, AF_REPORT2_CATEGORY_TREE *catTree);

AE_API int AE_Book_Report2_ReadMatchingSplits(AE_BOOK *b, AF_REPORT2 *rp, 
                                              const GWEN_DATE *fromDate,
                                              const GWEN_DATE *toDate,
                                              const char *accountPattern,
                                              const char *commodity);

AE_API void AF_Book_Report2_SplitList2Report(AE_BOOK *bk, AF_REPORT2 *rp, const AE_SPLIT_LIST *spList);

/*@}*/




/*@}*/ /* defgroup */


#ifdef __cplusplus
}
#endif


#endif
