/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */


int AE_Book_ApplyPartner(AE_BOOK *b, AE_PARTNER *ptNew, const AE_PARTNER *ptOld, int doLock) {
  AQDB_ID id;
  int rv;
  AE_ADDRESS *addr;
  AE_BANKACCOUNT *ba;
  AE_RULE *rule;

  /* possibly lock */
  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  /* add or write partner */
  id=AE_Partner_GetId(ptNew);
  if (id) {
    rv=AE_Book_WritePartner(b, AE_Book_TableType_Partner, ptNew, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }
  else {
    /* new partner */
    rv=AE_Book_AddPartner(b, AE_Book_TableType_Partner, ptNew, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
    id=AE_Partner_GetId(ptNew);
  }

  /* write new addresses */
  addr=AE_Partner_GetFirstAddress(ptNew);
  while(addr) {
    AQDB_ID aid;

    /* store owner type and id */
    AE_Address_SetOwnerType(addr, AE_Book_TableType_Partner);
    AE_Address_SetOwnerId(addr, id);

    /* write or add address */
    aid=AE_Address_GetId(addr);
    if (aid) {
      rv=AE_Book_WriteAddress(b, AE_Book_TableType_Address, addr, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }
    else {
      rv=AE_Book_AddAddress(b, AE_Book_TableType_Address, addr, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }

    addr=AE_Address_List_Next(addr);
  }

  /* remove deleted addresses */
  if (ptOld) {
    /* for every old address: check whether it still exists in the new object,
     * delete it if not */
    addr=AE_Partner_GetFirstAddress(ptOld);
    while(addr) {
      AQDB_ID aid;

      aid=AE_Address_GetId(addr);
      if (aid) {
	AE_ADDRESS *naddr;

	naddr=AE_Partner_GetFirstAddress(ptNew);
	while(naddr) {
	  if (AE_Address_GetId(naddr)==aid)
	    break;
	  naddr=AE_Address_List_Next(naddr);
	}

	if (naddr==NULL) {
	  /* not found in new list, delete */
	  rv=AE_Book_DeleteAddress(b, AE_Book_TableType_Address, addr, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
      }

      addr=AE_Address_List_Next(addr);
    }
  }


  /* write new bank accounts */
  ba=AE_Partner_GetFirstBankAccount(ptNew);
  while(ba) {
    AQDB_ID aid;

    /* store owner type and id */
    AE_BankAccount_SetOwnerType(ba, AE_Book_TableType_Partner);
    AE_BankAccount_SetOwnerId(ba, id);

    /* write or add address */
    aid=AE_BankAccount_GetId(ba);
    if (aid) {
      rv=AE_Book_WriteBankAccount(b, AE_Book_TableType_BankAccount, ba, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }
    else {
      rv=AE_Book_AddBankAccount(b, AE_Book_TableType_BankAccount, ba, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }

    ba=AE_BankAccount_List_Next(ba);
  }

  /* remove deleted bank accounts */
  if (ptOld) {
    /* for every old account: check whether it still exists in the new object,
     * delete it if not */
    ba=AE_Partner_GetFirstBankAccount(ptOld);
    while(ba) {
      AQDB_ID aid;

      aid=AE_BankAccount_GetId(ba);
      if (aid) {
	AE_BANKACCOUNT *nba;

	nba=AE_Partner_GetFirstBankAccount(ptNew);
	while(nba) {
	  if (AE_BankAccount_GetId(nba)==aid)
	    break;
	  nba=AE_BankAccount_List_Next(nba);
	}

	if (nba==NULL) {
	  /* not found in new list, delete */
	  rv=AE_Book_DeleteBankAccount(b, AE_Book_TableType_BankAccount, ba, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
      }

      ba=AE_BankAccount_List_Next(ba);
    }
  }

  /* write new rules */
  rule=AE_Partner_GetFirstRule(ptNew);
  while(rule) {
    AQDB_ID rid;

    /* store owner type and id */
    AE_Rule_SetOwnerType(rule, AE_Book_TableType_Partner);
    AE_Rule_SetOwnerId(rule, id);

    /* write or add address */
    rid=AE_Rule_GetId(rule);
    if (rid) {
      rv=AE_Book_WriteRule(b, AE_Book_TableType_PartnerRule, rule, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }
    else {
      rv=AE_Book_AddRule(b, AE_Book_TableType_PartnerRule, rule, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }

    rule=AE_Rule_List_Next(rule);
  }

  /* remove deleted rules */
  if (ptOld) {
    /* for every old rule: check whether it still exists in the new object,
     * delete it if not */
    rule=AE_Partner_GetFirstRule(ptOld);
    while(rule) {
      AQDB_ID rid;

      rid=AE_Rule_GetId(rule);
      if (rid) {
	AE_RULE *nrule;

	nrule=AE_Partner_GetFirstRule(ptNew);
	while(nrule) {
	  if (AE_Rule_GetId(nrule)==rid)
	    break;
	  nrule=AE_Rule_List_Next(nrule);
	}

	if (nrule==NULL) {
	  /* not found in new list, delete */
	  rv=AE_Book_DeleteRule(b, AE_Book_TableType_PartnerRule, rule, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
      }

      rule=AE_Rule_List_Next(rule);
    }
  }

  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }

  return 0;
}



int AE_Book_ApplyJob(AE_BOOK *b, AE_JOB *jNew, const AE_JOB *jOld, int doLock) {
  AQDB_ID id;
  int rv;
  AE_ITEM *item;
  AQDB_ID coId=0;

  /* get commodity id */
  if (jNew) {
    item=AE_Job_GetFirstItem(jNew);
    if (item) {
      coId=AE_Item_GetCommodityId(item);
      AE_Job_SetCommodityId(jNew, coId);
    }
  }

  /* possibly lock */
  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  /* add or write partner */
  id=AE_Job_GetId(jNew);
  if (id) {
    rv=AE_Book_WriteJob(b, AE_Book_TableType_Job, jNew, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }
  else {
    /* new partner */
    rv=AE_Book_AddJob(b, AE_Book_TableType_Job, jNew, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
    id=AE_Job_GetId(jNew);
  }

  /* write new itemesses */
  item=AE_Job_GetFirstItem(jNew);
  while(item) {
    AQDB_ID iid;

    /* preset */
    AE_Item_SetJobId(item, id);

    /* write or add items */
    iid=AE_Item_GetId(item);
    if (iid) {
      rv=AE_Book_WriteItem(b, AE_Book_TableType_Item, item, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }
    else {
      rv=AE_Book_AddItem(b, AE_Book_TableType_Item, item, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	return rv;
      }
    }

    item=AE_Item_List_Next(item);
  }

  /* remove deleted itemesses */
  if (jOld) {
    /* for every old itemess: check whether it still exists in the new object,
     * delete it if not */
    item=AE_Job_GetFirstItem(jOld);
    while(item) {
      AQDB_ID iid;

      iid=AE_Item_GetId(item);
      if (iid) {
	AE_ITEM *nItem;

	nItem=AE_Job_GetFirstItem(jNew);
	while(nItem) {
	  if (AE_Item_GetId(nItem)==iid)
	    break;
	  nItem=AE_Item_List_Next(nItem);
	}

	if (nItem==NULL) {
	  /* not found in new list, delete */
	  rv=AE_Book_DeleteItem(b, AE_Book_TableType_Item, item, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
      }

      item=AE_Item_List_Next(item);
    }
  }

  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }

  return 0;
}



int AE_Book_ApplyTransactionWithBalances(AE_BOOK *b,
					 AE_ACCOUNT_TREE *at,
					 AE_TRANSACTION *tNew,
					 const AE_TRANSACTION *tOld,
					 int doLock) {
  AQDB_ID id;
  int rv;
  AE_SPLIT *split;

  /* adjust account balances */
  if (at) {
    /* subtract old splits from account balances */
    if (tOld) {
      split=AE_Transaction_GetFirstSplit(tOld);
      while(split) {
	rv=AE_Account_Tree_SubFromBalance(at,
					  AE_Split_GetAccountId(split),
					  AE_Split_GetAmount(split),
					  AE_Split_GetCommodityId(split));
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  return rv;
	}
	split=AE_Split_List_Next(split);
      }
    }

    /* add new splits to account balances */
    if (tNew) {
      split=AE_Transaction_GetFirstSplit(tNew);
      while(split) {
	rv=AE_Account_Tree_AddToBalance(at,
					AE_Split_GetAccountId(split),
					AE_Split_GetAmount(split),
					AE_Split_GetCommodityId(split));
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  return rv;
	}
	split=AE_Split_List_Next(split);
      }
    }
  }

  /* possibly lock */
  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  /* add or write transaction */
  if (tNew) {
    id=AE_Transaction_GetId(tNew);
    if (id==0 || AE_Transaction_GetRuntimeFlags(tNew) & AE_TRANSACTION_RTFLAGS_MODIFIED) {
      if (id) {
	rv=AE_Book_WriteTransaction(b, AE_Book_TableType_Transaction, tNew, 0);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  if (doLock)
	    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  return rv;
	}
      }
      else {
	/* new transaction */
	rv=AE_Book_AddTransaction(b, AE_Book_TableType_Transaction, tNew, 0);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  if (doLock)
	    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  return rv;
	}
	id=AE_Transaction_GetId(tNew);
      }
      AE_Transaction_SubRuntimeFlags(tNew, AE_TRANSACTION_RTFLAGS_MODIFIED);
    }
  
    /* write new splits */
    split=AE_Transaction_GetFirstSplit(tNew);
    while(split) {
      AQDB_ID sid;
  
      sid=AE_Split_GetId(split);
      if (sid==0 || AE_Split_GetRuntimeFlags(split) & AE_SPLIT_RTFLAGS_MODIFIED) {
	AE_Split_SetTransactionId(split, id);
	/* write or add splits */
	if (sid) {
	  rv=AE_Book_WriteSplit(b, AE_Book_TableType_Split, split, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
	else {
	  rv=AE_Book_AddSplit(b, AE_Book_TableType_Split, split, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
	AE_Split_SubRuntimeFlags(split, AE_SPLIT_RTFLAGS_MODIFIED);
      }
  
      split=AE_Split_List_Next(split);
    }
  }

  /* remove deleted splits */
  if (tOld) {
    /* for every old splits: check whether it still exists in the new object,
     * delete it if not */
    split=AE_Transaction_GetFirstSplit(tOld);
    while(split) {
      AQDB_ID sid;

      sid=AE_Split_GetId(split);
      if (sid) {
	AE_SPLIT *nSplit=NULL;

	if (tNew) {
	  nSplit=AE_Transaction_GetFirstSplit(tNew);
	  while(nSplit) {
	    if (AE_Split_GetId(nSplit)==sid)
	      break;
	    nSplit=AE_Split_List_Next(nSplit);
	  }
	}

	if (nSplit==NULL) {
	  /* not found in new list, delete */
	  rv=AE_Book_DeleteSplit(b, AE_Book_TableType_Split, split, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    return rv;
	  }
	}
      }

      split=AE_Split_List_Next(split);
    }
  }

  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }

  return 0;
}



int AE_Book_ApplyTransaction(AE_BOOK *b,
			     AE_TRANSACTION *tNew,
			     const AE_TRANSACTION *tOld,
			     int doLock) {
  AE_ACCOUNT_TREE *at;
  int rv;

  /* possibly lock */
  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  at=AE_Account_Tree_new();

  /* read account tree */
  rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
  if (rv<0) {
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  /* read balances */
  rv=AE_Book_ReadAccountTreeBalances(b, at);
  if (rv<0) {
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  /* apply transaction */
  rv=AE_Book_ApplyTransactionWithBalances(b, at, tNew, tOld, 0);
  if (rv<0) {
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  /* write back balances */
  rv=AE_Book_WriteAccountTreeBalances(b, at, 0);
  if (rv<0) {
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  /* possibly unlock database */
  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      AE_Account_Tree_free(at);
      return rv;
    }
  }
  AE_Account_Tree_free(at);

  return 0;
}



int AE_Book_ApplyTransactionToAccountTree(AE_BOOK *b,
					  AE_TRANSACTION *tNew, const AE_TRANSACTION *tOld,
					  AE_ACCOUNT_TREE *at) {
  AE_SPLIT *split;

  if (tOld) {
    /* remove old splits from their accounts */
    split=AE_Transaction_GetFirstSplit(tOld);
    while(split) {
      const AQDB_VALUE *vSrc;

      vSrc=AE_Split_GetAmount(split);
      if (vSrc) {
	AQDB_ID aid;

	aid=AE_Split_GetAccountId(split);
	if (aid>0) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_Tree_GetById(at, aid);
	  if (acc) {
	    AE_ACCOUNTBALANCE *bal;
	    AQDB_VALUE *vDest;

	    bal=AE_Account_GetBalance(acc);
	    if (bal==NULL) {
	      bal=AE_AccountBalance_new();
	      AE_AccountBalance_SetAccountId(bal, aid);
	      AE_AccountBalance_SetCommodityId(bal, AE_Account_GetPrimaryCommodityId(acc));
              AE_Account_SetBalance(acc, bal);
	    }
	    vDest=AE_AccountBalance_GetValue(bal);
	    AQDB_Value_SubValue(vDest, vSrc);
	    AE_AccountBalance_AddRuntimeFlags(bal, AE_ACCOUNTBALANCE_RTFLAGS_MODIFIED);
	  }
	}
      }

      split=AE_Split_List_Next(split);
    }
  }

  if (tNew) {
    /* add new splits to their accounts */
    split=AE_Transaction_GetFirstSplit(tNew);
    while(split) {
      const AQDB_VALUE *vSrc;

      vSrc=AE_Split_GetAmount(split);
      if (vSrc) {
	AQDB_ID aid;

	aid=AE_Split_GetAccountId(split);
	if (aid>0) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_Tree_GetById(at, aid);
	  if (acc) {
	    AE_ACCOUNTBALANCE *bal;
	    AQDB_VALUE *vDest;

	    bal=AE_Account_GetBalance(acc);
	    if (bal==NULL) {
	      bal=AE_AccountBalance_new();
	      AE_AccountBalance_SetAccountId(bal, aid);
	      AE_AccountBalance_SetCommodityId(bal, AE_Account_GetPrimaryCommodityId(acc));
	      AE_Account_SetBalance(acc, bal);
	    }
	    vDest=AE_AccountBalance_GetValue(bal);
	    AQDB_Value_AddValue(vDest, vSrc);
	    AE_AccountBalance_AddRuntimeFlags(bal, AE_ACCOUNTBALANCE_RTFLAGS_MODIFIED);
	  }
	}
      }

      split=AE_Split_List_Next(split);
    }
  }


  return 0;
}





