/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */


int addConflictingObjects(AE_BOOK *b,
			  AE_BOOK_TABLE_TYPE tt,
			  const char *qexpr,
			  AE_DBCONFLICT *cfl) {
  int rv;
  AQDB_ID qid=0;
  AQDB_ID oids[16];
  int len;

  /* query */
  rv=AQDB_DB_QuerySubmit(AE_Book_GetDataBase(b),
                         AE_Book_GetTableId(b, tt),
			 0,
			 qexpr,
			 0, /* query flags */
			 &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* browse all objects */
  len=16;
  rv=AQDB_DB_QueryGetFirst(AE_Book_GetDataBase(b), qid, oids, &len);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  while(rv==0) {
    int i;

    for (i=0; i<len; i++) {
      AQDB_OBJECT *o=NULL;
      int64_t id;

      rv=AQDB_DB_ReadObject(AE_Book_GetDataBase(b),
			    AE_Book_GetTableId(b, tt),
			    oids[i],
			    &o);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
	AQDB_DB_QueryClose(AE_Book_GetDataBase(b), qid);
	return rv;
      }

      rv=AQDB_Object_GetFieldInt64(o, 0, &id);
      AQDB_Object_free(o);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AQDB_DB_QueryClose(AE_Book_GetDataBase(b), qid);
	return rv;
      }
      GWEN_IdList64_AddId(AE_DbConflict_GetIdList(cfl), id);
    }

    len=16;
    rv=AQDB_DB_QueryGetNext(AE_Book_GetDataBase(b), qid, oids, &len);
  }
  AQDB_DB_QueryClose(AE_Book_GetDataBase(b), qid);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



int AE_Book_GetConflicts(AE_BOOK *b,
			 AQDB_ACTION_TYPE action,
			 AE_BOOK_TABLE_TYPE tt,
			 AQDB_ID id,
			 AE_DBCONFLICT_LIST *clist) {
  if (action==AQDB_ActionType_Remove) {
    switch(tt) {
    case AE_Book_TableType_Account:
      return AE_Book_GetConflictsDeleteAccount(b, tt, id, clist);
    case AE_Book_TableType_Transaction:
      return AE_Book_GetConflictsDeleteTransaction(b, tt, id, clist);
    case AE_Book_TableType_BankStatement:
      return AE_Book_GetConflictsDeleteBankStatement(b, tt, id, clist);
    case AE_Book_TableType_BankAccount:
      return AE_Book_GetConflictsDeleteBankAccount(b, tt, id, clist);
    case AE_Book_TableType_Job:
      return AE_Book_GetConflictsDeleteJob(b, tt, id, clist);
    case AE_Book_TableType_Task:
      return AE_Book_GetConflictsDeleteTask(b, tt, id, clist);
    case AE_Book_TableType_Partner:
      return AE_Book_GetConflictsDeletePartner(b, tt, id, clist);
    case AE_Book_TableType_PartnerGroup:
      return AE_Book_GetConflictsDeletePartnerGroup(b, tt, id, clist);

     default:
      break;
    }
  }

  return 0;
}



