/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */



static int deleteAccountSolutionAccountToParentExec(AE_DBCONFLICTSOLUTION *sol,
						    AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_ACCOUNT *a=NULL;
      int rv;

      rv=AE_Book_ReadAccount(AE_DbConflict_GetBook(cfl),
			     AE_DbConflict_GetDbType(cfl),
			     id,
			     &a);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Account_SetParentId(a, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteAccount(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), a, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Account_free(a);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Account_free(a);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deleteAccountSolutionSplitsToParentExec(AE_DBCONFLICTSOLUTION *sol,
						   AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_SPLIT *sp=NULL;
      int rv;

      rv=AE_Book_ReadSplit(AE_DbConflict_GetBook(cfl),
			   AE_DbConflict_GetDbType(cfl),
			   id,
			   &sp);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      /* TODO: add to balance of new account */
      AE_Split_SetAccountId(sp, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteSplit(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), sp, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Split_free(sp);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Split_free(sp);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deleteAccountSolutionPartnerPayableToParentExec(AE_DBCONFLICTSOLUTION *sol,
							   AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_PARTNER *pt=NULL;
      int rv;

      rv=AE_Book_ReadPartner(AE_DbConflict_GetBook(cfl),
			     AE_DbConflict_GetDbType(cfl),
			     id,
			     &pt);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Partner_SetPayableAccountId(pt, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WritePartner(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), pt, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Partner_free(pt);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Partner_free(pt);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deleteAccountSolutionPartnerReceivableToParentExec(AE_DBCONFLICTSOLUTION *sol,
							      AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_PARTNER *pt=NULL;
      int rv;

      rv=AE_Book_ReadPartner(AE_DbConflict_GetBook(cfl),
			     AE_DbConflict_GetDbType(cfl),
			     id,
			     &pt);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Partner_SetReceivableAccountId(pt, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WritePartner(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), pt, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Partner_free(pt);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Partner_free(pt);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deleteAccountSolutionTaxToParentExec(AE_DBCONFLICTSOLUTION *sol,
						AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_TAX *tx=NULL;
      int rv;

      rv=AE_Book_ReadTax(AE_DbConflict_GetBook(cfl),
			 AE_DbConflict_GetDbType(cfl),
			 id,
			 &tx);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Tax_SetAccountId(tx, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteTax(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), tx, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Tax_free(tx);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Tax_free(tx);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deleteAccountSolutionRuleRemoveExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_RULE *r=NULL;
      int rv;

      rv=AE_Book_ReadRule(AE_DbConflict_GetBook(cfl),
			  AE_DbConflict_GetDbType(cfl),
			  id,
			  &r);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      rv=AE_Book_DeleteRule(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), r, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Rule_free(r);
	GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Rule_free(r);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deleteAccountSolutionRuleToParentExec(AE_DBCONFLICTSOLUTION *sol,
						 AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_RULE *r=NULL;
      int rv;

      rv=AE_Book_ReadRule(AE_DbConflict_GetBook(cfl),
			  AE_DbConflict_GetDbType(cfl),
			  id,
			  &r);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Rule_SetOwnerId(r, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteRule(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), r, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Rule_free(r);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Rule_free(r);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int AE_Book_GetConflictsDeleteAccount(AE_BOOK *b,
					     AE_BOOK_TABLE_TYPE tt,
					     AQDB_ID id,
					     AE_DBCONFLICT_LIST *clist) {
  char qexpr[64];
  int rv;
  AE_DBCONFLICT *cfl;
  AE_DBCONFLICTSOLUTION *sol;
  AE_ACCOUNT *a;

  rv=AE_Book_ReadAccount(b, tt, id, &a);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* check for sub-accounts */
  snprintf(qexpr, sizeof(qexpr)-1, "$parentId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Account);

  rv=addConflictingObjects(b, AE_Book_TableType_Account, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    /* add solutions */
    parentId=AE_Account_GetParentId(a);
    AE_DbConflict_SetText(cfl, I18N("There is at least one account below this account."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Move account to parent account"));
    AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionAccountToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Account_free(a);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }


  /* check for splits */
  snprintf(qexpr, sizeof(qexpr)-1, "$accountId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Split);

  rv=addConflictingObjects(b, AE_Book_TableType_Split, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    /* add solutions */
    parentId=AE_Account_GetParentId(a);
    if (parentId) {
      AE_DbConflict_SetText(cfl, I18N("There is at least one transaction split in this account."));
      sol=AE_DbConflictSolution_new();
      AE_DbConflictSolution_SetText(sol, I18N("Move transaction splits to parent account"));
      AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionSplitsToParentExec);
      AE_DbConflictSolution_SetDestinationId(sol, parentId);
      AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
      AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
      AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

      /* add conflict */
      AE_DbConflict_List_Add(cfl, clist);
    }
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Account_free(a);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for payable accounts for partners */
  snprintf(qexpr, sizeof(qexpr)-1, "$payableAccountId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Partner);

  rv=addConflictingObjects(b, AE_Book_TableType_Partner, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    /* add solutions */
    parentId=AE_Account_GetParentId(a);
    AE_DbConflict_SetText(cfl, I18N("There is at least one partner having this account as payable account."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Set partner's payable account to parent account"));
    AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionPartnerPayableToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Account_free(a);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for receivable accounts for partners */
  snprintf(qexpr, sizeof(qexpr)-1, "$receivableAccountId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Partner);

  rv=addConflictingObjects(b, AE_Book_TableType_Partner, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    /* add solutions */
    parentId=AE_Account_GetParentId(a);
    AE_DbConflict_SetText(cfl, I18N("There is at least one partner having this account as receivable account."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Set partner's receivable account to parent account"));
    AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionPartnerReceivableToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Account_free(a);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for taxes */
  snprintf(qexpr, sizeof(qexpr)-1, "$accountId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Tax);

  rv=addConflictingObjects(b, AE_Book_TableType_Tax, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    /* add solutions */
    parentId=AE_Account_GetParentId(a);
    AE_DbConflict_SetText(cfl, I18N("There is at least one tax referring to this account."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Set taxe's account to parent account"));
    AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionTaxToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Account_free(a);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for rules */
  snprintf(qexpr, sizeof(qexpr)-1, "($ownertype==%d)&&($ownerid==%d)", tt, id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Rule);

  rv=addConflictingObjects(b, AE_Book_TableType_Rule, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    /* add solutions */
    parentId=AE_Account_GetParentId(a);
    AE_DbConflict_SetText(cfl, I18N("There is at least one matcher rule referring to this account."));

    /* solution */
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove matcher rules for this account"));
    AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionRuleRemoveExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Remove);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* solution */
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Move matcher rule to parent account"));
    AE_DbConflictSolution_SetExecFn(sol, deleteAccountSolutionRuleToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Account_free(a);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  AE_Account_free(a);

  return 0;
}





