/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */



static int deleteBankAccountSolutionStatementDeleteExec(AE_DBCONFLICTSOLUTION *sol,
							AE_DBCONFLICT *cfl) {
  uint32_t cnt;

  cnt=(uint32_t) GWEN_IdList64_GetEntryCount(AE_DbConflict_GetIdList(cfl));
  if (cnt) {
    uint32_t n=0;
    uint32_t pid;
    GWEN_IDLIST64_ITERATOR *it;
    AQDB_ID id;

    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
			       GWEN_GUI_PROGRESS_SHOW_ABORT,
			       I18N("Resolving Conflicts"),
			       I18N("Deleting statements"),
			       cnt,
			       0);

    it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
    assert(it);

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_STATEMENT *st=NULL;
      int rv;

      rv=AE_Book_ReadStatement(AE_DbConflict_GetBook(cfl),
			       AE_DbConflict_GetDbType(cfl),
			       id,
			       &st);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error reading statement: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }
      rv=AE_Book_DeleteStatement(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), st, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Statement_free(st);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error deleting statement: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }
      AE_Statement_free(st);
      n++;

      rv=GWEN_Gui_ProgressAdvance(pid, n);
      if (rv==GWEN_ERROR_USER_ABORTED) {
	GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user"));
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
    GWEN_Gui_ProgressEnd(pid);
  }

  return 0;
}



static int deleteBankAccountSolutionStatementDeleteCheck(AE_DBCONFLICTSOLUTION *sol,
                                                         AE_DBCONFLICT *cfl,
							 AE_DBCONFLICT_LIST *clist) {
  uint32_t cnt;

  cnt=(uint32_t) GWEN_IdList64_GetEntryCount(AE_DbConflict_GetIdList(cfl));
  if (cnt) {
    uint32_t n=0;
    uint32_t pid;
    GWEN_IDLIST64_ITERATOR *it;
    AQDB_ID id;

    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
			       GWEN_GUI_PROGRESS_SHOW_ABORT,
			       I18N("Resolving Conflicts"),
			       I18N("Checking for conflicts regarding deletion of statements."),
			       cnt,
			       0);

    it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
    assert(it);

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      int rv;

      rv=AE_Book_GetConflicts(AE_DbConflict_GetBook(cfl), AQDB_ActionType_Remove,
			      AE_DbConflict_GetDbType(cfl),
			      id, clist);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error checking for conflicts: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      n++;
      rv=GWEN_Gui_ProgressAdvance(pid, n);
      if (rv==GWEN_ERROR_USER_ABORTED) {
	GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user"));
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
    GWEN_Gui_ProgressEnd(pid);
  }

  return 0;
}


static int AE_Book_GetConflictsDeleteBankAccount(AE_BOOK *b,
						 AE_BOOK_TABLE_TYPE tt,
						 AQDB_ID id,
						 AE_DBCONFLICT_LIST *clist) {
  char qexpr[64];
  int rv;
  AE_DBCONFLICT *cfl;
  AE_DBCONFLICTSOLUTION *sol;
  AE_BANKACCOUNT *ba=NULL;

  rv=AE_Book_ReadBankAccount(b, tt, id, &ba);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* check for jobs */
  snprintf(qexpr, sizeof(qexpr)-1, "$bankAccountId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_BankStatement);
  rv=addConflictingObjects(b, AE_Book_TableType_BankStatement, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one statement in this bank account."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Delete statement(s)"));
    AE_DbConflictSolution_SetExecFn(sol, deleteBankAccountSolutionStatementDeleteExec);
    AE_DbConflictSolution_SetCheckConflictsFn(sol, deleteBankAccountSolutionStatementDeleteCheck);
    AE_DbConflictSolution_SetDestinationId(sol, 0);
    /*AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);*/
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Remove);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_BankAccount_free(ba);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  AE_BankAccount_free(ba);

  return 0;
}





