/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */

#include <gwenhywfar/gui.h>



static int deleteBankStatementSolutionTransactionTo0Exec(AE_DBCONFLICTSOLUTION *sol,
							 AE_DBCONFLICT *cfl) {
  uint32_t cnt;

  cnt=(uint32_t) GWEN_IdList64_GetEntryCount(AE_DbConflict_GetIdList(cfl));
  if (cnt) {
    uint32_t n=0;
    uint32_t pid;
    GWEN_IDLIST64_ITERATOR *it;
    AQDB_ID id;

    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
			       GWEN_GUI_PROGRESS_SHOW_ABORT,
			       I18N("Resolving Conflicts"),
			       I18N("Remove statements from transactions"),
			       cnt,
			       0);

    it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
    assert(it);

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_TRANSACTION *t=NULL;
      int rv;

      rv=AE_Book_ReadTransaction(AE_DbConflict_GetBook(cfl),
				 AE_DbConflict_GetDbType(cfl),
				 id,
				 &t);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error reading transaction: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }
      AE_Transaction_SetStatementId(t, 0);
      rv=AE_Book_WriteTransaction(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), t, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Transaction_free(t);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error writing transaction: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }
      AE_Transaction_free(t);
      n++;

      rv=GWEN_Gui_ProgressAdvance(pid, n);
      if (rv==GWEN_ERROR_USER_ABORTED) {
	GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user"));
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
    GWEN_Gui_ProgressEnd(pid);
  }

  return 0;
}



static int AE_Book_GetConflictsDeleteBankStatement(AE_BOOK *b,
						   AE_BOOK_TABLE_TYPE tt,
						   AQDB_ID id,
						   AE_DBCONFLICT_LIST *clist) {
  char qexpr[64];
  int rv;
  AE_DBCONFLICT *cfl;
  AE_DBCONFLICTSOLUTION *sol;
  AE_STATEMENT *st=NULL;

  rv=AE_Book_ReadStatement(b, tt, id, &st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* check for jobs */
  snprintf(qexpr, sizeof(qexpr)-1, "$statementId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Transaction);
  rv=addConflictingObjects(b, AE_Book_TableType_Transaction, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one transaction referring to this statement."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove statement(s) from transaction"));
    AE_DbConflictSolution_SetExecFn(sol, deleteBankStatementSolutionTransactionTo0Exec);
    AE_DbConflictSolution_SetDestinationId(sol, 0);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Statement_free(st);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  AE_Statement_free(st);

  return 0;
}





