/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */


static int deletePartnerSolutionDeleteAddressExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_ADDRESS *addr=NULL;
      int rv;

      rv=AE_Book_ReadAddress(AE_DbConflict_GetBook(cfl),
			     AE_DbConflict_GetDbType(cfl),
			     id,
			     &addr);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      rv=AE_Book_DeleteAddress(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), addr, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Address_free(addr);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Address_free(addr);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerSolutionDeleteAddressCheck(AE_DBCONFLICTSOLUTION *sol,
                                                   AE_DBCONFLICT *cfl,
                                                   AE_DBCONFLICT_LIST *clist) {
  uint32_t cnt;

  cnt=(uint32_t) GWEN_IdList64_GetEntryCount(AE_DbConflict_GetIdList(cfl));
  if (cnt) {
    uint32_t n=0;
    uint32_t pid;
    GWEN_IDLIST64_ITERATOR *it;
    AQDB_ID id;

    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
			       GWEN_GUI_PROGRESS_SHOW_ABORT,
			       I18N("Resolving Conflicts"),
                               I18N("Checking for conflicts regarding deletion of addresses."),
			       cnt,
			       0);

    it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
    assert(it);

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      int rv;

      rv=AE_Book_GetConflicts(AE_DbConflict_GetBook(cfl), AQDB_ActionType_Remove,
			      AE_DbConflict_GetDbType(cfl),
			      id, clist);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error checking for conflicts: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      n++;
      rv=GWEN_Gui_ProgressAdvance(pid, n);
      if (rv==GWEN_ERROR_USER_ABORTED) {
	GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user"));
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
    GWEN_Gui_ProgressEnd(pid);
  }

  return 0;
}



static int deletePartnerSolutionDeleteBankAccountExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_BANKACCOUNT *ba=NULL;
      int rv;

      rv=AE_Book_ReadBankAccount(AE_DbConflict_GetBook(cfl),
				 AE_DbConflict_GetDbType(cfl),
				 id,
				 &ba);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      rv=AE_Book_DeleteBankAccount(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), ba, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_BankAccount_free(ba);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_BankAccount_free(ba);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerSolutionDeleteBankAccountCheck(AE_DBCONFLICTSOLUTION *sol,
                                                       AE_DBCONFLICT *cfl,
                                                       AE_DBCONFLICT_LIST *clist) {
  uint32_t cnt;

  cnt=(uint32_t) GWEN_IdList64_GetEntryCount(AE_DbConflict_GetIdList(cfl));
  if (cnt) {
    uint32_t n=0;
    uint32_t pid;
    GWEN_IDLIST64_ITERATOR *it;
    AQDB_ID id;

    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
			       GWEN_GUI_PROGRESS_SHOW_ABORT,
			       I18N("Resolving Conflicts"),
                               I18N("Checking for conflicts regarding deletion of bank accounts."),
			       cnt,
			       0);

    it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
    assert(it);

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      int rv;

      rv=AE_Book_GetConflicts(AE_DbConflict_GetBook(cfl), AQDB_ActionType_Remove,
			      AE_DbConflict_GetDbType(cfl),
			      id, clist);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error checking for conflicts: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      n++;
      rv=GWEN_Gui_ProgressAdvance(pid, n);
      if (rv==GWEN_ERROR_USER_ABORTED) {
	GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user"));
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
    GWEN_Gui_ProgressEnd(pid);
  }

  return 0;
}




static int deletePartnerSolutionAccountUnsetExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_ACCOUNT *a=NULL;
      int rv;

      rv=AE_Book_ReadAccount(AE_DbConflict_GetBook(cfl),
			     AE_DbConflict_GetDbType(cfl),
			     id,
			     &a);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Account_SetPartnerId(a, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteAccount(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), a, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Account_free(a);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Account_free(a);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerSolutionTransactionUnsetExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_TRANSACTION *t=NULL;
      int rv;

      rv=AE_Book_ReadTransaction(AE_DbConflict_GetBook(cfl),
				 AE_DbConflict_GetDbType(cfl),
				 id,
				 &t);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Transaction_SetOwnerId(t, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteTransaction(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), t, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Transaction_free(t);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Transaction_free(t);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerSolutionTaskUnsetExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_TASK *tk=NULL;
      int rv;

      rv=AE_Book_ReadTask(AE_DbConflict_GetBook(cfl),
                          AE_DbConflict_GetDbType(cfl),
                          id,
                          &tk);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Task_SetPartnerId(tk, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteTask(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), tk, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Task_free(tk);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Task_free(tk);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerSolutionJobUnsetExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_JOB *j=NULL;
      int rv;

      rv=AE_Book_ReadJob(AE_DbConflict_GetBook(cfl),
                         AE_DbConflict_GetDbType(cfl),
                         id,
                         &j);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Job_SetPartnerId(j, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteJob(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), j, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Job_free(j);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Job_free(j);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerSolutionStatementUnsetExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_STATEMENT *st=NULL;
      int rv;

      rv=AE_Book_ReadStatement(AE_DbConflict_GetBook(cfl),
                               AE_DbConflict_GetDbType(cfl),
                               id,
                               &st);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Statement_SetPartnerId(st, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteStatement(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), st, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Statement_free(st);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Statement_free(st);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}









static int AE_Book_GetConflictsDeletePartner(AE_BOOK *b,
					     AE_BOOK_TABLE_TYPE tt,
					     AQDB_ID id,
					     AE_DBCONFLICT_LIST *clist) {
  char qexpr[64];
  int rv;
  AE_DBCONFLICT *cfl;
  AE_DBCONFLICTSOLUTION *sol;
  AE_PARTNER *pt;

  rv=AE_Book_ReadPartner(b, tt, id, &pt);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* check for addresses */
  snprintf(qexpr, sizeof(qexpr)-1, "($ownerType==%d)&&($ownerId==%ld)", tt, (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Address);

  rv=addConflictingObjects(b, AE_Book_TableType_Address, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one address associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Delete address(es) of this partner"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionDeleteAddressExec);
    AE_DbConflictSolution_SetCheckConflictsFn(sol, deletePartnerSolutionDeleteAddressCheck);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Remove);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }


  /* check for bank accounts */
  snprintf(qexpr, sizeof(qexpr)-1, "($ownerType==%d)&&($ownerId==%ld)", tt, (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_BankAccount);

  rv=addConflictingObjects(b, AE_Book_TableType_BankAccount, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one bank account associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Delete bank account(s) of this partner"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionDeleteBankAccountExec);
    AE_DbConflictSolution_SetCheckConflictsFn(sol, deletePartnerSolutionDeleteBankAccountCheck);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Remove);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }


  /* check for accounts */
  snprintf(qexpr, sizeof(qexpr)-1, "$partnerId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Account);

  rv=addConflictingObjects(b, AE_Book_TableType_Account, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one account associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove partner from account(s)"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionAccountUnsetExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for tasks */
  snprintf(qexpr, sizeof(qexpr)-1, "$partnerId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Task);

  rv=addConflictingObjects(b, AE_Book_TableType_Task, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one task associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove partner from task(s)"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionTaskUnsetExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* TODO: add solution for removing task */

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for jobs */
  snprintf(qexpr, sizeof(qexpr)-1, "$partnerId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Job);

  rv=addConflictingObjects(b, AE_Book_TableType_Job, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one job associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove partner from job(s)"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionJobUnsetExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* TODO: add solution for removing job */

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  /* check for statements */
  snprintf(qexpr, sizeof(qexpr)-1, "$partnerId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_BankStatement);

  rv=addConflictingObjects(b, AE_Book_TableType_BankStatement, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one bank statement associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove partner from bank statement(s)"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionStatementUnsetExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }


  /* check for transactions */
  snprintf(qexpr, sizeof(qexpr)-1, "($ownerType==%d)&&($ownerId==%ld)", tt, (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Transaction);

  rv=addConflictingObjects(b, AE_Book_TableType_Transaction, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one transaction associated with this partner."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove partner from transaction(s)"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerSolutionTransactionUnsetExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Partner_free(pt);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }


  AE_Partner_free(pt);

  return 0;
}



