/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */


static int deletePartnerGroupSolutionDeletePartnerExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_PARTNER *pt=NULL;
      int rv;

      rv=AE_Book_ReadPartner(AE_DbConflict_GetBook(cfl),
                             AE_DbConflict_GetDbType(cfl),
                             id,
                             &pt);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      rv=AE_Book_DeletePartner(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), pt, 0);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Partner_free(pt);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Partner_free(pt);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerGroupSolutionDeletePartnerCheck(AE_DBCONFLICTSOLUTION *sol,
                                                        AE_DBCONFLICT *cfl,
                                                        AE_DBCONFLICT_LIST *clist) {
  uint32_t cnt;

  cnt=(uint32_t) GWEN_IdList64_GetEntryCount(AE_DbConflict_GetIdList(cfl));
  if (cnt) {
    uint32_t n=0;
    uint32_t pid;
    GWEN_IDLIST64_ITERATOR *it;
    AQDB_ID id;

    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
			       GWEN_GUI_PROGRESS_SHOW_ABORT,
			       I18N("Resolving Conflicts"),
			       I18N("Checking for conflicts regarding deletion of partners."),
			       cnt,
			       0);

    it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
    assert(it);

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      int rv;

      rv=AE_Book_GetConflicts(AE_DbConflict_GetBook(cfl), AQDB_ActionType_Remove,
			      AE_DbConflict_GetDbType(cfl),
			      id, clist);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error, I18N("Error checking for conflicts: %d"), rv);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      n++;
      rv=GWEN_Gui_ProgressAdvance(pid, n);
      if (rv==GWEN_ERROR_USER_ABORTED) {
	GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user"));
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
    GWEN_Gui_ProgressEnd(pid);
  }

  return 0;
}



static int deletePartnerGroupSolutionPartnerToParentExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_PARTNER *pt=NULL;
      int rv;

      rv=AE_Book_ReadPartner(AE_DbConflict_GetBook(cfl),
                             AE_DbConflict_GetDbType(cfl),
                             id,
                             &pt);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Partner_SetGroupId(pt, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WritePartner(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), pt, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Partner_free(pt);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Partner_free(pt);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerGroupSolutionDeleteGroupExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_GROUP *grp=NULL;
      int rv;

      rv=AE_Book_ReadGroup(AE_DbConflict_GetBook(cfl),
                           AE_DbConflict_GetDbType(cfl),
                           id,
                           &grp);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      rv=AE_Book_DeleteGroup(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), grp, 0);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Group_free(grp);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Group_free(grp);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int deletePartnerGroupSolutionGroupToParentExec(AE_DBCONFLICTSOLUTION *sol, AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_GROUP *grp=NULL;
      int rv;

      rv=AE_Book_ReadGroup(AE_DbConflict_GetBook(cfl),
                             AE_DbConflict_GetDbType(cfl),
                             id,
                             &grp);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Group_SetParentId(grp, AE_DbConflictSolution_GetDestinationId(sol));
      rv=AE_Book_WriteGroup(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), grp, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Group_free(grp);
        GWEN_IdList64_Iterator_free(it);
        return rv;
      }
      AE_Group_free(grp);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}









static int AE_Book_GetConflictsDeletePartnerGroup(AE_BOOK *b,
                                                  AE_BOOK_TABLE_TYPE tt,
                                                  AQDB_ID id,
                                                  AE_DBCONFLICT_LIST *clist) {
  char qexpr[64];
  int rv;
  AE_DBCONFLICT *cfl;
  AE_DBCONFLICTSOLUTION *sol;
  AE_GROUP *grp;

  rv=AE_Book_ReadGroup(b, tt, id, &grp);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }


  /* check for sub-groups */
  snprintf(qexpr, sizeof(qexpr)-1, "$parentId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_PartnerGroup);

  rv=addConflictingObjects(b, AE_Book_TableType_PartnerGroup, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    parentId=AE_Group_GetParentId(grp);

    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one partner group below this partner group."));

    /* move to parent */
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Move partner group(s) to parent group"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerGroupSolutionGroupToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* delete group */
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Delete partner(s) in this group"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerGroupSolutionDeleteGroupExec);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Remove);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Group_free(grp);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }


  /* check for partners */
  snprintf(qexpr, sizeof(qexpr)-1, "$groupId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Partner);

  rv=addConflictingObjects(b, AE_Book_TableType_Partner, qexpr, cfl);
  if (rv>=0) {
    AQDB_ID parentId;

    parentId=AE_Group_GetParentId(grp);

    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one partner associated with this partner group."));

    /* move to partner */
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Move partner(s) to parent group"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerGroupSolutionPartnerToParentExec);
    AE_DbConflictSolution_SetDestinationId(sol, parentId);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* delete partner */
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Delete partner(s) in this group"));
    AE_DbConflictSolution_SetExecFn(sol, deletePartnerGroupSolutionDeletePartnerExec);
    AE_DbConflictSolution_SetCheckConflictsFn(sol, deletePartnerGroupSolutionDeletePartnerCheck);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Remove);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Group_free(grp);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }



  AE_Group_free(grp);

  return 0;
}



