/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */



static int deleteTransactionSolutionJobTo0Exec(AE_DBCONFLICTSOLUTION *sol,
					       AE_DBCONFLICT *cfl) {
  GWEN_IDLIST64_ITERATOR *it;

  it=GWEN_IdList64_Iterator_new(AE_DbConflict_GetIdList(cfl));
  if (it) {
    AQDB_ID id;

    id=GWEN_IdList64_Iterator_GetFirstId(it);
    while(id) {
      AE_JOB *j=NULL;
      int rv;

      rv=AE_Book_ReadJob(AE_DbConflict_GetBook(cfl),
			 AE_DbConflict_GetDbType(cfl),
			 id,
			 &j);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Job_SetTransactionId(j, 0);
      rv=AE_Book_WriteJob(AE_DbConflict_GetBook(cfl), AE_DbConflict_GetDbType(cfl), j, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Job_free(j);
	GWEN_IdList64_Iterator_free(it);
	return rv;
      }
      AE_Job_free(j);

      id=GWEN_IdList64_Iterator_GetNextId(it);
    }
    GWEN_IdList64_Iterator_free(it);
  }

  return 0;
}



static int AE_Book_GetConflictsDeleteTransaction(AE_BOOK *b,
						 AE_BOOK_TABLE_TYPE tt,
						 AQDB_ID id,
						 AE_DBCONFLICT_LIST *clist) {
  char qexpr[64];
  int rv;
  AE_DBCONFLICT *cfl;
  AE_DBCONFLICTSOLUTION *sol;
  AE_TRANSACTION *t=NULL;

  rv=AE_Book_ReadTransaction(b, tt, id, &t);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* check for jobs */
  snprintf(qexpr, sizeof(qexpr)-1, "$transactionId==%ld", (unsigned long int) id);
  qexpr[sizeof(qexpr)-1]=0;

  cfl=AE_DbConflict_new();
  AE_DbConflict_SetBook(cfl, b);
  AE_DbConflict_SetDbType(cfl, AE_Book_TableType_Job);

  rv=addConflictingObjects(b, AE_Book_TableType_Job, qexpr, cfl);
  if (rv>=0) {
    /* add solutions */
    AE_DbConflict_SetText(cfl, I18N("There is at least one job referring to this transaction."));
    sol=AE_DbConflictSolution_new();
    AE_DbConflictSolution_SetText(sol, I18N("Remove transaction from job"));
    AE_DbConflictSolution_SetExecFn(sol, deleteTransactionSolutionJobTo0Exec);
    AE_DbConflictSolution_SetDestinationId(sol, 0);
    AE_DbConflictSolution_AddFlags(sol, AE_DBCONFLICTSOLUTION_FLAGS_NEEDS_DEST);
    AE_DbConflictSolution_SetActionType(sol, AQDB_ActionType_Modify);
    AE_DbConflictSolution_List_Add(sol, AE_DbConflict_GetSolutionList(cfl));

    /* add conflict */
    AE_DbConflict_List_Add(cfl, clist);
  }
  else if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbConflict_free(cfl);
      AE_Transaction_free(t);
      return rv;
    }
    AE_DbConflict_free(cfl);
  }

  AE_Transaction_free(t);

  return 0;
}





