/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */


void AE_Book_Report_ToFlatList(AE_BOOK *b, AF_REPORT_DATA *rd) {
  AF_REPORT_CATEGORY *cat;

  cat=AF_Report_Category_List_First(AF_Report_Data_GetCategoryList(rd));
  while(cat) {
    AF_REPORT_YEAR *ry;

    ry=AF_Report_Year_List_First(AF_Report_Category_GetYearList(cat));
    while(ry) {
      AF_REPORT_MONTH *rm;

      rm=AF_Report_Month_List_First(AF_Report_Year_GetMonthList(ry));
      while(rm) {
        AF_REPORT_DAY *rday;

        rday=AF_Report_Day_List_First(AF_Report_Month_GetDayList(rm));
        while(rday) {
          AF_REPORT_DATAPAIR *dp;

	  dp=AF_Report_DataPair_List_First(AF_Report_Day_GetDataPairList(rday));
	  while(dp) {
	    AF_REPORT_DATAPAIR *dpNew;

	    dpNew=AF_Report_DataPair_dup(dp);
	    if (dpNew) {
	      /* add to linear list */
	      AF_Report_Data_AddDataPairToList(rd, dpNew);
	    }
	    dp=AF_Report_DataPair_List_Next(dp);
	  }
	  rday=AF_Report_Day_List_Next(rday);
        }

        rm=AF_Report_Month_List_Next(rm);
      }
      ry=AF_Report_Year_List_Next(ry);
    }

    cat=AF_Report_Category_List_Next(cat);
  }
}



void AE_Book_Report_ToFlatDayList(AE_BOOK *b, AF_REPORT_DATA *rd, uint32_t flags) {
  AF_REPORT_AXIS_LIST *al;
  AF_REPORT_AXIS *ax;
  AF_REPORT_CATEGORY *cat;

  al=AF_Report_Data_GetAxisList(rd);
  assert(al);
  AF_Report_Axis_List_Clear(al);

  /* create date column */
  ax=AF_Report_Axis_new();
  AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT|Date"));
  AF_Report_Axis_SetDataType(ax, AQDB_DataType_Date);
  AF_Report_Axis_List_Add(ax, al);

  /* create pos amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_POS_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT|Deposit"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  /* create neg amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_NEG_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT_DATE|Withdrawal"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  /* create sum amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_SUM_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT_DATE|Balance"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  AF_Report_DataPair_List_Clear(AF_Report_Data_GetDataPairList(rd));

  cat=AF_Report_Category_List_First(AF_Report_Data_GetCategoryList(rd));
  while(cat) {
    AF_REPORT_YEAR *ry;

    ry=AF_Report_Year_List_First(AF_Report_Category_GetYearList(cat));
    while(ry) {
      AF_REPORT_MONTH *rm;

      rm=AF_Report_Month_List_First(AF_Report_Year_GetMonthList(ry));
      while(rm) {
        AF_REPORT_DAY *rday;

        rday=AF_Report_Day_List_First(AF_Report_Month_GetDayList(rm));
        while(rday) {
          GWEN_DATE *dt;
          AF_REPORT_DATAPAIR *dp;
          AF_REPORT_DATAFIELD *df;
          AF_REPORT_DATAFIELD_LIST *dfl;
          const AQDB_VALUE *vPos;
          const AQDB_VALUE *vNeg;
          AQDB_VALUE *v;

          DBG_ERROR(0, "Adding day %04d/%02d/%02d",
                    AF_Report_Year_GetYear(ry),
                    AF_Report_Month_GetMonth(rm),
                    AF_Report_Day_GetDay(rday));
          dt=GWEN_Date_fromGregorian(AF_Report_Year_GetYear(ry),
                                     AF_Report_Month_GetMonth(rm),
                                     AF_Report_Day_GetDay(rday));
          vPos=AF_Report_Day_GetPosAmount(rday);
          vNeg=AF_Report_Day_GetNegAmount(rday);

          v=AQDB_Value_dup(vPos);
          AQDB_Value_AddValue(v, vNeg);

          dp=AF_Report_DataPair_new();
          dfl=AF_Report_DataPair_GetDataFieldList(dp);
          assert(dfl);

          /* add date column */
          df=AF_Report_DataField_new();
          AF_Report_DataField_SetDataAsDate(df, dt);
          AF_Report_DataField_List_Add(df, dfl);
          GWEN_Date_free(dt);

          /* add pos amount column */
          if (flags & AE_BOOK_REPORT_FLAG_ADD_POS_AMT) {
            df=AF_Report_DataField_new();
            AF_Report_DataField_SetDataAsValue(df, vPos);
            AF_Report_DataField_List_Add(df, dfl);
          }

          /* add neg amount column */
          if (flags & AE_BOOK_REPORT_FLAG_ADD_NEG_AMT) {
            df=AF_Report_DataField_new();
            AF_Report_DataField_SetDataAsValue(df, vNeg);
            AF_Report_DataField_List_Add(df, dfl);
          }

          /* add sum amount column */
          if (flags & AE_BOOK_REPORT_FLAG_ADD_SUM_AMT) {
            df=AF_Report_DataField_new();
            AF_Report_DataField_SetDataAsValue(df, v);
            AF_Report_DataField_List_Add(df, dfl);
          }

          AQDB_Value_free(v);

          /* add to linear list */
          AF_Report_Data_AddDataPairToList(rd, dp);

          rday=AF_Report_Day_List_Next(rday);
        }

        rm=AF_Report_Month_List_Next(rm);
      }
      ry=AF_Report_Year_List_Next(ry);
    }

    cat=AF_Report_Category_List_Next(cat);
  }
}



void AE_Book_Report_ToFlatMonthList(AE_BOOK *b, AF_REPORT_DATA *rd, uint32_t flags) {
  AF_REPORT_AXIS_LIST *al;
  AF_REPORT_AXIS *ax;
  AF_REPORT_CATEGORY *cat;

  al=AF_Report_Data_GetAxisList(rd);
  assert(al);
  AF_Report_Axis_List_Clear(al);

  /* create date column */
  ax=AF_Report_Axis_new();
  AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT|Date"));
  AF_Report_Axis_SetDataType(ax, AQDB_DataType_Date);
  AF_Report_Axis_List_Add(ax, al);

  /* create pos amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_POS_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT|Deposit"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  /* create neg amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_NEG_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT_DATE|Withdrawal"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  /* create sum amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_SUM_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT_DATE|Balance"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }


  cat=AF_Report_Category_List_First(AF_Report_Data_GetCategoryList(rd));
  while(cat) {
    AF_REPORT_YEAR *ry;

    ry=AF_Report_Year_List_First(AF_Report_Category_GetYearList(cat));
    while(ry) {
      AF_REPORT_MONTH *rm;

      rm=AF_Report_Month_List_First(AF_Report_Year_GetMonthList(ry));
      while(rm) {
        GWEN_DATE *dt;
        AF_REPORT_DATAPAIR *dp;
        const AQDB_VALUE *vPos;
        const AQDB_VALUE *vNeg;
        AF_REPORT_DATAFIELD *df;
        AF_REPORT_DATAFIELD_LIST *dfl;
        AQDB_VALUE *v;

        dt=GWEN_Date_fromGregorian(AF_Report_Year_GetYear(ry),
                                   AF_Report_Month_GetMonth(rm),
                                   1);
        vPos=AF_Report_Month_GetPosAmount(rm);
        vNeg=AF_Report_Month_GetNegAmount(rm);

        v=AQDB_Value_dup(vPos);
        AQDB_Value_AddValue(v, vNeg);

        dp=AF_Report_DataPair_new();
        dfl=AF_Report_DataPair_GetDataFieldList(dp);
        assert(dfl);

        /* add date column */
        df=AF_Report_DataField_new();
        AF_Report_DataField_SetDataAsDate(df, dt);
        AF_Report_DataField_List_Add(df, dfl);
        GWEN_Date_free(dt);

        /* add pos amount column */
        if (flags & AE_BOOK_REPORT_FLAG_ADD_POS_AMT) {
          df=AF_Report_DataField_new();
          AF_Report_DataField_SetDataAsValue(df, vPos);
          AF_Report_DataField_List_Add(df, dfl);
        }

        /* add neg amount column */
        if (flags & AE_BOOK_REPORT_FLAG_ADD_NEG_AMT) {
          df=AF_Report_DataField_new();
          AF_Report_DataField_SetDataAsValue(df, vNeg);
          AF_Report_DataField_List_Add(df, dfl);
        }

        /* add sum amount column */
        if (flags & AE_BOOK_REPORT_FLAG_ADD_SUM_AMT) {
          df=AF_Report_DataField_new();
          AF_Report_DataField_SetDataAsValue(df, v);
          AF_Report_DataField_List_Add(df, dfl);
        }
        AQDB_Value_free(v);

        /* add to linear list */
        AF_Report_Data_AddDataPairToList(rd, dp);

        rm=AF_Report_Month_List_Next(rm);
      }
      ry=AF_Report_Year_List_Next(ry);
    }

    cat=AF_Report_Category_List_Next(cat);
  }
}



void AE_Book_Report_ToFlatCategoryList(AE_BOOK *b, AF_REPORT_DATA *rd, uint32_t flags) {
  AF_REPORT_AXIS_LIST *al;
  AF_REPORT_AXIS *ax;
  AF_REPORT_CATEGORY *cat;

  al=AF_Report_Data_GetAxisList(rd);
  assert(al);
  AF_Report_Axis_List_Clear(al);

  /* create date column */
  ax=AF_Report_Axis_new();
  AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT|Category"));
  AF_Report_Axis_SetDataType(ax, AQDB_DataType_String);
  AF_Report_Axis_List_Add(ax, al);

  /* create pos amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_POS_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT|Deposit"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  /* create neg amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_NEG_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT_DATE|Withdrawal"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  /* create sum amount column */
  if (flags & AE_BOOK_REPORT_FLAG_ADD_SUM_AMT) {
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetLabel(ax, I18N("AE_BOOK_REPORT_DATE|Balance"));
    AF_Report_Axis_SetDataType(ax, AQDB_DataType_Amount);
    AF_Report_Axis_List_Add(ax, al);
  }

  AF_Report_DataPair_List_Clear(AF_Report_Data_GetDataPairList(rd));

  cat=AF_Report_Category_List_First(AF_Report_Data_GetCategoryList(rd));
  while(cat) {
    AF_REPORT_DATAPAIR *dp;
    AF_REPORT_DATAFIELD *df;
    AF_REPORT_DATAFIELD_LIST *dfl;
    const AQDB_VALUE *vPos;
    const AQDB_VALUE *vNeg;
    AQDB_VALUE *v;
    const char *s;
  
    vPos=AF_Report_Category_GetPosAmount(cat);
    vNeg=AF_Report_Category_GetNegAmount(cat);
  
    v=AQDB_Value_dup(vPos);
    AQDB_Value_AddValue(v, vNeg);
  
    dp=AF_Report_DataPair_new();
    dfl=AF_Report_DataPair_GetDataFieldList(dp);
    assert(dfl);
  
    /* add category column */
    s=AF_Report_Category_GetLabel(cat);

    df=AF_Report_DataField_new();
    AF_Report_DataField_SetData(df, s);
    AF_Report_DataField_List_Add(df, dfl);
  
    /* add pos amount column */
    if (flags & AE_BOOK_REPORT_FLAG_ADD_POS_AMT) {
      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsValue(df, vPos);
      AF_Report_DataField_List_Add(df, dfl);
    }
  
    /* add neg amount column */
    if (flags & AE_BOOK_REPORT_FLAG_ADD_NEG_AMT) {
      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsValue(df, vNeg);
      AF_Report_DataField_List_Add(df, dfl);
    }
  
    /* add sum amount column */
    if (flags & AE_BOOK_REPORT_FLAG_ADD_SUM_AMT) {
      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsValue(df, v);
      AF_Report_DataField_List_Add(df, dfl);
    }
  
    AQDB_Value_free(v);
  
    /* add to linear list */
    AF_Report_Data_AddDataPairToList(rd, dp);

    cat=AF_Report_Category_List_Next(cat);
  }
}



void AE_Book_Report_CreateColumns(AE_BOOK *b,
				  const AF_REPORT_CFG *cfg,
				  AF_REPORT_DATA *rd) {
  const AF_REPORT_CFG_COLUMN_LIST *cl;
  AF_REPORT_CFG_COLUMN *col;

  cl=AF_Report_Cfg_GetColumnList(cfg);
  assert(cl);
  col=AF_Report_Cfg_Column_List_First(cl);
  while(col) {
    AF_REPORT_AXIS *ax;
    const char *s;

    /* create and setup axis */
    ax=AF_Report_Axis_new();
    AF_Report_Axis_SetDataType(ax, AF_Report_Cfg_Column_GetDataType(col));
    s=AF_Report_Cfg_Column_GetHeader(col);
    AF_Report_Axis_SetLabel(ax, s);

    /* add axis */
    AF_Report_Axis_List_Add(ax, AF_Report_Data_GetAxisList(rd));

    col=AF_Report_Cfg_Column_List_Next(col);
  }
}



int AE_Book_Report_DbToDataPair(AE_BOOK *b,
				const AF_REPORT_CFG *cfg,
				AF_REPORT_DATAPAIR *dp,
				GWEN_DB_NODE *dbData) {
  const AF_REPORT_CFG_COLUMN_LIST *cl;
  AF_REPORT_CFG_COLUMN *col;

  cl=AF_Report_Cfg_GetColumnList(cfg);
  assert(cl);
  col=AF_Report_Cfg_Column_List_First(cl);
  while(col) {
    AF_REPORT_DATAFIELD *df;
    int dataType;
    const char *varName;
    int v;
    const char *s;

    /* create and setup axis */
    df=AF_Report_DataField_new();

    varName=AF_Report_Cfg_Column_GetVarName(col);
    assert(varName);
    dataType=AF_Report_Cfg_Column_GetDataType(col);
    switch(dataType) {
      case AQDB_DataType_Int:
	v=GWEN_DB_GetIntValue(dbData, varName, 0, 0);
	AF_Report_DataField_SetDataAsInt(df, v);
	break;
  
      default:
      case AQDB_DataType_String:
      case AQDB_DataType_Date:
      case AQDB_DataType_Amount:
	s=GWEN_DB_GetCharValue(dbData, varName, 0, NULL);
	AF_Report_DataField_SetData(df, s);
	break;
    }

    /* add data field */
    AF_Report_DataField_List_Add(df, AF_Report_DataPair_GetDataFieldList(dp));

    col=AF_Report_Cfg_Column_List_Next(col);
  }

  return 0;
}




void AE_Book_Report_AddStatementsToReportData(AE_BOOK *b,
                                              AF_REPORT_DATA *rd,
					      const AF_REPORT_CFG *cfg,
                                              const AE_STATEMENT_LIST *sl) {
  AE_STATEMENT *st;
  const AF_REPORT_CFG_COLUMN_LIST *cl;

  cl=AF_Report_Cfg_GetColumnList(cfg);
  assert(cl);
  st=AE_Statement_List_First(sl);
  while(st) {
    const GWEN_DATE *d;
    const AQDB_VALUE *v;

    d=AE_Statement_GetDate(st);
    v=AE_Statement_GetValue(st);
    if (d && v) {
      GWEN_DB_NODE *dbData;
      int rv;

      dbData=GWEN_DB_Group_new("data");
      rv=AE_Statement_toReportDb(st, dbData);
      if (rv<0) {
	DBG_ERROR(AE_LOGDOMAIN, "Error in AE_Statement_toReportDb (%d), ignoring statement", rv);
      }
      else {
	AF_REPORT_DATAPAIR *dp;

	dp=AF_Report_DataPair_new();
	rv=AE_Book_Report_DbToDataPair(b, cfg, dp, dbData);
	if (rv<0) {
	  DBG_ERROR(AE_LOGDOMAIN, "Error in AE_Book_Report_DbToDataPair (%d), ignoring statement", rv);
	  AF_Report_DataPair_free(dp);
	}
	else {
	  /* add to date-based list */
	  AF_Report_Data_AddDataPairToCategories(rd, 0, NULL, d, v, dp);
	}
      }
      GWEN_DB_Group_free(dbData);
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Statement needs both amount and date, ignoring");
    }

    st=AE_Statement_List_Next(st);
  }
}



void AE_Book_Report_AddSplitsToReportData(AE_BOOK *b,
                                          AF_REPORT_DATA *rd,
                                          const AF_REPORT_CFG *cfg,
                                          const AE_SPLIT_LIST *sl) {
  AE_SPLIT *sp;
  const AF_REPORT_CFG_COLUMN_LIST *cl;

  cl=AF_Report_Cfg_GetColumnList(cfg);
  assert(cl);
  sp=AE_Split_List_First(sl);
  while(sp) {
    const AE_TRANSACTION *t;

    t=AE_Split_GetTmpTransaction(sp);
    if (t) {
      const GWEN_DATE *d;
      const AQDB_VALUE *v;

      d=AE_Transaction_GetDate(t);
      v=AE_Split_GetAmount(sp);
      if (d && v) {
	GWEN_DB_NODE *dbData;
	int rv;

	dbData=GWEN_DB_Group_new("data");
	rv=AE_Split_toReportDb(sp, dbData);
	if (rv<0) {
	  DBG_ERROR(AE_LOGDOMAIN, "Error in AE_Split_toReportDb (%d), ignoring split", rv);
	}
	else {
	  AF_REPORT_DATAPAIR *dp;

	  dp=AF_Report_DataPair_new();
	  rv=AE_Book_Report_DbToDataPair(b, cfg, dp, dbData);
	  if (rv<0) {
	    DBG_ERROR(AE_LOGDOMAIN, "Error in AE_Book_Report_DbToDataPair (%d), ignoring split", rv);
	    AF_Report_DataPair_free(dp);
	  }
          else {
            if (AF_Report_Cfg_GetFlags(cfg) & AF_REPORT_CFG_FLAG_REVERSE_SIGN) {
              AQDB_VALUE *nv;

              nv=AQDB_Value_dup(v);
              AQDB_Value_Negate(nv);
              /* add to date-based list */
              AF_Report_Data_AddDataPairToCategories(rd,
                                                     AE_Split_GetAccountId(sp),
                                                     AE_Split_GetTmpAccountPath(sp),
                                                     d, nv, dp);
              AQDB_Value_free(nv);
            }
            else {
              /* add to date-based list */
              AF_Report_Data_AddDataPairToCategories(rd,
                                                     AE_Split_GetAccountId(sp),
                                                     AE_Split_GetTmpAccountPath(sp),
                                                     d, v, dp);
            }
	  }
	}
	GWEN_DB_Group_free(dbData);
      }
      else {
	DBG_ERROR(AE_LOGDOMAIN, "Statement needs both amount and date, ignoring");
      }
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Split needs a transaction pointer, ignoring");
    }

    sp=AE_Split_List_Next(sp);
  }
}



int AE_Book_Report_SumSplitsIntoReportData(AE_BOOK *b,
					   AF_REPORT_DATA *rd,
					   const AF_REPORT_CFG *cfg,
					   const AE_SPLIT_LIST *sl) {
  AE_ACCOUNT_TREE *at;
  const AE_SPLIT *sp;
  int rv;
  AF_REPORT_CATEGORY_LIST *cl;

  /* create tree */
  at=AE_Account_Tree_new();
  rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  cl=AF_Report_Data_GetCategoryList(rd);

  sp=AE_Split_List_First(sl);
  while(sp) {
    AQDB_ID aid;

    aid=AE_Split_GetAccountId(sp);
    if (aid) {
      const AQDB_VALUE *val;

      val=AE_Split_GetAmount(sp);
      if (val) {
	AE_ACCOUNT *acc;

	acc=AE_Account_Tree_GetById(at, aid);
	while(acc) {
	  AF_REPORT_CATEGORY *rc;

	  rc=AF_Report_Category_List_GetByCategory(cl, AE_Account_GetId(acc));
          if (rc==NULL) {
            GWEN_BUFFER *tbuf;

            tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            AE_Account_PathToBuffer(acc, tbuf);
	    rc=AF_Report_Category_new();
	    AF_Report_Category_SetCategory(rc, AE_Account_GetId(acc));
            AF_Report_Category_SetLabel(rc, GWEN_Buffer_GetStart(tbuf));
            AF_Report_Category_List_Add(rc, cl);
            GWEN_Buffer_free(tbuf);
	  }

          if (AF_Report_Cfg_GetFlags(cfg) & AF_REPORT_CFG_FLAG_REVERSE_SIGN) {
            AQDB_VALUE *nv;

            nv=AQDB_Value_dup(val);
            AQDB_Value_Negate(nv);

            if (AQDB_Value_IsNegative(nv))
              AQDB_Value_AddValue(AF_Report_Category_GetNegAmount(rc), nv);
            else
              AQDB_Value_AddValue(AF_Report_Category_GetPosAmount(rc), nv);
            AQDB_Value_free(nv);
          }
          else {
            if (AQDB_Value_IsNegative(val))
              AQDB_Value_AddValue(AF_Report_Category_GetNegAmount(rc), val);
            else
              AQDB_Value_AddValue(AF_Report_Category_GetPosAmount(rc), val);
          }

	  acc=AE_Account_Tree_GetParent(acc);
	}
      }
    }

    sp=AE_Split_List_Next(sp);
  }

  AE_Account_Tree_free(at);
  return 0;
}









