/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */

#include "af_report2_l.h"




static void AF_Book_Report2__AccountTree2CategoryTree(AE_ACCOUNT *acc, AF_REPORT2_CATEGORY_TREE *catTree,
                                                      AF_REPORT2_CATEGORY *parentCat) {
  AF_REPORT2_CATEGORY *cat;

  cat=AF_Report2_Category_new();
  AF_Report2_Category_SetId(cat, AE_Account_GetId(acc));
  AF_Report2_Category_SetName(cat, AE_Account_GetName(acc));
  if (parentCat)
    AF_Report2_Category_Tree_AddChild(parentCat, cat);
  else
    AF_Report2_Category_Tree_Add(catTree, cat);

  acc=AE_Account_Tree_GetFirstChild(acc);
  while(acc) {
    AF_Book_Report2__AccountTree2CategoryTree(acc, catTree, cat);
    acc=AE_Account_Tree_GetNext(acc);
  }
}




int AF_Book_Report2_AccountTree2CategoryTree(AE_BOOK *bk, AF_REPORT2_CATEGORY_TREE *catTree) {
  int rv;
  AE_ACCOUNT_TREE *at;
  AE_ACCOUNT *acc;

  assert(bk);

  /* read account tree */
  at=AE_Account_Tree_new();
  rv=AE_Book_ReadAccountTree(bk, AE_Book_TableType_Account, at);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  acc=AE_Account_Tree_GetFirst(at);
  while(acc) {
    AF_Book_Report2__AccountTree2CategoryTree(acc, catTree, NULL);
    acc=AE_Account_Tree_GetNext(acc);
  }

  AE_Account_Tree_free(at);
  return 0;
}



void AF_Book_Report2_SplitList2Report(AE_BOOK *bk, AF_REPORT2 *rp, const AE_SPLIT_LIST *spList) {
  const AE_SPLIT *sp;

  sp=AE_Split_List_First(spList);
  while(sp) {
    const AE_TRANSACTION *t;

    t=AE_Split_GetTmpTransaction(sp);
    if (t) {
      const GWEN_DATE *dt;
      const AQDB_VALUE *v;

      dt=AE_Transaction_GetDate(t);
      v=AE_Split_GetAmount(sp);
      if (dt && sp) {
        AF_REPORT2_DATAPAIR *dp;
        const char *s;

        dp=AF_Report2_DataPair_new();
        AF_Report2_DataPair_SetCategoryId(dp, AE_Split_GetAccountId(sp));

        AF_Report2_DataPair_SetDate(dp, dt);
        if (AE_Split_GetRuntimeFlags(sp) & AE_SPLIT_RTFLAGS_REVERSE)
          AF_Report2_DataPair_SetValue(dp, -AQDB_Value_GetValueAsDouble(v));
        else
          AF_Report2_DataPair_SetValue(dp, AQDB_Value_GetValueAsDouble(v));
        s=AE_Split_GetMemo(sp);
        if (!(s && *s))
          s=AE_Transaction_GetMemo(t);
        AF_Report2_DataPair_SetLabel(dp, s);

#if 0
        DBG_ERROR(0, "%s: Adding %f [%s] [%s]",
                  GWEN_Date_GetString(dt),
                  AF_Report2_DataPair_GetValue(dp),
                  AE_Split_GetTmpAccountPath(sp),
                  s);
#endif
        AF_Report2_AddDataPair(rp, dp);
      }
    }

    sp=AE_Split_List_Next(sp);
  }
}



int AE_Book_Report2_ReadMatchingSplits(AE_BOOK *b, AF_REPORT2 *rp, 
                                       const GWEN_DATE *fromDate,
                                       const GWEN_DATE *toDate,
                                       const char *accountPattern,
                                       const char *commodity) {
  AE_SPLIT_LIST *spList;
  int rv;

  spList=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndAccount(b, fromDate, toDate, accountPattern, commodity, spList);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Split_List_free(spList);
    return rv;
  }
  DBG_ERROR(0, "Got %d splits", AE_Split_List_GetCount(spList));
  AF_Book_Report2_SplitList2Report(b, rp, spList);
  AE_Split_List_free(spList);

  return 0;
}












