/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */


#define WRITE_TYPE(macro_prefix, macro_type)                                      \
int AE_Book_Add##macro_prefix(AE_BOOK *b,                                         \
                       AE_BOOK_TABLE_TYPE tt,                                     \
                       AE_##macro_type *ba,                                       \
                       int doLock) {                                              \
  int rv;                                                                         \
  AQDB_ID nextId;                                                                 \
  AQDB_ID dbId=0;                                                                 \
  AQDB_OBJECT *o=NULL;                                                            \
  AQDB_ID tableId;                                                                \
                                                                                  \
  assert(b);                                                                      \
  assert(b->refCount);                                                            \
                                                                                  \
  if (b->dbOpened<1) {                                                            \
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");                                     \
    return GWEN_ERROR_NOT_OPEN;                                                   \
  }                                                                               \
                                                                                  \
  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {                \
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);                             \
    return GWEN_ERROR_INVALID;                                                    \
  }                                                                               \
                                                                                  \
  tableId=AE_Book_GetTableId(b, tt);                                              \
  if (tableId==0) {                                                               \
    DBG_ERROR(AE_LOGDOMAIN, "Table type %d not open", (int)tt);                   \
    return GWEN_ERROR_NOT_OPEN;                                                   \
  }                                                                               \
                                                                                  \
  if (doLock) {                                                                   \
    rv=AE_Book_BeginEdit(b, 0);                                                   \
    if (rv<0) {                                                                   \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                    \
      return rv;                                                                  \
    }                                                                             \
  }                                                                               \
                                                                                  \
  rv=AE_Book_GetNextId(b, tt, &nextId);                                           \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  rv=AE_##macro_prefix##_CreateObject(b->dataBase, tableId, nextId, &o);          \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  AE_##macro_prefix##_SetId(ba, nextId);                                          \
  rv=AE_##macro_prefix##_toObject(ba, o);                                         \
  AE_##macro_prefix##_SetId(ba, 0);                                               \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    AQDB_Object_free(o);                                                          \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  rv=AQDB_DB_AddObject(b->dataBase, tableId, o);                                  \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    AQDB_Object_free(o);                                                          \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
  dbId=AQDB_Object_GetObjectId(o);                                                \
                                                                                  \
  AQDB_Object_free(o);                                                            \
  if (doLock) {                                                                   \
    rv=AE_Book_EndEdit(b, 0);                                                     \
    if (rv<0) {                                                                   \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                    \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
      return rv;                                                                  \
    }                                                                             \
  }                                                                               \
                                                                                  \
  AE_##macro_prefix##_SetId(ba, nextId);                                          \
  AE_##macro_prefix##_SetDbId(ba, dbId);                                          \
                                                                                  \
  return 0;                                                                       \
}                                                                                 \
                                                                                  \
                                                                                  \
                                                                                  \
int AE_Book_Write##macro_prefix(AE_BOOK *b,                                       \
                         AE_BOOK_TABLE_TYPE tt,                                   \
                         const AE_##macro_type *ba,                             \
                         int doLock) {                                            \
  int rv;                                                                         \
  AQDB_OBJECT *o=NULL;                                                            \
  AQDB_ID tableId;                                                                \
                                                                                  \
  assert(b);                                                                      \
  assert(b->refCount);                                                            \
                                                                                  \
  if (b->dbOpened<1) {                                                            \
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");                                     \
    return GWEN_ERROR_NOT_OPEN;                                                   \
  }                                                                               \
                                                                                  \
  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {                \
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);                             \
    return GWEN_ERROR_INVALID;                                                    \
  }                                                                               \
                                                                                  \
  tableId=AE_Book_GetTableId(b, tt);                                              \
  if (tableId==0) {                                                               \
    DBG_ERROR(AE_LOGDOMAIN, "Table type %d not open", (int)tt);                   \
    return GWEN_ERROR_NOT_OPEN;                                                   \
  }                                                                               \
                                                                                  \
  if (doLock) {                                                                   \
    rv=AE_Book_BeginEdit(b, 0);                                                   \
    if (rv<0) {                                                                   \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                    \
      return rv;                                                                  \
    }                                                                             \
  }                                                                               \
                                                                                  \
  rv=AE_##macro_prefix##_CreateObject(b->dataBase, tableId,                       \
                                 AE_##macro_prefix##_GetDbId(ba),                 \
                                 &o);                                             \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  rv=AE_##macro_prefix##_toObject(ba, o);                                         \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    AQDB_Object_free(o);                                                          \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  rv=AQDB_DB_WriteObject(b->dataBase, o);                                         \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    AQDB_Object_free(o);                                                          \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  AQDB_Object_free(o);                                                            \
  if (doLock) {                                                                   \
    rv=AE_Book_EndEdit(b, 0);                                                     \
    if (rv<0) {                                                                   \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                    \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
      return rv;                                                                  \
    }                                                                             \
  }                                                                               \
                                                                                  \
  return 0;                                                                       \
}                                                                                 \
                                                                                  \
                                                                                  \
                                                                                  \
int AE_Book_Delete##macro_prefix(AE_BOOK *b,                                      \
                          AE_BOOK_TABLE_TYPE tt,                                  \
                          const AE_##macro_type *ba,                              \
                          int doLock) {                                           \
  int rv;                                                                         \
  AQDB_ID tableId;                                                                \
                                                                                  \
  assert(b);                                                                      \
  assert(b->refCount);                                                            \
                                                                                  \
  if (b->dbOpened<1) {                                                            \
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");                                     \
    return GWEN_ERROR_NOT_OPEN;                                                   \
  }                                                                               \
                                                                                  \
  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {                \
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);                             \
    return GWEN_ERROR_INVALID;                                                    \
  }                                                                               \
                                                                                  \
  tableId=AE_Book_GetTableId(b, tt);                                              \
  if (tableId==0) {                                                               \
    DBG_ERROR(AE_LOGDOMAIN, "Table type %d not open", (int)tt);                   \
    return GWEN_ERROR_NOT_OPEN;                                                   \
  }                                                                               \
                                                                                  \
  if (AE_##macro_prefix##_GetDbId(ba)==0) {                                       \
    DBG_ERROR(AE_LOGDOMAIN, "No db id");                                          \
    abort();                                                                      \
    return GWEN_ERROR_INVALID;                                                    \
  }                                                                               \
                                                                                  \
  if (doLock) {                                                                   \
    rv=AE_Book_BeginEdit(b, 0);                                                   \
    if (rv<0) {                                                                   \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                    \
      return rv;                                                                  \
    }                                                                             \
  }                                                                               \
                                                                                  \
  rv=AQDB_DB_DeleteObject(b->dataBase, tableId, AE_##macro_prefix##_GetDbId(ba)); \
  if (rv<0) {                                                                     \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                      \
    if (doLock)                                                                   \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
    return rv;                                                                    \
  }                                                                               \
                                                                                  \
  if (doLock) {                                                                   \
    rv=AE_Book_EndEdit(b, 0);                                                     \
    if (rv<0) {                                                                   \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                    \
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);                                \
      return rv;                                                                  \
    }                                                                             \
  }                                                                               \
                                                                                  \
  return 0;                                                                       \
}



WRITE_TYPE(Group, GROUP)
WRITE_TYPE(BankBalance, BANKBALANCE)
WRITE_TYPE(Statement, STATEMENT)
WRITE_TYPE(BankAccount, BANKACCOUNT)
WRITE_TYPE(Commodity, COMMODITY)
WRITE_TYPE(Transaction, TRANSACTION)
WRITE_TYPE(Split, SPLIT)
WRITE_TYPE(Account, ACCOUNT)
WRITE_TYPE(AccountBalance, ACCOUNTBALANCE)
WRITE_TYPE(Rule, RULE)
WRITE_TYPE(Address, ADDRESS)
WRITE_TYPE(Partner, PARTNER)
WRITE_TYPE(Task, TASK)
WRITE_TYPE(Job, JOB)
WRITE_TYPE(Item, ITEM)
WRITE_TYPE(Tax, TAX)
WRITE_TYPE(LoadCellPhone, LOAD_CELLPHONE)
WRITE_TYPE(Recurrence, RECURRENCE)
WRITE_TYPE(Vehicle, VEHICLE)
WRITE_TYPE(Refueling, REFUELING)
WRITE_TYPE(Data10k, DATA_10K)
WRITE_TYPE(Share, SHARE)


#undef WRITE_TYPE




int AE_Book_WriteAccountTreeBalances(AE_BOOK *b, AE_ACCOUNT_TREE *at, int doLock) {
  int rv;
  AE_ACCOUNT *acc;

  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  acc=AE_Account_Tree_GetFirst(at);
  while(acc) {
    AE_ACCOUNTBALANCE *bal;

    bal=AE_Account_GetBalance(acc);
    if (bal) {
      AQDB_ID bid;

      AE_AccountBalance_SetAccountId(bal, AE_Account_GetId(acc));
      bid=AE_AccountBalance_GetId(bal);
      if (bid==0) {
	rv=AE_Book_AddAccountBalance(b, AE_Book_TableType_AccountBalance, bal, 0);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  if (doLock)
	    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
          return rv;
	}
      }
      else {
	rv=AE_Book_WriteAccountBalance(b, AE_Book_TableType_AccountBalance, bal, 0);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  if (doLock)
	    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	  return rv;
	}
      }
    }

    acc=AE_Account_Tree_GetBelow(acc);
  }

  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }

  /* done */
  return 0;
}



void AE_Book_WriteAccountTreeXml(AE_BOOK *b,
				 const AE_ACCOUNT_TREE *at,
				 const AE_ACCOUNT *acc,
				 GWEN_XMLNODE *node) {

  if (acc) {
    GWEN_XMLNODE *n;

    n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "account");
    AE_Account_toXml(acc, n);
    GWEN_XMLNode_AddChild(node, n);

    acc=AE_Account_Tree_GetFirstChild(acc);
    while(acc) {
      AE_Book_WriteAccountTreeXml(b, at, acc, n);
      acc=AE_Account_Tree_GetNext(acc);
    }
  }
  else {
    acc=AE_Account_Tree_GetFirst(at);
    while(acc) {
      GWEN_XMLNODE *n;

      n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "account");
      AE_Account_toXml(acc, n);
      GWEN_XMLNode_AddChild(node, n);

      acc=AE_Account_Tree_GetBelow(acc);
    }
  }
}




