/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



void AE_Job_AddItem(AE_JOB *j, AE_ITEM *item) {
  assert(j);
  assert(j->_refCount);

  assert(j->items);
  AE_Item_List_Add(item, j->items);
}



AE_ITEM* AE_Job_GetFirstItem(const AE_JOB *j) {
  assert(j);
  assert(j->_refCount);

  if (j->items)
    return AE_Item_List_First(j->items);
  return NULL;
}



int AE_Job_SampleBuyTaxes(const AE_JOB *j, AQDB_ID *idBuffer, int size) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;
    int cnt=0;

    item=AE_Item_List_First(j->items);
    while(item) {
      AQDB_ID id;

      id=AE_Item_GetBuyTaxId(item);
      if (id) {
	int i;

	for (i=0; i<cnt;i++)
	  if (idBuffer[i]==id)
	    break;
	if (i>=cnt) {
	  /* add tax */
	  if (cnt>=size) {
	    DBG_ERROR(0, "Too many different taxes in this job");
	    break;
	  }
	  else
	    idBuffer[cnt++]=id;
	}
      }
      item=AE_Item_List_Next(item);
    }

    return cnt;
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
    return 0;
  }
}



int AE_Job_SampleSellTaxes(const AE_JOB *j, AQDB_ID *idBuffer, int size) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;
    int cnt=0;

    item=AE_Item_List_First(j->items);
    while(item) {
      AQDB_ID id;

      id=AE_Item_GetSellTaxId(item);
      if (id) {
	int i;

	for (i=0; i<cnt;i++)
	  if (idBuffer[i]==id)
	    break;
	if (i>=cnt) {
	  /* add tax */
	  if (cnt>=size) {
	    DBG_ERROR(0, "Too many different taxes in this job");
	    break;
	  }
	  else
	    idBuffer[cnt++]=id;
	}
      }
      item=AE_Item_List_Next(item);
    }

    return cnt;
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
    return 0;
  }
}



int AE_Job_CalculateBuyTax(const AE_JOB *j, AQDB_ID taxId, AQDB_VALUE *v) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;

    item=AE_Item_List_First(j->items);
    while(item) {
      AQDB_ID id;

      id=AE_Item_GetBuyTaxId(item);
      if (id && id==taxId) {
	const AQDB_VALUE *vt;

	vt=AE_Item_GetBuyTaxSumValue(item);
	if (vt)
          AQDB_Value_AddValue(v, vt);
      }

      item=AE_Item_List_Next(item);
    }
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
  }
  return 0;
}



int AE_Job_CalculateSellTax(const AE_JOB *j, AQDB_ID taxId, AQDB_VALUE *v) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;

    item=AE_Item_List_First(j->items);
    while(item) {
      AQDB_ID id;

      id=AE_Item_GetSellTaxId(item);
      if (id && id==taxId) {
	const AQDB_VALUE *vt;

	vt=AE_Item_GetSellTaxSumValue(item);
	if (vt)
          AQDB_Value_AddValue(v, vt);
      }

      item=AE_Item_List_Next(item);
    }
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
  }
  return 0;
}



int AE_Job_CalculateBuyNetSum(const AE_JOB *j, AQDB_VALUE *v) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;

    item=AE_Item_List_First(j->items);
    while(item) {
      const AQDB_VALUE *vt;

      vt=AE_Item_GetBuyPriceValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);

      item=AE_Item_List_Next(item);
    }
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
  }

  return 0;
}



int AE_Job_CalculateSellNetSum(const AE_JOB *j, AQDB_VALUE *v) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;

    item=AE_Item_List_First(j->items);
    while(item) {
      const AQDB_VALUE *vt;

      vt=AE_Item_GetSellPriceValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);

      item=AE_Item_List_Next(item);
    }
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
  }

  return 0;
}



int AE_Job_CalculateBuyTotalSum(const AE_JOB *j, AQDB_VALUE *v) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;

    item=AE_Item_List_First(j->items);
    while(item) {
      const AQDB_VALUE *vt;

      vt=AE_Item_GetBuyPriceValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);

      vt=AE_Item_GetBuyTaxSumValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);

      item=AE_Item_List_Next(item);
    }
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
  }

  return 0;
}



int AE_Job_CalculateSellTotalSum(const AE_JOB *j, AQDB_VALUE *v) {
  assert(j);
  assert(j->_refCount);

  if (j->items) {
    const AE_ITEM *item;

    item=AE_Item_List_First(j->items);
    while(item) {
      const AQDB_VALUE *vt;

      vt=AE_Item_GetSellPriceValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);

      vt=AE_Item_GetSellTaxSumValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);

      item=AE_Item_List_Next(item);
    }
  }
  else {
    /* no items */
    DBG_ERROR(AE_LOGDOMAIN, "No items");
  }

  return 0;
}









