/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_query_stm_p.h"

#include <aqdatabase/aqdb_query_be.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>



AQDB_QUERY *AE_StatementQuery_new(const char *s) {
  AQDB_QUERY *q;
  int rv;

  q=AQDB_Query_new();

  AQDB_Query_SetMkFieldExprFn(q, AE_StatementQuery_MkFieldExpr);
  AQDB_Query_SetGetFieldDataFn(q, AE_StatementQuery_GetFieldData);
  rv=AQDB_Query_SetString(q, s);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AQDB_Query_free(q);
    return NULL;
  }

  return q;
}



int AE_StatementQuery_Check(AQDB_QUERY *q, const AE_STATEMENT *st) {
  int rv;

  q->object=(const void*) st;
  rv=AQDB_Query_Calculate(q);
  q->object=NULL;
  return rv;
}



AQDB_QUERY_EXPR GWENHYWFAR_CB *AE_StatementQuery_MkFieldExpr(AQDB_QUERY *q, const char *name) {
  if (strcasecmp(name, "id")==0)
    return AQDB_Query_Expr_Field_new(q, 0, AQDB_DataType_Int);
  else if (strcasecmp(name, "bankAccountId")==0)
    return AQDB_Query_Expr_Field_new(q, 1, AQDB_DataType_Int);
  else if (strcasecmp(name, "localCountry")==0)
    return AQDB_Query_Expr_Field_new(q, 2, AQDB_DataType_String);
  else if (strcasecmp(name, "localBankCode")==0)
    return AQDB_Query_Expr_Field_new(q, 3, AQDB_DataType_String);
  else if (strcasecmp(name, "localAccountNumber")==0)
    return AQDB_Query_Expr_Field_new(q, 4, AQDB_DataType_String);
  else if (strcasecmp(name, "localiban")==0)
    return AQDB_Query_Expr_Field_new(q, 5, AQDB_DataType_String);
  else if (strcasecmp(name, "localbic")==0)
    return AQDB_Query_Expr_Field_new(q, 6, AQDB_DataType_String);
  else if (strcasecmp(name, "localname")==0)
    return AQDB_Query_Expr_Field_new(q, 7, AQDB_DataType_String);
  else if (strcasecmp(name, "remoteCountry")==0)
    return AQDB_Query_Expr_Field_new(q, 8, AQDB_DataType_String);
  else if (strcasecmp(name, "remoteBankCode")==0)
    return AQDB_Query_Expr_Field_new(q, 9, AQDB_DataType_String);
  else if (strcasecmp(name, "remoteAccountNumber")==0)
    return AQDB_Query_Expr_Field_new(q, 10, AQDB_DataType_String);
  else if (strcasecmp(name, "remoteiban")==0)
    return AQDB_Query_Expr_Field_new(q, 11, AQDB_DataType_String);
  else if (strcasecmp(name, "remotebic")==0)
    return AQDB_Query_Expr_Field_new(q, 12, AQDB_DataType_String);
  else if (strcasecmp(name, "remotename")==0)
    return AQDB_Query_Expr_Field_new(q, 13, AQDB_DataType_String);
  else if (strcasecmp(name, "date")==0)
    return AQDB_Query_Expr_Field_new(q, 14, AQDB_DataType_Date);
  else if (strcasecmp(name, "valutadate")==0)
    return AQDB_Query_Expr_Field_new(q, 15, AQDB_DataType_Date);
  else if (strcasecmp(name, "amount")==0)
    return AQDB_Query_Expr_Field_new(q, 16, AQDB_DataType_Amount);
  else if (strcasecmp(name, "currency")==0)
    return AQDB_Query_Expr_Field_new(q, 17, AQDB_DataType_String);
  else if (strcasecmp(name, "fiid")==0)
    return AQDB_Query_Expr_Field_new(q, 18, AQDB_DataType_String);
  else if (strcasecmp(name, "textkey")==0)
    return AQDB_Query_Expr_Field_new(q, 19, AQDB_DataType_Int);
  else if (strcasecmp(name, "textkeyExt")==0)
    return AQDB_Query_Expr_Field_new(q, 20, AQDB_DataType_Int);
  else if (strcasecmp(name, "infoText")==0)
    return AQDB_Query_Expr_Field_new(q, 21, AQDB_DataType_String);
  else if (strcasecmp(name, "customerRef")==0)
    return AQDB_Query_Expr_Field_new(q, 22, AQDB_DataType_String);
  else if (strcasecmp(name, "bankRef")==0)
    return AQDB_Query_Expr_Field_new(q, 23, AQDB_DataType_String);
  else if (strcasecmp(name, "purpose")==0)
    return AQDB_Query_Expr_Field_new(q, 24, AQDB_DataType_String);
  else if (strcasecmp(name, "status")==0)
    return AQDB_Query_Expr_Field_new(q, 25, AQDB_DataType_Int);
  else if (strcasecmp(name, "transferId")==0)
    return AQDB_Query_Expr_Field_new(q, 26, AQDB_DataType_Int);
  else if (strcasecmp(name, "groupId")==0)
    return AQDB_Query_Expr_Field_new(q, 27, AQDB_DataType_Int);
  else if (strcasecmp(name, "category")==0)
    return AQDB_Query_Expr_Field_new(q, 28, AQDB_DataType_String);
  else if (strcasecmp(name, "flags")==0)
    return AQDB_Query_Expr_Field_new(q, 29, AQDB_DataType_Int);
  else
    return NULL;
}




int GWENHYWFAR_CB AE_StatementQuery_GetFieldData(AQDB_QUERY *q, AQDB_QUERY_EXPR *expr) {
  const AE_STATEMENT *st=(const AE_STATEMENT*)(q->object);
  const char *s;

  if (expr->dataPtr) {
    free(expr->dataPtr);
    expr->dataPtr=NULL;
    expr->dataLen=0;
  }

#define CPYSTR(macro_fn)                           \
  s=AE_Statement_Get##macro_fn(st);                \
  if (s && *s) {                                   \
    expr->dataPtr=strdup(s);                       \
    expr->dataLen=strlen(s);                       \
  }                                                \
  break

#define CPYINT(macro_fn)                           \
  expr->intData=AE_Statement_Get##macro_fn(st);    \
  break

#define CPYAMT(macro_fn)                           \
  expr->typedPtr.value=AE_Statement_Get##macro_fn(st); \
  break

#define CPYDATE(macro_fn)                          \
  expr->typedPtr.date=AE_Statement_Get##macro_fn(st);  \
  break


  switch(expr->fieldIndex) {
  case 0: CPYINT(Id);
  case 1: CPYINT(BankAccountId);

  case 2: CPYSTR(LocalCountry);
  case 3: CPYSTR(LocalBankCode);
  case 4: CPYSTR(LocalAccountNumber);
  case 5: CPYSTR(LocalIban);
  case 6: CPYSTR(LocalBic);
  case 7: CPYSTR(LocalName);

  case 8: CPYSTR(RemoteCountry);
  case 9: CPYSTR(RemoteBankCode);
  case 10: CPYSTR(RemoteAccountNumber);
  case 11: CPYSTR(RemoteIban);
  case 12: CPYSTR(RemoteBic);
  case 13: CPYSTR(RemoteName);

  case 14: CPYDATE(Date);
  case 15: CPYDATE(ValutaDate);
  case 16: CPYAMT(Value);
  case 17: CPYSTR(Currency);
  case 18: CPYSTR(FiId);
  case 19: CPYINT(TextKey);
  case 20: CPYINT(TextKeyExt);
  case 21: CPYSTR(InfoText);
  case 22: CPYSTR(CustomerRef);
  case 23: CPYSTR(BankRef);
  case 24: CPYSTR(Purpose);
  case 25: CPYINT(Status);
  case 26: CPYINT(TransferId);
  case 27: CPYINT(GroupId);
  case 28: CPYSTR(Category);
  case 29: CPYINT(Flags);
  default:
    DBG_ERROR(AE_LOGDOMAIN, "Invalid field %d", q->expression->fieldIndex);
    assert(0);
    return GWEN_ERROR_INVALID;
  }
#undef CPYSTR
#undef CPYINT
#undef CPYAMT
#undef CPYDATE

  return 0;
}










