/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#include <aqfinance/engine/book/ae_utils.h>

#include <ctype.h>



int AE_Split_toReportDb(const AE_SPLIT *sp, GWEN_DB_NODE *db) {
  int rv;

  rv=AE_Split_toDb(sp, db);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  if (sp->tmpCommodity) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "commodity");
    assert(dbT);
    AE_Commodity_toDb(sp->tmpCommodity, dbT);
  }

  if (sp->tmpTransaction) {
    GWEN_DB_NODE *dbT;

    dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "transaction");
    assert(dbT);
    AE_Transaction_toDb(sp->tmpTransaction, dbT);
  }

  if (sp->tmpAccountPath)
    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "account/path", sp->tmpAccountPath);

  return 0;
}





AF_REPORT_CFG_COLUMN_LIST *AE_Split_CreateReportColumnList() {
  AF_REPORT_CFG_COLUMN_LIST *cl;
  AF_REPORT_CFG_COLUMN *c;

  cl=AF_Report_Cfg_Column_List_new();

  /* member "memo" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Memo"));
  AF_Report_Cfg_Column_SetVarName(c, "memo");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "value" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Amount);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Amount"));
  AF_Report_Cfg_Column_SetVarName(c, "amount");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "commodity/nameSpace" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Commodity Namespace"));
  AF_Report_Cfg_Column_SetVarName(c, "commodity/nameSpace");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "commodity/name" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Commodity Name"));
  AF_Report_Cfg_Column_SetVarName(c, "commodity/name");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "commodity/symbol" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Commodity Symbol"));
  AF_Report_Cfg_Column_SetVarName(c, "commodity/symbol");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "commodity/code" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Commodity Code"));
  AF_Report_Cfg_Column_SetVarName(c, "commodity/code");
  AF_Report_Cfg_Column_List_Add(c, cl);


  /* member "account/path" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Account Path"));
  AF_Report_Cfg_Column_SetVarName(c, "account/path");
  AF_Report_Cfg_Column_List_Add(c, cl);


  /* member "transaction/memo" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Transaction Memo"));
  AF_Report_Cfg_Column_SetVarName(c, "transaction/memo");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "transaction/date" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Date);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Date"));
  AF_Report_Cfg_Column_SetVarName(c, "transaction/date");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "transaction/valutaDate" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Date);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Valuta Date"));
  AF_Report_Cfg_Column_SetVarName(c, "transaction/valutaDate");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "transaction/checkNumber" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Check Number"));
  AF_Report_Cfg_Column_SetVarName(c, "transaction/checkNumber");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "transaction/receipt" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_SPLIT|Receipt"));
  AF_Report_Cfg_Column_SetVarName(c, "transaction/receipt");
  AF_Report_Cfg_Column_List_Add(c, cl);

  return cl;
}







