/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#include <aqfinance/engine/book/ae_utils.h>


void AE_Task_CalcOpenOrderedItems(AE_TASK *tk) {
  AE_JOB *j;

  j=AE_Job_List_First(AE_Task_GetJobs(tk));
  while(j) {
    int jt;

    jt=AE_Job_GetType(j);
    if (jt==AE_Job_TypeOrder) {
      AE_ITEM *itm;

      itm=AE_Item_List_First(AE_Job_GetItems(j));
      while(itm) {
	const char *s;

	s=AE_Item_GetCode(itm);
	if (s && *s) {
	  AE_ITEM *ti;

	  ti=AE_Item_List_First(AE_Task_GetOpenOrderedItems(tk));
	  while(ti) {
	    const char *t;

	    t=AE_Item_GetCode(ti);
	    if (t && *t && strcasecmp(s, t)==0)
              break;
	    ti=AE_Item_List_Next(ti);
	  }
          if (ti==NULL) {
	    ti=AE_Item_new();
            AE_Item_SetCode(ti, s);
            AE_Item_SetName(ti, AE_Item_GetName(itm));
            AE_Item_SetMemo(ti, AE_Item_GetMemo(itm));
            AE_Item_SetUnits(ti, AE_Item_GetUnits(itm));
	    AE_Item_List_Add(ti, AE_Task_GetOpenOrderedItems(tk));
	  }

	  if (ti) {
	    AQDB_VALUE *v;
	    const AQDB_VALUE *cv;

	    /* calculate amount */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyPriceSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyPriceValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetBuyPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetBuyTaxSumValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate sellPriceValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellPriceValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetSellPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetSellTaxSumValue(ti, v);
	    AQDB_Value_free(v);
	  }

	}
        itm=AE_Item_List_Next(itm);
      }
    }

    else if (jt==AE_Job_TypeDelivery) {
      AE_ITEM *itm;

      itm=AE_Item_List_First(AE_Job_GetItems(j));
      while(itm) {
	const char *s;

	s=AE_Item_GetCode(itm);
	if (s && *s) {
	  AE_ITEM *ti;

	  ti=AE_Item_List_First(AE_Task_GetOpenOrderedItems(tk));
	  while(ti) {
	    const char *t;

	    t=AE_Item_GetCode(ti);
	    if (t && *t && strcasecmp(s, t)==0)
              break;
	    ti=AE_Item_List_Next(ti);
	  }

	  if (ti) {
	    AQDB_VALUE *v;
	    const AQDB_VALUE *cv;

	    /* calculate amount */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyPriceSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyPriceValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetBuyPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetBuyTaxSumValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate sellPriceValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellPriceValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetSellPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetSellTaxSumValue(ti, v);
	    AQDB_Value_free(v);
	  }
	}
        itm=AE_Item_List_Next(itm);
      }
    }

    j=AE_Job_List_Next(j);
  }

  /* remove empty items */
  if (1) {
    AE_ITEM *itm;

    itm=AE_Item_List_First(AE_Task_GetOpenOrderedItems(tk));
    while(itm) {
      const AQDB_VALUE *cv;
      AE_ITEM *next;

      next=AE_Item_List_Next(itm);
      cv=AE_Item_GetValue(itm);
      if (cv==NULL || (cv && AQDB_Value_IsZero(cv))) {
	AE_Item_List_Del(itm);
	AE_Item_free(itm);
      }
      itm=next;
    }
  }

}



void AE_Task_CalcOpenDeliveredItems(AE_TASK *tk) {
  AE_JOB *j;

  j=AE_Job_List_First(AE_Task_GetJobs(tk));
  while(j) {
    int jt;

    jt=AE_Job_GetType(j);
    if (jt==AE_Job_TypeDelivery) {
      AE_ITEM *itm;

      itm=AE_Item_List_First(AE_Job_GetItems(j));
      while(itm) {
	const char *s;

	s=AE_Item_GetCode(itm);
	if (s && *s) {
	  AE_ITEM *ti;

	  ti=AE_Item_List_First(AE_Task_GetOpenDeliveredItems(tk));
	  while(ti) {
	    const char *t;

	    t=AE_Item_GetCode(ti);
	    if (t && *t && strcasecmp(s, t)==0)
              break;
	    ti=AE_Item_List_Next(ti);
	  }
	  if (ti==NULL) {
	    ti=AE_Item_new();
            AE_Item_SetCode(ti, s);
            AE_Item_SetName(ti, AE_Item_GetName(itm));
            AE_Item_SetMemo(ti, AE_Item_GetMemo(itm));
            AE_Item_SetUnits(ti, AE_Item_GetUnits(itm));
	    AE_Item_List_Add(ti, AE_Task_GetOpenDeliveredItems(tk));
	  }

	  if (ti) {
	    AQDB_VALUE *v;
	    const AQDB_VALUE *cv;

	    /* calculate amount */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyPriceSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyPriceValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetBuyPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetBuyTaxSumValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate sellPriceValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellPriceValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetSellPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    AE_Item_SetSellTaxSumValue(ti, v);
	    AQDB_Value_free(v);
	  }

	}
        itm=AE_Item_List_Next(itm);
      }
    }

    else if (jt==AE_Job_TypeInvoice) {
      AE_ITEM *itm;

      itm=AE_Item_List_First(AE_Job_GetItems(j));
      while(itm) {
	const char *s;

	s=AE_Item_GetCode(itm);
	if (s && *s) {
	  AE_ITEM *ti;

	  ti=AE_Item_List_First(AE_Task_GetOpenDeliveredItems(tk));
	  while(ti) {
	    const char *t;

	    t=AE_Item_GetCode(ti);
	    if (t && *t && strcasecmp(s, t)==0)
              break;
	    ti=AE_Item_List_Next(ti);
	  }
	  if (ti==NULL) {
	    ti=AE_Item_new();
	    AE_Item_SetCode(ti, s);
            AE_Item_SetName(ti, AE_Item_GetName(itm));
            AE_Item_SetMemo(ti, AE_Item_GetMemo(itm));
            AE_Item_SetUnits(ti, AE_Item_GetUnits(itm));
	    AE_Item_List_Add(ti, AE_Task_GetOpenDeliveredItems(tk));
	  }

	  if (ti) {
	    AQDB_VALUE *v;
	    const AQDB_VALUE *cv;

	    /* calculate amount */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyPriceSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyPriceValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetBuyPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetBuyTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetBuyTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetBuyTaxSumValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate sellPriceValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellPriceValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellPriceValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetSellPriceValue(ti, v);
	    AQDB_Value_free(v);

	    /* calculate buyTaxSumValue */
	    v=AQDB_Value_new();
	    cv=AE_Item_GetSellTaxSumValue(ti);
	    if (cv)
	      AQDB_Value_AddValue(v, cv);
	    cv=AE_Item_GetSellTaxSumValue(itm);
	    if (cv)
	      AQDB_Value_SubValue(v, cv);
	    AE_Item_SetSellTaxSumValue(ti, v);
	    AQDB_Value_free(v);
	  }
	}
        itm=AE_Item_List_Next(itm);
      }
    }

    j=AE_Job_List_Next(j);
  }

  /* remove empty items */
  if (1) {
    AE_ITEM *itm;

    itm=AE_Item_List_First(AE_Task_GetOpenDeliveredItems(tk));
    while(itm) {
      const AQDB_VALUE *cv;
      AE_ITEM *next;

      next=AE_Item_List_Next(itm);
      cv=AE_Item_GetValue(itm);
      if (cv==NULL || (cv && AQDB_Value_IsZero(cv))) {
	AE_Item_List_Del(itm);
	AE_Item_free(itm);
      }
      itm=next;
    }
  }

}



