/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_idgen.h"


static char _idCharSet[]="CEFGHJKLPQRSTWXYZ235679";



int AE_IdGen_MkId(const char *prefix, GWEN_BUFFER *tbuf) {
  GWEN_TIME *now;
  GWEN_TIME *t0;
  uint64_t d;
  char numbuf[256];
  uint64_t csLen;
  int i;
  int j;
  uint32_t pos;
  const char *s;
  int rv;

  now=GWEN_CurrentTime();
  assert(now);

  t0=GWEN_Time_fromUtcString("20100101-00:00:00", "YYYYMMDD-hh:mm:ss");
  assert(t0);

  d=(uint64_t)GWEN_Time_DiffSeconds(now, t0);
  csLen=strlen(_idCharSet);

  i=0;
  while(d && i<(sizeof(numbuf)-1)) {
    uint64_t r;

    r=d % csLen;
    numbuf[i++]=_idCharSet[r];
    d/=csLen;
  }
  if (i==0)
    numbuf[i++]=_idCharSet[0];
  numbuf[i]=0;

  pos=GWEN_Buffer_GetPos(tbuf);
  if (prefix && *prefix)
    GWEN_Buffer_AppendString(tbuf, prefix);
  for (j=i; j>0; j--) {
    GWEN_Buffer_AppendByte(tbuf, numbuf[j-1]);
  }

  /* generate checksum */
  s=GWEN_Buffer_GetStart(tbuf)+pos;
  j=0;
  while(*s) {
    if (strchr(_idCharSet, *s)) {
      j+=*s;
      j=j % csLen;
    }
    s++;
  }
  j=(csLen-j) % csLen;
  GWEN_Buffer_AppendByte(tbuf, _idCharSet[j]);

  GWEN_Time_free(t0);
  GWEN_Time_free(now);

  rv=AE_IdGen_CheckId(GWEN_Buffer_GetStart(tbuf)+pos);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }
  return 0;
}



int AE_IdGen_CheckId(const char *id) {
  uint64_t csLen;
  int cs;
  int slen;
  int i;

  csLen=strlen(_idCharSet);
  cs=0;
  slen=strlen(id)-1;
  for (i=0; i<slen; i++) {
    if (strchr(_idCharSet, id[i])) {
      cs+=id[i];
      cs=cs % csLen;
    }
  }

  cs=(csLen-cs) % csLen;
  if (id[i]!=_idCharSet[cs]) {
    DBG_ERROR(AE_LOGDOMAIN, "Id [%s] is invalid", id);
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}





