/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AE_MODULES_STATEMENTIMPORT_P_H
#define AE_MODULES_STATEMENTIMPORT_P_H


#include <aqfinance/engine/modules/ae_statementimport.h>

#include <aqbanking/transaction.h>

#include <gwenhywfar/list1.h>


typedef struct AE_STATEMENT_DAY AE_STATEMENT_DAY;
GWEN_LIST_FUNCTION_DEFS(AE_STATEMENT_DAY, AE_StatementDay)


struct AE_STATEMENT_DAY {
  GWEN_LIST_ELEMENT(AE_STATEMENT_DAY)

  int day;
  int month;
  int year;

  AE_STATEMENT_LIST *savedList;
  AE_STATEMENT_LIST *importedList;
};



static AE_STATEMENT_DAY *AE_StatementDay_new(int day, int month, int year);
static void AE_StatementDay_free(AE_STATEMENT_DAY *sd);

static void AE_StatementDay_AddSavedStatement(AE_STATEMENT_DAY *sd, AE_STATEMENT *st);
static void AE_StatementDay_AddImportedStatement(AE_STATEMENT_DAY *sd, AE_STATEMENT *st);

static int AE_StatementDay_CountSavedMatches(const AE_STATEMENT_DAY *sd, const AE_STATEMENT *mst);
static int AE_StatementDay_CountImportedMatches(const AE_STATEMENT_DAY *sd, const AE_STATEMENT *mst);


static AE_STATEMENT_DAY *AE_StatementImport_GetDay(AE_STATEMENT_DAY_LIST *dl,
						   const GWEN_DATE *dt);



static int AE_StatementImport_ImportStatement(AE_STATEMENT_DAY_LIST *dl,
					      AE_STATEMENT *st);

static int AE_StatementImport_ImportTransaction(AE_STATEMENT_DAY_LIST *dl,
                                                AB_TRANSACTION *t,
                                                AQDB_ID bankAccountId);


static void GWENHYWFAR_CB *AE_StatementImport_AddSavedStatement(AE_BOOK *bk,
								AE_BOOK_TABLE_TYPE tt,
								AQDB_OBJECT *o,
								void *data);
static int AE_StatementImport_FillSavedDay(AE_BOOK *b,
					   AE_STATEMENT_DAY *sd,
					   AQDB_ID refQueryId,
					   AE_BOOK_TABLE_TYPE tt);


static int AE_StatementImport_TransactionStatusToStatementStatus(AB_TRANSACTION_STATUS t);



struct AE_HANDLE_ACCOUNT_DATA {
  AE_BOOK *book;
  AE_ACCOUNT_TREE *accountTree;
  AE_ACCOUNT *account;
  AQDB_ID commodityId;
  int doLock;
};


#endif
