/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_trans4job.h"

#include <aqfinance/engine/book/ae_book.h>
#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_transaction.h>
#include <aqfinance/engine/book/ae_tax.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/gui.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>





int AE_Transaction4SellInvoice(AE_BOOK *b, AE_JOB *j,
			       AE_TRANSACTION *t,
			       AQDB_ID receivableAccountId) {
  AE_ITEM *item;
  AE_TAX_LIST *taxes;
  AE_SPLIT *sp;
  AQDB_VALUE *v;
  int rv;

  /* read taxes for later */
  taxes=AE_Tax_List_new();
  rv=AE_Book_ReadTaxList(b, AE_Book_TableType_Tax, taxes);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Tax_List_free(taxes);
    return rv;
  }

  item=AE_Item_List_First(AE_Job_GetItems(j));
  while(item) {
    const AQDB_VALUE *vt;
    AQDB_ID aid;
    AQDB_ID tid;

    /* Offene Forderungen an Warenverkauf, Steuern */
    /* Offene Forderungen */
    sp=AE_Split_new();
    AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
    AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
    v=AQDB_Value_new();
    vt=AE_Item_GetSellPriceValue(item);
    if (vt)
      AQDB_Value_AddValue(v, vt);
    vt=AE_Item_GetSellTaxSumValue(item);
    if (vt)
      AQDB_Value_AddValue(v, vt);
    AE_Split_SetAmount(sp, v);
    AE_Split_SetAccountId(sp, receivableAccountId);
    AE_Transaction_AddSplit(t, sp);
    AQDB_Value_free(v);

    /* Warenverkauf (-) */
    aid=AE_Item_GetSellAccountId(item);
    sp=AE_Split_new();
    AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
    AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
    vt=AE_Item_GetSellPriceValue(item);
    if (vt) {
      v=AQDB_Value_dup(vt);
      AQDB_Value_Negate(v);
      AE_Split_SetAmount(sp, v);
      AQDB_Value_free(v);
    }
    AE_Split_SetAccountId(sp, aid);
    AE_Transaction_AddSplit(t, sp);

    /* Steuern */
    tid=AE_Item_GetSellTaxId(item);
    if (tid) {
      AE_TAX *tax;

      sp=AE_Split_new();
      AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
      AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
      tax=AE_Tax_List_GetById(taxes, tid);
      if (tax) {
	AE_Split_SetAccountId(sp, AE_Tax_GetAccountId(tax));
      }
      else {
	DBG_ERROR(AE_LOGDOMAIN, "Tax not found");
	AE_Split_SetMemo(sp, I18N("Split for unknown tax"));
      }

      vt=AE_Item_GetSellTaxSumValue(item);
      if (vt) {
	v=AQDB_Value_dup(vt);
	AQDB_Value_Negate(v);
	AE_Split_SetAmount(sp, v);
	AQDB_Value_free(v);
      }
      AE_Transaction_AddSplit(t, sp);
    }

    /* Wareneinsatz an Wareneinkauf */
    if (AE_Item_GetBuyAccountId(item) &&
	AE_Item_GetCostOfSalesAccountId(item)) {

      /* Wareneinsatz (-) */
      aid=AE_Item_GetCostOfSalesAccountId(item);
      sp=AE_Split_new();
      AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
      AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
      vt=AE_Item_GetBuyPriceValue(item);
      if (vt)
	AE_Split_SetAmount(sp, vt);
      AE_Split_SetAccountId(sp, aid);
      AE_Transaction_AddSplit(t, sp);

      /* Wareneinkauf (+) */
      aid=AE_Item_GetBuyAccountId(item);
      sp=AE_Split_new();
      AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
      AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
      vt=AE_Item_GetBuyPriceValue(item);
      if (vt) {
	v=AQDB_Value_dup(vt);
        AQDB_Value_Negate(v);
	AE_Split_SetAmount(sp, v);
        AQDB_Value_free(v);
      }
      AE_Split_SetAccountId(sp, aid);
      AE_Transaction_AddSplit(t, sp);
    }

    item=AE_Item_List_Next(item);
  }

  AE_Tax_List_free(taxes);
  return 0;
}



int AE_Transaction4BuyInvoice(AE_BOOK *b, AE_JOB *j,
			      AE_TRANSACTION *t,
			      AQDB_ID payableAccountId) {
  AE_ITEM *item;
  AE_TAX_LIST *taxes;
  AE_SPLIT *sp;
  AQDB_VALUE *v;
  int rv;

  /* read taxes for later */
  taxes=AE_Tax_List_new();
  rv=AE_Book_ReadTaxList(b, AE_Book_TableType_Tax, taxes);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Tax_List_free(taxes);
    return rv;
  }

  item=AE_Item_List_First(AE_Job_GetItems(j));
  while(item) {
    const AQDB_VALUE *vt;
    AQDB_ID aid;
    AQDB_ID tid;

    /* Wareneinkauf, Steuern an offene Verbindlichkeiten */
    /* Offene Verbindlichkeiten (-) */
    sp=AE_Split_new();
    AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
    AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
    v=AQDB_Value_new();
    vt=AE_Item_GetBuyPriceValue(item);
    if (vt)
      AQDB_Value_AddValue(v, vt);
    vt=AE_Item_GetBuyTaxSumValue(item);
    if (vt)
      AQDB_Value_AddValue(v, vt);
    AQDB_Value_Negate(v);
    AE_Split_SetAmount(sp, v);
    AE_Split_SetAccountId(sp, payableAccountId);
    AE_Transaction_AddSplit(t, sp);
    AQDB_Value_free(v);

    /* Wareneinkauf (+) */
    aid=AE_Item_GetBuyAccountId(item);
    sp=AE_Split_new();
    AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
    AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
    vt=AE_Item_GetBuyPriceValue(item);
    if (vt)
      AE_Split_SetAmount(sp, vt);
    AE_Split_SetAccountId(sp, aid);
    AE_Transaction_AddSplit(t, sp);

    /* Steuern (+) */
    tid=AE_Item_GetBuyTaxId(item);
    if (tid) {
      AE_TAX *tax;

      sp=AE_Split_new();
      AE_Split_SetTransactionId(sp, AE_Transaction_GetId(t));
      AE_Split_SetCommodityId(sp, AE_Item_GetCommodityId(item));
      tax=AE_Tax_List_GetById(taxes, tid);
      if (tax) {
	AE_Split_SetAccountId(sp, AE_Tax_GetAccountId(tax));
      }
      else {
	DBG_ERROR(AE_LOGDOMAIN, "Tax not found");
	AE_Split_SetMemo(sp, I18N("Split for unknown tax"));
      }

      vt=AE_Item_GetBuyTaxSumValue(item);
      if (vt)
	AE_Split_SetAmount(sp, vt);
      AE_Transaction_AddSplit(t, sp);
    }

    item=AE_Item_List_Next(item);
  }

  AE_Tax_List_free(taxes);
  return 0;
}



int AE_Transaction4SellPayment(AE_BOOK *b, AE_JOB *j,
			       AE_TRANSACTION *t,
			       AQDB_ID receivableAccountId,
			       AQDB_ID bankAccountId) {
  int rv;
  AQDB_VALUE *v;
  AE_SPLIT *sp;

  /* calculate total sum, create a receivable split */
  v=AQDB_Value_new();
  rv=AE_Job_CalculateSellTotalSum(j, v);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AQDB_Value_free(v);
    return rv;
  }

  /* bank account */
  sp=AE_Split_new();
  AE_Split_SetAmount(sp, v);
  AE_Split_SetCommodityId(sp, AE_Job_GetCommodityId(j));
  AE_Split_SetAccountId(sp, bankAccountId);
  AE_Transaction_AddSplit(t, sp);

  /* receivable account */
  sp=AE_Split_new();
  AQDB_Value_Negate(v);
  AE_Split_SetAmount(sp, v);
  AE_Split_SetCommodityId(sp, AE_Job_GetCommodityId(j));
  AE_Split_SetAccountId(sp, receivableAccountId);
  AE_Transaction_AddSplit(t, sp);


  return 0;
}



int AE_Transaction4BuyPayment(AE_BOOK *b, AE_JOB *j,
			      AE_TRANSACTION *t,
			      AQDB_ID payableAccountId,
			      AQDB_ID bankAccountId) {
  int rv;
  AQDB_VALUE *v;
  AE_SPLIT *sp;

  /* calculate total sum, create a payable split */
  v=AQDB_Value_new();
  rv=AE_Job_CalculateBuyTotalSum(j, v);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AQDB_Value_free(v);
    return rv;
  }

  /* payable */
  sp=AE_Split_new();
  AE_Split_SetAmount(sp, v);
  AE_Split_SetCommodityId(sp, AE_Job_GetCommodityId(j));
  AE_Split_SetAccountId(sp, payableAccountId);
  AE_Transaction_AddSplit(t, sp);

  /* bank account */
  sp=AE_Split_new();
  AQDB_Value_Negate(v);
  AE_Split_SetAmount(sp, v);
  AE_Split_SetCommodityId(sp, AE_Job_GetCommodityId(j));
  AE_Split_SetAccountId(sp, bankAccountId);
  AE_Transaction_AddSplit(t, sp);

  return 0;
}









