/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "af_report_p.h"

#include <aqfinance/report/htmlbase.h>
#include <aqfinance/report/af_report_cfg_htmlbase.h>



static int _buildReportForSplits(AE_BOOK *b,
                                 int reverseSign,
                                 int sumParents,
                                 AE_SPLIT_LIST *origList,
                                 AE_ACCOUNT_GROUP grp,
                                 const char *title,
                                 GWEN_SYNCIO *sio) {
  AE_SPLIT_LIST *sl;
  AF_REPORT_DATA *rdata;
  AF_REPORT_CFG *cfg;
  AF_REPORT_CFG_COLUMN_LIST *cl;
  AE_SPLIT *sp;

  rdata=AF_Report_Data_new();
  //AF_Report_Data_SetTitle(rdata, title);
  AF_Report_Data_SetSubTitle(rdata, title);


  cfg=AF_Report_Cfg_new();
  if (reverseSign)
    AF_Report_Cfg_AddFlags(cfg, AF_REPORT_CFG_FLAG_REVERSE_SIGN);
  cl=AE_Split_CreateReportColumnList();
  AF_Report_Cfg_SetColumnList(cfg, cl);
  AE_Book_Report_CreateColumns(b, cfg, rdata);

  sl=AE_Split_List_new();
  sp=AE_Split_List_First(origList);
  while(sp) {
    if (AE_AccountType_GetGroup(AE_Split_GetTmpAccountType(sp))==grp) {
      AE_SPLIT *nsp;

      nsp=AE_Split_dup(sp);
      AE_Split_List_Add(nsp, sl);
    }

    sp=AE_Split_List_Next(sp);
  }

  if (sumParents)
    AE_Book_Report_SumSplitsIntoReportData(b, rdata, cfg, sl);
  else
    AE_Book_Report_AddSplitsToReportData(b, rdata, cfg, sl);
  AE_Split_List_free(sl);

  DBG_ERROR(0, "Creating flat category list...");
  AE_Book_Report_ToFlatCategoryList(b, rdata,
                                    AE_BOOK_REPORT_FLAG_ADD_POS_AMT |
                                    AE_BOOK_REPORT_FLAG_ADD_NEG_AMT |
                                    AE_BOOK_REPORT_FLAG_ADD_SUM_AMT |
                                    0);
  AF_Report_Data_SortValueByIdx(rdata, 0, 3);
  AF_Report_Data_AdjustAxisRange(rdata, 1);

  if (1) {
    GWEN_XMLNODE *n;
    GWEN_XMLNODE *nn;

    n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "root");
    nn=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "report");
    GWEN_XMLNode_AddChild(n, nn);
    AF_Report_Data_toXml(rdata, nn);
    GWEN_XMLNode_WriteFile(n, "/tmp/test-splits.xml", GWEN_XML_FLAGS_SIMPLE | GWEN_XML_FLAGS_INDENT);
    GWEN_XMLNode_free(n);
  }

  if (1) {
    int rv;
    AF_REPORT_CFG_HTMLBASE *cfg;

    DBG_ERROR(0, "Creating HTML report...");
    cfg=AF_Report_Cfg_HtmlBase_new();
    AF_Report_Cfg_HtmlBase_AddFlags(cfg,
                                    AF_REPORT_CFG_HTMLBASE_FLAG_VALUE_LOCAL |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_RED_NEGATIVES |
                                    AF_REPORT_CFG_HTMLBASE_FLAG_SHOW_TITLE |
                                    0);
    AF_Report_Cfg_HtmlBase_SetBorderWidth(cfg, 1);
    //AF_Report_Cfg_HtmlBase_SetColumnWidth1(cfg, 400);

    rv=AF_Report_Builder_HtmlBase_toSio(sio, rdata, cfg);
    if (rv<0) {
      DBG_ERROR(0, "Error building file (%d)", rv);
      return 4;
    }
    DBG_ERROR(0, "Done");
  }

  return 0;
}




int AF_Report_BuildHtmlSplitReport(AE_BOOK *b,
				   const char *mainTitle,
				   AE_SPLIT_LIST *sl,
				   uint32_t flags,
                                   GWEN_SYNCIO *sio) {
  int rv;
  int sumParents;

  /* header */
  GWEN_SyncIo_WriteString(sio,
			  "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" "
			  "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
			  "<html>\n"
			  "<head>\n"
			  "<meta name=\"generator\" content=\"AqFinance\" />\n"
			  "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf8\" />\n");
  GWEN_SyncIo_WriteString(sio, "<title>");
  GWEN_SyncIo_WriteString(sio, "AqFinance Report");
  GWEN_SyncIo_WriteString(sio, "</title>\n");
  GWEN_SyncIo_WriteString(sio, "</head>\n<body>\n");


  if (mainTitle && *mainTitle) {
    GWEN_SyncIo_WriteString(sio, "<h1>");
    GWEN_SyncIo_WriteString(sio, mainTitle);
    GWEN_SyncIo_WriteString(sio, "</h1>\n");
  }

  if (flags & AF_REPORT_HTMLSPLIT_FLAGS_SUM_PARENTS)
    sumParents=1;
  else
    sumParents=0;

  if (flags & AF_REPORT_HTMLSPLIT_FLAGS_WITH_ASSETS) {
    rv=_buildReportForSplits(b, 0, sumParents, sl, AE_AccountGroup_Asset,
			     I18N("REPORT_HEADER|Assets"), sio);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }


  if (flags & AF_REPORT_HTMLSPLIT_FLAGS_WITH_LIABILITIES) {
    rv=_buildReportForSplits(b, 1, sumParents, sl, AE_AccountGroup_Liability,
			     I18N("REPORT_HEADER|Liabilities"), sio);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }

  if (flags & AF_REPORT_HTMLSPLIT_FLAGS_WITH_INCOME) {
    rv=_buildReportForSplits(b, 1, sumParents, sl, AE_AccountGroup_Income,
			     I18N("REPORT_HEADER|Income"), sio);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }

  if (flags & AF_REPORT_HTMLSPLIT_FLAGS_WITH_EXPENSE) {
    rv=_buildReportForSplits(b, 0, sumParents, sl, AE_AccountGroup_Expense,
			     I18N("REPORT_HEADER|Expense"), sio);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }

  if (flags & AF_REPORT_HTMLSPLIT_FLAGS_WITH_EQUITY) {
    rv=_buildReportForSplits(b, 0, sumParents, sl, AE_AccountGroup_Equity,
			     I18N("REPORT_HEADER|Equity"), sio);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }


  /* footer */
  GWEN_SyncIo_WriteString(sio, "</body>\n</html>\n");

  /* flush SIO */
  rv=GWEN_SyncIo_Flush(sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}






