/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



static void _getStartAndEndDates(AF_REPORT_AXIS *ax, GWEN_DATE **pStart, GWEN_DATE **pEnd) {
  const char *s;
  GWEN_DATE *minDate=NULL;
  GWEN_DATE *maxDate=NULL;

  s=AF_Report_Axis_GetMinValue(ax);
  if (s && *s) {
    GWEN_DATE *v;

    v=GWEN_Date_fromString(s);
    if (v==NULL) {
      DBG_WARN(0, "Bad date value for minValue [%s]", s);
    }
    else
      minDate=v;
  }

  s=AF_Report_Axis_GetMaxValue(ax);
  if (s && *s) {
    GWEN_DATE *v;

    v=GWEN_Date_fromString(s);
    if (v==NULL) {
      DBG_WARN(0, "Bad date value for maxValue [%s]", s);
    }
    else
      maxDate=v;
  }

  if (minDate==NULL || maxDate==NULL) {
    if (minDate==NULL && maxDate!=NULL)
      minDate=GWEN_Date_dup(maxDate);
    else if (minDate!=NULL && maxDate==NULL)
      maxDate=GWEN_Date_dup(minDate);
    else {
      minDate=GWEN_Date_CurrentDate();
      maxDate=GWEN_Date_dup(minDate);
    }
  }

  if (pStart)
    *pStart=minDate;
  if (pEnd)
    *pEnd=maxDate;
}






void AF_Report_Axis_AddTicksForDays(AF_REPORT_AXIS *ax, int level, uint32_t flags, const char *dateTempl, int colorWD, int colorWE) {
  GWEN_DATE *minDate=NULL;
  GWEN_DATE *maxDate=NULL;
  int minJul;
  int maxJul;
  int i;
  AF_REPORT_MARK_LIST *ml;
  GWEN_BUFFER *mbuf;

  _getStartAndEndDates(ax, &minDate, &maxDate);
  minJul=GWEN_Date_GetJulian(minDate);
  maxJul=GWEN_Date_GetJulian(maxDate);
  GWEN_Date_free(minDate);
  GWEN_Date_free(maxDate);

  assert(ax->marks);
  ml=AF_Report_Marks_GetMarkList(ax->marks);
  assert(ml);
  mbuf=GWEN_Buffer_new(0, 256, 0, 1);

  /* add days */
  for (i=minJul; i<=maxJul; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      AF_REPORT_MARK *m;
      int rv;

      rv=GWEN_Date_toStringWithTemplate(d, dateTempl, mbuf);
      if (rv<0) {
	DBG_INFO(0, "here (%d)", rv);
      }
      else {
	int wd;

	wd=GWEN_Date_WeekDay(d);
	m=AF_Report_Mark_new();
	AF_Report_Mark_SetLevel(m, level);

	if (wd==0 || wd==6)
	  AF_Report_Mark_SetColor(m, colorWE);
	else
	  AF_Report_Mark_SetColor(m, colorWD);
	AF_Report_Mark_SetValue(m, GWEN_Date_GetString(d));
	AF_Report_Mark_SetLabel(m, GWEN_Buffer_GetStart(mbuf));
	AF_Report_Mark_SetFlags(m, flags);
	AF_Report_Mark_List_Add(m, ml);
      }
      GWEN_Buffer_Reset(mbuf);
      GWEN_Date_free(d);
    }
  }
  GWEN_Buffer_free(mbuf);
}



void AF_Report_Axis_AddTicksForWeeks(AF_REPORT_AXIS *ax, int level, uint32_t flags, int color, int weekStartsSunday) {
  GWEN_DATE *minDate=NULL;
  GWEN_DATE *maxDate=NULL;
  int minJul;
  int maxJul;
  int i;
  AF_REPORT_MARK_LIST *ml;

  _getStartAndEndDates(ax, &minDate, &maxDate);
  minJul=GWEN_Date_GetJulian(minDate);
  maxJul=GWEN_Date_GetJulian(maxDate);
  GWEN_Date_free(minDate);
  GWEN_Date_free(maxDate);

  assert(ax->marks);
  ml=AF_Report_Marks_GetMarkList(ax->marks);
  assert(ml);

  /* add days */
  for (i=minJul; i<=maxJul; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      int wd;

      wd=GWEN_Date_WeekDay(d);
      if ((weekStartsSunday && wd==0) ||
	  (!weekStartsSunday && wd==1)) {
	AF_REPORT_MARK *m;

	m=AF_Report_Mark_new();
	AF_Report_Mark_SetLevel(m, level);

	AF_Report_Mark_SetColor(m, color);
	AF_Report_Mark_SetValue(m, GWEN_Date_GetString(d));
	AF_Report_Mark_SetFlags(m, flags);
	AF_Report_Mark_List_Add(m, ml);
      }
      GWEN_Date_free(d);
    }
  }
}



void AF_Report_Axis_AddTicksForMonths(AF_REPORT_AXIS *ax, int level, uint32_t flags, int color, const char *dateTempl) {
  GWEN_DATE *minDate=NULL;
  GWEN_DATE *maxDate=NULL;
  int minJul;
  int maxJul;
  int i;
  AF_REPORT_MARK_LIST *ml;

  _getStartAndEndDates(ax, &minDate, &maxDate);
  minJul=GWEN_Date_GetJulian(minDate);
  maxJul=GWEN_Date_GetJulian(maxDate);
  GWEN_Date_free(minDate);
  GWEN_Date_free(maxDate);

  assert(ax->marks);
  ml=AF_Report_Marks_GetMarkList(ax->marks);
  assert(ml);

  /* add days */
  for (i=minJul; i<=maxJul; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      int wd;

      wd=GWEN_Date_GetDay(d);
      if (wd==1) {
	AF_REPORT_MARK *m;

	m=AF_Report_Mark_new();
	AF_Report_Mark_SetLevel(m, level);

	AF_Report_Mark_SetColor(m, color);
	AF_Report_Mark_SetValue(m, GWEN_Date_GetString(d));
	AF_Report_Mark_SetLabel(m, dateTempl);
	AF_Report_Mark_SetFlags(m, flags);
	AF_Report_Mark_List_Add(m, ml);
      }
      GWEN_Date_free(d);
    }
  }
}



void AF_Report_Axis_AddTicksForYears(AF_REPORT_AXIS *ax, int level, uint32_t flags, int color, const char *dateTempl) {
  GWEN_DATE *minDate=NULL;
  GWEN_DATE *maxDate=NULL;
  int minJul;
  int maxJul;
  int i;
  AF_REPORT_MARK_LIST *ml;
  GWEN_BUFFER *mbuf;

  _getStartAndEndDates(ax, &minDate, &maxDate);
  minJul=GWEN_Date_GetJulian(minDate);
  maxJul=GWEN_Date_GetJulian(maxDate);
  GWEN_Date_free(minDate);
  GWEN_Date_free(maxDate);

  assert(ax->marks);
  ml=AF_Report_Marks_GetMarkList(ax->marks);
  assert(ml);

  mbuf=GWEN_Buffer_new(0, 256, 0, 1);

  /* add days */
  for (i=minJul; i<=maxJul; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      if (GWEN_Date_DaysInYear(d)==0) {
	int rv;

	rv=GWEN_Date_toStringWithTemplate(d, dateTempl, mbuf);
	if (rv<0) {
	  DBG_INFO(0, "here (%d)", rv);
	}
	else {
	  AF_REPORT_MARK *m;

	  m=AF_Report_Mark_new();
	  AF_Report_Mark_SetLevel(m, level);

	  AF_Report_Mark_SetColor(m, color);
	  AF_Report_Mark_SetValue(m, GWEN_Date_GetString(d));
	  AF_Report_Mark_SetLabel(m, GWEN_Buffer_GetStart(mbuf));
	  AF_Report_Mark_SetFlags(m, flags);
	  AF_Report_Mark_List_Add(m, ml);
	}
      }
      GWEN_Date_free(d);
      GWEN_Buffer_Reset(mbuf);
    }
  }
  GWEN_Buffer_free(mbuf);
}



void AF_Report_Axis_AddTicksForInt(AF_REPORT_AXIS *ax, int level, uint32_t flags, int color, int step) {
  const char *s;
  AF_REPORT_MARK_LIST *ml;
  int minVal=0;
  int maxVal=0;
  int i;

  assert(ax->marks);
  ml=AF_Report_Marks_GetMarkList(ax->marks);
  assert(ml);

  s=AF_Report_Axis_GetMinValue(ax);
  if (s && *s) {
    if (1!=sscanf(s, "%i", &minVal)) {
      DBG_WARN(0, "Bad date value for minValue [%s]", s);
      minVal=0;
    }
  }

  s=AF_Report_Axis_GetMaxValue(ax);
  if (s && *s) {
    if (1!=sscanf(s, "%i", &maxVal)) {
      DBG_WARN(0, "Bad date value for maxValue [%s]", s);
      maxVal=minVal;
    }
  }

  /* add days */
  for (i=minVal; i<=maxVal; i+=step) {
    AF_REPORT_MARK *m;
    char buf[64];

    snprintf(buf, sizeof(buf)-1, "%d", i);
    buf[sizeof(buf)-1]=0;
    m=AF_Report_Mark_new();
    AF_Report_Mark_SetLevel(m, level);

    AF_Report_Mark_SetColor(m, color);
    AF_Report_Mark_SetValue(m, buf);
    AF_Report_Mark_SetLabel(m, buf);
    AF_Report_Mark_SetFlags(m, flags);
    AF_Report_Mark_List_Add(m, ml);
  }
}



void AF_Report_Axis_AddTicksForDouble(AF_REPORT_AXIS *ax, int level, uint32_t flags, int color, double step, int prec) {
  const char *s;
  AF_REPORT_MARK_LIST *ml;
  double minVal=0.0;
  double maxVal=0.0;
  double i;

  assert(ax->marks);
  ml=AF_Report_Marks_GetMarkList(ax->marks);
  assert(ml);

  s=AF_Report_Axis_GetMinValue(ax);
  if (s && *s) {
    if (1!=sscanf(s, "%lf", &minVal)) {
      DBG_WARN(0, "Bad date value for minValue [%s]", s);
      minVal=0;
    }
  }

  s=AF_Report_Axis_GetMaxValue(ax);
  if (s && *s) {
    if (1!=sscanf(s, "%lf", &maxVal)) {
      DBG_WARN(0, "Bad date value for maxValue [%s]", s);
      maxVal=minVal;
    }
  }

  /* add days */
  i=minVal;
  while(i<=maxVal) {
    AF_REPORT_MARK *m;
    char buf1[64];
    char buf2[64];

    snprintf(buf1, sizeof(buf1)-1, "%.*f", prec?prec:2, i);
    buf1[sizeof(buf1)-1]=0;

    snprintf(buf2, sizeof(buf2)-1, "%f", i);
    buf2[sizeof(buf2)-1]=0;

    m=AF_Report_Mark_new();
    AF_Report_Mark_SetLevel(m, level);

    AF_Report_Mark_SetColor(m, color);
    AF_Report_Mark_SetLabel(m, buf1);
    AF_Report_Mark_SetValue(m, buf2);
    AF_Report_Mark_SetFlags(m, flags);
    AF_Report_Mark_List_Add(m, ml);
    i+=step;
  }
}








