/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/







void AF_Report_Data_AddDataPairToCategories(AF_REPORT_DATA *rd,
                                            AQDB_ID cat,
                                            const char *catName,
                                            const GWEN_DATE *d,
                                            const AQDB_VALUE *val,
                                            AF_REPORT_DATAPAIR *dp) {
  AF_REPORT_CATEGORY *rc;
  AF_REPORT_YEAR *ry;
  AF_REPORT_MONTH *rm;
  AF_REPORT_DAY *rday;

  rc=AF_Report_Category_List_GetByCategory(rd->categoryList, cat);
  if (rc==NULL) {
    rc=AF_Report_Category_new();
    AF_Report_Category_SetCategory(rc, cat);
    AF_Report_Category_SetLabel(rc, catName);
    AF_Report_Category_List_Add(rc, rd->categoryList);
  }
  if (val) {
    if (AQDB_Value_IsNegative(val))
      AQDB_Value_AddValue(AF_Report_Category_GetNegAmount(rc), val);
    else
      AQDB_Value_AddValue(AF_Report_Category_GetPosAmount(rc), val);
  }

  ry=AF_Report_Year_List_GetByYear(AF_Report_Category_GetYearList(rc), GWEN_Date_GetYear(d));
  if (ry==NULL) {
    ry=AF_Report_Year_new();
    AF_Report_Year_SetYear(ry, GWEN_Date_GetYear(d));
     AF_Report_Year_List_Add(ry, AF_Report_Category_GetYearList(rc));
  }
  if (val) {
    if (AQDB_Value_IsNegative(val))
      AQDB_Value_AddValue(AF_Report_Year_GetNegAmount(ry), val);
    else
      AQDB_Value_AddValue(AF_Report_Year_GetPosAmount(ry), val);
  }

  rm=AF_Report_Month_List_GetByMonth(AF_Report_Year_GetMonthList(ry), GWEN_Date_GetMonth(d));
  if (rm==NULL) {
    rm=AF_Report_Month_new();
    AF_Report_Month_SetMonth(rm, GWEN_Date_GetMonth(d));
    AF_Report_Month_List_Add(rm, AF_Report_Year_GetMonthList(ry));
  }
  if (val) {
    if (AQDB_Value_IsNegative(val))
      AQDB_Value_AddValue(AF_Report_Month_GetNegAmount(rm), val);
    else
      AQDB_Value_AddValue(AF_Report_Month_GetPosAmount(rm), val);
  }

  rday=AF_Report_Day_List_GetByDay(AF_Report_Month_GetDayList(rm), GWEN_Date_GetDay(d));
  if (rday==NULL) {
    rday=AF_Report_Day_new();
    AF_Report_Day_SetDay(rday, GWEN_Date_GetDay(d));
    AF_Report_Day_List_Add(rday, AF_Report_Month_GetDayList(rm));
  }
  if (val) {
    if (AQDB_Value_IsNegative(val))
      AQDB_Value_AddValue(AF_Report_Day_GetNegAmount(rday), val);
    else
      AQDB_Value_AddValue(AF_Report_Day_GetPosAmount(rday), val);
  }

  AF_Report_DataPair_List_Add(dp, AF_Report_Day_GetDataPairList(rday));
}



void AF_Report_Data_AddDataPairToList(AF_REPORT_DATA *rd, AF_REPORT_DATAPAIR *dp) {
  AF_Report_DataPair_List_Add(dp, rd->dataPairList);
}



void AF_Report_Data_AdjustAxisRange(AF_REPORT_DATA *rd, int idx) {
  AF_REPORT_AXIS *ax;
  int i;

  assert(rd);
  assert(rd->dataPairList);

  i=idx;
  ax=AF_Report_Axis_List_First(rd->axisList);
  while(ax && i--)
    ax=AF_Report_Axis_List_Next(ax);
  assert(ax);

  switch(AF_Report_Axis_GetDataType(ax)) {
  case AQDB_DataType_Int: {
    int minVal, maxVal;
    int hasMinVal=0, hasMaxVal=0;
    AF_REPORT_DATAPAIR *dp;

    minVal=0;
    maxVal=0;

    dp=AF_Report_DataPair_List_First(rd->dataPairList);
    while(dp) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataPair_GetDataField(dp, idx);
      if (df) {
        int v;

        v=AF_Report_DataField_GetDataAsInt(df);
        if (!hasMinVal) {
          minVal=v;
          hasMinVal=1;
        }
        else if (v<minVal)
          minVal=v;

        if (!hasMaxVal) {
          maxVal=v;
          hasMaxVal=1;
        }
        else if (v>maxVal)
          maxVal=v;
      }
      else {
        DBG_ERROR(0, "No value");
      }
      dp=AF_Report_DataPair_List_Next(dp);
    }

    if (hasMinVal) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsInt(df, minVal);
      AF_Report_Axis_SetMinDataValue(ax, df);
    }
    if (hasMaxVal) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsInt(df, maxVal);
      AF_Report_Axis_SetMaxDataValue(ax, df);
    }
    break;
  }

  case AQDB_DataType_Amount: {
    AQDB_VALUE *minVal=NULL;
    AQDB_VALUE *maxVal=NULL;
    int hasMinVal=0, hasMaxVal=0;
    AF_REPORT_DATAPAIR *dp;

    dp=AF_Report_DataPair_List_First(rd->dataPairList);
    while(dp) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataPair_GetDataField(dp, idx);
      if (df) {
	const AQDB_VALUE *v;

	v=AF_Report_DataField_GetDataAsValue(df);
	if (v) {
	  if (!hasMinVal) {
	    minVal=AQDB_Value_dup(v);
	    hasMinVal=1;
	  }
	  else if (AQDB_Value_Compare(v, minVal)<0)
	    minVal=AQDB_Value_dup(v);
  
	  if (!hasMaxVal) {
	    maxVal=AQDB_Value_dup(v);
	    hasMaxVal=1;
	  }
	  else if (AQDB_Value_Compare(maxVal, v)<0)
	    maxVal=AQDB_Value_dup(v);
	}
	else {
	  DBG_ERROR(0, "Invalid value");
	}
      }
      else {
        DBG_ERROR(0, "No value");
      }
      dp=AF_Report_DataPair_List_Next(dp);
    }

    if (hasMinVal) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsValue(df, minVal);
      AF_Report_Axis_SetMinDataValue(ax, df);
      AQDB_Value_free(minVal);
    }
    if (hasMaxVal) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsValue(df, maxVal);
      AF_Report_Axis_SetMaxDataValue(ax, df);
      AQDB_Value_free(maxVal);
    }
    break;
  }

  case AQDB_DataType_Date: {
    GWEN_DATE *minVal=NULL;
    GWEN_DATE *maxVal=NULL;
    int hasMinVal=0, hasMaxVal=0;
    AF_REPORT_DATAPAIR *dp;

    dp=AF_Report_DataPair_List_First(rd->dataPairList);
    while(dp) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataPair_GetDataField(dp, idx);
      if (df) {
	const GWEN_DATE *v;

	v=AF_Report_DataField_GetDataAsDate(df);
	if (v) {
	  if (!hasMinVal) {
	    minVal=GWEN_Date_dup(v);
	    hasMinVal=1;
	  }
	  else if (GWEN_Date_Compare(v, minVal)<0)
	    minVal=GWEN_Date_dup(v);
  
	  if (!hasMaxVal) {
	    maxVal=GWEN_Date_dup(v);
	    hasMaxVal=1;
	  }
	  else if (GWEN_Date_Compare(maxVal, v)<0)
	    maxVal=GWEN_Date_dup(v);
	}
	else {
	  DBG_ERROR(0, "Invalid value");
	}
      }
      else {
        DBG_ERROR(0, "No value");
      }
      dp=AF_Report_DataPair_List_Next(dp);
    }

    if (hasMinVal) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsDate(df, minVal);
      AF_Report_Axis_SetMinDataValue(ax, df);
      GWEN_Date_free(minVal);
    }
    if (hasMaxVal) {
      AF_REPORT_DATAFIELD *df;

      df=AF_Report_DataField_new();
      AF_Report_DataField_SetDataAsDate(df, maxVal);
      AF_Report_Axis_SetMaxDataValue(ax, df);
      GWEN_Date_free(maxVal);
    }
    break;
  }

  }
}



static int GWENHYWFAR_CB _compareInt(const AF_REPORT_DATAPAIR *a,                        
                                     const AF_REPORT_DATAPAIR *b,
                                     int ascIdx) {
  int asc;
  int idx;
  int vA=0;
  int vB=0;
  AF_REPORT_DATAFIELD *dfA;
  AF_REPORT_DATAFIELD *dfB;

  idx=(ascIdx>>1);
  asc=ascIdx & 1;

  dfA=AF_Report_DataPair_GetDataField(a, idx);
  dfB=AF_Report_DataPair_GetDataField(b, idx);

  if (dfA)
    vA=AF_Report_DataField_GetDataAsInt(dfA);
  if (dfB)
    vB=AF_Report_DataField_GetDataAsInt(dfB);

  if (asc) {
    if (vA>vB)                                                             
      return 1;                                                            
    else if (vA<vB)                                                        
      return -1;                                                           
    else                                                                   
      return 0;                                                            
  }                                                                        
  else  {                                                                  
    if (vA>vB)                                                             
      return -1;                                                           
    else if (vA<vB)                                                        
      return 1;                                                            
    else                                                                   
      return 0;                                                            
  }                                                                        
}



static int GWENHYWFAR_CB _compareValue(const AF_REPORT_DATAPAIR *a,
                                       const AF_REPORT_DATAPAIR *b,
                                       int ascIdx) {
  int asc;
  int idx;
  const AQDB_VALUE *vA=NULL;
  const AQDB_VALUE *vB=NULL;
  AF_REPORT_DATAFIELD *dfA;
  AF_REPORT_DATAFIELD *dfB;

  idx=(ascIdx>>1);
  asc=ascIdx & 1;

  dfA=AF_Report_DataPair_GetDataField(a, idx);
  dfB=AF_Report_DataPair_GetDataField(b, idx);

  if (dfA)
    vA=AF_Report_DataField_GetDataAsValue(dfA);
  if (dfB)
    vB=AF_Report_DataField_GetDataAsValue(dfB);

  if (asc) {                                                               
    if (vA==NULL && vB!=NULL)                                              
      return 1;                                                            
    else if (vA!=NULL && vB==NULL)                                         
      return -1;                                                           
    else if (vA==NULL && vB==NULL)                                         
      return 0;                                                            
    else                                                                   
      return AQDB_Value_Compare(vA, vB);
  }                                                                        
  else  {                                                                  
    if (vA==NULL && vB!=NULL)                                              
      return -1;                                                           
    else if (vA!=NULL && vB==NULL)                                         
      return 1;                                                            
    else if (vA==NULL && vB==NULL)                                         
      return 0;                                                            
    else                                                                   
      return AQDB_Value_Compare(vB, vA);
  }
}



static int GWENHYWFAR_CB _compareDate(const AF_REPORT_DATAPAIR *a,
                                      const AF_REPORT_DATAPAIR *b,
                                      int ascIdx) {
  int asc;
  int idx;
  const GWEN_DATE *vA=NULL;
  const GWEN_DATE *vB=NULL;
  AF_REPORT_DATAFIELD *dfA;
  AF_REPORT_DATAFIELD *dfB;

  idx=(ascIdx>>1);
  asc=ascIdx & 1;

  dfA=AF_Report_DataPair_GetDataField(a, idx);
  dfB=AF_Report_DataPair_GetDataField(b, idx);

  if (dfA)
    vA=AF_Report_DataField_GetDataAsDate(dfA);
  if (dfB)
    vB=AF_Report_DataField_GetDataAsDate(dfB);

  if (asc) {                                                               
    if (vA==NULL && vB!=NULL)                                              
      return 1;                                                            
    else if (vA!=NULL && vB==NULL)                                         
      return -1;                                                           
    else if (vA==NULL && vB==NULL)                                         
      return 0;                                                            
    else                                                                   
      return GWEN_Date_Compare(vA, vB);
  }                                                                        
  else  {                                                                  
    if (vA==NULL && vB!=NULL)                                              
      return -1;                                                           
    else if (vA!=NULL && vB==NULL)                                         
      return 1;                                                            
    else if (vA==NULL && vB==NULL)                                         
      return 0;                                                            
    else                                                                   
      return GWEN_Date_Compare(vB, vA);
  }
}



static int GWENHYWFAR_CB _compareString(const AF_REPORT_DATAPAIR *a,
                                        const AF_REPORT_DATAPAIR *b,
                                        int ascIdx) {
  int asc;
  int idx;
  const char *vA=NULL;
  const char *vB=NULL;
  AF_REPORT_DATAFIELD *dfA;
  AF_REPORT_DATAFIELD *dfB;

  idx=(ascIdx>>1);
  asc=ascIdx & 1;

  dfA=AF_Report_DataPair_GetDataField(a, idx);
  dfB=AF_Report_DataPair_GetDataField(b, idx);

  if (dfA)
    vA=AF_Report_DataField_GetData(dfA);
  if (dfB)
    vB=AF_Report_DataField_GetData(dfB);

  if (asc) {                                                               
    if (vA==NULL && vB!=NULL)                                              
      return -1;                                                           
    else if (vA!=NULL && vB==NULL)
      return 1;                                                            
    else if (vA==NULL && vB==NULL)                                         
      return 0;                                                            
    else                                                                   
      return strcasecmp(vA, vB);                                           
  }                                                                        
  else  {                                                                  
    if (vA==NULL && vB!=NULL)                                              
      return 1;                                                            
    else if (vA!=NULL && vB==NULL)
      return -1;                                                           
    else if (vA==NULL && vB==NULL)                                         
      return 0;                                                            
    else                                                                   
      return strcasecmp(vB, vA);                                           
  }                                                                        
}



void AF_Report_Data_SortIntByIdx(AF_REPORT_DATA *rd, int asc, int idx) {
  AF_REPORT_DATAPAIR_LIST_SORT_FN oldFn;
  int ascIdx;

  ascIdx=(idx<<1) | (asc?1:0);
  oldFn=AF_Report_DataPair_List_SetSortFn(rd->dataPairList, _compareInt);
  AF_Report_DataPair_List_Sort(rd->dataPairList, ascIdx);
  AF_Report_DataPair_List_SetSortFn(rd->dataPairList, oldFn);
}



void AF_Report_Data_SortValueByIdx(AF_REPORT_DATA *rd, int asc, int idx) {
  AF_REPORT_DATAPAIR_LIST_SORT_FN oldFn;
  int ascIdx;

  ascIdx=(idx<<1) | (asc?1:0);
  oldFn=AF_Report_DataPair_List_SetSortFn(rd->dataPairList, _compareValue);
  AF_Report_DataPair_List_Sort(rd->dataPairList, ascIdx);
  AF_Report_DataPair_List_SetSortFn(rd->dataPairList, oldFn);
}



void AF_Report_Data_SortDateByIdx(AF_REPORT_DATA *rd, int asc, int idx) {
  AF_REPORT_DATAPAIR_LIST_SORT_FN oldFn;
  int ascIdx;

  ascIdx=(idx<<1) | (asc?1:0);
  oldFn=AF_Report_DataPair_List_SetSortFn(rd->dataPairList, _compareDate);
  AF_Report_DataPair_List_Sort(rd->dataPairList, ascIdx);
  AF_Report_DataPair_List_SetSortFn(rd->dataPairList, oldFn);
}



void AF_Report_Data_SortStringByIdx(AF_REPORT_DATA *rd, int asc, int idx) {
  AF_REPORT_DATAPAIR_LIST_SORT_FN oldFn;
  int ascIdx;

  ascIdx=(idx<<1) | (asc?1:0);
  oldFn=AF_Report_DataPair_List_SetSortFn(rd->dataPairList, _compareString);
  AF_Report_DataPair_List_Sort(rd->dataPairList, ascIdx);
  AF_Report_DataPair_List_SetSortFn(rd->dataPairList, oldFn);
}




void AF_Report_Data_AddTicksForString(AF_REPORT_DATA *rd,
				      int idx,
				      int level,
				      uint32_t flags,
				      int color) {
  GWEN_STRINGLIST *sl;                                                           
  AF_REPORT_DATAPAIR *dp;                                                        
  GWEN_STRINGLISTENTRY *se;                                                      
  AF_REPORT_AXIS *ax;
  AF_REPORT_MARKS *rm;
  AF_REPORT_MARK_LIST *ml;
  int i;

  /* get axis with given index */
  i=idx;
  ax=AF_Report_Axis_List_First(AF_Report_Data_GetAxisList(rd));
  while(ax && i--)
    AF_Report_Axis_List_Next(ax);
  assert(ax);

  rm=AF_Report_Axis_GetMarks(ax);
  assert(rm);
  ml=AF_Report_Marks_GetMarkList(rm);
  assert(ml);
										 
  sl=GWEN_StringList_new();                                                      
  dp=AF_Report_DataPair_List_First(rd->dataPairList);                            
  while(dp) {                                                                    
    AF_REPORT_DATAFIELD *df;

    df=AF_Report_DataPair_GetDataField(dp, idx);
    if (df) {
      const char *s;

      s=AF_Report_DataField_GetData(df);
      if (s && *s)
	GWEN_StringList_AppendString(sl, s, 0, 1);
      else
	GWEN_StringList_AppendString(sl, "", 0, 1);
    }
    dp=AF_Report_DataPair_List_Next(dp);
  }
										 
  se=GWEN_StringList_FirstEntry(sl);                                             
  while(se) {                                                                    
    const char *s;                                                               
										 
    s=GWEN_StringListEntry_Data(se);                                             
    if (s) {                                                                     
      AF_REPORT_MARK *m;                                                         
										 
      m=AF_Report_Mark_new();                                                    
      AF_Report_Mark_SetLevel(m, level);                                         
										 
      AF_Report_Mark_SetColor(m, color);                                         
      AF_Report_Mark_SetValue(m, s);                                             
      AF_Report_Mark_SetLabel(m, s);                                             
      AF_Report_Mark_SetFlags(m, flags);                                         
      AF_Report_Mark_List_Add(m, ml);                                            
    }                                                                            
    se=GWEN_StringListEntry_Next(se);                                            
  }                                                                              
  GWEN_StringList_free(sl);                                                      
}



