/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



static void AF_Report_DataField_ClearExtraData(AF_REPORT_DATAFIELD *df) {
  assert(df);

  if (df->dataAsValue)
    AQDB_Value_free(df->dataAsValue);
  df->dataAsValue=NULL;

  if (df->dataAsDate)
    GWEN_Date_free(df->dataAsDate);
  df->dataAsDate=NULL;

  df->dataType=AQDB_DataType_None;
}




void AF_Report_DataField_SetData(AF_REPORT_DATAFIELD *df, const char *v) {
  assert(df);

  if (df->data)
    free(df->data);
  if (v)
    df->data=strdup(v);
  else
    df->data=NULL;

  AF_Report_DataField_ClearExtraData(df);
}




void AF_Report_DataField_SetDataAsValue(AF_REPORT_DATAFIELD *df, const AQDB_VALUE *v) {
  assert(df);

  if (df->dataAsValue)
    AQDB_Value_free(df->dataAsValue);
  if (v) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AQDB_Value_toString(v, tbuf);
    if (df->data)
      free(df->data);
    df->data=strdup(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
    df->dataAsValue=AQDB_Value_dup(v);
  }
  else {
    df->dataAsValue=NULL;
    if (df->data)
      free(df->data);
    df->data=NULL;
  }
  df->dataType=AQDB_DataType_Amount;
}



const AQDB_VALUE *AF_Report_DataField_GetDataAsValue(AF_REPORT_DATAFIELD *df) {
  assert(df);
  if ((df->dataType==AQDB_DataType_Amount) &&
      (df->dataAsValue!=NULL))
    return df->dataAsValue;
  else {
    if (df->data!=NULL) {
      AQDB_VALUE *v;

      v=AQDB_Value_fromString(df->data);
      if (v) {
	df->dataAsValue=v;
	df->dataType=AQDB_DataType_Amount;
	return df->dataAsValue;
      }
    }
  }

  return NULL;
}



void AF_Report_DataField_SetDataAsInt(AF_REPORT_DATAFIELD *df, int d) {
  char tbuf[64];

  assert(df);

  snprintf(tbuf, sizeof(tbuf)-1, "%d", d);
  tbuf[sizeof(tbuf)-1]=0;
  if (df->data)
    free(df->data);
  df->data=strdup(tbuf);

  df->dataAsInt=d;
  df->dataType=AQDB_DataType_Int;
}



int AF_Report_DataField_GetDataAsInt(AF_REPORT_DATAFIELD *df) {
  assert(df);
  if (df->dataType==AQDB_DataType_Int)
    return df->dataAsInt;
  else {
    if (df->data!=NULL) {
      int i;

      if (1==sscanf(df->data, "%i", &i)) {
	df->dataAsInt=i;
	df->dataType=AQDB_DataType_Int;
	return df->dataAsInt;
      }
    }
  }

  return 0;
}



void AF_Report_DataField_SetDataAsDate(AF_REPORT_DATAFIELD *df, const GWEN_DATE *d) {
  assert(df);

  if (df->dataAsDate)
    GWEN_Date_free(df->dataAsDate);
  if (d) {
    if (df->data)
      free(df->data);
    df->data=strdup(GWEN_Date_GetString(d));
    df->dataAsDate=GWEN_Date_dup(d);
  }
  else {
    df->dataAsDate=NULL;
    if (df->data)
      free(df->data);
    df->data=NULL;
  }
  df->dataType=AQDB_DataType_Date;
}



const GWEN_DATE *AF_Report_DataField_GetDataAsDate(AF_REPORT_DATAFIELD *df) {
  assert(df);
  if ((df->dataType==AQDB_DataType_Date) &&
      (df->dataAsDate!=NULL))
    return df->dataAsDate;
  else {
    if (df->data!=NULL) {
      GWEN_DATE *v;

      v=GWEN_Date_fromString(df->data);
      if (v) {
	df->dataAsDate=v;
	df->dataType=AQDB_DataType_Date;
	return df->dataAsDate;
      }
    }
  }

  return NULL;
}





