/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/



#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "dbgroup_p.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>



GWEN_INHERIT_FUNCTIONS(AE_DBGROUP)





AE_DBGROUP *AE_DbGroup_new(const char *groupName,
			   AE_DBGROUP *parent,
			   GWEN_XML_CONTEXT *ctx) {
  AE_DBGROUP *g;

  GWEN_NEW_OBJECT(AE_DBGROUP, g);
  GWEN_INHERIT_INIT(AE_DBGROUP, g);
  g->parent=parent;
  g->xmlContext=ctx;
  if (groupName)
    g->groupName=strdup(groupName);
  if (g->xmlContext==NULL && g->parent)
    g->xmlContext=parent->xmlContext;

  return g;
}



void AE_DbGroup_free(AE_DBGROUP *g) {
  if (g) {
    GWEN_INHERIT_FINI(AE_DBGROUP, g);
    free(g->groupName);
    GWEN_FREE_OBJECT(g);
  }
}



AE_DBGROUP *AE_DbGroup_GetParent(const AE_DBGROUP *g) {
  assert(g);
  return g->parent;
}



GWEN_XML_CONTEXT *AE_DbGroup_GetXmlContext(const AE_DBGROUP *g) {
  assert(g);
  return g->xmlContext;
}



const char *AE_DbGroup_GetGroupName(const AE_DBGROUP *g) {
  assert(g);
  return g->groupName;
}








AE_DBGROUP_STARTTAG_FN
AE_DbGroup_SetStartTagFn(AE_DBGROUP *g,
			 AE_DBGROUP_STARTTAG_FN f) {
  AE_DBGROUP_STARTTAG_FN oldFn;

  assert(g);
  oldFn=g->startTagFn;
  g->startTagFn=f;
  return oldFn;
}



AE_DBGROUP_ENDTAG_FN
AE_DbGroup_SetEndTagFn(AE_DBGROUP *g,
		       AE_DBGROUP_ENDTAG_FN f) {
  AE_DBGROUP_ENDTAG_FN oldFn;

  assert(g);
  oldFn=g->endTagFn;
  g->endTagFn=f;
  return oldFn;
}



AE_DBGROUP_ADDDATA_FN
AE_DbGroup_SetAddDataFn(AE_DBGROUP *g,
			AE_DBGROUP_ADDDATA_FN f) {
  AE_DBGROUP_ADDDATA_FN oldFn;

  assert(g);
  oldFn=g->addDataFn;
  g->addDataFn=f;
  return oldFn;
}



AE_DBGROUP_ENDSUBGROUP_FN
AE_DbGroup_SetEndSubGroupFn(AE_DBGROUP *g,
			    AE_DBGROUP_ENDSUBGROUP_FN f) {
  AE_DBGROUP_ENDSUBGROUP_FN oldFn;

  assert(g);
  oldFn=g->endSubGroupFn;
  g->endSubGroupFn=f;
  return oldFn;
}





int AE_DbGroup_StartTag(AE_DBGROUP *g, const char *tagName) {
  assert(g);
  if (g->startTagFn)
    return g->startTagFn(g, tagName);
  else
    return GWEN_ERROR_NOT_IMPLEMENTED;
}



int AE_DbGroup_EndTag(AE_DBGROUP *g, const char *tagName) {
  assert(g);
  if (g->endTagFn)
    return g->endTagFn(g, tagName);
  else
    return GWEN_ERROR_NOT_IMPLEMENTED;
}



int AE_DbGroup_AddData(AE_DBGROUP *g, const char *data) {
  assert(g);
  if (g->addDataFn)
    return g->addDataFn(g, data);
  else
    return GWEN_ERROR_NOT_IMPLEMENTED;
}



int AE_DbGroup_EndSubGroup(AE_DBGROUP *g, AE_DBGROUP *sg) {
  assert(g);
  if (g->endSubGroupFn)
    return g->endSubGroupFn(g, sg);
  else
    return GWEN_ERROR_NOT_IMPLEMENTED;
}





