/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "g_aqfinance_p.h"
#include "dbxmlctx_l.h"

#include "g_table_l.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>




AE_DBGROUP *AE_DbGroup_AqFinance_new(const char *groupName,
				     AE_DBGROUP *parent,
				     GWEN_XML_CONTEXT *ctx) {
  AE_DBGROUP *g;

  /* create base group */
  g=AE_DbGroup_Generic_new(groupName, parent, ctx);
  assert(g);

  /* set virtual functions */
  AE_DbGroup_SetStartTagFn(g, AE_DbGroup_AqFinance_StartTag);

  return g;
}



int AE_DbGroup_AqFinance_StartTag(AE_DBGROUP *g, const char *tagName) {
  AE_DBGROUP *gNew=NULL;
  GWEN_XML_CONTEXT *ctx;
  GWEN_DB_NODE *dbAttribs;

  assert(g);

  ctx=AE_DbGroup_GetXmlContext(g);
  dbAttribs=AE_DbXmlCtx_GetCurrentAttributes(ctx);

  if (strcasecmp(tagName, "table")==0) {
    const char *s;

    s=GWEN_DB_GetCharValue(dbAttribs, "name", 0, NULL);
    if (s && *s)
      gNew=AE_DbGroup_Table_new(tagName, g, ctx, s);
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Missing table name");
      return GWEN_ERROR_BAD_DATA;
    }
  }
  else {
    DBG_WARN(AE_LOGDOMAIN,
	     "Ignoring group [%s]", tagName);
    return GWEN_ERROR_BAD_DATA;
  }

  if (gNew) {
    AE_DbXmlCtx_SetCurrentGroup(ctx, gNew);
    GWEN_XmlCtx_IncDepth(ctx);
  }

  return 0;
}




