/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "g_generic_p.h"
#include "dbxmlctx_l.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>




AE_DBGROUP *AE_DbGroup_Generic_new(const char *groupName,
					AE_DBGROUP *parent,
					GWEN_XML_CONTEXT *ctx) {
  AE_DBGROUP *g;

  /* create base group */
  g=AE_DbGroup_new(groupName, parent, ctx);
  assert(g);

  /* set virtual functions */
  AE_DbGroup_SetEndTagFn(g, AE_DbGroup_Generic_EndTag);
  AE_DbGroup_SetAddDataFn(g, AE_DbGroup_Generic_AddData);
  AE_DbGroup_SetEndSubGroupFn(g, AE_DbGroup_Generic_EndSubGroup);

  return g;
}



int AE_DbGroup_Generic_EndTag(AE_DBGROUP *g, const char *tagName) {
  assert(g);

  if (strcasecmp(AE_DbGroup_GetGroupName(g), tagName)!=0) {
    DBG_INFO(AE_LOGDOMAIN,
	     "Tag [%s] does not close [%s], ignoring",
	     tagName, AE_DbGroup_GetGroupName(g));
    return GWEN_ERROR_BAD_DATA;
  }

  /* always end this tag */
  return 1;
}



int AE_DbGroup_Generic_AddData(AE_DBGROUP *g, const char *data) {
  assert(g);

  /* just ignore the data */
  return 0;
}



int AE_DbGroup_Generic_EndSubGroup(AE_DBGROUP *g, AE_DBGROUP *sg){
  assert(g);

  /* just ignore the end of sub group */
  return 0;
}







