/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "g_object_p.h"
#include "g_table_l.h"
#include "g_column_l.h"
#include "dbxmlctx_l.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>


GWEN_INHERIT(AE_DBGROUP, AE_DBGROUP_OBJECT)




AE_DBGROUP *AE_DbGroup_Object_new(const char *groupName,
				 AE_DBGROUP *parent,
				 GWEN_XML_CONTEXT *ctx) {
  AE_DBGROUP *g;
  AE_DBGROUP_OBJECT *xg;

  assert(parent);
  g=AE_DbGroup_Generic_new(groupName, parent, ctx);
  assert(g);

  GWEN_NEW_OBJECT(AE_DBGROUP_OBJECT, xg);
  GWEN_INHERIT_SETDATA(AE_DBGROUP, AE_DBGROUP_OBJECT, g, xg, AE_DbGroup_Object_FreeData);

  /* set virtual functions */
  AE_DbGroup_SetStartTagFn(g, AE_DbGroup_Object_StartTag);

  xg->columns=AE_DbGroup_Table_GetColumns(parent);
  if (xg->columns==NULL) {
    DBG_INFO(AE_LOGDOMAIN, "here");
    AE_DbGroup_free(g);
    return NULL;
  }

  xg->object=AQDB_Object_new(AE_DbGroup_Table_GetTableId(parent), 0,
			     AQDB_Column_List_GetCount(xg->columns));

  return g;
}



void GWENHYWFAR_CB AE_DbGroup_Object_FreeData(void *bp, void *p) {
  AE_DBGROUP_OBJECT *xg;

  xg=(AE_DBGROUP_OBJECT*) p;

  if (xg->object)
    AQDB_Object_free(xg->object);

  GWEN_FREE_OBJECT(xg);
}



AQDB_OBJECT *AE_DbGroup_Object_GetObject(const AE_DBGROUP *g) {
  AE_DBGROUP_OBJECT *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_OBJECT, g);
  assert(xg);

  return xg->object;
}



AQDB_COLUMN_LIST *AE_DbGroup_Object_GetColumns(const AE_DBGROUP *g) {
  AE_DBGROUP_OBJECT *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_OBJECT, g);
  assert(xg);

  return xg->columns;
}



int AE_DbGroup_Object_StartTag(AE_DBGROUP *g, const char *tagName) {
  AE_DBGROUP_OBJECT *xg;
  AE_DBGROUP *gNew=NULL;
  GWEN_XML_CONTEXT *ctx;
  GWEN_DB_NODE *dbAttribs;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_OBJECT, g);
  assert(xg);

  ctx=AE_DbGroup_GetXmlContext(g);
  dbAttribs=AE_DbXmlCtx_GetCurrentAttributes(ctx);

  if (strcasecmp(tagName, "column")==0) {
    const char *s;

    s=GWEN_DB_GetCharValue(dbAttribs, "name", 0, NULL);
    if (s && *s) {
      AQDB_COLUMN *col;
      int idx=0;

      col=AQDB_Column_List_First(xg->columns);
      while(col) {
	const char *t;

	t=AQDB_Column_GetName(col);
	if (t && *t && strcasecmp(s, t)==0)
	  break;
        idx++;
	col=AQDB_Column_List_Next(col);
      }
      if (col==NULL) {
	DBG_ERROR(AE_LOGDOMAIN, "Column [%s] not found", s);
	return GWEN_ERROR_BAD_DATA;
      }
      gNew=AE_DbGroup_Column_new(tagName, g, ctx, col, idx, xg->object);
      assert(gNew);
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Missing table name");
      return GWEN_ERROR_BAD_DATA;
    }
  }
  else {
    DBG_WARN(AE_LOGDOMAIN,
	     "Unexpected group [%s]", tagName);
    return GWEN_ERROR_BAD_DATA;
  }

  if (gNew) {
    AE_DbXmlCtx_SetCurrentGroup(ctx, gNew);
    GWEN_XmlCtx_IncDepth(ctx);
  }

  return 0;
}



