/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "af_graphics_cairo_p.h"

#include <gwenhywfar/debug.h>



GWEN_INHERIT(AF_GRAPHICS, AF_GRAPHICS_CAIRO);



const double AF_GraphicsCairo_DashPattern1[2]={1.0, 1.0};
const double AF_GraphicsCairo_DashPattern2[2]={1.0, 2.0};
const double AF_GraphicsCairo_DashPattern3[2]={1.0, 5.0};
const double AF_GraphicsCairo_DashPattern4[2]={1.0, 10.0};
const double AF_GraphicsCairo_DashPattern5[2]={10.0, 2.0};






AF_GRAPHICS *AF_GraphicsCairo_Pdf_new(const char *fileName, int w, int h) {
  AF_GRAPHICS *g;
  AF_GRAPHICS_CAIRO *xg;

  g=AF_Graphics_new();
  GWEN_NEW_OBJECT(AF_GRAPHICS_CAIRO, xg);
  assert(xg);
  GWEN_INHERIT_SETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g, xg, AF_GraphicsCairo_FreeData);

  xg->fontList=AF_Graphics_Font_List_new();
  xg->penList=AF_Graphics_Pen_List_new();

  if (fileName)
    xg->fileName=strdup(fileName);
  xg->width=w;
  xg->height=h;

  xg->surface=cairo_pdf_surface_create(fileName, w, h);
  if (xg->surface==NULL) {
    DBG_ERROR(0, "Unable to create surface");
  }

  xg->cr=cairo_create(xg->surface);
  if (xg->cr==NULL) {
    DBG_ERROR(0, "Could not create cairo context");
  }

  /* set virtual functions */
  AF_Graphics_SetPenCreateFn(g, AF_GraphicsCairo_PenCreate);
  AF_Graphics_SetPenReleaseFn(g, AF_GraphicsCairo_PenRelease);
  AF_Graphics_SetDrawLineFn(g, AF_GraphicsCairo_DrawLine);
  AF_Graphics_SetDrawRectFn(g, AF_GraphicsCairo_DrawRect);
  AF_Graphics_SetDrawFilledRectFn(g, AF_GraphicsCairo_DrawFilledRect);
  AF_Graphics_SetFontCreateFn(g, AF_GraphicsCairo_FontCreate);
  AF_Graphics_SetFontReleaseFn(g, AF_GraphicsCairo_FontRelease);
  AF_Graphics_SetDrawTextFn(g, AF_GraphicsCairo_DrawText);
  AF_Graphics_SetGetTextWidthFn(g, AF_GraphicsCairo_GetTextWidth);
  AF_Graphics_SetGetTextHeightFn(g, AF_GraphicsCairo_GetTextHeight);
  AF_Graphics_SetFinishFn(g, AF_GraphicsCairo_Finish);

  return g;
}



AF_GRAPHICS *AF_GraphicsCairo_Ps_new(const char *fileName, int w, int h) {
  AF_GRAPHICS *g;
  AF_GRAPHICS_CAIRO *xg;

  g=AF_Graphics_new();
  GWEN_NEW_OBJECT(AF_GRAPHICS_CAIRO, xg);
  assert(xg);
  GWEN_INHERIT_SETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g, xg, AF_GraphicsCairo_FreeData);

  xg->fontList=AF_Graphics_Font_List_new();
  xg->penList=AF_Graphics_Pen_List_new();

  if (fileName)
    xg->fileName=strdup(fileName);
  xg->width=w;
  xg->height=h;

  xg->surface=cairo_ps_surface_create(fileName, w, h);
  if (xg->surface==NULL) {
    DBG_ERROR(0, "Unable to create surface");
  }

  xg->cr=cairo_create(xg->surface);
  if (xg->cr==NULL) {
    DBG_ERROR(0, "Could not create cairo context");
  }

  /* set virtual functions */
  AF_Graphics_SetPenCreateFn(g, AF_GraphicsCairo_PenCreate);
  AF_Graphics_SetPenReleaseFn(g, AF_GraphicsCairo_PenRelease);
  AF_Graphics_SetDrawLineFn(g, AF_GraphicsCairo_DrawLine);
  AF_Graphics_SetDrawRectFn(g, AF_GraphicsCairo_DrawRect);
  AF_Graphics_SetDrawFilledRectFn(g, AF_GraphicsCairo_DrawFilledRect);
  AF_Graphics_SetFontCreateFn(g, AF_GraphicsCairo_FontCreate);
  AF_Graphics_SetFontReleaseFn(g, AF_GraphicsCairo_FontRelease);
  AF_Graphics_SetDrawTextFn(g, AF_GraphicsCairo_DrawText);
  AF_Graphics_SetGetTextWidthFn(g, AF_GraphicsCairo_GetTextWidth);
  AF_Graphics_SetGetTextHeightFn(g, AF_GraphicsCairo_GetTextHeight);
  AF_Graphics_SetFinishFn(g, AF_GraphicsCairo_Finish);

  return g;
}



AF_GRAPHICS *AF_GraphicsCairo_Png_new(const char *fileName, int w, int h) {
  AF_GRAPHICS *g;
  AF_GRAPHICS_CAIRO *xg;

  g=AF_Graphics_new();
  GWEN_NEW_OBJECT(AF_GRAPHICS_CAIRO, xg);
  assert(xg);
  GWEN_INHERIT_SETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g, xg, AF_GraphicsCairo_FreeData);

  xg->fontList=AF_Graphics_Font_List_new();
  xg->penList=AF_Graphics_Pen_List_new();

  if (fileName)
    xg->fileName=strdup(fileName);
  xg->width=w;
  xg->height=h;
  xg->stride=cairo_format_stride_for_width(CAIRO_FORMAT_RGB24, w);
  xg->writePng=1;

  xg->surface=cairo_image_surface_create(CAIRO_FORMAT_RGB24, w, h);
  if (xg->surface==NULL) {
    DBG_ERROR(0, "Unable to create surface");
  }

  xg->cr=cairo_create(xg->surface);
  if (xg->cr==NULL) {
    DBG_ERROR(0, "Could not create cairo context");
  }

  /* set virtual functions */
  AF_Graphics_SetPenCreateFn(g, AF_GraphicsCairo_PenCreate);
  AF_Graphics_SetPenReleaseFn(g, AF_GraphicsCairo_PenRelease);
  AF_Graphics_SetDrawLineFn(g, AF_GraphicsCairo_DrawLine);
  AF_Graphics_SetDrawRectFn(g, AF_GraphicsCairo_DrawRect);
  AF_Graphics_SetDrawFilledRectFn(g, AF_GraphicsCairo_DrawFilledRect);
  AF_Graphics_SetFontCreateFn(g, AF_GraphicsCairo_FontCreate);
  AF_Graphics_SetFontReleaseFn(g, AF_GraphicsCairo_FontRelease);
  AF_Graphics_SetDrawTextFn(g, AF_GraphicsCairo_DrawText);
  AF_Graphics_SetGetTextWidthFn(g, AF_GraphicsCairo_GetTextWidth);
  AF_Graphics_SetGetTextHeightFn(g, AF_GraphicsCairo_GetTextHeight);
  AF_Graphics_SetFinishFn(g, AF_GraphicsCairo_Finish);

  return g;
}




GWENHYWFAR_CB void AF_GraphicsCairo_FreeData(void *bp, void *p) {
  AF_GRAPHICS_CAIRO *xg;

  xg=(AF_GRAPHICS_CAIRO*) p;

  if (xg->cr) {
    cairo_destroy(xg->cr);
    xg->cr=NULL;
  }

  if (xg->surface) {
    cairo_surface_destroy(xg->surface);
    xg->surface=NULL;
  }

  free(xg->fileName);

  AF_Graphics_Font_List_free(xg->fontList);
  xg->fontList=NULL;

  AF_Graphics_Pen_List_free(xg->penList);
  xg->penList=NULL;

  GWEN_FREE_OBJECT(xg);
}





int AF_GraphicsCairo_PenCreate(AF_GRAPHICS *g, uint32_t frontColour, int lineWidth, int dashType) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_PEN *pen;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  pen=AF_Graphics_Pen_new();
  AF_Graphics_Pen_SetId(pen, ++(xg->lastPenId));
  AF_Graphics_Pen_SetFrontColour(pen, frontColour);
  AF_Graphics_Pen_SetLineWidth(pen, lineWidth);
  AF_Graphics_Pen_SetDashType(pen, dashType);

  AF_Graphics_Pen_List_Add(pen, xg->penList);
  return xg->lastPenId;
}



int AF_GraphicsCairo_PenRelease(AF_GRAPHICS *g, int penId) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_PEN *pen;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  pen=AF_Graphics_Pen_List_GetById(xg->penList, penId);
  if (pen==NULL) {
    DBG_INFO(0, "Pen %d not found", penId);
    return GWEN_ERROR_NOT_FOUND;
  }

  AF_Graphics_Pen_List_Del(pen);
  AF_Graphics_Pen_free(pen);
  return 0;
}



int AF_GraphicsCairo_DrawLine(AF_GRAPHICS *g, int penId, int xStart, int yStart, int xEnd, int yEnd) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_PEN *pen;
  uint32_t fgCol;
  double red, green, blue;
  int i;
  int dashType;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_ERROR(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  pen=AF_Graphics_Pen_List_GetById(xg->penList, penId);
  if (pen==NULL) {
    DBG_ERROR(0, "Pen %d not found", penId);
    return GWEN_ERROR_NOT_FOUND;
  }

  /*  set foreground color */;
  fgCol=AF_Graphics_Pen_GetFrontColour(pen);
  red=(double)((fgCol>>24) & 0xff)/255.0;
  green=(double)((fgCol>>16) & 0xff)/255.0;
  blue=(double)((fgCol>>8) & 0xff)/255.0;
  cairo_set_source_rgb(xg->cr, red, green, blue);

  /* set line width */
  i=AF_Graphics_Pen_GetLineWidth(pen);
  cairo_set_line_width(xg->cr, i);

  dashType=AF_Graphics_Pen_GetDashType(pen);

  switch(dashType) {
    default:
    case 0:
      cairo_set_dash(xg->cr, NULL, 0, 0.0);
      break;
    case 1:
      cairo_set_dash(xg->cr, AF_GraphicsCairo_DashPattern1, 2, 0.0);
      break;
    case 2:
      cairo_set_dash(xg->cr, AF_GraphicsCairo_DashPattern2, 2, 0.0);
      break;
    case 3:
      cairo_set_dash(xg->cr, AF_GraphicsCairo_DashPattern3, 2, 0.0);
      break;
    case 4:
      cairo_set_dash(xg->cr, AF_GraphicsCairo_DashPattern4, 2, 0.0);
      break;
    case 5:
      cairo_set_dash(xg->cr, AF_GraphicsCairo_DashPattern5, 2, 0.0);
      break;
  }

  /* draw line from start to end position */
  cairo_move_to(xg->cr, xStart, yStart);
  cairo_line_to(xg->cr, xEnd, yEnd);
  cairo_stroke(xg->cr);

  return 0;
}



int AF_GraphicsCairo_DrawRect(AF_GRAPHICS *g, int penId, int x, int y, int w, int h) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_PEN *pen;
  uint32_t fgCol;
  double red, green, blue;
  int i;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_INFO(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  pen=AF_Graphics_Pen_List_GetById(xg->penList, penId);
  if (pen==NULL) {
    DBG_INFO(0, "Pen %d not found", penId);
    return GWEN_ERROR_NOT_FOUND;
  }

  /*  set foreground color */;
  fgCol=AF_Graphics_Pen_GetFrontColour(pen);
  red=(double)((fgCol>>24) & 0xff)/255.0;
  green=(double)((fgCol>>16) & 0xff)/255.0;
  blue=(double)((fgCol>>8) & 0xff)/255.0;
  cairo_set_source_rgb(xg->cr, red, green, blue);

  /* set line width */
  i=AF_Graphics_Pen_GetLineWidth(pen);
  cairo_set_line_width(xg->cr, i);

  cairo_set_dash(xg->cr, NULL, 0, 0.0);

  /* draw line from start to end position */
  cairo_rectangle(xg->cr, x, y, w, h);
  cairo_stroke(xg->cr);

  return 0;
}



int AF_GraphicsCairo_DrawFilledRect(AF_GRAPHICS *g, int penId, int x, int y, int w, int h) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_PEN *pen;
  uint32_t fgCol;
  double red, green, blue;
  int i;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_INFO(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  pen=AF_Graphics_Pen_List_GetById(xg->penList, penId);
  if (pen==NULL) {
    DBG_INFO(0, "Pen %d not found", penId);
    return GWEN_ERROR_NOT_FOUND;
  }

  /*  set foreground color */;
  fgCol=AF_Graphics_Pen_GetFrontColour(pen);
  red=(double)((fgCol>>24) & 0xff)/255.0;
  green=(double)((fgCol>>16) & 0xff)/255.0;
  blue=(double)((fgCol>>8) & 0xff)/255.0;
  cairo_set_source_rgb(xg->cr, red, green, blue);

  /* set line width */
  i=AF_Graphics_Pen_GetLineWidth(pen);
  cairo_set_line_width(xg->cr, i);

  cairo_set_dash(xg->cr, NULL, 0, 0.0);

  /* draw line from start to end position */
  cairo_rectangle(xg->cr, x, y, w, h);
  cairo_stroke_preserve(xg->cr);
  cairo_fill(xg->cr);

  return 0;
}



int AF_GraphicsCairo_FontCreate(AF_GRAPHICS *g, const char* face, int fontSize, int fontSlant, int fontWeight) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_FONT *font;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  font=AF_Graphics_Font_new();
  AF_Graphics_Font_SetId(font, ++(xg->lastFontId));
  AF_Graphics_Font_SetFontName(font, face);
  AF_Graphics_Font_SetFontSize(font, fontSize);
  AF_Graphics_Font_SetFontSlant(font, fontSlant);
  AF_Graphics_Font_SetFontWeight(font, fontWeight);

  AF_Graphics_Font_List_Add(font, xg->fontList);
  return xg->lastFontId;

}



int AF_GraphicsCairo_FontRelease(AF_GRAPHICS *g, int fontId) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_FONT *font;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  font=AF_Graphics_Font_List_GetById(xg->fontList, fontId);
  if (font==NULL) {
    DBG_INFO(0, "Font %d not found", fontId);
    return GWEN_ERROR_NOT_FOUND;
  }

  AF_Graphics_Font_List_Del(font);
  AF_Graphics_Font_free(font);
  return 0;
}



int AF_GraphicsCairo_DrawText(AF_GRAPHICS *g, int penId, int fontId, int direction, int x, int y, const char* text) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_FONT *font;
  int crSlant;
  int crWeight;
  AF_GRAPHICS_PEN *pen;
  uint32_t fgCol;
  double red, green, blue;
  int i;
  cairo_font_extents_t extents;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_INFO(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  /* get and handle pen */
  pen=AF_Graphics_Pen_List_GetById(xg->penList, penId);
  if (pen==NULL) {
    DBG_INFO(0, "Pen %d not found", penId);
    return GWEN_ERROR_NOT_FOUND;
  }

  /*  set foreground color */;
  fgCol=AF_Graphics_Pen_GetFrontColour(pen);
  red=(double)((fgCol>>24) & 0xff)/255.0;
  green=(double)((fgCol>>16) & 0xff)/255.0;
  blue=(double)((fgCol>>8) & 0xff)/255.0;
  cairo_set_source_rgb(xg->cr, red, green, blue);

  /* set line width */
  i=AF_Graphics_Pen_GetLineWidth(pen);
  cairo_set_line_width(xg->cr, i);

  /* get and handle font */
  font=AF_Graphics_Font_List_GetById(xg->fontList, fontId);
  if (font==NULL) {
    DBG_INFO(0, "Font %d not found", fontId);
    return GWEN_ERROR_NOT_FOUND;
  }

  switch(AF_Graphics_Font_GetFontSlant(font)) {
    default:
    case AF_Graphics_Slant_None:   crSlant=CAIRO_FONT_SLANT_NORMAL; break;
    case AF_Graphics_Slant_Italic: crSlant=CAIRO_FONT_SLANT_ITALIC; break;
  }

  switch(AF_Graphics_Font_GetFontWeight(font)) {
    default:
    case AF_Graphics_Weight_None:  crWeight=CAIRO_FONT_WEIGHT_NORMAL; break;
    case AF_Graphics_Weight_Bold:  crWeight=CAIRO_FONT_WEIGHT_BOLD;   break;
  }
  cairo_select_font_face(xg->cr, AF_Graphics_Font_GetFontName(font), crSlant, crWeight);
  cairo_set_font_size(xg->cr, AF_Graphics_Font_GetFontSize(font));

  cairo_font_extents(xg->cr, &extents);

  /* TODO: translate pos and rotate according to given direction (use cairo_save and cairo_restore) */
  cairo_move_to(xg->cr, x, y+extents.ascent);
  cairo_show_text(xg->cr, text);

  return 0;
}



int AF_GraphicsCairo_GetTextWidth(AF_GRAPHICS *g, int fontId, const char* text) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_FONT *font;
  int crSlant;
  int crWeight;
  cairo_text_extents_t extents;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_INFO(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  /* get and handle font */
  font=AF_Graphics_Font_List_GetById(xg->fontList, fontId);
  if (font==NULL) {
    DBG_INFO(0, "Font %d not found", fontId);
    return GWEN_ERROR_NOT_FOUND;
  }

  switch(AF_Graphics_Font_GetFontSlant(font)) {
    default:
    case AF_Graphics_Slant_None:   crSlant=CAIRO_FONT_SLANT_NORMAL; break;
    case AF_Graphics_Slant_Italic: crSlant=CAIRO_FONT_SLANT_ITALIC; break;
  }

  switch(AF_Graphics_Font_GetFontWeight(font)) {
    default:
    case AF_Graphics_Weight_None:  crWeight=CAIRO_FONT_WEIGHT_NORMAL; break;
    case AF_Graphics_Weight_Bold:  crWeight=CAIRO_FONT_WEIGHT_BOLD;   break;
  }
  cairo_select_font_face(xg->cr, AF_Graphics_Font_GetFontName(font), crSlant, crWeight);
  cairo_set_font_size(xg->cr, AF_Graphics_Font_GetFontSize(font));

  cairo_text_extents(xg->cr, text, &extents);

  return (int)(extents.width);
}



int AF_GraphicsCairo_GetTextHeight(AF_GRAPHICS *g, int fontId, const char* text) {
  AF_GRAPHICS_CAIRO *xg;
  AF_GRAPHICS_FONT *font;
  int crSlant;
  int crWeight;
  cairo_text_extents_t extents;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_INFO(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  /* get and handle font */
  font=AF_Graphics_Font_List_GetById(xg->fontList, fontId);
  if (font==NULL) {
    DBG_INFO(0, "Font %d not found", fontId);
    return GWEN_ERROR_NOT_FOUND;
  }

  switch(AF_Graphics_Font_GetFontSlant(font)) {
    default:
    case AF_Graphics_Slant_None:   crSlant=CAIRO_FONT_SLANT_NORMAL; break;
    case AF_Graphics_Slant_Italic: crSlant=CAIRO_FONT_SLANT_ITALIC; break;
  }

  switch(AF_Graphics_Font_GetFontWeight(font)) {
    default:
    case AF_Graphics_Weight_None:  crWeight=CAIRO_FONT_WEIGHT_NORMAL; break;
    case AF_Graphics_Weight_Bold:  crWeight=CAIRO_FONT_WEIGHT_BOLD;   break;
  }
  cairo_select_font_face(xg->cr, AF_Graphics_Font_GetFontName(font), crSlant, crWeight);
  cairo_set_font_size(xg->cr, AF_Graphics_Font_GetFontSize(font));

  cairo_text_extents(xg->cr, text, &extents);

  return (int)(extents.height);
}



int AF_GraphicsCairo_Finish(AF_GRAPHICS *g) {
  AF_GRAPHICS_CAIRO *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AF_GRAPHICS, AF_GRAPHICS_CAIRO, g);
  assert(xg);

  if (xg->surface==NULL) {
    DBG_ERROR(0, "No surface created");
    return GWEN_ERROR_INVALID;
  }

  if (xg->cr==NULL) {
    DBG_INFO(0, "Graphics context not active, call \"AF_Graphics_BeginDraw()\" before");
    return GWEN_ERROR_INVALID;
  }

  if (xg->writePng) {
    cairo_status_t st;

    cairo_surface_flush(xg->surface);
    st=cairo_surface_write_to_png(xg->surface, xg->fileName);
    if (st!=CAIRO_STATUS_SUCCESS) {
      DBG_ERROR(0, "Cairo Status: %d (%s)", st, cairo_status_to_string(st));
    }
  }

#if 0
  if (1) {
    cairo_status_t st;

    fprintf(stderr, "Writing to PNG\n");
    cairo_surface_flush(xg->surface);
    st=cairo_surface_write_to_png(xg->surface, "/tmp/test.png");
    if (st!=CAIRO_STATUS_SUCCESS) {
      fprintf(stderr, "Cairo Status: %d (%s)\n", st, cairo_status_to_string(st));
    }
  }
#endif

  cairo_surface_finish(xg->surface);

  return 0;
}






