/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AF_GRAPHICS_CAIRO_P_H
#define AF_GRAPHICS_CAIRO_P_H


#include <aqfinance/graphics/af_graphics_cairo.h>

#include <aqfinance/graphics/af_graphics.h>
#include <aqfinance/graphics/af_graphics_font.h>
#include <aqfinance/graphics/af_graphics_pen.h>

#include <cairo/cairo.h>
#include <cairo/cairo-pdf.h>
#include <cairo/cairo-ps.h>




typedef struct AF_GRAPHICS_CAIRO AF_GRAPHICS_CAIRO;
struct AF_GRAPHICS_CAIRO {
  char *fileName;
  int width;
  int height;

  cairo_surface_t *surface;
  cairo_t *cr;

  AF_GRAPHICS_FONT_LIST *fontList;
  AF_GRAPHICS_PEN_LIST *penList;

  int lastFontId;
  int lastPenId;

  int stride; /* only for memory buffer surface */
  int writePng;
};


GWENHYWFAR_CB void AF_GraphicsCairo_FreeData(void *bp, void *p);




int AF_GraphicsCairo_PenCreate(AF_GRAPHICS *g, uint32_t frontColour, int lineWidth, int dashType);

int AF_GraphicsCairo_PenRelease(AF_GRAPHICS *g, int penId);

int AF_GraphicsCairo_DrawLine(AF_GRAPHICS *g, int penId, int xStart, int yStart, int xEnd, int yEnd);

int AF_GraphicsCairo_DrawRect(AF_GRAPHICS *g, int penId, int x, int y, int w, int h);

int AF_GraphicsCairo_DrawFilledRect(AF_GRAPHICS *g, int penId, int x, int y, int w, int h);

int AF_GraphicsCairo_FontCreate(AF_GRAPHICS *g, const char* face, int fontSize, int fontSlant, int fontWeight);

int AF_GraphicsCairo_FontRelease(AF_GRAPHICS *g, int fontId);

int AF_GraphicsCairo_DrawText(AF_GRAPHICS *g, int penId, int fontId, int direction, int x, int y, const char* text);

int AF_GraphicsCairo_GetTextWidth(AF_GRAPHICS *g, int fontId, const char* text);
int AF_GraphicsCairo_GetTextHeight(AF_GRAPHICS *g, int fontId, const char* text);

int AF_GraphicsCairo_Finish(AF_GRAPHICS *g);


#endif


