/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/




void AF_Graph_DataSet_AdjustXRange(AF_GRAPH_DATASET *ds) {
  assert(ds);
  if (ds->dataPairList) {
    AF_GRAPH_DATAPAIR *dp;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    if (dp) {
      double minVal;
      double maxVal;

      minVal=AF_Graph_DataPair_GetValueX(dp);
      maxVal=minVal;

      dp=AF_Graph_DataPair_List_Next(dp);
      while(dp) {
        double v;

        v=AF_Graph_DataPair_GetValueX(dp);
        if (v<minVal)
          minVal=v;
        if (v>maxVal)
          maxVal=v;

        dp=AF_Graph_DataPair_List_Next(dp);
      }

      ds->minValueX=minVal;
      ds->maxValueX=maxVal;
    }
  }
}



void AF_Graph_DataSet_AdjustYRange(AF_GRAPH_DATASET *ds) {
  assert(ds);
  if (ds->dataPairList) {
    AF_GRAPH_DATAPAIR *dp;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    if (dp) {
      double minVal;
      double maxVal;

      minVal=AF_Graph_DataPair_GetValueY(dp);
      maxVal=minVal;

      dp=AF_Graph_DataPair_List_Next(dp);
      while(dp) {
        double v;

        v=AF_Graph_DataPair_GetValueY(dp);
        if (v<minVal)
          minVal=v;
        if (v>maxVal)
          maxVal=v;

        dp=AF_Graph_DataPair_List_Next(dp);
      }

      ds->minValueY=minVal;
      ds->maxValueY=maxVal;
    }
  }
}




void AF_Graph_DataSet_AddTicks(AF_GRAPH_DATASET *ds) {
  if (ds->xTickList)
    AF_Graph_Tick_AddTicks(ds->xTickList, ds->dataTypeX, ds->minValueX, ds->maxValueX, ds->precisionX);
  if (ds->yTickList)
    AF_Graph_Tick_AddTicks(ds->yTickList, ds->dataTypeY, ds->minValueY, ds->maxValueY, ds->precisionY);
}



void AF_Graph_DataSet_AddSubGraph(AF_GRAPH_DATASET *ds, const char *label,
                                  int level, int graphType, int dataMode, int colour, int lineWidth, int dashType) {
  AF_GRAPH_SUBGRAPH *sg;

  if (ds->subGraphList==NULL)
    ds->subGraphList=AF_Graph_SubGraph_List_new();

  sg=AF_Graph_SubGraph_new();
  AF_Graph_SubGraph_SetLabel(sg, label);
  AF_Graph_SubGraph_SetLevel(sg, level);
  AF_Graph_SubGraph_SetGraphType(sg, graphType);
  AF_Graph_SubGraph_SetColour(sg, colour);
  AF_Graph_SubGraph_SetLineWidth(sg, lineWidth);
  AF_Graph_SubGraph_SetDashType(sg, dashType);

  AF_Graph_SubGraph_List_Add(sg, ds->subGraphList);
}



AF_GRAPH_SUBGRAPH *AF_Graph_DataSet_GetSubGraphByLevel(AF_GRAPH_DATASET *ds, int level) {
  if (ds->subGraphList)
    return AF_Graph_SubGraph_List_GetByLevel(ds->subGraphList, level);

  return NULL;
}



void AF_Graph_DataSet_AddDataPair(AF_GRAPH_DATASET *ds, AF_GRAPH_DATAPAIR *dp) {
  assert(ds);
  if (ds->dataPairList==NULL)
    ds->dataPairList=AF_Graph_DataPair_List_new();
  AF_Graph_DataPair_List_Add(dp, ds->dataPairList);
}



void AF_Graph_DataSet_SortDataPairs(AF_GRAPH_DATASET *ds, int ascending) {
  assert(ds);
  if (ds->dataPairList)
    AF_Graph_DataPair_List_SortByValueX(ds->dataPairList, ascending);
}



void AF_Graph_DataSet_AccumulateDataPairs(AF_GRAPH_DATASET *ds, int level) {
  assert(ds);
  if (ds->dataPairList) {
    AF_GRAPH_DATAPAIR *dp;
    double totalSum=0.0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double v;

        v=AF_Graph_DataPair_GetValueY(dp);
        totalSum+=v;
        AF_Graph_DataPair_SetValueY(dp, totalSum);
      }
      dp=AF_Graph_DataPair_List_Next(dp);
    }
  }
}



void AF_Graph_DataSet_AverageDataPairs(AF_GRAPH_DATASET *ds, int level) {
  assert(ds);
  if (ds->dataPairList) {
    AF_GRAPH_DATAPAIR *dp;
    double totalSum=0.0;
    int cnt=0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double v;

        v=AF_Graph_DataPair_GetValueY(dp);
        totalSum+=v;
        cnt++;
      }

      dp=AF_Graph_DataPair_List_Next(dp);
    }

    if (cnt) {
      double average;

      average=totalSum/((double)cnt);
      dp=AF_Graph_DataPair_List_First(ds->dataPairList);
      while(dp) {
        if (AF_Graph_DataPair_GetLevel(dp)==level) {
          AF_Graph_DataPair_SetValueY(dp, average);
        }
        dp=AF_Graph_DataPair_List_Next(dp);
      }
    }
  }
}



void AF_Graph_DataSet_FloatingAverageDataPairs(AF_GRAPH_DATASET *ds, int level, int num) {
  assert(ds);
  if (ds->dataPairList) {
    AF_GRAPH_DATAPAIR *dp;
    double lastValues[num];
    int cnt=0;
    int idx=0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double v;
        int i;

        v=AF_Graph_DataPair_GetValueY(dp);
        if (idx>=num)
          idx=0;
        lastValues[idx]=v;
        idx++;
        cnt++;

        if (cnt<num) {
          /* buffer not full, average over the values we already have */
          if (cnt) {
            double totalSum=0.0;

            for (i=0; i<cnt; i++) {
              totalSum+=lastValues[i];
            }
            AF_Graph_DataPair_SetValueY(dp, totalSum/((double)cnt));
          }
        }
        else {
          double totalSum=0.0;

          /* buffer full, average over full buffer */
          for (i=0; i<num; i++) {
            totalSum+=lastValues[i];
          }
          AF_Graph_DataPair_SetValueY(dp, totalSum/((double)num));
        }
      }

      dp=AF_Graph_DataPair_List_Next(dp);
    }
  }
}



int AF_Graph_DataSet_GetAverageValueDistX(const AF_GRAPH_DATASET *ds, int level, double *pResult) {
  if (ds->dataPairList) {
    const AF_GRAPH_DATAPAIR *dp;
    double sumOfDists=0.0;
    double lastValue=0.0;
    int cnt=0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double d;

        d=AF_Graph_DataPair_GetValueX(dp);
        if (cnt)
          sumOfDists+=abs(d-lastValue);
        lastValue=d;
        cnt++;
      }
      dp=AF_Graph_DataPair_List_Next(dp);
    }

    if (cnt>1) {
      *pResult=sumOfDists/((double) (cnt-1));
      return 0;
    }
    else
      return 1;
  }
  else {
    return GWEN_ERROR_NO_DATA;
  }
}



int AF_Graph_DataSet_GetAverageValueDistY(const AF_GRAPH_DATASET *ds, int level, double *pResult) {
  if (ds->dataPairList) {
    const AF_GRAPH_DATAPAIR *dp;
    double sumOfDists=0.0;
    double lastValue=0.0;
    int cnt=0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double d;

        d=AF_Graph_DataPair_GetValueY(dp);
        if (cnt)
          sumOfDists+=abs(d-lastValue);
        lastValue=d;
        cnt++;
      }
      dp=AF_Graph_DataPair_List_Next(dp);
    }

    if (cnt>1) {
      *pResult=sumOfDists/((double) (cnt-1));
      return 0;
    }
    else
      return 1;
  }
  else {
    return GWEN_ERROR_NO_DATA;
  }
}



int AF_Graph_DataSet_GetMinValueDistX(const AF_GRAPH_DATASET *ds, int level, double *pResult) {
  if (ds->dataPairList) {
    const AF_GRAPH_DATAPAIR *dp;
    double minDist=0.0;
    double lastValue=0.0;
    int cnt=0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double d;

        d=AF_Graph_DataPair_GetValueX(dp);
        if (cnt) {
          double dist;

          dist=abs(d-lastValue);
          if (cnt>1) {
            if (dist<minDist)
              minDist=dist;
          }
          else
            minDist=dist;
        }
        lastValue=d;
        cnt++;
      }
      dp=AF_Graph_DataPair_List_Next(dp);
    }

    if (cnt>1) {
      *pResult=minDist;
      return 0;
    }
    else
      return 1;
  }
  else {
    return GWEN_ERROR_NO_DATA;
  }
}



int AF_Graph_DataSet_GetMinValueDistY(const AF_GRAPH_DATASET *ds, int level, double *pResult) {
  if (ds->dataPairList) {
    const AF_GRAPH_DATAPAIR *dp;
    double minDist=0.0;
    double lastValue=0.0;
    int cnt=0;

    dp=AF_Graph_DataPair_List_First(ds->dataPairList);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double d;

        d=AF_Graph_DataPair_GetValueY(dp);
        if (cnt) {
          double dist;

          dist=abs(d-lastValue);
          if (dist<minDist)
            minDist=dist;
        }
        lastValue=d;
        cnt++;
      }
      dp=AF_Graph_DataPair_List_Next(dp);
    }

    if (cnt>1) {
      *pResult=minDist;
      return 0;
    }
    else
      return 1;
  }
  else {
    return GWEN_ERROR_NO_DATA;
  }
}





