/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AF_GRAPH_P_H
#define AF_GRAPH_P_H


#include <aqfinance/graphs/af_graph.h>



struct AF_GRAPH {
  GWEN_LIST_ELEMENT(AF_GRAPH)

  AF_GRAPH_DATASET *dataSet;
  AF_GRAPHICS *graphics;
  int width;
  int height;

  int penTitle;
  int penSubTitle;
  int penAxisLabel;
  int penAxis;
  int penGraph;
  int penGraphBackground;
  int penGrid;
  int penBackground;

  int fontTitle;
  int fontSubTitle;
  int fontAxisLabel;
  int fontTickLabel;

  int xTickLabelMaxWidth;
  int xTickLabelMaxHeight;
  int xTickLabelPosY;

  int yTickLabelMaxWidth;
  int yTickLabelMaxHeight;
  int yTickLabelPosX;

  int titleWidth;
  int titleHeight;
  int titlePosX;
  int titlePosY;

  int subTitleWidth;
  int subTitleHeight;
  int subTitlePosX;
  int subTitlePosY;

  int xAxisLabelWidth;
  int xAxisLabelHeight;
  int xAxisLabelPosX;
  int xAxisLabelPosY;

  int yAxisLabelWidth;
  int yAxisLabelHeight;
  int yAxisLabelPosX;
  int yAxisLabelPosY;

  int graphAreaWidth;
  int graphAreaHeight;
  int graphAreaPosX;
  int graphAreaPosY;

  double graphFactorX;
  double graphFactorY;

  int graphOriginX;
  int graphOriginY;

  int yAxisPosX;
  int xAxisPosY;

  uint32_t flags;
};



int AF_Graph_CreatePenFromConfig(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig, const char *name);
int AF_Graph_CreateFontFromConfig(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig, const char *name, int defaultFontSize);
int AF_Graph_CreatePensAndFonts(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig);

void AF_Graph_ReleasePensAndFonts(AF_GRAPH *gr);
void AF_Graph_GetTickLabelMaxSizes(AF_GRAPH *gr, AF_GRAPH_TICK_LIST *tl, int *pMaxWidth, int *pMaxHeight);
void AF_Graph_SampleSizes(AF_GRAPH *gr, AF_GRAPH_DATASET *ds);
void AF_Graph_Layout(AF_GRAPH *gr, AF_GRAPH_DATASET *ds);
void AF_Graph_CalcGraphArea(AF_GRAPH *gr, AF_GRAPH_DATASET *ds);

void AF_Graph_PrepareGraph(AF_GRAPH *gr);


void AF_Graph_DrawDataAsBars(AF_GRAPH *gr, int level, int penId);
void AF_Graph_DrawDataAsLine(AF_GRAPH *gr, int level, int penId);





#endif



