/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/





double AF_Graph_Tick_GetRoundedMaxVal(double val) {
  double maxVal;

  if (val<0.0)
    val=-val;

  if (val>100000000.0)
    maxVal=1000000000.0;
  else if (val>10000000.0)
    maxVal=100000000.0;
  else if (val>1000000.0)
    maxVal=10000000.0;
  else if (val>100000.0)
    maxVal=1000000.0;
  else if (val>10000.0)
    maxVal=100000.0;
  else if (val>1000.0)
    maxVal=10000.0;
  else if (val>100.0)
    maxVal=1000.0;
  else if (val>10.0)
    maxVal=100.0;
  else if (val>10.0)
    maxVal=100.0;
  else if (val>1.0)
    maxVal=10.0;
  else if (val>0.1)
    maxVal=1.0;
  else if (val>0.01)
    maxVal=0.1;
  else if (val>0.001)
    maxVal=0.01;
  else if (val>0.0001)
    maxVal=0.001;
  else if (val>0.00001)
    maxVal=0.0001;
  else if (val>0.000001)
    maxVal=0.00001;
  else if (val<0.000001)
    maxVal=0.000001;

  return maxVal;
}



static void AF_Graph_Tick_AddTicks_IntSubTicks(AF_GRAPH_TICK_LIST *tickList,
                                               double minVal, double maxVal, double stepFactor, int level) {
  double vCurrent;
  double vMin, vMax, v;
  double steps;

  vMin=AF_Graph_Tick_GetRoundedMaxVal(minVal);
  vMax=AF_Graph_Tick_GetRoundedMaxVal(maxVal);

  v=(vMin>vMax)?vMin:vMax;
  if (minVal<0.0)
    vMin=-vMin;
  if (maxVal<0.0)
    vMax=-vMax;
  steps=v/stepFactor;

  if (vMin<0.0 && vMax>0.0) {
    /* create ticks for negative values */
    vCurrent=-v;
    while(vCurrent<0.0) {
      if (vCurrent>=minVal) {
        AF_GRAPH_TICK *t;
        char numbuf[64];

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, vCurrent);
        AF_Graph_Tick_SetLevel(t, level);
        snprintf(numbuf, sizeof(numbuf)-1, "%d", (int)vCurrent);
        numbuf[sizeof(numbuf)-1]=0;
        DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
        AF_Graph_Tick_SetLabel(t, numbuf);
        AF_Graph_Tick_List_Add(t, tickList);
      }

      vCurrent+=steps;
    }

    /* create ticks for positive values */
    vCurrent=0.0;
    while(vCurrent<=vMax) {
      AF_GRAPH_TICK *t;
      char numbuf[64];
  
      t=AF_Graph_Tick_new();
      AF_Graph_Tick_SetValue(t, vCurrent);
      AF_Graph_Tick_SetLevel(t, level);
      snprintf(numbuf, sizeof(numbuf)-1, "%d", (int)vCurrent);
      numbuf[sizeof(numbuf)-1]=0;
      DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
      AF_Graph_Tick_SetLabel(t, numbuf);
      AF_Graph_Tick_List_Add(t, tickList);
  
      vCurrent+=steps;
    }
  }
  else if (vMin<0.0 && vMax<0.0) {
    /* create ticks for negative values */
    vCurrent=-v;
    while(vCurrent<=vMax) {
      if (vCurrent>=vMin) {
        AF_GRAPH_TICK *t;
        char numbuf[64];

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, vCurrent);
        AF_Graph_Tick_SetLevel(t, level);
        snprintf(numbuf, sizeof(numbuf)-1, "%d", (int)vCurrent);
        numbuf[sizeof(numbuf)-1]=0;
        DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
        AF_Graph_Tick_SetLabel(t, numbuf);
        AF_Graph_Tick_List_Add(t, tickList);
      }
      vCurrent-=steps;
    }
  }
  else if (vMin>0.0 && vMax>0.0) {
    /* create ticks for negative values */
    vCurrent=0.0;
    while(vCurrent<=vMax) {
      if (vCurrent>=vMin) {
        AF_GRAPH_TICK *t;
        char numbuf[64];

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, vCurrent);
        AF_Graph_Tick_SetLevel(t, level);
        snprintf(numbuf, sizeof(numbuf)-1, "%d", (int)vCurrent);
        numbuf[sizeof(numbuf)-1]=0;
        DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
        AF_Graph_Tick_SetLabel(t, numbuf);
        AF_Graph_Tick_List_Add(t, tickList);
      }

      vCurrent+=steps;
    }
  }
}



void AF_Graph_Tick_AddTicks_Int(AF_GRAPH_TICK_LIST *tickList, double minVal, double maxVal) {
  DBG_ERROR(0, "Creating ticks from minVal=%f to maxVal=%f", minVal, maxVal);

#if 0
  AF_Graph_Tick_AddTicks_IntSubTicks(tickList, minVal, maxVal, 10.0, 1);
  AF_Graph_Tick_AddTicks_IntSubTicks(tickList, minVal, maxVal, 20.0, 2);
  AF_Graph_Tick_AddTicks_IntSubTicks(tickList, minVal, maxVal, 100.0, 3);
#else
  AF_Graph_Tick_AddTicks_IntSubTicks(tickList, minVal, maxVal, 2.0, 1);
  AF_Graph_Tick_AddTicks_IntSubTicks(tickList, minVal, maxVal, 20.0, 2);
  AF_Graph_Tick_AddTicks_IntSubTicks(tickList, minVal, maxVal, 200.0, 3);
#endif
}













static void AF_Graph_Tick_AddTicks_DoubleSubTicks(AF_GRAPH_TICK_LIST *tickList,
                                                  double minVal, double maxVal, double stepFactor,
                                                  int prec, int level, int withText) {
  double vCurrent;
  double vMin, vMax, v;
  double steps;

  vMin=AF_Graph_Tick_GetRoundedMaxVal(minVal);
  vMax=AF_Graph_Tick_GetRoundedMaxVal(maxVal);

  v=(vMin>vMax)?vMin:vMax;
  if (minVal<0.0)
    vMin=-vMin;
  if (maxVal<0.0)
    vMax=-vMax;
  steps=v/stepFactor;

  if (vMin<0.0 && vMax>0.0) {
    /* create ticks for negative values */
    vCurrent=-v;
    while(vCurrent<0.0) {
      if (vCurrent>=minVal) {
        AF_GRAPH_TICK *t;
        char numbuf[64];

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, vCurrent);
        AF_Graph_Tick_SetLevel(t, level);
        snprintf(numbuf, sizeof(numbuf)-1, "%.*f", prec?prec:2, vCurrent);
        numbuf[sizeof(numbuf)-1]=0;
        DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
        if (withText)
          AF_Graph_Tick_SetLabel(t, numbuf);
        AF_Graph_Tick_List_Add(t, tickList);
      }

      vCurrent+=steps;
    }

    /* create ticks for positive values */
    vCurrent=0.0;
    while(vCurrent<=maxVal) {
      AF_GRAPH_TICK *t;
      char numbuf[64];
  
      t=AF_Graph_Tick_new();
      AF_Graph_Tick_SetValue(t, vCurrent);
      AF_Graph_Tick_SetLevel(t, level);
      snprintf(numbuf, sizeof(numbuf)-1, "%.*f", prec?prec:2, vCurrent);
      numbuf[sizeof(numbuf)-1]=0;
      DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
      if (withText)
        AF_Graph_Tick_SetLabel(t, numbuf);
      AF_Graph_Tick_List_Add(t, tickList);
  
      vCurrent+=steps;
    }
  }
  else if (vMin<0.0 && vMax<0.0) {
    /* create ticks for negative values */
    vCurrent=-v;
    while(vCurrent<maxVal) {
      if (vCurrent>=minVal) {
        AF_GRAPH_TICK *t;
        char numbuf[64];

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, vCurrent);
        AF_Graph_Tick_SetLevel(t, level);
        snprintf(numbuf, sizeof(numbuf)-1, "%.*f", prec?prec:2, vCurrent);
        numbuf[sizeof(numbuf)-1]=0;
        DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
        if (withText)
          AF_Graph_Tick_SetLabel(t, numbuf);
        AF_Graph_Tick_List_Add(t, tickList);
      }

      vCurrent+=steps;
    }
  }
  else if (vMin>0.0 && vMax>0.0) {
    /* create ticks for negative values */
    vCurrent=0.0;
    while(vCurrent<=maxVal) {
      if (vCurrent>=minVal) {
        AF_GRAPH_TICK *t;
        char numbuf[64];
    
        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, vCurrent);
        AF_Graph_Tick_SetLevel(t, level);
        snprintf(numbuf, sizeof(numbuf)-1, "%.*f", prec?prec:2, vCurrent);
        numbuf[sizeof(numbuf)-1]=0;
        DBG_ERROR(0, "Tick: %s (steps %f)", numbuf, steps);
        if (withText)
          AF_Graph_Tick_SetLabel(t, numbuf);
        AF_Graph_Tick_List_Add(t, tickList);
      }
      vCurrent+=steps;
    }
  }
}






static void AF_Graph_Tick_AddTicks_Double(AF_GRAPH_TICK_LIST *tickList, double minVal, double maxVal, int prec) {
  int level=1;

  DBG_ERROR(0, "Creating ticks from minVal=%f to maxVal=%f", minVal, maxVal);

#if 0
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 10.0, prec, level++, 1);
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 20.0, prec, level++, 1);
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 100.0, prec, level++, 1);
#else
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 2.0, prec, level++, 1);
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 10.0, prec, level++, 0);
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 20.0, prec, level++, 1);
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 100.0, prec, level++, 0);
  AF_Graph_Tick_AddTicks_DoubleSubTicks(tickList, minVal, maxVal, 200.0, prec, level++, 1);
#endif
}



void AF_Graph_Tick_AddTicks_DateYears(AF_GRAPH_TICK_LIST *tickList, int level, double minVal, double maxVal) {
  /* add ticks for years */
  int i;

  for (i=(int)minVal; i<=(int) maxVal; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      if (GWEN_Date_DaysInYear(d)==0) {
        int rv;
        GWEN_BUFFER *tbuf;

        tbuf=GWEN_Buffer_new(0, 256, 0, 1);
        rv=GWEN_Date_toStringWithTemplate(d, I18N("YYYY"), tbuf);
        if (rv<0) {
          DBG_INFO(0, "here (%d)", rv);
        }
        else {
          AF_GRAPH_TICK *t;

          t=AF_Graph_Tick_new();
          AF_Graph_Tick_SetValue(t, i);
          AF_Graph_Tick_SetLevel(t, level);
          AF_Graph_Tick_SetLabel(t, GWEN_Buffer_GetStart(tbuf));
          AF_Graph_Tick_List_Add(t, tickList);
        }
        GWEN_Buffer_free(tbuf);
      }
      GWEN_Date_free(d);
    }
  }
}



void AF_Graph_Tick_AddTicks_DateMonths(AF_GRAPH_TICK_LIST *tickList, int level, double minVal, double maxVal) {
  int i;

  for (i=(int)minVal; i<=(int) maxVal; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      if (GWEN_Date_GetDay(d)==1) {
        int rv;
        GWEN_BUFFER *tbuf;

        tbuf=GWEN_Buffer_new(0, 256, 0, 1);
        rv=GWEN_Date_toStringWithTemplate(d, I18N("YY/MM"), tbuf);
        if (rv<0) {
          DBG_INFO(0, "here (%d)", rv);
        }
        else {
          AF_GRAPH_TICK *t;

          t=AF_Graph_Tick_new();
          AF_Graph_Tick_SetValue(t, i);
          AF_Graph_Tick_SetLevel(t, level);
          AF_Graph_Tick_SetLabel(t, GWEN_Buffer_GetStart(tbuf));
          AF_Graph_Tick_List_Add(t, tickList);
        }
        GWEN_Buffer_free(tbuf);
      }
      GWEN_Date_free(d);
    }
  }
}



void AF_Graph_Tick_AddTicks_DateWeeks(AF_GRAPH_TICK_LIST *tickList, int level, double minVal, double maxVal) {
  int i;

  for (i=(int)minVal; i<=(int) maxVal; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      if (GWEN_Date_WeekDay(d)==1) { /* week starts at monday */
        GWEN_BUFFER *tbuf;
        int rv;

        tbuf=GWEN_Buffer_new(0, 256, 0, 1);
        rv=GWEN_Date_toStringWithTemplate(d, I18N("YY/MM/DD"), tbuf);
        if (rv<0) {
          DBG_INFO(0, "here (%d)", rv);
        }
        else {
          AF_GRAPH_TICK *t;

          t=AF_Graph_Tick_new();
          AF_Graph_Tick_SetValue(t, i);
          AF_Graph_Tick_SetLevel(t, level);
          AF_Graph_Tick_SetLabel(t, GWEN_Buffer_GetStart(tbuf));
          AF_Graph_Tick_List_Add(t, tickList);
        }
        GWEN_Buffer_free(tbuf);
      }
      GWEN_Date_free(d);
    }
  }
}



void AF_Graph_Tick_AddTicks_DateDays(AF_GRAPH_TICK_LIST *tickList, int level, double minVal, double maxVal) {
  int i;

  for (i=(int)minVal; i<=(int) maxVal; i++) {
    GWEN_DATE *d;

    d=GWEN_Date_fromJulian(i);
    if (d==NULL) {
      DBG_ERROR(0, "Invalid julian date %d", i);
    }
    else {
      int rv;
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      rv=GWEN_Date_toStringWithTemplate(d, I18N("YY/MM/DD"), tbuf);
      if (rv<0) {
        DBG_INFO(0, "here (%d)", rv);
      }
      else {
        AF_GRAPH_TICK *t;

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, i);
        AF_Graph_Tick_SetLevel(t, level);
        AF_Graph_Tick_SetLabel(t, GWEN_Buffer_GetStart(tbuf));
        AF_Graph_Tick_List_Add(t, tickList);
      }
      GWEN_Buffer_free(tbuf);
      GWEN_Date_free(d);
    }
  }
}



static void AF_Graph_Tick_AddTicks_Date(AF_GRAPH_TICK_LIST *tickList, double minVal, double maxVal) {
  int dist;
  int level;

  dist=maxVal-minVal;
  level=1;

  /* add ticks for years */
  if (dist>365)
    AF_Graph_Tick_AddTicks_DateYears(tickList, level++, minVal, maxVal);

  /* add ticks for months */
  if (dist>30)
    AF_Graph_Tick_AddTicks_DateMonths(tickList, level++, minVal, maxVal);

  /* add ticks for weeks */
  if (dist>7)
    AF_Graph_Tick_AddTicks_DateWeeks(tickList, level++, minVal, maxVal);

  /* add ticks for days */
  if (dist>1)
    AF_Graph_Tick_AddTicks_DateDays(tickList, level++, minVal, maxVal);
}









void AF_Graph_Tick_AddTicks(AF_GRAPH_TICK_LIST *tickList, int dataType, double minVal, double maxVal, int prec) {
  switch(dataType) {
  case AF_Graph_DataSet_DataType_Integer:
    AF_Graph_Tick_AddTicks_Int(tickList, minVal, maxVal);
    break;
  case AF_Graph_DataSet_DataType_Double:
    AF_Graph_Tick_AddTicks_Double(tickList, minVal, maxVal, prec);
    break;
  case AF_Graph_DataSet_DataType_Date:
    AF_Graph_Tick_AddTicks_Date(tickList, minVal, maxVal);
    break;
  default:
    break;
  }
}





static int GWENHYWFAR_CB AF_Graph_Tick_List_Compare_ValueAndLevel(const AF_GRAPH_TICK *p_a, const AF_GRAPH_TICK *p_b, int p_ascending) {
  int p_rv;

  if (p_b->value==p_a->value) {
    if (p_b->level==p_a->level) {
      p_rv=0;
    }
    else if (p_a->level<p_b->level)
      p_rv=-1;
    else
      p_rv=1;
  }
  else if (p_a->value<p_b->value)
    p_rv=-1;
  else
    p_rv=1;

  if (p_ascending)
    return p_rv;
  else
    return -p_rv;
}



void AF_Graph_Tick_List_SortByValueAndLevel(AF_GRAPH_TICK_LIST *p_list, int p_ascending) {
  AF_GRAPH_TICK_LIST_SORT_FN oldSortFn;

  oldSortFn=AF_Graph_Tick_List_SetSortFn(p_list, AF_Graph_Tick_List_Compare_ValueAndLevel);
  AF_Graph_Tick_List_Sort(p_list, p_ascending);
  AF_Graph_Tick_List_SetSortFn(p_list, oldSortFn);
}



int AF_Graph_Tick_List_GetAverageValueDist(const AF_GRAPH_TICK_LIST *tl, int level, double *pResult) {
  if (tl) {
    const AF_GRAPH_TICK *t;
    double sumOfDists=0.0;
    double lastValue=0.0;
    int cnt=0;

    t=AF_Graph_Tick_List_First(tl);
    while(t) {
      if (AF_Graph_Tick_GetLevel(t)==level) {
        double d;

        d=AF_Graph_Tick_GetValue(t);
        if (cnt)
          sumOfDists+=abs(d-lastValue);
        lastValue=d;
        cnt++;
      }
      t=AF_Graph_Tick_List_Next(t);
    }

    if (cnt>1) {
      *pResult=sumOfDists/((double) (cnt-1));
      return 0;
    }
    else
      return 1;
  }
  else {
    return GWEN_ERROR_NO_DATA;
  }
}






