/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#ifdef HAVE_CAIRO


#include "af_htmlctx_cairo_p.h"

#include <gwenhywfar/directory.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>

#include <gwenhywfar/o_image_be.h>

#include <cairo/cairo.h>
#include <cairo/cairo-pdf.h>
#include <cairo/cairo-ps.h>



GWEN_INHERIT(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO);
GWEN_INHERIT(HTML_IMAGE, cairo_surface_t);




GWEN_XML_CONTEXT *AF_HtmlCtxCairo_Pdf_new(uint32_t flags, const char *filename, double width, double height) {
  GWEN_XML_CONTEXT *ctx;
  AF_HTMLCTX_CAIRO *xc;
  HTML_PROPS *pr;
  HTML_FONT *fnt;

  ctx=HtmlCtx_new(flags);
  GWEN_NEW_OBJECT(AF_HTMLCTX_CAIRO, xc);
  GWEN_INHERIT_SETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx, xc, AF_HtmlCtxCairo_FreeData);

  xc->fontList=HtmlFont_List_new();

  xc->surface=cairo_pdf_surface_create(filename, width, height);
  xc->cr=cairo_create(xc->surface);

  HtmlCtx_SetGetTextWidthFn(ctx, AF_HtmlCtxCairo_GetTextWidth);
  HtmlCtx_SetGetTextHeightFn(ctx, AF_HtmlCtxCairo_GetTextHeight);
  HtmlCtx_SetGetColorFromNameFn(ctx, AF_HtmlCtxCairo_GetColorFromName);
  HtmlCtx_SetGetFontFn(ctx, AF_HtmlCtxCairo_GetFont);
  HtmlCtx_SetGetImageFn(ctx, AF_HtmlCtxCairo_GetImage);

  HtmlCtx_SetResolutionX(ctx, 300);
  HtmlCtx_SetResolutionY(ctx, 300);

  pr=HtmlProps_new();
  fnt=AF_HtmlCtxCairo_GetFont(ctx, "", 48, 0);
  HtmlProps_SetFont(pr, fnt);
  HtmlFont_free(fnt);
  HtmlProps_SetForegroundColor(pr, 0x000000);
  HtmlProps_SetBackgroundColor(pr, 0xffffff);
  HtmlCtx_SetStandardProps(ctx, pr);

  HtmlProps_free(pr);

  return ctx;
}



GWEN_XML_CONTEXT *AF_HtmlCtxCairo_Ps_new(uint32_t flags, const char *filename, double width, double height) {
  GWEN_XML_CONTEXT *ctx;
  AF_HTMLCTX_CAIRO *xc;
  HTML_PROPS *pr;
  HTML_FONT *fnt;

  ctx=HtmlCtx_new(flags);
  GWEN_NEW_OBJECT(AF_HTMLCTX_CAIRO, xc);
  GWEN_INHERIT_SETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx, xc, AF_HtmlCtxCairo_FreeData);

  xc->fontList=HtmlFont_List_new();

  xc->surface=cairo_ps_surface_create(filename, width, height);
  xc->cr=cairo_create(xc->surface);

  HtmlCtx_SetGetTextWidthFn(ctx, AF_HtmlCtxCairo_GetTextWidth);
  HtmlCtx_SetGetTextHeightFn(ctx, AF_HtmlCtxCairo_GetTextHeight);
  HtmlCtx_SetGetColorFromNameFn(ctx, AF_HtmlCtxCairo_GetColorFromName);
  HtmlCtx_SetGetFontFn(ctx, AF_HtmlCtxCairo_GetFont);

  pr=HtmlProps_new();
  fnt=AF_HtmlCtxCairo_GetFont(ctx, "", 48, 0);
  HtmlProps_SetFont(pr, fnt);
  HtmlFont_free(fnt);
  HtmlProps_SetForegroundColor(pr, 0x000000);
  HtmlProps_SetBackgroundColor(pr, 0xffffff);
  HtmlCtx_SetStandardProps(ctx, pr);
  HtmlProps_free(pr);

  return ctx;
}



GWENHYWFAR_CB void AF_HtmlCtxCairo_FreeData(void *bp, void *p) {
  AF_HTMLCTX_CAIRO *xc;

  xc=(AF_HTMLCTX_CAIRO*) p;

  if (xc->cr)
    cairo_destroy(xc->cr);
  if (xc->surface)
    cairo_surface_destroy(xc->surface);

  HtmlFont_List_free(xc->fontList);
  GWEN_FREE_OBJECT(xc);
}



void AF_HtmlCtxCairo_SetFontSize(GWEN_XML_CONTEXT *ctx, int size) {
  HTML_PROPS *pr;
  HTML_FONT *fnt;

  pr=HtmlCtx_GetStandardProps(ctx);
  fnt=AF_HtmlCtxCairo_GetFont(ctx, "", size, 0);
  HtmlProps_SetFont(pr, fnt);
  HtmlFont_free(fnt);
}




static void AF_HtmlCtxCairo_SelectFont(GWEN_XML_CONTEXT *ctx, HTML_FONT *fnt) {
  AF_HTMLCTX_CAIRO *xc;
  cairo_font_slant_t slant=CAIRO_FONT_SLANT_NORMAL;
  cairo_font_weight_t weight=CAIRO_FONT_WEIGHT_NORMAL;
  uint32_t flags;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  flags=HtmlFont_GetFontFlags(fnt);

  if (flags & HTML_FONT_FLAGS_ITALIC)
    slant=CAIRO_FONT_SLANT_ITALIC;
  if (flags & HTML_FONT_FLAGS_STRONG)
    weight=CAIRO_FONT_WEIGHT_BOLD;
  cairo_select_font_face(xc->cr, HtmlFont_GetFontName(fnt), slant, weight);
  cairo_set_font_size(xc->cr, HtmlFont_GetFontSize(fnt));
}




int AF_HtmlCtxCairo_GetTextWidth(GWEN_XML_CONTEXT *ctx,
				 HTML_FONT *fnt,
				 const char *s) {
  AF_HTMLCTX_CAIRO *xc;
  cairo_text_extents_t extents;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  AF_HtmlCtxCairo_SelectFont(ctx, fnt);
  cairo_text_extents(xc->cr, s, &extents);

  return (int)(extents.width);
}



int AF_HtmlCtxCairo_GetTextHeight(GWEN_XML_CONTEXT *ctx,
				  HTML_FONT *fnt,
				  const char *s) {
  AF_HTMLCTX_CAIRO *xc;
  cairo_text_extents_t extents;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  AF_HtmlCtxCairo_SelectFont(ctx, fnt);
  cairo_text_extents(xc->cr, s, &extents);

  return (int)(extents.height);
}



uint32_t AF_HtmlCtxCairo_GetColorFromName(const GWEN_XML_CONTEXT *ctx, const char *s) {
  if (strcasecmp(s, "black")==0)
    return 0x000000;
  else if (strcasecmp(s, "red")==0)
    return 0xff0000;
  else if (strcasecmp(s, "green")==0)
    return 0x00ff00;
  else if (strcasecmp(s, "blue")==0)
    return 0x0000ff;
  else if (strcasecmp(s, "white")==0)
    return 0xffffff;

  return 0xffffff;
}





static HTML_FONT *AF_HtmlCtxCairo_FindFont(const GWEN_XML_CONTEXT *ctx,
					   const char *fontName,
					   int fontSize,
					   uint32_t fontFlags) {
  AF_HTMLCTX_CAIRO *xc;
  HTML_FONT *fnt;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  assert(xc->fontList);
  fnt=HtmlFont_List_First(xc->fontList);
  while(fnt) {
    const char *s;

    s=HtmlFont_GetFontName(fnt);
    if (s && *s &&
	HtmlFont_GetFontSize(fnt)==fontSize &&
	HtmlFont_GetFontFlags(fnt)==fontFlags &&
	strcasecmp(s, fontName)==0)
      break;
    fnt=HtmlFont_List_Next(fnt);
  }

  return fnt;
}



HTML_FONT *AF_HtmlCtxCairo_GetFont(GWEN_XML_CONTEXT *ctx,
				   const char *fontName,
				   int fontSize,
				   uint32_t fontFlags) {
  AF_HTMLCTX_CAIRO *xc;
  HTML_FONT *fnt;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  fnt=AF_HtmlCtxCairo_FindFont(ctx, fontName, fontSize, fontFlags);
  if (fnt)
    return fnt;
  else {
    fnt=HtmlFont_new();
    HtmlFont_SetFontName(fnt, fontName);
    HtmlFont_SetFontSize(fnt, fontSize);
    HtmlFont_SetFontFlags(fnt, fontFlags);
    HtmlFont_List_Add(fnt, xc->fontList);
    return fnt;
  }
}



static GWENHYWFAR_CB void AF_HtmlCtxCairo_freeImageData(void *bp, void *p) {
  cairo_surface_t *ximg;

  ximg=(cairo_surface_t*) p;
  cairo_surface_destroy(ximg);
}



HTML_IMAGE *AF_HtmlCtxCairo_GetImage(GWEN_XML_CONTEXT *ctx,
				     const char *fileName) {
  GWEN_STRINGLIST *sl;

  sl=HtmlCtx_GetMediaPaths(ctx);
  if (sl) {
    GWEN_BUFFER *tbuf;
    int rv;
    cairo_surface_t *ximg;
    HTML_IMAGE *img;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    rv=GWEN_Directory_FindFileInPaths(sl, fileName, tbuf);
    if (rv<0) {
      DBG_ERROR(GWEN_LOGDOMAIN, "here (%d)", rv);
      GWEN_Buffer_free(tbuf);
      return NULL;
    }

    DBG_DEBUG(GWEN_LOGDOMAIN, "Loading [%s]", GWEN_Buffer_GetStart(tbuf));
    ximg=cairo_image_surface_create_from_png(GWEN_Buffer_GetStart(tbuf));
    if (ximg==NULL) {
      DBG_ERROR(GWEN_LOGDOMAIN, "Could not load icon [%s]", GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
      return NULL;
    }

    img=HtmlImage_new();
    HtmlImage_SetImageName(img, GWEN_Buffer_GetStart(tbuf));
    HtmlImage_SetWidth(img, cairo_image_surface_get_width(ximg));
    HtmlImage_SetHeight(img, cairo_image_surface_get_height(ximg));

    GWEN_INHERIT_SETDATA(HTML_IMAGE, cairo_surface_t, img, ximg,
			 AF_HtmlCtxCairo_freeImageData);
    GWEN_Buffer_free(tbuf);
    return img;
  }
  else {
    DBG_ERROR(GWEN_LOGDOMAIN, "No media paths in dialog");
    return NULL;
  }
}



static void AF_HtmlCtxCairo__Paint(GWEN_XML_CONTEXT *ctx, HTML_OBJECT *o, int xOffset, int yOffset) {
  AF_HTMLCTX_CAIRO *xc;
  HTML_OBJECT *c;
  HTML_PROPS *stdProps;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  xOffset+=HtmlObject_GetX(o);
  yOffset+=HtmlObject_GetY(o);

  stdProps=HtmlCtx_GetStandardProps(ctx);

  switch(HtmlObject_GetObjectType(o)) {
  case HtmlObjectType_Word: {
    HTML_PROPS *pr;
    HTML_FONT *fnt;
    int ascent=0;
    uint32_t col;
    cairo_font_extents_t extents;
    double red, green, blue;

    pr=HtmlObject_GetProperties(o);

    /* select font */
    fnt=HtmlProps_GetFont(pr);
    AF_HtmlCtxCairo_SelectFont(ctx, fnt);
    /* get ascent */
    cairo_font_extents(xc->cr, &extents);
    ascent=extents.ascent;

    /* select foreground color */
    col=HtmlProps_GetForegroundColor(pr);
    if (col==HTML_PROPS_NOCOLOR) {
      col=HtmlProps_GetForegroundColor(stdProps);
    }

    red=(double)((col>>16) & 0xff)/255.0;
    green=(double)((col>>8) & 0xff)/255.0;
    blue=(double)(col & 0xff)/255.0;

    /*  set foreground color */;
    cairo_set_source_rgb(xc->cr, red, green, blue);

#if 0
    if (1) {
      const char *s;

      s=HtmlObject_GetText(o);
      DBG_ERROR(0, "Writing text to %d / %d", xOffset, yOffset+ascent);
      fprintf(stderr, "Writing text to %d / %d\n", xOffset, yOffset+ascent);
      if (s && *s)
	GWEN_Text_DumpString(s, strlen(s), stderr, 2);
      else {
	DBG_ERROR(0, "Upps...");
	fprintf(stderr, "Upps...\n");
      }
    }
#endif

    cairo_move_to(xc->cr, xOffset, yOffset+ascent);
    cairo_show_text(xc->cr, HtmlObject_GetText(o));
    break;
  }
  case HtmlObjectType_Image: {
    HTML_IMAGE *img;

    img=HtmlObject_Image_GetImage(o);
    if (img) {
      cairo_surface_t *ximg;
      ximg=GWEN_INHERIT_GETDATA(HTML_IMAGE, cairo_surface_t, img);
      if (ximg) {
	cairo_set_source_surface(xc->cr, ximg, xOffset, yOffset);
	cairo_paint(xc->cr);
      }
    }
    break;
  }

  default:
    break;
  }

  c=HtmlObject_Tree_GetFirstChild(o);
  while(c) {
    AF_HtmlCtxCairo__Paint(ctx, c, xOffset, yOffset);
    c=HtmlObject_Tree_GetNext(c);
  }
}



void AF_HtmlCtxCairo_Paint(GWEN_XML_CONTEXT *ctx, int xOffset, int yOffset) {
  AF_HTMLCTX_CAIRO *xc;
  HTML_OBJECT *o;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  o=HtmlCtx_GetRootObject(ctx);
  if (o)
    AF_HtmlCtxCairo__Paint(ctx, o, xOffset, yOffset);

}



void AF_HtmlCtxCairo_EndPage(GWEN_XML_CONTEXT *ctx) {
  AF_HTMLCTX_CAIRO *xc;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

#if 0
  if (1) {
    cairo_status_t st;

    fprintf(stderr, "Writing to PNG\n");
    cairo_surface_flush(xc->surface);
    st=cairo_surface_write_to_png(xc->surface, "/tmp/test.png");
    if (st!=CAIRO_STATUS_SUCCESS) {
      fprintf(stderr, "Cairo Status: %d (%s)\n", st, cairo_status_to_string(st));
    }
  }
#endif

  cairo_surface_show_page(xc->surface);
}



void AF_HtmlCtxCairo_Finish(GWEN_XML_CONTEXT *ctx) {
  AF_HTMLCTX_CAIRO *xc;

  assert(ctx);
  xc=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AF_HTMLCTX_CAIRO, ctx);
  assert(xc);

  cairo_surface_finish(xc->surface);
}




#endif


