/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AF_PRINT_H
#define AF_PRINT_H


#include "af_print_doc.h"
#include "af_print_table_descr.h"

#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_commodity.h>
#include <aqfinance/engine/book/ae_tax.h>

#include <gwenhywfar/xml.h>


typedef struct AF_PRINT AF_PRINT;


#ifdef __cplusplus
extern "C" {
#endif


AE_API AF_PRINT *AF_Print_newForPdf(const char *filename, AF_PRINT_DOC *docTmpl);
AE_API AF_PRINT *AF_Print_newForPs(const char *filename, AF_PRINT_DOC *docTmpl);

AE_API void AF_Print_free(AF_PRINT *pr);

AE_API AF_PRINT_DOC *AF_Print_GetCurrentDoc(const AF_PRINT *pr);
AE_API void AF_Print_AddMediaPath(AF_PRINT *pr, const char *s);

/**
 * @return 0 if text fits, bit0=1: too wide, bit1=1: too high, <0: error
 */
AE_API int AF_Print_CheckText(AF_PRINT *pr, AF_PRINT_BOX_ID boxId, const char *text);

AE_API int AF_Print_SetText(AF_PRINT *pr, AF_PRINT_BOX_ID boxId, const char *text);

AE_API int AF_Print_NextPage(AF_PRINT *pr);

AE_API int AF_Print_Finish(AF_PRINT *pr);


AE_API AF_PRINT_DOC *AF_Print_GenerateTemplate_A4(double dpi);
AE_API AF_PRINT_DOC *AF_Print_GenerateTemplate_A4Letter(double dpi);

AE_API int AF_Print_LoadDocTemplate(const char *name, AF_PRINT_DOC **pDoc);


AE_API int AF_Print_Job(AF_PRINT *pr,
                        const AF_PRINT_TABLE_DESCR *descr,
                        const AE_COMMODITY_LIST *coList,
                        const AE_TAX_LIST *taxes,
                        const AE_JOB *j);


#ifdef __cplusplus
}
#endif


#endif

