/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/




int AF_Report2_Category_AddDataPair(AF_REPORT2_CATEGORY *cat, AF_REPORT2_DATAPAIR *dp) {
  AF_REPORT2_YEARINFO_LIST *yil;
  AF_REPORT2_YEARINFO *yi;
  AF_REPORT2_MONTHINFO_LIST *mil;
  AF_REPORT2_MONTHINFO *mi;
  AF_REPORT2_DAYINFO_LIST *dil;
  AF_REPORT2_DAYINFO *di;
  AF_REPORT2_DATAPAIR_LIST *dpl;
  AQDB_ID tmpId;
  const GWEN_DATE *dt;
  double v;

  assert(cat);
  assert(dp);

  v=AF_Report2_DataPair_GetValue(dp);
  dt=AF_Report2_DataPair_GetDate(dp);
  if (dt==NULL) {
    DBG_ERROR(0, "No date in data pair");
    return GWEN_ERROR_BAD_DATA;
  }

  /* category */
  AF_Report2_Category_SetTotalSum(cat, AF_Report2_Category_GetTotalSum(cat)+v);
  if (v<0.0)
    AF_Report2_Category_SetNegativeSum(cat, AF_Report2_Category_GetNegativeSum(cat)+v);
  else
    AF_Report2_Category_SetPositiveSum(cat, AF_Report2_Category_GetPositiveSum(cat)+v);


  /* yearInfo */
  tmpId=GWEN_Date_GetYear(dt);
  yil=AF_Report2_Category_GetYearInfoList(cat);
  if (yil==NULL) {
    yil=AF_Report2_YearInfo_List_new();
    AF_Report2_Category_SetYearInfoList(cat, yil);
  }

  yi=AF_Report2_YearInfo_List_GetById(yil, tmpId);
  if (yi==NULL) {
    yi=AF_Report2_YearInfo_new();
    AF_Report2_YearInfo_SetId(yi, tmpId);
    AF_Report2_YearInfo_List_Add(yi, yil);
  }
  AF_Report2_YearInfo_SetTotalSum(yi, AF_Report2_YearInfo_GetTotalSum(yi)+v);
  if (v<0.0)
    AF_Report2_YearInfo_SetNegativeSum(yi, AF_Report2_YearInfo_GetNegativeSum(yi)+v);
  else
    AF_Report2_YearInfo_SetPositiveSum(yi, AF_Report2_YearInfo_GetPositiveSum(yi)+v);


  /* monthInfo */
  tmpId=(GWEN_Date_GetYear(dt)<<16)+(GWEN_Date_GetMonth(dt)<<8);
  mil=AF_Report2_YearInfo_GetMonthInfoList(yi);
  if (mil==NULL) {
    mil=AF_Report2_MonthInfo_List_new();
    AF_Report2_YearInfo_SetMonthInfoList(yi, mil);
  }

  mi=AF_Report2_MonthInfo_List_GetById(mil, tmpId);
  if (mi==NULL) {
    mi=AF_Report2_MonthInfo_new();
    AF_Report2_MonthInfo_SetId(mi, tmpId);
    AF_Report2_MonthInfo_List_Add(mi, mil);
  }
  AF_Report2_MonthInfo_SetTotalSum(mi, AF_Report2_MonthInfo_GetTotalSum(mi)+v);
  if (v<0.0)
    AF_Report2_MonthInfo_SetNegativeSum(mi, AF_Report2_MonthInfo_GetNegativeSum(mi)+v);
  else
    AF_Report2_MonthInfo_SetPositiveSum(mi, AF_Report2_MonthInfo_GetPositiveSum(mi)+v);


  /* dayInfo */
  tmpId=(GWEN_Date_GetYear(dt)<<16)+(GWEN_Date_GetMonth(dt)<<8)+GWEN_Date_GetDay(dt);
  dil=AF_Report2_MonthInfo_GetDayInfoList(mi);
  if (dil==NULL) {
    dil=AF_Report2_DayInfo_List_new();
    AF_Report2_MonthInfo_SetDayInfoList(mi, dil);
  }

  di=AF_Report2_DayInfo_List_GetById(dil, tmpId);
  if (di==NULL) {
    di=AF_Report2_DayInfo_new();
    AF_Report2_DayInfo_SetId(di, tmpId);
    AF_Report2_DayInfo_SetDate(di, dt);
    AF_Report2_DayInfo_List_Add(di, dil);
  }
  AF_Report2_DayInfo_SetTotalSum(di, AF_Report2_DayInfo_GetTotalSum(di)+v);
  if (v<0.0)
    AF_Report2_DayInfo_SetNegativeSum(di, AF_Report2_DayInfo_GetNegativeSum(di)+v);
  else
    AF_Report2_DayInfo_SetPositiveSum(di, AF_Report2_DayInfo_GetPositiveSum(di)+v);


  /* data pair */
  dpl=AF_Report2_DayInfo_GetDataPairList(di);
  if (dpl==NULL) {
    dpl=AF_Report2_DataPair_List_new();
    AF_Report2_DayInfo_SetDataPairList(di, dpl);
  }

  AF_Report2_DataPair_List_Add(dp, dpl);
  return 0;
}


