/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/




void AF_Report2_Settings_SetDefaults(AF_REPORT2_SETTINGS *rs) {
  AF_GRAPH_SETTINGS *gs;

  gs=AF_Graph_Settings_new();
  AF_Graph_Settings_SetDefaults(gs);
  AF_Report2_Settings_SetGraphSettings(rs, gs);
  AF_Report2_Settings_SetPrecisionY(rs, 2);
}



int AF_Report2_Settings_fromData10k(const AE_DATA_10K *o, AF_REPORT2_SETTINGS **pT) {
  AF_REPORT2_SETTINGS *al;
  const char *s;
  GWEN_BINDATA binData;

  al=AF_Report2_Settings_new();
  al->dbId=AE_Data10k_GetDbId(o);

  s=AE_Data10k_GetDataTypeName(o);
  if (!(s && *s && strcasecmp(s, "xml")==0)) {
    DBG_ERROR(AE_LOGDOMAIN, "No XML data (instead: %s)", s?s:"(NULL)");
    AF_Report2_Settings_free(al);
    return GWEN_ERROR_BAD_DATA;
  }

  binData=AE_Data10k_GetData(o);
  if (binData.pointer && binData.length) {
    GWEN_XMLNODE *node=NULL;

    node=GWEN_XMLNode_fromString((const char*) binData.pointer, binData.length, GWEN_XML_FLAGS_DEFAULT);
    if (node==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Error in XML data.");
      AF_Report2_Settings_free(al);
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      GWEN_XMLNODE *rootNode;

      rootNode=GWEN_XMLNode_FindNode(node, GWEN_XMLNodeTypeTag, "af_report2_settings");
      if (rootNode==NULL)
        rootNode=node;
      AF_Report2_Settings_ReadXml(al, rootNode);
    }
    GWEN_XMLNode_free(node);
  }

  al->id=AE_Data10k_GetId(o);
  al->groupId=AE_Data10k_GetGroupId(o);
  al->dbId=AE_Data10k_GetDbId(o);
  AF_Report2_Settings_SetName(al, AE_Data10k_GetName(o));

  *pT=al;
  return 0;
}



int AF_Report2_Settings_toData10k(const AF_REPORT2_SETTINGS *al, AE_DATA_10K *o) {
  assert(al);
  assert(o);

  AE_Data10k_SetId(o, al->id);
  AE_Data10k_SetGroupId(o, al->groupId);
  AE_Data10k_SetName(o, al->name);
  AE_Data10k_SetDataTypeName(o, "xml");
  AE_Data10k_SetDbId(o, al->dbId);

  if (1) {
    GWEN_BUFFER *tbuf;
    GWEN_XMLNODE *node;
    GWEN_XMLNODE *rootNode;
    GWEN_BINDATA binData;
    int rv;

    node=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "root");
    rootNode=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "af_report2_settings");
    GWEN_XMLNode_AddChild(node, rootNode);

    AF_Report2_Settings_toXml(al, rootNode);

    tbuf=GWEN_Buffer_new(0, 1024, 0, 1);
    rv=GWEN_XMLNode_toBuffer(node, tbuf, GWEN_XML_FLAGS_DEFAULT|GWEN_XML_FLAGS_SIMPLE);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_Buffer_free(tbuf);
      GWEN_XMLNode_free(node);
      return rv;
    }

    binData.pointer=(uint8_t*) GWEN_Buffer_GetStart(tbuf);
    binData.length=GWEN_Buffer_GetUsedBytes(tbuf);
    AE_Data10k_SetData(o, binData);

    GWEN_Buffer_free(tbuf);
    GWEN_XMLNode_free(node);
  }

  return 0;
}










